
/* Copyright (c) 2015-2023 Valve Corporation
 * Copyright (c) 2015-2023 LunarG, Inc.
 * Copyright (c) 2015-2017, 2019 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author: Lenny Komow <lenny@lunarg.com>
 * Author: Joey Bzdek <joey@lunarg.com>
 * Author: Shannon McPherson <shannon@lunarg.com>
 * Author: David Pinedo <david@lunarg.com>
 * Author: Charles Giessen <charles@lunarg.com>
 */

/*
 * This file is generated from the Khronos Vulkan XML API Registry.
 */

#pragma once

#include "api_dump.h"
#include "api_dump_video_json.h"

void dump_json_pNext_trampoline(const void* object, const ApiDumpSettings& settings, int indents);


void dump_json_VkClearColorValue(const VkClearColorValue& object, const ApiDumpSettings& settings, int indents);
void dump_json_VkClearValue(const VkClearValue& object, const ApiDumpSettings& settings, int indents);
void dump_json_VkPerformanceCounterResultKHR(const VkPerformanceCounterResultKHR& object, const ApiDumpSettings& settings, int indents);
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_VkDeviceOrHostAddressConstAMDX(const VkDeviceOrHostAddressConstAMDX& object, const ApiDumpSettings& settings, int indents);
#endif // VK_ENABLE_BETA_EXTENSIONS
void dump_json_VkDeviceOrHostAddressKHR(const VkDeviceOrHostAddressKHR& object, const ApiDumpSettings& settings, int indents);
void dump_json_VkDeviceOrHostAddressConstKHR(const VkDeviceOrHostAddressConstKHR& object, const ApiDumpSettings& settings, int indents);
void dump_json_VkAccelerationStructureGeometryDataKHR(const VkAccelerationStructureGeometryDataKHR& object, const ApiDumpSettings& settings, int indents);
void dump_json_VkPerformanceValueDataINTEL(const VkPerformanceValueDataINTEL& object, const ApiDumpSettings& settings, int indents);
void dump_json_VkPipelineExecutableStatisticValueKHR(const VkPipelineExecutableStatisticValueKHR& object, const ApiDumpSettings& settings, int indents);
void dump_json_VkDescriptorDataEXT(const VkDescriptorDataEXT& object, const ApiDumpSettings& settings, int indents);
void dump_json_VkAccelerationStructureMotionInstanceDataNV(const VkAccelerationStructureMotionInstanceDataNV& object, const ApiDumpSettings& settings, int indents);

//=========================== Type Implementations ==========================//

void dump_json_uint64_t(uint64_t object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
void dump_json_size_t(size_t object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
void dump_json_char(char object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
void dump_json_float(float object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
void dump_json_int64_t(int64_t object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
void dump_json_double(double object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}

//========================= Basetype Implementations ========================//

void dump_json_VkBool32(VkBool32 object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
void dump_json_VkDeviceAddress(VkDeviceAddress object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
void dump_json_VkDeviceSize(VkDeviceSize object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
void dump_json_VkFlags(VkFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
void dump_json_VkSampleMask(VkSampleMask object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
void dump_json_VkFlags64(VkFlags64 object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_MTLDevice_id(MTLDevice_id object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_MTLCommandQueue_id(MTLCommandQueue_id object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_MTLBuffer_id(MTLBuffer_id object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_MTLTexture_id(MTLTexture_id object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_IOSurfaceRef(IOSurfaceRef object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_MTLSharedEvent_id(MTLSharedEvent_id object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
#endif // VK_USE_PLATFORM_METAL_EXT
void dump_json_VkRemoteAddressNV(VkRemoteAddressNV object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}


#if defined(VK_USE_PLATFORM_ANDROID_KHR)
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_ANativeWindow(const ANativeWindow* object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
#endif
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_AHardwareBuffer(const AHardwareBuffer* object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
#endif
#endif // VK_USE_PLATFORM_ANDROID_KHR


#if defined(VK_USE_PLATFORM_METAL_EXT)
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_CAMetalLayer(CAMetalLayer object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << "\"" << object << "\"";
}
#endif
#endif // VK_USE_PLATFORM_METAL_EXT

//======================= System Type Implementations =======================//

#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_DWORD(const DWORD object, const ApiDumpSettings& settings, int indents)
{
    if (settings.showAddress())
        settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_XLIB_KHR)
void dump_json_Display(const Display* object, const ApiDumpSettings& settings, int indents)
{
    OutputAddressJSON(settings, object);
    settings.stream() << "\n";
}
#endif // VK_USE_PLATFORM_XLIB_KHR
#if defined(VK_USE_PLATFORM_GGP)
void dump_json_GgpFrameToken(const GgpFrameToken object, const ApiDumpSettings& settings, int indents)
{
    if (settings.showAddress())
        settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
#endif // VK_USE_PLATFORM_GGP
#if defined(VK_USE_PLATFORM_GGP)
void dump_json_GgpStreamDescriptor(const GgpStreamDescriptor object, const ApiDumpSettings& settings, int indents)
{
    if (settings.showAddress())
        settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
#endif // VK_USE_PLATFORM_GGP
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_HANDLE(const HANDLE object, const ApiDumpSettings& settings, int indents)
{
    if (settings.showAddress())
        settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_HINSTANCE(const HINSTANCE object, const ApiDumpSettings& settings, int indents)
{
    if (settings.showAddress())
        settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_HMONITOR(const HMONITOR object, const ApiDumpSettings& settings, int indents)
{
    if (settings.showAddress())
        settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_HWND(const HWND object, const ApiDumpSettings& settings, int indents)
{
    if (settings.showAddress())
        settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_DIRECTFB_EXT)
void dump_json_IDirectFB(const IDirectFB object, const ApiDumpSettings& settings, int indents)
{
    if (settings.showAddress())
        settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
#endif // VK_USE_PLATFORM_DIRECTFB_EXT
#if defined(VK_USE_PLATFORM_DIRECTFB_EXT)
void dump_json_IDirectFBSurface(const IDirectFBSurface object, const ApiDumpSettings& settings, int indents)
{
    if (settings.showAddress())
        settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
#endif // VK_USE_PLATFORM_DIRECTFB_EXT
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_LPCWSTR(const LPCWSTR object, const ApiDumpSettings& settings, int indents)
{
    if (settings.showAddress())
        settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_SECURITY_ATTRIBUTES(const SECURITY_ATTRIBUTES* object, const ApiDumpSettings& settings, int indents)
{
    OutputAddressJSON(settings, object);
    settings.stream() << "\n";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_XLIB_KHR)
void dump_json_VisualID(const VisualID object, const ApiDumpSettings& settings, int indents)
{
    if (settings.showAddress())
        settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
#endif // VK_USE_PLATFORM_XLIB_KHR
#if defined(VK_USE_PLATFORM_XLIB_KHR)
void dump_json_Window(const Window object, const ApiDumpSettings& settings, int indents)
{
    if (settings.showAddress())
        settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
#endif // VK_USE_PLATFORM_XLIB_KHR
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
void dump_json__screen_buffer(const _screen_buffer* object, const ApiDumpSettings& settings, int indents)
{
    OutputAddressJSON(settings, object);
    settings.stream() << "\n";
}
#endif // VK_USE_PLATFORM_SCREEN_QNX
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
void dump_json__screen_context(const _screen_context* object, const ApiDumpSettings& settings, int indents)
{
    OutputAddressJSON(settings, object);
    settings.stream() << "\n";
}
#endif // VK_USE_PLATFORM_SCREEN_QNX
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
void dump_json__screen_window(const _screen_window* object, const ApiDumpSettings& settings, int indents)
{
    OutputAddressJSON(settings, object);
    settings.stream() << "\n";
}
#endif // VK_USE_PLATFORM_SCREEN_QNX
#if defined(VK_USE_PLATFORM_WAYLAND_KHR)
void dump_json_wl_display(const wl_display* object, const ApiDumpSettings& settings, int indents)
{
    OutputAddressJSON(settings, object);
    settings.stream() << "\n";
}
#endif // VK_USE_PLATFORM_WAYLAND_KHR
#if defined(VK_USE_PLATFORM_WAYLAND_KHR)
void dump_json_wl_surface(const wl_surface* object, const ApiDumpSettings& settings, int indents)
{
    OutputAddressJSON(settings, object);
    settings.stream() << "\n";
}
#endif // VK_USE_PLATFORM_WAYLAND_KHR
#if defined(VK_USE_PLATFORM_XCB_KHR)
void dump_json_xcb_connection_t(const xcb_connection_t* object, const ApiDumpSettings& settings, int indents)
{
    OutputAddressJSON(settings, object);
    settings.stream() << "\n";
}
#endif // VK_USE_PLATFORM_XCB_KHR
#if defined(VK_USE_PLATFORM_XCB_KHR)
void dump_json_xcb_visualid_t(const xcb_visualid_t object, const ApiDumpSettings& settings, int indents)
{
    if (settings.showAddress())
        settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
#endif // VK_USE_PLATFORM_XCB_KHR
#if defined(VK_USE_PLATFORM_XCB_KHR)
void dump_json_xcb_window_t(const xcb_window_t object, const ApiDumpSettings& settings, int indents)
{
    if (settings.showAddress())
        settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
#endif // VK_USE_PLATFORM_XCB_KHR
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_zx_handle_t(const zx_handle_t object, const ApiDumpSettings& settings, int indents)
{
    if (settings.showAddress())
        settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
#endif // VK_USE_PLATFORM_FUCHSIA

//========================== Handle Implementations =========================//

void dump_json_VkBuffer(const VkBuffer object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkImage(const VkImage object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkInstance(const VkInstance object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkPhysicalDevice(const VkPhysicalDevice object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkDevice(const VkDevice object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkQueue(const VkQueue object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkSemaphore(const VkSemaphore object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkCommandBuffer(const VkCommandBuffer object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkFence(const VkFence object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkDeviceMemory(const VkDeviceMemory object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkEvent(const VkEvent object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkQueryPool(const VkQueryPool object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkBufferView(const VkBufferView object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkImageView(const VkImageView object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkShaderModule(const VkShaderModule object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkPipelineCache(const VkPipelineCache object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkPipelineLayout(const VkPipelineLayout object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkPipeline(const VkPipeline object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkRenderPass(const VkRenderPass object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkDescriptorSetLayout(const VkDescriptorSetLayout object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkSampler(const VkSampler object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkDescriptorSet(const VkDescriptorSet object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkDescriptorPool(const VkDescriptorPool object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkFramebuffer(const VkFramebuffer object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkCommandPool(const VkCommandPool object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkSamplerYcbcrConversion(const VkSamplerYcbcrConversion object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkDescriptorUpdateTemplate(const VkDescriptorUpdateTemplate object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkPrivateDataSlot(const VkPrivateDataSlot object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkSurfaceKHR(const VkSurfaceKHR object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkSwapchainKHR(const VkSwapchainKHR object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkDisplayKHR(const VkDisplayKHR object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkDisplayModeKHR(const VkDisplayModeKHR object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkDebugReportCallbackEXT(const VkDebugReportCallbackEXT object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkVideoSessionKHR(const VkVideoSessionKHR object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkVideoSessionParametersKHR(const VkVideoSessionParametersKHR object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkCuModuleNVX(const VkCuModuleNVX object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkCuFunctionNVX(const VkCuFunctionNVX object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkDebugUtilsMessengerEXT(const VkDebugUtilsMessengerEXT object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkAccelerationStructureKHR(const VkAccelerationStructureKHR object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkDeferredOperationKHR(const VkDeferredOperationKHR object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkValidationCacheEXT(const VkValidationCacheEXT object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkAccelerationStructureNV(const VkAccelerationStructureNV object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkPerformanceConfigurationINTEL(const VkPerformanceConfigurationINTEL object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkIndirectCommandsLayoutNV(const VkIndirectCommandsLayoutNV object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkCudaModuleNV(const VkCudaModuleNV object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkCudaFunctionNV(const VkCudaFunctionNV object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkBufferCollectionFUCHSIA(const VkBufferCollectionFUCHSIA object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
#endif // VK_USE_PLATFORM_FUCHSIA
void dump_json_VkMicromapEXT(const VkMicromapEXT object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkOpticalFlowSessionNV(const VkOpticalFlowSessionNV object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}
void dump_json_VkShaderEXT(const VkShaderEXT object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress()) {
        settings.stream() << "\"" << object << "\"";
    } else {
        settings.stream() << "\"address\"";
    }
}

//=========================== Enum Implementations ==========================//

void dump_json_VkResult(VkResult object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SUCCESS\"";
        break;
    case 1:
        settings.stream() << "\"VK_NOT_READY\"";
        break;
    case 2:
        settings.stream() << "\"VK_TIMEOUT\"";
        break;
    case 3:
        settings.stream() << "\"VK_EVENT_SET\"";
        break;
    case 4:
        settings.stream() << "\"VK_EVENT_RESET\"";
        break;
    case 5:
        settings.stream() << "\"VK_INCOMPLETE\"";
        break;
    case -1:
        settings.stream() << "\"VK_ERROR_OUT_OF_HOST_MEMORY\"";
        break;
    case -2:
        settings.stream() << "\"VK_ERROR_OUT_OF_DEVICE_MEMORY\"";
        break;
    case -3:
        settings.stream() << "\"VK_ERROR_INITIALIZATION_FAILED\"";
        break;
    case -4:
        settings.stream() << "\"VK_ERROR_DEVICE_LOST\"";
        break;
    case -5:
        settings.stream() << "\"VK_ERROR_MEMORY_MAP_FAILED\"";
        break;
    case -6:
        settings.stream() << "\"VK_ERROR_LAYER_NOT_PRESENT\"";
        break;
    case -7:
        settings.stream() << "\"VK_ERROR_EXTENSION_NOT_PRESENT\"";
        break;
    case -8:
        settings.stream() << "\"VK_ERROR_FEATURE_NOT_PRESENT\"";
        break;
    case -9:
        settings.stream() << "\"VK_ERROR_INCOMPATIBLE_DRIVER\"";
        break;
    case -10:
        settings.stream() << "\"VK_ERROR_TOO_MANY_OBJECTS\"";
        break;
    case -11:
        settings.stream() << "\"VK_ERROR_FORMAT_NOT_SUPPORTED\"";
        break;
    case -12:
        settings.stream() << "\"VK_ERROR_FRAGMENTED_POOL\"";
        break;
    case -13:
        settings.stream() << "\"VK_ERROR_UNKNOWN\"";
        break;
    case -1000069000:
        settings.stream() << "\"VK_ERROR_OUT_OF_POOL_MEMORY\"";
        break;
    case -1000072003:
        settings.stream() << "\"VK_ERROR_INVALID_EXTERNAL_HANDLE\"";
        break;
    case -1000161000:
        settings.stream() << "\"VK_ERROR_FRAGMENTATION\"";
        break;
    case -1000257000:
        settings.stream() << "\"VK_ERROR_INVALID_OPAQUE_CAPTURE_ADDRESS\"";
        break;
    case 1000297000:
        settings.stream() << "\"VK_PIPELINE_COMPILE_REQUIRED\"";
        break;
    case -1000000000:
        settings.stream() << "\"VK_ERROR_SURFACE_LOST_KHR\"";
        break;
    case -1000000001:
        settings.stream() << "\"VK_ERROR_NATIVE_WINDOW_IN_USE_KHR\"";
        break;
    case 1000001003:
        settings.stream() << "\"VK_SUBOPTIMAL_KHR\"";
        break;
    case -1000001004:
        settings.stream() << "\"VK_ERROR_OUT_OF_DATE_KHR\"";
        break;
    case -1000003001:
        settings.stream() << "\"VK_ERROR_INCOMPATIBLE_DISPLAY_KHR\"";
        break;
    case -1000011001:
        settings.stream() << "\"VK_ERROR_VALIDATION_FAILED_EXT\"";
        break;
    case -1000012000:
        settings.stream() << "\"VK_ERROR_INVALID_SHADER_NV\"";
        break;
    case -1000023000:
        settings.stream() << "\"VK_ERROR_IMAGE_USAGE_NOT_SUPPORTED_KHR\"";
        break;
    case -1000023001:
        settings.stream() << "\"VK_ERROR_VIDEO_PICTURE_LAYOUT_NOT_SUPPORTED_KHR\"";
        break;
    case -1000023002:
        settings.stream() << "\"VK_ERROR_VIDEO_PROFILE_OPERATION_NOT_SUPPORTED_KHR\"";
        break;
    case -1000023003:
        settings.stream() << "\"VK_ERROR_VIDEO_PROFILE_FORMAT_NOT_SUPPORTED_KHR\"";
        break;
    case -1000023004:
        settings.stream() << "\"VK_ERROR_VIDEO_PROFILE_CODEC_NOT_SUPPORTED_KHR\"";
        break;
    case -1000023005:
        settings.stream() << "\"VK_ERROR_VIDEO_STD_VERSION_NOT_SUPPORTED_KHR\"";
        break;
    case -1000158000:
        settings.stream() << "\"VK_ERROR_INVALID_DRM_FORMAT_MODIFIER_PLANE_LAYOUT_EXT\"";
        break;
    case -1000174001:
        settings.stream() << "\"VK_ERROR_NOT_PERMITTED_KHR\"";
        break;
    case -1000255000:
        settings.stream() << "\"VK_ERROR_FULL_SCREEN_EXCLUSIVE_MODE_LOST_EXT\"";
        break;
    case 1000268000:
        settings.stream() << "\"VK_THREAD_IDLE_KHR\"";
        break;
    case 1000268001:
        settings.stream() << "\"VK_THREAD_DONE_KHR\"";
        break;
    case 1000268002:
        settings.stream() << "\"VK_OPERATION_DEFERRED_KHR\"";
        break;
    case 1000268003:
        settings.stream() << "\"VK_OPERATION_NOT_DEFERRED_KHR\"";
        break;
    case -1000299000:
        settings.stream() << "\"VK_ERROR_INVALID_VIDEO_STD_PARAMETERS_KHR\"";
        break;
    case -1000338000:
        settings.stream() << "\"VK_ERROR_COMPRESSION_EXHAUSTED_EXT\"";
        break;
    case 1000482000:
        settings.stream() << "\"VK_INCOMPATIBLE_SHADER_BINARY_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkStructureType(VkStructureType object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_STRUCTURE_TYPE_APPLICATION_INFO\"";
        break;
    case 1:
        settings.stream() << "\"VK_STRUCTURE_TYPE_INSTANCE_CREATE_INFO\"";
        break;
    case 2:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_QUEUE_CREATE_INFO\"";
        break;
    case 3:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_CREATE_INFO\"";
        break;
    case 4:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SUBMIT_INFO\"";
        break;
    case 5:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_ALLOCATE_INFO\"";
        break;
    case 6:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MAPPED_MEMORY_RANGE\"";
        break;
    case 7:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BIND_SPARSE_INFO\"";
        break;
    case 8:
        settings.stream() << "\"VK_STRUCTURE_TYPE_FENCE_CREATE_INFO\"";
        break;
    case 9:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SEMAPHORE_CREATE_INFO\"";
        break;
    case 10:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EVENT_CREATE_INFO\"";
        break;
    case 11:
        settings.stream() << "\"VK_STRUCTURE_TYPE_QUERY_POOL_CREATE_INFO\"";
        break;
    case 12:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_CREATE_INFO\"";
        break;
    case 13:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_VIEW_CREATE_INFO\"";
        break;
    case 14:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_CREATE_INFO\"";
        break;
    case 15:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_VIEW_CREATE_INFO\"";
        break;
    case 16:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SHADER_MODULE_CREATE_INFO\"";
        break;
    case 17:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_CACHE_CREATE_INFO\"";
        break;
    case 18:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO\"";
        break;
    case 19:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_VERTEX_INPUT_STATE_CREATE_INFO\"";
        break;
    case 20:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_INPUT_ASSEMBLY_STATE_CREATE_INFO\"";
        break;
    case 21:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_TESSELLATION_STATE_CREATE_INFO\"";
        break;
    case 22:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_STATE_CREATE_INFO\"";
        break;
    case 23:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_CREATE_INFO\"";
        break;
    case 24:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_MULTISAMPLE_STATE_CREATE_INFO\"";
        break;
    case 25:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_DEPTH_STENCIL_STATE_CREATE_INFO\"";
        break;
    case 26:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_COLOR_BLEND_STATE_CREATE_INFO\"";
        break;
    case 27:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_DYNAMIC_STATE_CREATE_INFO\"";
        break;
    case 28:
        settings.stream() << "\"VK_STRUCTURE_TYPE_GRAPHICS_PIPELINE_CREATE_INFO\"";
        break;
    case 29:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COMPUTE_PIPELINE_CREATE_INFO\"";
        break;
    case 30:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_LAYOUT_CREATE_INFO\"";
        break;
    case 31:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SAMPLER_CREATE_INFO\"";
        break;
    case 32:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_CREATE_INFO\"";
        break;
    case 33:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DESCRIPTOR_POOL_CREATE_INFO\"";
        break;
    case 34:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DESCRIPTOR_SET_ALLOCATE_INFO\"";
        break;
    case 35:
        settings.stream() << "\"VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET\"";
        break;
    case 36:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COPY_DESCRIPTOR_SET\"";
        break;
    case 37:
        settings.stream() << "\"VK_STRUCTURE_TYPE_FRAMEBUFFER_CREATE_INFO\"";
        break;
    case 38:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDER_PASS_CREATE_INFO\"";
        break;
    case 39:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COMMAND_POOL_CREATE_INFO\"";
        break;
    case 40:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COMMAND_BUFFER_ALLOCATE_INFO\"";
        break;
    case 41:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_INFO\"";
        break;
    case 42:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COMMAND_BUFFER_BEGIN_INFO\"";
        break;
    case 43:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDER_PASS_BEGIN_INFO\"";
        break;
    case 44:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_MEMORY_BARRIER\"";
        break;
    case 45:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER\"";
        break;
    case 46:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_BARRIER\"";
        break;
    case 47:
        settings.stream() << "\"VK_STRUCTURE_TYPE_LOADER_INSTANCE_CREATE_INFO\"";
        break;
    case 48:
        settings.stream() << "\"VK_STRUCTURE_TYPE_LOADER_DEVICE_CREATE_INFO\"";
        break;
    case 1000094000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBGROUP_PROPERTIES\"";
        break;
    case 1000157000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BIND_BUFFER_MEMORY_INFO\"";
        break;
    case 1000157001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BIND_IMAGE_MEMORY_INFO\"";
        break;
    case 1000083000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_16BIT_STORAGE_FEATURES\"";
        break;
    case 1000127000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_DEDICATED_REQUIREMENTS\"";
        break;
    case 1000127001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_DEDICATED_ALLOCATE_INFO\"";
        break;
    case 1000060000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_ALLOCATE_FLAGS_INFO\"";
        break;
    case 1000060003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_GROUP_RENDER_PASS_BEGIN_INFO\"";
        break;
    case 1000060004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_GROUP_COMMAND_BUFFER_BEGIN_INFO\"";
        break;
    case 1000060005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_GROUP_SUBMIT_INFO\"";
        break;
    case 1000060006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_GROUP_BIND_SPARSE_INFO\"";
        break;
    case 1000060013:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BIND_BUFFER_MEMORY_DEVICE_GROUP_INFO\"";
        break;
    case 1000060014:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BIND_IMAGE_MEMORY_DEVICE_GROUP_INFO\"";
        break;
    case 1000070000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_GROUP_PROPERTIES\"";
        break;
    case 1000070001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_GROUP_DEVICE_CREATE_INFO\"";
        break;
    case 1000146000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_MEMORY_REQUIREMENTS_INFO_2\"";
        break;
    case 1000146001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_MEMORY_REQUIREMENTS_INFO_2\"";
        break;
    case 1000146002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_SPARSE_MEMORY_REQUIREMENTS_INFO_2\"";
        break;
    case 1000146003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_REQUIREMENTS_2\"";
        break;
    case 1000146004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SPARSE_IMAGE_MEMORY_REQUIREMENTS_2\"";
        break;
    case 1000059000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FEATURES_2\"";
        break;
    case 1000059001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROPERTIES_2\"";
        break;
    case 1000059002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_FORMAT_PROPERTIES_2\"";
        break;
    case 1000059003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_FORMAT_PROPERTIES_2\"";
        break;
    case 1000059004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_FORMAT_INFO_2\"";
        break;
    case 1000059005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_QUEUE_FAMILY_PROPERTIES_2\"";
        break;
    case 1000059006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_PROPERTIES_2\"";
        break;
    case 1000059007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SPARSE_IMAGE_FORMAT_PROPERTIES_2\"";
        break;
    case 1000059008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SPARSE_IMAGE_FORMAT_INFO_2\"";
        break;
    case 1000117000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_POINT_CLIPPING_PROPERTIES\"";
        break;
    case 1000117001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDER_PASS_INPUT_ATTACHMENT_ASPECT_CREATE_INFO\"";
        break;
    case 1000117002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_VIEW_USAGE_CREATE_INFO\"";
        break;
    case 1000117003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_TESSELLATION_DOMAIN_ORIGIN_STATE_CREATE_INFO\"";
        break;
    case 1000053000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDER_PASS_MULTIVIEW_CREATE_INFO\"";
        break;
    case 1000053001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTIVIEW_FEATURES\"";
        break;
    case 1000053002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTIVIEW_PROPERTIES\"";
        break;
    case 1000120000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VARIABLE_POINTERS_FEATURES\"";
        break;
    case 1000145000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PROTECTED_SUBMIT_INFO\"";
        break;
    case 1000145001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROTECTED_MEMORY_FEATURES\"";
        break;
    case 1000145002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROTECTED_MEMORY_PROPERTIES\"";
        break;
    case 1000145003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_QUEUE_INFO_2\"";
        break;
    case 1000156000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SAMPLER_YCBCR_CONVERSION_CREATE_INFO\"";
        break;
    case 1000156001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SAMPLER_YCBCR_CONVERSION_INFO\"";
        break;
    case 1000156002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BIND_IMAGE_PLANE_MEMORY_INFO\"";
        break;
    case 1000156003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_PLANE_MEMORY_REQUIREMENTS_INFO\"";
        break;
    case 1000156004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SAMPLER_YCBCR_CONVERSION_FEATURES\"";
        break;
    case 1000156005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SAMPLER_YCBCR_CONVERSION_IMAGE_FORMAT_PROPERTIES\"";
        break;
    case 1000085000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DESCRIPTOR_UPDATE_TEMPLATE_CREATE_INFO\"";
        break;
    case 1000071000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_IMAGE_FORMAT_INFO\"";
        break;
    case 1000071001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXTERNAL_IMAGE_FORMAT_PROPERTIES\"";
        break;
    case 1000071002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_BUFFER_INFO\"";
        break;
    case 1000071003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXTERNAL_BUFFER_PROPERTIES\"";
        break;
    case 1000071004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ID_PROPERTIES\"";
        break;
    case 1000072000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXTERNAL_MEMORY_BUFFER_CREATE_INFO\"";
        break;
    case 1000072001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXTERNAL_MEMORY_IMAGE_CREATE_INFO\"";
        break;
    case 1000072002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_MEMORY_ALLOCATE_INFO\"";
        break;
    case 1000112000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_FENCE_INFO\"";
        break;
    case 1000112001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXTERNAL_FENCE_PROPERTIES\"";
        break;
    case 1000113000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_FENCE_CREATE_INFO\"";
        break;
    case 1000077000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_SEMAPHORE_CREATE_INFO\"";
        break;
    case 1000076000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_SEMAPHORE_INFO\"";
        break;
    case 1000076001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXTERNAL_SEMAPHORE_PROPERTIES\"";
        break;
    case 1000168000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_3_PROPERTIES\"";
        break;
    case 1000168001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_SUPPORT\"";
        break;
    case 1000063000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_DRAW_PARAMETERS_FEATURES\"";
        break;
    case 49:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_1_FEATURES\"";
        break;
    case 50:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_1_PROPERTIES\"";
        break;
    case 51:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_2_FEATURES\"";
        break;
    case 52:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_2_PROPERTIES\"";
        break;
    case 1000147000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_FORMAT_LIST_CREATE_INFO\"";
        break;
    case 1000109000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ATTACHMENT_DESCRIPTION_2\"";
        break;
    case 1000109001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ATTACHMENT_REFERENCE_2\"";
        break;
    case 1000109002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SUBPASS_DESCRIPTION_2\"";
        break;
    case 1000109003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SUBPASS_DEPENDENCY_2\"";
        break;
    case 1000109004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDER_PASS_CREATE_INFO_2\"";
        break;
    case 1000109005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SUBPASS_BEGIN_INFO\"";
        break;
    case 1000109006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SUBPASS_END_INFO\"";
        break;
    case 1000177000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_8BIT_STORAGE_FEATURES\"";
        break;
    case 1000196000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DRIVER_PROPERTIES\"";
        break;
    case 1000180000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_ATOMIC_INT64_FEATURES\"";
        break;
    case 1000082000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_FLOAT16_INT8_FEATURES\"";
        break;
    case 1000197000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FLOAT_CONTROLS_PROPERTIES\"";
        break;
    case 1000161000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_BINDING_FLAGS_CREATE_INFO\"";
        break;
    case 1000161001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_INDEXING_FEATURES\"";
        break;
    case 1000161002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_INDEXING_PROPERTIES\"";
        break;
    case 1000161003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DESCRIPTOR_SET_VARIABLE_DESCRIPTOR_COUNT_ALLOCATE_INFO\"";
        break;
    case 1000161004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DESCRIPTOR_SET_VARIABLE_DESCRIPTOR_COUNT_LAYOUT_SUPPORT\"";
        break;
    case 1000199000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_STENCIL_RESOLVE_PROPERTIES\"";
        break;
    case 1000199001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SUBPASS_DESCRIPTION_DEPTH_STENCIL_RESOLVE\"";
        break;
    case 1000221000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SCALAR_BLOCK_LAYOUT_FEATURES\"";
        break;
    case 1000246000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_STENCIL_USAGE_CREATE_INFO\"";
        break;
    case 1000130000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SAMPLER_FILTER_MINMAX_PROPERTIES\"";
        break;
    case 1000130001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SAMPLER_REDUCTION_MODE_CREATE_INFO\"";
        break;
    case 1000211000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_MEMORY_MODEL_FEATURES\"";
        break;
    case 1000108000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGELESS_FRAMEBUFFER_FEATURES\"";
        break;
    case 1000108001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_FRAMEBUFFER_ATTACHMENTS_CREATE_INFO\"";
        break;
    case 1000108002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_FRAMEBUFFER_ATTACHMENT_IMAGE_INFO\"";
        break;
    case 1000108003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDER_PASS_ATTACHMENT_BEGIN_INFO\"";
        break;
    case 1000253000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_UNIFORM_BUFFER_STANDARD_LAYOUT_FEATURES\"";
        break;
    case 1000175000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_SUBGROUP_EXTENDED_TYPES_FEATURES\"";
        break;
    case 1000241000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SEPARATE_DEPTH_STENCIL_LAYOUTS_FEATURES\"";
        break;
    case 1000241001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ATTACHMENT_REFERENCE_STENCIL_LAYOUT\"";
        break;
    case 1000241002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ATTACHMENT_DESCRIPTION_STENCIL_LAYOUT\"";
        break;
    case 1000261000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_HOST_QUERY_RESET_FEATURES\"";
        break;
    case 1000207000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TIMELINE_SEMAPHORE_FEATURES\"";
        break;
    case 1000207001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TIMELINE_SEMAPHORE_PROPERTIES\"";
        break;
    case 1000207002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SEMAPHORE_TYPE_CREATE_INFO\"";
        break;
    case 1000207003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_TIMELINE_SEMAPHORE_SUBMIT_INFO\"";
        break;
    case 1000207004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SEMAPHORE_WAIT_INFO\"";
        break;
    case 1000207005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SEMAPHORE_SIGNAL_INFO\"";
        break;
    case 1000257000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_BUFFER_DEVICE_ADDRESS_FEATURES\"";
        break;
    case 1000244001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_DEVICE_ADDRESS_INFO\"";
        break;
    case 1000257002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_OPAQUE_CAPTURE_ADDRESS_CREATE_INFO\"";
        break;
    case 1000257003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_OPAQUE_CAPTURE_ADDRESS_ALLOCATE_INFO\"";
        break;
    case 1000257004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_MEMORY_OPAQUE_CAPTURE_ADDRESS_INFO\"";
        break;
    case 53:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_3_FEATURES\"";
        break;
    case 54:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_3_PROPERTIES\"";
        break;
    case 1000192000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_CREATION_FEEDBACK_CREATE_INFO\"";
        break;
    case 1000215000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_TERMINATE_INVOCATION_FEATURES\"";
        break;
    case 1000245000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TOOL_PROPERTIES\"";
        break;
    case 1000276000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_DEMOTE_TO_HELPER_INVOCATION_FEATURES\"";
        break;
    case 1000295000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRIVATE_DATA_FEATURES\"";
        break;
    case 1000295001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_PRIVATE_DATA_CREATE_INFO\"";
        break;
    case 1000295002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PRIVATE_DATA_SLOT_CREATE_INFO\"";
        break;
    case 1000297000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PIPELINE_CREATION_CACHE_CONTROL_FEATURES\"";
        break;
    case 1000314000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_BARRIER_2\"";
        break;
    case 1000314001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_MEMORY_BARRIER_2\"";
        break;
    case 1000314002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER_2\"";
        break;
    case 1000314003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEPENDENCY_INFO\"";
        break;
    case 1000314004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SUBMIT_INFO_2\"";
        break;
    case 1000314005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SEMAPHORE_SUBMIT_INFO\"";
        break;
    case 1000314006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COMMAND_BUFFER_SUBMIT_INFO\"";
        break;
    case 1000314007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SYNCHRONIZATION_2_FEATURES\"";
        break;
    case 1000325000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ZERO_INITIALIZE_WORKGROUP_MEMORY_FEATURES\"";
        break;
    case 1000335000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_ROBUSTNESS_FEATURES\"";
        break;
    case 1000337000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COPY_BUFFER_INFO_2\"";
        break;
    case 1000337001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COPY_IMAGE_INFO_2\"";
        break;
    case 1000337002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COPY_BUFFER_TO_IMAGE_INFO_2\"";
        break;
    case 1000337003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COPY_IMAGE_TO_BUFFER_INFO_2\"";
        break;
    case 1000337004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BLIT_IMAGE_INFO_2\"";
        break;
    case 1000337005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RESOLVE_IMAGE_INFO_2\"";
        break;
    case 1000337006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_COPY_2\"";
        break;
    case 1000337007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_COPY_2\"";
        break;
    case 1000337008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_BLIT_2\"";
        break;
    case 1000337009:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_IMAGE_COPY_2\"";
        break;
    case 1000337010:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_RESOLVE_2\"";
        break;
    case 1000225000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBGROUP_SIZE_CONTROL_PROPERTIES\"";
        break;
    case 1000225001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_REQUIRED_SUBGROUP_SIZE_CREATE_INFO\"";
        break;
    case 1000225002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBGROUP_SIZE_CONTROL_FEATURES\"";
        break;
    case 1000138000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_INLINE_UNIFORM_BLOCK_FEATURES\"";
        break;
    case 1000138001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_INLINE_UNIFORM_BLOCK_PROPERTIES\"";
        break;
    case 1000138002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET_INLINE_UNIFORM_BLOCK\"";
        break;
    case 1000138003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DESCRIPTOR_POOL_INLINE_UNIFORM_BLOCK_CREATE_INFO\"";
        break;
    case 1000066000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TEXTURE_COMPRESSION_ASTC_HDR_FEATURES\"";
        break;
    case 1000044000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDERING_INFO\"";
        break;
    case 1000044001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDERING_ATTACHMENT_INFO\"";
        break;
    case 1000044002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_RENDERING_CREATE_INFO\"";
        break;
    case 1000044003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DYNAMIC_RENDERING_FEATURES\"";
        break;
    case 1000044004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_RENDERING_INFO\"";
        break;
    case 1000280000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_INTEGER_DOT_PRODUCT_FEATURES\"";
        break;
    case 1000280001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_INTEGER_DOT_PRODUCT_PROPERTIES\"";
        break;
    case 1000281001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TEXEL_BUFFER_ALIGNMENT_PROPERTIES\"";
        break;
    case 1000360000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_FORMAT_PROPERTIES_3\"";
        break;
    case 1000413000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_4_FEATURES\"";
        break;
    case 1000413001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_4_PROPERTIES\"";
        break;
    case 1000413002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_BUFFER_MEMORY_REQUIREMENTS\"";
        break;
    case 1000413003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_IMAGE_MEMORY_REQUIREMENTS\"";
        break;
    case 1000001000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SWAPCHAIN_CREATE_INFO_KHR\"";
        break;
    case 1000001001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PRESENT_INFO_KHR\"";
        break;
    case 1000060007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_GROUP_PRESENT_CAPABILITIES_KHR\"";
        break;
    case 1000060008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_SWAPCHAIN_CREATE_INFO_KHR\"";
        break;
    case 1000060009:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BIND_IMAGE_MEMORY_SWAPCHAIN_INFO_KHR\"";
        break;
    case 1000060010:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACQUIRE_NEXT_IMAGE_INFO_KHR\"";
        break;
    case 1000060011:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_GROUP_PRESENT_INFO_KHR\"";
        break;
    case 1000060012:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_GROUP_SWAPCHAIN_CREATE_INFO_KHR\"";
        break;
    case 1000002000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DISPLAY_MODE_CREATE_INFO_KHR\"";
        break;
    case 1000002001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DISPLAY_SURFACE_CREATE_INFO_KHR\"";
        break;
    case 1000003000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DISPLAY_PRESENT_INFO_KHR\"";
        break;
    case 1000004000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_XLIB_SURFACE_CREATE_INFO_KHR\"";
        break;
    case 1000005000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_XCB_SURFACE_CREATE_INFO_KHR\"";
        break;
    case 1000006000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_WAYLAND_SURFACE_CREATE_INFO_KHR\"";
        break;
    case 1000008000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ANDROID_SURFACE_CREATE_INFO_KHR\"";
        break;
    case 1000009000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_WIN32_SURFACE_CREATE_INFO_KHR\"";
        break;
    case 1000010000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_NATIVE_BUFFER_ANDROID\"";
        break;
    case 1000010001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SWAPCHAIN_IMAGE_CREATE_INFO_ANDROID\"";
        break;
    case 1000010002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRESENTATION_PROPERTIES_ANDROID\"";
        break;
    case 1000011000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEBUG_REPORT_CALLBACK_CREATE_INFO_EXT\"";
        break;
    case 1000018000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_RASTERIZATION_ORDER_AMD\"";
        break;
    case 1000022000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEBUG_MARKER_OBJECT_NAME_INFO_EXT\"";
        break;
    case 1000022001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEBUG_MARKER_OBJECT_TAG_INFO_EXT\"";
        break;
    case 1000022002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEBUG_MARKER_MARKER_INFO_EXT\"";
        break;
    case 1000023000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_PROFILE_INFO_KHR\"";
        break;
    case 1000023001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_CAPABILITIES_KHR\"";
        break;
    case 1000023002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_PICTURE_RESOURCE_INFO_KHR\"";
        break;
    case 1000023003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_SESSION_MEMORY_REQUIREMENTS_KHR\"";
        break;
    case 1000023004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BIND_VIDEO_SESSION_MEMORY_INFO_KHR\"";
        break;
    case 1000023005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_SESSION_CREATE_INFO_KHR\"";
        break;
    case 1000023006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_SESSION_PARAMETERS_CREATE_INFO_KHR\"";
        break;
    case 1000023007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_SESSION_PARAMETERS_UPDATE_INFO_KHR\"";
        break;
    case 1000023008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_BEGIN_CODING_INFO_KHR\"";
        break;
    case 1000023009:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_END_CODING_INFO_KHR\"";
        break;
    case 1000023010:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_CODING_CONTROL_INFO_KHR\"";
        break;
    case 1000023011:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_REFERENCE_SLOT_INFO_KHR\"";
        break;
    case 1000023012:
        settings.stream() << "\"VK_STRUCTURE_TYPE_QUEUE_FAMILY_VIDEO_PROPERTIES_KHR\"";
        break;
    case 1000023013:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_PROFILE_LIST_INFO_KHR\"";
        break;
    case 1000023014:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VIDEO_FORMAT_INFO_KHR\"";
        break;
    case 1000023015:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_FORMAT_PROPERTIES_KHR\"";
        break;
    case 1000023016:
        settings.stream() << "\"VK_STRUCTURE_TYPE_QUEUE_FAMILY_QUERY_RESULT_STATUS_PROPERTIES_KHR\"";
        break;
    case 1000024000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_INFO_KHR\"";
        break;
    case 1000024001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_CAPABILITIES_KHR\"";
        break;
    case 1000024002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_USAGE_INFO_KHR\"";
        break;
    case 1000026000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEDICATED_ALLOCATION_IMAGE_CREATE_INFO_NV\"";
        break;
    case 1000026001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEDICATED_ALLOCATION_BUFFER_CREATE_INFO_NV\"";
        break;
    case 1000026002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEDICATED_ALLOCATION_MEMORY_ALLOCATE_INFO_NV\"";
        break;
    case 1000028000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TRANSFORM_FEEDBACK_FEATURES_EXT\"";
        break;
    case 1000028001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TRANSFORM_FEEDBACK_PROPERTIES_EXT\"";
        break;
    case 1000028002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_STREAM_CREATE_INFO_EXT\"";
        break;
    case 1000029000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_CU_MODULE_CREATE_INFO_NVX\"";
        break;
    case 1000029001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_CU_FUNCTION_CREATE_INFO_NVX\"";
        break;
    case 1000029002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_CU_LAUNCH_INFO_NVX\"";
        break;
    case 1000030000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_VIEW_HANDLE_INFO_NVX\"";
        break;
    case 1000030001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_VIEW_ADDRESS_PROPERTIES_NVX\"";
        break;
    case 1000038000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H264_CAPABILITIES_KHR\"";
        break;
    case 1000038001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H264_SESSION_PARAMETERS_CREATE_INFO_KHR\"";
        break;
    case 1000038002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H264_SESSION_PARAMETERS_ADD_INFO_KHR\"";
        break;
    case 1000038003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H264_PICTURE_INFO_KHR\"";
        break;
    case 1000038004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H264_DPB_SLOT_INFO_KHR\"";
        break;
    case 1000038005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H264_NALU_SLICE_INFO_KHR\"";
        break;
    case 1000038006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H264_GOP_REMAINING_FRAME_INFO_KHR\"";
        break;
    case 1000038007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H264_PROFILE_INFO_KHR\"";
        break;
    case 1000038008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H264_RATE_CONTROL_INFO_KHR\"";
        break;
    case 1000038009:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H264_RATE_CONTROL_LAYER_INFO_KHR\"";
        break;
    case 1000038010:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H264_SESSION_CREATE_INFO_KHR\"";
        break;
    case 1000038011:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H264_QUALITY_LEVEL_PROPERTIES_KHR\"";
        break;
    case 1000038012:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H264_SESSION_PARAMETERS_GET_INFO_KHR\"";
        break;
    case 1000038013:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H264_SESSION_PARAMETERS_FEEDBACK_INFO_KHR\"";
        break;
    case 1000039000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H265_CAPABILITIES_KHR\"";
        break;
    case 1000039001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H265_SESSION_PARAMETERS_CREATE_INFO_KHR\"";
        break;
    case 1000039002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H265_SESSION_PARAMETERS_ADD_INFO_KHR\"";
        break;
    case 1000039003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H265_PICTURE_INFO_KHR\"";
        break;
    case 1000039004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H265_DPB_SLOT_INFO_KHR\"";
        break;
    case 1000039005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H265_NALU_SLICE_SEGMENT_INFO_KHR\"";
        break;
    case 1000039006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H265_GOP_REMAINING_FRAME_INFO_KHR\"";
        break;
    case 1000039007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H265_PROFILE_INFO_KHR\"";
        break;
    case 1000039009:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H265_RATE_CONTROL_INFO_KHR\"";
        break;
    case 1000039010:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H265_RATE_CONTROL_LAYER_INFO_KHR\"";
        break;
    case 1000039011:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H265_SESSION_CREATE_INFO_KHR\"";
        break;
    case 1000039012:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H265_QUALITY_LEVEL_PROPERTIES_KHR\"";
        break;
    case 1000039013:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H265_SESSION_PARAMETERS_GET_INFO_KHR\"";
        break;
    case 1000039014:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_H265_SESSION_PARAMETERS_FEEDBACK_INFO_KHR\"";
        break;
    case 1000040000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_H264_CAPABILITIES_KHR\"";
        break;
    case 1000040001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_H264_PICTURE_INFO_KHR\"";
        break;
    case 1000040003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_H264_PROFILE_INFO_KHR\"";
        break;
    case 1000040004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_H264_SESSION_PARAMETERS_CREATE_INFO_KHR\"";
        break;
    case 1000040005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_H264_SESSION_PARAMETERS_ADD_INFO_KHR\"";
        break;
    case 1000040006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_H264_DPB_SLOT_INFO_KHR\"";
        break;
    case 1000041000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_TEXTURE_LOD_GATHER_FORMAT_PROPERTIES_AMD\"";
        break;
    case 1000044006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDERING_FRAGMENT_SHADING_RATE_ATTACHMENT_INFO_KHR\"";
        break;
    case 1000044007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDERING_FRAGMENT_DENSITY_MAP_ATTACHMENT_INFO_EXT\"";
        break;
    case 1000044008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ATTACHMENT_SAMPLE_COUNT_INFO_AMD\"";
        break;
    case 1000044009:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MULTIVIEW_PER_VIEW_ATTRIBUTES_INFO_NVX\"";
        break;
    case 1000049000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_STREAM_DESCRIPTOR_SURFACE_CREATE_INFO_GGP\"";
        break;
    case 1000050000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CORNER_SAMPLED_IMAGE_FEATURES_NV\"";
        break;
    case 1000051000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PRIVATE_VENDOR_INFO_PLACEHOLDER_OFFSET_0_NV\"";
        break;
    case 1000056000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXTERNAL_MEMORY_IMAGE_CREATE_INFO_NV\"";
        break;
    case 1000056001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_MEMORY_ALLOCATE_INFO_NV\"";
        break;
    case 1000057000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_MEMORY_WIN32_HANDLE_INFO_NV\"";
        break;
    case 1000057001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_MEMORY_WIN32_HANDLE_INFO_NV\"";
        break;
    case 1000058000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_WIN32_KEYED_MUTEX_ACQUIRE_RELEASE_INFO_NV\"";
        break;
    case 1000061000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VALIDATION_FLAGS_EXT\"";
        break;
    case 1000062000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VI_SURFACE_CREATE_INFO_NN\"";
        break;
    case 1000067000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_VIEW_ASTC_DECODE_MODE_EXT\"";
        break;
    case 1000067001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ASTC_DECODE_FEATURES_EXT\"";
        break;
    case 1000068000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_ROBUSTNESS_CREATE_INFO_EXT\"";
        break;
    case 1000068001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PIPELINE_ROBUSTNESS_FEATURES_EXT\"";
        break;
    case 1000068002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PIPELINE_ROBUSTNESS_PROPERTIES_EXT\"";
        break;
    case 1000073000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_MEMORY_WIN32_HANDLE_INFO_KHR\"";
        break;
    case 1000073001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_MEMORY_WIN32_HANDLE_INFO_KHR\"";
        break;
    case 1000073002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_WIN32_HANDLE_PROPERTIES_KHR\"";
        break;
    case 1000073003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_GET_WIN32_HANDLE_INFO_KHR\"";
        break;
    case 1000074000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_MEMORY_FD_INFO_KHR\"";
        break;
    case 1000074001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_FD_PROPERTIES_KHR\"";
        break;
    case 1000074002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_GET_FD_INFO_KHR\"";
        break;
    case 1000075000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_WIN32_KEYED_MUTEX_ACQUIRE_RELEASE_INFO_KHR\"";
        break;
    case 1000078000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_SEMAPHORE_WIN32_HANDLE_INFO_KHR\"";
        break;
    case 1000078001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_SEMAPHORE_WIN32_HANDLE_INFO_KHR\"";
        break;
    case 1000078002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_D3D12_FENCE_SUBMIT_INFO_KHR\"";
        break;
    case 1000078003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SEMAPHORE_GET_WIN32_HANDLE_INFO_KHR\"";
        break;
    case 1000079000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_SEMAPHORE_FD_INFO_KHR\"";
        break;
    case 1000079001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SEMAPHORE_GET_FD_INFO_KHR\"";
        break;
    case 1000080000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PUSH_DESCRIPTOR_PROPERTIES_KHR\"";
        break;
    case 1000081000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_CONDITIONAL_RENDERING_INFO_EXT\"";
        break;
    case 1000081001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CONDITIONAL_RENDERING_FEATURES_EXT\"";
        break;
    case 1000081002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_CONDITIONAL_RENDERING_BEGIN_INFO_EXT\"";
        break;
    case 1000084000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PRESENT_REGIONS_KHR\"";
        break;
    case 1000087000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_W_SCALING_STATE_CREATE_INFO_NV\"";
        break;
    case 1000090000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SURFACE_CAPABILITIES_2_EXT\"";
        break;
    case 1000091000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DISPLAY_POWER_INFO_EXT\"";
        break;
    case 1000091001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_EVENT_INFO_EXT\"";
        break;
    case 1000091002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DISPLAY_EVENT_INFO_EXT\"";
        break;
    case 1000091003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SWAPCHAIN_COUNTER_CREATE_INFO_EXT\"";
        break;
    case 1000092000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PRESENT_TIMES_INFO_GOOGLE\"";
        break;
    case 1000097000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTIVIEW_PER_VIEW_ATTRIBUTES_PROPERTIES_NVX\"";
        break;
    case 1000098000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_SWIZZLE_STATE_CREATE_INFO_NV\"";
        break;
    case 1000099000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DISCARD_RECTANGLE_PROPERTIES_EXT\"";
        break;
    case 1000099001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_DISCARD_RECTANGLE_STATE_CREATE_INFO_EXT\"";
        break;
    case 1000101000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CONSERVATIVE_RASTERIZATION_PROPERTIES_EXT\"";
        break;
    case 1000101001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_CONSERVATIVE_STATE_CREATE_INFO_EXT\"";
        break;
    case 1000102000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_CLIP_ENABLE_FEATURES_EXT\"";
        break;
    case 1000102001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_DEPTH_CLIP_STATE_CREATE_INFO_EXT\"";
        break;
    case 1000105000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_HDR_METADATA_EXT\"";
        break;
    case 1000110000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RELAXED_LINE_RASTERIZATION_FEATURES_IMG\"";
        break;
    case 1000111000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SHARED_PRESENT_SURFACE_CAPABILITIES_KHR\"";
        break;
    case 1000114000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_FENCE_WIN32_HANDLE_INFO_KHR\"";
        break;
    case 1000114001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_FENCE_WIN32_HANDLE_INFO_KHR\"";
        break;
    case 1000114002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_FENCE_GET_WIN32_HANDLE_INFO_KHR\"";
        break;
    case 1000115000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_FENCE_FD_INFO_KHR\"";
        break;
    case 1000115001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_FENCE_GET_FD_INFO_KHR\"";
        break;
    case 1000116000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PERFORMANCE_QUERY_FEATURES_KHR\"";
        break;
    case 1000116001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PERFORMANCE_QUERY_PROPERTIES_KHR\"";
        break;
    case 1000116002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_QUERY_POOL_PERFORMANCE_CREATE_INFO_KHR\"";
        break;
    case 1000116003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PERFORMANCE_QUERY_SUBMIT_INFO_KHR\"";
        break;
    case 1000116004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACQUIRE_PROFILING_LOCK_INFO_KHR\"";
        break;
    case 1000116005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PERFORMANCE_COUNTER_KHR\"";
        break;
    case 1000116006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PERFORMANCE_COUNTER_DESCRIPTION_KHR\"";
        break;
    case 1000119000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SURFACE_INFO_2_KHR\"";
        break;
    case 1000119001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SURFACE_CAPABILITIES_2_KHR\"";
        break;
    case 1000119002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SURFACE_FORMAT_2_KHR\"";
        break;
    case 1000121000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DISPLAY_PROPERTIES_2_KHR\"";
        break;
    case 1000121001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DISPLAY_PLANE_PROPERTIES_2_KHR\"";
        break;
    case 1000121002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DISPLAY_MODE_PROPERTIES_2_KHR\"";
        break;
    case 1000121003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DISPLAY_PLANE_INFO_2_KHR\"";
        break;
    case 1000121004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DISPLAY_PLANE_CAPABILITIES_2_KHR\"";
        break;
    case 1000122000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IOS_SURFACE_CREATE_INFO_MVK\"";
        break;
    case 1000123000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MACOS_SURFACE_CREATE_INFO_MVK\"";
        break;
    case 1000128000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEBUG_UTILS_OBJECT_NAME_INFO_EXT\"";
        break;
    case 1000128001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEBUG_UTILS_OBJECT_TAG_INFO_EXT\"";
        break;
    case 1000128002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEBUG_UTILS_LABEL_EXT\"";
        break;
    case 1000128003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEBUG_UTILS_MESSENGER_CALLBACK_DATA_EXT\"";
        break;
    case 1000128004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEBUG_UTILS_MESSENGER_CREATE_INFO_EXT\"";
        break;
    case 1000129000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ANDROID_HARDWARE_BUFFER_USAGE_ANDROID\"";
        break;
    case 1000129001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ANDROID_HARDWARE_BUFFER_PROPERTIES_ANDROID\"";
        break;
    case 1000129002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ANDROID_HARDWARE_BUFFER_FORMAT_PROPERTIES_ANDROID\"";
        break;
    case 1000129003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_ANDROID_HARDWARE_BUFFER_INFO_ANDROID\"";
        break;
    case 1000129004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_GET_ANDROID_HARDWARE_BUFFER_INFO_ANDROID\"";
        break;
    case 1000129005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXTERNAL_FORMAT_ANDROID\"";
        break;
    case 1000129006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ANDROID_HARDWARE_BUFFER_FORMAT_PROPERTIES_2_ANDROID\"";
        break;
    case 1000134000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_ENQUEUE_FEATURES_AMDX\"";
        break;
    case 1000134001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_ENQUEUE_PROPERTIES_AMDX\"";
        break;
    case 1000134002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXECUTION_GRAPH_PIPELINE_SCRATCH_SIZE_AMDX\"";
        break;
    case 1000134003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXECUTION_GRAPH_PIPELINE_CREATE_INFO_AMDX\"";
        break;
    case 1000134004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_NODE_CREATE_INFO_AMDX\"";
        break;
    case 1000143000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SAMPLE_LOCATIONS_INFO_EXT\"";
        break;
    case 1000143001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDER_PASS_SAMPLE_LOCATIONS_BEGIN_INFO_EXT\"";
        break;
    case 1000143002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_SAMPLE_LOCATIONS_STATE_CREATE_INFO_EXT\"";
        break;
    case 1000143003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SAMPLE_LOCATIONS_PROPERTIES_EXT\"";
        break;
    case 1000143004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MULTISAMPLE_PROPERTIES_EXT\"";
        break;
    case 1000148000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_BLEND_OPERATION_ADVANCED_FEATURES_EXT\"";
        break;
    case 1000148001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_BLEND_OPERATION_ADVANCED_PROPERTIES_EXT\"";
        break;
    case 1000148002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_COLOR_BLEND_ADVANCED_STATE_CREATE_INFO_EXT\"";
        break;
    case 1000149000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_COVERAGE_TO_COLOR_STATE_CREATE_INFO_NV\"";
        break;
    case 1000150007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET_ACCELERATION_STRUCTURE_KHR\"";
        break;
    case 1000150000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_BUILD_GEOMETRY_INFO_KHR\"";
        break;
    case 1000150002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_DEVICE_ADDRESS_INFO_KHR\"";
        break;
    case 1000150003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_AABBS_DATA_KHR\"";
        break;
    case 1000150004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_INSTANCES_DATA_KHR\"";
        break;
    case 1000150005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_TRIANGLES_DATA_KHR\"";
        break;
    case 1000150006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_KHR\"";
        break;
    case 1000150009:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_VERSION_INFO_KHR\"";
        break;
    case 1000150010:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COPY_ACCELERATION_STRUCTURE_INFO_KHR\"";
        break;
    case 1000150011:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COPY_ACCELERATION_STRUCTURE_TO_MEMORY_INFO_KHR\"";
        break;
    case 1000150012:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COPY_MEMORY_TO_ACCELERATION_STRUCTURE_INFO_KHR\"";
        break;
    case 1000150013:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ACCELERATION_STRUCTURE_FEATURES_KHR\"";
        break;
    case 1000150014:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ACCELERATION_STRUCTURE_PROPERTIES_KHR\"";
        break;
    case 1000150017:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_CREATE_INFO_KHR\"";
        break;
    case 1000150020:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_BUILD_SIZES_INFO_KHR\"";
        break;
    case 1000347000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_PIPELINE_FEATURES_KHR\"";
        break;
    case 1000347001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_PIPELINE_PROPERTIES_KHR\"";
        break;
    case 1000150015:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RAY_TRACING_PIPELINE_CREATE_INFO_KHR\"";
        break;
    case 1000150016:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RAY_TRACING_SHADER_GROUP_CREATE_INFO_KHR\"";
        break;
    case 1000150018:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RAY_TRACING_PIPELINE_INTERFACE_CREATE_INFO_KHR\"";
        break;
    case 1000348013:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_QUERY_FEATURES_KHR\"";
        break;
    case 1000152000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_COVERAGE_MODULATION_STATE_CREATE_INFO_NV\"";
        break;
    case 1000154000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_SM_BUILTINS_FEATURES_NV\"";
        break;
    case 1000154001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_SM_BUILTINS_PROPERTIES_NV\"";
        break;
    case 1000158000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DRM_FORMAT_MODIFIER_PROPERTIES_LIST_EXT\"";
        break;
    case 1000158002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_DRM_FORMAT_MODIFIER_INFO_EXT\"";
        break;
    case 1000158003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_DRM_FORMAT_MODIFIER_LIST_CREATE_INFO_EXT\"";
        break;
    case 1000158004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_DRM_FORMAT_MODIFIER_EXPLICIT_CREATE_INFO_EXT\"";
        break;
    case 1000158005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_DRM_FORMAT_MODIFIER_PROPERTIES_EXT\"";
        break;
    case 1000158006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DRM_FORMAT_MODIFIER_PROPERTIES_LIST_2_EXT\"";
        break;
    case 1000160000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VALIDATION_CACHE_CREATE_INFO_EXT\"";
        break;
    case 1000160001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SHADER_MODULE_VALIDATION_CACHE_CREATE_INFO_EXT\"";
        break;
    case 1000163000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PORTABILITY_SUBSET_FEATURES_KHR\"";
        break;
    case 1000163001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PORTABILITY_SUBSET_PROPERTIES_KHR\"";
        break;
    case 1000164000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_SHADING_RATE_IMAGE_STATE_CREATE_INFO_NV\"";
        break;
    case 1000164001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADING_RATE_IMAGE_FEATURES_NV\"";
        break;
    case 1000164002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADING_RATE_IMAGE_PROPERTIES_NV\"";
        break;
    case 1000164005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_COARSE_SAMPLE_ORDER_STATE_CREATE_INFO_NV\"";
        break;
    case 1000165000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RAY_TRACING_PIPELINE_CREATE_INFO_NV\"";
        break;
    case 1000165001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_CREATE_INFO_NV\"";
        break;
    case 1000165003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_GEOMETRY_NV\"";
        break;
    case 1000165004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_GEOMETRY_TRIANGLES_NV\"";
        break;
    case 1000165005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_GEOMETRY_AABB_NV\"";
        break;
    case 1000165006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BIND_ACCELERATION_STRUCTURE_MEMORY_INFO_NV\"";
        break;
    case 1000165007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET_ACCELERATION_STRUCTURE_NV\"";
        break;
    case 1000165008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_MEMORY_REQUIREMENTS_INFO_NV\"";
        break;
    case 1000165009:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_PROPERTIES_NV\"";
        break;
    case 1000165011:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RAY_TRACING_SHADER_GROUP_CREATE_INFO_NV\"";
        break;
    case 1000165012:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_INFO_NV\"";
        break;
    case 1000166000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_REPRESENTATIVE_FRAGMENT_TEST_FEATURES_NV\"";
        break;
    case 1000166001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_REPRESENTATIVE_FRAGMENT_TEST_STATE_CREATE_INFO_NV\"";
        break;
    case 1000170000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_VIEW_IMAGE_FORMAT_INFO_EXT\"";
        break;
    case 1000170001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_FILTER_CUBIC_IMAGE_VIEW_IMAGE_FORMAT_PROPERTIES_EXT\"";
        break;
    case 1000178000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_MEMORY_HOST_POINTER_INFO_EXT\"";
        break;
    case 1000178001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_HOST_POINTER_PROPERTIES_EXT\"";
        break;
    case 1000178002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_MEMORY_HOST_PROPERTIES_EXT\"";
        break;
    case 1000181000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_CLOCK_FEATURES_KHR\"";
        break;
    case 1000183000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_COMPILER_CONTROL_CREATE_INFO_AMD\"";
        break;
    case 1000185000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_CORE_PROPERTIES_AMD\"";
        break;
    case 1000187000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_H265_CAPABILITIES_KHR\"";
        break;
    case 1000187001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_H265_SESSION_PARAMETERS_CREATE_INFO_KHR\"";
        break;
    case 1000187002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_H265_SESSION_PARAMETERS_ADD_INFO_KHR\"";
        break;
    case 1000187003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_H265_PROFILE_INFO_KHR\"";
        break;
    case 1000187004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_H265_PICTURE_INFO_KHR\"";
        break;
    case 1000187005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_H265_DPB_SLOT_INFO_KHR\"";
        break;
    case 1000174000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_QUEUE_GLOBAL_PRIORITY_CREATE_INFO_KHR\"";
        break;
    case 1000388000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_GLOBAL_PRIORITY_QUERY_FEATURES_KHR\"";
        break;
    case 1000388001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_QUEUE_FAMILY_GLOBAL_PRIORITY_PROPERTIES_KHR\"";
        break;
    case 1000189000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_MEMORY_OVERALLOCATION_CREATE_INFO_AMD\"";
        break;
    case 1000190000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_ATTRIBUTE_DIVISOR_PROPERTIES_EXT\"";
        break;
    case 1000191000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PRESENT_FRAME_TOKEN_GGP\"";
        break;
    case 1000201000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COMPUTE_SHADER_DERIVATIVES_FEATURES_NV\"";
        break;
    case 1000202000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MESH_SHADER_FEATURES_NV\"";
        break;
    case 1000202001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MESH_SHADER_PROPERTIES_NV\"";
        break;
    case 1000204000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_IMAGE_FOOTPRINT_FEATURES_NV\"";
        break;
    case 1000205000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_EXCLUSIVE_SCISSOR_STATE_CREATE_INFO_NV\"";
        break;
    case 1000205002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXCLUSIVE_SCISSOR_FEATURES_NV\"";
        break;
    case 1000206000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_CHECKPOINT_DATA_NV\"";
        break;
    case 1000206001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_QUEUE_FAMILY_CHECKPOINT_PROPERTIES_NV\"";
        break;
    case 1000209000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_INTEGER_FUNCTIONS_2_FEATURES_INTEL\"";
        break;
    case 1000210000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_QUERY_POOL_PERFORMANCE_QUERY_CREATE_INFO_INTEL\"";
        break;
    case 1000210001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_INITIALIZE_PERFORMANCE_API_INFO_INTEL\"";
        break;
    case 1000210002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PERFORMANCE_MARKER_INFO_INTEL\"";
        break;
    case 1000210003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PERFORMANCE_STREAM_MARKER_INFO_INTEL\"";
        break;
    case 1000210004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PERFORMANCE_OVERRIDE_INFO_INTEL\"";
        break;
    case 1000210005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PERFORMANCE_CONFIGURATION_ACQUIRE_INFO_INTEL\"";
        break;
    case 1000212000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PCI_BUS_INFO_PROPERTIES_EXT\"";
        break;
    case 1000213000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DISPLAY_NATIVE_HDR_SURFACE_CAPABILITIES_AMD\"";
        break;
    case 1000213001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SWAPCHAIN_DISPLAY_NATIVE_HDR_CREATE_INFO_AMD\"";
        break;
    case 1000214000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGEPIPE_SURFACE_CREATE_INFO_FUCHSIA\"";
        break;
    case 1000217000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_METAL_SURFACE_CREATE_INFO_EXT\"";
        break;
    case 1000218000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_DENSITY_MAP_FEATURES_EXT\"";
        break;
    case 1000218001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_DENSITY_MAP_PROPERTIES_EXT\"";
        break;
    case 1000218002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDER_PASS_FRAGMENT_DENSITY_MAP_CREATE_INFO_EXT\"";
        break;
    case 1000226000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_FRAGMENT_SHADING_RATE_ATTACHMENT_INFO_KHR\"";
        break;
    case 1000226001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_FRAGMENT_SHADING_RATE_STATE_CREATE_INFO_KHR\"";
        break;
    case 1000226002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADING_RATE_PROPERTIES_KHR\"";
        break;
    case 1000226003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADING_RATE_FEATURES_KHR\"";
        break;
    case 1000226004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADING_RATE_KHR\"";
        break;
    case 1000227000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_CORE_PROPERTIES_2_AMD\"";
        break;
    case 1000229000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COHERENT_MEMORY_FEATURES_AMD\"";
        break;
    case 1000232000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DYNAMIC_RENDERING_LOCAL_READ_FEATURES_KHR\"";
        break;
    case 1000232001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDERING_ATTACHMENT_LOCATION_INFO_KHR\"";
        break;
    case 1000232002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDERING_INPUT_ATTACHMENT_INDEX_INFO_KHR\"";
        break;
    case 1000234000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_IMAGE_ATOMIC_INT64_FEATURES_EXT\"";
        break;
    case 1000235000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_QUAD_CONTROL_FEATURES_KHR\"";
        break;
    case 1000237000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_BUDGET_PROPERTIES_EXT\"";
        break;
    case 1000238000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_PRIORITY_FEATURES_EXT\"";
        break;
    case 1000238001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_PRIORITY_ALLOCATE_INFO_EXT\"";
        break;
    case 1000239000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SURFACE_PROTECTED_CAPABILITIES_KHR\"";
        break;
    case 1000240000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEDICATED_ALLOCATION_IMAGE_ALIASING_FEATURES_NV\"";
        break;
    case 1000244000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_BUFFER_DEVICE_ADDRESS_FEATURES_EXT\"";
        break;
    case 1000244002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_DEVICE_ADDRESS_CREATE_INFO_EXT\"";
        break;
    case 1000247000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VALIDATION_FEATURES_EXT\"";
        break;
    case 1000248000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRESENT_WAIT_FEATURES_KHR\"";
        break;
    case 1000249000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COOPERATIVE_MATRIX_FEATURES_NV\"";
        break;
    case 1000249001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COOPERATIVE_MATRIX_PROPERTIES_NV\"";
        break;
    case 1000249002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COOPERATIVE_MATRIX_PROPERTIES_NV\"";
        break;
    case 1000250000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COVERAGE_REDUCTION_MODE_FEATURES_NV\"";
        break;
    case 1000250001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_COVERAGE_REDUCTION_STATE_CREATE_INFO_NV\"";
        break;
    case 1000250002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_FRAMEBUFFER_MIXED_SAMPLES_COMBINATION_NV\"";
        break;
    case 1000251000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADER_INTERLOCK_FEATURES_EXT\"";
        break;
    case 1000252000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_YCBCR_IMAGE_ARRAYS_FEATURES_EXT\"";
        break;
    case 1000254000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROVOKING_VERTEX_FEATURES_EXT\"";
        break;
    case 1000254001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_PROVOKING_VERTEX_STATE_CREATE_INFO_EXT\"";
        break;
    case 1000254002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROVOKING_VERTEX_PROPERTIES_EXT\"";
        break;
    case 1000255000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SURFACE_FULL_SCREEN_EXCLUSIVE_INFO_EXT\"";
        break;
    case 1000255002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SURFACE_CAPABILITIES_FULL_SCREEN_EXCLUSIVE_EXT\"";
        break;
    case 1000255001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SURFACE_FULL_SCREEN_EXCLUSIVE_WIN32_INFO_EXT\"";
        break;
    case 1000256000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_HEADLESS_SURFACE_CREATE_INFO_EXT\"";
        break;
    case 1000260000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_ATOMIC_FLOAT_FEATURES_EXT\"";
        break;
    case 1000267000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_FEATURES_EXT\"";
        break;
    case 1000269000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PIPELINE_EXECUTABLE_PROPERTIES_FEATURES_KHR\"";
        break;
    case 1000269001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_INFO_KHR\"";
        break;
    case 1000269002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_EXECUTABLE_PROPERTIES_KHR\"";
        break;
    case 1000269003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_EXECUTABLE_INFO_KHR\"";
        break;
    case 1000269004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_EXECUTABLE_STATISTIC_KHR\"";
        break;
    case 1000269005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_EXECUTABLE_INTERNAL_REPRESENTATION_KHR\"";
        break;
    case 1000270000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_HOST_IMAGE_COPY_FEATURES_EXT\"";
        break;
    case 1000270001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_HOST_IMAGE_COPY_PROPERTIES_EXT\"";
        break;
    case 1000270002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_TO_IMAGE_COPY_EXT\"";
        break;
    case 1000270003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_TO_MEMORY_COPY_EXT\"";
        break;
    case 1000270004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COPY_IMAGE_TO_MEMORY_INFO_EXT\"";
        break;
    case 1000270005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COPY_MEMORY_TO_IMAGE_INFO_EXT\"";
        break;
    case 1000270006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_HOST_IMAGE_LAYOUT_TRANSITION_INFO_EXT\"";
        break;
    case 1000270007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COPY_IMAGE_TO_IMAGE_INFO_EXT\"";
        break;
    case 1000270008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SUBRESOURCE_HOST_MEMCPY_SIZE_EXT\"";
        break;
    case 1000270009:
        settings.stream() << "\"VK_STRUCTURE_TYPE_HOST_IMAGE_COPY_DEVICE_PERFORMANCE_QUERY_EXT\"";
        break;
    case 1000271000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_MAP_INFO_KHR\"";
        break;
    case 1000271001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_UNMAP_INFO_KHR\"";
        break;
    case 1000272000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAP_MEMORY_PLACED_FEATURES_EXT\"";
        break;
    case 1000272001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAP_MEMORY_PLACED_PROPERTIES_EXT\"";
        break;
    case 1000272002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_MAP_PLACED_INFO_EXT\"";
        break;
    case 1000273000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_ATOMIC_FLOAT_2_FEATURES_EXT\"";
        break;
    case 1000274000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SURFACE_PRESENT_MODE_EXT\"";
        break;
    case 1000274001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SURFACE_PRESENT_SCALING_CAPABILITIES_EXT\"";
        break;
    case 1000274002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SURFACE_PRESENT_MODE_COMPATIBILITY_EXT\"";
        break;
    case 1000275000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SWAPCHAIN_MAINTENANCE_1_FEATURES_EXT\"";
        break;
    case 1000275001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SWAPCHAIN_PRESENT_FENCE_INFO_EXT\"";
        break;
    case 1000275002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SWAPCHAIN_PRESENT_MODES_CREATE_INFO_EXT\"";
        break;
    case 1000275003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SWAPCHAIN_PRESENT_MODE_INFO_EXT\"";
        break;
    case 1000275004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SWAPCHAIN_PRESENT_SCALING_CREATE_INFO_EXT\"";
        break;
    case 1000275005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RELEASE_SWAPCHAIN_IMAGES_INFO_EXT\"";
        break;
    case 1000277000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEVICE_GENERATED_COMMANDS_PROPERTIES_NV\"";
        break;
    case 1000277001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_GRAPHICS_SHADER_GROUP_CREATE_INFO_NV\"";
        break;
    case 1000277002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_GRAPHICS_PIPELINE_SHADER_GROUPS_CREATE_INFO_NV\"";
        break;
    case 1000277003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_INDIRECT_COMMANDS_LAYOUT_TOKEN_NV\"";
        break;
    case 1000277004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_INDIRECT_COMMANDS_LAYOUT_CREATE_INFO_NV\"";
        break;
    case 1000277005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_GENERATED_COMMANDS_INFO_NV\"";
        break;
    case 1000277006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_GENERATED_COMMANDS_MEMORY_REQUIREMENTS_INFO_NV\"";
        break;
    case 1000277007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEVICE_GENERATED_COMMANDS_FEATURES_NV\"";
        break;
    case 1000278000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_INHERITED_VIEWPORT_SCISSOR_FEATURES_NV\"";
        break;
    case 1000278001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_VIEWPORT_SCISSOR_INFO_NV\"";
        break;
    case 1000281000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TEXEL_BUFFER_ALIGNMENT_FEATURES_EXT\"";
        break;
    case 1000282000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_RENDER_PASS_TRANSFORM_INFO_QCOM\"";
        break;
    case 1000282001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDER_PASS_TRANSFORM_BEGIN_INFO_QCOM\"";
        break;
    case 1000283000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_BIAS_CONTROL_FEATURES_EXT\"";
        break;
    case 1000283001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEPTH_BIAS_INFO_EXT\"";
        break;
    case 1000283002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEPTH_BIAS_REPRESENTATION_INFO_EXT\"";
        break;
    case 1000284000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEVICE_MEMORY_REPORT_FEATURES_EXT\"";
        break;
    case 1000284001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_DEVICE_MEMORY_REPORT_CREATE_INFO_EXT\"";
        break;
    case 1000284002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_MEMORY_REPORT_CALLBACK_DATA_EXT\"";
        break;
    case 1000286000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ROBUSTNESS_2_FEATURES_EXT\"";
        break;
    case 1000286001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ROBUSTNESS_2_PROPERTIES_EXT\"";
        break;
    case 1000287000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SAMPLER_CUSTOM_BORDER_COLOR_CREATE_INFO_EXT\"";
        break;
    case 1000287001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CUSTOM_BORDER_COLOR_PROPERTIES_EXT\"";
        break;
    case 1000287002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CUSTOM_BORDER_COLOR_FEATURES_EXT\"";
        break;
    case 1000290000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_LIBRARY_CREATE_INFO_KHR\"";
        break;
    case 1000292000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRESENT_BARRIER_FEATURES_NV\"";
        break;
    case 1000292001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SURFACE_CAPABILITIES_PRESENT_BARRIER_NV\"";
        break;
    case 1000292002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SWAPCHAIN_PRESENT_BARRIER_CREATE_INFO_NV\"";
        break;
    case 1000294000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PRESENT_ID_KHR\"";
        break;
    case 1000294001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRESENT_ID_FEATURES_KHR\"";
        break;
    case 1000299000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_INFO_KHR\"";
        break;
    case 1000299001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_RATE_CONTROL_INFO_KHR\"";
        break;
    case 1000299002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_RATE_CONTROL_LAYER_INFO_KHR\"";
        break;
    case 1000299003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_CAPABILITIES_KHR\"";
        break;
    case 1000299004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_USAGE_INFO_KHR\"";
        break;
    case 1000299005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_QUERY_POOL_VIDEO_ENCODE_FEEDBACK_CREATE_INFO_KHR\"";
        break;
    case 1000299006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VIDEO_ENCODE_QUALITY_LEVEL_INFO_KHR\"";
        break;
    case 1000299007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_QUALITY_LEVEL_PROPERTIES_KHR\"";
        break;
    case 1000299008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_QUALITY_LEVEL_INFO_KHR\"";
        break;
    case 1000299009:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_SESSION_PARAMETERS_GET_INFO_KHR\"";
        break;
    case 1000299010:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_ENCODE_SESSION_PARAMETERS_FEEDBACK_INFO_KHR\"";
        break;
    case 1000300000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DIAGNOSTICS_CONFIG_FEATURES_NV\"";
        break;
    case 1000300001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_DIAGNOSTICS_CONFIG_CREATE_INFO_NV\"";
        break;
    case 1000307000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_CUDA_MODULE_CREATE_INFO_NV\"";
        break;
    case 1000307001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_CUDA_FUNCTION_CREATE_INFO_NV\"";
        break;
    case 1000307002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_CUDA_LAUNCH_INFO_NV\"";
        break;
    case 1000307003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CUDA_KERNEL_LAUNCH_FEATURES_NV\"";
        break;
    case 1000307004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CUDA_KERNEL_LAUNCH_PROPERTIES_NV\"";
        break;
    case 1000308000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_REFRESH_OBJECT_LIST_KHR\"";
        break;
    case 1000310000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_QUERY_LOW_LATENCY_SUPPORT_NV\"";
        break;
    case 1000311000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_METAL_OBJECT_CREATE_INFO_EXT\"";
        break;
    case 1000311001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_METAL_OBJECTS_INFO_EXT\"";
        break;
    case 1000311002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_METAL_DEVICE_INFO_EXT\"";
        break;
    case 1000311003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_METAL_COMMAND_QUEUE_INFO_EXT\"";
        break;
    case 1000311004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_METAL_BUFFER_INFO_EXT\"";
        break;
    case 1000311005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_METAL_BUFFER_INFO_EXT\"";
        break;
    case 1000311006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_METAL_TEXTURE_INFO_EXT\"";
        break;
    case 1000311007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_METAL_TEXTURE_INFO_EXT\"";
        break;
    case 1000311008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_METAL_IO_SURFACE_INFO_EXT\"";
        break;
    case 1000311009:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_METAL_IO_SURFACE_INFO_EXT\"";
        break;
    case 1000311010:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_METAL_SHARED_EVENT_INFO_EXT\"";
        break;
    case 1000311011:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_METAL_SHARED_EVENT_INFO_EXT\"";
        break;
    case 1000314008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_QUEUE_FAMILY_CHECKPOINT_PROPERTIES_2_NV\"";
        break;
    case 1000314009:
        settings.stream() << "\"VK_STRUCTURE_TYPE_CHECKPOINT_DATA_2_NV\"";
        break;
    case 1000316000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_BUFFER_PROPERTIES_EXT\"";
        break;
    case 1000316001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_BUFFER_DENSITY_MAP_PROPERTIES_EXT\"";
        break;
    case 1000316002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_BUFFER_FEATURES_EXT\"";
        break;
    case 1000316003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DESCRIPTOR_ADDRESS_INFO_EXT\"";
        break;
    case 1000316004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DESCRIPTOR_GET_INFO_EXT\"";
        break;
    case 1000316005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_CAPTURE_DESCRIPTOR_DATA_INFO_EXT\"";
        break;
    case 1000316006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_CAPTURE_DESCRIPTOR_DATA_INFO_EXT\"";
        break;
    case 1000316007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_VIEW_CAPTURE_DESCRIPTOR_DATA_INFO_EXT\"";
        break;
    case 1000316008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SAMPLER_CAPTURE_DESCRIPTOR_DATA_INFO_EXT\"";
        break;
    case 1000316010:
        settings.stream() << "\"VK_STRUCTURE_TYPE_OPAQUE_CAPTURE_DESCRIPTOR_DATA_CREATE_INFO_EXT\"";
        break;
    case 1000316011:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DESCRIPTOR_BUFFER_BINDING_INFO_EXT\"";
        break;
    case 1000316012:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DESCRIPTOR_BUFFER_BINDING_PUSH_DESCRIPTOR_BUFFER_HANDLE_EXT\"";
        break;
    case 1000316009:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_CAPTURE_DESCRIPTOR_DATA_INFO_EXT\"";
        break;
    case 1000320000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_GRAPHICS_PIPELINE_LIBRARY_FEATURES_EXT\"";
        break;
    case 1000320001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_GRAPHICS_PIPELINE_LIBRARY_PROPERTIES_EXT\"";
        break;
    case 1000320002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_GRAPHICS_PIPELINE_LIBRARY_CREATE_INFO_EXT\"";
        break;
    case 1000321000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_EARLY_AND_LATE_FRAGMENT_TESTS_FEATURES_AMD\"";
        break;
    case 1000203000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADER_BARYCENTRIC_FEATURES_KHR\"";
        break;
    case 1000322000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADER_BARYCENTRIC_PROPERTIES_KHR\"";
        break;
    case 1000323000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_SUBGROUP_UNIFORM_CONTROL_FLOW_FEATURES_KHR\"";
        break;
    case 1000326000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADING_RATE_ENUMS_PROPERTIES_NV\"";
        break;
    case 1000326001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADING_RATE_ENUMS_FEATURES_NV\"";
        break;
    case 1000326002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_FRAGMENT_SHADING_RATE_ENUM_STATE_CREATE_INFO_NV\"";
        break;
    case 1000327000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_MOTION_TRIANGLES_DATA_NV\"";
        break;
    case 1000327001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_MOTION_BLUR_FEATURES_NV\"";
        break;
    case 1000327002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_MOTION_INFO_NV\"";
        break;
    case 1000328000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MESH_SHADER_FEATURES_EXT\"";
        break;
    case 1000328001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MESH_SHADER_PROPERTIES_EXT\"";
        break;
    case 1000330000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_YCBCR_2_PLANE_444_FORMATS_FEATURES_EXT\"";
        break;
    case 1000332000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_DENSITY_MAP_2_FEATURES_EXT\"";
        break;
    case 1000332001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_DENSITY_MAP_2_PROPERTIES_EXT\"";
        break;
    case 1000333000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COPY_COMMAND_TRANSFORM_INFO_QCOM\"";
        break;
    case 1000336000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_WORKGROUP_MEMORY_EXPLICIT_LAYOUT_FEATURES_KHR\"";
        break;
    case 1000338000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_COMPRESSION_CONTROL_FEATURES_EXT\"";
        break;
    case 1000338001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_COMPRESSION_CONTROL_EXT\"";
        break;
    case 1000338004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_COMPRESSION_PROPERTIES_EXT\"";
        break;
    case 1000339000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ATTACHMENT_FEEDBACK_LOOP_LAYOUT_FEATURES_EXT\"";
        break;
    case 1000340000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_4444_FORMATS_FEATURES_EXT\"";
        break;
    case 1000341000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FAULT_FEATURES_EXT\"";
        break;
    case 1000341001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_FAULT_COUNTS_EXT\"";
        break;
    case 1000341002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_FAULT_INFO_EXT\"";
        break;
    case 1000344000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RGBA10X6_FORMATS_FEATURES_EXT\"";
        break;
    case 1000346000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DIRECTFB_SURFACE_CREATE_INFO_EXT\"";
        break;
    case 1000352000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_INPUT_DYNAMIC_STATE_FEATURES_EXT\"";
        break;
    case 1000352001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VERTEX_INPUT_BINDING_DESCRIPTION_2_EXT\"";
        break;
    case 1000352002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VERTEX_INPUT_ATTRIBUTE_DESCRIPTION_2_EXT\"";
        break;
    case 1000353000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DRM_PROPERTIES_EXT\"";
        break;
    case 1000354000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ADDRESS_BINDING_REPORT_FEATURES_EXT\"";
        break;
    case 1000354001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_ADDRESS_BINDING_CALLBACK_DATA_EXT\"";
        break;
    case 1000355000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_CLIP_CONTROL_FEATURES_EXT\"";
        break;
    case 1000355001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_DEPTH_CLIP_CONTROL_CREATE_INFO_EXT\"";
        break;
    case 1000356000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRIMITIVE_TOPOLOGY_LIST_RESTART_FEATURES_EXT\"";
        break;
    case 1000364000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_MEMORY_ZIRCON_HANDLE_INFO_FUCHSIA\"";
        break;
    case 1000364001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_ZIRCON_HANDLE_PROPERTIES_FUCHSIA\"";
        break;
    case 1000364002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_GET_ZIRCON_HANDLE_INFO_FUCHSIA\"";
        break;
    case 1000365000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_SEMAPHORE_ZIRCON_HANDLE_INFO_FUCHSIA\"";
        break;
    case 1000365001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SEMAPHORE_GET_ZIRCON_HANDLE_INFO_FUCHSIA\"";
        break;
    case 1000366000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_COLLECTION_CREATE_INFO_FUCHSIA\"";
        break;
    case 1000366001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_MEMORY_BUFFER_COLLECTION_FUCHSIA\"";
        break;
    case 1000366002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_COLLECTION_IMAGE_CREATE_INFO_FUCHSIA\"";
        break;
    case 1000366003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_COLLECTION_PROPERTIES_FUCHSIA\"";
        break;
    case 1000366004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_CONSTRAINTS_INFO_FUCHSIA\"";
        break;
    case 1000366005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_COLLECTION_BUFFER_CREATE_INFO_FUCHSIA\"";
        break;
    case 1000366006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_CONSTRAINTS_INFO_FUCHSIA\"";
        break;
    case 1000366007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_FORMAT_CONSTRAINTS_INFO_FUCHSIA\"";
        break;
    case 1000366008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SYSMEM_COLOR_SPACE_FUCHSIA\"";
        break;
    case 1000366009:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_COLLECTION_CONSTRAINTS_INFO_FUCHSIA\"";
        break;
    case 1000369000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SUBPASS_SHADING_PIPELINE_CREATE_INFO_HUAWEI\"";
        break;
    case 1000369001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBPASS_SHADING_FEATURES_HUAWEI\"";
        break;
    case 1000369002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBPASS_SHADING_PROPERTIES_HUAWEI\"";
        break;
    case 1000370000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_INVOCATION_MASK_FEATURES_HUAWEI\"";
        break;
    case 1000371000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_GET_REMOTE_ADDRESS_INFO_NV\"";
        break;
    case 1000371001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_MEMORY_RDMA_FEATURES_NV\"";
        break;
    case 1000372000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_PROPERTIES_IDENTIFIER_EXT\"";
        break;
    case 1000372001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PIPELINE_PROPERTIES_FEATURES_EXT\"";
        break;
    case 1000373000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_FENCE_SCI_SYNC_INFO_NV\"";
        break;
    case 1000373001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_FENCE_SCI_SYNC_INFO_NV\"";
        break;
    case 1000373002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_FENCE_GET_SCI_SYNC_INFO_NV\"";
        break;
    case 1000373003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SCI_SYNC_ATTRIBUTES_INFO_NV\"";
        break;
    case 1000373004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_SEMAPHORE_SCI_SYNC_INFO_NV\"";
        break;
    case 1000373005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_SEMAPHORE_SCI_SYNC_INFO_NV\"";
        break;
    case 1000373006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SEMAPHORE_GET_SCI_SYNC_INFO_NV\"";
        break;
    case 1000373007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_SCI_SYNC_FEATURES_NV\"";
        break;
    case 1000374000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_MEMORY_SCI_BUF_INFO_NV\"";
        break;
    case 1000374001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXPORT_MEMORY_SCI_BUF_INFO_NV\"";
        break;
    case 1000374002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_GET_SCI_BUF_INFO_NV\"";
        break;
    case 1000374003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MEMORY_SCI_BUF_PROPERTIES_NV\"";
        break;
    case 1000374004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_MEMORY_SCI_BUF_FEATURES_NV\"";
        break;
    case 1000375000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAME_BOUNDARY_FEATURES_EXT\"";
        break;
    case 1000375001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_FRAME_BOUNDARY_EXT\"";
        break;
    case 1000376000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTISAMPLED_RENDER_TO_SINGLE_SAMPLED_FEATURES_EXT\"";
        break;
    case 1000376001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SUBPASS_RESOLVE_PERFORMANCE_QUERY_EXT\"";
        break;
    case 1000376002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MULTISAMPLED_RENDER_TO_SINGLE_SAMPLED_INFO_EXT\"";
        break;
    case 1000377000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_2_FEATURES_EXT\"";
        break;
    case 1000378000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SCREEN_SURFACE_CREATE_INFO_QNX\"";
        break;
    case 1000381000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COLOR_WRITE_ENABLE_FEATURES_EXT\"";
        break;
    case 1000381001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_COLOR_WRITE_CREATE_INFO_EXT\"";
        break;
    case 1000382000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRIMITIVES_GENERATED_QUERY_FEATURES_EXT\"";
        break;
    case 1000386000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_MAINTENANCE_1_FEATURES_KHR\"";
        break;
    case 1000391000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_VIEW_MIN_LOD_FEATURES_EXT\"";
        break;
    case 1000391001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_VIEW_MIN_LOD_CREATE_INFO_EXT\"";
        break;
    case 1000392000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTI_DRAW_FEATURES_EXT\"";
        break;
    case 1000392001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTI_DRAW_PROPERTIES_EXT\"";
        break;
    case 1000393000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_2D_VIEW_OF_3D_FEATURES_EXT\"";
        break;
    case 1000395000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_TILE_IMAGE_FEATURES_EXT\"";
        break;
    case 1000395001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_TILE_IMAGE_PROPERTIES_EXT\"";
        break;
    case 1000396000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MICROMAP_BUILD_INFO_EXT\"";
        break;
    case 1000396001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MICROMAP_VERSION_INFO_EXT\"";
        break;
    case 1000396002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COPY_MICROMAP_INFO_EXT\"";
        break;
    case 1000396003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COPY_MICROMAP_TO_MEMORY_INFO_EXT\"";
        break;
    case 1000396004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COPY_MEMORY_TO_MICROMAP_INFO_EXT\"";
        break;
    case 1000396005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_OPACITY_MICROMAP_FEATURES_EXT\"";
        break;
    case 1000396006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_OPACITY_MICROMAP_PROPERTIES_EXT\"";
        break;
    case 1000396007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MICROMAP_CREATE_INFO_EXT\"";
        break;
    case 1000396008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MICROMAP_BUILD_SIZES_INFO_EXT\"";
        break;
    case 1000396009:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_TRIANGLES_OPACITY_MICROMAP_EXT\"";
        break;
    case 1000397000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DISPLACEMENT_MICROMAP_FEATURES_NV\"";
        break;
    case 1000397001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DISPLACEMENT_MICROMAP_PROPERTIES_NV\"";
        break;
    case 1000397002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_TRIANGLES_DISPLACEMENT_MICROMAP_NV\"";
        break;
    case 1000404000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CLUSTER_CULLING_SHADER_FEATURES_HUAWEI\"";
        break;
    case 1000404001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CLUSTER_CULLING_SHADER_PROPERTIES_HUAWEI\"";
        break;
    case 1000404002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CLUSTER_CULLING_SHADER_VRS_FEATURES_HUAWEI\"";
        break;
    case 1000411000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_BORDER_COLOR_SWIZZLE_FEATURES_EXT\"";
        break;
    case 1000411001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SAMPLER_BORDER_COLOR_COMPONENT_MAPPING_CREATE_INFO_EXT\"";
        break;
    case 1000412000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PAGEABLE_DEVICE_LOCAL_MEMORY_FEATURES_EXT\"";
        break;
    case 1000415000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_CORE_PROPERTIES_ARM\"";
        break;
    case 1000416000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_SUBGROUP_ROTATE_FEATURES_KHR\"";
        break;
    case 1000417000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_QUEUE_SHADER_CORE_CONTROL_CREATE_INFO_ARM\"";
        break;
    case 1000417001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SCHEDULING_CONTROLS_FEATURES_ARM\"";
        break;
    case 1000417002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SCHEDULING_CONTROLS_PROPERTIES_ARM\"";
        break;
    case 1000418000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_SLICED_VIEW_OF_3D_FEATURES_EXT\"";
        break;
    case 1000418001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_VIEW_SLICED_CREATE_INFO_EXT\"";
        break;
    case 1000420000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_SET_HOST_MAPPING_FEATURES_VALVE\"";
        break;
    case 1000420001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DESCRIPTOR_SET_BINDING_REFERENCE_VALVE\"";
        break;
    case 1000420002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_HOST_MAPPING_INFO_VALVE\"";
        break;
    case 1000421000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_CLAMP_ZERO_ONE_FEATURES_EXT\"";
        break;
    case 1000422000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_NON_SEAMLESS_CUBE_MAP_FEATURES_EXT\"";
        break;
    case 1000424000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RENDER_PASS_STRIPED_FEATURES_ARM\"";
        break;
    case 1000424001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RENDER_PASS_STRIPED_PROPERTIES_ARM\"";
        break;
    case 1000424002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDER_PASS_STRIPE_BEGIN_INFO_ARM\"";
        break;
    case 1000424003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDER_PASS_STRIPE_INFO_ARM\"";
        break;
    case 1000424004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDER_PASS_STRIPE_SUBMIT_INFO_ARM\"";
        break;
    case 1000425000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_DENSITY_MAP_OFFSET_FEATURES_QCOM\"";
        break;
    case 1000425001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_DENSITY_MAP_OFFSET_PROPERTIES_QCOM\"";
        break;
    case 1000425002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SUBPASS_FRAGMENT_DENSITY_MAP_OFFSET_END_INFO_QCOM\"";
        break;
    case 1000426000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COPY_MEMORY_INDIRECT_FEATURES_NV\"";
        break;
    case 1000426001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COPY_MEMORY_INDIRECT_PROPERTIES_NV\"";
        break;
    case 1000427000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_DECOMPRESSION_FEATURES_NV\"";
        break;
    case 1000427001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_DECOMPRESSION_PROPERTIES_NV\"";
        break;
    case 1000428000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEVICE_GENERATED_COMMANDS_COMPUTE_FEATURES_NV\"";
        break;
    case 1000428001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COMPUTE_PIPELINE_INDIRECT_BUFFER_INFO_NV\"";
        break;
    case 1000428002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_INDIRECT_DEVICE_ADDRESS_INFO_NV\"";
        break;
    case 1000430000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LINEAR_COLOR_ATTACHMENT_FEATURES_NV\"";
        break;
    case 1000434000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_MAXIMAL_RECONVERGENCE_FEATURES_KHR\"";
        break;
    case 1000435000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_APPLICATION_PARAMETERS_EXT\"";
        break;
    case 1000437000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_COMPRESSION_CONTROL_SWAPCHAIN_FEATURES_EXT\"";
        break;
    case 1000440000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_PROCESSING_FEATURES_QCOM\"";
        break;
    case 1000440001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_PROCESSING_PROPERTIES_QCOM\"";
        break;
    case 1000440002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_VIEW_SAMPLE_WEIGHT_CREATE_INFO_QCOM\"";
        break;
    case 1000451000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_NESTED_COMMAND_BUFFER_FEATURES_EXT\"";
        break;
    case 1000451001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_NESTED_COMMAND_BUFFER_PROPERTIES_EXT\"";
        break;
    case 1000453000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXTERNAL_MEMORY_ACQUIRE_UNMODIFIED_EXT\"";
        break;
    case 1000455000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_3_FEATURES_EXT\"";
        break;
    case 1000455001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_3_PROPERTIES_EXT\"";
        break;
    case 1000458000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBPASS_MERGE_FEEDBACK_FEATURES_EXT\"";
        break;
    case 1000458001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDER_PASS_CREATION_CONTROL_EXT\"";
        break;
    case 1000458002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDER_PASS_CREATION_FEEDBACK_CREATE_INFO_EXT\"";
        break;
    case 1000458003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDER_PASS_SUBPASS_FEEDBACK_CREATE_INFO_EXT\"";
        break;
    case 1000459000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DIRECT_DRIVER_LOADING_INFO_LUNARG\"";
        break;
    case 1000459001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DIRECT_DRIVER_LOADING_LIST_LUNARG\"";
        break;
    case 1000462000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_MODULE_IDENTIFIER_FEATURES_EXT\"";
        break;
    case 1000462001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_MODULE_IDENTIFIER_PROPERTIES_EXT\"";
        break;
    case 1000462002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_MODULE_IDENTIFIER_CREATE_INFO_EXT\"";
        break;
    case 1000462003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SHADER_MODULE_IDENTIFIER_EXT\"";
        break;
    case 1000342000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RASTERIZATION_ORDER_ATTACHMENT_ACCESS_FEATURES_EXT\"";
        break;
    case 1000464000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_OPTICAL_FLOW_FEATURES_NV\"";
        break;
    case 1000464001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_OPTICAL_FLOW_PROPERTIES_NV\"";
        break;
    case 1000464002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_OPTICAL_FLOW_IMAGE_FORMAT_INFO_NV\"";
        break;
    case 1000464003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_OPTICAL_FLOW_IMAGE_FORMAT_PROPERTIES_NV\"";
        break;
    case 1000464004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_OPTICAL_FLOW_SESSION_CREATE_INFO_NV\"";
        break;
    case 1000464005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_OPTICAL_FLOW_EXECUTE_INFO_NV\"";
        break;
    case 1000464010:
        settings.stream() << "\"VK_STRUCTURE_TYPE_OPTICAL_FLOW_SESSION_CREATE_PRIVATE_DATA_INFO_NV\"";
        break;
    case 1000465000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LEGACY_DITHERING_FEATURES_EXT\"";
        break;
    case 1000466000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PIPELINE_PROTECTED_ACCESS_FEATURES_EXT\"";
        break;
    case 1000468000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_FORMAT_RESOLVE_FEATURES_ANDROID\"";
        break;
    case 1000468001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_FORMAT_RESOLVE_PROPERTIES_ANDROID\"";
        break;
    case 1000468002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_ANDROID_HARDWARE_BUFFER_FORMAT_RESOLVE_PROPERTIES_ANDROID\"";
        break;
    case 1000470000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_5_FEATURES_KHR\"";
        break;
    case 1000470001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_5_PROPERTIES_KHR\"";
        break;
    case 1000470003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_RENDERING_AREA_INFO_KHR\"";
        break;
    case 1000470004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_DEVICE_IMAGE_SUBRESOURCE_INFO_KHR\"";
        break;
    case 1000338002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SUBRESOURCE_LAYOUT_2_KHR\"";
        break;
    case 1000338003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_SUBRESOURCE_2_KHR\"";
        break;
    case 1000470005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_CREATE_FLAGS_2_CREATE_INFO_KHR\"";
        break;
    case 1000470006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BUFFER_USAGE_FLAGS_2_CREATE_INFO_KHR\"";
        break;
    case 1000481000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_POSITION_FETCH_FEATURES_KHR\"";
        break;
    case 1000482000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_OBJECT_FEATURES_EXT\"";
        break;
    case 1000482001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_OBJECT_PROPERTIES_EXT\"";
        break;
    case 1000482002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SHADER_CREATE_INFO_EXT\"";
        break;
    case 1000484000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TILE_PROPERTIES_FEATURES_QCOM\"";
        break;
    case 1000484001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_TILE_PROPERTIES_QCOM\"";
        break;
    case 1000485000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_AMIGO_PROFILING_FEATURES_SEC\"";
        break;
    case 1000485001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_AMIGO_PROFILING_SUBMIT_INFO_SEC\"";
        break;
    case 1000488000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTIVIEW_PER_VIEW_VIEWPORTS_FEATURES_QCOM\"";
        break;
    case 1000489000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SEMAPHORE_SCI_SYNC_POOL_CREATE_INFO_NV\"";
        break;
    case 1000489001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SEMAPHORE_SCI_SYNC_CREATE_INFO_NV\"";
        break;
    case 1000489002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_SCI_SYNC_2_FEATURES_NV\"";
        break;
    case 1000490000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_INVOCATION_REORDER_FEATURES_NV\"";
        break;
    case 1000490001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_INVOCATION_REORDER_PROPERTIES_NV\"";
        break;
    case 1000492000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_SPARSE_ADDRESS_SPACE_FEATURES_NV\"";
        break;
    case 1000492001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_SPARSE_ADDRESS_SPACE_PROPERTIES_NV\"";
        break;
    case 1000351000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MUTABLE_DESCRIPTOR_TYPE_FEATURES_EXT\"";
        break;
    case 1000351002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MUTABLE_DESCRIPTOR_TYPE_CREATE_INFO_EXT\"";
        break;
    case 1000495000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LEGACY_VERTEX_ATTRIBUTES_FEATURES_EXT\"";
        break;
    case 1000495001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LEGACY_VERTEX_ATTRIBUTES_PROPERTIES_EXT\"";
        break;
    case 1000496000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_LAYER_SETTINGS_CREATE_INFO_EXT\"";
        break;
    case 1000497000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_CORE_BUILTINS_FEATURES_ARM\"";
        break;
    case 1000497001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_CORE_BUILTINS_PROPERTIES_ARM\"";
        break;
    case 1000498000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PIPELINE_LIBRARY_GROUP_HANDLES_FEATURES_EXT\"";
        break;
    case 1000499000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DYNAMIC_RENDERING_UNUSED_ATTACHMENTS_FEATURES_EXT\"";
        break;
    case 1000505000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_LATENCY_SLEEP_MODE_INFO_NV\"";
        break;
    case 1000505001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_LATENCY_SLEEP_INFO_NV\"";
        break;
    case 1000505002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SET_LATENCY_MARKER_INFO_NV\"";
        break;
    case 1000505003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_GET_LATENCY_MARKER_INFO_NV\"";
        break;
    case 1000505004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_LATENCY_TIMINGS_FRAME_REPORT_NV\"";
        break;
    case 1000505005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_LATENCY_SUBMISSION_PRESENT_ID_NV\"";
        break;
    case 1000505006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_OUT_OF_BAND_QUEUE_TYPE_INFO_NV\"";
        break;
    case 1000505007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SWAPCHAIN_LATENCY_CREATE_INFO_NV\"";
        break;
    case 1000505008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_LATENCY_SURFACE_CAPABILITIES_NV\"";
        break;
    case 1000506000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COOPERATIVE_MATRIX_FEATURES_KHR\"";
        break;
    case 1000506001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_COOPERATIVE_MATRIX_PROPERTIES_KHR\"";
        break;
    case 1000506002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COOPERATIVE_MATRIX_PROPERTIES_KHR\"";
        break;
    case 1000510000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTIVIEW_PER_VIEW_RENDER_AREAS_FEATURES_QCOM\"";
        break;
    case 1000510001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_MULTIVIEW_PER_VIEW_RENDER_AREAS_RENDER_PASS_BEGIN_INFO_QCOM\"";
        break;
    case 1000512000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_AV1_CAPABILITIES_KHR\"";
        break;
    case 1000512001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_AV1_PICTURE_INFO_KHR\"";
        break;
    case 1000512003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_AV1_PROFILE_INFO_KHR\"";
        break;
    case 1000512004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_AV1_SESSION_PARAMETERS_CREATE_INFO_KHR\"";
        break;
    case 1000512005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_DECODE_AV1_DPB_SLOT_INFO_KHR\"";
        break;
    case 1000515000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VIDEO_MAINTENANCE_1_FEATURES_KHR\"";
        break;
    case 1000515001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_VIDEO_INLINE_QUERY_INFO_KHR\"";
        break;
    case 1000516000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PER_STAGE_DESCRIPTOR_SET_FEATURES_NV\"";
        break;
    case 1000518000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_PROCESSING_2_FEATURES_QCOM\"";
        break;
    case 1000518001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_PROCESSING_2_PROPERTIES_QCOM\"";
        break;
    case 1000518002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SAMPLER_BLOCK_MATCH_WINDOW_CREATE_INFO_QCOM\"";
        break;
    case 1000519000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SAMPLER_CUBIC_WEIGHTS_CREATE_INFO_QCOM\"";
        break;
    case 1000519001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CUBIC_WEIGHTS_FEATURES_QCOM\"";
        break;
    case 1000519002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BLIT_IMAGE_CUBIC_WEIGHTS_INFO_QCOM\"";
        break;
    case 1000520000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_YCBCR_DEGAMMA_FEATURES_QCOM\"";
        break;
    case 1000520001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SAMPLER_YCBCR_CONVERSION_YCBCR_DEGAMMA_CREATE_INFO_QCOM\"";
        break;
    case 1000521000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CUBIC_CLAMP_FEATURES_QCOM\"";
        break;
    case 1000524000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ATTACHMENT_FEEDBACK_LOOP_DYNAMIC_STATE_FEATURES_EXT\"";
        break;
    case 1000525000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_ATTRIBUTE_DIVISOR_PROPERTIES_KHR\"";
        break;
    case 1000190001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_VERTEX_INPUT_DIVISOR_STATE_CREATE_INFO_KHR\"";
        break;
    case 1000190002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_ATTRIBUTE_DIVISOR_FEATURES_KHR\"";
        break;
    case 1000528000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_FLOAT_CONTROLS_2_FEATURES_KHR\"";
        break;
    case 1000529000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SCREEN_BUFFER_PROPERTIES_QNX\"";
        break;
    case 1000529001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SCREEN_BUFFER_FORMAT_PROPERTIES_QNX\"";
        break;
    case 1000529002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMPORT_SCREEN_BUFFER_INFO_QNX\"";
        break;
    case 1000529003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_EXTERNAL_FORMAT_QNX\"";
        break;
    case 1000529004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_MEMORY_SCREEN_BUFFER_FEATURES_QNX\"";
        break;
    case 1000530000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LAYERED_DRIVER_PROPERTIES_MSFT\"";
        break;
    case 1000265000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_INDEX_TYPE_UINT8_FEATURES_KHR\"";
        break;
    case 1000259000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LINE_RASTERIZATION_FEATURES_KHR\"";
        break;
    case 1000259001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_LINE_STATE_CREATE_INFO_KHR\"";
        break;
    case 1000259002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LINE_RASTERIZATION_PROPERTIES_KHR\"";
        break;
    case 1000184000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_CALIBRATED_TIMESTAMP_INFO_KHR\"";
        break;
    case 1000544000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_EXPECT_ASSUME_FEATURES_KHR\"";
        break;
    case 1000545000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_6_FEATURES_KHR\"";
        break;
    case 1000545001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_6_PROPERTIES_KHR\"";
        break;
    case 1000545002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BIND_MEMORY_STATUS_KHR\"";
        break;
    case 1000545003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BIND_DESCRIPTOR_SETS_INFO_KHR\"";
        break;
    case 1000545004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PUSH_CONSTANTS_INFO_KHR\"";
        break;
    case 1000545005:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PUSH_DESCRIPTOR_SET_INFO_KHR\"";
        break;
    case 1000545006:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PUSH_DESCRIPTOR_SET_WITH_TEMPLATE_INFO_KHR\"";
        break;
    case 1000545007:
        settings.stream() << "\"VK_STRUCTURE_TYPE_SET_DESCRIPTOR_BUFFER_OFFSETS_INFO_EXT\"";
        break;
    case 1000545008:
        settings.stream() << "\"VK_STRUCTURE_TYPE_BIND_DESCRIPTOR_BUFFER_EMBEDDED_SAMPLERS_INFO_EXT\"";
        break;
    case 1000546000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_POOL_OVERALLOCATION_FEATURES_NV\"";
        break;
    case 1000555000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAW_ACCESS_CHAINS_FEATURES_NV\"";
        break;
    case 1000558000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_RELAXED_EXTENDED_INSTRUCTION_FEATURES_KHR\"";
        break;
    case 1000562000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_7_FEATURES_KHR\"";
        break;
    case 1000562001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_7_PROPERTIES_KHR\"";
        break;
    case 1000562002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LAYERED_API_PROPERTIES_LIST_KHR\"";
        break;
    case 1000562003:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LAYERED_API_PROPERTIES_KHR\"";
        break;
    case 1000562004:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LAYERED_API_VULKAN_PROPERTIES_KHR\"";
        break;
    case 1000563000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_ATOMIC_FLOAT16_VECTOR_FEATURES_NV\"";
        break;
    case 1000564000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_REPLICATED_COMPOSITES_FEATURES_EXT\"";
        break;
    case 1000568000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_VALIDATION_FEATURES_NV\"";
        break;
    case 1000575000:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_ALIGNMENT_CONTROL_FEATURES_MESA\"";
        break;
    case 1000575001:
        settings.stream() << "\"VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_ALIGNMENT_CONTROL_PROPERTIES_MESA\"";
        break;
    case 1000575002:
        settings.stream() << "\"VK_STRUCTURE_TYPE_IMAGE_ALIGNMENT_CONTROL_CREATE_INFO_MESA\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPipelineCacheHeaderVersion(VkPipelineCacheHeaderVersion object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 1:
        settings.stream() << "\"VK_PIPELINE_CACHE_HEADER_VERSION_ONE\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkImageLayout(VkImageLayout object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_IMAGE_LAYOUT_UNDEFINED\"";
        break;
    case 1:
        settings.stream() << "\"VK_IMAGE_LAYOUT_GENERAL\"";
        break;
    case 2:
        settings.stream() << "\"VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL\"";
        break;
    case 3:
        settings.stream() << "\"VK_IMAGE_LAYOUT_DEPTH_STENCIL_ATTACHMENT_OPTIMAL\"";
        break;
    case 4:
        settings.stream() << "\"VK_IMAGE_LAYOUT_DEPTH_STENCIL_READ_ONLY_OPTIMAL\"";
        break;
    case 5:
        settings.stream() << "\"VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL\"";
        break;
    case 6:
        settings.stream() << "\"VK_IMAGE_LAYOUT_TRANSFER_SRC_OPTIMAL\"";
        break;
    case 7:
        settings.stream() << "\"VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL\"";
        break;
    case 8:
        settings.stream() << "\"VK_IMAGE_LAYOUT_PREINITIALIZED\"";
        break;
    case 1000117000:
        settings.stream() << "\"VK_IMAGE_LAYOUT_DEPTH_READ_ONLY_STENCIL_ATTACHMENT_OPTIMAL\"";
        break;
    case 1000117001:
        settings.stream() << "\"VK_IMAGE_LAYOUT_DEPTH_ATTACHMENT_STENCIL_READ_ONLY_OPTIMAL\"";
        break;
    case 1000241000:
        settings.stream() << "\"VK_IMAGE_LAYOUT_DEPTH_ATTACHMENT_OPTIMAL\"";
        break;
    case 1000241001:
        settings.stream() << "\"VK_IMAGE_LAYOUT_DEPTH_READ_ONLY_OPTIMAL\"";
        break;
    case 1000241002:
        settings.stream() << "\"VK_IMAGE_LAYOUT_STENCIL_ATTACHMENT_OPTIMAL\"";
        break;
    case 1000241003:
        settings.stream() << "\"VK_IMAGE_LAYOUT_STENCIL_READ_ONLY_OPTIMAL\"";
        break;
    case 1000314000:
        settings.stream() << "\"VK_IMAGE_LAYOUT_READ_ONLY_OPTIMAL\"";
        break;
    case 1000314001:
        settings.stream() << "\"VK_IMAGE_LAYOUT_ATTACHMENT_OPTIMAL\"";
        break;
    case 1000001002:
        settings.stream() << "\"VK_IMAGE_LAYOUT_PRESENT_SRC_KHR\"";
        break;
    case 1000024000:
        settings.stream() << "\"VK_IMAGE_LAYOUT_VIDEO_DECODE_DST_KHR\"";
        break;
    case 1000024001:
        settings.stream() << "\"VK_IMAGE_LAYOUT_VIDEO_DECODE_SRC_KHR\"";
        break;
    case 1000024002:
        settings.stream() << "\"VK_IMAGE_LAYOUT_VIDEO_DECODE_DPB_KHR\"";
        break;
    case 1000111000:
        settings.stream() << "\"VK_IMAGE_LAYOUT_SHARED_PRESENT_KHR\"";
        break;
    case 1000218000:
        settings.stream() << "\"VK_IMAGE_LAYOUT_FRAGMENT_DENSITY_MAP_OPTIMAL_EXT\"";
        break;
    case 1000164003:
        settings.stream() << "\"VK_IMAGE_LAYOUT_FRAGMENT_SHADING_RATE_ATTACHMENT_OPTIMAL_KHR\"";
        break;
    case 1000232000:
        settings.stream() << "\"VK_IMAGE_LAYOUT_RENDERING_LOCAL_READ_KHR\"";
        break;
    case 1000299000:
        settings.stream() << "\"VK_IMAGE_LAYOUT_VIDEO_ENCODE_DST_KHR\"";
        break;
    case 1000299001:
        settings.stream() << "\"VK_IMAGE_LAYOUT_VIDEO_ENCODE_SRC_KHR\"";
        break;
    case 1000299002:
        settings.stream() << "\"VK_IMAGE_LAYOUT_VIDEO_ENCODE_DPB_KHR\"";
        break;
    case 1000339000:
        settings.stream() << "\"VK_IMAGE_LAYOUT_ATTACHMENT_FEEDBACK_LOOP_OPTIMAL_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkObjectType(VkObjectType object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_OBJECT_TYPE_UNKNOWN\"";
        break;
    case 1:
        settings.stream() << "\"VK_OBJECT_TYPE_INSTANCE\"";
        break;
    case 2:
        settings.stream() << "\"VK_OBJECT_TYPE_PHYSICAL_DEVICE\"";
        break;
    case 3:
        settings.stream() << "\"VK_OBJECT_TYPE_DEVICE\"";
        break;
    case 4:
        settings.stream() << "\"VK_OBJECT_TYPE_QUEUE\"";
        break;
    case 5:
        settings.stream() << "\"VK_OBJECT_TYPE_SEMAPHORE\"";
        break;
    case 6:
        settings.stream() << "\"VK_OBJECT_TYPE_COMMAND_BUFFER\"";
        break;
    case 7:
        settings.stream() << "\"VK_OBJECT_TYPE_FENCE\"";
        break;
    case 8:
        settings.stream() << "\"VK_OBJECT_TYPE_DEVICE_MEMORY\"";
        break;
    case 9:
        settings.stream() << "\"VK_OBJECT_TYPE_BUFFER\"";
        break;
    case 10:
        settings.stream() << "\"VK_OBJECT_TYPE_IMAGE\"";
        break;
    case 11:
        settings.stream() << "\"VK_OBJECT_TYPE_EVENT\"";
        break;
    case 12:
        settings.stream() << "\"VK_OBJECT_TYPE_QUERY_POOL\"";
        break;
    case 13:
        settings.stream() << "\"VK_OBJECT_TYPE_BUFFER_VIEW\"";
        break;
    case 14:
        settings.stream() << "\"VK_OBJECT_TYPE_IMAGE_VIEW\"";
        break;
    case 15:
        settings.stream() << "\"VK_OBJECT_TYPE_SHADER_MODULE\"";
        break;
    case 16:
        settings.stream() << "\"VK_OBJECT_TYPE_PIPELINE_CACHE\"";
        break;
    case 17:
        settings.stream() << "\"VK_OBJECT_TYPE_PIPELINE_LAYOUT\"";
        break;
    case 18:
        settings.stream() << "\"VK_OBJECT_TYPE_RENDER_PASS\"";
        break;
    case 19:
        settings.stream() << "\"VK_OBJECT_TYPE_PIPELINE\"";
        break;
    case 20:
        settings.stream() << "\"VK_OBJECT_TYPE_DESCRIPTOR_SET_LAYOUT\"";
        break;
    case 21:
        settings.stream() << "\"VK_OBJECT_TYPE_SAMPLER\"";
        break;
    case 22:
        settings.stream() << "\"VK_OBJECT_TYPE_DESCRIPTOR_POOL\"";
        break;
    case 23:
        settings.stream() << "\"VK_OBJECT_TYPE_DESCRIPTOR_SET\"";
        break;
    case 24:
        settings.stream() << "\"VK_OBJECT_TYPE_FRAMEBUFFER\"";
        break;
    case 25:
        settings.stream() << "\"VK_OBJECT_TYPE_COMMAND_POOL\"";
        break;
    case 1000156000:
        settings.stream() << "\"VK_OBJECT_TYPE_SAMPLER_YCBCR_CONVERSION\"";
        break;
    case 1000085000:
        settings.stream() << "\"VK_OBJECT_TYPE_DESCRIPTOR_UPDATE_TEMPLATE\"";
        break;
    case 1000295000:
        settings.stream() << "\"VK_OBJECT_TYPE_PRIVATE_DATA_SLOT\"";
        break;
    case 1000000000:
        settings.stream() << "\"VK_OBJECT_TYPE_SURFACE_KHR\"";
        break;
    case 1000001000:
        settings.stream() << "\"VK_OBJECT_TYPE_SWAPCHAIN_KHR\"";
        break;
    case 1000002000:
        settings.stream() << "\"VK_OBJECT_TYPE_DISPLAY_KHR\"";
        break;
    case 1000002001:
        settings.stream() << "\"VK_OBJECT_TYPE_DISPLAY_MODE_KHR\"";
        break;
    case 1000011000:
        settings.stream() << "\"VK_OBJECT_TYPE_DEBUG_REPORT_CALLBACK_EXT\"";
        break;
    case 1000023000:
        settings.stream() << "\"VK_OBJECT_TYPE_VIDEO_SESSION_KHR\"";
        break;
    case 1000023001:
        settings.stream() << "\"VK_OBJECT_TYPE_VIDEO_SESSION_PARAMETERS_KHR\"";
        break;
    case 1000029000:
        settings.stream() << "\"VK_OBJECT_TYPE_CU_MODULE_NVX\"";
        break;
    case 1000029001:
        settings.stream() << "\"VK_OBJECT_TYPE_CU_FUNCTION_NVX\"";
        break;
    case 1000128000:
        settings.stream() << "\"VK_OBJECT_TYPE_DEBUG_UTILS_MESSENGER_EXT\"";
        break;
    case 1000150000:
        settings.stream() << "\"VK_OBJECT_TYPE_ACCELERATION_STRUCTURE_KHR\"";
        break;
    case 1000160000:
        settings.stream() << "\"VK_OBJECT_TYPE_VALIDATION_CACHE_EXT\"";
        break;
    case 1000165000:
        settings.stream() << "\"VK_OBJECT_TYPE_ACCELERATION_STRUCTURE_NV\"";
        break;
    case 1000210000:
        settings.stream() << "\"VK_OBJECT_TYPE_PERFORMANCE_CONFIGURATION_INTEL\"";
        break;
    case 1000268000:
        settings.stream() << "\"VK_OBJECT_TYPE_DEFERRED_OPERATION_KHR\"";
        break;
    case 1000277000:
        settings.stream() << "\"VK_OBJECT_TYPE_INDIRECT_COMMANDS_LAYOUT_NV\"";
        break;
    case 1000307000:
        settings.stream() << "\"VK_OBJECT_TYPE_CUDA_MODULE_NV\"";
        break;
    case 1000307001:
        settings.stream() << "\"VK_OBJECT_TYPE_CUDA_FUNCTION_NV\"";
        break;
    case 1000366000:
        settings.stream() << "\"VK_OBJECT_TYPE_BUFFER_COLLECTION_FUCHSIA\"";
        break;
    case 1000396000:
        settings.stream() << "\"VK_OBJECT_TYPE_MICROMAP_EXT\"";
        break;
    case 1000464000:
        settings.stream() << "\"VK_OBJECT_TYPE_OPTICAL_FLOW_SESSION_NV\"";
        break;
    case 1000482000:
        settings.stream() << "\"VK_OBJECT_TYPE_SHADER_EXT\"";
        break;
    case 1000489000:
        settings.stream() << "\"VK_OBJECT_TYPE_SEMAPHORE_SCI_SYNC_POOL_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkVendorId(VkVendorId object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0x10000:
        settings.stream() << "\"VK_VENDOR_ID_KHRONOS\"";
        break;
    case 0x10001:
        settings.stream() << "\"VK_VENDOR_ID_VIV\"";
        break;
    case 0x10002:
        settings.stream() << "\"VK_VENDOR_ID_VSI\"";
        break;
    case 0x10003:
        settings.stream() << "\"VK_VENDOR_ID_KAZAN\"";
        break;
    case 0x10004:
        settings.stream() << "\"VK_VENDOR_ID_CODEPLAY\"";
        break;
    case 0x10005:
        settings.stream() << "\"VK_VENDOR_ID_MESA\"";
        break;
    case 0x10006:
        settings.stream() << "\"VK_VENDOR_ID_POCL\"";
        break;
    case 0x10007:
        settings.stream() << "\"VK_VENDOR_ID_MOBILEYE\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkSystemAllocationScope(VkSystemAllocationScope object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SYSTEM_ALLOCATION_SCOPE_COMMAND\"";
        break;
    case 1:
        settings.stream() << "\"VK_SYSTEM_ALLOCATION_SCOPE_OBJECT\"";
        break;
    case 2:
        settings.stream() << "\"VK_SYSTEM_ALLOCATION_SCOPE_CACHE\"";
        break;
    case 3:
        settings.stream() << "\"VK_SYSTEM_ALLOCATION_SCOPE_DEVICE\"";
        break;
    case 4:
        settings.stream() << "\"VK_SYSTEM_ALLOCATION_SCOPE_INSTANCE\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkInternalAllocationType(VkInternalAllocationType object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_INTERNAL_ALLOCATION_TYPE_EXECUTABLE\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkFormat(VkFormat object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_FORMAT_UNDEFINED\"";
        break;
    case 1:
        settings.stream() << "\"VK_FORMAT_R4G4_UNORM_PACK8\"";
        break;
    case 2:
        settings.stream() << "\"VK_FORMAT_R4G4B4A4_UNORM_PACK16\"";
        break;
    case 3:
        settings.stream() << "\"VK_FORMAT_B4G4R4A4_UNORM_PACK16\"";
        break;
    case 4:
        settings.stream() << "\"VK_FORMAT_R5G6B5_UNORM_PACK16\"";
        break;
    case 5:
        settings.stream() << "\"VK_FORMAT_B5G6R5_UNORM_PACK16\"";
        break;
    case 6:
        settings.stream() << "\"VK_FORMAT_R5G5B5A1_UNORM_PACK16\"";
        break;
    case 7:
        settings.stream() << "\"VK_FORMAT_B5G5R5A1_UNORM_PACK16\"";
        break;
    case 8:
        settings.stream() << "\"VK_FORMAT_A1R5G5B5_UNORM_PACK16\"";
        break;
    case 9:
        settings.stream() << "\"VK_FORMAT_R8_UNORM\"";
        break;
    case 10:
        settings.stream() << "\"VK_FORMAT_R8_SNORM\"";
        break;
    case 11:
        settings.stream() << "\"VK_FORMAT_R8_USCALED\"";
        break;
    case 12:
        settings.stream() << "\"VK_FORMAT_R8_SSCALED\"";
        break;
    case 13:
        settings.stream() << "\"VK_FORMAT_R8_UINT\"";
        break;
    case 14:
        settings.stream() << "\"VK_FORMAT_R8_SINT\"";
        break;
    case 15:
        settings.stream() << "\"VK_FORMAT_R8_SRGB\"";
        break;
    case 16:
        settings.stream() << "\"VK_FORMAT_R8G8_UNORM\"";
        break;
    case 17:
        settings.stream() << "\"VK_FORMAT_R8G8_SNORM\"";
        break;
    case 18:
        settings.stream() << "\"VK_FORMAT_R8G8_USCALED\"";
        break;
    case 19:
        settings.stream() << "\"VK_FORMAT_R8G8_SSCALED\"";
        break;
    case 20:
        settings.stream() << "\"VK_FORMAT_R8G8_UINT\"";
        break;
    case 21:
        settings.stream() << "\"VK_FORMAT_R8G8_SINT\"";
        break;
    case 22:
        settings.stream() << "\"VK_FORMAT_R8G8_SRGB\"";
        break;
    case 23:
        settings.stream() << "\"VK_FORMAT_R8G8B8_UNORM\"";
        break;
    case 24:
        settings.stream() << "\"VK_FORMAT_R8G8B8_SNORM\"";
        break;
    case 25:
        settings.stream() << "\"VK_FORMAT_R8G8B8_USCALED\"";
        break;
    case 26:
        settings.stream() << "\"VK_FORMAT_R8G8B8_SSCALED\"";
        break;
    case 27:
        settings.stream() << "\"VK_FORMAT_R8G8B8_UINT\"";
        break;
    case 28:
        settings.stream() << "\"VK_FORMAT_R8G8B8_SINT\"";
        break;
    case 29:
        settings.stream() << "\"VK_FORMAT_R8G8B8_SRGB\"";
        break;
    case 30:
        settings.stream() << "\"VK_FORMAT_B8G8R8_UNORM\"";
        break;
    case 31:
        settings.stream() << "\"VK_FORMAT_B8G8R8_SNORM\"";
        break;
    case 32:
        settings.stream() << "\"VK_FORMAT_B8G8R8_USCALED\"";
        break;
    case 33:
        settings.stream() << "\"VK_FORMAT_B8G8R8_SSCALED\"";
        break;
    case 34:
        settings.stream() << "\"VK_FORMAT_B8G8R8_UINT\"";
        break;
    case 35:
        settings.stream() << "\"VK_FORMAT_B8G8R8_SINT\"";
        break;
    case 36:
        settings.stream() << "\"VK_FORMAT_B8G8R8_SRGB\"";
        break;
    case 37:
        settings.stream() << "\"VK_FORMAT_R8G8B8A8_UNORM\"";
        break;
    case 38:
        settings.stream() << "\"VK_FORMAT_R8G8B8A8_SNORM\"";
        break;
    case 39:
        settings.stream() << "\"VK_FORMAT_R8G8B8A8_USCALED\"";
        break;
    case 40:
        settings.stream() << "\"VK_FORMAT_R8G8B8A8_SSCALED\"";
        break;
    case 41:
        settings.stream() << "\"VK_FORMAT_R8G8B8A8_UINT\"";
        break;
    case 42:
        settings.stream() << "\"VK_FORMAT_R8G8B8A8_SINT\"";
        break;
    case 43:
        settings.stream() << "\"VK_FORMAT_R8G8B8A8_SRGB\"";
        break;
    case 44:
        settings.stream() << "\"VK_FORMAT_B8G8R8A8_UNORM\"";
        break;
    case 45:
        settings.stream() << "\"VK_FORMAT_B8G8R8A8_SNORM\"";
        break;
    case 46:
        settings.stream() << "\"VK_FORMAT_B8G8R8A8_USCALED\"";
        break;
    case 47:
        settings.stream() << "\"VK_FORMAT_B8G8R8A8_SSCALED\"";
        break;
    case 48:
        settings.stream() << "\"VK_FORMAT_B8G8R8A8_UINT\"";
        break;
    case 49:
        settings.stream() << "\"VK_FORMAT_B8G8R8A8_SINT\"";
        break;
    case 50:
        settings.stream() << "\"VK_FORMAT_B8G8R8A8_SRGB\"";
        break;
    case 51:
        settings.stream() << "\"VK_FORMAT_A8B8G8R8_UNORM_PACK32\"";
        break;
    case 52:
        settings.stream() << "\"VK_FORMAT_A8B8G8R8_SNORM_PACK32\"";
        break;
    case 53:
        settings.stream() << "\"VK_FORMAT_A8B8G8R8_USCALED_PACK32\"";
        break;
    case 54:
        settings.stream() << "\"VK_FORMAT_A8B8G8R8_SSCALED_PACK32\"";
        break;
    case 55:
        settings.stream() << "\"VK_FORMAT_A8B8G8R8_UINT_PACK32\"";
        break;
    case 56:
        settings.stream() << "\"VK_FORMAT_A8B8G8R8_SINT_PACK32\"";
        break;
    case 57:
        settings.stream() << "\"VK_FORMAT_A8B8G8R8_SRGB_PACK32\"";
        break;
    case 58:
        settings.stream() << "\"VK_FORMAT_A2R10G10B10_UNORM_PACK32\"";
        break;
    case 59:
        settings.stream() << "\"VK_FORMAT_A2R10G10B10_SNORM_PACK32\"";
        break;
    case 60:
        settings.stream() << "\"VK_FORMAT_A2R10G10B10_USCALED_PACK32\"";
        break;
    case 61:
        settings.stream() << "\"VK_FORMAT_A2R10G10B10_SSCALED_PACK32\"";
        break;
    case 62:
        settings.stream() << "\"VK_FORMAT_A2R10G10B10_UINT_PACK32\"";
        break;
    case 63:
        settings.stream() << "\"VK_FORMAT_A2R10G10B10_SINT_PACK32\"";
        break;
    case 64:
        settings.stream() << "\"VK_FORMAT_A2B10G10R10_UNORM_PACK32\"";
        break;
    case 65:
        settings.stream() << "\"VK_FORMAT_A2B10G10R10_SNORM_PACK32\"";
        break;
    case 66:
        settings.stream() << "\"VK_FORMAT_A2B10G10R10_USCALED_PACK32\"";
        break;
    case 67:
        settings.stream() << "\"VK_FORMAT_A2B10G10R10_SSCALED_PACK32\"";
        break;
    case 68:
        settings.stream() << "\"VK_FORMAT_A2B10G10R10_UINT_PACK32\"";
        break;
    case 69:
        settings.stream() << "\"VK_FORMAT_A2B10G10R10_SINT_PACK32\"";
        break;
    case 70:
        settings.stream() << "\"VK_FORMAT_R16_UNORM\"";
        break;
    case 71:
        settings.stream() << "\"VK_FORMAT_R16_SNORM\"";
        break;
    case 72:
        settings.stream() << "\"VK_FORMAT_R16_USCALED\"";
        break;
    case 73:
        settings.stream() << "\"VK_FORMAT_R16_SSCALED\"";
        break;
    case 74:
        settings.stream() << "\"VK_FORMAT_R16_UINT\"";
        break;
    case 75:
        settings.stream() << "\"VK_FORMAT_R16_SINT\"";
        break;
    case 76:
        settings.stream() << "\"VK_FORMAT_R16_SFLOAT\"";
        break;
    case 77:
        settings.stream() << "\"VK_FORMAT_R16G16_UNORM\"";
        break;
    case 78:
        settings.stream() << "\"VK_FORMAT_R16G16_SNORM\"";
        break;
    case 79:
        settings.stream() << "\"VK_FORMAT_R16G16_USCALED\"";
        break;
    case 80:
        settings.stream() << "\"VK_FORMAT_R16G16_SSCALED\"";
        break;
    case 81:
        settings.stream() << "\"VK_FORMAT_R16G16_UINT\"";
        break;
    case 82:
        settings.stream() << "\"VK_FORMAT_R16G16_SINT\"";
        break;
    case 83:
        settings.stream() << "\"VK_FORMAT_R16G16_SFLOAT\"";
        break;
    case 84:
        settings.stream() << "\"VK_FORMAT_R16G16B16_UNORM\"";
        break;
    case 85:
        settings.stream() << "\"VK_FORMAT_R16G16B16_SNORM\"";
        break;
    case 86:
        settings.stream() << "\"VK_FORMAT_R16G16B16_USCALED\"";
        break;
    case 87:
        settings.stream() << "\"VK_FORMAT_R16G16B16_SSCALED\"";
        break;
    case 88:
        settings.stream() << "\"VK_FORMAT_R16G16B16_UINT\"";
        break;
    case 89:
        settings.stream() << "\"VK_FORMAT_R16G16B16_SINT\"";
        break;
    case 90:
        settings.stream() << "\"VK_FORMAT_R16G16B16_SFLOAT\"";
        break;
    case 91:
        settings.stream() << "\"VK_FORMAT_R16G16B16A16_UNORM\"";
        break;
    case 92:
        settings.stream() << "\"VK_FORMAT_R16G16B16A16_SNORM\"";
        break;
    case 93:
        settings.stream() << "\"VK_FORMAT_R16G16B16A16_USCALED\"";
        break;
    case 94:
        settings.stream() << "\"VK_FORMAT_R16G16B16A16_SSCALED\"";
        break;
    case 95:
        settings.stream() << "\"VK_FORMAT_R16G16B16A16_UINT\"";
        break;
    case 96:
        settings.stream() << "\"VK_FORMAT_R16G16B16A16_SINT\"";
        break;
    case 97:
        settings.stream() << "\"VK_FORMAT_R16G16B16A16_SFLOAT\"";
        break;
    case 98:
        settings.stream() << "\"VK_FORMAT_R32_UINT\"";
        break;
    case 99:
        settings.stream() << "\"VK_FORMAT_R32_SINT\"";
        break;
    case 100:
        settings.stream() << "\"VK_FORMAT_R32_SFLOAT\"";
        break;
    case 101:
        settings.stream() << "\"VK_FORMAT_R32G32_UINT\"";
        break;
    case 102:
        settings.stream() << "\"VK_FORMAT_R32G32_SINT\"";
        break;
    case 103:
        settings.stream() << "\"VK_FORMAT_R32G32_SFLOAT\"";
        break;
    case 104:
        settings.stream() << "\"VK_FORMAT_R32G32B32_UINT\"";
        break;
    case 105:
        settings.stream() << "\"VK_FORMAT_R32G32B32_SINT\"";
        break;
    case 106:
        settings.stream() << "\"VK_FORMAT_R32G32B32_SFLOAT\"";
        break;
    case 107:
        settings.stream() << "\"VK_FORMAT_R32G32B32A32_UINT\"";
        break;
    case 108:
        settings.stream() << "\"VK_FORMAT_R32G32B32A32_SINT\"";
        break;
    case 109:
        settings.stream() << "\"VK_FORMAT_R32G32B32A32_SFLOAT\"";
        break;
    case 110:
        settings.stream() << "\"VK_FORMAT_R64_UINT\"";
        break;
    case 111:
        settings.stream() << "\"VK_FORMAT_R64_SINT\"";
        break;
    case 112:
        settings.stream() << "\"VK_FORMAT_R64_SFLOAT\"";
        break;
    case 113:
        settings.stream() << "\"VK_FORMAT_R64G64_UINT\"";
        break;
    case 114:
        settings.stream() << "\"VK_FORMAT_R64G64_SINT\"";
        break;
    case 115:
        settings.stream() << "\"VK_FORMAT_R64G64_SFLOAT\"";
        break;
    case 116:
        settings.stream() << "\"VK_FORMAT_R64G64B64_UINT\"";
        break;
    case 117:
        settings.stream() << "\"VK_FORMAT_R64G64B64_SINT\"";
        break;
    case 118:
        settings.stream() << "\"VK_FORMAT_R64G64B64_SFLOAT\"";
        break;
    case 119:
        settings.stream() << "\"VK_FORMAT_R64G64B64A64_UINT\"";
        break;
    case 120:
        settings.stream() << "\"VK_FORMAT_R64G64B64A64_SINT\"";
        break;
    case 121:
        settings.stream() << "\"VK_FORMAT_R64G64B64A64_SFLOAT\"";
        break;
    case 122:
        settings.stream() << "\"VK_FORMAT_B10G11R11_UFLOAT_PACK32\"";
        break;
    case 123:
        settings.stream() << "\"VK_FORMAT_E5B9G9R9_UFLOAT_PACK32\"";
        break;
    case 124:
        settings.stream() << "\"VK_FORMAT_D16_UNORM\"";
        break;
    case 125:
        settings.stream() << "\"VK_FORMAT_X8_D24_UNORM_PACK32\"";
        break;
    case 126:
        settings.stream() << "\"VK_FORMAT_D32_SFLOAT\"";
        break;
    case 127:
        settings.stream() << "\"VK_FORMAT_S8_UINT\"";
        break;
    case 128:
        settings.stream() << "\"VK_FORMAT_D16_UNORM_S8_UINT\"";
        break;
    case 129:
        settings.stream() << "\"VK_FORMAT_D24_UNORM_S8_UINT\"";
        break;
    case 130:
        settings.stream() << "\"VK_FORMAT_D32_SFLOAT_S8_UINT\"";
        break;
    case 131:
        settings.stream() << "\"VK_FORMAT_BC1_RGB_UNORM_BLOCK\"";
        break;
    case 132:
        settings.stream() << "\"VK_FORMAT_BC1_RGB_SRGB_BLOCK\"";
        break;
    case 133:
        settings.stream() << "\"VK_FORMAT_BC1_RGBA_UNORM_BLOCK\"";
        break;
    case 134:
        settings.stream() << "\"VK_FORMAT_BC1_RGBA_SRGB_BLOCK\"";
        break;
    case 135:
        settings.stream() << "\"VK_FORMAT_BC2_UNORM_BLOCK\"";
        break;
    case 136:
        settings.stream() << "\"VK_FORMAT_BC2_SRGB_BLOCK\"";
        break;
    case 137:
        settings.stream() << "\"VK_FORMAT_BC3_UNORM_BLOCK\"";
        break;
    case 138:
        settings.stream() << "\"VK_FORMAT_BC3_SRGB_BLOCK\"";
        break;
    case 139:
        settings.stream() << "\"VK_FORMAT_BC4_UNORM_BLOCK\"";
        break;
    case 140:
        settings.stream() << "\"VK_FORMAT_BC4_SNORM_BLOCK\"";
        break;
    case 141:
        settings.stream() << "\"VK_FORMAT_BC5_UNORM_BLOCK\"";
        break;
    case 142:
        settings.stream() << "\"VK_FORMAT_BC5_SNORM_BLOCK\"";
        break;
    case 143:
        settings.stream() << "\"VK_FORMAT_BC6H_UFLOAT_BLOCK\"";
        break;
    case 144:
        settings.stream() << "\"VK_FORMAT_BC6H_SFLOAT_BLOCK\"";
        break;
    case 145:
        settings.stream() << "\"VK_FORMAT_BC7_UNORM_BLOCK\"";
        break;
    case 146:
        settings.stream() << "\"VK_FORMAT_BC7_SRGB_BLOCK\"";
        break;
    case 147:
        settings.stream() << "\"VK_FORMAT_ETC2_R8G8B8_UNORM_BLOCK\"";
        break;
    case 148:
        settings.stream() << "\"VK_FORMAT_ETC2_R8G8B8_SRGB_BLOCK\"";
        break;
    case 149:
        settings.stream() << "\"VK_FORMAT_ETC2_R8G8B8A1_UNORM_BLOCK\"";
        break;
    case 150:
        settings.stream() << "\"VK_FORMAT_ETC2_R8G8B8A1_SRGB_BLOCK\"";
        break;
    case 151:
        settings.stream() << "\"VK_FORMAT_ETC2_R8G8B8A8_UNORM_BLOCK\"";
        break;
    case 152:
        settings.stream() << "\"VK_FORMAT_ETC2_R8G8B8A8_SRGB_BLOCK\"";
        break;
    case 153:
        settings.stream() << "\"VK_FORMAT_EAC_R11_UNORM_BLOCK\"";
        break;
    case 154:
        settings.stream() << "\"VK_FORMAT_EAC_R11_SNORM_BLOCK\"";
        break;
    case 155:
        settings.stream() << "\"VK_FORMAT_EAC_R11G11_UNORM_BLOCK\"";
        break;
    case 156:
        settings.stream() << "\"VK_FORMAT_EAC_R11G11_SNORM_BLOCK\"";
        break;
    case 157:
        settings.stream() << "\"VK_FORMAT_ASTC_4x4_UNORM_BLOCK\"";
        break;
    case 158:
        settings.stream() << "\"VK_FORMAT_ASTC_4x4_SRGB_BLOCK\"";
        break;
    case 159:
        settings.stream() << "\"VK_FORMAT_ASTC_5x4_UNORM_BLOCK\"";
        break;
    case 160:
        settings.stream() << "\"VK_FORMAT_ASTC_5x4_SRGB_BLOCK\"";
        break;
    case 161:
        settings.stream() << "\"VK_FORMAT_ASTC_5x5_UNORM_BLOCK\"";
        break;
    case 162:
        settings.stream() << "\"VK_FORMAT_ASTC_5x5_SRGB_BLOCK\"";
        break;
    case 163:
        settings.stream() << "\"VK_FORMAT_ASTC_6x5_UNORM_BLOCK\"";
        break;
    case 164:
        settings.stream() << "\"VK_FORMAT_ASTC_6x5_SRGB_BLOCK\"";
        break;
    case 165:
        settings.stream() << "\"VK_FORMAT_ASTC_6x6_UNORM_BLOCK\"";
        break;
    case 166:
        settings.stream() << "\"VK_FORMAT_ASTC_6x6_SRGB_BLOCK\"";
        break;
    case 167:
        settings.stream() << "\"VK_FORMAT_ASTC_8x5_UNORM_BLOCK\"";
        break;
    case 168:
        settings.stream() << "\"VK_FORMAT_ASTC_8x5_SRGB_BLOCK\"";
        break;
    case 169:
        settings.stream() << "\"VK_FORMAT_ASTC_8x6_UNORM_BLOCK\"";
        break;
    case 170:
        settings.stream() << "\"VK_FORMAT_ASTC_8x6_SRGB_BLOCK\"";
        break;
    case 171:
        settings.stream() << "\"VK_FORMAT_ASTC_8x8_UNORM_BLOCK\"";
        break;
    case 172:
        settings.stream() << "\"VK_FORMAT_ASTC_8x8_SRGB_BLOCK\"";
        break;
    case 173:
        settings.stream() << "\"VK_FORMAT_ASTC_10x5_UNORM_BLOCK\"";
        break;
    case 174:
        settings.stream() << "\"VK_FORMAT_ASTC_10x5_SRGB_BLOCK\"";
        break;
    case 175:
        settings.stream() << "\"VK_FORMAT_ASTC_10x6_UNORM_BLOCK\"";
        break;
    case 176:
        settings.stream() << "\"VK_FORMAT_ASTC_10x6_SRGB_BLOCK\"";
        break;
    case 177:
        settings.stream() << "\"VK_FORMAT_ASTC_10x8_UNORM_BLOCK\"";
        break;
    case 178:
        settings.stream() << "\"VK_FORMAT_ASTC_10x8_SRGB_BLOCK\"";
        break;
    case 179:
        settings.stream() << "\"VK_FORMAT_ASTC_10x10_UNORM_BLOCK\"";
        break;
    case 180:
        settings.stream() << "\"VK_FORMAT_ASTC_10x10_SRGB_BLOCK\"";
        break;
    case 181:
        settings.stream() << "\"VK_FORMAT_ASTC_12x10_UNORM_BLOCK\"";
        break;
    case 182:
        settings.stream() << "\"VK_FORMAT_ASTC_12x10_SRGB_BLOCK\"";
        break;
    case 183:
        settings.stream() << "\"VK_FORMAT_ASTC_12x12_UNORM_BLOCK\"";
        break;
    case 184:
        settings.stream() << "\"VK_FORMAT_ASTC_12x12_SRGB_BLOCK\"";
        break;
    case 1000156000:
        settings.stream() << "\"VK_FORMAT_G8B8G8R8_422_UNORM\"";
        break;
    case 1000156001:
        settings.stream() << "\"VK_FORMAT_B8G8R8G8_422_UNORM\"";
        break;
    case 1000156002:
        settings.stream() << "\"VK_FORMAT_G8_B8_R8_3PLANE_420_UNORM\"";
        break;
    case 1000156003:
        settings.stream() << "\"VK_FORMAT_G8_B8R8_2PLANE_420_UNORM\"";
        break;
    case 1000156004:
        settings.stream() << "\"VK_FORMAT_G8_B8_R8_3PLANE_422_UNORM\"";
        break;
    case 1000156005:
        settings.stream() << "\"VK_FORMAT_G8_B8R8_2PLANE_422_UNORM\"";
        break;
    case 1000156006:
        settings.stream() << "\"VK_FORMAT_G8_B8_R8_3PLANE_444_UNORM\"";
        break;
    case 1000156007:
        settings.stream() << "\"VK_FORMAT_R10X6_UNORM_PACK16\"";
        break;
    case 1000156008:
        settings.stream() << "\"VK_FORMAT_R10X6G10X6_UNORM_2PACK16\"";
        break;
    case 1000156009:
        settings.stream() << "\"VK_FORMAT_R10X6G10X6B10X6A10X6_UNORM_4PACK16\"";
        break;
    case 1000156010:
        settings.stream() << "\"VK_FORMAT_G10X6B10X6G10X6R10X6_422_UNORM_4PACK16\"";
        break;
    case 1000156011:
        settings.stream() << "\"VK_FORMAT_B10X6G10X6R10X6G10X6_422_UNORM_4PACK16\"";
        break;
    case 1000156012:
        settings.stream() << "\"VK_FORMAT_G10X6_B10X6_R10X6_3PLANE_420_UNORM_3PACK16\"";
        break;
    case 1000156013:
        settings.stream() << "\"VK_FORMAT_G10X6_B10X6R10X6_2PLANE_420_UNORM_3PACK16\"";
        break;
    case 1000156014:
        settings.stream() << "\"VK_FORMAT_G10X6_B10X6_R10X6_3PLANE_422_UNORM_3PACK16\"";
        break;
    case 1000156015:
        settings.stream() << "\"VK_FORMAT_G10X6_B10X6R10X6_2PLANE_422_UNORM_3PACK16\"";
        break;
    case 1000156016:
        settings.stream() << "\"VK_FORMAT_G10X6_B10X6_R10X6_3PLANE_444_UNORM_3PACK16\"";
        break;
    case 1000156017:
        settings.stream() << "\"VK_FORMAT_R12X4_UNORM_PACK16\"";
        break;
    case 1000156018:
        settings.stream() << "\"VK_FORMAT_R12X4G12X4_UNORM_2PACK16\"";
        break;
    case 1000156019:
        settings.stream() << "\"VK_FORMAT_R12X4G12X4B12X4A12X4_UNORM_4PACK16\"";
        break;
    case 1000156020:
        settings.stream() << "\"VK_FORMAT_G12X4B12X4G12X4R12X4_422_UNORM_4PACK16\"";
        break;
    case 1000156021:
        settings.stream() << "\"VK_FORMAT_B12X4G12X4R12X4G12X4_422_UNORM_4PACK16\"";
        break;
    case 1000156022:
        settings.stream() << "\"VK_FORMAT_G12X4_B12X4_R12X4_3PLANE_420_UNORM_3PACK16\"";
        break;
    case 1000156023:
        settings.stream() << "\"VK_FORMAT_G12X4_B12X4R12X4_2PLANE_420_UNORM_3PACK16\"";
        break;
    case 1000156024:
        settings.stream() << "\"VK_FORMAT_G12X4_B12X4_R12X4_3PLANE_422_UNORM_3PACK16\"";
        break;
    case 1000156025:
        settings.stream() << "\"VK_FORMAT_G12X4_B12X4R12X4_2PLANE_422_UNORM_3PACK16\"";
        break;
    case 1000156026:
        settings.stream() << "\"VK_FORMAT_G12X4_B12X4_R12X4_3PLANE_444_UNORM_3PACK16\"";
        break;
    case 1000156027:
        settings.stream() << "\"VK_FORMAT_G16B16G16R16_422_UNORM\"";
        break;
    case 1000156028:
        settings.stream() << "\"VK_FORMAT_B16G16R16G16_422_UNORM\"";
        break;
    case 1000156029:
        settings.stream() << "\"VK_FORMAT_G16_B16_R16_3PLANE_420_UNORM\"";
        break;
    case 1000156030:
        settings.stream() << "\"VK_FORMAT_G16_B16R16_2PLANE_420_UNORM\"";
        break;
    case 1000156031:
        settings.stream() << "\"VK_FORMAT_G16_B16_R16_3PLANE_422_UNORM\"";
        break;
    case 1000156032:
        settings.stream() << "\"VK_FORMAT_G16_B16R16_2PLANE_422_UNORM\"";
        break;
    case 1000156033:
        settings.stream() << "\"VK_FORMAT_G16_B16_R16_3PLANE_444_UNORM\"";
        break;
    case 1000330000:
        settings.stream() << "\"VK_FORMAT_G8_B8R8_2PLANE_444_UNORM\"";
        break;
    case 1000330001:
        settings.stream() << "\"VK_FORMAT_G10X6_B10X6R10X6_2PLANE_444_UNORM_3PACK16\"";
        break;
    case 1000330002:
        settings.stream() << "\"VK_FORMAT_G12X4_B12X4R12X4_2PLANE_444_UNORM_3PACK16\"";
        break;
    case 1000330003:
        settings.stream() << "\"VK_FORMAT_G16_B16R16_2PLANE_444_UNORM\"";
        break;
    case 1000340000:
        settings.stream() << "\"VK_FORMAT_A4R4G4B4_UNORM_PACK16\"";
        break;
    case 1000340001:
        settings.stream() << "\"VK_FORMAT_A4B4G4R4_UNORM_PACK16\"";
        break;
    case 1000066000:
        settings.stream() << "\"VK_FORMAT_ASTC_4x4_SFLOAT_BLOCK\"";
        break;
    case 1000066001:
        settings.stream() << "\"VK_FORMAT_ASTC_5x4_SFLOAT_BLOCK\"";
        break;
    case 1000066002:
        settings.stream() << "\"VK_FORMAT_ASTC_5x5_SFLOAT_BLOCK\"";
        break;
    case 1000066003:
        settings.stream() << "\"VK_FORMAT_ASTC_6x5_SFLOAT_BLOCK\"";
        break;
    case 1000066004:
        settings.stream() << "\"VK_FORMAT_ASTC_6x6_SFLOAT_BLOCK\"";
        break;
    case 1000066005:
        settings.stream() << "\"VK_FORMAT_ASTC_8x5_SFLOAT_BLOCK\"";
        break;
    case 1000066006:
        settings.stream() << "\"VK_FORMAT_ASTC_8x6_SFLOAT_BLOCK\"";
        break;
    case 1000066007:
        settings.stream() << "\"VK_FORMAT_ASTC_8x8_SFLOAT_BLOCK\"";
        break;
    case 1000066008:
        settings.stream() << "\"VK_FORMAT_ASTC_10x5_SFLOAT_BLOCK\"";
        break;
    case 1000066009:
        settings.stream() << "\"VK_FORMAT_ASTC_10x6_SFLOAT_BLOCK\"";
        break;
    case 1000066010:
        settings.stream() << "\"VK_FORMAT_ASTC_10x8_SFLOAT_BLOCK\"";
        break;
    case 1000066011:
        settings.stream() << "\"VK_FORMAT_ASTC_10x10_SFLOAT_BLOCK\"";
        break;
    case 1000066012:
        settings.stream() << "\"VK_FORMAT_ASTC_12x10_SFLOAT_BLOCK\"";
        break;
    case 1000066013:
        settings.stream() << "\"VK_FORMAT_ASTC_12x12_SFLOAT_BLOCK\"";
        break;
    case 1000054000:
        settings.stream() << "\"VK_FORMAT_PVRTC1_2BPP_UNORM_BLOCK_IMG\"";
        break;
    case 1000054001:
        settings.stream() << "\"VK_FORMAT_PVRTC1_4BPP_UNORM_BLOCK_IMG\"";
        break;
    case 1000054002:
        settings.stream() << "\"VK_FORMAT_PVRTC2_2BPP_UNORM_BLOCK_IMG\"";
        break;
    case 1000054003:
        settings.stream() << "\"VK_FORMAT_PVRTC2_4BPP_UNORM_BLOCK_IMG\"";
        break;
    case 1000054004:
        settings.stream() << "\"VK_FORMAT_PVRTC1_2BPP_SRGB_BLOCK_IMG\"";
        break;
    case 1000054005:
        settings.stream() << "\"VK_FORMAT_PVRTC1_4BPP_SRGB_BLOCK_IMG\"";
        break;
    case 1000054006:
        settings.stream() << "\"VK_FORMAT_PVRTC2_2BPP_SRGB_BLOCK_IMG\"";
        break;
    case 1000054007:
        settings.stream() << "\"VK_FORMAT_PVRTC2_4BPP_SRGB_BLOCK_IMG\"";
        break;
    case 1000288000:
        settings.stream() << "\"VK_FORMAT_ASTC_3x3x3_UNORM_BLOCK_EXT\"";
        break;
    case 1000288001:
        settings.stream() << "\"VK_FORMAT_ASTC_3x3x3_SRGB_BLOCK_EXT\"";
        break;
    case 1000288002:
        settings.stream() << "\"VK_FORMAT_ASTC_3x3x3_SFLOAT_BLOCK_EXT\"";
        break;
    case 1000288003:
        settings.stream() << "\"VK_FORMAT_ASTC_4x3x3_UNORM_BLOCK_EXT\"";
        break;
    case 1000288004:
        settings.stream() << "\"VK_FORMAT_ASTC_4x3x3_SRGB_BLOCK_EXT\"";
        break;
    case 1000288005:
        settings.stream() << "\"VK_FORMAT_ASTC_4x3x3_SFLOAT_BLOCK_EXT\"";
        break;
    case 1000288006:
        settings.stream() << "\"VK_FORMAT_ASTC_4x4x3_UNORM_BLOCK_EXT\"";
        break;
    case 1000288007:
        settings.stream() << "\"VK_FORMAT_ASTC_4x4x3_SRGB_BLOCK_EXT\"";
        break;
    case 1000288008:
        settings.stream() << "\"VK_FORMAT_ASTC_4x4x3_SFLOAT_BLOCK_EXT\"";
        break;
    case 1000288009:
        settings.stream() << "\"VK_FORMAT_ASTC_4x4x4_UNORM_BLOCK_EXT\"";
        break;
    case 1000288010:
        settings.stream() << "\"VK_FORMAT_ASTC_4x4x4_SRGB_BLOCK_EXT\"";
        break;
    case 1000288011:
        settings.stream() << "\"VK_FORMAT_ASTC_4x4x4_SFLOAT_BLOCK_EXT\"";
        break;
    case 1000288012:
        settings.stream() << "\"VK_FORMAT_ASTC_5x4x4_UNORM_BLOCK_EXT\"";
        break;
    case 1000288013:
        settings.stream() << "\"VK_FORMAT_ASTC_5x4x4_SRGB_BLOCK_EXT\"";
        break;
    case 1000288014:
        settings.stream() << "\"VK_FORMAT_ASTC_5x4x4_SFLOAT_BLOCK_EXT\"";
        break;
    case 1000288015:
        settings.stream() << "\"VK_FORMAT_ASTC_5x5x4_UNORM_BLOCK_EXT\"";
        break;
    case 1000288016:
        settings.stream() << "\"VK_FORMAT_ASTC_5x5x4_SRGB_BLOCK_EXT\"";
        break;
    case 1000288017:
        settings.stream() << "\"VK_FORMAT_ASTC_5x5x4_SFLOAT_BLOCK_EXT\"";
        break;
    case 1000288018:
        settings.stream() << "\"VK_FORMAT_ASTC_5x5x5_UNORM_BLOCK_EXT\"";
        break;
    case 1000288019:
        settings.stream() << "\"VK_FORMAT_ASTC_5x5x5_SRGB_BLOCK_EXT\"";
        break;
    case 1000288020:
        settings.stream() << "\"VK_FORMAT_ASTC_5x5x5_SFLOAT_BLOCK_EXT\"";
        break;
    case 1000288021:
        settings.stream() << "\"VK_FORMAT_ASTC_6x5x5_UNORM_BLOCK_EXT\"";
        break;
    case 1000288022:
        settings.stream() << "\"VK_FORMAT_ASTC_6x5x5_SRGB_BLOCK_EXT\"";
        break;
    case 1000288023:
        settings.stream() << "\"VK_FORMAT_ASTC_6x5x5_SFLOAT_BLOCK_EXT\"";
        break;
    case 1000288024:
        settings.stream() << "\"VK_FORMAT_ASTC_6x6x5_UNORM_BLOCK_EXT\"";
        break;
    case 1000288025:
        settings.stream() << "\"VK_FORMAT_ASTC_6x6x5_SRGB_BLOCK_EXT\"";
        break;
    case 1000288026:
        settings.stream() << "\"VK_FORMAT_ASTC_6x6x5_SFLOAT_BLOCK_EXT\"";
        break;
    case 1000288027:
        settings.stream() << "\"VK_FORMAT_ASTC_6x6x6_UNORM_BLOCK_EXT\"";
        break;
    case 1000288028:
        settings.stream() << "\"VK_FORMAT_ASTC_6x6x6_SRGB_BLOCK_EXT\"";
        break;
    case 1000288029:
        settings.stream() << "\"VK_FORMAT_ASTC_6x6x6_SFLOAT_BLOCK_EXT\"";
        break;
    case 1000464000:
        settings.stream() << "\"VK_FORMAT_R16G16_SFIXED5_NV\"";
        break;
    case 1000470000:
        settings.stream() << "\"VK_FORMAT_A1B5G5R5_UNORM_PACK16_KHR\"";
        break;
    case 1000470001:
        settings.stream() << "\"VK_FORMAT_A8_UNORM_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkImageTiling(VkImageTiling object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_IMAGE_TILING_OPTIMAL\"";
        break;
    case 1:
        settings.stream() << "\"VK_IMAGE_TILING_LINEAR\"";
        break;
    case 1000158000:
        settings.stream() << "\"VK_IMAGE_TILING_DRM_FORMAT_MODIFIER_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkImageType(VkImageType object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_IMAGE_TYPE_1D\"";
        break;
    case 1:
        settings.stream() << "\"VK_IMAGE_TYPE_2D\"";
        break;
    case 2:
        settings.stream() << "\"VK_IMAGE_TYPE_3D\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPhysicalDeviceType(VkPhysicalDeviceType object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PHYSICAL_DEVICE_TYPE_OTHER\"";
        break;
    case 1:
        settings.stream() << "\"VK_PHYSICAL_DEVICE_TYPE_INTEGRATED_GPU\"";
        break;
    case 2:
        settings.stream() << "\"VK_PHYSICAL_DEVICE_TYPE_DISCRETE_GPU\"";
        break;
    case 3:
        settings.stream() << "\"VK_PHYSICAL_DEVICE_TYPE_VIRTUAL_GPU\"";
        break;
    case 4:
        settings.stream() << "\"VK_PHYSICAL_DEVICE_TYPE_CPU\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkQueryType(VkQueryType object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_QUERY_TYPE_OCCLUSION\"";
        break;
    case 1:
        settings.stream() << "\"VK_QUERY_TYPE_PIPELINE_STATISTICS\"";
        break;
    case 2:
        settings.stream() << "\"VK_QUERY_TYPE_TIMESTAMP\"";
        break;
    case 1000023000:
        settings.stream() << "\"VK_QUERY_TYPE_RESULT_STATUS_ONLY_KHR\"";
        break;
    case 1000028004:
        settings.stream() << "\"VK_QUERY_TYPE_TRANSFORM_FEEDBACK_STREAM_EXT\"";
        break;
    case 1000116000:
        settings.stream() << "\"VK_QUERY_TYPE_PERFORMANCE_QUERY_KHR\"";
        break;
    case 1000150000:
        settings.stream() << "\"VK_QUERY_TYPE_ACCELERATION_STRUCTURE_COMPACTED_SIZE_KHR\"";
        break;
    case 1000150001:
        settings.stream() << "\"VK_QUERY_TYPE_ACCELERATION_STRUCTURE_SERIALIZATION_SIZE_KHR\"";
        break;
    case 1000165000:
        settings.stream() << "\"VK_QUERY_TYPE_ACCELERATION_STRUCTURE_COMPACTED_SIZE_NV\"";
        break;
    case 1000210000:
        settings.stream() << "\"VK_QUERY_TYPE_PERFORMANCE_QUERY_INTEL\"";
        break;
    case 1000299000:
        settings.stream() << "\"VK_QUERY_TYPE_VIDEO_ENCODE_FEEDBACK_KHR\"";
        break;
    case 1000328000:
        settings.stream() << "\"VK_QUERY_TYPE_MESH_PRIMITIVES_GENERATED_EXT\"";
        break;
    case 1000382000:
        settings.stream() << "\"VK_QUERY_TYPE_PRIMITIVES_GENERATED_EXT\"";
        break;
    case 1000386000:
        settings.stream() << "\"VK_QUERY_TYPE_ACCELERATION_STRUCTURE_SERIALIZATION_BOTTOM_LEVEL_POINTERS_KHR\"";
        break;
    case 1000386001:
        settings.stream() << "\"VK_QUERY_TYPE_ACCELERATION_STRUCTURE_SIZE_KHR\"";
        break;
    case 1000396000:
        settings.stream() << "\"VK_QUERY_TYPE_MICROMAP_SERIALIZATION_SIZE_EXT\"";
        break;
    case 1000396001:
        settings.stream() << "\"VK_QUERY_TYPE_MICROMAP_COMPACTED_SIZE_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkSharingMode(VkSharingMode object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SHARING_MODE_EXCLUSIVE\"";
        break;
    case 1:
        settings.stream() << "\"VK_SHARING_MODE_CONCURRENT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkComponentSwizzle(VkComponentSwizzle object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_COMPONENT_SWIZZLE_IDENTITY\"";
        break;
    case 1:
        settings.stream() << "\"VK_COMPONENT_SWIZZLE_ZERO\"";
        break;
    case 2:
        settings.stream() << "\"VK_COMPONENT_SWIZZLE_ONE\"";
        break;
    case 3:
        settings.stream() << "\"VK_COMPONENT_SWIZZLE_R\"";
        break;
    case 4:
        settings.stream() << "\"VK_COMPONENT_SWIZZLE_G\"";
        break;
    case 5:
        settings.stream() << "\"VK_COMPONENT_SWIZZLE_B\"";
        break;
    case 6:
        settings.stream() << "\"VK_COMPONENT_SWIZZLE_A\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkImageViewType(VkImageViewType object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_IMAGE_VIEW_TYPE_1D\"";
        break;
    case 1:
        settings.stream() << "\"VK_IMAGE_VIEW_TYPE_2D\"";
        break;
    case 2:
        settings.stream() << "\"VK_IMAGE_VIEW_TYPE_3D\"";
        break;
    case 3:
        settings.stream() << "\"VK_IMAGE_VIEW_TYPE_CUBE\"";
        break;
    case 4:
        settings.stream() << "\"VK_IMAGE_VIEW_TYPE_1D_ARRAY\"";
        break;
    case 5:
        settings.stream() << "\"VK_IMAGE_VIEW_TYPE_2D_ARRAY\"";
        break;
    case 6:
        settings.stream() << "\"VK_IMAGE_VIEW_TYPE_CUBE_ARRAY\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkBlendFactor(VkBlendFactor object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_BLEND_FACTOR_ZERO\"";
        break;
    case 1:
        settings.stream() << "\"VK_BLEND_FACTOR_ONE\"";
        break;
    case 2:
        settings.stream() << "\"VK_BLEND_FACTOR_SRC_COLOR\"";
        break;
    case 3:
        settings.stream() << "\"VK_BLEND_FACTOR_ONE_MINUS_SRC_COLOR\"";
        break;
    case 4:
        settings.stream() << "\"VK_BLEND_FACTOR_DST_COLOR\"";
        break;
    case 5:
        settings.stream() << "\"VK_BLEND_FACTOR_ONE_MINUS_DST_COLOR\"";
        break;
    case 6:
        settings.stream() << "\"VK_BLEND_FACTOR_SRC_ALPHA\"";
        break;
    case 7:
        settings.stream() << "\"VK_BLEND_FACTOR_ONE_MINUS_SRC_ALPHA\"";
        break;
    case 8:
        settings.stream() << "\"VK_BLEND_FACTOR_DST_ALPHA\"";
        break;
    case 9:
        settings.stream() << "\"VK_BLEND_FACTOR_ONE_MINUS_DST_ALPHA\"";
        break;
    case 10:
        settings.stream() << "\"VK_BLEND_FACTOR_CONSTANT_COLOR\"";
        break;
    case 11:
        settings.stream() << "\"VK_BLEND_FACTOR_ONE_MINUS_CONSTANT_COLOR\"";
        break;
    case 12:
        settings.stream() << "\"VK_BLEND_FACTOR_CONSTANT_ALPHA\"";
        break;
    case 13:
        settings.stream() << "\"VK_BLEND_FACTOR_ONE_MINUS_CONSTANT_ALPHA\"";
        break;
    case 14:
        settings.stream() << "\"VK_BLEND_FACTOR_SRC_ALPHA_SATURATE\"";
        break;
    case 15:
        settings.stream() << "\"VK_BLEND_FACTOR_SRC1_COLOR\"";
        break;
    case 16:
        settings.stream() << "\"VK_BLEND_FACTOR_ONE_MINUS_SRC1_COLOR\"";
        break;
    case 17:
        settings.stream() << "\"VK_BLEND_FACTOR_SRC1_ALPHA\"";
        break;
    case 18:
        settings.stream() << "\"VK_BLEND_FACTOR_ONE_MINUS_SRC1_ALPHA\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkBlendOp(VkBlendOp object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_BLEND_OP_ADD\"";
        break;
    case 1:
        settings.stream() << "\"VK_BLEND_OP_SUBTRACT\"";
        break;
    case 2:
        settings.stream() << "\"VK_BLEND_OP_REVERSE_SUBTRACT\"";
        break;
    case 3:
        settings.stream() << "\"VK_BLEND_OP_MIN\"";
        break;
    case 4:
        settings.stream() << "\"VK_BLEND_OP_MAX\"";
        break;
    case 1000148000:
        settings.stream() << "\"VK_BLEND_OP_ZERO_EXT\"";
        break;
    case 1000148001:
        settings.stream() << "\"VK_BLEND_OP_SRC_EXT\"";
        break;
    case 1000148002:
        settings.stream() << "\"VK_BLEND_OP_DST_EXT\"";
        break;
    case 1000148003:
        settings.stream() << "\"VK_BLEND_OP_SRC_OVER_EXT\"";
        break;
    case 1000148004:
        settings.stream() << "\"VK_BLEND_OP_DST_OVER_EXT\"";
        break;
    case 1000148005:
        settings.stream() << "\"VK_BLEND_OP_SRC_IN_EXT\"";
        break;
    case 1000148006:
        settings.stream() << "\"VK_BLEND_OP_DST_IN_EXT\"";
        break;
    case 1000148007:
        settings.stream() << "\"VK_BLEND_OP_SRC_OUT_EXT\"";
        break;
    case 1000148008:
        settings.stream() << "\"VK_BLEND_OP_DST_OUT_EXT\"";
        break;
    case 1000148009:
        settings.stream() << "\"VK_BLEND_OP_SRC_ATOP_EXT\"";
        break;
    case 1000148010:
        settings.stream() << "\"VK_BLEND_OP_DST_ATOP_EXT\"";
        break;
    case 1000148011:
        settings.stream() << "\"VK_BLEND_OP_XOR_EXT\"";
        break;
    case 1000148012:
        settings.stream() << "\"VK_BLEND_OP_MULTIPLY_EXT\"";
        break;
    case 1000148013:
        settings.stream() << "\"VK_BLEND_OP_SCREEN_EXT\"";
        break;
    case 1000148014:
        settings.stream() << "\"VK_BLEND_OP_OVERLAY_EXT\"";
        break;
    case 1000148015:
        settings.stream() << "\"VK_BLEND_OP_DARKEN_EXT\"";
        break;
    case 1000148016:
        settings.stream() << "\"VK_BLEND_OP_LIGHTEN_EXT\"";
        break;
    case 1000148017:
        settings.stream() << "\"VK_BLEND_OP_COLORDODGE_EXT\"";
        break;
    case 1000148018:
        settings.stream() << "\"VK_BLEND_OP_COLORBURN_EXT\"";
        break;
    case 1000148019:
        settings.stream() << "\"VK_BLEND_OP_HARDLIGHT_EXT\"";
        break;
    case 1000148020:
        settings.stream() << "\"VK_BLEND_OP_SOFTLIGHT_EXT\"";
        break;
    case 1000148021:
        settings.stream() << "\"VK_BLEND_OP_DIFFERENCE_EXT\"";
        break;
    case 1000148022:
        settings.stream() << "\"VK_BLEND_OP_EXCLUSION_EXT\"";
        break;
    case 1000148023:
        settings.stream() << "\"VK_BLEND_OP_INVERT_EXT\"";
        break;
    case 1000148024:
        settings.stream() << "\"VK_BLEND_OP_INVERT_RGB_EXT\"";
        break;
    case 1000148025:
        settings.stream() << "\"VK_BLEND_OP_LINEARDODGE_EXT\"";
        break;
    case 1000148026:
        settings.stream() << "\"VK_BLEND_OP_LINEARBURN_EXT\"";
        break;
    case 1000148027:
        settings.stream() << "\"VK_BLEND_OP_VIVIDLIGHT_EXT\"";
        break;
    case 1000148028:
        settings.stream() << "\"VK_BLEND_OP_LINEARLIGHT_EXT\"";
        break;
    case 1000148029:
        settings.stream() << "\"VK_BLEND_OP_PINLIGHT_EXT\"";
        break;
    case 1000148030:
        settings.stream() << "\"VK_BLEND_OP_HARDMIX_EXT\"";
        break;
    case 1000148031:
        settings.stream() << "\"VK_BLEND_OP_HSL_HUE_EXT\"";
        break;
    case 1000148032:
        settings.stream() << "\"VK_BLEND_OP_HSL_SATURATION_EXT\"";
        break;
    case 1000148033:
        settings.stream() << "\"VK_BLEND_OP_HSL_COLOR_EXT\"";
        break;
    case 1000148034:
        settings.stream() << "\"VK_BLEND_OP_HSL_LUMINOSITY_EXT\"";
        break;
    case 1000148035:
        settings.stream() << "\"VK_BLEND_OP_PLUS_EXT\"";
        break;
    case 1000148036:
        settings.stream() << "\"VK_BLEND_OP_PLUS_CLAMPED_EXT\"";
        break;
    case 1000148037:
        settings.stream() << "\"VK_BLEND_OP_PLUS_CLAMPED_ALPHA_EXT\"";
        break;
    case 1000148038:
        settings.stream() << "\"VK_BLEND_OP_PLUS_DARKER_EXT\"";
        break;
    case 1000148039:
        settings.stream() << "\"VK_BLEND_OP_MINUS_EXT\"";
        break;
    case 1000148040:
        settings.stream() << "\"VK_BLEND_OP_MINUS_CLAMPED_EXT\"";
        break;
    case 1000148041:
        settings.stream() << "\"VK_BLEND_OP_CONTRAST_EXT\"";
        break;
    case 1000148042:
        settings.stream() << "\"VK_BLEND_OP_INVERT_OVG_EXT\"";
        break;
    case 1000148043:
        settings.stream() << "\"VK_BLEND_OP_RED_EXT\"";
        break;
    case 1000148044:
        settings.stream() << "\"VK_BLEND_OP_GREEN_EXT\"";
        break;
    case 1000148045:
        settings.stream() << "\"VK_BLEND_OP_BLUE_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkCompareOp(VkCompareOp object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_COMPARE_OP_NEVER\"";
        break;
    case 1:
        settings.stream() << "\"VK_COMPARE_OP_LESS\"";
        break;
    case 2:
        settings.stream() << "\"VK_COMPARE_OP_EQUAL\"";
        break;
    case 3:
        settings.stream() << "\"VK_COMPARE_OP_LESS_OR_EQUAL\"";
        break;
    case 4:
        settings.stream() << "\"VK_COMPARE_OP_GREATER\"";
        break;
    case 5:
        settings.stream() << "\"VK_COMPARE_OP_NOT_EQUAL\"";
        break;
    case 6:
        settings.stream() << "\"VK_COMPARE_OP_GREATER_OR_EQUAL\"";
        break;
    case 7:
        settings.stream() << "\"VK_COMPARE_OP_ALWAYS\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkDynamicState(VkDynamicState object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_DYNAMIC_STATE_VIEWPORT\"";
        break;
    case 1:
        settings.stream() << "\"VK_DYNAMIC_STATE_SCISSOR\"";
        break;
    case 2:
        settings.stream() << "\"VK_DYNAMIC_STATE_LINE_WIDTH\"";
        break;
    case 3:
        settings.stream() << "\"VK_DYNAMIC_STATE_DEPTH_BIAS\"";
        break;
    case 4:
        settings.stream() << "\"VK_DYNAMIC_STATE_BLEND_CONSTANTS\"";
        break;
    case 5:
        settings.stream() << "\"VK_DYNAMIC_STATE_DEPTH_BOUNDS\"";
        break;
    case 6:
        settings.stream() << "\"VK_DYNAMIC_STATE_STENCIL_COMPARE_MASK\"";
        break;
    case 7:
        settings.stream() << "\"VK_DYNAMIC_STATE_STENCIL_WRITE_MASK\"";
        break;
    case 8:
        settings.stream() << "\"VK_DYNAMIC_STATE_STENCIL_REFERENCE\"";
        break;
    case 1000267000:
        settings.stream() << "\"VK_DYNAMIC_STATE_CULL_MODE\"";
        break;
    case 1000267001:
        settings.stream() << "\"VK_DYNAMIC_STATE_FRONT_FACE\"";
        break;
    case 1000267002:
        settings.stream() << "\"VK_DYNAMIC_STATE_PRIMITIVE_TOPOLOGY\"";
        break;
    case 1000267003:
        settings.stream() << "\"VK_DYNAMIC_STATE_VIEWPORT_WITH_COUNT\"";
        break;
    case 1000267004:
        settings.stream() << "\"VK_DYNAMIC_STATE_SCISSOR_WITH_COUNT\"";
        break;
    case 1000267005:
        settings.stream() << "\"VK_DYNAMIC_STATE_VERTEX_INPUT_BINDING_STRIDE\"";
        break;
    case 1000267006:
        settings.stream() << "\"VK_DYNAMIC_STATE_DEPTH_TEST_ENABLE\"";
        break;
    case 1000267007:
        settings.stream() << "\"VK_DYNAMIC_STATE_DEPTH_WRITE_ENABLE\"";
        break;
    case 1000267008:
        settings.stream() << "\"VK_DYNAMIC_STATE_DEPTH_COMPARE_OP\"";
        break;
    case 1000267009:
        settings.stream() << "\"VK_DYNAMIC_STATE_DEPTH_BOUNDS_TEST_ENABLE\"";
        break;
    case 1000267010:
        settings.stream() << "\"VK_DYNAMIC_STATE_STENCIL_TEST_ENABLE\"";
        break;
    case 1000267011:
        settings.stream() << "\"VK_DYNAMIC_STATE_STENCIL_OP\"";
        break;
    case 1000377001:
        settings.stream() << "\"VK_DYNAMIC_STATE_RASTERIZER_DISCARD_ENABLE\"";
        break;
    case 1000377002:
        settings.stream() << "\"VK_DYNAMIC_STATE_DEPTH_BIAS_ENABLE\"";
        break;
    case 1000377004:
        settings.stream() << "\"VK_DYNAMIC_STATE_PRIMITIVE_RESTART_ENABLE\"";
        break;
    case 1000087000:
        settings.stream() << "\"VK_DYNAMIC_STATE_VIEWPORT_W_SCALING_NV\"";
        break;
    case 1000099000:
        settings.stream() << "\"VK_DYNAMIC_STATE_DISCARD_RECTANGLE_EXT\"";
        break;
    case 1000099001:
        settings.stream() << "\"VK_DYNAMIC_STATE_DISCARD_RECTANGLE_ENABLE_EXT\"";
        break;
    case 1000099002:
        settings.stream() << "\"VK_DYNAMIC_STATE_DISCARD_RECTANGLE_MODE_EXT\"";
        break;
    case 1000143000:
        settings.stream() << "\"VK_DYNAMIC_STATE_SAMPLE_LOCATIONS_EXT\"";
        break;
    case 1000347000:
        settings.stream() << "\"VK_DYNAMIC_STATE_RAY_TRACING_PIPELINE_STACK_SIZE_KHR\"";
        break;
    case 1000164004:
        settings.stream() << "\"VK_DYNAMIC_STATE_VIEWPORT_SHADING_RATE_PALETTE_NV\"";
        break;
    case 1000164006:
        settings.stream() << "\"VK_DYNAMIC_STATE_VIEWPORT_COARSE_SAMPLE_ORDER_NV\"";
        break;
    case 1000205000:
        settings.stream() << "\"VK_DYNAMIC_STATE_EXCLUSIVE_SCISSOR_ENABLE_NV\"";
        break;
    case 1000205001:
        settings.stream() << "\"VK_DYNAMIC_STATE_EXCLUSIVE_SCISSOR_NV\"";
        break;
    case 1000226000:
        settings.stream() << "\"VK_DYNAMIC_STATE_FRAGMENT_SHADING_RATE_KHR\"";
        break;
    case 1000352000:
        settings.stream() << "\"VK_DYNAMIC_STATE_VERTEX_INPUT_EXT\"";
        break;
    case 1000377000:
        settings.stream() << "\"VK_DYNAMIC_STATE_PATCH_CONTROL_POINTS_EXT\"";
        break;
    case 1000377003:
        settings.stream() << "\"VK_DYNAMIC_STATE_LOGIC_OP_EXT\"";
        break;
    case 1000381000:
        settings.stream() << "\"VK_DYNAMIC_STATE_COLOR_WRITE_ENABLE_EXT\"";
        break;
    case 1000455003:
        settings.stream() << "\"VK_DYNAMIC_STATE_DEPTH_CLAMP_ENABLE_EXT\"";
        break;
    case 1000455004:
        settings.stream() << "\"VK_DYNAMIC_STATE_POLYGON_MODE_EXT\"";
        break;
    case 1000455005:
        settings.stream() << "\"VK_DYNAMIC_STATE_RASTERIZATION_SAMPLES_EXT\"";
        break;
    case 1000455006:
        settings.stream() << "\"VK_DYNAMIC_STATE_SAMPLE_MASK_EXT\"";
        break;
    case 1000455007:
        settings.stream() << "\"VK_DYNAMIC_STATE_ALPHA_TO_COVERAGE_ENABLE_EXT\"";
        break;
    case 1000455008:
        settings.stream() << "\"VK_DYNAMIC_STATE_ALPHA_TO_ONE_ENABLE_EXT\"";
        break;
    case 1000455009:
        settings.stream() << "\"VK_DYNAMIC_STATE_LOGIC_OP_ENABLE_EXT\"";
        break;
    case 1000455010:
        settings.stream() << "\"VK_DYNAMIC_STATE_COLOR_BLEND_ENABLE_EXT\"";
        break;
    case 1000455011:
        settings.stream() << "\"VK_DYNAMIC_STATE_COLOR_BLEND_EQUATION_EXT\"";
        break;
    case 1000455012:
        settings.stream() << "\"VK_DYNAMIC_STATE_COLOR_WRITE_MASK_EXT\"";
        break;
    case 1000455002:
        settings.stream() << "\"VK_DYNAMIC_STATE_TESSELLATION_DOMAIN_ORIGIN_EXT\"";
        break;
    case 1000455013:
        settings.stream() << "\"VK_DYNAMIC_STATE_RASTERIZATION_STREAM_EXT\"";
        break;
    case 1000455014:
        settings.stream() << "\"VK_DYNAMIC_STATE_CONSERVATIVE_RASTERIZATION_MODE_EXT\"";
        break;
    case 1000455015:
        settings.stream() << "\"VK_DYNAMIC_STATE_EXTRA_PRIMITIVE_OVERESTIMATION_SIZE_EXT\"";
        break;
    case 1000455016:
        settings.stream() << "\"VK_DYNAMIC_STATE_DEPTH_CLIP_ENABLE_EXT\"";
        break;
    case 1000455017:
        settings.stream() << "\"VK_DYNAMIC_STATE_SAMPLE_LOCATIONS_ENABLE_EXT\"";
        break;
    case 1000455018:
        settings.stream() << "\"VK_DYNAMIC_STATE_COLOR_BLEND_ADVANCED_EXT\"";
        break;
    case 1000455019:
        settings.stream() << "\"VK_DYNAMIC_STATE_PROVOKING_VERTEX_MODE_EXT\"";
        break;
    case 1000455020:
        settings.stream() << "\"VK_DYNAMIC_STATE_LINE_RASTERIZATION_MODE_EXT\"";
        break;
    case 1000455021:
        settings.stream() << "\"VK_DYNAMIC_STATE_LINE_STIPPLE_ENABLE_EXT\"";
        break;
    case 1000455022:
        settings.stream() << "\"VK_DYNAMIC_STATE_DEPTH_CLIP_NEGATIVE_ONE_TO_ONE_EXT\"";
        break;
    case 1000455023:
        settings.stream() << "\"VK_DYNAMIC_STATE_VIEWPORT_W_SCALING_ENABLE_NV\"";
        break;
    case 1000455024:
        settings.stream() << "\"VK_DYNAMIC_STATE_VIEWPORT_SWIZZLE_NV\"";
        break;
    case 1000455025:
        settings.stream() << "\"VK_DYNAMIC_STATE_COVERAGE_TO_COLOR_ENABLE_NV\"";
        break;
    case 1000455026:
        settings.stream() << "\"VK_DYNAMIC_STATE_COVERAGE_TO_COLOR_LOCATION_NV\"";
        break;
    case 1000455027:
        settings.stream() << "\"VK_DYNAMIC_STATE_COVERAGE_MODULATION_MODE_NV\"";
        break;
    case 1000455028:
        settings.stream() << "\"VK_DYNAMIC_STATE_COVERAGE_MODULATION_TABLE_ENABLE_NV\"";
        break;
    case 1000455029:
        settings.stream() << "\"VK_DYNAMIC_STATE_COVERAGE_MODULATION_TABLE_NV\"";
        break;
    case 1000455030:
        settings.stream() << "\"VK_DYNAMIC_STATE_SHADING_RATE_IMAGE_ENABLE_NV\"";
        break;
    case 1000455031:
        settings.stream() << "\"VK_DYNAMIC_STATE_REPRESENTATIVE_FRAGMENT_TEST_ENABLE_NV\"";
        break;
    case 1000455032:
        settings.stream() << "\"VK_DYNAMIC_STATE_COVERAGE_REDUCTION_MODE_NV\"";
        break;
    case 1000524000:
        settings.stream() << "\"VK_DYNAMIC_STATE_ATTACHMENT_FEEDBACK_LOOP_ENABLE_EXT\"";
        break;
    case 1000259000:
        settings.stream() << "\"VK_DYNAMIC_STATE_LINE_STIPPLE_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkFrontFace(VkFrontFace object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_FRONT_FACE_COUNTER_CLOCKWISE\"";
        break;
    case 1:
        settings.stream() << "\"VK_FRONT_FACE_CLOCKWISE\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkVertexInputRate(VkVertexInputRate object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_VERTEX_INPUT_RATE_VERTEX\"";
        break;
    case 1:
        settings.stream() << "\"VK_VERTEX_INPUT_RATE_INSTANCE\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPrimitiveTopology(VkPrimitiveTopology object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PRIMITIVE_TOPOLOGY_POINT_LIST\"";
        break;
    case 1:
        settings.stream() << "\"VK_PRIMITIVE_TOPOLOGY_LINE_LIST\"";
        break;
    case 2:
        settings.stream() << "\"VK_PRIMITIVE_TOPOLOGY_LINE_STRIP\"";
        break;
    case 3:
        settings.stream() << "\"VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST\"";
        break;
    case 4:
        settings.stream() << "\"VK_PRIMITIVE_TOPOLOGY_TRIANGLE_STRIP\"";
        break;
    case 5:
        settings.stream() << "\"VK_PRIMITIVE_TOPOLOGY_TRIANGLE_FAN\"";
        break;
    case 6:
        settings.stream() << "\"VK_PRIMITIVE_TOPOLOGY_LINE_LIST_WITH_ADJACENCY\"";
        break;
    case 7:
        settings.stream() << "\"VK_PRIMITIVE_TOPOLOGY_LINE_STRIP_WITH_ADJACENCY\"";
        break;
    case 8:
        settings.stream() << "\"VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST_WITH_ADJACENCY\"";
        break;
    case 9:
        settings.stream() << "\"VK_PRIMITIVE_TOPOLOGY_TRIANGLE_STRIP_WITH_ADJACENCY\"";
        break;
    case 10:
        settings.stream() << "\"VK_PRIMITIVE_TOPOLOGY_PATCH_LIST\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPolygonMode(VkPolygonMode object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_POLYGON_MODE_FILL\"";
        break;
    case 1:
        settings.stream() << "\"VK_POLYGON_MODE_LINE\"";
        break;
    case 2:
        settings.stream() << "\"VK_POLYGON_MODE_POINT\"";
        break;
    case 1000153000:
        settings.stream() << "\"VK_POLYGON_MODE_FILL_RECTANGLE_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkStencilOp(VkStencilOp object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_STENCIL_OP_KEEP\"";
        break;
    case 1:
        settings.stream() << "\"VK_STENCIL_OP_ZERO\"";
        break;
    case 2:
        settings.stream() << "\"VK_STENCIL_OP_REPLACE\"";
        break;
    case 3:
        settings.stream() << "\"VK_STENCIL_OP_INCREMENT_AND_CLAMP\"";
        break;
    case 4:
        settings.stream() << "\"VK_STENCIL_OP_DECREMENT_AND_CLAMP\"";
        break;
    case 5:
        settings.stream() << "\"VK_STENCIL_OP_INVERT\"";
        break;
    case 6:
        settings.stream() << "\"VK_STENCIL_OP_INCREMENT_AND_WRAP\"";
        break;
    case 7:
        settings.stream() << "\"VK_STENCIL_OP_DECREMENT_AND_WRAP\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkLogicOp(VkLogicOp object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_LOGIC_OP_CLEAR\"";
        break;
    case 1:
        settings.stream() << "\"VK_LOGIC_OP_AND\"";
        break;
    case 2:
        settings.stream() << "\"VK_LOGIC_OP_AND_REVERSE\"";
        break;
    case 3:
        settings.stream() << "\"VK_LOGIC_OP_COPY\"";
        break;
    case 4:
        settings.stream() << "\"VK_LOGIC_OP_AND_INVERTED\"";
        break;
    case 5:
        settings.stream() << "\"VK_LOGIC_OP_NO_OP\"";
        break;
    case 6:
        settings.stream() << "\"VK_LOGIC_OP_XOR\"";
        break;
    case 7:
        settings.stream() << "\"VK_LOGIC_OP_OR\"";
        break;
    case 8:
        settings.stream() << "\"VK_LOGIC_OP_NOR\"";
        break;
    case 9:
        settings.stream() << "\"VK_LOGIC_OP_EQUIVALENT\"";
        break;
    case 10:
        settings.stream() << "\"VK_LOGIC_OP_INVERT\"";
        break;
    case 11:
        settings.stream() << "\"VK_LOGIC_OP_OR_REVERSE\"";
        break;
    case 12:
        settings.stream() << "\"VK_LOGIC_OP_COPY_INVERTED\"";
        break;
    case 13:
        settings.stream() << "\"VK_LOGIC_OP_OR_INVERTED\"";
        break;
    case 14:
        settings.stream() << "\"VK_LOGIC_OP_NAND\"";
        break;
    case 15:
        settings.stream() << "\"VK_LOGIC_OP_SET\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkBorderColor(VkBorderColor object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_BORDER_COLOR_FLOAT_TRANSPARENT_BLACK\"";
        break;
    case 1:
        settings.stream() << "\"VK_BORDER_COLOR_INT_TRANSPARENT_BLACK\"";
        break;
    case 2:
        settings.stream() << "\"VK_BORDER_COLOR_FLOAT_OPAQUE_BLACK\"";
        break;
    case 3:
        settings.stream() << "\"VK_BORDER_COLOR_INT_OPAQUE_BLACK\"";
        break;
    case 4:
        settings.stream() << "\"VK_BORDER_COLOR_FLOAT_OPAQUE_WHITE\"";
        break;
    case 5:
        settings.stream() << "\"VK_BORDER_COLOR_INT_OPAQUE_WHITE\"";
        break;
    case 1000287003:
        settings.stream() << "\"VK_BORDER_COLOR_FLOAT_CUSTOM_EXT\"";
        break;
    case 1000287004:
        settings.stream() << "\"VK_BORDER_COLOR_INT_CUSTOM_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkFilter(VkFilter object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_FILTER_NEAREST\"";
        break;
    case 1:
        settings.stream() << "\"VK_FILTER_LINEAR\"";
        break;
    case 1000015000:
        settings.stream() << "\"VK_FILTER_CUBIC_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkSamplerAddressMode(VkSamplerAddressMode object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SAMPLER_ADDRESS_MODE_REPEAT\"";
        break;
    case 1:
        settings.stream() << "\"VK_SAMPLER_ADDRESS_MODE_MIRRORED_REPEAT\"";
        break;
    case 2:
        settings.stream() << "\"VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE\"";
        break;
    case 3:
        settings.stream() << "\"VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER\"";
        break;
    case 4:
        settings.stream() << "\"VK_SAMPLER_ADDRESS_MODE_MIRROR_CLAMP_TO_EDGE\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkSamplerMipmapMode(VkSamplerMipmapMode object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SAMPLER_MIPMAP_MODE_NEAREST\"";
        break;
    case 1:
        settings.stream() << "\"VK_SAMPLER_MIPMAP_MODE_LINEAR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkDescriptorType(VkDescriptorType object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_SAMPLER\"";
        break;
    case 1:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER\"";
        break;
    case 2:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE\"";
        break;
    case 3:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_STORAGE_IMAGE\"";
        break;
    case 4:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER\"";
        break;
    case 5:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER\"";
        break;
    case 6:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER\"";
        break;
    case 7:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_STORAGE_BUFFER\"";
        break;
    case 8:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC\"";
        break;
    case 9:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC\"";
        break;
    case 10:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT\"";
        break;
    case 1000138000:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK\"";
        break;
    case 1000150000:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR\"";
        break;
    case 1000165000:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_NV\"";
        break;
    case 1000440000:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_SAMPLE_WEIGHT_IMAGE_QCOM\"";
        break;
    case 1000440001:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_BLOCK_MATCH_IMAGE_QCOM\"";
        break;
    case 1000351000:
        settings.stream() << "\"VK_DESCRIPTOR_TYPE_MUTABLE_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkAttachmentLoadOp(VkAttachmentLoadOp object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_ATTACHMENT_LOAD_OP_LOAD\"";
        break;
    case 1:
        settings.stream() << "\"VK_ATTACHMENT_LOAD_OP_CLEAR\"";
        break;
    case 2:
        settings.stream() << "\"VK_ATTACHMENT_LOAD_OP_DONT_CARE\"";
        break;
    case 1000400000:
        settings.stream() << "\"VK_ATTACHMENT_LOAD_OP_NONE_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkAttachmentStoreOp(VkAttachmentStoreOp object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_ATTACHMENT_STORE_OP_STORE\"";
        break;
    case 1:
        settings.stream() << "\"VK_ATTACHMENT_STORE_OP_DONT_CARE\"";
        break;
    case 1000301000:
        settings.stream() << "\"VK_ATTACHMENT_STORE_OP_NONE\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPipelineBindPoint(VkPipelineBindPoint object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PIPELINE_BIND_POINT_GRAPHICS\"";
        break;
    case 1:
        settings.stream() << "\"VK_PIPELINE_BIND_POINT_COMPUTE\"";
        break;
    case 1000134000:
        settings.stream() << "\"VK_PIPELINE_BIND_POINT_EXECUTION_GRAPH_AMDX\"";
        break;
    case 1000165000:
        settings.stream() << "\"VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR\"";
        break;
    case 1000369003:
        settings.stream() << "\"VK_PIPELINE_BIND_POINT_SUBPASS_SHADING_HUAWEI\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkCommandBufferLevel(VkCommandBufferLevel object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_COMMAND_BUFFER_LEVEL_PRIMARY\"";
        break;
    case 1:
        settings.stream() << "\"VK_COMMAND_BUFFER_LEVEL_SECONDARY\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkIndexType(VkIndexType object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_INDEX_TYPE_UINT16\"";
        break;
    case 1:
        settings.stream() << "\"VK_INDEX_TYPE_UINT32\"";
        break;
    case 1000165000:
        settings.stream() << "\"VK_INDEX_TYPE_NONE_KHR\"";
        break;
    case 1000265000:
        settings.stream() << "\"VK_INDEX_TYPE_UINT8_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkSubpassContents(VkSubpassContents object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SUBPASS_CONTENTS_INLINE\"";
        break;
    case 1:
        settings.stream() << "\"VK_SUBPASS_CONTENTS_SECONDARY_COMMAND_BUFFERS\"";
        break;
    case 1000451000:
        settings.stream() << "\"VK_SUBPASS_CONTENTS_INLINE_AND_SECONDARY_COMMAND_BUFFERS_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPointClippingBehavior(VkPointClippingBehavior object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_POINT_CLIPPING_BEHAVIOR_ALL_CLIP_PLANES\"";
        break;
    case 1:
        settings.stream() << "\"VK_POINT_CLIPPING_BEHAVIOR_USER_CLIP_PLANES_ONLY\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkTessellationDomainOrigin(VkTessellationDomainOrigin object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_TESSELLATION_DOMAIN_ORIGIN_UPPER_LEFT\"";
        break;
    case 1:
        settings.stream() << "\"VK_TESSELLATION_DOMAIN_ORIGIN_LOWER_LEFT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkSamplerYcbcrModelConversion(VkSamplerYcbcrModelConversion object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SAMPLER_YCBCR_MODEL_CONVERSION_RGB_IDENTITY\"";
        break;
    case 1:
        settings.stream() << "\"VK_SAMPLER_YCBCR_MODEL_CONVERSION_YCBCR_IDENTITY\"";
        break;
    case 2:
        settings.stream() << "\"VK_SAMPLER_YCBCR_MODEL_CONVERSION_YCBCR_709\"";
        break;
    case 3:
        settings.stream() << "\"VK_SAMPLER_YCBCR_MODEL_CONVERSION_YCBCR_601\"";
        break;
    case 4:
        settings.stream() << "\"VK_SAMPLER_YCBCR_MODEL_CONVERSION_YCBCR_2020\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkSamplerYcbcrRange(VkSamplerYcbcrRange object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SAMPLER_YCBCR_RANGE_ITU_FULL\"";
        break;
    case 1:
        settings.stream() << "\"VK_SAMPLER_YCBCR_RANGE_ITU_NARROW\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkChromaLocation(VkChromaLocation object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_CHROMA_LOCATION_COSITED_EVEN\"";
        break;
    case 1:
        settings.stream() << "\"VK_CHROMA_LOCATION_MIDPOINT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkDescriptorUpdateTemplateType(VkDescriptorUpdateTemplateType object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_DESCRIPTOR_UPDATE_TEMPLATE_TYPE_DESCRIPTOR_SET\"";
        break;
    case 1:
        settings.stream() << "\"VK_DESCRIPTOR_UPDATE_TEMPLATE_TYPE_PUSH_DESCRIPTORS_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkDriverId(VkDriverId object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 1:
        settings.stream() << "\"VK_DRIVER_ID_AMD_PROPRIETARY\"";
        break;
    case 2:
        settings.stream() << "\"VK_DRIVER_ID_AMD_OPEN_SOURCE\"";
        break;
    case 3:
        settings.stream() << "\"VK_DRIVER_ID_MESA_RADV\"";
        break;
    case 4:
        settings.stream() << "\"VK_DRIVER_ID_NVIDIA_PROPRIETARY\"";
        break;
    case 5:
        settings.stream() << "\"VK_DRIVER_ID_INTEL_PROPRIETARY_WINDOWS\"";
        break;
    case 6:
        settings.stream() << "\"VK_DRIVER_ID_INTEL_OPEN_SOURCE_MESA\"";
        break;
    case 7:
        settings.stream() << "\"VK_DRIVER_ID_IMAGINATION_PROPRIETARY\"";
        break;
    case 8:
        settings.stream() << "\"VK_DRIVER_ID_QUALCOMM_PROPRIETARY\"";
        break;
    case 9:
        settings.stream() << "\"VK_DRIVER_ID_ARM_PROPRIETARY\"";
        break;
    case 10:
        settings.stream() << "\"VK_DRIVER_ID_GOOGLE_SWIFTSHADER\"";
        break;
    case 11:
        settings.stream() << "\"VK_DRIVER_ID_GGP_PROPRIETARY\"";
        break;
    case 12:
        settings.stream() << "\"VK_DRIVER_ID_BROADCOM_PROPRIETARY\"";
        break;
    case 13:
        settings.stream() << "\"VK_DRIVER_ID_MESA_LLVMPIPE\"";
        break;
    case 14:
        settings.stream() << "\"VK_DRIVER_ID_MOLTENVK\"";
        break;
    case 15:
        settings.stream() << "\"VK_DRIVER_ID_COREAVI_PROPRIETARY\"";
        break;
    case 16:
        settings.stream() << "\"VK_DRIVER_ID_JUICE_PROPRIETARY\"";
        break;
    case 17:
        settings.stream() << "\"VK_DRIVER_ID_VERISILICON_PROPRIETARY\"";
        break;
    case 18:
        settings.stream() << "\"VK_DRIVER_ID_MESA_TURNIP\"";
        break;
    case 19:
        settings.stream() << "\"VK_DRIVER_ID_MESA_V3DV\"";
        break;
    case 20:
        settings.stream() << "\"VK_DRIVER_ID_MESA_PANVK\"";
        break;
    case 21:
        settings.stream() << "\"VK_DRIVER_ID_SAMSUNG_PROPRIETARY\"";
        break;
    case 22:
        settings.stream() << "\"VK_DRIVER_ID_MESA_VENUS\"";
        break;
    case 23:
        settings.stream() << "\"VK_DRIVER_ID_MESA_DOZEN\"";
        break;
    case 24:
        settings.stream() << "\"VK_DRIVER_ID_MESA_NVK\"";
        break;
    case 25:
        settings.stream() << "\"VK_DRIVER_ID_IMAGINATION_OPEN_SOURCE_MESA\"";
        break;
    case 26:
        settings.stream() << "\"VK_DRIVER_ID_MESA_HONEYKRISP\"";
        break;
    case 27:
        settings.stream() << "\"VK_DRIVER_ID_RESERVED_27\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkShaderFloatControlsIndependence(VkShaderFloatControlsIndependence object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SHADER_FLOAT_CONTROLS_INDEPENDENCE_32_BIT_ONLY\"";
        break;
    case 1:
        settings.stream() << "\"VK_SHADER_FLOAT_CONTROLS_INDEPENDENCE_ALL\"";
        break;
    case 2:
        settings.stream() << "\"VK_SHADER_FLOAT_CONTROLS_INDEPENDENCE_NONE\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkSamplerReductionMode(VkSamplerReductionMode object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SAMPLER_REDUCTION_MODE_WEIGHTED_AVERAGE\"";
        break;
    case 1:
        settings.stream() << "\"VK_SAMPLER_REDUCTION_MODE_MIN\"";
        break;
    case 2:
        settings.stream() << "\"VK_SAMPLER_REDUCTION_MODE_MAX\"";
        break;
    case 1000521000:
        settings.stream() << "\"VK_SAMPLER_REDUCTION_MODE_WEIGHTED_AVERAGE_RANGECLAMP_QCOM\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkSemaphoreType(VkSemaphoreType object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SEMAPHORE_TYPE_BINARY\"";
        break;
    case 1:
        settings.stream() << "\"VK_SEMAPHORE_TYPE_TIMELINE\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPresentModeKHR(VkPresentModeKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PRESENT_MODE_IMMEDIATE_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_PRESENT_MODE_MAILBOX_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_PRESENT_MODE_FIFO_KHR\"";
        break;
    case 3:
        settings.stream() << "\"VK_PRESENT_MODE_FIFO_RELAXED_KHR\"";
        break;
    case 1000111000:
        settings.stream() << "\"VK_PRESENT_MODE_SHARED_DEMAND_REFRESH_KHR\"";
        break;
    case 1000111001:
        settings.stream() << "\"VK_PRESENT_MODE_SHARED_CONTINUOUS_REFRESH_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkColorSpaceKHR(VkColorSpaceKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_COLOR_SPACE_SRGB_NONLINEAR_KHR\"";
        break;
    case 1000104001:
        settings.stream() << "\"VK_COLOR_SPACE_DISPLAY_P3_NONLINEAR_EXT\"";
        break;
    case 1000104002:
        settings.stream() << "\"VK_COLOR_SPACE_EXTENDED_SRGB_LINEAR_EXT\"";
        break;
    case 1000104003:
        settings.stream() << "\"VK_COLOR_SPACE_DISPLAY_P3_LINEAR_EXT\"";
        break;
    case 1000104004:
        settings.stream() << "\"VK_COLOR_SPACE_DCI_P3_NONLINEAR_EXT\"";
        break;
    case 1000104005:
        settings.stream() << "\"VK_COLOR_SPACE_BT709_LINEAR_EXT\"";
        break;
    case 1000104006:
        settings.stream() << "\"VK_COLOR_SPACE_BT709_NONLINEAR_EXT\"";
        break;
    case 1000104007:
        settings.stream() << "\"VK_COLOR_SPACE_BT2020_LINEAR_EXT\"";
        break;
    case 1000104008:
        settings.stream() << "\"VK_COLOR_SPACE_HDR10_ST2084_EXT\"";
        break;
    case 1000104009:
        settings.stream() << "\"VK_COLOR_SPACE_DOLBYVISION_EXT\"";
        break;
    case 1000104010:
        settings.stream() << "\"VK_COLOR_SPACE_HDR10_HLG_EXT\"";
        break;
    case 1000104011:
        settings.stream() << "\"VK_COLOR_SPACE_ADOBERGB_LINEAR_EXT\"";
        break;
    case 1000104012:
        settings.stream() << "\"VK_COLOR_SPACE_ADOBERGB_NONLINEAR_EXT\"";
        break;
    case 1000104013:
        settings.stream() << "\"VK_COLOR_SPACE_PASS_THROUGH_EXT\"";
        break;
    case 1000104014:
        settings.stream() << "\"VK_COLOR_SPACE_EXTENDED_SRGB_NONLINEAR_EXT\"";
        break;
    case 1000213000:
        settings.stream() << "\"VK_COLOR_SPACE_DISPLAY_NATIVE_AMD\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkDebugReportObjectTypeEXT(VkDebugReportObjectTypeEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_UNKNOWN_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_INSTANCE_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_PHYSICAL_DEVICE_EXT\"";
        break;
    case 3:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_DEVICE_EXT\"";
        break;
    case 4:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_QUEUE_EXT\"";
        break;
    case 5:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_SEMAPHORE_EXT\"";
        break;
    case 6:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_COMMAND_BUFFER_EXT\"";
        break;
    case 7:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_FENCE_EXT\"";
        break;
    case 8:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_DEVICE_MEMORY_EXT\"";
        break;
    case 9:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_BUFFER_EXT\"";
        break;
    case 10:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_IMAGE_EXT\"";
        break;
    case 11:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_EVENT_EXT\"";
        break;
    case 12:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_QUERY_POOL_EXT\"";
        break;
    case 13:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_BUFFER_VIEW_EXT\"";
        break;
    case 14:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_IMAGE_VIEW_EXT\"";
        break;
    case 15:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_SHADER_MODULE_EXT\"";
        break;
    case 16:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_PIPELINE_CACHE_EXT\"";
        break;
    case 17:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_PIPELINE_LAYOUT_EXT\"";
        break;
    case 18:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_RENDER_PASS_EXT\"";
        break;
    case 19:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_PIPELINE_EXT\"";
        break;
    case 20:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_DESCRIPTOR_SET_LAYOUT_EXT\"";
        break;
    case 21:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_SAMPLER_EXT\"";
        break;
    case 22:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_DESCRIPTOR_POOL_EXT\"";
        break;
    case 23:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_DESCRIPTOR_SET_EXT\"";
        break;
    case 24:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_FRAMEBUFFER_EXT\"";
        break;
    case 25:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_COMMAND_POOL_EXT\"";
        break;
    case 26:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_SURFACE_KHR_EXT\"";
        break;
    case 27:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_SWAPCHAIN_KHR_EXT\"";
        break;
    case 28:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_DEBUG_REPORT_CALLBACK_EXT_EXT\"";
        break;
    case 29:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_DISPLAY_KHR_EXT\"";
        break;
    case 30:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_DISPLAY_MODE_KHR_EXT\"";
        break;
    case 33:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_VALIDATION_CACHE_EXT_EXT\"";
        break;
    case 1000156000:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_SAMPLER_YCBCR_CONVERSION_EXT\"";
        break;
    case 1000085000:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_DESCRIPTOR_UPDATE_TEMPLATE_EXT\"";
        break;
    case 1000029000:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_CU_MODULE_NVX_EXT\"";
        break;
    case 1000029001:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_CU_FUNCTION_NVX_EXT\"";
        break;
    case 1000150000:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_ACCELERATION_STRUCTURE_KHR_EXT\"";
        break;
    case 1000165000:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_ACCELERATION_STRUCTURE_NV_EXT\"";
        break;
    case 1000307000:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_CUDA_MODULE_NV_EXT\"";
        break;
    case 1000307001:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_CUDA_FUNCTION_NV_EXT\"";
        break;
    case 1000366000:
        settings.stream() << "\"VK_DEBUG_REPORT_OBJECT_TYPE_BUFFER_COLLECTION_FUCHSIA_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkRasterizationOrderAMD(VkRasterizationOrderAMD object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_RASTERIZATION_ORDER_STRICT_AMD\"";
        break;
    case 1:
        settings.stream() << "\"VK_RASTERIZATION_ORDER_RELAXED_AMD\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkQueryResultStatusKHR(VkQueryResultStatusKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case -1:
        settings.stream() << "\"VK_QUERY_RESULT_STATUS_ERROR_KHR\"";
        break;
    case 0:
        settings.stream() << "\"VK_QUERY_RESULT_STATUS_NOT_READY_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_QUERY_RESULT_STATUS_COMPLETE_KHR\"";
        break;
    case -1000299000:
        settings.stream() << "\"VK_QUERY_RESULT_STATUS_INSUFFICIENT_BITSTREAM_BUFFER_RANGE_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkShaderInfoTypeAMD(VkShaderInfoTypeAMD object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SHADER_INFO_TYPE_STATISTICS_AMD\"";
        break;
    case 1:
        settings.stream() << "\"VK_SHADER_INFO_TYPE_BINARY_AMD\"";
        break;
    case 2:
        settings.stream() << "\"VK_SHADER_INFO_TYPE_DISASSEMBLY_AMD\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkValidationCheckEXT(VkValidationCheckEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_VALIDATION_CHECK_ALL_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_VALIDATION_CHECK_SHADERS_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPipelineRobustnessBufferBehaviorEXT(VkPipelineRobustnessBufferBehaviorEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_DEVICE_DEFAULT_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_DISABLED_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_EXT\"";
        break;
    case 3:
        settings.stream() << "\"VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_2_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPipelineRobustnessImageBehaviorEXT(VkPipelineRobustnessImageBehaviorEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PIPELINE_ROBUSTNESS_IMAGE_BEHAVIOR_DEVICE_DEFAULT_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_PIPELINE_ROBUSTNESS_IMAGE_BEHAVIOR_DISABLED_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_PIPELINE_ROBUSTNESS_IMAGE_BEHAVIOR_ROBUST_IMAGE_ACCESS_EXT\"";
        break;
    case 3:
        settings.stream() << "\"VK_PIPELINE_ROBUSTNESS_IMAGE_BEHAVIOR_ROBUST_IMAGE_ACCESS_2_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkDisplayPowerStateEXT(VkDisplayPowerStateEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_DISPLAY_POWER_STATE_OFF_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_DISPLAY_POWER_STATE_SUSPEND_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_DISPLAY_POWER_STATE_ON_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkDeviceEventTypeEXT(VkDeviceEventTypeEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_DEVICE_EVENT_TYPE_DISPLAY_HOTPLUG_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkDisplayEventTypeEXT(VkDisplayEventTypeEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_DISPLAY_EVENT_TYPE_FIRST_PIXEL_OUT_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkViewportCoordinateSwizzleNV(VkViewportCoordinateSwizzleNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_VIEWPORT_COORDINATE_SWIZZLE_POSITIVE_X_NV\"";
        break;
    case 1:
        settings.stream() << "\"VK_VIEWPORT_COORDINATE_SWIZZLE_NEGATIVE_X_NV\"";
        break;
    case 2:
        settings.stream() << "\"VK_VIEWPORT_COORDINATE_SWIZZLE_POSITIVE_Y_NV\"";
        break;
    case 3:
        settings.stream() << "\"VK_VIEWPORT_COORDINATE_SWIZZLE_NEGATIVE_Y_NV\"";
        break;
    case 4:
        settings.stream() << "\"VK_VIEWPORT_COORDINATE_SWIZZLE_POSITIVE_Z_NV\"";
        break;
    case 5:
        settings.stream() << "\"VK_VIEWPORT_COORDINATE_SWIZZLE_NEGATIVE_Z_NV\"";
        break;
    case 6:
        settings.stream() << "\"VK_VIEWPORT_COORDINATE_SWIZZLE_POSITIVE_W_NV\"";
        break;
    case 7:
        settings.stream() << "\"VK_VIEWPORT_COORDINATE_SWIZZLE_NEGATIVE_W_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkDiscardRectangleModeEXT(VkDiscardRectangleModeEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_DISCARD_RECTANGLE_MODE_INCLUSIVE_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_DISCARD_RECTANGLE_MODE_EXCLUSIVE_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkConservativeRasterizationModeEXT(VkConservativeRasterizationModeEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_CONSERVATIVE_RASTERIZATION_MODE_DISABLED_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_CONSERVATIVE_RASTERIZATION_MODE_OVERESTIMATE_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_CONSERVATIVE_RASTERIZATION_MODE_UNDERESTIMATE_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPerformanceCounterUnitKHR(VkPerformanceCounterUnitKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_UNIT_GENERIC_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_UNIT_PERCENTAGE_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_UNIT_NANOSECONDS_KHR\"";
        break;
    case 3:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_UNIT_BYTES_KHR\"";
        break;
    case 4:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_UNIT_BYTES_PER_SECOND_KHR\"";
        break;
    case 5:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_UNIT_KELVIN_KHR\"";
        break;
    case 6:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_UNIT_WATTS_KHR\"";
        break;
    case 7:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_UNIT_VOLTS_KHR\"";
        break;
    case 8:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_UNIT_AMPS_KHR\"";
        break;
    case 9:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_UNIT_HERTZ_KHR\"";
        break;
    case 10:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_UNIT_CYCLES_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPerformanceCounterScopeKHR(VkPerformanceCounterScopeKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_SCOPE_COMMAND_BUFFER_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_SCOPE_RENDER_PASS_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_SCOPE_COMMAND_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPerformanceCounterStorageKHR(VkPerformanceCounterStorageKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_STORAGE_INT32_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_STORAGE_INT64_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_STORAGE_UINT32_KHR\"";
        break;
    case 3:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_STORAGE_UINT64_KHR\"";
        break;
    case 4:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_STORAGE_FLOAT32_KHR\"";
        break;
    case 5:
        settings.stream() << "\"VK_PERFORMANCE_COUNTER_STORAGE_FLOAT64_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkBlendOverlapEXT(VkBlendOverlapEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_BLEND_OVERLAP_UNCORRELATED_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_BLEND_OVERLAP_DISJOINT_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_BLEND_OVERLAP_CONJOINT_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkAccelerationStructureTypeKHR(VkAccelerationStructureTypeKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_ACCELERATION_STRUCTURE_TYPE_TOP_LEVEL_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_ACCELERATION_STRUCTURE_TYPE_BOTTOM_LEVEL_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_ACCELERATION_STRUCTURE_TYPE_GENERIC_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkBuildAccelerationStructureModeKHR(VkBuildAccelerationStructureModeKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_BUILD_ACCELERATION_STRUCTURE_MODE_BUILD_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_BUILD_ACCELERATION_STRUCTURE_MODE_UPDATE_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkGeometryTypeKHR(VkGeometryTypeKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_GEOMETRY_TYPE_TRIANGLES_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_GEOMETRY_TYPE_AABBS_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_GEOMETRY_TYPE_INSTANCES_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkAccelerationStructureBuildTypeKHR(VkAccelerationStructureBuildTypeKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_ACCELERATION_STRUCTURE_BUILD_TYPE_HOST_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_ACCELERATION_STRUCTURE_BUILD_TYPE_DEVICE_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_ACCELERATION_STRUCTURE_BUILD_TYPE_HOST_OR_DEVICE_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkCopyAccelerationStructureModeKHR(VkCopyAccelerationStructureModeKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_COPY_ACCELERATION_STRUCTURE_MODE_CLONE_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_COPY_ACCELERATION_STRUCTURE_MODE_COMPACT_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_COPY_ACCELERATION_STRUCTURE_MODE_SERIALIZE_KHR\"";
        break;
    case 3:
        settings.stream() << "\"VK_COPY_ACCELERATION_STRUCTURE_MODE_DESERIALIZE_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkAccelerationStructureCompatibilityKHR(VkAccelerationStructureCompatibilityKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_ACCELERATION_STRUCTURE_COMPATIBILITY_COMPATIBLE_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_ACCELERATION_STRUCTURE_COMPATIBILITY_INCOMPATIBLE_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkCoverageModulationModeNV(VkCoverageModulationModeNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_COVERAGE_MODULATION_MODE_NONE_NV\"";
        break;
    case 1:
        settings.stream() << "\"VK_COVERAGE_MODULATION_MODE_RGB_NV\"";
        break;
    case 2:
        settings.stream() << "\"VK_COVERAGE_MODULATION_MODE_ALPHA_NV\"";
        break;
    case 3:
        settings.stream() << "\"VK_COVERAGE_MODULATION_MODE_RGBA_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkValidationCacheHeaderVersionEXT(VkValidationCacheHeaderVersionEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 1:
        settings.stream() << "\"VK_VALIDATION_CACHE_HEADER_VERSION_ONE_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkShadingRatePaletteEntryNV(VkShadingRatePaletteEntryNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SHADING_RATE_PALETTE_ENTRY_NO_INVOCATIONS_NV\"";
        break;
    case 1:
        settings.stream() << "\"VK_SHADING_RATE_PALETTE_ENTRY_16_INVOCATIONS_PER_PIXEL_NV\"";
        break;
    case 2:
        settings.stream() << "\"VK_SHADING_RATE_PALETTE_ENTRY_8_INVOCATIONS_PER_PIXEL_NV\"";
        break;
    case 3:
        settings.stream() << "\"VK_SHADING_RATE_PALETTE_ENTRY_4_INVOCATIONS_PER_PIXEL_NV\"";
        break;
    case 4:
        settings.stream() << "\"VK_SHADING_RATE_PALETTE_ENTRY_2_INVOCATIONS_PER_PIXEL_NV\"";
        break;
    case 5:
        settings.stream() << "\"VK_SHADING_RATE_PALETTE_ENTRY_1_INVOCATION_PER_PIXEL_NV\"";
        break;
    case 6:
        settings.stream() << "\"VK_SHADING_RATE_PALETTE_ENTRY_1_INVOCATION_PER_2X1_PIXELS_NV\"";
        break;
    case 7:
        settings.stream() << "\"VK_SHADING_RATE_PALETTE_ENTRY_1_INVOCATION_PER_1X2_PIXELS_NV\"";
        break;
    case 8:
        settings.stream() << "\"VK_SHADING_RATE_PALETTE_ENTRY_1_INVOCATION_PER_2X2_PIXELS_NV\"";
        break;
    case 9:
        settings.stream() << "\"VK_SHADING_RATE_PALETTE_ENTRY_1_INVOCATION_PER_4X2_PIXELS_NV\"";
        break;
    case 10:
        settings.stream() << "\"VK_SHADING_RATE_PALETTE_ENTRY_1_INVOCATION_PER_2X4_PIXELS_NV\"";
        break;
    case 11:
        settings.stream() << "\"VK_SHADING_RATE_PALETTE_ENTRY_1_INVOCATION_PER_4X4_PIXELS_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkCoarseSampleOrderTypeNV(VkCoarseSampleOrderTypeNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_COARSE_SAMPLE_ORDER_TYPE_DEFAULT_NV\"";
        break;
    case 1:
        settings.stream() << "\"VK_COARSE_SAMPLE_ORDER_TYPE_CUSTOM_NV\"";
        break;
    case 2:
        settings.stream() << "\"VK_COARSE_SAMPLE_ORDER_TYPE_PIXEL_MAJOR_NV\"";
        break;
    case 3:
        settings.stream() << "\"VK_COARSE_SAMPLE_ORDER_TYPE_SAMPLE_MAJOR_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkRayTracingShaderGroupTypeKHR(VkRayTracingShaderGroupTypeKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_RAY_TRACING_SHADER_GROUP_TYPE_GENERAL_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_RAY_TRACING_SHADER_GROUP_TYPE_TRIANGLES_HIT_GROUP_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_RAY_TRACING_SHADER_GROUP_TYPE_PROCEDURAL_HIT_GROUP_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkAccelerationStructureMemoryRequirementsTypeNV(VkAccelerationStructureMemoryRequirementsTypeNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_ACCELERATION_STRUCTURE_MEMORY_REQUIREMENTS_TYPE_OBJECT_NV\"";
        break;
    case 1:
        settings.stream() << "\"VK_ACCELERATION_STRUCTURE_MEMORY_REQUIREMENTS_TYPE_BUILD_SCRATCH_NV\"";
        break;
    case 2:
        settings.stream() << "\"VK_ACCELERATION_STRUCTURE_MEMORY_REQUIREMENTS_TYPE_UPDATE_SCRATCH_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkQueueGlobalPriorityKHR(VkQueueGlobalPriorityKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 128:
        settings.stream() << "\"VK_QUEUE_GLOBAL_PRIORITY_LOW_KHR\"";
        break;
    case 256:
        settings.stream() << "\"VK_QUEUE_GLOBAL_PRIORITY_MEDIUM_KHR\"";
        break;
    case 512:
        settings.stream() << "\"VK_QUEUE_GLOBAL_PRIORITY_HIGH_KHR\"";
        break;
    case 1024:
        settings.stream() << "\"VK_QUEUE_GLOBAL_PRIORITY_REALTIME_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkTimeDomainKHR(VkTimeDomainKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_TIME_DOMAIN_DEVICE_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_TIME_DOMAIN_CLOCK_MONOTONIC_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_TIME_DOMAIN_CLOCK_MONOTONIC_RAW_KHR\"";
        break;
    case 3:
        settings.stream() << "\"VK_TIME_DOMAIN_QUERY_PERFORMANCE_COUNTER_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkMemoryOverallocationBehaviorAMD(VkMemoryOverallocationBehaviorAMD object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_MEMORY_OVERALLOCATION_BEHAVIOR_DEFAULT_AMD\"";
        break;
    case 1:
        settings.stream() << "\"VK_MEMORY_OVERALLOCATION_BEHAVIOR_ALLOWED_AMD\"";
        break;
    case 2:
        settings.stream() << "\"VK_MEMORY_OVERALLOCATION_BEHAVIOR_DISALLOWED_AMD\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPerformanceConfigurationTypeINTEL(VkPerformanceConfigurationTypeINTEL object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PERFORMANCE_CONFIGURATION_TYPE_COMMAND_QUEUE_METRICS_DISCOVERY_ACTIVATED_INTEL\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkQueryPoolSamplingModeINTEL(VkQueryPoolSamplingModeINTEL object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_QUERY_POOL_SAMPLING_MODE_MANUAL_INTEL\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPerformanceOverrideTypeINTEL(VkPerformanceOverrideTypeINTEL object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PERFORMANCE_OVERRIDE_TYPE_NULL_HARDWARE_INTEL\"";
        break;
    case 1:
        settings.stream() << "\"VK_PERFORMANCE_OVERRIDE_TYPE_FLUSH_GPU_CACHES_INTEL\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPerformanceParameterTypeINTEL(VkPerformanceParameterTypeINTEL object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PERFORMANCE_PARAMETER_TYPE_HW_COUNTERS_SUPPORTED_INTEL\"";
        break;
    case 1:
        settings.stream() << "\"VK_PERFORMANCE_PARAMETER_TYPE_STREAM_MARKER_VALID_BITS_INTEL\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPerformanceValueTypeINTEL(VkPerformanceValueTypeINTEL object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PERFORMANCE_VALUE_TYPE_UINT32_INTEL\"";
        break;
    case 1:
        settings.stream() << "\"VK_PERFORMANCE_VALUE_TYPE_UINT64_INTEL\"";
        break;
    case 2:
        settings.stream() << "\"VK_PERFORMANCE_VALUE_TYPE_FLOAT_INTEL\"";
        break;
    case 3:
        settings.stream() << "\"VK_PERFORMANCE_VALUE_TYPE_BOOL_INTEL\"";
        break;
    case 4:
        settings.stream() << "\"VK_PERFORMANCE_VALUE_TYPE_STRING_INTEL\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkFragmentShadingRateCombinerOpKHR(VkFragmentShadingRateCombinerOpKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_COMBINER_OP_KEEP_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_COMBINER_OP_REPLACE_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_COMBINER_OP_MIN_KHR\"";
        break;
    case 3:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_COMBINER_OP_MAX_KHR\"";
        break;
    case 4:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_COMBINER_OP_MUL_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkValidationFeatureEnableEXT(VkValidationFeatureEnableEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_VALIDATION_FEATURE_ENABLE_GPU_ASSISTED_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_VALIDATION_FEATURE_ENABLE_GPU_ASSISTED_RESERVE_BINDING_SLOT_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_VALIDATION_FEATURE_ENABLE_BEST_PRACTICES_EXT\"";
        break;
    case 3:
        settings.stream() << "\"VK_VALIDATION_FEATURE_ENABLE_DEBUG_PRINTF_EXT\"";
        break;
    case 4:
        settings.stream() << "\"VK_VALIDATION_FEATURE_ENABLE_SYNCHRONIZATION_VALIDATION_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkValidationFeatureDisableEXT(VkValidationFeatureDisableEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_VALIDATION_FEATURE_DISABLE_ALL_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_VALIDATION_FEATURE_DISABLE_SHADERS_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_VALIDATION_FEATURE_DISABLE_THREAD_SAFETY_EXT\"";
        break;
    case 3:
        settings.stream() << "\"VK_VALIDATION_FEATURE_DISABLE_API_PARAMETERS_EXT\"";
        break;
    case 4:
        settings.stream() << "\"VK_VALIDATION_FEATURE_DISABLE_OBJECT_LIFETIMES_EXT\"";
        break;
    case 5:
        settings.stream() << "\"VK_VALIDATION_FEATURE_DISABLE_CORE_CHECKS_EXT\"";
        break;
    case 6:
        settings.stream() << "\"VK_VALIDATION_FEATURE_DISABLE_UNIQUE_HANDLES_EXT\"";
        break;
    case 7:
        settings.stream() << "\"VK_VALIDATION_FEATURE_DISABLE_SHADER_VALIDATION_CACHE_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkComponentTypeKHR(VkComponentTypeKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_COMPONENT_TYPE_FLOAT16_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_COMPONENT_TYPE_FLOAT32_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_COMPONENT_TYPE_FLOAT64_KHR\"";
        break;
    case 3:
        settings.stream() << "\"VK_COMPONENT_TYPE_SINT8_KHR\"";
        break;
    case 4:
        settings.stream() << "\"VK_COMPONENT_TYPE_SINT16_KHR\"";
        break;
    case 5:
        settings.stream() << "\"VK_COMPONENT_TYPE_SINT32_KHR\"";
        break;
    case 6:
        settings.stream() << "\"VK_COMPONENT_TYPE_SINT64_KHR\"";
        break;
    case 7:
        settings.stream() << "\"VK_COMPONENT_TYPE_UINT8_KHR\"";
        break;
    case 8:
        settings.stream() << "\"VK_COMPONENT_TYPE_UINT16_KHR\"";
        break;
    case 9:
        settings.stream() << "\"VK_COMPONENT_TYPE_UINT32_KHR\"";
        break;
    case 10:
        settings.stream() << "\"VK_COMPONENT_TYPE_UINT64_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkScopeKHR(VkScopeKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 1:
        settings.stream() << "\"VK_SCOPE_DEVICE_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_SCOPE_WORKGROUP_KHR\"";
        break;
    case 3:
        settings.stream() << "\"VK_SCOPE_SUBGROUP_KHR\"";
        break;
    case 5:
        settings.stream() << "\"VK_SCOPE_QUEUE_FAMILY_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkCoverageReductionModeNV(VkCoverageReductionModeNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_COVERAGE_REDUCTION_MODE_MERGE_NV\"";
        break;
    case 1:
        settings.stream() << "\"VK_COVERAGE_REDUCTION_MODE_TRUNCATE_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkProvokingVertexModeEXT(VkProvokingVertexModeEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PROVOKING_VERTEX_MODE_FIRST_VERTEX_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_PROVOKING_VERTEX_MODE_LAST_VERTEX_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkFullScreenExclusiveEXT(VkFullScreenExclusiveEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_FULL_SCREEN_EXCLUSIVE_DEFAULT_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_FULL_SCREEN_EXCLUSIVE_ALLOWED_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_FULL_SCREEN_EXCLUSIVE_DISALLOWED_EXT\"";
        break;
    case 3:
        settings.stream() << "\"VK_FULL_SCREEN_EXCLUSIVE_APPLICATION_CONTROLLED_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
#endif // VK_USE_PLATFORM_WIN32_KHR
void dump_json_VkLineRasterizationModeKHR(VkLineRasterizationModeKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_LINE_RASTERIZATION_MODE_DEFAULT_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_LINE_RASTERIZATION_MODE_RECTANGULAR_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_LINE_RASTERIZATION_MODE_BRESENHAM_KHR\"";
        break;
    case 3:
        settings.stream() << "\"VK_LINE_RASTERIZATION_MODE_RECTANGULAR_SMOOTH_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPipelineExecutableStatisticFormatKHR(VkPipelineExecutableStatisticFormatKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PIPELINE_EXECUTABLE_STATISTIC_FORMAT_BOOL32_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_PIPELINE_EXECUTABLE_STATISTIC_FORMAT_INT64_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_PIPELINE_EXECUTABLE_STATISTIC_FORMAT_UINT64_KHR\"";
        break;
    case 3:
        settings.stream() << "\"VK_PIPELINE_EXECUTABLE_STATISTIC_FORMAT_FLOAT64_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkIndirectCommandsTokenTypeNV(VkIndirectCommandsTokenTypeNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_INDIRECT_COMMANDS_TOKEN_TYPE_SHADER_GROUP_NV\"";
        break;
    case 1:
        settings.stream() << "\"VK_INDIRECT_COMMANDS_TOKEN_TYPE_STATE_FLAGS_NV\"";
        break;
    case 2:
        settings.stream() << "\"VK_INDIRECT_COMMANDS_TOKEN_TYPE_INDEX_BUFFER_NV\"";
        break;
    case 3:
        settings.stream() << "\"VK_INDIRECT_COMMANDS_TOKEN_TYPE_VERTEX_BUFFER_NV\"";
        break;
    case 4:
        settings.stream() << "\"VK_INDIRECT_COMMANDS_TOKEN_TYPE_PUSH_CONSTANT_NV\"";
        break;
    case 5:
        settings.stream() << "\"VK_INDIRECT_COMMANDS_TOKEN_TYPE_DRAW_INDEXED_NV\"";
        break;
    case 6:
        settings.stream() << "\"VK_INDIRECT_COMMANDS_TOKEN_TYPE_DRAW_NV\"";
        break;
    case 7:
        settings.stream() << "\"VK_INDIRECT_COMMANDS_TOKEN_TYPE_DRAW_TASKS_NV\"";
        break;
    case 1000328000:
        settings.stream() << "\"VK_INDIRECT_COMMANDS_TOKEN_TYPE_DRAW_MESH_TASKS_NV\"";
        break;
    case 1000428003:
        settings.stream() << "\"VK_INDIRECT_COMMANDS_TOKEN_TYPE_PIPELINE_NV\"";
        break;
    case 1000428004:
        settings.stream() << "\"VK_INDIRECT_COMMANDS_TOKEN_TYPE_DISPATCH_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkDepthBiasRepresentationEXT(VkDepthBiasRepresentationEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_DEPTH_BIAS_REPRESENTATION_LEAST_REPRESENTABLE_VALUE_FORMAT_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_DEPTH_BIAS_REPRESENTATION_LEAST_REPRESENTABLE_VALUE_FORCE_UNORM_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_DEPTH_BIAS_REPRESENTATION_FLOAT_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkDeviceMemoryReportEventTypeEXT(VkDeviceMemoryReportEventTypeEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_DEVICE_MEMORY_REPORT_EVENT_TYPE_ALLOCATE_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_DEVICE_MEMORY_REPORT_EVENT_TYPE_FREE_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_DEVICE_MEMORY_REPORT_EVENT_TYPE_IMPORT_EXT\"";
        break;
    case 3:
        settings.stream() << "\"VK_DEVICE_MEMORY_REPORT_EVENT_TYPE_UNIMPORT_EXT\"";
        break;
    case 4:
        settings.stream() << "\"VK_DEVICE_MEMORY_REPORT_EVENT_TYPE_ALLOCATION_FAILED_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkVideoEncodeTuningModeKHR(VkVideoEncodeTuningModeKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_VIDEO_ENCODE_TUNING_MODE_DEFAULT_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_VIDEO_ENCODE_TUNING_MODE_HIGH_QUALITY_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_VIDEO_ENCODE_TUNING_MODE_LOW_LATENCY_KHR\"";
        break;
    case 3:
        settings.stream() << "\"VK_VIDEO_ENCODE_TUNING_MODE_ULTRA_LOW_LATENCY_KHR\"";
        break;
    case 4:
        settings.stream() << "\"VK_VIDEO_ENCODE_TUNING_MODE_LOSSLESS_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkFragmentShadingRateTypeNV(VkFragmentShadingRateTypeNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_TYPE_FRAGMENT_SIZE_NV\"";
        break;
    case 1:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_TYPE_ENUMS_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkFragmentShadingRateNV(VkFragmentShadingRateNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_1_INVOCATION_PER_PIXEL_NV\"";
        break;
    case 1:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_1_INVOCATION_PER_1X2_PIXELS_NV\"";
        break;
    case 4:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_1_INVOCATION_PER_2X1_PIXELS_NV\"";
        break;
    case 5:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_1_INVOCATION_PER_2X2_PIXELS_NV\"";
        break;
    case 6:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_1_INVOCATION_PER_2X4_PIXELS_NV\"";
        break;
    case 9:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_1_INVOCATION_PER_4X2_PIXELS_NV\"";
        break;
    case 10:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_1_INVOCATION_PER_4X4_PIXELS_NV\"";
        break;
    case 11:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_2_INVOCATIONS_PER_PIXEL_NV\"";
        break;
    case 12:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_4_INVOCATIONS_PER_PIXEL_NV\"";
        break;
    case 13:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_8_INVOCATIONS_PER_PIXEL_NV\"";
        break;
    case 14:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_16_INVOCATIONS_PER_PIXEL_NV\"";
        break;
    case 15:
        settings.stream() << "\"VK_FRAGMENT_SHADING_RATE_NO_INVOCATIONS_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkAccelerationStructureMotionInstanceTypeNV(VkAccelerationStructureMotionInstanceTypeNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_ACCELERATION_STRUCTURE_MOTION_INSTANCE_TYPE_STATIC_NV\"";
        break;
    case 1:
        settings.stream() << "\"VK_ACCELERATION_STRUCTURE_MOTION_INSTANCE_TYPE_MATRIX_MOTION_NV\"";
        break;
    case 2:
        settings.stream() << "\"VK_ACCELERATION_STRUCTURE_MOTION_INSTANCE_TYPE_SRT_MOTION_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkDeviceFaultAddressTypeEXT(VkDeviceFaultAddressTypeEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_DEVICE_FAULT_ADDRESS_TYPE_NONE_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_DEVICE_FAULT_ADDRESS_TYPE_READ_INVALID_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_DEVICE_FAULT_ADDRESS_TYPE_WRITE_INVALID_EXT\"";
        break;
    case 3:
        settings.stream() << "\"VK_DEVICE_FAULT_ADDRESS_TYPE_EXECUTE_INVALID_EXT\"";
        break;
    case 4:
        settings.stream() << "\"VK_DEVICE_FAULT_ADDRESS_TYPE_INSTRUCTION_POINTER_UNKNOWN_EXT\"";
        break;
    case 5:
        settings.stream() << "\"VK_DEVICE_FAULT_ADDRESS_TYPE_INSTRUCTION_POINTER_INVALID_EXT\"";
        break;
    case 6:
        settings.stream() << "\"VK_DEVICE_FAULT_ADDRESS_TYPE_INSTRUCTION_POINTER_FAULT_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkDeviceFaultVendorBinaryHeaderVersionEXT(VkDeviceFaultVendorBinaryHeaderVersionEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 1:
        settings.stream() << "\"VK_DEVICE_FAULT_VENDOR_BINARY_HEADER_VERSION_ONE_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkShaderGroupShaderKHR(VkShaderGroupShaderKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SHADER_GROUP_SHADER_GENERAL_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_SHADER_GROUP_SHADER_CLOSEST_HIT_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_SHADER_GROUP_SHADER_ANY_HIT_KHR\"";
        break;
    case 3:
        settings.stream() << "\"VK_SHADER_GROUP_SHADER_INTERSECTION_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkDeviceAddressBindingTypeEXT(VkDeviceAddressBindingTypeEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_DEVICE_ADDRESS_BINDING_TYPE_BIND_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_DEVICE_ADDRESS_BINDING_TYPE_UNBIND_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkMicromapTypeEXT(VkMicromapTypeEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_MICROMAP_TYPE_OPACITY_MICROMAP_EXT\"";
        break;
    case 1000397000:
        settings.stream() << "\"VK_MICROMAP_TYPE_DISPLACEMENT_MICROMAP_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkBuildMicromapModeEXT(VkBuildMicromapModeEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_BUILD_MICROMAP_MODE_BUILD_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkCopyMicromapModeEXT(VkCopyMicromapModeEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_COPY_MICROMAP_MODE_CLONE_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_COPY_MICROMAP_MODE_SERIALIZE_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_COPY_MICROMAP_MODE_DESERIALIZE_EXT\"";
        break;
    case 3:
        settings.stream() << "\"VK_COPY_MICROMAP_MODE_COMPACT_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkOpacityMicromapFormatEXT(VkOpacityMicromapFormatEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 1:
        settings.stream() << "\"VK_OPACITY_MICROMAP_FORMAT_2_STATE_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_OPACITY_MICROMAP_FORMAT_4_STATE_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkOpacityMicromapSpecialIndexEXT(VkOpacityMicromapSpecialIndexEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case -1:
        settings.stream() << "\"VK_OPACITY_MICROMAP_SPECIAL_INDEX_FULLY_TRANSPARENT_EXT\"";
        break;
    case -2:
        settings.stream() << "\"VK_OPACITY_MICROMAP_SPECIAL_INDEX_FULLY_OPAQUE_EXT\"";
        break;
    case -3:
        settings.stream() << "\"VK_OPACITY_MICROMAP_SPECIAL_INDEX_FULLY_UNKNOWN_TRANSPARENT_EXT\"";
        break;
    case -4:
        settings.stream() << "\"VK_OPACITY_MICROMAP_SPECIAL_INDEX_FULLY_UNKNOWN_OPAQUE_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_VkDisplacementMicromapFormatNV(VkDisplacementMicromapFormatNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 1:
        settings.stream() << "\"VK_DISPLACEMENT_MICROMAP_FORMAT_64_TRIANGLES_64_BYTES_NV\"";
        break;
    case 2:
        settings.stream() << "\"VK_DISPLACEMENT_MICROMAP_FORMAT_256_TRIANGLES_128_BYTES_NV\"";
        break;
    case 3:
        settings.stream() << "\"VK_DISPLACEMENT_MICROMAP_FORMAT_1024_TRIANGLES_128_BYTES_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
#endif // VK_ENABLE_BETA_EXTENSIONS
void dump_json_VkSubpassMergeStatusEXT(VkSubpassMergeStatusEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SUBPASS_MERGE_STATUS_MERGED_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_SUBPASS_MERGE_STATUS_DISALLOWED_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_SUBPASS_MERGE_STATUS_NOT_MERGED_SIDE_EFFECTS_EXT\"";
        break;
    case 3:
        settings.stream() << "\"VK_SUBPASS_MERGE_STATUS_NOT_MERGED_SAMPLES_MISMATCH_EXT\"";
        break;
    case 4:
        settings.stream() << "\"VK_SUBPASS_MERGE_STATUS_NOT_MERGED_VIEWS_MISMATCH_EXT\"";
        break;
    case 5:
        settings.stream() << "\"VK_SUBPASS_MERGE_STATUS_NOT_MERGED_ALIASING_EXT\"";
        break;
    case 6:
        settings.stream() << "\"VK_SUBPASS_MERGE_STATUS_NOT_MERGED_DEPENDENCIES_EXT\"";
        break;
    case 7:
        settings.stream() << "\"VK_SUBPASS_MERGE_STATUS_NOT_MERGED_INCOMPATIBLE_INPUT_ATTACHMENT_EXT\"";
        break;
    case 8:
        settings.stream() << "\"VK_SUBPASS_MERGE_STATUS_NOT_MERGED_TOO_MANY_ATTACHMENTS_EXT\"";
        break;
    case 9:
        settings.stream() << "\"VK_SUBPASS_MERGE_STATUS_NOT_MERGED_INSUFFICIENT_STORAGE_EXT\"";
        break;
    case 10:
        settings.stream() << "\"VK_SUBPASS_MERGE_STATUS_NOT_MERGED_DEPTH_STENCIL_COUNT_EXT\"";
        break;
    case 11:
        settings.stream() << "\"VK_SUBPASS_MERGE_STATUS_NOT_MERGED_RESOLVE_ATTACHMENT_REUSE_EXT\"";
        break;
    case 12:
        settings.stream() << "\"VK_SUBPASS_MERGE_STATUS_NOT_MERGED_SINGLE_SUBPASS_EXT\"";
        break;
    case 13:
        settings.stream() << "\"VK_SUBPASS_MERGE_STATUS_NOT_MERGED_UNSPECIFIED_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkDirectDriverLoadingModeLUNARG(VkDirectDriverLoadingModeLUNARG object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_DIRECT_DRIVER_LOADING_MODE_EXCLUSIVE_LUNARG\"";
        break;
    case 1:
        settings.stream() << "\"VK_DIRECT_DRIVER_LOADING_MODE_INCLUSIVE_LUNARG\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkOpticalFlowPerformanceLevelNV(VkOpticalFlowPerformanceLevelNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_OPTICAL_FLOW_PERFORMANCE_LEVEL_UNKNOWN_NV\"";
        break;
    case 1:
        settings.stream() << "\"VK_OPTICAL_FLOW_PERFORMANCE_LEVEL_SLOW_NV\"";
        break;
    case 2:
        settings.stream() << "\"VK_OPTICAL_FLOW_PERFORMANCE_LEVEL_MEDIUM_NV\"";
        break;
    case 3:
        settings.stream() << "\"VK_OPTICAL_FLOW_PERFORMANCE_LEVEL_FAST_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkOpticalFlowSessionBindingPointNV(VkOpticalFlowSessionBindingPointNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_OPTICAL_FLOW_SESSION_BINDING_POINT_UNKNOWN_NV\"";
        break;
    case 1:
        settings.stream() << "\"VK_OPTICAL_FLOW_SESSION_BINDING_POINT_INPUT_NV\"";
        break;
    case 2:
        settings.stream() << "\"VK_OPTICAL_FLOW_SESSION_BINDING_POINT_REFERENCE_NV\"";
        break;
    case 3:
        settings.stream() << "\"VK_OPTICAL_FLOW_SESSION_BINDING_POINT_HINT_NV\"";
        break;
    case 4:
        settings.stream() << "\"VK_OPTICAL_FLOW_SESSION_BINDING_POINT_FLOW_VECTOR_NV\"";
        break;
    case 5:
        settings.stream() << "\"VK_OPTICAL_FLOW_SESSION_BINDING_POINT_BACKWARD_FLOW_VECTOR_NV\"";
        break;
    case 6:
        settings.stream() << "\"VK_OPTICAL_FLOW_SESSION_BINDING_POINT_COST_NV\"";
        break;
    case 7:
        settings.stream() << "\"VK_OPTICAL_FLOW_SESSION_BINDING_POINT_BACKWARD_COST_NV\"";
        break;
    case 8:
        settings.stream() << "\"VK_OPTICAL_FLOW_SESSION_BINDING_POINT_GLOBAL_FLOW_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkShaderCodeTypeEXT(VkShaderCodeTypeEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_SHADER_CODE_TYPE_BINARY_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_SHADER_CODE_TYPE_SPIRV_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkRayTracingInvocationReorderModeNV(VkRayTracingInvocationReorderModeNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_RAY_TRACING_INVOCATION_REORDER_MODE_NONE_NV\"";
        break;
    case 1:
        settings.stream() << "\"VK_RAY_TRACING_INVOCATION_REORDER_MODE_REORDER_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkLayerSettingTypeEXT(VkLayerSettingTypeEXT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_LAYER_SETTING_TYPE_BOOL32_EXT\"";
        break;
    case 1:
        settings.stream() << "\"VK_LAYER_SETTING_TYPE_INT32_EXT\"";
        break;
    case 2:
        settings.stream() << "\"VK_LAYER_SETTING_TYPE_INT64_EXT\"";
        break;
    case 3:
        settings.stream() << "\"VK_LAYER_SETTING_TYPE_UINT32_EXT\"";
        break;
    case 4:
        settings.stream() << "\"VK_LAYER_SETTING_TYPE_UINT64_EXT\"";
        break;
    case 5:
        settings.stream() << "\"VK_LAYER_SETTING_TYPE_FLOAT32_EXT\"";
        break;
    case 6:
        settings.stream() << "\"VK_LAYER_SETTING_TYPE_FLOAT64_EXT\"";
        break;
    case 7:
        settings.stream() << "\"VK_LAYER_SETTING_TYPE_STRING_EXT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkLatencyMarkerNV(VkLatencyMarkerNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_LATENCY_MARKER_SIMULATION_START_NV\"";
        break;
    case 1:
        settings.stream() << "\"VK_LATENCY_MARKER_SIMULATION_END_NV\"";
        break;
    case 2:
        settings.stream() << "\"VK_LATENCY_MARKER_RENDERSUBMIT_START_NV\"";
        break;
    case 3:
        settings.stream() << "\"VK_LATENCY_MARKER_RENDERSUBMIT_END_NV\"";
        break;
    case 4:
        settings.stream() << "\"VK_LATENCY_MARKER_PRESENT_START_NV\"";
        break;
    case 5:
        settings.stream() << "\"VK_LATENCY_MARKER_PRESENT_END_NV\"";
        break;
    case 6:
        settings.stream() << "\"VK_LATENCY_MARKER_INPUT_SAMPLE_NV\"";
        break;
    case 7:
        settings.stream() << "\"VK_LATENCY_MARKER_TRIGGER_FLASH_NV\"";
        break;
    case 8:
        settings.stream() << "\"VK_LATENCY_MARKER_OUT_OF_BAND_RENDERSUBMIT_START_NV\"";
        break;
    case 9:
        settings.stream() << "\"VK_LATENCY_MARKER_OUT_OF_BAND_RENDERSUBMIT_END_NV\"";
        break;
    case 10:
        settings.stream() << "\"VK_LATENCY_MARKER_OUT_OF_BAND_PRESENT_START_NV\"";
        break;
    case 11:
        settings.stream() << "\"VK_LATENCY_MARKER_OUT_OF_BAND_PRESENT_END_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkOutOfBandQueueTypeNV(VkOutOfBandQueueTypeNV object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_OUT_OF_BAND_QUEUE_TYPE_RENDER_NV\"";
        break;
    case 1:
        settings.stream() << "\"VK_OUT_OF_BAND_QUEUE_TYPE_PRESENT_NV\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkBlockMatchWindowCompareModeQCOM(VkBlockMatchWindowCompareModeQCOM object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_BLOCK_MATCH_WINDOW_COMPARE_MODE_MIN_QCOM\"";
        break;
    case 1:
        settings.stream() << "\"VK_BLOCK_MATCH_WINDOW_COMPARE_MODE_MAX_QCOM\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkCubicFilterWeightsQCOM(VkCubicFilterWeightsQCOM object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_CUBIC_FILTER_WEIGHTS_CATMULL_ROM_QCOM\"";
        break;
    case 1:
        settings.stream() << "\"VK_CUBIC_FILTER_WEIGHTS_ZERO_TANGENT_CARDINAL_QCOM\"";
        break;
    case 2:
        settings.stream() << "\"VK_CUBIC_FILTER_WEIGHTS_B_SPLINE_QCOM\"";
        break;
    case 3:
        settings.stream() << "\"VK_CUBIC_FILTER_WEIGHTS_MITCHELL_NETRAVALI_QCOM\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkLayeredDriverUnderlyingApiMSFT(VkLayeredDriverUnderlyingApiMSFT object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_LAYERED_DRIVER_UNDERLYING_API_NONE_MSFT\"";
        break;
    case 1:
        settings.stream() << "\"VK_LAYERED_DRIVER_UNDERLYING_API_D3D12_MSFT\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}
void dump_json_VkPhysicalDeviceLayeredApiKHR(VkPhysicalDeviceLayeredApiKHR object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) object)
    {
    case 0:
        settings.stream() << "\"VK_PHYSICAL_DEVICE_LAYERED_API_VULKAN_KHR\"";
        break;
    case 1:
        settings.stream() << "\"VK_PHYSICAL_DEVICE_LAYERED_API_D3D12_KHR\"";
        break;
    case 2:
        settings.stream() << "\"VK_PHYSICAL_DEVICE_LAYERED_API_METAL_KHR\"";
        break;
    case 3:
        settings.stream() << "\"VK_PHYSICAL_DEVICE_LAYERED_API_OPENGL_KHR\"";
        break;
    case 4:
        settings.stream() << "\"VK_PHYSICAL_DEVICE_LAYERED_API_OPENGLES_KHR\"";
        break;
    default:
        settings.stream() << "\"UNKNOWN (" << object << ")\"";
    }
}

//========================= Bitmask Implementations =========================//

void dump_json_VkAccessFlagBits(VkAccessFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_INDIRECT_COMMAND_READ_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_INDEX_READ_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_VERTEX_ATTRIBUTE_READ_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_UNIFORM_READ_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_INPUT_ATTACHMENT_READ_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_SHADER_READ_BIT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_SHADER_WRITE_BIT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_COLOR_ATTACHMENT_READ_BIT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_DEPTH_STENCIL_ATTACHMENT_READ_BIT"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_DEPTH_STENCIL_ATTACHMENT_WRITE_BIT"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_TRANSFER_READ_BIT"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_TRANSFER_WRITE_BIT"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_HOST_READ_BIT"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_HOST_WRITE_BIT"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_MEMORY_READ_BIT"; is_first = false;
    }
    if(object & 65536) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_MEMORY_WRITE_BIT"; is_first = false;
    }
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_NONE"; is_first = false;
    }
    if(object & 33554432) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_TRANSFORM_FEEDBACK_WRITE_BIT_EXT"; is_first = false;
    }
    if(object & 67108864) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_TRANSFORM_FEEDBACK_COUNTER_READ_BIT_EXT"; is_first = false;
    }
    if(object & 134217728) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_TRANSFORM_FEEDBACK_COUNTER_WRITE_BIT_EXT"; is_first = false;
    }
    if(object & 1048576) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_CONDITIONAL_RENDERING_READ_BIT_EXT"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_COLOR_ATTACHMENT_READ_NONCOHERENT_BIT_EXT"; is_first = false;
    }
    if(object & 2097152) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_ACCELERATION_STRUCTURE_READ_BIT_KHR"; is_first = false;
    }
    if(object & 4194304) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_ACCELERATION_STRUCTURE_WRITE_BIT_KHR"; is_first = false;
    }
    if(object & 16777216) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_FRAGMENT_DENSITY_MAP_READ_BIT_EXT"; is_first = false;
    }
    if(object & 8388608) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_FRAGMENT_SHADING_RATE_ATTACHMENT_READ_BIT_KHR"; is_first = false;
    }
    if(object & 131072) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_COMMAND_PREPROCESS_READ_BIT_NV"; is_first = false;
    }
    if(object & 262144) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_COMMAND_PREPROCESS_WRITE_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkImageAspectFlagBits(VkImageAspectFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_ASPECT_COLOR_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_ASPECT_DEPTH_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_ASPECT_STENCIL_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_ASPECT_METADATA_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_ASPECT_PLANE_0_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_ASPECT_PLANE_1_BIT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_ASPECT_PLANE_2_BIT"; is_first = false;
    }
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_ASPECT_NONE"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_ASPECT_MEMORY_PLANE_0_BIT_EXT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_ASPECT_MEMORY_PLANE_1_BIT_EXT"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_ASPECT_MEMORY_PLANE_2_BIT_EXT"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_ASPECT_MEMORY_PLANE_3_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkFormatFeatureFlagBits(VkFormatFeatureFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_SAMPLED_IMAGE_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_STORAGE_IMAGE_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_STORAGE_IMAGE_ATOMIC_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_UNIFORM_TEXEL_BUFFER_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_STORAGE_TEXEL_BUFFER_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_STORAGE_TEXEL_BUFFER_ATOMIC_BIT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_VERTEX_BUFFER_BIT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BIT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BLEND_BIT"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_DEPTH_STENCIL_ATTACHMENT_BIT"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_BLIT_SRC_BIT"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_BLIT_DST_BIT"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_SAMPLED_IMAGE_FILTER_LINEAR_BIT"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_TRANSFER_SRC_BIT"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_TRANSFER_DST_BIT"; is_first = false;
    }
    if(object & 131072) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_MIDPOINT_CHROMA_SAMPLES_BIT"; is_first = false;
    }
    if(object & 262144) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_SAMPLED_IMAGE_YCBCR_CONVERSION_LINEAR_FILTER_BIT"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_SAMPLED_IMAGE_YCBCR_CONVERSION_SEPARATE_RECONSTRUCTION_FILTER_BIT"; is_first = false;
    }
    if(object & 1048576) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_SAMPLED_IMAGE_YCBCR_CONVERSION_CHROMA_RECONSTRUCTION_EXPLICIT_BIT"; is_first = false;
    }
    if(object & 2097152) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_SAMPLED_IMAGE_YCBCR_CONVERSION_CHROMA_RECONSTRUCTION_EXPLICIT_FORCEABLE_BIT"; is_first = false;
    }
    if(object & 4194304) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_DISJOINT_BIT"; is_first = false;
    }
    if(object & 8388608) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_COSITED_CHROMA_SAMPLES_BIT"; is_first = false;
    }
    if(object & 65536) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_SAMPLED_IMAGE_FILTER_MINMAX_BIT"; is_first = false;
    }
    if(object & 33554432) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_VIDEO_DECODE_OUTPUT_BIT_KHR"; is_first = false;
    }
    if(object & 67108864) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_VIDEO_DECODE_DPB_BIT_KHR"; is_first = false;
    }
    if(object & 536870912) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_ACCELERATION_STRUCTURE_VERTEX_BUFFER_BIT_KHR"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_SAMPLED_IMAGE_FILTER_CUBIC_BIT_EXT"; is_first = false;
    }
    if(object & 16777216) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_FRAGMENT_DENSITY_MAP_BIT_EXT"; is_first = false;
    }
    if(object & 1073741824) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_FRAGMENT_SHADING_RATE_ATTACHMENT_BIT_KHR"; is_first = false;
    }
    if(object & 134217728) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_VIDEO_ENCODE_INPUT_BIT_KHR"; is_first = false;
    }
    if(object & 268435456) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_VIDEO_ENCODE_DPB_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkImageCreateFlagBits(VkImageCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_SPARSE_BINDING_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_SPARSE_RESIDENCY_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_SPARSE_ALIASED_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_MUTABLE_FORMAT_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_CUBE_COMPATIBLE_BIT"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_ALIAS_BIT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_SPLIT_INSTANCE_BIND_REGIONS_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_2D_ARRAY_COMPATIBLE_BIT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_BLOCK_TEXEL_VIEW_COMPATIBLE_BIT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_EXTENDED_USAGE_BIT"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_PROTECTED_BIT"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_DISJOINT_BIT"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_CORNER_SAMPLED_BIT_NV"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_SAMPLE_LOCATIONS_COMPATIBLE_DEPTH_BIT_EXT"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_SUBSAMPLED_BIT_EXT"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_RESERVED_19_BIT_EXT"; is_first = false;
    }
    if(object & 65536) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_DESCRIPTOR_BUFFER_CAPTURE_REPLAY_BIT_EXT"; is_first = false;
    }
    if(object & 262144) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_MULTISAMPLED_RENDER_TO_SINGLE_SAMPLED_BIT_EXT"; is_first = false;
    }
    if(object & 131072) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_2D_VIEW_COMPATIBLE_BIT_EXT"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_FRAGMENT_DENSITY_MAP_OFFSET_BIT_QCOM"; is_first = false;
    }
    if(object & 1048576) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CREATE_VIDEO_PROFILE_INDEPENDENT_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkSampleCountFlagBits(VkSampleCountFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SAMPLE_COUNT_1_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SAMPLE_COUNT_2_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SAMPLE_COUNT_4_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SAMPLE_COUNT_8_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SAMPLE_COUNT_16_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SAMPLE_COUNT_32_BIT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SAMPLE_COUNT_64_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkImageUsageFlagBits(VkImageUsageFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_TRANSFER_SRC_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_TRANSFER_DST_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_SAMPLED_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_STORAGE_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_TRANSIENT_ATTACHMENT_BIT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_INPUT_ATTACHMENT_BIT"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_VIDEO_DECODE_DST_BIT_KHR"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_VIDEO_DECODE_SRC_BIT_KHR"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_VIDEO_DECODE_DPB_BIT_KHR"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_FRAGMENT_DENSITY_MAP_BIT_EXT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_FRAGMENT_SHADING_RATE_ATTACHMENT_BIT_KHR"; is_first = false;
    }
    if(object & 4194304) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_HOST_TRANSFER_BIT_EXT"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_VIDEO_ENCODE_DST_BIT_KHR"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_VIDEO_ENCODE_SRC_BIT_KHR"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_VIDEO_ENCODE_DPB_BIT_KHR"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_ATTACHMENT_FEEDBACK_LOOP_BIT_EXT"; is_first = false;
    }
    if(object & 262144) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_INVOCATION_MASK_BIT_HUAWEI"; is_first = false;
    }
    if(object & 1048576) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_SAMPLE_WEIGHT_BIT_QCOM"; is_first = false;
    }
    if(object & 2097152) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_SAMPLE_BLOCK_MATCH_BIT_QCOM"; is_first = false;
    }
    if(object & 16777216) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_RESERVED_24_BIT_COREAVI"; is_first = false;
    }
    if(object & 8388608) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_RESERVED_23_BIT_EXT"; is_first = false;
    }
    if(object & 33554432) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_RESERVED_25_BIT_KHR"; is_first = false;
    }
    if(object & 67108864) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_USAGE_RESERVED_26_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkInstanceCreateFlagBits(VkInstanceCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_INSTANCE_CREATE_ENUMERATE_PORTABILITY_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkMemoryHeapFlagBits(VkMemoryHeapFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_HEAP_DEVICE_LOCAL_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_HEAP_MULTI_INSTANCE_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkMemoryPropertyFlagBits(VkMemoryPropertyFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_PROPERTY_HOST_COHERENT_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_PROPERTY_HOST_CACHED_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_PROPERTY_LAZILY_ALLOCATED_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_PROPERTY_PROTECTED_BIT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_PROPERTY_DEVICE_COHERENT_BIT_AMD"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_PROPERTY_DEVICE_UNCACHED_BIT_AMD"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_PROPERTY_RDMA_CAPABLE_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkQueueFlagBits(VkQueueFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUEUE_GRAPHICS_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUEUE_COMPUTE_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUEUE_TRANSFER_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUEUE_SPARSE_BINDING_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUEUE_PROTECTED_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUEUE_VIDEO_DECODE_BIT_KHR"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUEUE_RESERVED_9_BIT_EXT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUEUE_VIDEO_ENCODE_BIT_KHR"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUEUE_RESERVED_7_BIT_QCOM"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUEUE_RESERVED_11_BIT_ARM"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUEUE_OPTICAL_FLOW_BIT_NV"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUEUE_RESERVED_10_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkDeviceQueueCreateFlagBits(VkDeviceQueueCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEVICE_QUEUE_CREATE_PROTECTED_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEVICE_QUEUE_CREATE_RESERVED_1_BIT_QCOM"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPipelineStageFlagBits(VkPipelineStageFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_DRAW_INDIRECT_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_VERTEX_INPUT_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_VERTEX_SHADER_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_TESSELLATION_CONTROL_SHADER_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_TESSELLATION_EVALUATION_SHADER_BIT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_GEOMETRY_SHADER_BIT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_FRAGMENT_SHADER_BIT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_EARLY_FRAGMENT_TESTS_BIT"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_LATE_FRAGMENT_TESTS_BIT"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_COLOR_ATTACHMENT_OUTPUT_BIT"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_COMPUTE_SHADER_BIT"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_TRANSFER_BIT"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_BOTTOM_OF_PIPE_BIT"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_HOST_BIT"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_ALL_GRAPHICS_BIT"; is_first = false;
    }
    if(object & 65536) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_ALL_COMMANDS_BIT"; is_first = false;
    }
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_NONE"; is_first = false;
    }
    if(object & 16777216) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_TRANSFORM_FEEDBACK_BIT_EXT"; is_first = false;
    }
    if(object & 262144) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_CONDITIONAL_RENDERING_BIT_EXT"; is_first = false;
    }
    if(object & 33554432) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_ACCELERATION_STRUCTURE_BUILD_BIT_KHR"; is_first = false;
    }
    if(object & 2097152) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_RAY_TRACING_SHADER_BIT_KHR"; is_first = false;
    }
    if(object & 8388608) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_FRAGMENT_DENSITY_PROCESS_BIT_EXT"; is_first = false;
    }
    if(object & 4194304) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_FRAGMENT_SHADING_RATE_ATTACHMENT_BIT_KHR"; is_first = false;
    }
    if(object & 131072) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_COMMAND_PREPROCESS_BIT_NV"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_TASK_SHADER_BIT_EXT"; is_first = false;
    }
    if(object & 1048576) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_MESH_SHADER_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkMemoryMapFlagBits(VkMemoryMapFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_MAP_PLACED_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkSparseMemoryBindFlagBits(VkSparseMemoryBindFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SPARSE_MEMORY_BIND_METADATA_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkSparseImageFormatFlagBits(VkSparseImageFormatFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SPARSE_IMAGE_FORMAT_SINGLE_MIPTAIL_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SPARSE_IMAGE_FORMAT_ALIGNED_MIP_SIZE_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SPARSE_IMAGE_FORMAT_NONSTANDARD_BLOCK_SIZE_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkFenceCreateFlagBits(VkFenceCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FENCE_CREATE_SIGNALED_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkEventCreateFlagBits(VkEventCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EVENT_CREATE_DEVICE_ONLY_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkQueryPipelineStatisticFlagBits(VkQueryPipelineStatisticFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_PIPELINE_STATISTIC_INPUT_ASSEMBLY_VERTICES_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_PIPELINE_STATISTIC_INPUT_ASSEMBLY_PRIMITIVES_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_PIPELINE_STATISTIC_VERTEX_SHADER_INVOCATIONS_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_PIPELINE_STATISTIC_GEOMETRY_SHADER_INVOCATIONS_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_PIPELINE_STATISTIC_GEOMETRY_SHADER_PRIMITIVES_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_PIPELINE_STATISTIC_CLIPPING_INVOCATIONS_BIT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_PIPELINE_STATISTIC_CLIPPING_PRIMITIVES_BIT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_PIPELINE_STATISTIC_FRAGMENT_SHADER_INVOCATIONS_BIT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_PIPELINE_STATISTIC_TESSELLATION_CONTROL_SHADER_PATCHES_BIT"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_PIPELINE_STATISTIC_TESSELLATION_EVALUATION_SHADER_INVOCATIONS_BIT"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_PIPELINE_STATISTIC_COMPUTE_SHADER_INVOCATIONS_BIT"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_PIPELINE_STATISTIC_TASK_SHADER_INVOCATIONS_BIT_EXT"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_PIPELINE_STATISTIC_MESH_SHADER_INVOCATIONS_BIT_EXT"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_PIPELINE_STATISTIC_CLUSTER_CULLING_SHADER_INVOCATIONS_BIT_HUAWEI"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkQueryResultFlagBits(VkQueryResultFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_RESULT_64_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_RESULT_WAIT_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_RESULT_WITH_AVAILABILITY_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_RESULT_PARTIAL_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_RESULT_WITH_STATUS_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkBufferCreateFlagBits(VkBufferCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_CREATE_SPARSE_BINDING_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_CREATE_SPARSE_RESIDENCY_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_CREATE_SPARSE_ALIASED_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_CREATE_PROTECTED_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_CREATE_DEVICE_ADDRESS_CAPTURE_REPLAY_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_CREATE_DESCRIPTOR_BUFFER_CAPTURE_REPLAY_BIT_EXT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_CREATE_VIDEO_PROFILE_INDEPENDENT_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkBufferUsageFlagBits(VkBufferUsageFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_TRANSFER_SRC_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_TRANSFER_DST_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_UNIFORM_TEXEL_BUFFER_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_STORAGE_TEXEL_BUFFER_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_STORAGE_BUFFER_BIT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_INDEX_BUFFER_BIT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_VERTEX_BUFFER_BIT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT"; is_first = false;
    }
    if(object & 131072) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_VIDEO_DECODE_SRC_BIT_KHR"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_VIDEO_DECODE_DST_BIT_KHR"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_TRANSFORM_FEEDBACK_BUFFER_BIT_EXT"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_TRANSFORM_FEEDBACK_COUNTER_BUFFER_BIT_EXT"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_CONDITIONAL_RENDERING_BIT_EXT"; is_first = false;
    }
    if(object & 33554432) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_EXECUTION_GRAPH_SCRATCH_BIT_AMDX"; is_first = false;
    }
    if(object & 268435456) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_RESERVED_28_BIT_KHR"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_ACCELERATION_STRUCTURE_BUILD_INPUT_READ_ONLY_BIT_KHR"; is_first = false;
    }
    if(object & 1048576) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_ACCELERATION_STRUCTURE_STORAGE_BIT_KHR"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_SHADER_BINDING_TABLE_BIT_KHR"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_VIDEO_ENCODE_DST_BIT_KHR"; is_first = false;
    }
    if(object & 65536) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_VIDEO_ENCODE_SRC_BIT_KHR"; is_first = false;
    }
    if(object & 134217728) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_RESERVED_27_BIT_QCOM"; is_first = false;
    }
    if(object & 2097152) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_SAMPLER_DESCRIPTOR_BUFFER_BIT_EXT"; is_first = false;
    }
    if(object & 4194304) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_RESOURCE_DESCRIPTOR_BUFFER_BIT_EXT"; is_first = false;
    }
    if(object & 67108864) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_PUSH_DESCRIPTORS_DESCRIPTOR_BUFFER_BIT_EXT"; is_first = false;
    }
    if(object & 8388608) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_MICROMAP_BUILD_INPUT_READ_ONLY_BIT_EXT"; is_first = false;
    }
    if(object & 16777216) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_MICROMAP_STORAGE_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkImageViewCreateFlagBits(VkImageViewCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_VIEW_CREATE_FRAGMENT_DENSITY_MAP_DYNAMIC_BIT_EXT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_VIEW_CREATE_DESCRIPTOR_BUFFER_CAPTURE_REPLAY_BIT_EXT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_VIEW_CREATE_FRAGMENT_DENSITY_MAP_DEFERRED_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPipelineCacheCreateFlagBits(VkPipelineCacheCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CACHE_CREATE_EXTERNALLY_SYNCHRONIZED_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkColorComponentFlagBits(VkColorComponentFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COLOR_COMPONENT_R_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COLOR_COMPONENT_G_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COLOR_COMPONENT_B_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COLOR_COMPONENT_A_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPipelineCreateFlagBits(VkPipelineCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_DISABLE_OPTIMIZATION_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_ALLOW_DERIVATIVES_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_DERIVATIVE_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_VIEW_INDEX_FROM_DEVICE_INDEX_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_DISPATCH_BASE_BIT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_FAIL_ON_PIPELINE_COMPILE_REQUIRED_BIT"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_EARLY_RETURN_ON_FAILURE_BIT"; is_first = false;
    }
    if(object & 2097152) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_RENDERING_FRAGMENT_SHADING_RATE_ATTACHMENT_BIT_KHR"; is_first = false;
    }
    if(object & 4194304) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_RENDERING_FRAGMENT_DENSITY_MAP_ATTACHMENT_BIT_EXT"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_RAY_TRACING_NO_NULL_ANY_HIT_SHADERS_BIT_KHR"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_RAY_TRACING_NO_NULL_CLOSEST_HIT_SHADERS_BIT_KHR"; is_first = false;
    }
    if(object & 65536) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_RAY_TRACING_NO_NULL_MISS_SHADERS_BIT_KHR"; is_first = false;
    }
    if(object & 131072) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_RAY_TRACING_NO_NULL_INTERSECTION_SHADERS_BIT_KHR"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_RAY_TRACING_SKIP_TRIANGLES_BIT_KHR"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_RAY_TRACING_SKIP_AABBS_BIT_KHR"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_RAY_TRACING_SHADER_GROUP_HANDLE_CAPTURE_REPLAY_BIT_KHR"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_DEFER_COMPILE_BIT_NV"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_CAPTURE_STATISTICS_BIT_KHR"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_CAPTURE_INTERNAL_REPRESENTATIONS_BIT_KHR"; is_first = false;
    }
    if(object & 262144) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_INDIRECT_BINDABLE_BIT_NV"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_LIBRARY_BIT_KHR"; is_first = false;
    }
    if(object & 536870912) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_DESCRIPTOR_BUFFER_BIT_EXT"; is_first = false;
    }
    if(object & 8388608) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_RETAIN_LINK_TIME_OPTIMIZATION_INFO_BIT_EXT"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_LINK_TIME_OPTIMIZATION_BIT_EXT"; is_first = false;
    }
    if(object & 1048576) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_RAY_TRACING_ALLOW_MOTION_BIT_NV"; is_first = false;
    }
    if(object & 33554432) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_COLOR_ATTACHMENT_FEEDBACK_LOOP_BIT_EXT"; is_first = false;
    }
    if(object & 67108864) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_DEPTH_STENCIL_ATTACHMENT_FEEDBACK_LOOP_BIT_EXT"; is_first = false;
    }
    if(object & 16777216) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_RAY_TRACING_OPACITY_MICROMAP_BIT_EXT"; is_first = false;
    }
    if(object & 268435456) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_RAY_TRACING_DISPLACEMENT_MICROMAP_BIT_NV"; is_first = false;
    }
    if(object & 134217728) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_NO_PROTECTED_ACCESS_BIT_EXT"; is_first = false;
    }
    if(object & 1073741824) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_PROTECTED_ACCESS_ONLY_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPipelineShaderStageCreateFlagBits(VkPipelineShaderStageCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_SHADER_STAGE_CREATE_ALLOW_VARYING_SUBGROUP_SIZE_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_SHADER_STAGE_CREATE_REQUIRE_FULL_SUBGROUPS_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_SHADER_STAGE_CREATE_RESERVED_3_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkShaderStageFlagBits(VkShaderStageFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_VERTEX_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_GEOMETRY_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_FRAGMENT_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_COMPUTE_BIT"; is_first = false;
    }
    if(object == 0x0000001F) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_ALL_GRAPHICS"; is_first = false;
    }
    if(object == 0x7FFFFFFF) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_ALL"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_RAYGEN_BIT_KHR"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_ANY_HIT_BIT_KHR"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_CLOSEST_HIT_BIT_KHR"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_MISS_BIT_KHR"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_INTERSECTION_BIT_KHR"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_CALLABLE_BIT_KHR"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_TASK_BIT_EXT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_MESH_BIT_EXT"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_SUBPASS_SHADING_BIT_HUAWEI"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_STAGE_CLUSTER_CULLING_BIT_HUAWEI"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkCullModeFlagBits(VkCullModeFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_CULL_MODE_NONE"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_CULL_MODE_FRONT_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_CULL_MODE_BACK_BIT"; is_first = false;
    }
    if(object == 0x00000003) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_CULL_MODE_FRONT_AND_BACK"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPipelineDepthStencilStateCreateFlagBits(VkPipelineDepthStencilStateCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_DEPTH_STENCIL_STATE_CREATE_RASTERIZATION_ORDER_ATTACHMENT_DEPTH_ACCESS_BIT_EXT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_DEPTH_STENCIL_STATE_CREATE_RASTERIZATION_ORDER_ATTACHMENT_STENCIL_ACCESS_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPipelineColorBlendStateCreateFlagBits(VkPipelineColorBlendStateCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_COLOR_BLEND_STATE_CREATE_RASTERIZATION_ORDER_ATTACHMENT_ACCESS_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPipelineLayoutCreateFlagBits(VkPipelineLayoutCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_LAYOUT_CREATE_RESERVED_0_BIT_AMD"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_LAYOUT_CREATE_INDEPENDENT_SETS_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkSamplerCreateFlagBits(VkSamplerCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SAMPLER_CREATE_SUBSAMPLED_BIT_EXT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SAMPLER_CREATE_SUBSAMPLED_COARSE_RECONSTRUCTION_BIT_EXT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SAMPLER_CREATE_DESCRIPTOR_BUFFER_CAPTURE_REPLAY_BIT_EXT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SAMPLER_CREATE_NON_SEAMLESS_CUBE_MAP_BIT_EXT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SAMPLER_CREATE_IMAGE_PROCESSING_BIT_QCOM"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkDescriptorPoolCreateFlagBits(VkDescriptorPoolCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_POOL_CREATE_FREE_DESCRIPTOR_SET_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_POOL_CREATE_UPDATE_AFTER_BIND_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_POOL_CREATE_HOST_ONLY_BIT_EXT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_POOL_CREATE_ALLOW_OVERALLOCATION_SETS_BIT_NV"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_POOL_CREATE_ALLOW_OVERALLOCATION_POOLS_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkDescriptorSetLayoutCreateFlagBits(VkDescriptorSetLayoutCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_SET_LAYOUT_CREATE_UPDATE_AFTER_BIND_POOL_BIT"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_SET_LAYOUT_CREATE_PUSH_DESCRIPTOR_BIT_KHR"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_SET_LAYOUT_CREATE_DESCRIPTOR_BUFFER_BIT_EXT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_SET_LAYOUT_CREATE_EMBEDDED_IMMUTABLE_SAMPLERS_BIT_EXT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_SET_LAYOUT_CREATE_RESERVED_3_BIT_AMD"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_SET_LAYOUT_CREATE_INDIRECT_BINDABLE_BIT_NV"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_SET_LAYOUT_CREATE_HOST_ONLY_POOL_BIT_EXT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_SET_LAYOUT_CREATE_PER_STAGE_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkAttachmentDescriptionFlagBits(VkAttachmentDescriptionFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ATTACHMENT_DESCRIPTION_MAY_ALIAS_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkDependencyFlagBits(VkDependencyFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEPENDENCY_BY_REGION_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEPENDENCY_DEVICE_GROUP_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEPENDENCY_VIEW_LOCAL_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEPENDENCY_FEEDBACK_LOOP_BIT_EXT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEPENDENCY_EXTENSION_586_BIT_IMG"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkFramebufferCreateFlagBits(VkFramebufferCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FRAMEBUFFER_CREATE_IMAGELESS_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkRenderPassCreateFlagBits(VkRenderPassCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_RENDER_PASS_CREATE_RESERVED_0_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_RENDER_PASS_CREATE_TRANSFORM_BIT_QCOM"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkSubpassDescriptionFlagBits(VkSubpassDescriptionFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBPASS_DESCRIPTION_PER_VIEW_ATTRIBUTES_BIT_NVX"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBPASS_DESCRIPTION_PER_VIEW_POSITION_X_ONLY_BIT_NVX"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBPASS_DESCRIPTION_FRAGMENT_REGION_BIT_QCOM"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBPASS_DESCRIPTION_SHADER_RESOLVE_BIT_QCOM"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBPASS_DESCRIPTION_RASTERIZATION_ORDER_ATTACHMENT_COLOR_ACCESS_BIT_EXT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBPASS_DESCRIPTION_RASTERIZATION_ORDER_ATTACHMENT_DEPTH_ACCESS_BIT_EXT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBPASS_DESCRIPTION_RASTERIZATION_ORDER_ATTACHMENT_STENCIL_ACCESS_BIT_EXT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBPASS_DESCRIPTION_ENABLE_LEGACY_DITHERING_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkCommandPoolCreateFlagBits(VkCommandPoolCreateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COMMAND_POOL_CREATE_TRANSIENT_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COMMAND_POOL_CREATE_RESET_COMMAND_BUFFER_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COMMAND_POOL_CREATE_PROTECTED_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkCommandPoolResetFlagBits(VkCommandPoolResetFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COMMAND_POOL_RESET_RELEASE_RESOURCES_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COMMAND_POOL_RESET_RESERVED_1_BIT_COREAVI"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkCommandBufferUsageFlagBits(VkCommandBufferUsageFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COMMAND_BUFFER_USAGE_RENDER_PASS_CONTINUE_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COMMAND_BUFFER_USAGE_SIMULTANEOUS_USE_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkQueryControlFlagBits(VkQueryControlFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_QUERY_CONTROL_PRECISE_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkCommandBufferResetFlagBits(VkCommandBufferResetFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COMMAND_BUFFER_RESET_RELEASE_RESOURCES_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkStencilFaceFlagBits(VkStencilFaceFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_STENCIL_FACE_FRONT_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_STENCIL_FACE_BACK_BIT"; is_first = false;
    }
    if(object == 0x00000003) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_STENCIL_FACE_FRONT_AND_BACK"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkSubgroupFeatureFlagBits(VkSubgroupFeatureFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBGROUP_FEATURE_BASIC_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBGROUP_FEATURE_VOTE_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBGROUP_FEATURE_ARITHMETIC_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBGROUP_FEATURE_BALLOT_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBGROUP_FEATURE_SHUFFLE_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBGROUP_FEATURE_SHUFFLE_RELATIVE_BIT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBGROUP_FEATURE_CLUSTERED_BIT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBGROUP_FEATURE_QUAD_BIT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBGROUP_FEATURE_PARTITIONED_BIT_NV"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBGROUP_FEATURE_ROTATE_BIT_KHR"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBGROUP_FEATURE_ROTATE_CLUSTERED_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPeerMemoryFeatureFlagBits(VkPeerMemoryFeatureFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PEER_MEMORY_FEATURE_COPY_SRC_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PEER_MEMORY_FEATURE_COPY_DST_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PEER_MEMORY_FEATURE_GENERIC_SRC_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PEER_MEMORY_FEATURE_GENERIC_DST_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkMemoryAllocateFlagBits(VkMemoryAllocateFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_ALLOCATE_DEVICE_MASK_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_CAPTURE_REPLAY_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkExternalMemoryHandleTypeFlagBits(VkExternalMemoryHandleTypeFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_KMT_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D11_TEXTURE_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D11_TEXTURE_KMT_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D12_HEAP_BIT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D12_RESOURCE_BIT"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_DMA_BUF_BIT_EXT"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_ANDROID_HARDWARE_BUFFER_BIT_ANDROID"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_HOST_ALLOCATION_BIT_EXT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_HOST_MAPPED_FOREIGN_MEMORY_BIT_EXT"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_ZIRCON_VMO_BIT_FUCHSIA"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_RDMA_ADDRESS_BIT_NV"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_SCI_BUF_BIT_NV"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_SCREEN_BUFFER_BIT_QNX"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_590_BIT_HUAWEI"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkExternalMemoryFeatureFlagBits(VkExternalMemoryFeatureFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_FEATURE_DEDICATED_ONLY_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_FEATURE_EXPORTABLE_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkExternalFenceHandleTypeFlagBits(VkExternalFenceHandleTypeFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_FENCE_HANDLE_TYPE_OPAQUE_FD_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_FENCE_HANDLE_TYPE_OPAQUE_WIN32_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_FENCE_HANDLE_TYPE_OPAQUE_WIN32_KMT_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_FENCE_HANDLE_TYPE_SYNC_FD_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_FENCE_HANDLE_TYPE_SCI_SYNC_OBJ_BIT_NV"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_FENCE_HANDLE_TYPE_SCI_SYNC_FENCE_BIT_NV"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_FENCE_HANDLE_TYPE_SCI_SYNC_OBJ_BIT_NV"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_FENCE_HANDLE_TYPE_SCI_SYNC_FENCE_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkExternalFenceFeatureFlagBits(VkExternalFenceFeatureFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_FENCE_FEATURE_EXPORTABLE_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_FENCE_FEATURE_IMPORTABLE_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkFenceImportFlagBits(VkFenceImportFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FENCE_IMPORT_TEMPORARY_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkSemaphoreImportFlagBits(VkSemaphoreImportFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SEMAPHORE_IMPORT_TEMPORARY_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkExternalSemaphoreHandleTypeFlagBits(VkExternalSemaphoreHandleTypeFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_OPAQUE_FD_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_OPAQUE_WIN32_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_OPAQUE_WIN32_KMT_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_D3D12_FENCE_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_SYNC_FD_BIT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_ZIRCON_EVENT_BIT_FUCHSIA"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_SCI_SYNC_OBJ_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkExternalSemaphoreFeatureFlagBits(VkExternalSemaphoreFeatureFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_SEMAPHORE_FEATURE_EXPORTABLE_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_SEMAPHORE_FEATURE_IMPORTABLE_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkResolveModeFlagBits(VkResolveModeFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_RESOLVE_MODE_NONE"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_RESOLVE_MODE_SAMPLE_ZERO_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_RESOLVE_MODE_AVERAGE_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_RESOLVE_MODE_MIN_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_RESOLVE_MODE_MAX_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_RESOLVE_MODE_EXTERNAL_FORMAT_DOWNSAMPLE_ANDROID"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkDescriptorBindingFlagBits(VkDescriptorBindingFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_BINDING_UPDATE_AFTER_BIND_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_BINDING_UPDATE_UNUSED_WHILE_PENDING_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_BINDING_PARTIALLY_BOUND_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_BINDING_VARIABLE_DESCRIPTOR_COUNT_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DESCRIPTOR_BINDING_RESERVED_4_BIT_QCOM"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkSemaphoreWaitFlagBits(VkSemaphoreWaitFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SEMAPHORE_WAIT_ANY_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPipelineCreationFeedbackFlagBits(VkPipelineCreationFeedbackFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATION_FEEDBACK_VALID_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATION_FEEDBACK_APPLICATION_PIPELINE_CACHE_HIT_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATION_FEEDBACK_BASE_PIPELINE_ACCELERATION_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkToolPurposeFlagBits(VkToolPurposeFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_TOOL_PURPOSE_VALIDATION_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_TOOL_PURPOSE_PROFILING_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_TOOL_PURPOSE_TRACING_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_TOOL_PURPOSE_ADDITIONAL_FEATURES_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_TOOL_PURPOSE_MODIFYING_FEATURES_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_TOOL_PURPOSE_DEBUG_REPORTING_BIT_EXT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_TOOL_PURPOSE_DEBUG_MARKERS_BIT_EXT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_TOOL_PURPOSE_DEBUG_REPORTING_BIT_EXT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_TOOL_PURPOSE_DEBUG_MARKERS_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPipelineStageFlagBits2(VkPipelineStageFlagBits2 object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_NONE"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_TOP_OF_PIPE_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_DRAW_INDIRECT_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_VERTEX_INPUT_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_VERTEX_SHADER_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_TESSELLATION_CONTROL_SHADER_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_TESSELLATION_EVALUATION_SHADER_BIT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_GEOMETRY_SHADER_BIT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_FRAGMENT_SHADER_BIT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_EARLY_FRAGMENT_TESTS_BIT"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_LATE_FRAGMENT_TESTS_BIT"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_COLOR_ATTACHMENT_OUTPUT_BIT"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_COMPUTE_SHADER_BIT"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_ALL_TRANSFER_BIT"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_BOTTOM_OF_PIPE_BIT"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_HOST_BIT"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_ALL_GRAPHICS_BIT"; is_first = false;
    }
    if(object & 65536) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_ALL_COMMANDS_BIT"; is_first = false;
    }
    if(object & 4294967296) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_COPY_BIT"; is_first = false;
    }
    if(object & 8589934592) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_RESOLVE_BIT"; is_first = false;
    }
    if(object & 17179869184) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_BLIT_BIT"; is_first = false;
    }
    if(object & 34359738368) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_CLEAR_BIT"; is_first = false;
    }
    if(object & 68719476736) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_INDEX_INPUT_BIT"; is_first = false;
    }
    if(object & 137438953472) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_VERTEX_ATTRIBUTE_INPUT_BIT"; is_first = false;
    }
    if(object & 274877906944) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_PRE_RASTERIZATION_SHADERS_BIT"; is_first = false;
    }
    if(object & 67108864) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_VIDEO_DECODE_BIT_KHR"; is_first = false;
    }
    if(object & 134217728) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_VIDEO_ENCODE_BIT_KHR"; is_first = false;
    }
    if(object & 16777216) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_TRANSFORM_FEEDBACK_BIT_EXT"; is_first = false;
    }
    if(object & 262144) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_CONDITIONAL_RENDERING_BIT_EXT"; is_first = false;
    }
    if(object & 131072) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_COMMAND_PREPROCESS_BIT_NV"; is_first = false;
    }
    if(object & 4194304) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_FRAGMENT_SHADING_RATE_ATTACHMENT_BIT_KHR"; is_first = false;
    }
    if(object & 33554432) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_ACCELERATION_STRUCTURE_BUILD_BIT_KHR"; is_first = false;
    }
    if(object & 2097152) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_RAY_TRACING_SHADER_BIT_KHR"; is_first = false;
    }
    if(object & 8388608) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_FRAGMENT_DENSITY_PROCESS_BIT_EXT"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_TASK_SHADER_BIT_EXT"; is_first = false;
    }
    if(object & 1048576) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_MESH_SHADER_BIT_EXT"; is_first = false;
    }
    if(object & 549755813888) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_SUBPASS_SHADER_BIT_HUAWEI"; is_first = false;
    }
    if(object & 1099511627776) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_INVOCATION_MASK_BIT_HUAWEI"; is_first = false;
    }
    if(object & 268435456) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_ACCELERATION_STRUCTURE_COPY_BIT_KHR"; is_first = false;
    }
    if(object & 1073741824) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_MICROMAP_BUILD_BIT_EXT"; is_first = false;
    }
    if(object & 2199023255552) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_CLUSTER_CULLING_SHADER_BIT_HUAWEI"; is_first = false;
    }
    if(object & 8796093022208) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_RESERVED_43_BIT_ARM"; is_first = false;
    }
    if(object & 536870912) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_OPTICAL_FLOW_BIT_NV"; is_first = false;
    }
    if(object & 17592186044416) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_RESERVED_44_BIT_NV"; is_first = false;
    }
    if(object & 4398046511104) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_RESERVED_42_BIT_EXT"; is_first = false;
    }
    if(object & 35184372088832) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_STAGE_2_RESERVED_45_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkAccessFlagBits2(VkAccessFlagBits2 object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_NONE"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_INDIRECT_COMMAND_READ_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_INDEX_READ_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_VERTEX_ATTRIBUTE_READ_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_UNIFORM_READ_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_INPUT_ATTACHMENT_READ_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_SHADER_READ_BIT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_SHADER_WRITE_BIT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_COLOR_ATTACHMENT_READ_BIT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_COLOR_ATTACHMENT_WRITE_BIT"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_DEPTH_STENCIL_ATTACHMENT_READ_BIT"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_DEPTH_STENCIL_ATTACHMENT_WRITE_BIT"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_TRANSFER_READ_BIT"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_TRANSFER_WRITE_BIT"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_HOST_READ_BIT"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_HOST_WRITE_BIT"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_MEMORY_READ_BIT"; is_first = false;
    }
    if(object & 65536) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_MEMORY_WRITE_BIT"; is_first = false;
    }
    if(object & 4294967296) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_SHADER_SAMPLED_READ_BIT"; is_first = false;
    }
    if(object & 8589934592) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_SHADER_STORAGE_READ_BIT"; is_first = false;
    }
    if(object & 17179869184) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_SHADER_STORAGE_WRITE_BIT"; is_first = false;
    }
    if(object & 34359738368) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_VIDEO_DECODE_READ_BIT_KHR"; is_first = false;
    }
    if(object & 68719476736) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_VIDEO_DECODE_WRITE_BIT_KHR"; is_first = false;
    }
    if(object & 144115188075855872) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_RESERVED_57_BIT_KHR"; is_first = false;
    }
    if(object & 288230376151711744) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_RESERVED_58_BIT_KHR"; is_first = false;
    }
    if(object & 576460752303423488) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_RESERVED_59_BIT_KHR"; is_first = false;
    }
    if(object & 70368744177664) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_RESERVED_46_BIT_INTEL"; is_first = false;
    }
    if(object & 137438953472) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_VIDEO_ENCODE_READ_BIT_KHR"; is_first = false;
    }
    if(object & 274877906944) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_VIDEO_ENCODE_WRITE_BIT_KHR"; is_first = false;
    }
    if(object & 2251799813685248) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_RESERVED_51_BIT_QCOM"; is_first = false;
    }
    if(object & 4503599627370496) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_RESERVED_52_BIT_QCOM"; is_first = false;
    }
    if(object & 9007199254740992) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_RESERVED_53_BIT_QCOM"; is_first = false;
    }
    if(object & 18014398509481984) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_RESERVED_54_BIT_QCOM"; is_first = false;
    }
    if(object & 33554432) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_TRANSFORM_FEEDBACK_WRITE_BIT_EXT"; is_first = false;
    }
    if(object & 67108864) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_TRANSFORM_FEEDBACK_COUNTER_READ_BIT_EXT"; is_first = false;
    }
    if(object & 134217728) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_TRANSFORM_FEEDBACK_COUNTER_WRITE_BIT_EXT"; is_first = false;
    }
    if(object & 1048576) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_CONDITIONAL_RENDERING_READ_BIT_EXT"; is_first = false;
    }
    if(object & 131072) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_COMMAND_PREPROCESS_READ_BIT_NV"; is_first = false;
    }
    if(object & 262144) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_COMMAND_PREPROCESS_WRITE_BIT_NV"; is_first = false;
    }
    if(object & 8388608) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_FRAGMENT_SHADING_RATE_ATTACHMENT_READ_BIT_KHR"; is_first = false;
    }
    if(object & 2097152) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_ACCELERATION_STRUCTURE_READ_BIT_KHR"; is_first = false;
    }
    if(object & 4194304) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_ACCELERATION_STRUCTURE_WRITE_BIT_KHR"; is_first = false;
    }
    if(object & 16777216) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_FRAGMENT_DENSITY_MAP_READ_BIT_EXT"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_COLOR_ATTACHMENT_READ_NONCOHERENT_BIT_EXT"; is_first = false;
    }
    if(object & 2199023255552) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_DESCRIPTOR_BUFFER_READ_BIT_EXT"; is_first = false;
    }
    if(object & 549755813888) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_INVOCATION_MASK_READ_BIT_HUAWEI"; is_first = false;
    }
    if(object & 1099511627776) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_SHADER_BINDING_TABLE_READ_BIT_KHR"; is_first = false;
    }
    if(object & 17592186044416) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_MICROMAP_READ_BIT_EXT"; is_first = false;
    }
    if(object & 35184372088832) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_MICROMAP_WRITE_BIT_EXT"; is_first = false;
    }
    if(object & 562949953421312) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_RESERVED_49_BIT_ARM"; is_first = false;
    }
    if(object & 1125899906842624) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_RESERVED_50_BIT_ARM"; is_first = false;
    }
    if(object & 4398046511104) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_OPTICAL_FLOW_READ_BIT_NV"; is_first = false;
    }
    if(object & 8796093022208) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_OPTICAL_FLOW_WRITE_BIT_NV"; is_first = false;
    }
    if(object & 140737488355328) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_RESERVED_47_BIT_EXT"; is_first = false;
    }
    if(object & 281474976710656) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_RESERVED_48_BIT_EXT"; is_first = false;
    }
    if(object & 36028797018963968) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_RESERVED_55_BIT_NV"; is_first = false;
    }
    if(object & 72057594037927936) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCESS_2_RESERVED_56_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkSubmitFlagBits(VkSubmitFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SUBMIT_PROTECTED_BIT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkRenderingFlagBits(VkRenderingFlagBits object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_RENDERING_CONTENTS_SECONDARY_COMMAND_BUFFERS_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_RENDERING_SUSPENDING_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_RENDERING_RESUMING_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_RENDERING_ENABLE_LEGACY_DITHERING_BIT_EXT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_RENDERING_EXTENSION_505_BIT_EXT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_RENDERING_CONTENTS_INLINE_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkFormatFeatureFlagBits2(VkFormatFeatureFlagBits2 object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_SAMPLED_IMAGE_BIT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_STORAGE_IMAGE_BIT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_STORAGE_IMAGE_ATOMIC_BIT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_UNIFORM_TEXEL_BUFFER_BIT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_STORAGE_TEXEL_BUFFER_BIT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_STORAGE_TEXEL_BUFFER_ATOMIC_BIT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_VERTEX_BUFFER_BIT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_COLOR_ATTACHMENT_BIT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_COLOR_ATTACHMENT_BLEND_BIT"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_DEPTH_STENCIL_ATTACHMENT_BIT"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_BLIT_SRC_BIT"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_BLIT_DST_BIT"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_SAMPLED_IMAGE_FILTER_LINEAR_BIT"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_SAMPLED_IMAGE_FILTER_CUBIC_BIT"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_TRANSFER_SRC_BIT"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_TRANSFER_DST_BIT"; is_first = false;
    }
    if(object & 65536) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_SAMPLED_IMAGE_FILTER_MINMAX_BIT"; is_first = false;
    }
    if(object & 131072) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_MIDPOINT_CHROMA_SAMPLES_BIT"; is_first = false;
    }
    if(object & 262144) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_SAMPLED_IMAGE_YCBCR_CONVERSION_LINEAR_FILTER_BIT"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_SAMPLED_IMAGE_YCBCR_CONVERSION_SEPARATE_RECONSTRUCTION_FILTER_BIT"; is_first = false;
    }
    if(object & 1048576) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_SAMPLED_IMAGE_YCBCR_CONVERSION_CHROMA_RECONSTRUCTION_EXPLICIT_BIT"; is_first = false;
    }
    if(object & 2097152) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_SAMPLED_IMAGE_YCBCR_CONVERSION_CHROMA_RECONSTRUCTION_EXPLICIT_FORCEABLE_BIT"; is_first = false;
    }
    if(object & 4194304) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_DISJOINT_BIT"; is_first = false;
    }
    if(object & 8388608) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_COSITED_CHROMA_SAMPLES_BIT"; is_first = false;
    }
    if(object & 2147483648) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_STORAGE_READ_WITHOUT_FORMAT_BIT"; is_first = false;
    }
    if(object & 4294967296) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_STORAGE_WRITE_WITHOUT_FORMAT_BIT"; is_first = false;
    }
    if(object & 8589934592) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_SAMPLED_IMAGE_DEPTH_COMPARISON_BIT"; is_first = false;
    }
    if(object & 33554432) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_VIDEO_DECODE_OUTPUT_BIT_KHR"; is_first = false;
    }
    if(object & 67108864) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_VIDEO_DECODE_DPB_BIT_KHR"; is_first = false;
    }
    if(object & 536870912) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_ACCELERATION_STRUCTURE_VERTEX_BUFFER_BIT_KHR"; is_first = false;
    }
    if(object & 16777216) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_FRAGMENT_DENSITY_MAP_BIT_EXT"; is_first = false;
    }
    if(object & 1073741824) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_FRAGMENT_SHADING_RATE_ATTACHMENT_BIT_KHR"; is_first = false;
    }
    if(object & 17592186044416) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_RESERVED_44_BIT_EXT"; is_first = false;
    }
    if(object & 35184372088832) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_RESERVED_45_BIT_EXT"; is_first = false;
    }
    if(object & 70368744177664) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_HOST_IMAGE_TRANSFER_BIT_EXT"; is_first = false;
    }
    if(object & 134217728) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_VIDEO_ENCODE_INPUT_BIT_KHR"; is_first = false;
    }
    if(object & 268435456) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_VIDEO_ENCODE_DPB_BIT_KHR"; is_first = false;
    }
    if(object & 2251799813685248) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_RESERVED_51_BIT_EXT"; is_first = false;
    }
    if(object & 274877906944) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_LINEAR_COLOR_ATTACHMENT_BIT_NV"; is_first = false;
    }
    if(object & 17179869184) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_WEIGHT_IMAGE_BIT_QCOM"; is_first = false;
    }
    if(object & 34359738368) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_WEIGHT_SAMPLED_IMAGE_BIT_QCOM"; is_first = false;
    }
    if(object & 68719476736) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_BLOCK_MATCHING_BIT_QCOM"; is_first = false;
    }
    if(object & 137438953472) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_BOX_FILTER_SAMPLED_BIT_QCOM"; is_first = false;
    }
    if(object & 140737488355328) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_RESERVED_47_BIT_ARM"; is_first = false;
    }
    if(object & 549755813888) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_RESERVED_39_BIT_EXT"; is_first = false;
    }
    if(object & 1099511627776) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_OPTICAL_FLOW_IMAGE_BIT_NV"; is_first = false;
    }
    if(object & 2199023255552) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_OPTICAL_FLOW_VECTOR_BIT_NV"; is_first = false;
    }
    if(object & 4398046511104) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_OPTICAL_FLOW_COST_BIT_NV"; is_first = false;
    }
    if(object & 281474976710656) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_RESERVED_48_BIT_EXT"; is_first = false;
    }
    if(object & 562949953421312) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_RESERVED_49_BIT_KHR"; is_first = false;
    }
    if(object & 1125899906842624) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FORMAT_FEATURE_2_RESERVED_50_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkSurfaceTransformFlagBitsKHR(VkSurfaceTransformFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SURFACE_TRANSFORM_IDENTITY_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SURFACE_TRANSFORM_ROTATE_90_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SURFACE_TRANSFORM_ROTATE_180_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SURFACE_TRANSFORM_ROTATE_270_BIT_KHR"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SURFACE_TRANSFORM_HORIZONTAL_MIRROR_BIT_KHR"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SURFACE_TRANSFORM_HORIZONTAL_MIRROR_ROTATE_90_BIT_KHR"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SURFACE_TRANSFORM_HORIZONTAL_MIRROR_ROTATE_180_BIT_KHR"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SURFACE_TRANSFORM_HORIZONTAL_MIRROR_ROTATE_270_BIT_KHR"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SURFACE_TRANSFORM_INHERIT_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkCompositeAlphaFlagBitsKHR(VkCompositeAlphaFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COMPOSITE_ALPHA_OPAQUE_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COMPOSITE_ALPHA_PRE_MULTIPLIED_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COMPOSITE_ALPHA_POST_MULTIPLIED_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_COMPOSITE_ALPHA_INHERIT_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkSwapchainCreateFlagBitsKHR(VkSwapchainCreateFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SWAPCHAIN_CREATE_SPLIT_INSTANCE_BIND_REGIONS_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SWAPCHAIN_CREATE_PROTECTED_BIT_KHR"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SWAPCHAIN_CREATE_SPLIT_INSTANCE_BIND_REGIONS_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SWAPCHAIN_CREATE_DEFERRED_MEMORY_ALLOCATION_BIT_EXT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SWAPCHAIN_CREATE_RESERVED_4_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkDeviceGroupPresentModeFlagBitsKHR(VkDeviceGroupPresentModeFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEVICE_GROUP_PRESENT_MODE_LOCAL_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEVICE_GROUP_PRESENT_MODE_REMOTE_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEVICE_GROUP_PRESENT_MODE_SUM_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEVICE_GROUP_PRESENT_MODE_LOCAL_MULTI_DEVICE_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkDisplayPlaneAlphaFlagBitsKHR(VkDisplayPlaneAlphaFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DISPLAY_PLANE_ALPHA_OPAQUE_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DISPLAY_PLANE_ALPHA_GLOBAL_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DISPLAY_PLANE_ALPHA_PER_PIXEL_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DISPLAY_PLANE_ALPHA_PER_PIXEL_PREMULTIPLIED_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkDebugReportFlagBitsEXT(VkDebugReportFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEBUG_REPORT_INFORMATION_BIT_EXT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEBUG_REPORT_WARNING_BIT_EXT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEBUG_REPORT_ERROR_BIT_EXT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEBUG_REPORT_DEBUG_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoCodecOperationFlagBitsKHR(VkVideoCodecOperationFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CODEC_OPERATION_NONE_KHR"; is_first = false;
    }
    if(object & 65536) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CODEC_OPERATION_ENCODE_H264_BIT_KHR"; is_first = false;
    }
    if(object & 131072) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CODEC_OPERATION_ENCODE_H265_BIT_KHR"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CODEC_OPERATION_DECODE_H264_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CODEC_OPERATION_DECODE_H265_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CODEC_OPERATION_DECODE_AV1_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoChromaSubsamplingFlagBitsKHR(VkVideoChromaSubsamplingFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CHROMA_SUBSAMPLING_INVALID_KHR"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CHROMA_SUBSAMPLING_MONOCHROME_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CHROMA_SUBSAMPLING_420_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CHROMA_SUBSAMPLING_422_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CHROMA_SUBSAMPLING_444_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoComponentBitDepthFlagBitsKHR(VkVideoComponentBitDepthFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_COMPONENT_BIT_DEPTH_INVALID_KHR"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_COMPONENT_BIT_DEPTH_8_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_COMPONENT_BIT_DEPTH_10_BIT_KHR"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_COMPONENT_BIT_DEPTH_12_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoCapabilityFlagBitsKHR(VkVideoCapabilityFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CAPABILITY_PROTECTED_CONTENT_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CAPABILITY_SEPARATE_REFERENCE_IMAGES_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoSessionCreateFlagBitsKHR(VkVideoSessionCreateFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_SESSION_CREATE_PROTECTED_CONTENT_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_SESSION_CREATE_ALLOW_ENCODE_PARAMETER_OPTIMIZATIONS_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_SESSION_CREATE_INLINE_QUERIES_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_SESSION_CREATE_RESERVED_3_BIT_KHR"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_SESSION_CREATE_RESERVED_4_BIT_KHR"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_SESSION_CREATE_RESERVED_5_BIT_KHR"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_SESSION_CREATE_RESERVED_6_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoCodingControlFlagBitsKHR(VkVideoCodingControlFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CODING_CONTROL_RESET_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CODING_CONTROL_ENCODE_RATE_CONTROL_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_CODING_CONTROL_ENCODE_QUALITY_LEVEL_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoDecodeCapabilityFlagBitsKHR(VkVideoDecodeCapabilityFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_DECODE_CAPABILITY_DPB_AND_OUTPUT_COINCIDE_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_DECODE_CAPABILITY_DPB_AND_OUTPUT_DISTINCT_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoDecodeUsageFlagBitsKHR(VkVideoDecodeUsageFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_DECODE_USAGE_DEFAULT_KHR"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_DECODE_USAGE_TRANSCODING_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_DECODE_USAGE_OFFLINE_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_DECODE_USAGE_STREAMING_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoEncodeH264CapabilityFlagBitsKHR(VkVideoEncodeH264CapabilityFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_CAPABILITY_HRD_COMPLIANCE_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_CAPABILITY_PREDICTION_WEIGHT_TABLE_GENERATED_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_CAPABILITY_ROW_UNALIGNED_SLICE_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_CAPABILITY_DIFFERENT_SLICE_TYPE_BIT_KHR"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_CAPABILITY_B_FRAME_IN_L0_LIST_BIT_KHR"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_CAPABILITY_B_FRAME_IN_L1_LIST_BIT_KHR"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_CAPABILITY_PER_PICTURE_TYPE_MIN_MAX_QP_BIT_KHR"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_CAPABILITY_PER_SLICE_CONSTANT_QP_BIT_KHR"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_CAPABILITY_GENERATE_PREFIX_NALU_BIT_KHR"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_CAPABILITY_RESERVED_9_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoEncodeH264StdFlagBitsKHR(VkVideoEncodeH264StdFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_SEPARATE_COLOR_PLANE_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_QPPRIME_Y_ZERO_TRANSFORM_BYPASS_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_SCALING_MATRIX_PRESENT_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_CHROMA_QP_INDEX_OFFSET_BIT_KHR"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_SECOND_CHROMA_QP_INDEX_OFFSET_BIT_KHR"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_PIC_INIT_QP_MINUS26_BIT_KHR"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_WEIGHTED_PRED_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_WEIGHTED_BIPRED_IDC_EXPLICIT_BIT_KHR"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_WEIGHTED_BIPRED_IDC_IMPLICIT_BIT_KHR"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_TRANSFORM_8X8_MODE_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_DIRECT_SPATIAL_MV_PRED_FLAG_UNSET_BIT_KHR"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_ENTROPY_CODING_MODE_FLAG_UNSET_BIT_KHR"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_ENTROPY_CODING_MODE_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_DIRECT_8X8_INFERENCE_FLAG_UNSET_BIT_KHR"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_CONSTRAINED_INTRA_PRED_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_DEBLOCKING_FILTER_DISABLED_BIT_KHR"; is_first = false;
    }
    if(object & 65536) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_DEBLOCKING_FILTER_ENABLED_BIT_KHR"; is_first = false;
    }
    if(object & 131072) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_DEBLOCKING_FILTER_PARTIAL_BIT_KHR"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_SLICE_QP_DELTA_BIT_KHR"; is_first = false;
    }
    if(object & 1048576) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_STD_DIFFERENT_SLICE_QP_DELTA_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoEncodeH264RateControlFlagBitsKHR(VkVideoEncodeH264RateControlFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_RATE_CONTROL_ATTEMPT_HRD_COMPLIANCE_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_RATE_CONTROL_REGULAR_GOP_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_RATE_CONTROL_REFERENCE_PATTERN_FLAT_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_RATE_CONTROL_REFERENCE_PATTERN_DYADIC_BIT_KHR"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H264_RATE_CONTROL_TEMPORAL_LAYER_PATTERN_DYADIC_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoEncodeH265CapabilityFlagBitsKHR(VkVideoEncodeH265CapabilityFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_CAPABILITY_HRD_COMPLIANCE_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_CAPABILITY_PREDICTION_WEIGHT_TABLE_GENERATED_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_CAPABILITY_ROW_UNALIGNED_SLICE_SEGMENT_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_CAPABILITY_DIFFERENT_SLICE_SEGMENT_TYPE_BIT_KHR"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_CAPABILITY_B_FRAME_IN_L0_LIST_BIT_KHR"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_CAPABILITY_B_FRAME_IN_L1_LIST_BIT_KHR"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_CAPABILITY_PER_PICTURE_TYPE_MIN_MAX_QP_BIT_KHR"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_CAPABILITY_PER_SLICE_SEGMENT_CONSTANT_QP_BIT_KHR"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_CAPABILITY_MULTIPLE_TILES_PER_SLICE_SEGMENT_BIT_KHR"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_CAPABILITY_MULTIPLE_SLICE_SEGMENTS_PER_TILE_BIT_KHR"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_CAPABILITY_RESERVED_10_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoEncodeH265StdFlagBitsKHR(VkVideoEncodeH265StdFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_SEPARATE_COLOR_PLANE_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_SAMPLE_ADAPTIVE_OFFSET_ENABLED_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_SCALING_LIST_DATA_PRESENT_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_PCM_ENABLED_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_SPS_TEMPORAL_MVP_ENABLED_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_INIT_QP_MINUS26_BIT_KHR"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_WEIGHTED_PRED_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_WEIGHTED_BIPRED_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_LOG2_PARALLEL_MERGE_LEVEL_MINUS2_BIT_KHR"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_SIGN_DATA_HIDING_ENABLED_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_TRANSFORM_SKIP_ENABLED_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_TRANSFORM_SKIP_ENABLED_FLAG_UNSET_BIT_KHR"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_PPS_SLICE_CHROMA_QP_OFFSETS_PRESENT_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_TRANSQUANT_BYPASS_ENABLED_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_CONSTRAINED_INTRA_PRED_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_ENTROPY_CODING_SYNC_ENABLED_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 65536) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_DEBLOCKING_FILTER_OVERRIDE_ENABLED_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 131072) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_DEPENDENT_SLICE_SEGMENTS_ENABLED_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 262144) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_DEPENDENT_SLICE_SEGMENT_FLAG_SET_BIT_KHR"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_SLICE_QP_DELTA_BIT_KHR"; is_first = false;
    }
    if(object & 1048576) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_STD_DIFFERENT_SLICE_QP_DELTA_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoEncodeH265CtbSizeFlagBitsKHR(VkVideoEncodeH265CtbSizeFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_CTB_SIZE_16_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_CTB_SIZE_32_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_CTB_SIZE_64_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoEncodeH265TransformBlockSizeFlagBitsKHR(VkVideoEncodeH265TransformBlockSizeFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_TRANSFORM_BLOCK_SIZE_4_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_TRANSFORM_BLOCK_SIZE_8_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_TRANSFORM_BLOCK_SIZE_16_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_TRANSFORM_BLOCK_SIZE_32_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoEncodeH265RateControlFlagBitsKHR(VkVideoEncodeH265RateControlFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_RATE_CONTROL_ATTEMPT_HRD_COMPLIANCE_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_RATE_CONTROL_REGULAR_GOP_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_RATE_CONTROL_REFERENCE_PATTERN_FLAT_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_RATE_CONTROL_REFERENCE_PATTERN_DYADIC_BIT_KHR"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_H265_RATE_CONTROL_TEMPORAL_SUB_LAYER_PATTERN_DYADIC_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoDecodeH264PictureLayoutFlagBitsKHR(VkVideoDecodeH264PictureLayoutFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_DECODE_H264_PICTURE_LAYOUT_PROGRESSIVE_KHR"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_DECODE_H264_PICTURE_LAYOUT_INTERLACED_INTERLEAVED_LINES_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_DECODE_H264_PICTURE_LAYOUT_INTERLACED_SEPARATE_PLANES_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkExternalMemoryHandleTypeFlagBitsNV(VkExternalMemoryHandleTypeFlagBitsNV object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_BIT_NV"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_KMT_BIT_NV"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D11_IMAGE_BIT_NV"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D11_IMAGE_KMT_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkExternalMemoryFeatureFlagBitsNV(VkExternalMemoryFeatureFlagBitsNV object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_FEATURE_DEDICATED_ONLY_BIT_NV"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_FEATURE_EXPORTABLE_BIT_NV"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkConditionalRenderingFlagBitsEXT(VkConditionalRenderingFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_CONDITIONAL_RENDERING_INVERTED_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkSurfaceCounterFlagBitsEXT(VkSurfaceCounterFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SURFACE_COUNTER_VBLANK_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPerformanceCounterDescriptionFlagBitsKHR(VkPerformanceCounterDescriptionFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PERFORMANCE_COUNTER_DESCRIPTION_PERFORMANCE_IMPACTING_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PERFORMANCE_COUNTER_DESCRIPTION_CONCURRENTLY_IMPACTED_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkAcquireProfilingLockFlagBitsKHR(VkAcquireProfilingLockFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkDebugUtilsMessageSeverityFlagBitsEXT(VkDebugUtilsMessageSeverityFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEBUG_UTILS_MESSAGE_SEVERITY_VERBOSE_BIT_EXT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEBUG_UTILS_MESSAGE_SEVERITY_INFO_BIT_EXT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEBUG_UTILS_MESSAGE_SEVERITY_WARNING_BIT_EXT"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEBUG_UTILS_MESSAGE_SEVERITY_ERROR_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkDebugUtilsMessageTypeFlagBitsEXT(VkDebugUtilsMessageTypeFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEBUG_UTILS_MESSAGE_TYPE_GENERAL_BIT_EXT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEBUG_UTILS_MESSAGE_TYPE_VALIDATION_BIT_EXT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEBUG_UTILS_MESSAGE_TYPE_PERFORMANCE_BIT_EXT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEBUG_UTILS_MESSAGE_TYPE_DEVICE_ADDRESS_BINDING_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkBuildAccelerationStructureFlagBitsKHR(VkBuildAccelerationStructureFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_UPDATE_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_COMPACTION_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUILD_ACCELERATION_STRUCTURE_PREFER_FAST_TRACE_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUILD_ACCELERATION_STRUCTURE_PREFER_FAST_BUILD_BIT_KHR"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUILD_ACCELERATION_STRUCTURE_LOW_MEMORY_BIT_KHR"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUILD_ACCELERATION_STRUCTURE_MOTION_BIT_NV"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_OPACITY_MICROMAP_UPDATE_EXT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_DISABLE_OPACITY_MICROMAPS_EXT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_OPACITY_MICROMAP_DATA_UPDATE_EXT"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_DISPLACEMENT_MICROMAP_UPDATE_NV"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_DATA_ACCESS_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkGeometryFlagBitsKHR(VkGeometryFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_GEOMETRY_OPAQUE_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_GEOMETRY_NO_DUPLICATE_ANY_HIT_INVOCATION_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkGeometryInstanceFlagBitsKHR(VkGeometryInstanceFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_GEOMETRY_INSTANCE_TRIANGLE_FACING_CULL_DISABLE_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_GEOMETRY_INSTANCE_TRIANGLE_FLIP_FACING_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_GEOMETRY_INSTANCE_FORCE_OPAQUE_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_GEOMETRY_INSTANCE_FORCE_NO_OPAQUE_BIT_KHR"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_GEOMETRY_INSTANCE_FORCE_OPACITY_MICROMAP_2_STATE_EXT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_GEOMETRY_INSTANCE_DISABLE_OPACITY_MICROMAPS_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkAccelerationStructureCreateFlagBitsKHR(VkAccelerationStructureCreateFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCELERATION_STRUCTURE_CREATE_DEVICE_ADDRESS_CAPTURE_REPLAY_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCELERATION_STRUCTURE_CREATE_DESCRIPTOR_BUFFER_CAPTURE_REPLAY_BIT_EXT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_ACCELERATION_STRUCTURE_CREATE_MOTION_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPipelineCompilerControlFlagBitsAMD(VkPipelineCompilerControlFlagBitsAMD object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkShaderCorePropertiesFlagBitsAMD(VkShaderCorePropertiesFlagBitsAMD object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkHostImageCopyFlagBitsEXT(VkHostImageCopyFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_HOST_IMAGE_COPY_MEMCPY_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkMemoryUnmapFlagBitsKHR(VkMemoryUnmapFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_UNMAP_RESERVE_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPresentScalingFlagBitsEXT(VkPresentScalingFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PRESENT_SCALING_ONE_TO_ONE_BIT_EXT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PRESENT_SCALING_ASPECT_RATIO_STRETCH_BIT_EXT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PRESENT_SCALING_STRETCH_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPresentGravityFlagBitsEXT(VkPresentGravityFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PRESENT_GRAVITY_MIN_BIT_EXT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PRESENT_GRAVITY_MAX_BIT_EXT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PRESENT_GRAVITY_CENTERED_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkIndirectStateFlagBitsNV(VkIndirectStateFlagBitsNV object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_INDIRECT_STATE_FLAG_FRONTFACE_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkIndirectCommandsLayoutUsageFlagBitsNV(VkIndirectCommandsLayoutUsageFlagBitsNV object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_INDIRECT_COMMANDS_LAYOUT_USAGE_EXPLICIT_PREPROCESS_BIT_NV"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_INDIRECT_COMMANDS_LAYOUT_USAGE_INDEXED_SEQUENCES_BIT_NV"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_INDIRECT_COMMANDS_LAYOUT_USAGE_UNORDERED_SEQUENCES_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoEncodeFlagBitsKHR(VkVideoEncodeFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_RESERVED_0_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_RESERVED_1_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoEncodeCapabilityFlagBitsKHR(VkVideoEncodeCapabilityFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_CAPABILITY_PRECEDING_EXTERNALLY_ENCODED_BYTES_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_CAPABILITY_INSUFFICIENT_BITSTREAM_BUFFER_RANGE_DETECTION_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_CAPABILITY_RESERVED_2_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_CAPABILITY_RESERVED_3_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoEncodeRateControlModeFlagBitsKHR(VkVideoEncodeRateControlModeFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_RATE_CONTROL_MODE_DEFAULT_KHR"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_RATE_CONTROL_MODE_DISABLED_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_RATE_CONTROL_MODE_CBR_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_RATE_CONTROL_MODE_VBR_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoEncodeFeedbackFlagBitsKHR(VkVideoEncodeFeedbackFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_FEEDBACK_BITSTREAM_BUFFER_OFFSET_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_FEEDBACK_BITSTREAM_BYTES_WRITTEN_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_FEEDBACK_BITSTREAM_HAS_OVERRIDES_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoEncodeUsageFlagBitsKHR(VkVideoEncodeUsageFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_USAGE_DEFAULT_KHR"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_USAGE_TRANSCODING_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_USAGE_STREAMING_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_USAGE_RECORDING_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_USAGE_CONFERENCING_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkVideoEncodeContentFlagBitsKHR(VkVideoEncodeContentFlagBitsKHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_CONTENT_DEFAULT_KHR"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_CONTENT_CAMERA_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_CONTENT_DESKTOP_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_VIDEO_ENCODE_CONTENT_RENDERED_BIT_KHR"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkDeviceDiagnosticsConfigFlagBitsNV(VkDeviceDiagnosticsConfigFlagBitsNV object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEVICE_DIAGNOSTICS_CONFIG_ENABLE_SHADER_DEBUG_INFO_BIT_NV"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEVICE_DIAGNOSTICS_CONFIG_ENABLE_RESOURCE_TRACKING_BIT_NV"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEVICE_DIAGNOSTICS_CONFIG_ENABLE_AUTOMATIC_CHECKPOINTS_BIT_NV"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEVICE_DIAGNOSTICS_CONFIG_ENABLE_SHADER_ERROR_REPORTING_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkExportMetalObjectTypeFlagBitsEXT(VkExportMetalObjectTypeFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXPORT_METAL_OBJECT_TYPE_METAL_DEVICE_BIT_EXT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXPORT_METAL_OBJECT_TYPE_METAL_COMMAND_QUEUE_BIT_EXT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXPORT_METAL_OBJECT_TYPE_METAL_BUFFER_BIT_EXT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXPORT_METAL_OBJECT_TYPE_METAL_TEXTURE_BIT_EXT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXPORT_METAL_OBJECT_TYPE_METAL_IOSURFACE_BIT_EXT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_EXPORT_METAL_OBJECT_TYPE_METAL_SHARED_EVENT_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
#endif // VK_USE_PLATFORM_METAL_EXT
void dump_json_VkGraphicsPipelineLibraryFlagBitsEXT(VkGraphicsPipelineLibraryFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_GRAPHICS_PIPELINE_LIBRARY_VERTEX_INPUT_INTERFACE_BIT_EXT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_GRAPHICS_PIPELINE_LIBRARY_PRE_RASTERIZATION_SHADERS_BIT_EXT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_SHADER_BIT_EXT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_OUTPUT_INTERFACE_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkImageCompressionFlagBitsEXT(VkImageCompressionFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_DEFAULT_EXT"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_DEFAULT_EXT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_EXPLICIT_EXT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_DISABLED_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkImageCompressionFixedRateFlagBitsEXT(VkImageCompressionFixedRateFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_NONE_EXT"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_1BPC_BIT_EXT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_2BPC_BIT_EXT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_3BPC_BIT_EXT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_4BPC_BIT_EXT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_5BPC_BIT_EXT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_6BPC_BIT_EXT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_7BPC_BIT_EXT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_8BPC_BIT_EXT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_9BPC_BIT_EXT"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_10BPC_BIT_EXT"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_11BPC_BIT_EXT"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_12BPC_BIT_EXT"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_13BPC_BIT_EXT"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_14BPC_BIT_EXT"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_15BPC_BIT_EXT"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_16BPC_BIT_EXT"; is_first = false;
    }
    if(object & 65536) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_17BPC_BIT_EXT"; is_first = false;
    }
    if(object & 131072) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_18BPC_BIT_EXT"; is_first = false;
    }
    if(object & 262144) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_19BPC_BIT_EXT"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_20BPC_BIT_EXT"; is_first = false;
    }
    if(object & 1048576) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_21BPC_BIT_EXT"; is_first = false;
    }
    if(object & 2097152) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_22BPC_BIT_EXT"; is_first = false;
    }
    if(object & 4194304) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_23BPC_BIT_EXT"; is_first = false;
    }
    if(object & 8388608) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_COMPRESSION_FIXED_RATE_24BPC_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkDeviceAddressBindingFlagBitsEXT(VkDeviceAddressBindingFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_DEVICE_ADDRESS_BINDING_INTERNAL_OBJECT_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkImageConstraintsInfoFlagBitsFUCHSIA(VkImageConstraintsInfoFlagBitsFUCHSIA object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CONSTRAINTS_INFO_CPU_READ_RARELY_FUCHSIA"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CONSTRAINTS_INFO_CPU_READ_OFTEN_FUCHSIA"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CONSTRAINTS_INFO_CPU_WRITE_RARELY_FUCHSIA"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CONSTRAINTS_INFO_CPU_WRITE_OFTEN_FUCHSIA"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_IMAGE_CONSTRAINTS_INFO_PROTECTED_OPTIONAL_FUCHSIA"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
#endif // VK_USE_PLATFORM_FUCHSIA
void dump_json_VkFrameBoundaryFlagBitsEXT(VkFrameBoundaryFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_FRAME_BOUNDARY_FRAME_END_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkBuildMicromapFlagBitsEXT(VkBuildMicromapFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUILD_MICROMAP_PREFER_FAST_TRACE_BIT_EXT"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUILD_MICROMAP_PREFER_FAST_BUILD_BIT_EXT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUILD_MICROMAP_ALLOW_COMPACTION_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkMicromapCreateFlagBitsEXT(VkMicromapCreateFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MICROMAP_CREATE_DEVICE_ADDRESS_CAPTURE_REPLAY_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPhysicalDeviceSchedulingControlsFlagBitsARM(VkPhysicalDeviceSchedulingControlsFlagBitsARM object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PHYSICAL_DEVICE_SCHEDULING_CONTROLS_SHADER_CORE_COUNT_ARM"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkMemoryDecompressionMethodFlagBitsNV(VkMemoryDecompressionMethodFlagBitsNV object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_MEMORY_DECOMPRESSION_METHOD_GDEFLATE_1_0_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkOpticalFlowGridSizeFlagBitsNV(VkOpticalFlowGridSizeFlagBitsNV object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_GRID_SIZE_UNKNOWN_NV"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_GRID_SIZE_1X1_BIT_NV"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_GRID_SIZE_2X2_BIT_NV"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_GRID_SIZE_4X4_BIT_NV"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_GRID_SIZE_8X8_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkOpticalFlowUsageFlagBitsNV(VkOpticalFlowUsageFlagBitsNV object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object == 0) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_USAGE_UNKNOWN_NV"; is_first = false;
    }
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_USAGE_INPUT_BIT_NV"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_USAGE_OUTPUT_BIT_NV"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_USAGE_HINT_BIT_NV"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_USAGE_COST_BIT_NV"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_USAGE_GLOBAL_FLOW_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkOpticalFlowSessionCreateFlagBitsNV(VkOpticalFlowSessionCreateFlagBitsNV object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_SESSION_CREATE_ENABLE_HINT_BIT_NV"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_SESSION_CREATE_ENABLE_COST_BIT_NV"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_SESSION_CREATE_ENABLE_GLOBAL_FLOW_BIT_NV"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_SESSION_CREATE_ALLOW_REGIONS_BIT_NV"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_SESSION_CREATE_BOTH_DIRECTIONS_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkOpticalFlowExecuteFlagBitsNV(VkOpticalFlowExecuteFlagBitsNV object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_OPTICAL_FLOW_EXECUTE_DISABLE_TEMPORAL_HINTS_BIT_NV"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkPipelineCreateFlagBits2KHR(VkPipelineCreateFlagBits2KHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_DISABLE_OPTIMIZATION_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_ALLOW_DERIVATIVES_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_DERIVATIVE_BIT_KHR"; is_first = false;
    }
    if(object & 68719476736) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_RESERVED_36_BIT_KHR"; is_first = false;
    }
    if(object & 8589934592) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RESERVED_33_BIT_KHR"; is_first = false;
    }
    if(object & 17179869184) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_ENABLE_LEGACY_DITHERING_BIT_EXT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_VIEW_INDEX_FROM_DEVICE_INDEX_BIT_KHR"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_DISPATCH_BASE_BIT_KHR"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_DEFER_COMPILE_BIT_NV"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_CAPTURE_STATISTICS_BIT_KHR"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_CAPTURE_INTERNAL_REPRESENTATIONS_BIT_KHR"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_FAIL_ON_PIPELINE_COMPILE_REQUIRED_BIT_KHR"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_EARLY_RETURN_ON_FAILURE_BIT_KHR"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_LINK_TIME_OPTIMIZATION_BIT_EXT"; is_first = false;
    }
    if(object & 8388608) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RETAIN_LINK_TIME_OPTIMIZATION_INFO_BIT_EXT"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_LIBRARY_BIT_KHR"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RAY_TRACING_SKIP_TRIANGLES_BIT_KHR"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RAY_TRACING_SKIP_AABBS_BIT_KHR"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RAY_TRACING_NO_NULL_ANY_HIT_SHADERS_BIT_KHR"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RAY_TRACING_NO_NULL_CLOSEST_HIT_SHADERS_BIT_KHR"; is_first = false;
    }
    if(object & 65536) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RAY_TRACING_NO_NULL_MISS_SHADERS_BIT_KHR"; is_first = false;
    }
    if(object & 131072) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RAY_TRACING_NO_NULL_INTERSECTION_SHADERS_BIT_KHR"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RAY_TRACING_SHADER_GROUP_HANDLE_CAPTURE_REPLAY_BIT_KHR"; is_first = false;
    }
    if(object & 262144) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_INDIRECT_BINDABLE_BIT_NV"; is_first = false;
    }
    if(object & 1048576) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RAY_TRACING_ALLOW_MOTION_BIT_NV"; is_first = false;
    }
    if(object & 2097152) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RENDERING_FRAGMENT_SHADING_RATE_ATTACHMENT_BIT_KHR"; is_first = false;
    }
    if(object & 4194304) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RENDERING_FRAGMENT_DENSITY_MAP_ATTACHMENT_BIT_EXT"; is_first = false;
    }
    if(object & 16777216) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RAY_TRACING_OPACITY_MICROMAP_BIT_EXT"; is_first = false;
    }
    if(object & 33554432) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_COLOR_ATTACHMENT_FEEDBACK_LOOP_BIT_EXT"; is_first = false;
    }
    if(object & 67108864) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_DEPTH_STENCIL_ATTACHMENT_FEEDBACK_LOOP_BIT_EXT"; is_first = false;
    }
    if(object & 134217728) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_NO_PROTECTED_ACCESS_BIT_EXT"; is_first = false;
    }
    if(object & 1073741824) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_PROTECTED_ACCESS_ONLY_BIT_EXT"; is_first = false;
    }
    if(object & 268435456) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RAY_TRACING_DISPLACEMENT_MICROMAP_BIT_NV"; is_first = false;
    }
    if(object & 536870912) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_DESCRIPTOR_BUFFER_BIT_EXT"; is_first = false;
    }
    if(object & 4294967296) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RESERVED_32_BIT_KHR"; is_first = false;
    }
    if(object & 2147483648) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RESERVED_31_BIT_KHR"; is_first = false;
    }
    if(object & 274877906944) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_EXTENSION_573_BIT_EXT"; is_first = false;
    }
    if(object & 34359738368) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RESERVED_35_BIT_KHR"; is_first = false;
    }
    if(object & 137438953472) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_PIPELINE_CREATE_2_RESERVED_37_BIT_ARM"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkBufferUsageFlagBits2KHR(VkBufferUsageFlagBits2KHR object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_TRANSFER_SRC_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_TRANSFER_DST_BIT_KHR"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_UNIFORM_TEXEL_BUFFER_BIT_KHR"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_STORAGE_TEXEL_BUFFER_BIT_KHR"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_UNIFORM_BUFFER_BIT_KHR"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_STORAGE_BUFFER_BIT_KHR"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_INDEX_BUFFER_BIT_KHR"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_VERTEX_BUFFER_BIT_KHR"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_INDIRECT_BUFFER_BIT_KHR"; is_first = false;
    }
    if(object & 33554432) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_EXECUTION_GRAPH_SCRATCH_BIT_AMDX"; is_first = false;
    }
    if(object & 268435456) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_RESERVED_28_BIT_KHR"; is_first = false;
    }
    if(object & 134217728) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_RESERVED_27_BIT_QCOM"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_CONDITIONAL_RENDERING_BIT_EXT"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_SHADER_BINDING_TABLE_BIT_KHR"; is_first = false;
    }
    if(object & 2048) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_TRANSFORM_FEEDBACK_BUFFER_BIT_EXT"; is_first = false;
    }
    if(object & 4096) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_TRANSFORM_FEEDBACK_COUNTER_BUFFER_BIT_EXT"; is_first = false;
    }
    if(object & 8192) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_VIDEO_DECODE_SRC_BIT_KHR"; is_first = false;
    }
    if(object & 16384) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_VIDEO_DECODE_DST_BIT_KHR"; is_first = false;
    }
    if(object & 32768) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_VIDEO_ENCODE_DST_BIT_KHR"; is_first = false;
    }
    if(object & 65536) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_VIDEO_ENCODE_SRC_BIT_KHR"; is_first = false;
    }
    if(object & 131072) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_SHADER_DEVICE_ADDRESS_BIT_KHR"; is_first = false;
    }
    if(object & 524288) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_ACCELERATION_STRUCTURE_BUILD_INPUT_READ_ONLY_BIT_KHR"; is_first = false;
    }
    if(object & 1048576) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_ACCELERATION_STRUCTURE_STORAGE_BIT_KHR"; is_first = false;
    }
    if(object & 2097152) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_SAMPLER_DESCRIPTOR_BUFFER_BIT_EXT"; is_first = false;
    }
    if(object & 4194304) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_RESOURCE_DESCRIPTOR_BUFFER_BIT_EXT"; is_first = false;
    }
    if(object & 67108864) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_PUSH_DESCRIPTORS_DESCRIPTOR_BUFFER_BIT_EXT"; is_first = false;
    }
    if(object & 8388608) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_MICROMAP_BUILD_INPUT_READ_ONLY_BIT_EXT"; is_first = false;
    }
    if(object & 16777216) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_MICROMAP_STORAGE_BIT_EXT"; is_first = false;
    }
    if(object & 4294967296) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_BUFFER_USAGE_2_EXTENSION_573_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}
void dump_json_VkShaderCreateFlagBitsEXT(VkShaderCreateFlagBitsEXT object, const ApiDumpSettings& settings, int indents)
{
    bool is_first = true;
    settings.stream() << '"' << object;
    if(object & 1) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_CREATE_LINK_STAGE_BIT_EXT"; is_first = false;
    }
    if(object & 1024) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_CREATE_RESERVED_10_BIT_KHR"; is_first = false;
    }
    if(object & 2) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_CREATE_ALLOW_VARYING_SUBGROUP_SIZE_BIT_EXT"; is_first = false;
    }
    if(object & 4) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_CREATE_REQUIRE_FULL_SUBGROUPS_BIT_EXT"; is_first = false;
    }
    if(object & 8) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_CREATE_NO_TASK_SHADER_BIT_EXT"; is_first = false;
    }
    if(object & 16) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_CREATE_DISPATCH_BASE_BIT_EXT"; is_first = false;
    }
    if(object & 32) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_CREATE_FRAGMENT_SHADING_RATE_ATTACHMENT_BIT_EXT"; is_first = false;
    }
    if(object & 64) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_CREATE_FRAGMENT_DENSITY_MAP_ATTACHMENT_BIT_EXT"; is_first = false;
    }
    if(object & 128) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_CREATE_EXTENSION_573_BIT_EXT"; is_first = false;
    }
    if(object & 256) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_CREATE_RESERVED_8_BIT_EXT"; is_first = false;
    }
    if(object & 512) {
        settings.stream() << (is_first ? " (" : " | ") << "VK_SHADER_CREATE_RESERVED_9_BIT_EXT"; is_first = false;
    }
    if(!is_first)
        settings.stream() << ')';
    settings.stream() << "\"";
}

//=========================== Flag Implementations ==========================//

void dump_json_VkAccessFlags(VkAccessFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkAccessFlagBits((VkAccessFlagBits) object, settings, indents);
}
void dump_json_VkImageAspectFlags(VkImageAspectFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkImageAspectFlagBits((VkImageAspectFlagBits) object, settings, indents);
}
void dump_json_VkFormatFeatureFlags(VkFormatFeatureFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkFormatFeatureFlagBits((VkFormatFeatureFlagBits) object, settings, indents);
}
void dump_json_VkImageCreateFlags(VkImageCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkImageCreateFlagBits((VkImageCreateFlagBits) object, settings, indents);
}
void dump_json_VkSampleCountFlags(VkSampleCountFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkSampleCountFlagBits((VkSampleCountFlagBits) object, settings, indents);
}
void dump_json_VkImageUsageFlags(VkImageUsageFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkImageUsageFlagBits((VkImageUsageFlagBits) object, settings, indents);
}
void dump_json_VkInstanceCreateFlags(VkInstanceCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkInstanceCreateFlagBits((VkInstanceCreateFlagBits) object, settings, indents);
}
void dump_json_VkMemoryHeapFlags(VkMemoryHeapFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkMemoryHeapFlagBits((VkMemoryHeapFlagBits) object, settings, indents);
}
void dump_json_VkMemoryPropertyFlags(VkMemoryPropertyFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkMemoryPropertyFlagBits((VkMemoryPropertyFlagBits) object, settings, indents);
}
void dump_json_VkQueueFlags(VkQueueFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkQueueFlagBits((VkQueueFlagBits) object, settings, indents);
}
void dump_json_VkDeviceQueueCreateFlags(VkDeviceQueueCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkDeviceQueueCreateFlagBits((VkDeviceQueueCreateFlagBits) object, settings, indents);
}
void dump_json_VkPipelineStageFlags(VkPipelineStageFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPipelineStageFlagBits((VkPipelineStageFlagBits) object, settings, indents);
}
void dump_json_VkMemoryMapFlags(VkMemoryMapFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkMemoryMapFlagBits((VkMemoryMapFlagBits) object, settings, indents);
}
void dump_json_VkSparseMemoryBindFlags(VkSparseMemoryBindFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkSparseMemoryBindFlagBits((VkSparseMemoryBindFlagBits) object, settings, indents);
}
void dump_json_VkSparseImageFormatFlags(VkSparseImageFormatFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkSparseImageFormatFlagBits((VkSparseImageFormatFlagBits) object, settings, indents);
}
void dump_json_VkFenceCreateFlags(VkFenceCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkFenceCreateFlagBits((VkFenceCreateFlagBits) object, settings, indents);
}
void dump_json_VkEventCreateFlags(VkEventCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkEventCreateFlagBits((VkEventCreateFlagBits) object, settings, indents);
}
void dump_json_VkQueryPipelineStatisticFlags(VkQueryPipelineStatisticFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkQueryPipelineStatisticFlagBits((VkQueryPipelineStatisticFlagBits) object, settings, indents);
}
void dump_json_VkQueryResultFlags(VkQueryResultFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkQueryResultFlagBits((VkQueryResultFlagBits) object, settings, indents);
}
void dump_json_VkBufferCreateFlags(VkBufferCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkBufferCreateFlagBits((VkBufferCreateFlagBits) object, settings, indents);
}
void dump_json_VkBufferUsageFlags(VkBufferUsageFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkBufferUsageFlagBits((VkBufferUsageFlagBits) object, settings, indents);
}
void dump_json_VkImageViewCreateFlags(VkImageViewCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkImageViewCreateFlagBits((VkImageViewCreateFlagBits) object, settings, indents);
}
void dump_json_VkPipelineCacheCreateFlags(VkPipelineCacheCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPipelineCacheCreateFlagBits((VkPipelineCacheCreateFlagBits) object, settings, indents);
}
void dump_json_VkColorComponentFlags(VkColorComponentFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkColorComponentFlagBits((VkColorComponentFlagBits) object, settings, indents);
}
void dump_json_VkPipelineCreateFlags(VkPipelineCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPipelineCreateFlagBits((VkPipelineCreateFlagBits) object, settings, indents);
}
void dump_json_VkPipelineShaderStageCreateFlags(VkPipelineShaderStageCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPipelineShaderStageCreateFlagBits((VkPipelineShaderStageCreateFlagBits) object, settings, indents);
}
void dump_json_VkCullModeFlags(VkCullModeFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkCullModeFlagBits((VkCullModeFlagBits) object, settings, indents);
}
void dump_json_VkPipelineDepthStencilStateCreateFlags(VkPipelineDepthStencilStateCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPipelineDepthStencilStateCreateFlagBits((VkPipelineDepthStencilStateCreateFlagBits) object, settings, indents);
}
void dump_json_VkPipelineColorBlendStateCreateFlags(VkPipelineColorBlendStateCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPipelineColorBlendStateCreateFlagBits((VkPipelineColorBlendStateCreateFlagBits) object, settings, indents);
}
void dump_json_VkPipelineLayoutCreateFlags(VkPipelineLayoutCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPipelineLayoutCreateFlagBits((VkPipelineLayoutCreateFlagBits) object, settings, indents);
}
void dump_json_VkShaderStageFlags(VkShaderStageFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkShaderStageFlagBits((VkShaderStageFlagBits) object, settings, indents);
}
void dump_json_VkSamplerCreateFlags(VkSamplerCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkSamplerCreateFlagBits((VkSamplerCreateFlagBits) object, settings, indents);
}
void dump_json_VkDescriptorPoolCreateFlags(VkDescriptorPoolCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkDescriptorPoolCreateFlagBits((VkDescriptorPoolCreateFlagBits) object, settings, indents);
}
void dump_json_VkDescriptorSetLayoutCreateFlags(VkDescriptorSetLayoutCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkDescriptorSetLayoutCreateFlagBits((VkDescriptorSetLayoutCreateFlagBits) object, settings, indents);
}
void dump_json_VkAttachmentDescriptionFlags(VkAttachmentDescriptionFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkAttachmentDescriptionFlagBits((VkAttachmentDescriptionFlagBits) object, settings, indents);
}
void dump_json_VkDependencyFlags(VkDependencyFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkDependencyFlagBits((VkDependencyFlagBits) object, settings, indents);
}
void dump_json_VkFramebufferCreateFlags(VkFramebufferCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkFramebufferCreateFlagBits((VkFramebufferCreateFlagBits) object, settings, indents);
}
void dump_json_VkRenderPassCreateFlags(VkRenderPassCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkRenderPassCreateFlagBits((VkRenderPassCreateFlagBits) object, settings, indents);
}
void dump_json_VkSubpassDescriptionFlags(VkSubpassDescriptionFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkSubpassDescriptionFlagBits((VkSubpassDescriptionFlagBits) object, settings, indents);
}
void dump_json_VkCommandPoolCreateFlags(VkCommandPoolCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkCommandPoolCreateFlagBits((VkCommandPoolCreateFlagBits) object, settings, indents);
}
void dump_json_VkCommandPoolResetFlags(VkCommandPoolResetFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkCommandPoolResetFlagBits((VkCommandPoolResetFlagBits) object, settings, indents);
}
void dump_json_VkCommandBufferUsageFlags(VkCommandBufferUsageFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkCommandBufferUsageFlagBits((VkCommandBufferUsageFlagBits) object, settings, indents);
}
void dump_json_VkQueryControlFlags(VkQueryControlFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkQueryControlFlagBits((VkQueryControlFlagBits) object, settings, indents);
}
void dump_json_VkCommandBufferResetFlags(VkCommandBufferResetFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkCommandBufferResetFlagBits((VkCommandBufferResetFlagBits) object, settings, indents);
}
void dump_json_VkStencilFaceFlags(VkStencilFaceFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkStencilFaceFlagBits((VkStencilFaceFlagBits) object, settings, indents);
}
void dump_json_VkSubgroupFeatureFlags(VkSubgroupFeatureFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkSubgroupFeatureFlagBits((VkSubgroupFeatureFlagBits) object, settings, indents);
}
void dump_json_VkPeerMemoryFeatureFlags(VkPeerMemoryFeatureFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPeerMemoryFeatureFlagBits((VkPeerMemoryFeatureFlagBits) object, settings, indents);
}
void dump_json_VkMemoryAllocateFlags(VkMemoryAllocateFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkMemoryAllocateFlagBits((VkMemoryAllocateFlagBits) object, settings, indents);
}
void dump_json_VkExternalMemoryHandleTypeFlags(VkExternalMemoryHandleTypeFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkExternalMemoryHandleTypeFlagBits((VkExternalMemoryHandleTypeFlagBits) object, settings, indents);
}
void dump_json_VkExternalMemoryFeatureFlags(VkExternalMemoryFeatureFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkExternalMemoryFeatureFlagBits((VkExternalMemoryFeatureFlagBits) object, settings, indents);
}
void dump_json_VkExternalFenceHandleTypeFlags(VkExternalFenceHandleTypeFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkExternalFenceHandleTypeFlagBits((VkExternalFenceHandleTypeFlagBits) object, settings, indents);
}
void dump_json_VkExternalFenceFeatureFlags(VkExternalFenceFeatureFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkExternalFenceFeatureFlagBits((VkExternalFenceFeatureFlagBits) object, settings, indents);
}
void dump_json_VkFenceImportFlags(VkFenceImportFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkFenceImportFlagBits((VkFenceImportFlagBits) object, settings, indents);
}
void dump_json_VkSemaphoreImportFlags(VkSemaphoreImportFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkSemaphoreImportFlagBits((VkSemaphoreImportFlagBits) object, settings, indents);
}
void dump_json_VkExternalSemaphoreHandleTypeFlags(VkExternalSemaphoreHandleTypeFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkExternalSemaphoreHandleTypeFlagBits((VkExternalSemaphoreHandleTypeFlagBits) object, settings, indents);
}
void dump_json_VkExternalSemaphoreFeatureFlags(VkExternalSemaphoreFeatureFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkExternalSemaphoreFeatureFlagBits((VkExternalSemaphoreFeatureFlagBits) object, settings, indents);
}
void dump_json_VkResolveModeFlags(VkResolveModeFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkResolveModeFlagBits((VkResolveModeFlagBits) object, settings, indents);
}
void dump_json_VkDescriptorBindingFlags(VkDescriptorBindingFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkDescriptorBindingFlagBits((VkDescriptorBindingFlagBits) object, settings, indents);
}
void dump_json_VkSemaphoreWaitFlags(VkSemaphoreWaitFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkSemaphoreWaitFlagBits((VkSemaphoreWaitFlagBits) object, settings, indents);
}
void dump_json_VkPipelineCreationFeedbackFlags(VkPipelineCreationFeedbackFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPipelineCreationFeedbackFlagBits((VkPipelineCreationFeedbackFlagBits) object, settings, indents);
}
void dump_json_VkToolPurposeFlags(VkToolPurposeFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkToolPurposeFlagBits((VkToolPurposeFlagBits) object, settings, indents);
}
void dump_json_VkPipelineStageFlags2(VkPipelineStageFlags2 object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPipelineStageFlagBits2((VkPipelineStageFlagBits2) object, settings, indents);
}
void dump_json_VkAccessFlags2(VkAccessFlags2 object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkAccessFlagBits2((VkAccessFlagBits2) object, settings, indents);
}
void dump_json_VkSubmitFlags(VkSubmitFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkSubmitFlagBits((VkSubmitFlagBits) object, settings, indents);
}
void dump_json_VkRenderingFlags(VkRenderingFlags object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkRenderingFlagBits((VkRenderingFlagBits) object, settings, indents);
}
void dump_json_VkFormatFeatureFlags2(VkFormatFeatureFlags2 object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkFormatFeatureFlagBits2((VkFormatFeatureFlagBits2) object, settings, indents);
}
void dump_json_VkCompositeAlphaFlagsKHR(VkCompositeAlphaFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkCompositeAlphaFlagBitsKHR((VkCompositeAlphaFlagBitsKHR) object, settings, indents);
}
void dump_json_VkSurfaceTransformFlagsKHR(VkSurfaceTransformFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkSurfaceTransformFlagBitsKHR((VkSurfaceTransformFlagBitsKHR) object, settings, indents);
}
void dump_json_VkSwapchainCreateFlagsKHR(VkSwapchainCreateFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkSwapchainCreateFlagBitsKHR((VkSwapchainCreateFlagBitsKHR) object, settings, indents);
}
void dump_json_VkDeviceGroupPresentModeFlagsKHR(VkDeviceGroupPresentModeFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkDeviceGroupPresentModeFlagBitsKHR((VkDeviceGroupPresentModeFlagBitsKHR) object, settings, indents);
}
void dump_json_VkDisplayPlaneAlphaFlagsKHR(VkDisplayPlaneAlphaFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkDisplayPlaneAlphaFlagBitsKHR((VkDisplayPlaneAlphaFlagBitsKHR) object, settings, indents);
}
void dump_json_VkDebugReportFlagsEXT(VkDebugReportFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkDebugReportFlagBitsEXT((VkDebugReportFlagBitsEXT) object, settings, indents);
}
void dump_json_VkVideoCodecOperationFlagsKHR(VkVideoCodecOperationFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoCodecOperationFlagBitsKHR((VkVideoCodecOperationFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoChromaSubsamplingFlagsKHR(VkVideoChromaSubsamplingFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoChromaSubsamplingFlagBitsKHR((VkVideoChromaSubsamplingFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoComponentBitDepthFlagsKHR(VkVideoComponentBitDepthFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoComponentBitDepthFlagBitsKHR((VkVideoComponentBitDepthFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoCapabilityFlagsKHR(VkVideoCapabilityFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoCapabilityFlagBitsKHR((VkVideoCapabilityFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoSessionCreateFlagsKHR(VkVideoSessionCreateFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoSessionCreateFlagBitsKHR((VkVideoSessionCreateFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoCodingControlFlagsKHR(VkVideoCodingControlFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoCodingControlFlagBitsKHR((VkVideoCodingControlFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoDecodeCapabilityFlagsKHR(VkVideoDecodeCapabilityFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoDecodeCapabilityFlagBitsKHR((VkVideoDecodeCapabilityFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoDecodeUsageFlagsKHR(VkVideoDecodeUsageFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoDecodeUsageFlagBitsKHR((VkVideoDecodeUsageFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoEncodeH264CapabilityFlagsKHR(VkVideoEncodeH264CapabilityFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoEncodeH264CapabilityFlagBitsKHR((VkVideoEncodeH264CapabilityFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoEncodeH264StdFlagsKHR(VkVideoEncodeH264StdFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoEncodeH264StdFlagBitsKHR((VkVideoEncodeH264StdFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoEncodeH264RateControlFlagsKHR(VkVideoEncodeH264RateControlFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoEncodeH264RateControlFlagBitsKHR((VkVideoEncodeH264RateControlFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoEncodeH265CapabilityFlagsKHR(VkVideoEncodeH265CapabilityFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoEncodeH265CapabilityFlagBitsKHR((VkVideoEncodeH265CapabilityFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoEncodeH265StdFlagsKHR(VkVideoEncodeH265StdFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoEncodeH265StdFlagBitsKHR((VkVideoEncodeH265StdFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoEncodeH265CtbSizeFlagsKHR(VkVideoEncodeH265CtbSizeFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoEncodeH265CtbSizeFlagBitsKHR((VkVideoEncodeH265CtbSizeFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoEncodeH265TransformBlockSizeFlagsKHR(VkVideoEncodeH265TransformBlockSizeFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoEncodeH265TransformBlockSizeFlagBitsKHR((VkVideoEncodeH265TransformBlockSizeFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoEncodeH265RateControlFlagsKHR(VkVideoEncodeH265RateControlFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoEncodeH265RateControlFlagBitsKHR((VkVideoEncodeH265RateControlFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoDecodeH264PictureLayoutFlagsKHR(VkVideoDecodeH264PictureLayoutFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoDecodeH264PictureLayoutFlagBitsKHR((VkVideoDecodeH264PictureLayoutFlagBitsKHR) object, settings, indents);
}
void dump_json_VkExternalMemoryHandleTypeFlagsNV(VkExternalMemoryHandleTypeFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkExternalMemoryHandleTypeFlagBitsNV((VkExternalMemoryHandleTypeFlagBitsNV) object, settings, indents);
}
void dump_json_VkExternalMemoryFeatureFlagsNV(VkExternalMemoryFeatureFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkExternalMemoryFeatureFlagBitsNV((VkExternalMemoryFeatureFlagBitsNV) object, settings, indents);
}
void dump_json_VkConditionalRenderingFlagsEXT(VkConditionalRenderingFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkConditionalRenderingFlagBitsEXT((VkConditionalRenderingFlagBitsEXT) object, settings, indents);
}
void dump_json_VkSurfaceCounterFlagsEXT(VkSurfaceCounterFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkSurfaceCounterFlagBitsEXT((VkSurfaceCounterFlagBitsEXT) object, settings, indents);
}
void dump_json_VkPerformanceCounterDescriptionFlagsKHR(VkPerformanceCounterDescriptionFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPerformanceCounterDescriptionFlagBitsKHR((VkPerformanceCounterDescriptionFlagBitsKHR) object, settings, indents);
}
void dump_json_VkAcquireProfilingLockFlagsKHR(VkAcquireProfilingLockFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkAcquireProfilingLockFlagBitsKHR((VkAcquireProfilingLockFlagBitsKHR) object, settings, indents);
}
void dump_json_VkDebugUtilsMessageTypeFlagsEXT(VkDebugUtilsMessageTypeFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkDebugUtilsMessageTypeFlagBitsEXT((VkDebugUtilsMessageTypeFlagBitsEXT) object, settings, indents);
}
void dump_json_VkDebugUtilsMessageSeverityFlagsEXT(VkDebugUtilsMessageSeverityFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkDebugUtilsMessageSeverityFlagBitsEXT((VkDebugUtilsMessageSeverityFlagBitsEXT) object, settings, indents);
}
void dump_json_VkBuildAccelerationStructureFlagsKHR(VkBuildAccelerationStructureFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkBuildAccelerationStructureFlagBitsKHR((VkBuildAccelerationStructureFlagBitsKHR) object, settings, indents);
}
void dump_json_VkGeometryFlagsKHR(VkGeometryFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkGeometryFlagBitsKHR((VkGeometryFlagBitsKHR) object, settings, indents);
}
void dump_json_VkGeometryInstanceFlagsKHR(VkGeometryInstanceFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkGeometryInstanceFlagBitsKHR((VkGeometryInstanceFlagBitsKHR) object, settings, indents);
}
void dump_json_VkAccelerationStructureCreateFlagsKHR(VkAccelerationStructureCreateFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkAccelerationStructureCreateFlagBitsKHR((VkAccelerationStructureCreateFlagBitsKHR) object, settings, indents);
}
void dump_json_VkPipelineCompilerControlFlagsAMD(VkPipelineCompilerControlFlagsAMD object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPipelineCompilerControlFlagBitsAMD((VkPipelineCompilerControlFlagBitsAMD) object, settings, indents);
}
void dump_json_VkShaderCorePropertiesFlagsAMD(VkShaderCorePropertiesFlagsAMD object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkShaderCorePropertiesFlagBitsAMD((VkShaderCorePropertiesFlagBitsAMD) object, settings, indents);
}
void dump_json_VkHostImageCopyFlagsEXT(VkHostImageCopyFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkHostImageCopyFlagBitsEXT((VkHostImageCopyFlagBitsEXT) object, settings, indents);
}
void dump_json_VkMemoryUnmapFlagsKHR(VkMemoryUnmapFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkMemoryUnmapFlagBitsKHR((VkMemoryUnmapFlagBitsKHR) object, settings, indents);
}
void dump_json_VkPresentScalingFlagsEXT(VkPresentScalingFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPresentScalingFlagBitsEXT((VkPresentScalingFlagBitsEXT) object, settings, indents);
}
void dump_json_VkPresentGravityFlagsEXT(VkPresentGravityFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPresentGravityFlagBitsEXT((VkPresentGravityFlagBitsEXT) object, settings, indents);
}
void dump_json_VkIndirectStateFlagsNV(VkIndirectStateFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkIndirectStateFlagBitsNV((VkIndirectStateFlagBitsNV) object, settings, indents);
}
void dump_json_VkIndirectCommandsLayoutUsageFlagsNV(VkIndirectCommandsLayoutUsageFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkIndirectCommandsLayoutUsageFlagBitsNV((VkIndirectCommandsLayoutUsageFlagBitsNV) object, settings, indents);
}
void dump_json_VkVideoEncodeFlagsKHR(VkVideoEncodeFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoEncodeFlagBitsKHR((VkVideoEncodeFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoEncodeCapabilityFlagsKHR(VkVideoEncodeCapabilityFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoEncodeCapabilityFlagBitsKHR((VkVideoEncodeCapabilityFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoEncodeRateControlModeFlagsKHR(VkVideoEncodeRateControlModeFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoEncodeRateControlModeFlagBitsKHR((VkVideoEncodeRateControlModeFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoEncodeFeedbackFlagsKHR(VkVideoEncodeFeedbackFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoEncodeFeedbackFlagBitsKHR((VkVideoEncodeFeedbackFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoEncodeUsageFlagsKHR(VkVideoEncodeUsageFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoEncodeUsageFlagBitsKHR((VkVideoEncodeUsageFlagBitsKHR) object, settings, indents);
}
void dump_json_VkVideoEncodeContentFlagsKHR(VkVideoEncodeContentFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkVideoEncodeContentFlagBitsKHR((VkVideoEncodeContentFlagBitsKHR) object, settings, indents);
}
void dump_json_VkDeviceDiagnosticsConfigFlagsNV(VkDeviceDiagnosticsConfigFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkDeviceDiagnosticsConfigFlagBitsNV((VkDeviceDiagnosticsConfigFlagBitsNV) object, settings, indents);
}
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkExportMetalObjectTypeFlagsEXT(VkExportMetalObjectTypeFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkExportMetalObjectTypeFlagBitsEXT((VkExportMetalObjectTypeFlagBitsEXT) object, settings, indents);
}
#endif // VK_USE_PLATFORM_METAL_EXT
void dump_json_VkGraphicsPipelineLibraryFlagsEXT(VkGraphicsPipelineLibraryFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkGraphicsPipelineLibraryFlagBitsEXT((VkGraphicsPipelineLibraryFlagBitsEXT) object, settings, indents);
}
void dump_json_VkImageCompressionFlagsEXT(VkImageCompressionFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkImageCompressionFlagBitsEXT((VkImageCompressionFlagBitsEXT) object, settings, indents);
}
void dump_json_VkImageCompressionFixedRateFlagsEXT(VkImageCompressionFixedRateFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkImageCompressionFixedRateFlagBitsEXT((VkImageCompressionFixedRateFlagBitsEXT) object, settings, indents);
}
void dump_json_VkDeviceAddressBindingFlagsEXT(VkDeviceAddressBindingFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkDeviceAddressBindingFlagBitsEXT((VkDeviceAddressBindingFlagBitsEXT) object, settings, indents);
}
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkImageConstraintsInfoFlagsFUCHSIA(VkImageConstraintsInfoFlagsFUCHSIA object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkImageConstraintsInfoFlagBitsFUCHSIA((VkImageConstraintsInfoFlagBitsFUCHSIA) object, settings, indents);
}
#endif // VK_USE_PLATFORM_FUCHSIA
void dump_json_VkFrameBoundaryFlagsEXT(VkFrameBoundaryFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkFrameBoundaryFlagBitsEXT((VkFrameBoundaryFlagBitsEXT) object, settings, indents);
}
void dump_json_VkBuildMicromapFlagsEXT(VkBuildMicromapFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkBuildMicromapFlagBitsEXT((VkBuildMicromapFlagBitsEXT) object, settings, indents);
}
void dump_json_VkMicromapCreateFlagsEXT(VkMicromapCreateFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkMicromapCreateFlagBitsEXT((VkMicromapCreateFlagBitsEXT) object, settings, indents);
}
void dump_json_VkPhysicalDeviceSchedulingControlsFlagsARM(VkPhysicalDeviceSchedulingControlsFlagsARM object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPhysicalDeviceSchedulingControlsFlagBitsARM((VkPhysicalDeviceSchedulingControlsFlagBitsARM) object, settings, indents);
}
void dump_json_VkMemoryDecompressionMethodFlagsNV(VkMemoryDecompressionMethodFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkMemoryDecompressionMethodFlagBitsNV((VkMemoryDecompressionMethodFlagBitsNV) object, settings, indents);
}
void dump_json_VkOpticalFlowGridSizeFlagsNV(VkOpticalFlowGridSizeFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkOpticalFlowGridSizeFlagBitsNV((VkOpticalFlowGridSizeFlagBitsNV) object, settings, indents);
}
void dump_json_VkOpticalFlowUsageFlagsNV(VkOpticalFlowUsageFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkOpticalFlowUsageFlagBitsNV((VkOpticalFlowUsageFlagBitsNV) object, settings, indents);
}
void dump_json_VkOpticalFlowSessionCreateFlagsNV(VkOpticalFlowSessionCreateFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkOpticalFlowSessionCreateFlagBitsNV((VkOpticalFlowSessionCreateFlagBitsNV) object, settings, indents);
}
void dump_json_VkOpticalFlowExecuteFlagsNV(VkOpticalFlowExecuteFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkOpticalFlowExecuteFlagBitsNV((VkOpticalFlowExecuteFlagBitsNV) object, settings, indents);
}
void dump_json_VkPipelineCreateFlags2KHR(VkPipelineCreateFlags2KHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkPipelineCreateFlagBits2KHR((VkPipelineCreateFlagBits2KHR) object, settings, indents);
}
void dump_json_VkBufferUsageFlags2KHR(VkBufferUsageFlags2KHR object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkBufferUsageFlagBits2KHR((VkBufferUsageFlagBits2KHR) object, settings, indents);
}
void dump_json_VkShaderCreateFlagsEXT(VkShaderCreateFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    dump_json_VkShaderCreateFlagBitsEXT((VkShaderCreateFlagBitsEXT) object, settings, indents);
}


void dump_json_VkDeviceCreateFlags(VkDeviceCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkSemaphoreCreateFlags(VkSemaphoreCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkQueryPoolCreateFlags(VkQueryPoolCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkBufferViewCreateFlags(VkBufferViewCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkShaderModuleCreateFlags(VkShaderModuleCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkPipelineVertexInputStateCreateFlags(VkPipelineVertexInputStateCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkPipelineInputAssemblyStateCreateFlags(VkPipelineInputAssemblyStateCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkPipelineTessellationStateCreateFlags(VkPipelineTessellationStateCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkPipelineViewportStateCreateFlags(VkPipelineViewportStateCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkPipelineRasterizationStateCreateFlags(VkPipelineRasterizationStateCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkPipelineMultisampleStateCreateFlags(VkPipelineMultisampleStateCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkPipelineDynamicStateCreateFlags(VkPipelineDynamicStateCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkDescriptorPoolResetFlags(VkDescriptorPoolResetFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkCommandPoolTrimFlags(VkCommandPoolTrimFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkDescriptorUpdateTemplateCreateFlags(VkDescriptorUpdateTemplateCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkPrivateDataSlotCreateFlags(VkPrivateDataSlotCreateFlags object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkDisplayModeCreateFlagsKHR(VkDisplayModeCreateFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkDisplaySurfaceCreateFlagsKHR(VkDisplaySurfaceCreateFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#if defined(VK_USE_PLATFORM_XLIB_KHR)
void dump_json_VkXlibSurfaceCreateFlagsKHR(VkXlibSurfaceCreateFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#endif // VK_USE_PLATFORM_XLIB_KHR
#if defined(VK_USE_PLATFORM_XCB_KHR)
void dump_json_VkXcbSurfaceCreateFlagsKHR(VkXcbSurfaceCreateFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#endif // VK_USE_PLATFORM_XCB_KHR
#if defined(VK_USE_PLATFORM_WAYLAND_KHR)
void dump_json_VkWaylandSurfaceCreateFlagsKHR(VkWaylandSurfaceCreateFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#endif // VK_USE_PLATFORM_WAYLAND_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_VkAndroidSurfaceCreateFlagsKHR(VkAndroidSurfaceCreateFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkWin32SurfaceCreateFlagsKHR(VkWin32SurfaceCreateFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
void dump_json_VkVideoSessionParametersCreateFlagsKHR(VkVideoSessionParametersCreateFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkVideoBeginCodingFlagsKHR(VkVideoBeginCodingFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkVideoEndCodingFlagsKHR(VkVideoEndCodingFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkVideoDecodeFlagsKHR(VkVideoDecodeFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkPipelineRasterizationStateStreamCreateFlagsEXT(VkPipelineRasterizationStateStreamCreateFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#if defined(VK_USE_PLATFORM_GGP)
void dump_json_VkStreamDescriptorSurfaceCreateFlagsGGP(VkStreamDescriptorSurfaceCreateFlagsGGP object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#endif // VK_USE_PLATFORM_GGP
#if defined(VK_USE_PLATFORM_VI_NN)
void dump_json_VkViSurfaceCreateFlagsNN(VkViSurfaceCreateFlagsNN object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#endif // VK_USE_PLATFORM_VI_NN
void dump_json_VkPipelineViewportSwizzleStateCreateFlagsNV(VkPipelineViewportSwizzleStateCreateFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkPipelineDiscardRectangleStateCreateFlagsEXT(VkPipelineDiscardRectangleStateCreateFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkPipelineRasterizationConservativeStateCreateFlagsEXT(VkPipelineRasterizationConservativeStateCreateFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkPipelineRasterizationDepthClipStateCreateFlagsEXT(VkPipelineRasterizationDepthClipStateCreateFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#if defined(VK_USE_PLATFORM_IOS_MVK)
void dump_json_VkIOSSurfaceCreateFlagsMVK(VkIOSSurfaceCreateFlagsMVK object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#endif // VK_USE_PLATFORM_IOS_MVK
#if defined(VK_USE_PLATFORM_MACOS_MVK)
void dump_json_VkMacOSSurfaceCreateFlagsMVK(VkMacOSSurfaceCreateFlagsMVK object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#endif // VK_USE_PLATFORM_MACOS_MVK
void dump_json_VkDebugUtilsMessengerCallbackDataFlagsEXT(VkDebugUtilsMessengerCallbackDataFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkDebugUtilsMessengerCreateFlagsEXT(VkDebugUtilsMessengerCreateFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkPipelineCoverageToColorStateCreateFlagsNV(VkPipelineCoverageToColorStateCreateFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkPipelineCoverageModulationStateCreateFlagsNV(VkPipelineCoverageModulationStateCreateFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkValidationCacheCreateFlagsEXT(VkValidationCacheCreateFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkImagePipeSurfaceCreateFlagsFUCHSIA(VkImagePipeSurfaceCreateFlagsFUCHSIA object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkMetalSurfaceCreateFlagsEXT(VkMetalSurfaceCreateFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#endif // VK_USE_PLATFORM_METAL_EXT
void dump_json_VkPipelineCoverageReductionStateCreateFlagsNV(VkPipelineCoverageReductionStateCreateFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkHeadlessSurfaceCreateFlagsEXT(VkHeadlessSurfaceCreateFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkDeviceMemoryReportFlagsEXT(VkDeviceMemoryReportFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkVideoEncodeRateControlFlagsKHR(VkVideoEncodeRateControlFlagsKHR object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkAccelerationStructureMotionInfoFlagsNV(VkAccelerationStructureMotionInfoFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
void dump_json_VkAccelerationStructureMotionInstanceFlagsNV(VkAccelerationStructureMotionInstanceFlagsNV object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#if defined(VK_USE_PLATFORM_DIRECTFB_EXT)
void dump_json_VkDirectFBSurfaceCreateFlagsEXT(VkDirectFBSurfaceCreateFlagsEXT object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#endif // VK_USE_PLATFORM_DIRECTFB_EXT
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkImageFormatConstraintsFlagsFUCHSIA(VkImageFormatConstraintsFlagsFUCHSIA object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
void dump_json_VkScreenSurfaceCreateFlagsQNX(VkScreenSurfaceCreateFlagsQNX object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}
#endif // VK_USE_PLATFORM_SCREEN_QNX
void dump_json_VkDirectDriverLoadingFlagsLUNARG(VkDirectDriverLoadingFlagsLUNARG object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << '"' << object << "\"";
}

//======================= Func Pointer Implementations ======================//

void dump_json_PFN_vkAllocationFunction(PFN_vkAllocationFunction object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress())
       settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
void dump_json_PFN_vkFreeFunction(PFN_vkFreeFunction object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress())
       settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
void dump_json_PFN_vkInternalAllocationNotification(PFN_vkInternalAllocationNotification object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress())
       settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
void dump_json_PFN_vkInternalFreeNotification(PFN_vkInternalFreeNotification object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress())
       settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
void dump_json_PFN_vkReallocationFunction(PFN_vkReallocationFunction object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress())
       settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
void dump_json_PFN_vkVoidFunction(PFN_vkVoidFunction object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress())
       settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
void dump_json_PFN_vkDebugReportCallbackEXT(PFN_vkDebugReportCallbackEXT object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress())
       settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
void dump_json_PFN_vkDebugUtilsMessengerCallbackEXT(PFN_vkDebugUtilsMessengerCallbackEXT object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress())
       settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
void dump_json_PFN_vkDeviceMemoryReportCallbackEXT(PFN_vkDeviceMemoryReportCallbackEXT object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress())
       settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}
void dump_json_PFN_vkGetInstanceProcAddrLUNARG(PFN_vkGetInstanceProcAddrLUNARG object, const ApiDumpSettings& settings, int indents)
{
    if(settings.showAddress())
       settings.stream() << "\"" << object << "\"";
    else
        settings.stream() << "\"address\"";
}

//========================== Struct Implementations =========================//

void dump_json_VkExtent2D(const VkExtent2D& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.width, NULL, settings, "uint32_t", "width", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.height, NULL, settings, "uint32_t", "height", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExtent3D(const VkExtent3D& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.width, NULL, settings, "uint32_t", "width", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.height, NULL, settings, "uint32_t", "height", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.depth, NULL, settings, "uint32_t", "depth", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkOffset2D(const VkOffset2D& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const int32_t>(object.x, NULL, settings, "int32_t", "x", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.y, NULL, settings, "int32_t", "y", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkOffset3D(const VkOffset3D& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const int32_t>(object.x, NULL, settings, "int32_t", "x", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.y, NULL, settings, "int32_t", "y", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.z, NULL, settings, "int32_t", "z", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRect2D(const VkRect2D& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkOffset2D>(object.offset, NULL, settings, "VkOffset2D", "offset", true, false, indents + 1, dump_json_VkOffset2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.extent, NULL, settings, "VkExtent2D", "extent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBaseInStructure(const VkBaseInStructure& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    dump_json_pointer<const VkBaseInStructure>(object.pNext, settings, "const struct VkBaseInStructure*", "pNext", true, false, indents + 1, dump_json_VkBaseInStructure);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBaseOutStructure(const VkBaseOutStructure& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    dump_json_pointer<const VkBaseOutStructure>(object.pNext, settings, "struct VkBaseOutStructure*", "pNext", true, false, indents + 1, dump_json_VkBaseOutStructure);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBufferMemoryBarrier(const VkBufferMemoryBarrier& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags>(object.srcAccessMask, NULL, settings, "VkAccessFlags", "srcAccessMask", false, false, indents + 1, dump_json_VkAccessFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags>(object.dstAccessMask, NULL, settings, "VkAccessFlags", "dstAccessMask", false, false, indents + 1, dump_json_VkAccessFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.srcQueueFamilyIndex, NULL, settings, "uint32_t", "srcQueueFamilyIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dstQueueFamilyIndex, NULL, settings, "uint32_t", "dstQueueFamilyIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.offset, NULL, settings, "VkDeviceSize", "offset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDispatchIndirectCommand(const VkDispatchIndirectCommand& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.x, NULL, settings, "uint32_t", "x", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.y, NULL, settings, "uint32_t", "y", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.z, NULL, settings, "uint32_t", "z", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDrawIndexedIndirectCommand(const VkDrawIndexedIndirectCommand& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.indexCount, NULL, settings, "uint32_t", "indexCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.instanceCount, NULL, settings, "uint32_t", "instanceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.firstIndex, NULL, settings, "uint32_t", "firstIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.vertexOffset, NULL, settings, "int32_t", "vertexOffset", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.firstInstance, NULL, settings, "uint32_t", "firstInstance", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDrawIndirectCommand(const VkDrawIndirectCommand& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.vertexCount, NULL, settings, "uint32_t", "vertexCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.instanceCount, NULL, settings, "uint32_t", "instanceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.firstVertex, NULL, settings, "uint32_t", "firstVertex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.firstInstance, NULL, settings, "uint32_t", "firstInstance", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageSubresourceRange(const VkImageSubresourceRange& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkImageAspectFlags>(object.aspectMask, NULL, settings, "VkImageAspectFlags", "aspectMask", false, false, indents + 1, dump_json_VkImageAspectFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.baseMipLevel, NULL, settings, "uint32_t", "baseMipLevel", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.levelCount, NULL, settings, "uint32_t", "levelCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.baseArrayLayer, NULL, settings, "uint32_t", "baseArrayLayer", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.layerCount, NULL, settings, "uint32_t", "layerCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageMemoryBarrier(const VkImageMemoryBarrier& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags>(object.srcAccessMask, NULL, settings, "VkAccessFlags", "srcAccessMask", false, false, indents + 1, dump_json_VkAccessFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags>(object.dstAccessMask, NULL, settings, "VkAccessFlags", "dstAccessMask", false, false, indents + 1, dump_json_VkAccessFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.oldLayout, NULL, settings, "VkImageLayout", "oldLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.newLayout, NULL, settings, "VkImageLayout", "newLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.srcQueueFamilyIndex, NULL, settings, "uint32_t", "srcQueueFamilyIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dstQueueFamilyIndex, NULL, settings, "uint32_t", "dstQueueFamilyIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.image, NULL, settings, "VkImage", "image", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceRange>(object.subresourceRange, NULL, settings, "VkImageSubresourceRange", "subresourceRange", true, false, indents + 1, dump_json_VkImageSubresourceRange);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryBarrier(const VkMemoryBarrier& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags>(object.srcAccessMask, NULL, settings, "VkAccessFlags", "srcAccessMask", false, false, indents + 1, dump_json_VkAccessFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags>(object.dstAccessMask, NULL, settings, "VkAccessFlags", "dstAccessMask", false, false, indents + 1, dump_json_VkAccessFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineCacheHeaderVersionOne(const VkPipelineCacheHeaderVersionOne& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.headerSize, NULL, settings, "uint32_t", "headerSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineCacheHeaderVersion>(object.headerVersion, NULL, settings, "VkPipelineCacheHeaderVersion", "headerVersion", false, false, indents + 1, dump_json_VkPipelineCacheHeaderVersion);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.vendorID, NULL, settings, "uint32_t", "vendorID", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceID, NULL, settings, "uint32_t", "deviceID", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.pipelineCacheUUID, 16, settings, "uint8_t[VK_UUID_SIZE]", "uint8_t", "pipelineCacheUUID", false, false, indents + 1, dump_json_uint8_t); // IQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAllocationCallbacks(const VkAllocationCallbacks& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const void*>(object.pUserData, NULL, settings, "void*", "pUserData", false, false, indents + 1, dump_json_void);
    settings.stream() << ",\n";
    dump_json_value<const PFN_vkAllocationFunction>(object.pfnAllocation, NULL, settings, "PFN_vkAllocationFunction", "pfnAllocation", false, false, indents + 1, dump_json_PFN_vkAllocationFunction);
    settings.stream() << ",\n";
    dump_json_value<const PFN_vkReallocationFunction>(object.pfnReallocation, NULL, settings, "PFN_vkReallocationFunction", "pfnReallocation", false, false, indents + 1, dump_json_PFN_vkReallocationFunction);
    settings.stream() << ",\n";
    dump_json_value<const PFN_vkFreeFunction>(object.pfnFree, NULL, settings, "PFN_vkFreeFunction", "pfnFree", false, false, indents + 1, dump_json_PFN_vkFreeFunction);
    settings.stream() << ",\n";
    dump_json_value<const PFN_vkInternalAllocationNotification>(object.pfnInternalAllocation, NULL, settings, "PFN_vkInternalAllocationNotification", "pfnInternalAllocation", false, false, indents + 1, dump_json_PFN_vkInternalAllocationNotification);
    settings.stream() << ",\n";
    dump_json_value<const PFN_vkInternalFreeNotification>(object.pfnInternalFree, NULL, settings, "PFN_vkInternalFreeNotification", "pfnInternalFree", false, false, indents + 1, dump_json_PFN_vkInternalFreeNotification);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkApplicationInfo(const VkApplicationInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.pApplicationName, NULL, settings, "const char*", "pApplicationName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.applicationVersion, NULL, settings, "uint32_t", "applicationVersion", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.pEngineName, NULL, settings, "const char*", "pEngineName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.engineVersion, NULL, settings, "uint32_t", "engineVersion", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.apiVersion, NULL, settings, "uint32_t", "apiVersion", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkFormatProperties(const VkFormatProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkFormatFeatureFlags>(object.linearTilingFeatures, NULL, settings, "VkFormatFeatureFlags", "linearTilingFeatures", false, false, indents + 1, dump_json_VkFormatFeatureFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkFormatFeatureFlags>(object.optimalTilingFeatures, NULL, settings, "VkFormatFeatureFlags", "optimalTilingFeatures", false, false, indents + 1, dump_json_VkFormatFeatureFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkFormatFeatureFlags>(object.bufferFeatures, NULL, settings, "VkFormatFeatureFlags", "bufferFeatures", false, false, indents + 1, dump_json_VkFormatFeatureFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageFormatProperties(const VkImageFormatProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkExtent3D>(object.maxExtent, NULL, settings, "VkExtent3D", "maxExtent", true, false, indents + 1, dump_json_VkExtent3D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMipLevels, NULL, settings, "uint32_t", "maxMipLevels", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxArrayLayers, NULL, settings, "uint32_t", "maxArrayLayers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlags>(object.sampleCounts, NULL, settings, "VkSampleCountFlags", "sampleCounts", false, false, indents + 1, dump_json_VkSampleCountFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.maxResourceSize, NULL, settings, "VkDeviceSize", "maxResourceSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkInstanceCreateInfo(const VkInstanceCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkInstanceCreateFlags>(object.flags, NULL, settings, "VkInstanceCreateFlags", "flags", false, false, indents + 1, dump_json_VkInstanceCreateFlags);
    settings.stream() << ",\n";
    dump_json_pointer<const VkApplicationInfo>(object.pApplicationInfo, settings, "const VkApplicationInfo*", "pApplicationInfo", true, false, indents + 1, dump_json_VkApplicationInfo);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.enabledLayerCount, NULL, settings, "uint32_t", "enabledLayerCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const char*>(object.ppEnabledLayerNames, object.enabledLayerCount, settings, "const char* const*", "const char* const", "ppEnabledLayerNames", false, false, indents + 1, dump_json_cstring); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.enabledExtensionCount, NULL, settings, "uint32_t", "enabledExtensionCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const char*>(object.ppEnabledExtensionNames, object.enabledExtensionCount, settings, "const char* const*", "const char* const", "ppEnabledExtensionNames", false, false, indents + 1, dump_json_cstring); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryHeap(const VkMemoryHeap& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkMemoryHeapFlags>(object.flags, NULL, settings, "VkMemoryHeapFlags", "flags", false, false, indents + 1, dump_json_VkMemoryHeapFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryType(const VkMemoryType& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkMemoryPropertyFlags>(object.propertyFlags, NULL, settings, "VkMemoryPropertyFlags", "propertyFlags", false, false, indents + 1, dump_json_VkMemoryPropertyFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.heapIndex, NULL, settings, "uint32_t", "heapIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFeatures(const VkPhysicalDeviceFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkBool32>(object.robustBufferAccess, NULL, settings, "VkBool32", "robustBufferAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fullDrawIndexUint32, NULL, settings, "VkBool32", "fullDrawIndexUint32", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.imageCubeArray, NULL, settings, "VkBool32", "imageCubeArray", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.independentBlend, NULL, settings, "VkBool32", "independentBlend", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.geometryShader, NULL, settings, "VkBool32", "geometryShader", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.tessellationShader, NULL, settings, "VkBool32", "tessellationShader", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sampleRateShading, NULL, settings, "VkBool32", "sampleRateShading", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.dualSrcBlend, NULL, settings, "VkBool32", "dualSrcBlend", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.logicOp, NULL, settings, "VkBool32", "logicOp", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multiDrawIndirect, NULL, settings, "VkBool32", "multiDrawIndirect", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.drawIndirectFirstInstance, NULL, settings, "VkBool32", "drawIndirectFirstInstance", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthClamp, NULL, settings, "VkBool32", "depthClamp", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthBiasClamp, NULL, settings, "VkBool32", "depthBiasClamp", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fillModeNonSolid, NULL, settings, "VkBool32", "fillModeNonSolid", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthBounds, NULL, settings, "VkBool32", "depthBounds", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.wideLines, NULL, settings, "VkBool32", "wideLines", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.largePoints, NULL, settings, "VkBool32", "largePoints", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.alphaToOne, NULL, settings, "VkBool32", "alphaToOne", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multiViewport, NULL, settings, "VkBool32", "multiViewport", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.samplerAnisotropy, NULL, settings, "VkBool32", "samplerAnisotropy", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.textureCompressionETC2, NULL, settings, "VkBool32", "textureCompressionETC2", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.textureCompressionASTC_LDR, NULL, settings, "VkBool32", "textureCompressionASTC_LDR", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.textureCompressionBC, NULL, settings, "VkBool32", "textureCompressionBC", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.occlusionQueryPrecise, NULL, settings, "VkBool32", "occlusionQueryPrecise", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.pipelineStatisticsQuery, NULL, settings, "VkBool32", "pipelineStatisticsQuery", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.vertexPipelineStoresAndAtomics, NULL, settings, "VkBool32", "vertexPipelineStoresAndAtomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentStoresAndAtomics, NULL, settings, "VkBool32", "fragmentStoresAndAtomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderTessellationAndGeometryPointSize, NULL, settings, "VkBool32", "shaderTessellationAndGeometryPointSize", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderImageGatherExtended, NULL, settings, "VkBool32", "shaderImageGatherExtended", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageImageExtendedFormats, NULL, settings, "VkBool32", "shaderStorageImageExtendedFormats", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageImageMultisample, NULL, settings, "VkBool32", "shaderStorageImageMultisample", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageImageReadWithoutFormat, NULL, settings, "VkBool32", "shaderStorageImageReadWithoutFormat", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageImageWriteWithoutFormat, NULL, settings, "VkBool32", "shaderStorageImageWriteWithoutFormat", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderUniformBufferArrayDynamicIndexing, NULL, settings, "VkBool32", "shaderUniformBufferArrayDynamicIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSampledImageArrayDynamicIndexing, NULL, settings, "VkBool32", "shaderSampledImageArrayDynamicIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageBufferArrayDynamicIndexing, NULL, settings, "VkBool32", "shaderStorageBufferArrayDynamicIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageImageArrayDynamicIndexing, NULL, settings, "VkBool32", "shaderStorageImageArrayDynamicIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderClipDistance, NULL, settings, "VkBool32", "shaderClipDistance", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderCullDistance, NULL, settings, "VkBool32", "shaderCullDistance", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderFloat64, NULL, settings, "VkBool32", "shaderFloat64", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderInt64, NULL, settings, "VkBool32", "shaderInt64", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderInt16, NULL, settings, "VkBool32", "shaderInt16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderResourceResidency, NULL, settings, "VkBool32", "shaderResourceResidency", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderResourceMinLod, NULL, settings, "VkBool32", "shaderResourceMinLod", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sparseBinding, NULL, settings, "VkBool32", "sparseBinding", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sparseResidencyBuffer, NULL, settings, "VkBool32", "sparseResidencyBuffer", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sparseResidencyImage2D, NULL, settings, "VkBool32", "sparseResidencyImage2D", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sparseResidencyImage3D, NULL, settings, "VkBool32", "sparseResidencyImage3D", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sparseResidency2Samples, NULL, settings, "VkBool32", "sparseResidency2Samples", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sparseResidency4Samples, NULL, settings, "VkBool32", "sparseResidency4Samples", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sparseResidency8Samples, NULL, settings, "VkBool32", "sparseResidency8Samples", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sparseResidency16Samples, NULL, settings, "VkBool32", "sparseResidency16Samples", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sparseResidencyAliased, NULL, settings, "VkBool32", "sparseResidencyAliased", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.variableMultisampleRate, NULL, settings, "VkBool32", "variableMultisampleRate", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.inheritedQueries, NULL, settings, "VkBool32", "inheritedQueries", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceLimits(const VkPhysicalDeviceLimits& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.maxImageDimension1D, NULL, settings, "uint32_t", "maxImageDimension1D", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxImageDimension2D, NULL, settings, "uint32_t", "maxImageDimension2D", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxImageDimension3D, NULL, settings, "uint32_t", "maxImageDimension3D", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxImageDimensionCube, NULL, settings, "uint32_t", "maxImageDimensionCube", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxImageArrayLayers, NULL, settings, "uint32_t", "maxImageArrayLayers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTexelBufferElements, NULL, settings, "uint32_t", "maxTexelBufferElements", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxUniformBufferRange, NULL, settings, "uint32_t", "maxUniformBufferRange", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxStorageBufferRange, NULL, settings, "uint32_t", "maxStorageBufferRange", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPushConstantsSize, NULL, settings, "uint32_t", "maxPushConstantsSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMemoryAllocationCount, NULL, settings, "uint32_t", "maxMemoryAllocationCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxSamplerAllocationCount, NULL, settings, "uint32_t", "maxSamplerAllocationCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.bufferImageGranularity, NULL, settings, "VkDeviceSize", "bufferImageGranularity", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.sparseAddressSpaceSize, NULL, settings, "VkDeviceSize", "sparseAddressSpaceSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxBoundDescriptorSets, NULL, settings, "uint32_t", "maxBoundDescriptorSets", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorSamplers, NULL, settings, "uint32_t", "maxPerStageDescriptorSamplers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUniformBuffers, NULL, settings, "uint32_t", "maxPerStageDescriptorUniformBuffers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorStorageBuffers, NULL, settings, "uint32_t", "maxPerStageDescriptorStorageBuffers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorSampledImages, NULL, settings, "uint32_t", "maxPerStageDescriptorSampledImages", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorStorageImages, NULL, settings, "uint32_t", "maxPerStageDescriptorStorageImages", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorInputAttachments, NULL, settings, "uint32_t", "maxPerStageDescriptorInputAttachments", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageResources, NULL, settings, "uint32_t", "maxPerStageResources", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetSamplers, NULL, settings, "uint32_t", "maxDescriptorSetSamplers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUniformBuffers, NULL, settings, "uint32_t", "maxDescriptorSetUniformBuffers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUniformBuffersDynamic, NULL, settings, "uint32_t", "maxDescriptorSetUniformBuffersDynamic", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetStorageBuffers, NULL, settings, "uint32_t", "maxDescriptorSetStorageBuffers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetStorageBuffersDynamic, NULL, settings, "uint32_t", "maxDescriptorSetStorageBuffersDynamic", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetSampledImages, NULL, settings, "uint32_t", "maxDescriptorSetSampledImages", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetStorageImages, NULL, settings, "uint32_t", "maxDescriptorSetStorageImages", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetInputAttachments, NULL, settings, "uint32_t", "maxDescriptorSetInputAttachments", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxVertexInputAttributes, NULL, settings, "uint32_t", "maxVertexInputAttributes", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxVertexInputBindings, NULL, settings, "uint32_t", "maxVertexInputBindings", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxVertexInputAttributeOffset, NULL, settings, "uint32_t", "maxVertexInputAttributeOffset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxVertexInputBindingStride, NULL, settings, "uint32_t", "maxVertexInputBindingStride", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxVertexOutputComponents, NULL, settings, "uint32_t", "maxVertexOutputComponents", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTessellationGenerationLevel, NULL, settings, "uint32_t", "maxTessellationGenerationLevel", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTessellationPatchSize, NULL, settings, "uint32_t", "maxTessellationPatchSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTessellationControlPerVertexInputComponents, NULL, settings, "uint32_t", "maxTessellationControlPerVertexInputComponents", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTessellationControlPerVertexOutputComponents, NULL, settings, "uint32_t", "maxTessellationControlPerVertexOutputComponents", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTessellationControlPerPatchOutputComponents, NULL, settings, "uint32_t", "maxTessellationControlPerPatchOutputComponents", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTessellationControlTotalOutputComponents, NULL, settings, "uint32_t", "maxTessellationControlTotalOutputComponents", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTessellationEvaluationInputComponents, NULL, settings, "uint32_t", "maxTessellationEvaluationInputComponents", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTessellationEvaluationOutputComponents, NULL, settings, "uint32_t", "maxTessellationEvaluationOutputComponents", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxGeometryShaderInvocations, NULL, settings, "uint32_t", "maxGeometryShaderInvocations", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxGeometryInputComponents, NULL, settings, "uint32_t", "maxGeometryInputComponents", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxGeometryOutputComponents, NULL, settings, "uint32_t", "maxGeometryOutputComponents", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxGeometryOutputVertices, NULL, settings, "uint32_t", "maxGeometryOutputVertices", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxGeometryTotalOutputComponents, NULL, settings, "uint32_t", "maxGeometryTotalOutputComponents", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxFragmentInputComponents, NULL, settings, "uint32_t", "maxFragmentInputComponents", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxFragmentOutputAttachments, NULL, settings, "uint32_t", "maxFragmentOutputAttachments", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxFragmentDualSrcAttachments, NULL, settings, "uint32_t", "maxFragmentDualSrcAttachments", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxFragmentCombinedOutputResources, NULL, settings, "uint32_t", "maxFragmentCombinedOutputResources", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxComputeSharedMemorySize, NULL, settings, "uint32_t", "maxComputeSharedMemorySize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.maxComputeWorkGroupCount, 3, settings, "uint32_t[3]", "uint32_t", "maxComputeWorkGroupCount", false, false, indents + 1, dump_json_uint32_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxComputeWorkGroupInvocations, NULL, settings, "uint32_t", "maxComputeWorkGroupInvocations", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.maxComputeWorkGroupSize, 3, settings, "uint32_t[3]", "uint32_t", "maxComputeWorkGroupSize", false, false, indents + 1, dump_json_uint32_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.subPixelPrecisionBits, NULL, settings, "uint32_t", "subPixelPrecisionBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.subTexelPrecisionBits, NULL, settings, "uint32_t", "subTexelPrecisionBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.mipmapPrecisionBits, NULL, settings, "uint32_t", "mipmapPrecisionBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDrawIndexedIndexValue, NULL, settings, "uint32_t", "maxDrawIndexedIndexValue", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDrawIndirectCount, NULL, settings, "uint32_t", "maxDrawIndirectCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.maxSamplerLodBias, NULL, settings, "float", "maxSamplerLodBias", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.maxSamplerAnisotropy, NULL, settings, "float", "maxSamplerAnisotropy", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxViewports, NULL, settings, "uint32_t", "maxViewports", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.maxViewportDimensions, 2, settings, "uint32_t[2]", "uint32_t", "maxViewportDimensions", false, false, indents + 1, dump_json_uint32_t); // IQA
    settings.stream() << ",\n";
    dump_json_array<const float>(object.viewportBoundsRange, 2, settings, "float[2]", "float", "viewportBoundsRange", false, false, indents + 1, dump_json_float); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.viewportSubPixelBits, NULL, settings, "uint32_t", "viewportSubPixelBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.minMemoryMapAlignment, NULL, settings, "size_t", "minMemoryMapAlignment", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.minTexelBufferOffsetAlignment, NULL, settings, "VkDeviceSize", "minTexelBufferOffsetAlignment", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.minUniformBufferOffsetAlignment, NULL, settings, "VkDeviceSize", "minUniformBufferOffsetAlignment", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.minStorageBufferOffsetAlignment, NULL, settings, "VkDeviceSize", "minStorageBufferOffsetAlignment", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.minTexelOffset, NULL, settings, "int32_t", "minTexelOffset", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTexelOffset, NULL, settings, "uint32_t", "maxTexelOffset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.minTexelGatherOffset, NULL, settings, "int32_t", "minTexelGatherOffset", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTexelGatherOffset, NULL, settings, "uint32_t", "maxTexelGatherOffset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.minInterpolationOffset, NULL, settings, "float", "minInterpolationOffset", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.maxInterpolationOffset, NULL, settings, "float", "maxInterpolationOffset", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.subPixelInterpolationOffsetBits, NULL, settings, "uint32_t", "subPixelInterpolationOffsetBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxFramebufferWidth, NULL, settings, "uint32_t", "maxFramebufferWidth", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxFramebufferHeight, NULL, settings, "uint32_t", "maxFramebufferHeight", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxFramebufferLayers, NULL, settings, "uint32_t", "maxFramebufferLayers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlags>(object.framebufferColorSampleCounts, NULL, settings, "VkSampleCountFlags", "framebufferColorSampleCounts", false, false, indents + 1, dump_json_VkSampleCountFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlags>(object.framebufferDepthSampleCounts, NULL, settings, "VkSampleCountFlags", "framebufferDepthSampleCounts", false, false, indents + 1, dump_json_VkSampleCountFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlags>(object.framebufferStencilSampleCounts, NULL, settings, "VkSampleCountFlags", "framebufferStencilSampleCounts", false, false, indents + 1, dump_json_VkSampleCountFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlags>(object.framebufferNoAttachmentsSampleCounts, NULL, settings, "VkSampleCountFlags", "framebufferNoAttachmentsSampleCounts", false, false, indents + 1, dump_json_VkSampleCountFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxColorAttachments, NULL, settings, "uint32_t", "maxColorAttachments", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlags>(object.sampledImageColorSampleCounts, NULL, settings, "VkSampleCountFlags", "sampledImageColorSampleCounts", false, false, indents + 1, dump_json_VkSampleCountFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlags>(object.sampledImageIntegerSampleCounts, NULL, settings, "VkSampleCountFlags", "sampledImageIntegerSampleCounts", false, false, indents + 1, dump_json_VkSampleCountFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlags>(object.sampledImageDepthSampleCounts, NULL, settings, "VkSampleCountFlags", "sampledImageDepthSampleCounts", false, false, indents + 1, dump_json_VkSampleCountFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlags>(object.sampledImageStencilSampleCounts, NULL, settings, "VkSampleCountFlags", "sampledImageStencilSampleCounts", false, false, indents + 1, dump_json_VkSampleCountFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlags>(object.storageImageSampleCounts, NULL, settings, "VkSampleCountFlags", "storageImageSampleCounts", false, false, indents + 1, dump_json_VkSampleCountFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxSampleMaskWords, NULL, settings, "uint32_t", "maxSampleMaskWords", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.timestampComputeAndGraphics, NULL, settings, "VkBool32", "timestampComputeAndGraphics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.timestampPeriod, NULL, settings, "float", "timestampPeriod", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxClipDistances, NULL, settings, "uint32_t", "maxClipDistances", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxCullDistances, NULL, settings, "uint32_t", "maxCullDistances", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxCombinedClipAndCullDistances, NULL, settings, "uint32_t", "maxCombinedClipAndCullDistances", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.discreteQueuePriorities, NULL, settings, "uint32_t", "discreteQueuePriorities", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const float>(object.pointSizeRange, 2, settings, "float[2]", "float", "pointSizeRange", false, false, indents + 1, dump_json_float); // IQA
    settings.stream() << ",\n";
    dump_json_array<const float>(object.lineWidthRange, 2, settings, "float[2]", "float", "lineWidthRange", false, false, indents + 1, dump_json_float); // IQA
    settings.stream() << ",\n";
    dump_json_value<const float>(object.pointSizeGranularity, NULL, settings, "float", "pointSizeGranularity", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.lineWidthGranularity, NULL, settings, "float", "lineWidthGranularity", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.strictLines, NULL, settings, "VkBool32", "strictLines", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.standardSampleLocations, NULL, settings, "VkBool32", "standardSampleLocations", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.optimalBufferCopyOffsetAlignment, NULL, settings, "VkDeviceSize", "optimalBufferCopyOffsetAlignment", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.optimalBufferCopyRowPitchAlignment, NULL, settings, "VkDeviceSize", "optimalBufferCopyRowPitchAlignment", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.nonCoherentAtomSize, NULL, settings, "VkDeviceSize", "nonCoherentAtomSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMemoryProperties(const VkPhysicalDeviceMemoryProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.memoryTypeCount, NULL, settings, "uint32_t", "memoryTypeCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkMemoryType>(object.memoryTypes, object.memoryTypeCount, settings, "VkMemoryType[VK_MAX_MEMORY_TYPES]", "VkMemoryType", "memoryTypes", true, false, indents + 1, dump_json_VkMemoryType); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryHeapCount, NULL, settings, "uint32_t", "memoryHeapCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkMemoryHeap>(object.memoryHeaps, object.memoryHeapCount, settings, "VkMemoryHeap[VK_MAX_MEMORY_HEAPS]", "VkMemoryHeap", "memoryHeaps", true, false, indents + 1, dump_json_VkMemoryHeap); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSparseProperties(const VkPhysicalDeviceSparseProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkBool32>(object.residencyStandard2DBlockShape, NULL, settings, "VkBool32", "residencyStandard2DBlockShape", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.residencyStandard2DMultisampleBlockShape, NULL, settings, "VkBool32", "residencyStandard2DMultisampleBlockShape", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.residencyStandard3DBlockShape, NULL, settings, "VkBool32", "residencyStandard3DBlockShape", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.residencyAlignedMipSize, NULL, settings, "VkBool32", "residencyAlignedMipSize", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.residencyNonResidentStrict, NULL, settings, "VkBool32", "residencyNonResidentStrict", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceProperties(const VkPhysicalDeviceProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.apiVersion, NULL, settings, "uint32_t", "apiVersion", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.driverVersion, NULL, settings, "uint32_t", "driverVersion", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.vendorID, NULL, settings, "uint32_t", "vendorID", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceID, NULL, settings, "uint32_t", "deviceID", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkPhysicalDeviceType>(object.deviceType, NULL, settings, "VkPhysicalDeviceType", "deviceType", false, false, indents + 1, dump_json_VkPhysicalDeviceType);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.deviceName, NULL, settings, "char[VK_MAX_PHYSICAL_DEVICE_NAME_SIZE]", "deviceName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.pipelineCacheUUID, 16, settings, "uint8_t[VK_UUID_SIZE]", "uint8_t", "pipelineCacheUUID", false, false, indents + 1, dump_json_uint8_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const VkPhysicalDeviceLimits>(object.limits, NULL, settings, "VkPhysicalDeviceLimits", "limits", true, false, indents + 1, dump_json_VkPhysicalDeviceLimits);
    settings.stream() << ",\n";
    dump_json_value<const VkPhysicalDeviceSparseProperties>(object.sparseProperties, NULL, settings, "VkPhysicalDeviceSparseProperties", "sparseProperties", true, false, indents + 1, dump_json_VkPhysicalDeviceSparseProperties);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkQueueFamilyProperties(const VkQueueFamilyProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkQueueFlags>(object.queueFlags, NULL, settings, "VkQueueFlags", "queueFlags", false, false, indents + 1, dump_json_VkQueueFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queueCount, NULL, settings, "uint32_t", "queueCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.timestampValidBits, NULL, settings, "uint32_t", "timestampValidBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent3D>(object.minImageTransferGranularity, NULL, settings, "VkExtent3D", "minImageTransferGranularity", true, false, indents + 1, dump_json_VkExtent3D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceQueueCreateInfo(const VkDeviceQueueCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceQueueCreateFlags>(object.flags, NULL, settings, "VkDeviceQueueCreateFlags", "flags", false, false, indents + 1, dump_json_VkDeviceQueueCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queueFamilyIndex, NULL, settings, "uint32_t", "queueFamilyIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queueCount, NULL, settings, "uint32_t", "queueCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const float>(object.pQueuePriorities, object.queueCount, settings, "const float*", "const float", "pQueuePriorities", false, false, indents + 1, dump_json_float); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceCreateInfo(const VkDeviceCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceCreateFlags>(object.flags, NULL, settings, "VkDeviceCreateFlags", "flags", false, false, indents + 1, dump_json_VkDeviceCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queueCreateInfoCount, NULL, settings, "uint32_t", "queueCreateInfoCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDeviceQueueCreateInfo>(object.pQueueCreateInfos, object.queueCreateInfoCount, settings, "const VkDeviceQueueCreateInfo*", "const VkDeviceQueueCreateInfo", "pQueueCreateInfos", true, false, indents + 1, dump_json_VkDeviceQueueCreateInfo); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.enabledLayerCount, NULL, settings, "uint32_t", "enabledLayerCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const char*>(object.ppEnabledLayerNames, object.enabledLayerCount, settings, "const char* const*", "const char* const", "ppEnabledLayerNames", false, false, indents + 1, dump_json_cstring); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.enabledExtensionCount, NULL, settings, "uint32_t", "enabledExtensionCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const char*>(object.ppEnabledExtensionNames, object.enabledExtensionCount, settings, "const char* const*", "const char* const", "ppEnabledExtensionNames", false, false, indents + 1, dump_json_cstring); // JQA
    settings.stream() << ",\n";
    dump_json_pointer<const VkPhysicalDeviceFeatures>(object.pEnabledFeatures, settings, "const VkPhysicalDeviceFeatures*", "pEnabledFeatures", true, false, indents + 1, dump_json_VkPhysicalDeviceFeatures);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExtensionProperties(const VkExtensionProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const char*>(object.extensionName, NULL, settings, "char[VK_MAX_EXTENSION_NAME_SIZE]", "extensionName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.specVersion, NULL, settings, "uint32_t", "specVersion", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkLayerProperties(const VkLayerProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const char*>(object.layerName, NULL, settings, "char[VK_MAX_EXTENSION_NAME_SIZE]", "layerName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.specVersion, NULL, settings, "uint32_t", "specVersion", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.implementationVersion, NULL, settings, "uint32_t", "implementationVersion", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.description, NULL, settings, "char[VK_MAX_DESCRIPTION_SIZE]", "description", false, false, indents + 1, dump_json_cstring);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSubmitInfo(const VkSubmitInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.waitSemaphoreCount, NULL, settings, "uint32_t", "waitSemaphoreCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSemaphore>(object.pWaitSemaphores, object.waitSemaphoreCount, settings, "const VkSemaphore*", "const VkSemaphore", "pWaitSemaphores", false, false, indents + 1, dump_json_VkSemaphore); // JQA
    settings.stream() << ",\n";
    dump_json_array<const VkPipelineStageFlags>(object.pWaitDstStageMask, object.waitSemaphoreCount, settings, "const VkPipelineStageFlags*", "const VkPipelineStageFlags", "pWaitDstStageMask", false, false, indents + 1, dump_json_VkPipelineStageFlags); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.commandBufferCount, NULL, settings, "uint32_t", "commandBufferCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkCommandBuffer>(object.pCommandBuffers, object.commandBufferCount, settings, "const VkCommandBuffer*", "const VkCommandBuffer", "pCommandBuffers", false, false, indents + 1, dump_json_VkCommandBuffer); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.signalSemaphoreCount, NULL, settings, "uint32_t", "signalSemaphoreCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSemaphore>(object.pSignalSemaphores, object.signalSemaphoreCount, settings, "const VkSemaphore*", "const VkSemaphore", "pSignalSemaphores", false, false, indents + 1, dump_json_VkSemaphore); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMappedMemoryRange(const VkMappedMemoryRange& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.offset, NULL, settings, "VkDeviceSize", "offset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryAllocateInfo(const VkMemoryAllocateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.allocationSize, NULL, settings, "VkDeviceSize", "allocationSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryTypeIndex, NULL, settings, "uint32_t", "memoryTypeIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryRequirements(const VkMemoryRequirements& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.alignment, NULL, settings, "VkDeviceSize", "alignment", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryTypeBits, NULL, settings, "uint32_t", "memoryTypeBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSparseMemoryBind(const VkSparseMemoryBind& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceSize>(object.resourceOffset, NULL, settings, "VkDeviceSize", "resourceOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.memoryOffset, NULL, settings, "VkDeviceSize", "memoryOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkSparseMemoryBindFlags>(object.flags, NULL, settings, "VkSparseMemoryBindFlags", "flags", false, false, indents + 1, dump_json_VkSparseMemoryBindFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSparseBufferMemoryBindInfo(const VkSparseBufferMemoryBindInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.bindCount, NULL, settings, "uint32_t", "bindCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSparseMemoryBind>(object.pBinds, object.bindCount, settings, "const VkSparseMemoryBind*", "const VkSparseMemoryBind", "pBinds", true, false, indents + 1, dump_json_VkSparseMemoryBind); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSparseImageOpaqueMemoryBindInfo(const VkSparseImageOpaqueMemoryBindInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkImage>(object.image, NULL, settings, "VkImage", "image", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.bindCount, NULL, settings, "uint32_t", "bindCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSparseMemoryBind>(object.pBinds, object.bindCount, settings, "const VkSparseMemoryBind*", "const VkSparseMemoryBind", "pBinds", true, false, indents + 1, dump_json_VkSparseMemoryBind); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageSubresource(const VkImageSubresource& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkImageAspectFlags>(object.aspectMask, NULL, settings, "VkImageAspectFlags", "aspectMask", false, false, indents + 1, dump_json_VkImageAspectFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.mipLevel, NULL, settings, "uint32_t", "mipLevel", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.arrayLayer, NULL, settings, "uint32_t", "arrayLayer", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSparseImageMemoryBind(const VkSparseImageMemoryBind& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkImageSubresource>(object.subresource, NULL, settings, "VkImageSubresource", "subresource", true, false, indents + 1, dump_json_VkImageSubresource);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset3D>(object.offset, NULL, settings, "VkOffset3D", "offset", true, false, indents + 1, dump_json_VkOffset3D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent3D>(object.extent, NULL, settings, "VkExtent3D", "extent", true, false, indents + 1, dump_json_VkExtent3D);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.memoryOffset, NULL, settings, "VkDeviceSize", "memoryOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkSparseMemoryBindFlags>(object.flags, NULL, settings, "VkSparseMemoryBindFlags", "flags", false, false, indents + 1, dump_json_VkSparseMemoryBindFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSparseImageMemoryBindInfo(const VkSparseImageMemoryBindInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkImage>(object.image, NULL, settings, "VkImage", "image", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.bindCount, NULL, settings, "uint32_t", "bindCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSparseImageMemoryBind>(object.pBinds, object.bindCount, settings, "const VkSparseImageMemoryBind*", "const VkSparseImageMemoryBind", "pBinds", true, false, indents + 1, dump_json_VkSparseImageMemoryBind); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindSparseInfo(const VkBindSparseInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.waitSemaphoreCount, NULL, settings, "uint32_t", "waitSemaphoreCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSemaphore>(object.pWaitSemaphores, object.waitSemaphoreCount, settings, "const VkSemaphore*", "const VkSemaphore", "pWaitSemaphores", false, false, indents + 1, dump_json_VkSemaphore); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.bufferBindCount, NULL, settings, "uint32_t", "bufferBindCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSparseBufferMemoryBindInfo>(object.pBufferBinds, object.bufferBindCount, settings, "const VkSparseBufferMemoryBindInfo*", "const VkSparseBufferMemoryBindInfo", "pBufferBinds", true, false, indents + 1, dump_json_VkSparseBufferMemoryBindInfo); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.imageOpaqueBindCount, NULL, settings, "uint32_t", "imageOpaqueBindCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSparseImageOpaqueMemoryBindInfo>(object.pImageOpaqueBinds, object.imageOpaqueBindCount, settings, "const VkSparseImageOpaqueMemoryBindInfo*", "const VkSparseImageOpaqueMemoryBindInfo", "pImageOpaqueBinds", true, false, indents + 1, dump_json_VkSparseImageOpaqueMemoryBindInfo); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.imageBindCount, NULL, settings, "uint32_t", "imageBindCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSparseImageMemoryBindInfo>(object.pImageBinds, object.imageBindCount, settings, "const VkSparseImageMemoryBindInfo*", "const VkSparseImageMemoryBindInfo", "pImageBinds", true, false, indents + 1, dump_json_VkSparseImageMemoryBindInfo); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.signalSemaphoreCount, NULL, settings, "uint32_t", "signalSemaphoreCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSemaphore>(object.pSignalSemaphores, object.signalSemaphoreCount, settings, "const VkSemaphore*", "const VkSemaphore", "pSignalSemaphores", false, false, indents + 1, dump_json_VkSemaphore); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSparseImageFormatProperties(const VkSparseImageFormatProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkImageAspectFlags>(object.aspectMask, NULL, settings, "VkImageAspectFlags", "aspectMask", false, false, indents + 1, dump_json_VkImageAspectFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent3D>(object.imageGranularity, NULL, settings, "VkExtent3D", "imageGranularity", true, false, indents + 1, dump_json_VkExtent3D);
    settings.stream() << ",\n";
    dump_json_value<const VkSparseImageFormatFlags>(object.flags, NULL, settings, "VkSparseImageFormatFlags", "flags", false, false, indents + 1, dump_json_VkSparseImageFormatFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSparseImageMemoryRequirements(const VkSparseImageMemoryRequirements& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkSparseImageFormatProperties>(object.formatProperties, NULL, settings, "VkSparseImageFormatProperties", "formatProperties", true, false, indents + 1, dump_json_VkSparseImageFormatProperties);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.imageMipTailFirstLod, NULL, settings, "uint32_t", "imageMipTailFirstLod", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.imageMipTailSize, NULL, settings, "VkDeviceSize", "imageMipTailSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.imageMipTailOffset, NULL, settings, "VkDeviceSize", "imageMipTailOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.imageMipTailStride, NULL, settings, "VkDeviceSize", "imageMipTailStride", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkFenceCreateInfo(const VkFenceCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFenceCreateFlags>(object.flags, NULL, settings, "VkFenceCreateFlags", "flags", false, false, indents + 1, dump_json_VkFenceCreateFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSemaphoreCreateInfo(const VkSemaphoreCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphoreCreateFlags>(object.flags, NULL, settings, "VkSemaphoreCreateFlags", "flags", false, false, indents + 1, dump_json_VkSemaphoreCreateFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkEventCreateInfo(const VkEventCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkEventCreateFlags>(object.flags, NULL, settings, "VkEventCreateFlags", "flags", false, false, indents + 1, dump_json_VkEventCreateFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkQueryPoolCreateInfo(const VkQueryPoolCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkQueryPoolCreateFlags>(object.flags, NULL, settings, "VkQueryPoolCreateFlags", "flags", false, false, indents + 1, dump_json_VkQueryPoolCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkQueryType>(object.queryType, NULL, settings, "VkQueryType", "queryType", false, false, indents + 1, dump_json_VkQueryType);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queryCount, NULL, settings, "uint32_t", "queryCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkQueryPipelineStatisticFlags>(object.pipelineStatistics, NULL, settings, "VkQueryPipelineStatisticFlags", "pipelineStatistics", false, false, indents + 1, dump_json_VkQueryPipelineStatisticFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBufferCreateInfo(const VkBufferCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBufferCreateFlags>(object.flags, NULL, settings, "VkBufferCreateFlags", "flags", false, false, indents + 1, dump_json_VkBufferCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkBufferUsageFlags>(object.usage, NULL, settings, "VkBufferUsageFlags", "usage", false, false, indents + 1, dump_json_VkBufferUsageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSharingMode>(object.sharingMode, NULL, settings, "VkSharingMode", "sharingMode", false, false, indents + 1, dump_json_VkSharingMode);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queueFamilyIndexCount, NULL, settings, "uint32_t", "queueFamilyIndexCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    if(object.sharingMode == VK_SHARING_MODE_CONCURRENT)
    dump_json_array<const uint32_t>(object.pQueueFamilyIndices, object.queueFamilyIndexCount, settings, "const uint32_t*", "const uint32_t", "pQueueFamilyIndices", false, false, indents + 1, dump_json_uint32_t); // JQA
    else
        dump_json_UNUSED(settings, "const uint32_t*", "pQueueFamilyIndices", indents + 1);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBufferViewCreateInfo(const VkBufferViewCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBufferViewCreateFlags>(object.flags, NULL, settings, "VkBufferViewCreateFlags", "flags", false, false, indents + 1, dump_json_VkBufferViewCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.offset, NULL, settings, "VkDeviceSize", "offset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.range, NULL, settings, "VkDeviceSize", "range", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageCreateInfo(const VkImageCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageCreateFlags>(object.flags, NULL, settings, "VkImageCreateFlags", "flags", false, false, indents + 1, dump_json_VkImageCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkImageType>(object.imageType, NULL, settings, "VkImageType", "imageType", false, false, indents + 1, dump_json_VkImageType);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent3D>(object.extent, NULL, settings, "VkExtent3D", "extent", true, false, indents + 1, dump_json_VkExtent3D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.mipLevels, NULL, settings, "uint32_t", "mipLevels", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.arrayLayers, NULL, settings, "uint32_t", "arrayLayers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlagBits>(object.samples, NULL, settings, "VkSampleCountFlagBits", "samples", false, false, indents + 1, dump_json_VkSampleCountFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const VkImageTiling>(object.tiling, NULL, settings, "VkImageTiling", "tiling", false, false, indents + 1, dump_json_VkImageTiling);
    settings.stream() << ",\n";
    dump_json_value<const VkImageUsageFlags>(object.usage, NULL, settings, "VkImageUsageFlags", "usage", false, false, indents + 1, dump_json_VkImageUsageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSharingMode>(object.sharingMode, NULL, settings, "VkSharingMode", "sharingMode", false, false, indents + 1, dump_json_VkSharingMode);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queueFamilyIndexCount, NULL, settings, "uint32_t", "queueFamilyIndexCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    if(object.sharingMode == VK_SHARING_MODE_CONCURRENT)
    dump_json_array<const uint32_t>(object.pQueueFamilyIndices, object.queueFamilyIndexCount, settings, "const uint32_t*", "const uint32_t", "pQueueFamilyIndices", false, false, indents + 1, dump_json_uint32_t); // JQA
    else
        dump_json_UNUSED(settings, "const uint32_t*", "pQueueFamilyIndices", indents + 1);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.initialLayout, NULL, settings, "VkImageLayout", "initialLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSubresourceLayout(const VkSubresourceLayout& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceSize>(object.offset, NULL, settings, "VkDeviceSize", "offset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.rowPitch, NULL, settings, "VkDeviceSize", "rowPitch", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.arrayPitch, NULL, settings, "VkDeviceSize", "arrayPitch", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.depthPitch, NULL, settings, "VkDeviceSize", "depthPitch", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkComponentMapping(const VkComponentMapping& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkComponentSwizzle>(object.r, NULL, settings, "VkComponentSwizzle", "r", false, false, indents + 1, dump_json_VkComponentSwizzle);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentSwizzle>(object.g, NULL, settings, "VkComponentSwizzle", "g", false, false, indents + 1, dump_json_VkComponentSwizzle);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentSwizzle>(object.b, NULL, settings, "VkComponentSwizzle", "b", false, false, indents + 1, dump_json_VkComponentSwizzle);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentSwizzle>(object.a, NULL, settings, "VkComponentSwizzle", "a", false, false, indents + 1, dump_json_VkComponentSwizzle);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageViewCreateInfo(const VkImageViewCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageViewCreateFlags>(object.flags, NULL, settings, "VkImageViewCreateFlags", "flags", false, false, indents + 1, dump_json_VkImageViewCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.image, NULL, settings, "VkImage", "image", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageViewType>(object.viewType, NULL, settings, "VkImageViewType", "viewType", false, false, indents + 1, dump_json_VkImageViewType);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentMapping>(object.components, NULL, settings, "VkComponentMapping", "components", true, false, indents + 1, dump_json_VkComponentMapping);
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceRange>(object.subresourceRange, NULL, settings, "VkImageSubresourceRange", "subresourceRange", true, false, indents + 1, dump_json_VkImageSubresourceRange);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkShaderModuleCreateInfo(const VkShaderModuleCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkShaderModuleCreateFlags>(object.flags, NULL, settings, "VkShaderModuleCreateFlags", "flags", false, false, indents + 1, dump_json_VkShaderModuleCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.codeSize, NULL, settings, "size_t", "codeSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    if(settings.showShader())
        dump_json_array<const uint32_t>(object.pCode, object.codeSize / 4, settings, "const uint32_t*", "const uint32_t", "pCode", false, false, indents + 1, dump_json_uint32_t); // KQA
    else
        dump_json_special("SHADER DATA", settings, "const uint32_t*", "pCode", indents + 1);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineCacheCreateInfo(const VkPipelineCacheCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineCacheCreateFlags>(object.flags, NULL, settings, "VkPipelineCacheCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineCacheCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.initialDataSize, NULL, settings, "size_t", "initialDataSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pInitialData, NULL, settings, "const void*", "pInitialData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSpecializationMapEntry(const VkSpecializationMapEntry& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.constantID, NULL, settings, "uint32_t", "constantID", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.offset, NULL, settings, "uint32_t", "offset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.size, NULL, settings, "size_t", "size", false, false, indents + 1, dump_json_size_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSpecializationInfo(const VkSpecializationInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.mapEntryCount, NULL, settings, "uint32_t", "mapEntryCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSpecializationMapEntry>(object.pMapEntries, object.mapEntryCount, settings, "const VkSpecializationMapEntry*", "const VkSpecializationMapEntry", "pMapEntries", true, false, indents + 1, dump_json_VkSpecializationMapEntry); // JQA
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.dataSize, NULL, settings, "size_t", "dataSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pData, NULL, settings, "const void*", "pData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineShaderStageCreateInfo(const VkPipelineShaderStageCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineShaderStageCreateFlags>(object.flags, NULL, settings, "VkPipelineShaderStageCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineShaderStageCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlagBits>(object.stage, NULL, settings, "VkShaderStageFlagBits", "stage", false, false, indents + 1, dump_json_VkShaderStageFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderModule>(object.module, NULL, settings, "VkShaderModule", "module", false, false, indents + 1, dump_json_VkShaderModule);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.pName, NULL, settings, "const char*", "pName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_pointer<const VkSpecializationInfo>(object.pSpecializationInfo, settings, "const VkSpecializationInfo*", "pSpecializationInfo", true, false, indents + 1, dump_json_VkSpecializationInfo);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkComputePipelineCreateInfo(const VkComputePipelineCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineCreateFlags>(object.flags, NULL, settings, "VkPipelineCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineShaderStageCreateInfo>(object.stage, NULL, settings, "VkPipelineShaderStageCreateInfo", "stage", true, false, indents + 1, dump_json_VkPipelineShaderStageCreateInfo);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineLayout>(object.layout, NULL, settings, "VkPipelineLayout", "layout", false, false, indents + 1, dump_json_VkPipelineLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkPipeline>(object.basePipelineHandle, NULL, settings, "VkPipeline", "basePipelineHandle", false, false, indents + 1, dump_json_VkPipeline);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.basePipelineIndex, NULL, settings, "int32_t", "basePipelineIndex", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVertexInputBindingDescription(const VkVertexInputBindingDescription& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.binding, NULL, settings, "uint32_t", "binding", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stride, NULL, settings, "uint32_t", "stride", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkVertexInputRate>(object.inputRate, NULL, settings, "VkVertexInputRate", "inputRate", false, false, indents + 1, dump_json_VkVertexInputRate);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVertexInputAttributeDescription(const VkVertexInputAttributeDescription& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.location, NULL, settings, "uint32_t", "location", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.binding, NULL, settings, "uint32_t", "binding", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.offset, NULL, settings, "uint32_t", "offset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineVertexInputStateCreateInfo(const VkPipelineVertexInputStateCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineVertexInputStateCreateFlags>(object.flags, NULL, settings, "VkPipelineVertexInputStateCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineVertexInputStateCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.vertexBindingDescriptionCount, NULL, settings, "uint32_t", "vertexBindingDescriptionCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkVertexInputBindingDescription>(object.pVertexBindingDescriptions, object.vertexBindingDescriptionCount, settings, "const VkVertexInputBindingDescription*", "const VkVertexInputBindingDescription", "pVertexBindingDescriptions", true, false, indents + 1, dump_json_VkVertexInputBindingDescription); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.vertexAttributeDescriptionCount, NULL, settings, "uint32_t", "vertexAttributeDescriptionCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkVertexInputAttributeDescription>(object.pVertexAttributeDescriptions, object.vertexAttributeDescriptionCount, settings, "const VkVertexInputAttributeDescription*", "const VkVertexInputAttributeDescription", "pVertexAttributeDescriptions", true, false, indents + 1, dump_json_VkVertexInputAttributeDescription); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineInputAssemblyStateCreateInfo(const VkPipelineInputAssemblyStateCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineInputAssemblyStateCreateFlags>(object.flags, NULL, settings, "VkPipelineInputAssemblyStateCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineInputAssemblyStateCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkPrimitiveTopology>(object.topology, NULL, settings, "VkPrimitiveTopology", "topology", false, false, indents + 1, dump_json_VkPrimitiveTopology);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.primitiveRestartEnable, NULL, settings, "VkBool32", "primitiveRestartEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineTessellationStateCreateInfo(const VkPipelineTessellationStateCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineTessellationStateCreateFlags>(object.flags, NULL, settings, "VkPipelineTessellationStateCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineTessellationStateCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.patchControlPoints, NULL, settings, "uint32_t", "patchControlPoints", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkViewport(const VkViewport& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const float>(object.x, NULL, settings, "float", "x", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.y, NULL, settings, "float", "y", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.width, NULL, settings, "float", "width", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.height, NULL, settings, "float", "height", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.minDepth, NULL, settings, "float", "minDepth", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.maxDepth, NULL, settings, "float", "maxDepth", false, false, indents + 1, dump_json_float);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineViewportStateCreateInfo(const VkPipelineViewportStateCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineViewportStateCreateFlags>(object.flags, NULL, settings, "VkPipelineViewportStateCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineViewportStateCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.viewportCount, NULL, settings, "uint32_t", "viewportCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    if(!ApiDumpInstance::current().getIsDynamicViewport())
    dump_json_array<const VkViewport>(object.pViewports, object.viewportCount, settings, "const VkViewport*", "const VkViewport", "pViewports", true, false, indents + 1, dump_json_VkViewport); // JQA
    else
        dump_json_UNUSED(settings, "const VkViewport*", "pViewports", indents + 1);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.scissorCount, NULL, settings, "uint32_t", "scissorCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    if(!ApiDumpInstance::current().getIsDynamicScissor())
    dump_json_array<const VkRect2D>(object.pScissors, object.scissorCount, settings, "const VkRect2D*", "const VkRect2D", "pScissors", true, false, indents + 1, dump_json_VkRect2D); // JQA
    else
        dump_json_UNUSED(settings, "const VkRect2D*", "pScissors", indents + 1);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineRasterizationStateCreateInfo(const VkPipelineRasterizationStateCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineRasterizationStateCreateFlags>(object.flags, NULL, settings, "VkPipelineRasterizationStateCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineRasterizationStateCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthClampEnable, NULL, settings, "VkBool32", "depthClampEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rasterizerDiscardEnable, NULL, settings, "VkBool32", "rasterizerDiscardEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkPolygonMode>(object.polygonMode, NULL, settings, "VkPolygonMode", "polygonMode", false, false, indents + 1, dump_json_VkPolygonMode);
    settings.stream() << ",\n";
    dump_json_value<const VkCullModeFlags>(object.cullMode, NULL, settings, "VkCullModeFlags", "cullMode", false, false, indents + 1, dump_json_VkCullModeFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkFrontFace>(object.frontFace, NULL, settings, "VkFrontFace", "frontFace", false, false, indents + 1, dump_json_VkFrontFace);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthBiasEnable, NULL, settings, "VkBool32", "depthBiasEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.depthBiasConstantFactor, NULL, settings, "float", "depthBiasConstantFactor", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.depthBiasClamp, NULL, settings, "float", "depthBiasClamp", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.depthBiasSlopeFactor, NULL, settings, "float", "depthBiasSlopeFactor", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.lineWidth, NULL, settings, "float", "lineWidth", false, false, indents + 1, dump_json_float);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineMultisampleStateCreateInfo(const VkPipelineMultisampleStateCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineMultisampleStateCreateFlags>(object.flags, NULL, settings, "VkPipelineMultisampleStateCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineMultisampleStateCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlagBits>(object.rasterizationSamples, NULL, settings, "VkSampleCountFlagBits", "rasterizationSamples", false, false, indents + 1, dump_json_VkSampleCountFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sampleShadingEnable, NULL, settings, "VkBool32", "sampleShadingEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.minSampleShading, NULL, settings, "float", "minSampleShading", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_array<const VkSampleMask>(object.pSampleMask, (object.rasterizationSamples + 31) / 32, settings, "const VkSampleMask*", "const VkSampleMask", "pSampleMask", false, false, indents + 1, dump_json_VkSampleMask); // JQA
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.alphaToCoverageEnable, NULL, settings, "VkBool32", "alphaToCoverageEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.alphaToOneEnable, NULL, settings, "VkBool32", "alphaToOneEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkStencilOpState(const VkStencilOpState& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStencilOp>(object.failOp, NULL, settings, "VkStencilOp", "failOp", false, false, indents + 1, dump_json_VkStencilOp);
    settings.stream() << ",\n";
    dump_json_value<const VkStencilOp>(object.passOp, NULL, settings, "VkStencilOp", "passOp", false, false, indents + 1, dump_json_VkStencilOp);
    settings.stream() << ",\n";
    dump_json_value<const VkStencilOp>(object.depthFailOp, NULL, settings, "VkStencilOp", "depthFailOp", false, false, indents + 1, dump_json_VkStencilOp);
    settings.stream() << ",\n";
    dump_json_value<const VkCompareOp>(object.compareOp, NULL, settings, "VkCompareOp", "compareOp", false, false, indents + 1, dump_json_VkCompareOp);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.compareMask, NULL, settings, "uint32_t", "compareMask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.writeMask, NULL, settings, "uint32_t", "writeMask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.reference, NULL, settings, "uint32_t", "reference", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineDepthStencilStateCreateInfo(const VkPipelineDepthStencilStateCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineDepthStencilStateCreateFlags>(object.flags, NULL, settings, "VkPipelineDepthStencilStateCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineDepthStencilStateCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthTestEnable, NULL, settings, "VkBool32", "depthTestEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthWriteEnable, NULL, settings, "VkBool32", "depthWriteEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkCompareOp>(object.depthCompareOp, NULL, settings, "VkCompareOp", "depthCompareOp", false, false, indents + 1, dump_json_VkCompareOp);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthBoundsTestEnable, NULL, settings, "VkBool32", "depthBoundsTestEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.stencilTestEnable, NULL, settings, "VkBool32", "stencilTestEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkStencilOpState>(object.front, NULL, settings, "VkStencilOpState", "front", true, false, indents + 1, dump_json_VkStencilOpState);
    settings.stream() << ",\n";
    dump_json_value<const VkStencilOpState>(object.back, NULL, settings, "VkStencilOpState", "back", true, false, indents + 1, dump_json_VkStencilOpState);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.minDepthBounds, NULL, settings, "float", "minDepthBounds", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.maxDepthBounds, NULL, settings, "float", "maxDepthBounds", false, false, indents + 1, dump_json_float);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineColorBlendAttachmentState(const VkPipelineColorBlendAttachmentState& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkBool32>(object.blendEnable, NULL, settings, "VkBool32", "blendEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBlendFactor>(object.srcColorBlendFactor, NULL, settings, "VkBlendFactor", "srcColorBlendFactor", false, false, indents + 1, dump_json_VkBlendFactor);
    settings.stream() << ",\n";
    dump_json_value<const VkBlendFactor>(object.dstColorBlendFactor, NULL, settings, "VkBlendFactor", "dstColorBlendFactor", false, false, indents + 1, dump_json_VkBlendFactor);
    settings.stream() << ",\n";
    dump_json_value<const VkBlendOp>(object.colorBlendOp, NULL, settings, "VkBlendOp", "colorBlendOp", false, false, indents + 1, dump_json_VkBlendOp);
    settings.stream() << ",\n";
    dump_json_value<const VkBlendFactor>(object.srcAlphaBlendFactor, NULL, settings, "VkBlendFactor", "srcAlphaBlendFactor", false, false, indents + 1, dump_json_VkBlendFactor);
    settings.stream() << ",\n";
    dump_json_value<const VkBlendFactor>(object.dstAlphaBlendFactor, NULL, settings, "VkBlendFactor", "dstAlphaBlendFactor", false, false, indents + 1, dump_json_VkBlendFactor);
    settings.stream() << ",\n";
    dump_json_value<const VkBlendOp>(object.alphaBlendOp, NULL, settings, "VkBlendOp", "alphaBlendOp", false, false, indents + 1, dump_json_VkBlendOp);
    settings.stream() << ",\n";
    dump_json_value<const VkColorComponentFlags>(object.colorWriteMask, NULL, settings, "VkColorComponentFlags", "colorWriteMask", false, false, indents + 1, dump_json_VkColorComponentFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineColorBlendStateCreateInfo(const VkPipelineColorBlendStateCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineColorBlendStateCreateFlags>(object.flags, NULL, settings, "VkPipelineColorBlendStateCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineColorBlendStateCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.logicOpEnable, NULL, settings, "VkBool32", "logicOpEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkLogicOp>(object.logicOp, NULL, settings, "VkLogicOp", "logicOp", false, false, indents + 1, dump_json_VkLogicOp);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.attachmentCount, NULL, settings, "uint32_t", "attachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPipelineColorBlendAttachmentState>(object.pAttachments, object.attachmentCount, settings, "const VkPipelineColorBlendAttachmentState*", "const VkPipelineColorBlendAttachmentState", "pAttachments", true, false, indents + 1, dump_json_VkPipelineColorBlendAttachmentState); // JQA
    settings.stream() << ",\n";
    dump_json_array<const float>(object.blendConstants, 4, settings, "float[4]", "float", "blendConstants", false, false, indents + 1, dump_json_float); // IQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineDynamicStateCreateInfo(const VkPipelineDynamicStateCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineDynamicStateCreateFlags>(object.flags, NULL, settings, "VkPipelineDynamicStateCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineDynamicStateCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dynamicStateCount, NULL, settings, "uint32_t", "dynamicStateCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDynamicState>(object.pDynamicStates, object.dynamicStateCount, settings, "const VkDynamicState*", "const VkDynamicState", "pDynamicStates", false, false, indents + 1, dump_json_VkDynamicState); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkGraphicsPipelineCreateInfo(const VkGraphicsPipelineCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    ApiDumpInstance::current().setIsDynamicViewport(object.pDynamicState && std::count(object.pDynamicState->pDynamicStates, object.pDynamicState->pDynamicStates + object.pDynamicState->dynamicStateCount, VK_DYNAMIC_STATE_VIEWPORT) > 0);ApiDumpInstance::current().setIsDynamicScissor(object.pDynamicState && std::count(object.pDynamicState->pDynamicStates, object.pDynamicState->pDynamicStates + object.pDynamicState->dynamicStateCount, VK_DYNAMIC_STATE_SCISSOR));ApiDumpInstance::current().setIsGPLPreRasterOrFragmentShader(checkForGPLPreRasterOrFragmentShader(object));
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineCreateFlags>(object.flags, NULL, settings, "VkPipelineCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stageCount, NULL, settings, "uint32_t", "stageCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPipelineShaderStageCreateInfo>(object.pStages, object.stageCount, settings, "const VkPipelineShaderStageCreateInfo*", "const VkPipelineShaderStageCreateInfo", "pStages", true, false, indents + 1, dump_json_VkPipelineShaderStageCreateInfo); // JQA
    settings.stream() << ",\n";
    dump_json_pointer<const VkPipelineVertexInputStateCreateInfo>(object.pVertexInputState, settings, "const VkPipelineVertexInputStateCreateInfo*", "pVertexInputState", true, false, indents + 1, dump_json_VkPipelineVertexInputStateCreateInfo);
    settings.stream() << ",\n";
    dump_json_pointer<const VkPipelineInputAssemblyStateCreateInfo>(object.pInputAssemblyState, settings, "const VkPipelineInputAssemblyStateCreateInfo*", "pInputAssemblyState", true, false, indents + 1, dump_json_VkPipelineInputAssemblyStateCreateInfo);
    settings.stream() << ",\n";
    dump_json_pointer<const VkPipelineTessellationStateCreateInfo>(object.pTessellationState, settings, "const VkPipelineTessellationStateCreateInfo*", "pTessellationState", true, false, indents + 1, dump_json_VkPipelineTessellationStateCreateInfo);
    settings.stream() << ",\n";
    dump_json_pointer<const VkPipelineViewportStateCreateInfo>(object.pViewportState, settings, "const VkPipelineViewportStateCreateInfo*", "pViewportState", true, false, indents + 1, dump_json_VkPipelineViewportStateCreateInfo);
    settings.stream() << ",\n";
    dump_json_pointer<const VkPipelineRasterizationStateCreateInfo>(object.pRasterizationState, settings, "const VkPipelineRasterizationStateCreateInfo*", "pRasterizationState", true, false, indents + 1, dump_json_VkPipelineRasterizationStateCreateInfo);
    settings.stream() << ",\n";
    dump_json_pointer<const VkPipelineMultisampleStateCreateInfo>(object.pMultisampleState, settings, "const VkPipelineMultisampleStateCreateInfo*", "pMultisampleState", true, false, indents + 1, dump_json_VkPipelineMultisampleStateCreateInfo);
    settings.stream() << ",\n";
    dump_json_pointer<const VkPipelineDepthStencilStateCreateInfo>(object.pDepthStencilState, settings, "const VkPipelineDepthStencilStateCreateInfo*", "pDepthStencilState", true, false, indents + 1, dump_json_VkPipelineDepthStencilStateCreateInfo);
    settings.stream() << ",\n";
    dump_json_pointer<const VkPipelineColorBlendStateCreateInfo>(object.pColorBlendState, settings, "const VkPipelineColorBlendStateCreateInfo*", "pColorBlendState", true, false, indents + 1, dump_json_VkPipelineColorBlendStateCreateInfo);
    settings.stream() << ",\n";
    dump_json_pointer<const VkPipelineDynamicStateCreateInfo>(object.pDynamicState, settings, "const VkPipelineDynamicStateCreateInfo*", "pDynamicState", true, false, indents + 1, dump_json_VkPipelineDynamicStateCreateInfo);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineLayout>(object.layout, NULL, settings, "VkPipelineLayout", "layout", false, false, indents + 1, dump_json_VkPipelineLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkRenderPass>(object.renderPass, NULL, settings, "VkRenderPass", "renderPass", false, false, indents + 1, dump_json_VkRenderPass);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.subpass, NULL, settings, "uint32_t", "subpass", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkPipeline>(object.basePipelineHandle, NULL, settings, "VkPipeline", "basePipelineHandle", false, false, indents + 1, dump_json_VkPipeline);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.basePipelineIndex, NULL, settings, "int32_t", "basePipelineIndex", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPushConstantRange(const VkPushConstantRange& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkShaderStageFlags>(object.stageFlags, NULL, settings, "VkShaderStageFlags", "stageFlags", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.offset, NULL, settings, "uint32_t", "offset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.size, NULL, settings, "uint32_t", "size", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineLayoutCreateInfo(const VkPipelineLayoutCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineLayoutCreateFlags>(object.flags, NULL, settings, "VkPipelineLayoutCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineLayoutCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.setLayoutCount, NULL, settings, "uint32_t", "setLayoutCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDescriptorSetLayout>(object.pSetLayouts, object.setLayoutCount, settings, "const VkDescriptorSetLayout*", "const VkDescriptorSetLayout", "pSetLayouts", false, false, indents + 1, dump_json_VkDescriptorSetLayout); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.pushConstantRangeCount, NULL, settings, "uint32_t", "pushConstantRangeCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPushConstantRange>(object.pPushConstantRanges, object.pushConstantRangeCount, settings, "const VkPushConstantRange*", "const VkPushConstantRange", "pPushConstantRanges", true, false, indents + 1, dump_json_VkPushConstantRange); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSamplerCreateInfo(const VkSamplerCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerCreateFlags>(object.flags, NULL, settings, "VkSamplerCreateFlags", "flags", false, false, indents + 1, dump_json_VkSamplerCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkFilter>(object.magFilter, NULL, settings, "VkFilter", "magFilter", false, false, indents + 1, dump_json_VkFilter);
    settings.stream() << ",\n";
    dump_json_value<const VkFilter>(object.minFilter, NULL, settings, "VkFilter", "minFilter", false, false, indents + 1, dump_json_VkFilter);
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerMipmapMode>(object.mipmapMode, NULL, settings, "VkSamplerMipmapMode", "mipmapMode", false, false, indents + 1, dump_json_VkSamplerMipmapMode);
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerAddressMode>(object.addressModeU, NULL, settings, "VkSamplerAddressMode", "addressModeU", false, false, indents + 1, dump_json_VkSamplerAddressMode);
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerAddressMode>(object.addressModeV, NULL, settings, "VkSamplerAddressMode", "addressModeV", false, false, indents + 1, dump_json_VkSamplerAddressMode);
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerAddressMode>(object.addressModeW, NULL, settings, "VkSamplerAddressMode", "addressModeW", false, false, indents + 1, dump_json_VkSamplerAddressMode);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.mipLodBias, NULL, settings, "float", "mipLodBias", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.anisotropyEnable, NULL, settings, "VkBool32", "anisotropyEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.maxAnisotropy, NULL, settings, "float", "maxAnisotropy", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.compareEnable, NULL, settings, "VkBool32", "compareEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkCompareOp>(object.compareOp, NULL, settings, "VkCompareOp", "compareOp", false, false, indents + 1, dump_json_VkCompareOp);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.minLod, NULL, settings, "float", "minLod", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.maxLod, NULL, settings, "float", "maxLod", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const VkBorderColor>(object.borderColor, NULL, settings, "VkBorderColor", "borderColor", false, false, indents + 1, dump_json_VkBorderColor);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.unnormalizedCoordinates, NULL, settings, "VkBool32", "unnormalizedCoordinates", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyDescriptorSet(const VkCopyDescriptorSet& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorSet>(object.srcSet, NULL, settings, "VkDescriptorSet", "srcSet", false, false, indents + 1, dump_json_VkDescriptorSet);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.srcBinding, NULL, settings, "uint32_t", "srcBinding", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.srcArrayElement, NULL, settings, "uint32_t", "srcArrayElement", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorSet>(object.dstSet, NULL, settings, "VkDescriptorSet", "dstSet", false, false, indents + 1, dump_json_VkDescriptorSet);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dstBinding, NULL, settings, "uint32_t", "dstBinding", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dstArrayElement, NULL, settings, "uint32_t", "dstArrayElement", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.descriptorCount, NULL, settings, "uint32_t", "descriptorCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorBufferInfo(const VkDescriptorBufferInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.offset, NULL, settings, "VkDeviceSize", "offset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.range, NULL, settings, "VkDeviceSize", "range", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorImageInfo(const VkDescriptorImageInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkSampler>(object.sampler, NULL, settings, "VkSampler", "sampler", false, false, indents + 1, dump_json_VkSampler);
    settings.stream() << ",\n";
    dump_json_value<const VkImageView>(object.imageView, NULL, settings, "VkImageView", "imageView", false, false, indents + 1, dump_json_VkImageView);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.imageLayout, NULL, settings, "VkImageLayout", "imageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorPoolSize(const VkDescriptorPoolSize& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDescriptorType>(object.type, NULL, settings, "VkDescriptorType", "type", false, false, indents + 1, dump_json_VkDescriptorType);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.descriptorCount, NULL, settings, "uint32_t", "descriptorCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorPoolCreateInfo(const VkDescriptorPoolCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorPoolCreateFlags>(object.flags, NULL, settings, "VkDescriptorPoolCreateFlags", "flags", false, false, indents + 1, dump_json_VkDescriptorPoolCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxSets, NULL, settings, "uint32_t", "maxSets", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.poolSizeCount, NULL, settings, "uint32_t", "poolSizeCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDescriptorPoolSize>(object.pPoolSizes, object.poolSizeCount, settings, "const VkDescriptorPoolSize*", "const VkDescriptorPoolSize", "pPoolSizes", true, false, indents + 1, dump_json_VkDescriptorPoolSize); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorSetAllocateInfo(const VkDescriptorSetAllocateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorPool>(object.descriptorPool, NULL, settings, "VkDescriptorPool", "descriptorPool", false, false, indents + 1, dump_json_VkDescriptorPool);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.descriptorSetCount, NULL, settings, "uint32_t", "descriptorSetCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDescriptorSetLayout>(object.pSetLayouts, object.descriptorSetCount, settings, "const VkDescriptorSetLayout*", "const VkDescriptorSetLayout", "pSetLayouts", false, false, indents + 1, dump_json_VkDescriptorSetLayout); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorSetLayoutBinding(const VkDescriptorSetLayoutBinding& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.binding, NULL, settings, "uint32_t", "binding", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorType>(object.descriptorType, NULL, settings, "VkDescriptorType", "descriptorType", false, false, indents + 1, dump_json_VkDescriptorType);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.descriptorCount, NULL, settings, "uint32_t", "descriptorCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlags>(object.stageFlags, NULL, settings, "VkShaderStageFlags", "stageFlags", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << ",\n";
    if((object.descriptorType == VK_DESCRIPTOR_TYPE_SAMPLER) || (object.descriptorType == VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER))
    dump_json_array<const VkSampler>(object.pImmutableSamplers, object.descriptorCount, settings, "const VkSampler*", "const VkSampler", "pImmutableSamplers", false, false, indents + 1, dump_json_VkSampler); // JQA
    else
        dump_json_UNUSED(settings, "const VkSampler*", "pImmutableSamplers", indents + 1);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorSetLayoutCreateInfo(const VkDescriptorSetLayoutCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorSetLayoutCreateFlags>(object.flags, NULL, settings, "VkDescriptorSetLayoutCreateFlags", "flags", false, false, indents + 1, dump_json_VkDescriptorSetLayoutCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.bindingCount, NULL, settings, "uint32_t", "bindingCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDescriptorSetLayoutBinding>(object.pBindings, object.bindingCount, settings, "const VkDescriptorSetLayoutBinding*", "const VkDescriptorSetLayoutBinding", "pBindings", true, false, indents + 1, dump_json_VkDescriptorSetLayoutBinding); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkWriteDescriptorSet(const VkWriteDescriptorSet& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorSet>(object.dstSet, NULL, settings, "VkDescriptorSet", "dstSet", false, false, indents + 1, dump_json_VkDescriptorSet);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dstBinding, NULL, settings, "uint32_t", "dstBinding", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dstArrayElement, NULL, settings, "uint32_t", "dstArrayElement", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.descriptorCount, NULL, settings, "uint32_t", "descriptorCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorType>(object.descriptorType, NULL, settings, "VkDescriptorType", "descriptorType", false, false, indents + 1, dump_json_VkDescriptorType);
    settings.stream() << ",\n";
    if((object.descriptorType == VK_DESCRIPTOR_TYPE_SAMPLER) || (object.descriptorType == VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER) || (object.descriptorType == VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE) || (object.descriptorType == VK_DESCRIPTOR_TYPE_STORAGE_IMAGE) || (object.descriptorType == VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT))
    dump_json_array<const VkDescriptorImageInfo>(object.pImageInfo, object.descriptorCount, settings, "const VkDescriptorImageInfo*", "const VkDescriptorImageInfo", "pImageInfo", true, false, indents + 1, dump_json_VkDescriptorImageInfo); // JQA
    else
        dump_json_UNUSED(settings, "const VkDescriptorImageInfo*", "pImageInfo", indents + 1);
    settings.stream() << ",\n";
    if((object.descriptorType == VK_DESCRIPTOR_TYPE_STORAGE_BUFFER) || (object.descriptorType == VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER) || (object.descriptorType == VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC) || (object.descriptorType == VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC))
    dump_json_array<const VkDescriptorBufferInfo>(object.pBufferInfo, object.descriptorCount, settings, "const VkDescriptorBufferInfo*", "const VkDescriptorBufferInfo", "pBufferInfo", true, false, indents + 1, dump_json_VkDescriptorBufferInfo); // JQA
    else
        dump_json_UNUSED(settings, "const VkDescriptorBufferInfo*", "pBufferInfo", indents + 1);
    settings.stream() << ",\n";
    if((object.descriptorType == VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER) || (object.descriptorType == VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER))
    dump_json_array<const VkBufferView>(object.pTexelBufferView, object.descriptorCount, settings, "const VkBufferView*", "const VkBufferView", "pTexelBufferView", false, false, indents + 1, dump_json_VkBufferView); // JQA
    else
        dump_json_UNUSED(settings, "const VkBufferView*", "pTexelBufferView", indents + 1);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAttachmentDescription(const VkAttachmentDescription& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkAttachmentDescriptionFlags>(object.flags, NULL, settings, "VkAttachmentDescriptionFlags", "flags", false, false, indents + 1, dump_json_VkAttachmentDescriptionFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlagBits>(object.samples, NULL, settings, "VkSampleCountFlagBits", "samples", false, false, indents + 1, dump_json_VkSampleCountFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const VkAttachmentLoadOp>(object.loadOp, NULL, settings, "VkAttachmentLoadOp", "loadOp", false, false, indents + 1, dump_json_VkAttachmentLoadOp);
    settings.stream() << ",\n";
    dump_json_value<const VkAttachmentStoreOp>(object.storeOp, NULL, settings, "VkAttachmentStoreOp", "storeOp", false, false, indents + 1, dump_json_VkAttachmentStoreOp);
    settings.stream() << ",\n";
    dump_json_value<const VkAttachmentLoadOp>(object.stencilLoadOp, NULL, settings, "VkAttachmentLoadOp", "stencilLoadOp", false, false, indents + 1, dump_json_VkAttachmentLoadOp);
    settings.stream() << ",\n";
    dump_json_value<const VkAttachmentStoreOp>(object.stencilStoreOp, NULL, settings, "VkAttachmentStoreOp", "stencilStoreOp", false, false, indents + 1, dump_json_VkAttachmentStoreOp);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.initialLayout, NULL, settings, "VkImageLayout", "initialLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.finalLayout, NULL, settings, "VkImageLayout", "finalLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAttachmentReference(const VkAttachmentReference& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.attachment, NULL, settings, "uint32_t", "attachment", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.layout, NULL, settings, "VkImageLayout", "layout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkFramebufferCreateInfo(const VkFramebufferCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFramebufferCreateFlags>(object.flags, NULL, settings, "VkFramebufferCreateFlags", "flags", false, false, indents + 1, dump_json_VkFramebufferCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkRenderPass>(object.renderPass, NULL, settings, "VkRenderPass", "renderPass", false, false, indents + 1, dump_json_VkRenderPass);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.attachmentCount, NULL, settings, "uint32_t", "attachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkImageView>(object.pAttachments, object.attachmentCount, settings, "const VkImageView*", "const VkImageView", "pAttachments", false, false, indents + 1, dump_json_VkImageView); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.width, NULL, settings, "uint32_t", "width", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.height, NULL, settings, "uint32_t", "height", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.layers, NULL, settings, "uint32_t", "layers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSubpassDescription(const VkSubpassDescription& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkSubpassDescriptionFlags>(object.flags, NULL, settings, "VkSubpassDescriptionFlags", "flags", false, false, indents + 1, dump_json_VkSubpassDescriptionFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineBindPoint>(object.pipelineBindPoint, NULL, settings, "VkPipelineBindPoint", "pipelineBindPoint", false, false, indents + 1, dump_json_VkPipelineBindPoint);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.inputAttachmentCount, NULL, settings, "uint32_t", "inputAttachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkAttachmentReference>(object.pInputAttachments, object.inputAttachmentCount, settings, "const VkAttachmentReference*", "const VkAttachmentReference", "pInputAttachments", true, false, indents + 1, dump_json_VkAttachmentReference); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.colorAttachmentCount, NULL, settings, "uint32_t", "colorAttachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkAttachmentReference>(object.pColorAttachments, object.colorAttachmentCount, settings, "const VkAttachmentReference*", "const VkAttachmentReference", "pColorAttachments", true, false, indents + 1, dump_json_VkAttachmentReference); // JQA
    settings.stream() << ",\n";
    dump_json_array<const VkAttachmentReference>(object.pResolveAttachments, object.colorAttachmentCount, settings, "const VkAttachmentReference*", "const VkAttachmentReference", "pResolveAttachments", true, false, indents + 1, dump_json_VkAttachmentReference); // JQA
    settings.stream() << ",\n";
    dump_json_pointer<const VkAttachmentReference>(object.pDepthStencilAttachment, settings, "const VkAttachmentReference*", "pDepthStencilAttachment", true, false, indents + 1, dump_json_VkAttachmentReference);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.preserveAttachmentCount, NULL, settings, "uint32_t", "preserveAttachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pPreserveAttachments, object.preserveAttachmentCount, settings, "const uint32_t*", "const uint32_t", "pPreserveAttachments", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSubpassDependency(const VkSubpassDependency& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.srcSubpass, NULL, settings, "uint32_t", "srcSubpass", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dstSubpass, NULL, settings, "uint32_t", "dstSubpass", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineStageFlags>(object.srcStageMask, NULL, settings, "VkPipelineStageFlags", "srcStageMask", false, false, indents + 1, dump_json_VkPipelineStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineStageFlags>(object.dstStageMask, NULL, settings, "VkPipelineStageFlags", "dstStageMask", false, false, indents + 1, dump_json_VkPipelineStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags>(object.srcAccessMask, NULL, settings, "VkAccessFlags", "srcAccessMask", false, false, indents + 1, dump_json_VkAccessFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags>(object.dstAccessMask, NULL, settings, "VkAccessFlags", "dstAccessMask", false, false, indents + 1, dump_json_VkAccessFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkDependencyFlags>(object.dependencyFlags, NULL, settings, "VkDependencyFlags", "dependencyFlags", false, false, indents + 1, dump_json_VkDependencyFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassCreateInfo(const VkRenderPassCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkRenderPassCreateFlags>(object.flags, NULL, settings, "VkRenderPassCreateFlags", "flags", false, false, indents + 1, dump_json_VkRenderPassCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.attachmentCount, NULL, settings, "uint32_t", "attachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkAttachmentDescription>(object.pAttachments, object.attachmentCount, settings, "const VkAttachmentDescription*", "const VkAttachmentDescription", "pAttachments", true, false, indents + 1, dump_json_VkAttachmentDescription); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.subpassCount, NULL, settings, "uint32_t", "subpassCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSubpassDescription>(object.pSubpasses, object.subpassCount, settings, "const VkSubpassDescription*", "const VkSubpassDescription", "pSubpasses", true, false, indents + 1, dump_json_VkSubpassDescription); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dependencyCount, NULL, settings, "uint32_t", "dependencyCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSubpassDependency>(object.pDependencies, object.dependencyCount, settings, "const VkSubpassDependency*", "const VkSubpassDependency", "pDependencies", true, false, indents + 1, dump_json_VkSubpassDependency); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCommandPoolCreateInfo(const VkCommandPoolCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkCommandPoolCreateFlags>(object.flags, NULL, settings, "VkCommandPoolCreateFlags", "flags", false, false, indents + 1, dump_json_VkCommandPoolCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queueFamilyIndex, NULL, settings, "uint32_t", "queueFamilyIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCommandBufferAllocateInfo(const VkCommandBufferAllocateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkCommandPool>(object.commandPool, NULL, settings, "VkCommandPool", "commandPool", false, false, indents + 1, dump_json_VkCommandPool);
    settings.stream() << ",\n";
    dump_json_value<const VkCommandBufferLevel>(object.level, NULL, settings, "VkCommandBufferLevel", "level", false, false, indents + 1, dump_json_VkCommandBufferLevel);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.commandBufferCount, NULL, settings, "uint32_t", "commandBufferCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCommandBufferInheritanceInfo(const VkCommandBufferInheritanceInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkRenderPass>(object.renderPass, NULL, settings, "VkRenderPass", "renderPass", false, false, indents + 1, dump_json_VkRenderPass);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.subpass, NULL, settings, "uint32_t", "subpass", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkFramebuffer>(object.framebuffer, NULL, settings, "VkFramebuffer", "framebuffer", false, false, indents + 1, dump_json_VkFramebuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.occlusionQueryEnable, NULL, settings, "VkBool32", "occlusionQueryEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkQueryControlFlags>(object.queryFlags, NULL, settings, "VkQueryControlFlags", "queryFlags", false, false, indents + 1, dump_json_VkQueryControlFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkQueryPipelineStatisticFlags>(object.pipelineStatistics, NULL, settings, "VkQueryPipelineStatisticFlags", "pipelineStatistics", false, false, indents + 1, dump_json_VkQueryPipelineStatisticFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCommandBufferBeginInfo(const VkCommandBufferBeginInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkCommandBufferUsageFlags>(object.flags, NULL, settings, "VkCommandBufferUsageFlags", "flags", false, false, indents + 1, dump_json_VkCommandBufferUsageFlags);
    settings.stream() << ",\n";
    if(ApiDumpInstance::current().getCmdBufferLevel() == VK_COMMAND_BUFFER_LEVEL_SECONDARY)
    dump_json_pointer<const VkCommandBufferInheritanceInfo>(object.pInheritanceInfo, settings, "const VkCommandBufferInheritanceInfo*", "pInheritanceInfo", true, false, indents + 1, dump_json_VkCommandBufferInheritanceInfo);
    else
        dump_json_UNUSED(settings, "const VkCommandBufferInheritanceInfo*", "pInheritanceInfo", indents + 1);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBufferCopy(const VkBufferCopy& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceSize>(object.srcOffset, NULL, settings, "VkDeviceSize", "srcOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.dstOffset, NULL, settings, "VkDeviceSize", "dstOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageSubresourceLayers(const VkImageSubresourceLayers& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkImageAspectFlags>(object.aspectMask, NULL, settings, "VkImageAspectFlags", "aspectMask", false, false, indents + 1, dump_json_VkImageAspectFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.mipLevel, NULL, settings, "uint32_t", "mipLevel", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.baseArrayLayer, NULL, settings, "uint32_t", "baseArrayLayer", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.layerCount, NULL, settings, "uint32_t", "layerCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBufferImageCopy(const VkBufferImageCopy& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceSize>(object.bufferOffset, NULL, settings, "VkDeviceSize", "bufferOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.bufferRowLength, NULL, settings, "uint32_t", "bufferRowLength", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.bufferImageHeight, NULL, settings, "uint32_t", "bufferImageHeight", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceLayers>(object.imageSubresource, NULL, settings, "VkImageSubresourceLayers", "imageSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset3D>(object.imageOffset, NULL, settings, "VkOffset3D", "imageOffset", true, false, indents + 1, dump_json_VkOffset3D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent3D>(object.imageExtent, NULL, settings, "VkExtent3D", "imageExtent", true, false, indents + 1, dump_json_VkExtent3D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkClearDepthStencilValue(const VkClearDepthStencilValue& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const float>(object.depth, NULL, settings, "float", "depth", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stencil, NULL, settings, "uint32_t", "stencil", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkClearAttachment(const VkClearAttachment& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkImageAspectFlags>(object.aspectMask, NULL, settings, "VkImageAspectFlags", "aspectMask", false, false, indents + 1, dump_json_VkImageAspectFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.colorAttachment, NULL, settings, "uint32_t", "colorAttachment", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkClearValue>(object.clearValue, NULL, settings, "VkClearValue", "clearValue", false, true, indents + 1, dump_json_VkClearValue);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkClearRect(const VkClearRect& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkRect2D>(object.rect, NULL, settings, "VkRect2D", "rect", true, false, indents + 1, dump_json_VkRect2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.baseArrayLayer, NULL, settings, "uint32_t", "baseArrayLayer", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.layerCount, NULL, settings, "uint32_t", "layerCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageBlit(const VkImageBlit& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkImageSubresourceLayers>(object.srcSubresource, NULL, settings, "VkImageSubresourceLayers", "srcSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_array<const VkOffset3D>(object.srcOffsets, 2, settings, "VkOffset3D[2]", "VkOffset3D", "srcOffsets", true, false, indents + 1, dump_json_VkOffset3D); // IQA
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceLayers>(object.dstSubresource, NULL, settings, "VkImageSubresourceLayers", "dstSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_array<const VkOffset3D>(object.dstOffsets, 2, settings, "VkOffset3D[2]", "VkOffset3D", "dstOffsets", true, false, indents + 1, dump_json_VkOffset3D); // IQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageCopy(const VkImageCopy& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkImageSubresourceLayers>(object.srcSubresource, NULL, settings, "VkImageSubresourceLayers", "srcSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset3D>(object.srcOffset, NULL, settings, "VkOffset3D", "srcOffset", true, false, indents + 1, dump_json_VkOffset3D);
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceLayers>(object.dstSubresource, NULL, settings, "VkImageSubresourceLayers", "dstSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset3D>(object.dstOffset, NULL, settings, "VkOffset3D", "dstOffset", true, false, indents + 1, dump_json_VkOffset3D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent3D>(object.extent, NULL, settings, "VkExtent3D", "extent", true, false, indents + 1, dump_json_VkExtent3D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageResolve(const VkImageResolve& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkImageSubresourceLayers>(object.srcSubresource, NULL, settings, "VkImageSubresourceLayers", "srcSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset3D>(object.srcOffset, NULL, settings, "VkOffset3D", "srcOffset", true, false, indents + 1, dump_json_VkOffset3D);
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceLayers>(object.dstSubresource, NULL, settings, "VkImageSubresourceLayers", "dstSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset3D>(object.dstOffset, NULL, settings, "VkOffset3D", "dstOffset", true, false, indents + 1, dump_json_VkOffset3D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent3D>(object.extent, NULL, settings, "VkExtent3D", "extent", true, false, indents + 1, dump_json_VkExtent3D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassBeginInfo(const VkRenderPassBeginInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkRenderPass>(object.renderPass, NULL, settings, "VkRenderPass", "renderPass", false, false, indents + 1, dump_json_VkRenderPass);
    settings.stream() << ",\n";
    dump_json_value<const VkFramebuffer>(object.framebuffer, NULL, settings, "VkFramebuffer", "framebuffer", false, false, indents + 1, dump_json_VkFramebuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkRect2D>(object.renderArea, NULL, settings, "VkRect2D", "renderArea", true, false, indents + 1, dump_json_VkRect2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.clearValueCount, NULL, settings, "uint32_t", "clearValueCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkClearValue>(object.pClearValues, object.clearValueCount, settings, "const VkClearValue*", "const VkClearValue", "pClearValues", false, true, indents + 1, dump_json_VkClearValue); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSubgroupProperties(const VkPhysicalDeviceSubgroupProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.subgroupSize, NULL, settings, "uint32_t", "subgroupSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlags>(object.supportedStages, NULL, settings, "VkShaderStageFlags", "supportedStages", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSubgroupFeatureFlags>(object.supportedOperations, NULL, settings, "VkSubgroupFeatureFlags", "supportedOperations", false, false, indents + 1, dump_json_VkSubgroupFeatureFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.quadOperationsInAllStages, NULL, settings, "VkBool32", "quadOperationsInAllStages", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindBufferMemoryInfo(const VkBindBufferMemoryInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.memoryOffset, NULL, settings, "VkDeviceSize", "memoryOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindImageMemoryInfo(const VkBindImageMemoryInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.image, NULL, settings, "VkImage", "image", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.memoryOffset, NULL, settings, "VkDeviceSize", "memoryOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevice16BitStorageFeatures(const VkPhysicalDevice16BitStorageFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.storageBuffer16BitAccess, NULL, settings, "VkBool32", "storageBuffer16BitAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.uniformAndStorageBuffer16BitAccess, NULL, settings, "VkBool32", "uniformAndStorageBuffer16BitAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.storagePushConstant16, NULL, settings, "VkBool32", "storagePushConstant16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.storageInputOutput16, NULL, settings, "VkBool32", "storageInputOutput16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryDedicatedRequirements(const VkMemoryDedicatedRequirements& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.prefersDedicatedAllocation, NULL, settings, "VkBool32", "prefersDedicatedAllocation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.requiresDedicatedAllocation, NULL, settings, "VkBool32", "requiresDedicatedAllocation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryDedicatedAllocateInfo(const VkMemoryDedicatedAllocateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.image, NULL, settings, "VkImage", "image", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryAllocateFlagsInfo(const VkMemoryAllocateFlagsInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkMemoryAllocateFlags>(object.flags, NULL, settings, "VkMemoryAllocateFlags", "flags", false, false, indents + 1, dump_json_VkMemoryAllocateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceMask, NULL, settings, "uint32_t", "deviceMask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceGroupRenderPassBeginInfo(const VkDeviceGroupRenderPassBeginInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceMask, NULL, settings, "uint32_t", "deviceMask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceRenderAreaCount, NULL, settings, "uint32_t", "deviceRenderAreaCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkRect2D>(object.pDeviceRenderAreas, object.deviceRenderAreaCount, settings, "const VkRect2D*", "const VkRect2D", "pDeviceRenderAreas", true, false, indents + 1, dump_json_VkRect2D); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceGroupCommandBufferBeginInfo(const VkDeviceGroupCommandBufferBeginInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceMask, NULL, settings, "uint32_t", "deviceMask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceGroupSubmitInfo(const VkDeviceGroupSubmitInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.waitSemaphoreCount, NULL, settings, "uint32_t", "waitSemaphoreCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pWaitSemaphoreDeviceIndices, object.waitSemaphoreCount, settings, "const uint32_t*", "const uint32_t", "pWaitSemaphoreDeviceIndices", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.commandBufferCount, NULL, settings, "uint32_t", "commandBufferCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pCommandBufferDeviceMasks, object.commandBufferCount, settings, "const uint32_t*", "const uint32_t", "pCommandBufferDeviceMasks", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.signalSemaphoreCount, NULL, settings, "uint32_t", "signalSemaphoreCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pSignalSemaphoreDeviceIndices, object.signalSemaphoreCount, settings, "const uint32_t*", "const uint32_t", "pSignalSemaphoreDeviceIndices", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceGroupBindSparseInfo(const VkDeviceGroupBindSparseInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.resourceDeviceIndex, NULL, settings, "uint32_t", "resourceDeviceIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryDeviceIndex, NULL, settings, "uint32_t", "memoryDeviceIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindBufferMemoryDeviceGroupInfo(const VkBindBufferMemoryDeviceGroupInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceIndexCount, NULL, settings, "uint32_t", "deviceIndexCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pDeviceIndices, object.deviceIndexCount, settings, "const uint32_t*", "const uint32_t", "pDeviceIndices", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindImageMemoryDeviceGroupInfo(const VkBindImageMemoryDeviceGroupInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceIndexCount, NULL, settings, "uint32_t", "deviceIndexCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pDeviceIndices, object.deviceIndexCount, settings, "const uint32_t*", "const uint32_t", "pDeviceIndices", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.splitInstanceBindRegionCount, NULL, settings, "uint32_t", "splitInstanceBindRegionCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkRect2D>(object.pSplitInstanceBindRegions, object.splitInstanceBindRegionCount, settings, "const VkRect2D*", "const VkRect2D", "pSplitInstanceBindRegions", true, false, indents + 1, dump_json_VkRect2D); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceGroupProperties(const VkPhysicalDeviceGroupProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.physicalDeviceCount, NULL, settings, "uint32_t", "physicalDeviceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPhysicalDevice>(object.physicalDevices, object.physicalDeviceCount, settings, "VkPhysicalDevice[VK_MAX_DEVICE_GROUP_SIZE]", "VkPhysicalDevice", "physicalDevices", false, false, indents + 1, dump_json_VkPhysicalDevice); // JQA
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.subsetAllocation, NULL, settings, "VkBool32", "subsetAllocation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceGroupDeviceCreateInfo(const VkDeviceGroupDeviceCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.physicalDeviceCount, NULL, settings, "uint32_t", "physicalDeviceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPhysicalDevice>(object.pPhysicalDevices, object.physicalDeviceCount, settings, "const VkPhysicalDevice*", "const VkPhysicalDevice", "pPhysicalDevices", false, false, indents + 1, dump_json_VkPhysicalDevice); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBufferMemoryRequirementsInfo2(const VkBufferMemoryRequirementsInfo2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageMemoryRequirementsInfo2(const VkImageMemoryRequirementsInfo2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.image, NULL, settings, "VkImage", "image", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageSparseMemoryRequirementsInfo2(const VkImageSparseMemoryRequirementsInfo2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.image, NULL, settings, "VkImage", "image", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryRequirements2(const VkMemoryRequirements2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkMemoryRequirements>(object.memoryRequirements, NULL, settings, "VkMemoryRequirements", "memoryRequirements", true, false, indents + 1, dump_json_VkMemoryRequirements);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSparseImageMemoryRequirements2(const VkSparseImageMemoryRequirements2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSparseImageMemoryRequirements>(object.memoryRequirements, NULL, settings, "VkSparseImageMemoryRequirements", "memoryRequirements", true, false, indents + 1, dump_json_VkSparseImageMemoryRequirements);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFeatures2(const VkPhysicalDeviceFeatures2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPhysicalDeviceFeatures>(object.features, NULL, settings, "VkPhysicalDeviceFeatures", "features", true, false, indents + 1, dump_json_VkPhysicalDeviceFeatures);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceProperties2(const VkPhysicalDeviceProperties2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPhysicalDeviceProperties>(object.properties, NULL, settings, "VkPhysicalDeviceProperties", "properties", true, false, indents + 1, dump_json_VkPhysicalDeviceProperties);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkFormatProperties2(const VkFormatProperties2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFormatProperties>(object.formatProperties, NULL, settings, "VkFormatProperties", "formatProperties", true, false, indents + 1, dump_json_VkFormatProperties);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageFormatProperties2(const VkImageFormatProperties2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageFormatProperties>(object.imageFormatProperties, NULL, settings, "VkImageFormatProperties", "imageFormatProperties", true, false, indents + 1, dump_json_VkImageFormatProperties);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImageFormatInfo2(const VkPhysicalDeviceImageFormatInfo2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkImageType>(object.type, NULL, settings, "VkImageType", "type", false, false, indents + 1, dump_json_VkImageType);
    settings.stream() << ",\n";
    dump_json_value<const VkImageTiling>(object.tiling, NULL, settings, "VkImageTiling", "tiling", false, false, indents + 1, dump_json_VkImageTiling);
    settings.stream() << ",\n";
    dump_json_value<const VkImageUsageFlags>(object.usage, NULL, settings, "VkImageUsageFlags", "usage", false, false, indents + 1, dump_json_VkImageUsageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkImageCreateFlags>(object.flags, NULL, settings, "VkImageCreateFlags", "flags", false, false, indents + 1, dump_json_VkImageCreateFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkQueueFamilyProperties2(const VkQueueFamilyProperties2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkQueueFamilyProperties>(object.queueFamilyProperties, NULL, settings, "VkQueueFamilyProperties", "queueFamilyProperties", true, false, indents + 1, dump_json_VkQueueFamilyProperties);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMemoryProperties2(const VkPhysicalDeviceMemoryProperties2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    ApiDumpInstance::current().setMemoryHeapCount(object.memoryProperties.memoryHeapCount);
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPhysicalDeviceMemoryProperties>(object.memoryProperties, NULL, settings, "VkPhysicalDeviceMemoryProperties", "memoryProperties", true, false, indents + 1, dump_json_VkPhysicalDeviceMemoryProperties);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSparseImageFormatProperties2(const VkSparseImageFormatProperties2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSparseImageFormatProperties>(object.properties, NULL, settings, "VkSparseImageFormatProperties", "properties", true, false, indents + 1, dump_json_VkSparseImageFormatProperties);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSparseImageFormatInfo2(const VkPhysicalDeviceSparseImageFormatInfo2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkImageType>(object.type, NULL, settings, "VkImageType", "type", false, false, indents + 1, dump_json_VkImageType);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlagBits>(object.samples, NULL, settings, "VkSampleCountFlagBits", "samples", false, false, indents + 1, dump_json_VkSampleCountFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const VkImageUsageFlags>(object.usage, NULL, settings, "VkImageUsageFlags", "usage", false, false, indents + 1, dump_json_VkImageUsageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkImageTiling>(object.tiling, NULL, settings, "VkImageTiling", "tiling", false, false, indents + 1, dump_json_VkImageTiling);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePointClippingProperties(const VkPhysicalDevicePointClippingProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPointClippingBehavior>(object.pointClippingBehavior, NULL, settings, "VkPointClippingBehavior", "pointClippingBehavior", false, false, indents + 1, dump_json_VkPointClippingBehavior);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkInputAttachmentAspectReference(const VkInputAttachmentAspectReference& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.subpass, NULL, settings, "uint32_t", "subpass", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.inputAttachmentIndex, NULL, settings, "uint32_t", "inputAttachmentIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkImageAspectFlags>(object.aspectMask, NULL, settings, "VkImageAspectFlags", "aspectMask", false, false, indents + 1, dump_json_VkImageAspectFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassInputAttachmentAspectCreateInfo(const VkRenderPassInputAttachmentAspectCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.aspectReferenceCount, NULL, settings, "uint32_t", "aspectReferenceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkInputAttachmentAspectReference>(object.pAspectReferences, object.aspectReferenceCount, settings, "const VkInputAttachmentAspectReference*", "const VkInputAttachmentAspectReference", "pAspectReferences", true, false, indents + 1, dump_json_VkInputAttachmentAspectReference); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageViewUsageCreateInfo(const VkImageViewUsageCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageUsageFlags>(object.usage, NULL, settings, "VkImageUsageFlags", "usage", false, false, indents + 1, dump_json_VkImageUsageFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineTessellationDomainOriginStateCreateInfo(const VkPipelineTessellationDomainOriginStateCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkTessellationDomainOrigin>(object.domainOrigin, NULL, settings, "VkTessellationDomainOrigin", "domainOrigin", false, false, indents + 1, dump_json_VkTessellationDomainOrigin);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassMultiviewCreateInfo(const VkRenderPassMultiviewCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.subpassCount, NULL, settings, "uint32_t", "subpassCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pViewMasks, object.subpassCount, settings, "const uint32_t*", "const uint32_t", "pViewMasks", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dependencyCount, NULL, settings, "uint32_t", "dependencyCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const int32_t>(object.pViewOffsets, object.dependencyCount, settings, "const int32_t*", "const int32_t", "pViewOffsets", false, false, indents + 1, dump_json_int32_t); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.correlationMaskCount, NULL, settings, "uint32_t", "correlationMaskCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pCorrelationMasks, object.correlationMaskCount, settings, "const uint32_t*", "const uint32_t", "pCorrelationMasks", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMultiviewFeatures(const VkPhysicalDeviceMultiviewFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multiview, NULL, settings, "VkBool32", "multiview", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multiviewGeometryShader, NULL, settings, "VkBool32", "multiviewGeometryShader", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multiviewTessellationShader, NULL, settings, "VkBool32", "multiviewTessellationShader", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMultiviewProperties(const VkPhysicalDeviceMultiviewProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMultiviewViewCount, NULL, settings, "uint32_t", "maxMultiviewViewCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMultiviewInstanceIndex, NULL, settings, "uint32_t", "maxMultiviewInstanceIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceVariablePointersFeatures(const VkPhysicalDeviceVariablePointersFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.variablePointersStorageBuffer, NULL, settings, "VkBool32", "variablePointersStorageBuffer", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.variablePointers, NULL, settings, "VkBool32", "variablePointers", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceProtectedMemoryFeatures(const VkPhysicalDeviceProtectedMemoryFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.protectedMemory, NULL, settings, "VkBool32", "protectedMemory", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceProtectedMemoryProperties(const VkPhysicalDeviceProtectedMemoryProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.protectedNoFault, NULL, settings, "VkBool32", "protectedNoFault", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceQueueInfo2(const VkDeviceQueueInfo2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceQueueCreateFlags>(object.flags, NULL, settings, "VkDeviceQueueCreateFlags", "flags", false, false, indents + 1, dump_json_VkDeviceQueueCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queueFamilyIndex, NULL, settings, "uint32_t", "queueFamilyIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queueIndex, NULL, settings, "uint32_t", "queueIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkProtectedSubmitInfo(const VkProtectedSubmitInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.protectedSubmit, NULL, settings, "VkBool32", "protectedSubmit", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSamplerYcbcrConversionCreateInfo(const VkSamplerYcbcrConversionCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerYcbcrModelConversion>(object.ycbcrModel, NULL, settings, "VkSamplerYcbcrModelConversion", "ycbcrModel", false, false, indents + 1, dump_json_VkSamplerYcbcrModelConversion);
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerYcbcrRange>(object.ycbcrRange, NULL, settings, "VkSamplerYcbcrRange", "ycbcrRange", false, false, indents + 1, dump_json_VkSamplerYcbcrRange);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentMapping>(object.components, NULL, settings, "VkComponentMapping", "components", true, false, indents + 1, dump_json_VkComponentMapping);
    settings.stream() << ",\n";
    dump_json_value<const VkChromaLocation>(object.xChromaOffset, NULL, settings, "VkChromaLocation", "xChromaOffset", false, false, indents + 1, dump_json_VkChromaLocation);
    settings.stream() << ",\n";
    dump_json_value<const VkChromaLocation>(object.yChromaOffset, NULL, settings, "VkChromaLocation", "yChromaOffset", false, false, indents + 1, dump_json_VkChromaLocation);
    settings.stream() << ",\n";
    dump_json_value<const VkFilter>(object.chromaFilter, NULL, settings, "VkFilter", "chromaFilter", false, false, indents + 1, dump_json_VkFilter);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.forceExplicitReconstruction, NULL, settings, "VkBool32", "forceExplicitReconstruction", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSamplerYcbcrConversionInfo(const VkSamplerYcbcrConversionInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerYcbcrConversion>(object.conversion, NULL, settings, "VkSamplerYcbcrConversion", "conversion", false, false, indents + 1, dump_json_VkSamplerYcbcrConversion);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindImagePlaneMemoryInfo(const VkBindImagePlaneMemoryInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageAspectFlagBits>(object.planeAspect, NULL, settings, "VkImageAspectFlagBits", "planeAspect", false, false, indents + 1, dump_json_VkImageAspectFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImagePlaneMemoryRequirementsInfo(const VkImagePlaneMemoryRequirementsInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageAspectFlagBits>(object.planeAspect, NULL, settings, "VkImageAspectFlagBits", "planeAspect", false, false, indents + 1, dump_json_VkImageAspectFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSamplerYcbcrConversionFeatures(const VkPhysicalDeviceSamplerYcbcrConversionFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.samplerYcbcrConversion, NULL, settings, "VkBool32", "samplerYcbcrConversion", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSamplerYcbcrConversionImageFormatProperties(const VkSamplerYcbcrConversionImageFormatProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.combinedImageSamplerDescriptorCount, NULL, settings, "uint32_t", "combinedImageSamplerDescriptorCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorUpdateTemplateEntry(const VkDescriptorUpdateTemplateEntry& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.dstBinding, NULL, settings, "uint32_t", "dstBinding", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dstArrayElement, NULL, settings, "uint32_t", "dstArrayElement", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.descriptorCount, NULL, settings, "uint32_t", "descriptorCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorType>(object.descriptorType, NULL, settings, "VkDescriptorType", "descriptorType", false, false, indents + 1, dump_json_VkDescriptorType);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.offset, NULL, settings, "size_t", "offset", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.stride, NULL, settings, "size_t", "stride", false, false, indents + 1, dump_json_size_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorUpdateTemplateCreateInfo(const VkDescriptorUpdateTemplateCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorUpdateTemplateCreateFlags>(object.flags, NULL, settings, "VkDescriptorUpdateTemplateCreateFlags", "flags", false, false, indents + 1, dump_json_VkDescriptorUpdateTemplateCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.descriptorUpdateEntryCount, NULL, settings, "uint32_t", "descriptorUpdateEntryCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDescriptorUpdateTemplateEntry>(object.pDescriptorUpdateEntries, object.descriptorUpdateEntryCount, settings, "const VkDescriptorUpdateTemplateEntry*", "const VkDescriptorUpdateTemplateEntry", "pDescriptorUpdateEntries", true, false, indents + 1, dump_json_VkDescriptorUpdateTemplateEntry); // JQA
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorUpdateTemplateType>(object.templateType, NULL, settings, "VkDescriptorUpdateTemplateType", "templateType", false, false, indents + 1, dump_json_VkDescriptorUpdateTemplateType);
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorSetLayout>(object.descriptorSetLayout, NULL, settings, "VkDescriptorSetLayout", "descriptorSetLayout", false, false, indents + 1, dump_json_VkDescriptorSetLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineBindPoint>(object.pipelineBindPoint, NULL, settings, "VkPipelineBindPoint", "pipelineBindPoint", false, false, indents + 1, dump_json_VkPipelineBindPoint);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineLayout>(object.pipelineLayout, NULL, settings, "VkPipelineLayout", "pipelineLayout", false, false, indents + 1, dump_json_VkPipelineLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.set, NULL, settings, "uint32_t", "set", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExternalMemoryProperties(const VkExternalMemoryProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkExternalMemoryFeatureFlags>(object.externalMemoryFeatures, NULL, settings, "VkExternalMemoryFeatureFlags", "externalMemoryFeatures", false, false, indents + 1, dump_json_VkExternalMemoryFeatureFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlags>(object.exportFromImportedHandleTypes, NULL, settings, "VkExternalMemoryHandleTypeFlags", "exportFromImportedHandleTypes", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlags>(object.compatibleHandleTypes, NULL, settings, "VkExternalMemoryHandleTypeFlags", "compatibleHandleTypes", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceExternalImageFormatInfo(const VkPhysicalDeviceExternalImageFormatInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExternalImageFormatProperties(const VkExternalImageFormatProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryProperties>(object.externalMemoryProperties, NULL, settings, "VkExternalMemoryProperties", "externalMemoryProperties", true, false, indents + 1, dump_json_VkExternalMemoryProperties);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceExternalBufferInfo(const VkPhysicalDeviceExternalBufferInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBufferCreateFlags>(object.flags, NULL, settings, "VkBufferCreateFlags", "flags", false, false, indents + 1, dump_json_VkBufferCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkBufferUsageFlags>(object.usage, NULL, settings, "VkBufferUsageFlags", "usage", false, false, indents + 1, dump_json_VkBufferUsageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExternalBufferProperties(const VkExternalBufferProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryProperties>(object.externalMemoryProperties, NULL, settings, "VkExternalMemoryProperties", "externalMemoryProperties", true, false, indents + 1, dump_json_VkExternalMemoryProperties);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceIDProperties(const VkPhysicalDeviceIDProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.deviceUUID, 16, settings, "uint8_t[VK_UUID_SIZE]", "uint8_t", "deviceUUID", false, false, indents + 1, dump_json_uint8_t); // IQA
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.driverUUID, 16, settings, "uint8_t[VK_UUID_SIZE]", "uint8_t", "driverUUID", false, false, indents + 1, dump_json_uint8_t); // IQA
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.deviceLUID, 8, settings, "uint8_t[VK_LUID_SIZE]", "uint8_t", "deviceLUID", false, false, indents + 1, dump_json_uint8_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceNodeMask, NULL, settings, "uint32_t", "deviceNodeMask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.deviceLUIDValid, NULL, settings, "VkBool32", "deviceLUIDValid", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExternalMemoryImageCreateInfo(const VkExternalMemoryImageCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlags>(object.handleTypes, NULL, settings, "VkExternalMemoryHandleTypeFlags", "handleTypes", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExternalMemoryBufferCreateInfo(const VkExternalMemoryBufferCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlags>(object.handleTypes, NULL, settings, "VkExternalMemoryHandleTypeFlags", "handleTypes", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExportMemoryAllocateInfo(const VkExportMemoryAllocateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlags>(object.handleTypes, NULL, settings, "VkExternalMemoryHandleTypeFlags", "handleTypes", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceExternalFenceInfo(const VkPhysicalDeviceExternalFenceInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalFenceHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalFenceHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalFenceHandleTypeFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExternalFenceProperties(const VkExternalFenceProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalFenceHandleTypeFlags>(object.exportFromImportedHandleTypes, NULL, settings, "VkExternalFenceHandleTypeFlags", "exportFromImportedHandleTypes", false, false, indents + 1, dump_json_VkExternalFenceHandleTypeFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalFenceHandleTypeFlags>(object.compatibleHandleTypes, NULL, settings, "VkExternalFenceHandleTypeFlags", "compatibleHandleTypes", false, false, indents + 1, dump_json_VkExternalFenceHandleTypeFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalFenceFeatureFlags>(object.externalFenceFeatures, NULL, settings, "VkExternalFenceFeatureFlags", "externalFenceFeatures", false, false, indents + 1, dump_json_VkExternalFenceFeatureFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExportFenceCreateInfo(const VkExportFenceCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalFenceHandleTypeFlags>(object.handleTypes, NULL, settings, "VkExternalFenceHandleTypeFlags", "handleTypes", false, false, indents + 1, dump_json_VkExternalFenceHandleTypeFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExportSemaphoreCreateInfo(const VkExportSemaphoreCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalSemaphoreHandleTypeFlags>(object.handleTypes, NULL, settings, "VkExternalSemaphoreHandleTypeFlags", "handleTypes", false, false, indents + 1, dump_json_VkExternalSemaphoreHandleTypeFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceExternalSemaphoreInfo(const VkPhysicalDeviceExternalSemaphoreInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalSemaphoreHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalSemaphoreHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalSemaphoreHandleTypeFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExternalSemaphoreProperties(const VkExternalSemaphoreProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalSemaphoreHandleTypeFlags>(object.exportFromImportedHandleTypes, NULL, settings, "VkExternalSemaphoreHandleTypeFlags", "exportFromImportedHandleTypes", false, false, indents + 1, dump_json_VkExternalSemaphoreHandleTypeFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalSemaphoreHandleTypeFlags>(object.compatibleHandleTypes, NULL, settings, "VkExternalSemaphoreHandleTypeFlags", "compatibleHandleTypes", false, false, indents + 1, dump_json_VkExternalSemaphoreHandleTypeFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalSemaphoreFeatureFlags>(object.externalSemaphoreFeatures, NULL, settings, "VkExternalSemaphoreFeatureFlags", "externalSemaphoreFeatures", false, false, indents + 1, dump_json_VkExternalSemaphoreFeatureFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMaintenance3Properties(const VkPhysicalDeviceMaintenance3Properties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerSetDescriptors, NULL, settings, "uint32_t", "maxPerSetDescriptors", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.maxMemoryAllocationSize, NULL, settings, "VkDeviceSize", "maxMemoryAllocationSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorSetLayoutSupport(const VkDescriptorSetLayoutSupport& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.supported, NULL, settings, "VkBool32", "supported", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderDrawParametersFeatures(const VkPhysicalDeviceShaderDrawParametersFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDrawParameters, NULL, settings, "VkBool32", "shaderDrawParameters", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceVulkan11Features(const VkPhysicalDeviceVulkan11Features& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.storageBuffer16BitAccess, NULL, settings, "VkBool32", "storageBuffer16BitAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.uniformAndStorageBuffer16BitAccess, NULL, settings, "VkBool32", "uniformAndStorageBuffer16BitAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.storagePushConstant16, NULL, settings, "VkBool32", "storagePushConstant16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.storageInputOutput16, NULL, settings, "VkBool32", "storageInputOutput16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multiview, NULL, settings, "VkBool32", "multiview", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multiviewGeometryShader, NULL, settings, "VkBool32", "multiviewGeometryShader", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multiviewTessellationShader, NULL, settings, "VkBool32", "multiviewTessellationShader", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.variablePointersStorageBuffer, NULL, settings, "VkBool32", "variablePointersStorageBuffer", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.variablePointers, NULL, settings, "VkBool32", "variablePointers", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.protectedMemory, NULL, settings, "VkBool32", "protectedMemory", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.samplerYcbcrConversion, NULL, settings, "VkBool32", "samplerYcbcrConversion", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDrawParameters, NULL, settings, "VkBool32", "shaderDrawParameters", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceVulkan11Properties(const VkPhysicalDeviceVulkan11Properties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.deviceUUID, 16, settings, "uint8_t[VK_UUID_SIZE]", "uint8_t", "deviceUUID", false, false, indents + 1, dump_json_uint8_t); // IQA
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.driverUUID, 16, settings, "uint8_t[VK_UUID_SIZE]", "uint8_t", "driverUUID", false, false, indents + 1, dump_json_uint8_t); // IQA
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.deviceLUID, 8, settings, "uint8_t[VK_LUID_SIZE]", "uint8_t", "deviceLUID", false, false, indents + 1, dump_json_uint8_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceNodeMask, NULL, settings, "uint32_t", "deviceNodeMask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.deviceLUIDValid, NULL, settings, "VkBool32", "deviceLUIDValid", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.subgroupSize, NULL, settings, "uint32_t", "subgroupSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlags>(object.subgroupSupportedStages, NULL, settings, "VkShaderStageFlags", "subgroupSupportedStages", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSubgroupFeatureFlags>(object.subgroupSupportedOperations, NULL, settings, "VkSubgroupFeatureFlags", "subgroupSupportedOperations", false, false, indents + 1, dump_json_VkSubgroupFeatureFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.subgroupQuadOperationsInAllStages, NULL, settings, "VkBool32", "subgroupQuadOperationsInAllStages", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkPointClippingBehavior>(object.pointClippingBehavior, NULL, settings, "VkPointClippingBehavior", "pointClippingBehavior", false, false, indents + 1, dump_json_VkPointClippingBehavior);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMultiviewViewCount, NULL, settings, "uint32_t", "maxMultiviewViewCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMultiviewInstanceIndex, NULL, settings, "uint32_t", "maxMultiviewInstanceIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.protectedNoFault, NULL, settings, "VkBool32", "protectedNoFault", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerSetDescriptors, NULL, settings, "uint32_t", "maxPerSetDescriptors", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.maxMemoryAllocationSize, NULL, settings, "VkDeviceSize", "maxMemoryAllocationSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceVulkan12Features(const VkPhysicalDeviceVulkan12Features& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.samplerMirrorClampToEdge, NULL, settings, "VkBool32", "samplerMirrorClampToEdge", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.drawIndirectCount, NULL, settings, "VkBool32", "drawIndirectCount", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.storageBuffer8BitAccess, NULL, settings, "VkBool32", "storageBuffer8BitAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.uniformAndStorageBuffer8BitAccess, NULL, settings, "VkBool32", "uniformAndStorageBuffer8BitAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.storagePushConstant8, NULL, settings, "VkBool32", "storagePushConstant8", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderBufferInt64Atomics, NULL, settings, "VkBool32", "shaderBufferInt64Atomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSharedInt64Atomics, NULL, settings, "VkBool32", "shaderSharedInt64Atomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderFloat16, NULL, settings, "VkBool32", "shaderFloat16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderInt8, NULL, settings, "VkBool32", "shaderInt8", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorIndexing, NULL, settings, "VkBool32", "descriptorIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderInputAttachmentArrayDynamicIndexing, NULL, settings, "VkBool32", "shaderInputAttachmentArrayDynamicIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderUniformTexelBufferArrayDynamicIndexing, NULL, settings, "VkBool32", "shaderUniformTexelBufferArrayDynamicIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageTexelBufferArrayDynamicIndexing, NULL, settings, "VkBool32", "shaderStorageTexelBufferArrayDynamicIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderUniformBufferArrayNonUniformIndexing, NULL, settings, "VkBool32", "shaderUniformBufferArrayNonUniformIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSampledImageArrayNonUniformIndexing, NULL, settings, "VkBool32", "shaderSampledImageArrayNonUniformIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageBufferArrayNonUniformIndexing, NULL, settings, "VkBool32", "shaderStorageBufferArrayNonUniformIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageImageArrayNonUniformIndexing, NULL, settings, "VkBool32", "shaderStorageImageArrayNonUniformIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderInputAttachmentArrayNonUniformIndexing, NULL, settings, "VkBool32", "shaderInputAttachmentArrayNonUniformIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderUniformTexelBufferArrayNonUniformIndexing, NULL, settings, "VkBool32", "shaderUniformTexelBufferArrayNonUniformIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageTexelBufferArrayNonUniformIndexing, NULL, settings, "VkBool32", "shaderStorageTexelBufferArrayNonUniformIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingUniformBufferUpdateAfterBind, NULL, settings, "VkBool32", "descriptorBindingUniformBufferUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingSampledImageUpdateAfterBind, NULL, settings, "VkBool32", "descriptorBindingSampledImageUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingStorageImageUpdateAfterBind, NULL, settings, "VkBool32", "descriptorBindingStorageImageUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingStorageBufferUpdateAfterBind, NULL, settings, "VkBool32", "descriptorBindingStorageBufferUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingUniformTexelBufferUpdateAfterBind, NULL, settings, "VkBool32", "descriptorBindingUniformTexelBufferUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingStorageTexelBufferUpdateAfterBind, NULL, settings, "VkBool32", "descriptorBindingStorageTexelBufferUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingUpdateUnusedWhilePending, NULL, settings, "VkBool32", "descriptorBindingUpdateUnusedWhilePending", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingPartiallyBound, NULL, settings, "VkBool32", "descriptorBindingPartiallyBound", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingVariableDescriptorCount, NULL, settings, "VkBool32", "descriptorBindingVariableDescriptorCount", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.runtimeDescriptorArray, NULL, settings, "VkBool32", "runtimeDescriptorArray", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.samplerFilterMinmax, NULL, settings, "VkBool32", "samplerFilterMinmax", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.scalarBlockLayout, NULL, settings, "VkBool32", "scalarBlockLayout", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.imagelessFramebuffer, NULL, settings, "VkBool32", "imagelessFramebuffer", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.uniformBufferStandardLayout, NULL, settings, "VkBool32", "uniformBufferStandardLayout", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSubgroupExtendedTypes, NULL, settings, "VkBool32", "shaderSubgroupExtendedTypes", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.separateDepthStencilLayouts, NULL, settings, "VkBool32", "separateDepthStencilLayouts", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.hostQueryReset, NULL, settings, "VkBool32", "hostQueryReset", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.timelineSemaphore, NULL, settings, "VkBool32", "timelineSemaphore", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.bufferDeviceAddress, NULL, settings, "VkBool32", "bufferDeviceAddress", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.bufferDeviceAddressCaptureReplay, NULL, settings, "VkBool32", "bufferDeviceAddressCaptureReplay", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.bufferDeviceAddressMultiDevice, NULL, settings, "VkBool32", "bufferDeviceAddressMultiDevice", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.vulkanMemoryModel, NULL, settings, "VkBool32", "vulkanMemoryModel", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.vulkanMemoryModelDeviceScope, NULL, settings, "VkBool32", "vulkanMemoryModelDeviceScope", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.vulkanMemoryModelAvailabilityVisibilityChains, NULL, settings, "VkBool32", "vulkanMemoryModelAvailabilityVisibilityChains", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderOutputViewportIndex, NULL, settings, "VkBool32", "shaderOutputViewportIndex", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderOutputLayer, NULL, settings, "VkBool32", "shaderOutputLayer", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.subgroupBroadcastDynamicId, NULL, settings, "VkBool32", "subgroupBroadcastDynamicId", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkConformanceVersion(const VkConformanceVersion& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint8_t>(object.major, NULL, settings, "uint8_t", "major", false, false, indents + 1, dump_json_uint8_t);
    settings.stream() << ",\n";
    dump_json_value<const uint8_t>(object.minor, NULL, settings, "uint8_t", "minor", false, false, indents + 1, dump_json_uint8_t);
    settings.stream() << ",\n";
    dump_json_value<const uint8_t>(object.subminor, NULL, settings, "uint8_t", "subminor", false, false, indents + 1, dump_json_uint8_t);
    settings.stream() << ",\n";
    dump_json_value<const uint8_t>(object.patch, NULL, settings, "uint8_t", "patch", false, false, indents + 1, dump_json_uint8_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceVulkan12Properties(const VkPhysicalDeviceVulkan12Properties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDriverId>(object.driverID, NULL, settings, "VkDriverId", "driverID", false, false, indents + 1, dump_json_VkDriverId);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.driverName, NULL, settings, "char[VK_MAX_DRIVER_NAME_SIZE]", "driverName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.driverInfo, NULL, settings, "char[VK_MAX_DRIVER_INFO_SIZE]", "driverInfo", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const VkConformanceVersion>(object.conformanceVersion, NULL, settings, "VkConformanceVersion", "conformanceVersion", true, false, indents + 1, dump_json_VkConformanceVersion);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderFloatControlsIndependence>(object.denormBehaviorIndependence, NULL, settings, "VkShaderFloatControlsIndependence", "denormBehaviorIndependence", false, false, indents + 1, dump_json_VkShaderFloatControlsIndependence);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderFloatControlsIndependence>(object.roundingModeIndependence, NULL, settings, "VkShaderFloatControlsIndependence", "roundingModeIndependence", false, false, indents + 1, dump_json_VkShaderFloatControlsIndependence);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSignedZeroInfNanPreserveFloat16, NULL, settings, "VkBool32", "shaderSignedZeroInfNanPreserveFloat16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSignedZeroInfNanPreserveFloat32, NULL, settings, "VkBool32", "shaderSignedZeroInfNanPreserveFloat32", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSignedZeroInfNanPreserveFloat64, NULL, settings, "VkBool32", "shaderSignedZeroInfNanPreserveFloat64", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDenormPreserveFloat16, NULL, settings, "VkBool32", "shaderDenormPreserveFloat16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDenormPreserveFloat32, NULL, settings, "VkBool32", "shaderDenormPreserveFloat32", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDenormPreserveFloat64, NULL, settings, "VkBool32", "shaderDenormPreserveFloat64", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDenormFlushToZeroFloat16, NULL, settings, "VkBool32", "shaderDenormFlushToZeroFloat16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDenormFlushToZeroFloat32, NULL, settings, "VkBool32", "shaderDenormFlushToZeroFloat32", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDenormFlushToZeroFloat64, NULL, settings, "VkBool32", "shaderDenormFlushToZeroFloat64", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderRoundingModeRTEFloat16, NULL, settings, "VkBool32", "shaderRoundingModeRTEFloat16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderRoundingModeRTEFloat32, NULL, settings, "VkBool32", "shaderRoundingModeRTEFloat32", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderRoundingModeRTEFloat64, NULL, settings, "VkBool32", "shaderRoundingModeRTEFloat64", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderRoundingModeRTZFloat16, NULL, settings, "VkBool32", "shaderRoundingModeRTZFloat16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderRoundingModeRTZFloat32, NULL, settings, "VkBool32", "shaderRoundingModeRTZFloat32", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderRoundingModeRTZFloat64, NULL, settings, "VkBool32", "shaderRoundingModeRTZFloat64", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxUpdateAfterBindDescriptorsInAllPools, NULL, settings, "uint32_t", "maxUpdateAfterBindDescriptorsInAllPools", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderUniformBufferArrayNonUniformIndexingNative, NULL, settings, "VkBool32", "shaderUniformBufferArrayNonUniformIndexingNative", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSampledImageArrayNonUniformIndexingNative, NULL, settings, "VkBool32", "shaderSampledImageArrayNonUniformIndexingNative", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageBufferArrayNonUniformIndexingNative, NULL, settings, "VkBool32", "shaderStorageBufferArrayNonUniformIndexingNative", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageImageArrayNonUniformIndexingNative, NULL, settings, "VkBool32", "shaderStorageImageArrayNonUniformIndexingNative", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderInputAttachmentArrayNonUniformIndexingNative, NULL, settings, "VkBool32", "shaderInputAttachmentArrayNonUniformIndexingNative", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.robustBufferAccessUpdateAfterBind, NULL, settings, "VkBool32", "robustBufferAccessUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.quadDivergentImplicitLod, NULL, settings, "VkBool32", "quadDivergentImplicitLod", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUpdateAfterBindSamplers, NULL, settings, "uint32_t", "maxPerStageDescriptorUpdateAfterBindSamplers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUpdateAfterBindUniformBuffers, NULL, settings, "uint32_t", "maxPerStageDescriptorUpdateAfterBindUniformBuffers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUpdateAfterBindStorageBuffers, NULL, settings, "uint32_t", "maxPerStageDescriptorUpdateAfterBindStorageBuffers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUpdateAfterBindSampledImages, NULL, settings, "uint32_t", "maxPerStageDescriptorUpdateAfterBindSampledImages", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUpdateAfterBindStorageImages, NULL, settings, "uint32_t", "maxPerStageDescriptorUpdateAfterBindStorageImages", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUpdateAfterBindInputAttachments, NULL, settings, "uint32_t", "maxPerStageDescriptorUpdateAfterBindInputAttachments", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageUpdateAfterBindResources, NULL, settings, "uint32_t", "maxPerStageUpdateAfterBindResources", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindSamplers, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindSamplers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindUniformBuffers, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindUniformBuffers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindUniformBuffersDynamic, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindUniformBuffersDynamic", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindStorageBuffers, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindStorageBuffers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindStorageBuffersDynamic, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindStorageBuffersDynamic", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindSampledImages, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindSampledImages", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindStorageImages, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindStorageImages", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindInputAttachments, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindInputAttachments", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkResolveModeFlags>(object.supportedDepthResolveModes, NULL, settings, "VkResolveModeFlags", "supportedDepthResolveModes", false, false, indents + 1, dump_json_VkResolveModeFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkResolveModeFlags>(object.supportedStencilResolveModes, NULL, settings, "VkResolveModeFlags", "supportedStencilResolveModes", false, false, indents + 1, dump_json_VkResolveModeFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.independentResolveNone, NULL, settings, "VkBool32", "independentResolveNone", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.independentResolve, NULL, settings, "VkBool32", "independentResolve", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.filterMinmaxSingleComponentFormats, NULL, settings, "VkBool32", "filterMinmaxSingleComponentFormats", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.filterMinmaxImageComponentMapping, NULL, settings, "VkBool32", "filterMinmaxImageComponentMapping", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.maxTimelineSemaphoreValueDifference, NULL, settings, "uint64_t", "maxTimelineSemaphoreValueDifference", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlags>(object.framebufferIntegerColorSampleCounts, NULL, settings, "VkSampleCountFlags", "framebufferIntegerColorSampleCounts", false, false, indents + 1, dump_json_VkSampleCountFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageFormatListCreateInfo(const VkImageFormatListCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.viewFormatCount, NULL, settings, "uint32_t", "viewFormatCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkFormat>(object.pViewFormats, object.viewFormatCount, settings, "const VkFormat*", "const VkFormat", "pViewFormats", false, false, indents + 1, dump_json_VkFormat); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAttachmentDescription2(const VkAttachmentDescription2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAttachmentDescriptionFlags>(object.flags, NULL, settings, "VkAttachmentDescriptionFlags", "flags", false, false, indents + 1, dump_json_VkAttachmentDescriptionFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlagBits>(object.samples, NULL, settings, "VkSampleCountFlagBits", "samples", false, false, indents + 1, dump_json_VkSampleCountFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const VkAttachmentLoadOp>(object.loadOp, NULL, settings, "VkAttachmentLoadOp", "loadOp", false, false, indents + 1, dump_json_VkAttachmentLoadOp);
    settings.stream() << ",\n";
    dump_json_value<const VkAttachmentStoreOp>(object.storeOp, NULL, settings, "VkAttachmentStoreOp", "storeOp", false, false, indents + 1, dump_json_VkAttachmentStoreOp);
    settings.stream() << ",\n";
    dump_json_value<const VkAttachmentLoadOp>(object.stencilLoadOp, NULL, settings, "VkAttachmentLoadOp", "stencilLoadOp", false, false, indents + 1, dump_json_VkAttachmentLoadOp);
    settings.stream() << ",\n";
    dump_json_value<const VkAttachmentStoreOp>(object.stencilStoreOp, NULL, settings, "VkAttachmentStoreOp", "stencilStoreOp", false, false, indents + 1, dump_json_VkAttachmentStoreOp);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.initialLayout, NULL, settings, "VkImageLayout", "initialLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.finalLayout, NULL, settings, "VkImageLayout", "finalLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAttachmentReference2(const VkAttachmentReference2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.attachment, NULL, settings, "uint32_t", "attachment", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.layout, NULL, settings, "VkImageLayout", "layout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkImageAspectFlags>(object.aspectMask, NULL, settings, "VkImageAspectFlags", "aspectMask", false, false, indents + 1, dump_json_VkImageAspectFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSubpassDescription2(const VkSubpassDescription2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSubpassDescriptionFlags>(object.flags, NULL, settings, "VkSubpassDescriptionFlags", "flags", false, false, indents + 1, dump_json_VkSubpassDescriptionFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineBindPoint>(object.pipelineBindPoint, NULL, settings, "VkPipelineBindPoint", "pipelineBindPoint", false, false, indents + 1, dump_json_VkPipelineBindPoint);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.viewMask, NULL, settings, "uint32_t", "viewMask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.inputAttachmentCount, NULL, settings, "uint32_t", "inputAttachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkAttachmentReference2>(object.pInputAttachments, object.inputAttachmentCount, settings, "const VkAttachmentReference2*", "const VkAttachmentReference2", "pInputAttachments", true, false, indents + 1, dump_json_VkAttachmentReference2); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.colorAttachmentCount, NULL, settings, "uint32_t", "colorAttachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkAttachmentReference2>(object.pColorAttachments, object.colorAttachmentCount, settings, "const VkAttachmentReference2*", "const VkAttachmentReference2", "pColorAttachments", true, false, indents + 1, dump_json_VkAttachmentReference2); // JQA
    settings.stream() << ",\n";
    dump_json_array<const VkAttachmentReference2>(object.pResolveAttachments, object.colorAttachmentCount, settings, "const VkAttachmentReference2*", "const VkAttachmentReference2", "pResolveAttachments", true, false, indents + 1, dump_json_VkAttachmentReference2); // JQA
    settings.stream() << ",\n";
    dump_json_pointer<const VkAttachmentReference2>(object.pDepthStencilAttachment, settings, "const VkAttachmentReference2*", "pDepthStencilAttachment", true, false, indents + 1, dump_json_VkAttachmentReference2);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.preserveAttachmentCount, NULL, settings, "uint32_t", "preserveAttachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pPreserveAttachments, object.preserveAttachmentCount, settings, "const uint32_t*", "const uint32_t", "pPreserveAttachments", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSubpassDependency2(const VkSubpassDependency2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.srcSubpass, NULL, settings, "uint32_t", "srcSubpass", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dstSubpass, NULL, settings, "uint32_t", "dstSubpass", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineStageFlags>(object.srcStageMask, NULL, settings, "VkPipelineStageFlags", "srcStageMask", false, false, indents + 1, dump_json_VkPipelineStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineStageFlags>(object.dstStageMask, NULL, settings, "VkPipelineStageFlags", "dstStageMask", false, false, indents + 1, dump_json_VkPipelineStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags>(object.srcAccessMask, NULL, settings, "VkAccessFlags", "srcAccessMask", false, false, indents + 1, dump_json_VkAccessFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags>(object.dstAccessMask, NULL, settings, "VkAccessFlags", "dstAccessMask", false, false, indents + 1, dump_json_VkAccessFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkDependencyFlags>(object.dependencyFlags, NULL, settings, "VkDependencyFlags", "dependencyFlags", false, false, indents + 1, dump_json_VkDependencyFlags);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.viewOffset, NULL, settings, "int32_t", "viewOffset", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassCreateInfo2(const VkRenderPassCreateInfo2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkRenderPassCreateFlags>(object.flags, NULL, settings, "VkRenderPassCreateFlags", "flags", false, false, indents + 1, dump_json_VkRenderPassCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.attachmentCount, NULL, settings, "uint32_t", "attachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkAttachmentDescription2>(object.pAttachments, object.attachmentCount, settings, "const VkAttachmentDescription2*", "const VkAttachmentDescription2", "pAttachments", true, false, indents + 1, dump_json_VkAttachmentDescription2); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.subpassCount, NULL, settings, "uint32_t", "subpassCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSubpassDescription2>(object.pSubpasses, object.subpassCount, settings, "const VkSubpassDescription2*", "const VkSubpassDescription2", "pSubpasses", true, false, indents + 1, dump_json_VkSubpassDescription2); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dependencyCount, NULL, settings, "uint32_t", "dependencyCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSubpassDependency2>(object.pDependencies, object.dependencyCount, settings, "const VkSubpassDependency2*", "const VkSubpassDependency2", "pDependencies", true, false, indents + 1, dump_json_VkSubpassDependency2); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.correlatedViewMaskCount, NULL, settings, "uint32_t", "correlatedViewMaskCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pCorrelatedViewMasks, object.correlatedViewMaskCount, settings, "const uint32_t*", "const uint32_t", "pCorrelatedViewMasks", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSubpassBeginInfo(const VkSubpassBeginInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSubpassContents>(object.contents, NULL, settings, "VkSubpassContents", "contents", false, false, indents + 1, dump_json_VkSubpassContents);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSubpassEndInfo(const VkSubpassEndInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevice8BitStorageFeatures(const VkPhysicalDevice8BitStorageFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.storageBuffer8BitAccess, NULL, settings, "VkBool32", "storageBuffer8BitAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.uniformAndStorageBuffer8BitAccess, NULL, settings, "VkBool32", "uniformAndStorageBuffer8BitAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.storagePushConstant8, NULL, settings, "VkBool32", "storagePushConstant8", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDriverProperties(const VkPhysicalDeviceDriverProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDriverId>(object.driverID, NULL, settings, "VkDriverId", "driverID", false, false, indents + 1, dump_json_VkDriverId);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.driverName, NULL, settings, "char[VK_MAX_DRIVER_NAME_SIZE]", "driverName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.driverInfo, NULL, settings, "char[VK_MAX_DRIVER_INFO_SIZE]", "driverInfo", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const VkConformanceVersion>(object.conformanceVersion, NULL, settings, "VkConformanceVersion", "conformanceVersion", true, false, indents + 1, dump_json_VkConformanceVersion);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderAtomicInt64Features(const VkPhysicalDeviceShaderAtomicInt64Features& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderBufferInt64Atomics, NULL, settings, "VkBool32", "shaderBufferInt64Atomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSharedInt64Atomics, NULL, settings, "VkBool32", "shaderSharedInt64Atomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderFloat16Int8Features(const VkPhysicalDeviceShaderFloat16Int8Features& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderFloat16, NULL, settings, "VkBool32", "shaderFloat16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderInt8, NULL, settings, "VkBool32", "shaderInt8", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFloatControlsProperties(const VkPhysicalDeviceFloatControlsProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkShaderFloatControlsIndependence>(object.denormBehaviorIndependence, NULL, settings, "VkShaderFloatControlsIndependence", "denormBehaviorIndependence", false, false, indents + 1, dump_json_VkShaderFloatControlsIndependence);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderFloatControlsIndependence>(object.roundingModeIndependence, NULL, settings, "VkShaderFloatControlsIndependence", "roundingModeIndependence", false, false, indents + 1, dump_json_VkShaderFloatControlsIndependence);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSignedZeroInfNanPreserveFloat16, NULL, settings, "VkBool32", "shaderSignedZeroInfNanPreserveFloat16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSignedZeroInfNanPreserveFloat32, NULL, settings, "VkBool32", "shaderSignedZeroInfNanPreserveFloat32", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSignedZeroInfNanPreserveFloat64, NULL, settings, "VkBool32", "shaderSignedZeroInfNanPreserveFloat64", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDenormPreserveFloat16, NULL, settings, "VkBool32", "shaderDenormPreserveFloat16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDenormPreserveFloat32, NULL, settings, "VkBool32", "shaderDenormPreserveFloat32", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDenormPreserveFloat64, NULL, settings, "VkBool32", "shaderDenormPreserveFloat64", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDenormFlushToZeroFloat16, NULL, settings, "VkBool32", "shaderDenormFlushToZeroFloat16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDenormFlushToZeroFloat32, NULL, settings, "VkBool32", "shaderDenormFlushToZeroFloat32", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDenormFlushToZeroFloat64, NULL, settings, "VkBool32", "shaderDenormFlushToZeroFloat64", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderRoundingModeRTEFloat16, NULL, settings, "VkBool32", "shaderRoundingModeRTEFloat16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderRoundingModeRTEFloat32, NULL, settings, "VkBool32", "shaderRoundingModeRTEFloat32", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderRoundingModeRTEFloat64, NULL, settings, "VkBool32", "shaderRoundingModeRTEFloat64", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderRoundingModeRTZFloat16, NULL, settings, "VkBool32", "shaderRoundingModeRTZFloat16", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderRoundingModeRTZFloat32, NULL, settings, "VkBool32", "shaderRoundingModeRTZFloat32", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderRoundingModeRTZFloat64, NULL, settings, "VkBool32", "shaderRoundingModeRTZFloat64", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorSetLayoutBindingFlagsCreateInfo(const VkDescriptorSetLayoutBindingFlagsCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.bindingCount, NULL, settings, "uint32_t", "bindingCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDescriptorBindingFlags>(object.pBindingFlags, object.bindingCount, settings, "const VkDescriptorBindingFlags*", "const VkDescriptorBindingFlags", "pBindingFlags", false, false, indents + 1, dump_json_VkDescriptorBindingFlags); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDescriptorIndexingFeatures(const VkPhysicalDeviceDescriptorIndexingFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderInputAttachmentArrayDynamicIndexing, NULL, settings, "VkBool32", "shaderInputAttachmentArrayDynamicIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderUniformTexelBufferArrayDynamicIndexing, NULL, settings, "VkBool32", "shaderUniformTexelBufferArrayDynamicIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageTexelBufferArrayDynamicIndexing, NULL, settings, "VkBool32", "shaderStorageTexelBufferArrayDynamicIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderUniformBufferArrayNonUniformIndexing, NULL, settings, "VkBool32", "shaderUniformBufferArrayNonUniformIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSampledImageArrayNonUniformIndexing, NULL, settings, "VkBool32", "shaderSampledImageArrayNonUniformIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageBufferArrayNonUniformIndexing, NULL, settings, "VkBool32", "shaderStorageBufferArrayNonUniformIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageImageArrayNonUniformIndexing, NULL, settings, "VkBool32", "shaderStorageImageArrayNonUniformIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderInputAttachmentArrayNonUniformIndexing, NULL, settings, "VkBool32", "shaderInputAttachmentArrayNonUniformIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderUniformTexelBufferArrayNonUniformIndexing, NULL, settings, "VkBool32", "shaderUniformTexelBufferArrayNonUniformIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageTexelBufferArrayNonUniformIndexing, NULL, settings, "VkBool32", "shaderStorageTexelBufferArrayNonUniformIndexing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingUniformBufferUpdateAfterBind, NULL, settings, "VkBool32", "descriptorBindingUniformBufferUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingSampledImageUpdateAfterBind, NULL, settings, "VkBool32", "descriptorBindingSampledImageUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingStorageImageUpdateAfterBind, NULL, settings, "VkBool32", "descriptorBindingStorageImageUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingStorageBufferUpdateAfterBind, NULL, settings, "VkBool32", "descriptorBindingStorageBufferUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingUniformTexelBufferUpdateAfterBind, NULL, settings, "VkBool32", "descriptorBindingUniformTexelBufferUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingStorageTexelBufferUpdateAfterBind, NULL, settings, "VkBool32", "descriptorBindingStorageTexelBufferUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingUpdateUnusedWhilePending, NULL, settings, "VkBool32", "descriptorBindingUpdateUnusedWhilePending", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingPartiallyBound, NULL, settings, "VkBool32", "descriptorBindingPartiallyBound", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingVariableDescriptorCount, NULL, settings, "VkBool32", "descriptorBindingVariableDescriptorCount", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.runtimeDescriptorArray, NULL, settings, "VkBool32", "runtimeDescriptorArray", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDescriptorIndexingProperties(const VkPhysicalDeviceDescriptorIndexingProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxUpdateAfterBindDescriptorsInAllPools, NULL, settings, "uint32_t", "maxUpdateAfterBindDescriptorsInAllPools", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderUniformBufferArrayNonUniformIndexingNative, NULL, settings, "VkBool32", "shaderUniformBufferArrayNonUniformIndexingNative", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSampledImageArrayNonUniformIndexingNative, NULL, settings, "VkBool32", "shaderSampledImageArrayNonUniformIndexingNative", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageBufferArrayNonUniformIndexingNative, NULL, settings, "VkBool32", "shaderStorageBufferArrayNonUniformIndexingNative", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderStorageImageArrayNonUniformIndexingNative, NULL, settings, "VkBool32", "shaderStorageImageArrayNonUniformIndexingNative", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderInputAttachmentArrayNonUniformIndexingNative, NULL, settings, "VkBool32", "shaderInputAttachmentArrayNonUniformIndexingNative", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.robustBufferAccessUpdateAfterBind, NULL, settings, "VkBool32", "robustBufferAccessUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.quadDivergentImplicitLod, NULL, settings, "VkBool32", "quadDivergentImplicitLod", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUpdateAfterBindSamplers, NULL, settings, "uint32_t", "maxPerStageDescriptorUpdateAfterBindSamplers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUpdateAfterBindUniformBuffers, NULL, settings, "uint32_t", "maxPerStageDescriptorUpdateAfterBindUniformBuffers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUpdateAfterBindStorageBuffers, NULL, settings, "uint32_t", "maxPerStageDescriptorUpdateAfterBindStorageBuffers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUpdateAfterBindSampledImages, NULL, settings, "uint32_t", "maxPerStageDescriptorUpdateAfterBindSampledImages", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUpdateAfterBindStorageImages, NULL, settings, "uint32_t", "maxPerStageDescriptorUpdateAfterBindStorageImages", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUpdateAfterBindInputAttachments, NULL, settings, "uint32_t", "maxPerStageDescriptorUpdateAfterBindInputAttachments", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageUpdateAfterBindResources, NULL, settings, "uint32_t", "maxPerStageUpdateAfterBindResources", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindSamplers, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindSamplers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindUniformBuffers, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindUniformBuffers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindUniformBuffersDynamic, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindUniformBuffersDynamic", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindStorageBuffers, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindStorageBuffers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindStorageBuffersDynamic, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindStorageBuffersDynamic", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindSampledImages, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindSampledImages", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindStorageImages, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindStorageImages", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindInputAttachments, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindInputAttachments", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorSetVariableDescriptorCountAllocateInfo(const VkDescriptorSetVariableDescriptorCountAllocateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.descriptorSetCount, NULL, settings, "uint32_t", "descriptorSetCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pDescriptorCounts, object.descriptorSetCount, settings, "const uint32_t*", "const uint32_t", "pDescriptorCounts", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorSetVariableDescriptorCountLayoutSupport(const VkDescriptorSetVariableDescriptorCountLayoutSupport& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxVariableDescriptorCount, NULL, settings, "uint32_t", "maxVariableDescriptorCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSubpassDescriptionDepthStencilResolve(const VkSubpassDescriptionDepthStencilResolve& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkResolveModeFlagBits>(object.depthResolveMode, NULL, settings, "VkResolveModeFlagBits", "depthResolveMode", false, false, indents + 1, dump_json_VkResolveModeFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const VkResolveModeFlagBits>(object.stencilResolveMode, NULL, settings, "VkResolveModeFlagBits", "stencilResolveMode", false, false, indents + 1, dump_json_VkResolveModeFlagBits);
    settings.stream() << ",\n";
    dump_json_pointer<const VkAttachmentReference2>(object.pDepthStencilResolveAttachment, settings, "const VkAttachmentReference2*", "pDepthStencilResolveAttachment", true, false, indents + 1, dump_json_VkAttachmentReference2);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDepthStencilResolveProperties(const VkPhysicalDeviceDepthStencilResolveProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkResolveModeFlags>(object.supportedDepthResolveModes, NULL, settings, "VkResolveModeFlags", "supportedDepthResolveModes", false, false, indents + 1, dump_json_VkResolveModeFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkResolveModeFlags>(object.supportedStencilResolveModes, NULL, settings, "VkResolveModeFlags", "supportedStencilResolveModes", false, false, indents + 1, dump_json_VkResolveModeFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.independentResolveNone, NULL, settings, "VkBool32", "independentResolveNone", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.independentResolve, NULL, settings, "VkBool32", "independentResolve", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceScalarBlockLayoutFeatures(const VkPhysicalDeviceScalarBlockLayoutFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.scalarBlockLayout, NULL, settings, "VkBool32", "scalarBlockLayout", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageStencilUsageCreateInfo(const VkImageStencilUsageCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageUsageFlags>(object.stencilUsage, NULL, settings, "VkImageUsageFlags", "stencilUsage", false, false, indents + 1, dump_json_VkImageUsageFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSamplerReductionModeCreateInfo(const VkSamplerReductionModeCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerReductionMode>(object.reductionMode, NULL, settings, "VkSamplerReductionMode", "reductionMode", false, false, indents + 1, dump_json_VkSamplerReductionMode);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSamplerFilterMinmaxProperties(const VkPhysicalDeviceSamplerFilterMinmaxProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.filterMinmaxSingleComponentFormats, NULL, settings, "VkBool32", "filterMinmaxSingleComponentFormats", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.filterMinmaxImageComponentMapping, NULL, settings, "VkBool32", "filterMinmaxImageComponentMapping", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceVulkanMemoryModelFeatures(const VkPhysicalDeviceVulkanMemoryModelFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.vulkanMemoryModel, NULL, settings, "VkBool32", "vulkanMemoryModel", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.vulkanMemoryModelDeviceScope, NULL, settings, "VkBool32", "vulkanMemoryModelDeviceScope", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.vulkanMemoryModelAvailabilityVisibilityChains, NULL, settings, "VkBool32", "vulkanMemoryModelAvailabilityVisibilityChains", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImagelessFramebufferFeatures(const VkPhysicalDeviceImagelessFramebufferFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.imagelessFramebuffer, NULL, settings, "VkBool32", "imagelessFramebuffer", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkFramebufferAttachmentImageInfo(const VkFramebufferAttachmentImageInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageCreateFlags>(object.flags, NULL, settings, "VkImageCreateFlags", "flags", false, false, indents + 1, dump_json_VkImageCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkImageUsageFlags>(object.usage, NULL, settings, "VkImageUsageFlags", "usage", false, false, indents + 1, dump_json_VkImageUsageFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.width, NULL, settings, "uint32_t", "width", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.height, NULL, settings, "uint32_t", "height", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.layerCount, NULL, settings, "uint32_t", "layerCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.viewFormatCount, NULL, settings, "uint32_t", "viewFormatCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkFormat>(object.pViewFormats, object.viewFormatCount, settings, "const VkFormat*", "const VkFormat", "pViewFormats", false, false, indents + 1, dump_json_VkFormat); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkFramebufferAttachmentsCreateInfo(const VkFramebufferAttachmentsCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.attachmentImageInfoCount, NULL, settings, "uint32_t", "attachmentImageInfoCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkFramebufferAttachmentImageInfo>(object.pAttachmentImageInfos, object.attachmentImageInfoCount, settings, "const VkFramebufferAttachmentImageInfo*", "const VkFramebufferAttachmentImageInfo", "pAttachmentImageInfos", true, false, indents + 1, dump_json_VkFramebufferAttachmentImageInfo); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassAttachmentBeginInfo(const VkRenderPassAttachmentBeginInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.attachmentCount, NULL, settings, "uint32_t", "attachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkImageView>(object.pAttachments, object.attachmentCount, settings, "const VkImageView*", "const VkImageView", "pAttachments", false, false, indents + 1, dump_json_VkImageView); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceUniformBufferStandardLayoutFeatures(const VkPhysicalDeviceUniformBufferStandardLayoutFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.uniformBufferStandardLayout, NULL, settings, "VkBool32", "uniformBufferStandardLayout", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderSubgroupExtendedTypesFeatures(const VkPhysicalDeviceShaderSubgroupExtendedTypesFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSubgroupExtendedTypes, NULL, settings, "VkBool32", "shaderSubgroupExtendedTypes", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSeparateDepthStencilLayoutsFeatures(const VkPhysicalDeviceSeparateDepthStencilLayoutsFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.separateDepthStencilLayouts, NULL, settings, "VkBool32", "separateDepthStencilLayouts", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAttachmentReferenceStencilLayout(const VkAttachmentReferenceStencilLayout& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.stencilLayout, NULL, settings, "VkImageLayout", "stencilLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAttachmentDescriptionStencilLayout(const VkAttachmentDescriptionStencilLayout& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.stencilInitialLayout, NULL, settings, "VkImageLayout", "stencilInitialLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.stencilFinalLayout, NULL, settings, "VkImageLayout", "stencilFinalLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceHostQueryResetFeatures(const VkPhysicalDeviceHostQueryResetFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.hostQueryReset, NULL, settings, "VkBool32", "hostQueryReset", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceTimelineSemaphoreFeatures(const VkPhysicalDeviceTimelineSemaphoreFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.timelineSemaphore, NULL, settings, "VkBool32", "timelineSemaphore", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceTimelineSemaphoreProperties(const VkPhysicalDeviceTimelineSemaphoreProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.maxTimelineSemaphoreValueDifference, NULL, settings, "uint64_t", "maxTimelineSemaphoreValueDifference", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSemaphoreTypeCreateInfo(const VkSemaphoreTypeCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphoreType>(object.semaphoreType, NULL, settings, "VkSemaphoreType", "semaphoreType", false, false, indents + 1, dump_json_VkSemaphoreType);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.initialValue, NULL, settings, "uint64_t", "initialValue", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkTimelineSemaphoreSubmitInfo(const VkTimelineSemaphoreSubmitInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.waitSemaphoreValueCount, NULL, settings, "uint32_t", "waitSemaphoreValueCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint64_t>(object.pWaitSemaphoreValues, object.waitSemaphoreValueCount, settings, "const uint64_t*", "const uint64_t", "pWaitSemaphoreValues", false, false, indents + 1, dump_json_uint64_t); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.signalSemaphoreValueCount, NULL, settings, "uint32_t", "signalSemaphoreValueCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint64_t>(object.pSignalSemaphoreValues, object.signalSemaphoreValueCount, settings, "const uint64_t*", "const uint64_t", "pSignalSemaphoreValues", false, false, indents + 1, dump_json_uint64_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSemaphoreWaitInfo(const VkSemaphoreWaitInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphoreWaitFlags>(object.flags, NULL, settings, "VkSemaphoreWaitFlags", "flags", false, false, indents + 1, dump_json_VkSemaphoreWaitFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.semaphoreCount, NULL, settings, "uint32_t", "semaphoreCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSemaphore>(object.pSemaphores, object.semaphoreCount, settings, "const VkSemaphore*", "const VkSemaphore", "pSemaphores", false, false, indents + 1, dump_json_VkSemaphore); // JQA
    settings.stream() << ",\n";
    dump_json_array<const uint64_t>(object.pValues, object.semaphoreCount, settings, "const uint64_t*", "const uint64_t", "pValues", false, false, indents + 1, dump_json_uint64_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSemaphoreSignalInfo(const VkSemaphoreSignalInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphore>(object.semaphore, NULL, settings, "VkSemaphore", "semaphore", false, false, indents + 1, dump_json_VkSemaphore);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.value, NULL, settings, "uint64_t", "value", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceBufferDeviceAddressFeatures(const VkPhysicalDeviceBufferDeviceAddressFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.bufferDeviceAddress, NULL, settings, "VkBool32", "bufferDeviceAddress", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.bufferDeviceAddressCaptureReplay, NULL, settings, "VkBool32", "bufferDeviceAddressCaptureReplay", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.bufferDeviceAddressMultiDevice, NULL, settings, "VkBool32", "bufferDeviceAddressMultiDevice", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBufferDeviceAddressInfo(const VkBufferDeviceAddressInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBufferOpaqueCaptureAddressCreateInfo(const VkBufferOpaqueCaptureAddressCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.opaqueCaptureAddress, NULL, settings, "uint64_t", "opaqueCaptureAddress", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryOpaqueCaptureAddressAllocateInfo(const VkMemoryOpaqueCaptureAddressAllocateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.opaqueCaptureAddress, NULL, settings, "uint64_t", "opaqueCaptureAddress", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceMemoryOpaqueCaptureAddressInfo(const VkDeviceMemoryOpaqueCaptureAddressInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceVulkan13Features(const VkPhysicalDeviceVulkan13Features& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.robustImageAccess, NULL, settings, "VkBool32", "robustImageAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.inlineUniformBlock, NULL, settings, "VkBool32", "inlineUniformBlock", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingInlineUniformBlockUpdateAfterBind, NULL, settings, "VkBool32", "descriptorBindingInlineUniformBlockUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.pipelineCreationCacheControl, NULL, settings, "VkBool32", "pipelineCreationCacheControl", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.privateData, NULL, settings, "VkBool32", "privateData", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDemoteToHelperInvocation, NULL, settings, "VkBool32", "shaderDemoteToHelperInvocation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderTerminateInvocation, NULL, settings, "VkBool32", "shaderTerminateInvocation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.subgroupSizeControl, NULL, settings, "VkBool32", "subgroupSizeControl", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.computeFullSubgroups, NULL, settings, "VkBool32", "computeFullSubgroups", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.synchronization2, NULL, settings, "VkBool32", "synchronization2", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.textureCompressionASTC_HDR, NULL, settings, "VkBool32", "textureCompressionASTC_HDR", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderZeroInitializeWorkgroupMemory, NULL, settings, "VkBool32", "shaderZeroInitializeWorkgroupMemory", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.dynamicRendering, NULL, settings, "VkBool32", "dynamicRendering", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderIntegerDotProduct, NULL, settings, "VkBool32", "shaderIntegerDotProduct", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.maintenance4, NULL, settings, "VkBool32", "maintenance4", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceVulkan13Properties(const VkPhysicalDeviceVulkan13Properties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minSubgroupSize, NULL, settings, "uint32_t", "minSubgroupSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxSubgroupSize, NULL, settings, "uint32_t", "maxSubgroupSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxComputeWorkgroupSubgroups, NULL, settings, "uint32_t", "maxComputeWorkgroupSubgroups", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlags>(object.requiredSubgroupSizeStages, NULL, settings, "VkShaderStageFlags", "requiredSubgroupSizeStages", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxInlineUniformBlockSize, NULL, settings, "uint32_t", "maxInlineUniformBlockSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorInlineUniformBlocks, NULL, settings, "uint32_t", "maxPerStageDescriptorInlineUniformBlocks", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUpdateAfterBindInlineUniformBlocks, NULL, settings, "uint32_t", "maxPerStageDescriptorUpdateAfterBindInlineUniformBlocks", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetInlineUniformBlocks, NULL, settings, "uint32_t", "maxDescriptorSetInlineUniformBlocks", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindInlineUniformBlocks, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindInlineUniformBlocks", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxInlineUniformTotalSize, NULL, settings, "uint32_t", "maxInlineUniformTotalSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct8BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct8BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct8BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct8BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct8BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProduct8BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct4x8BitPackedUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct4x8BitPackedUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct4x8BitPackedSignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct4x8BitPackedSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct4x8BitPackedMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProduct4x8BitPackedMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct16BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct16BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct16BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct16BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct16BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProduct16BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct32BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct32BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct32BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct32BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct32BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProduct32BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct64BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct64BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct64BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct64BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct64BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProduct64BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating8BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating8BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating8BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating8BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating8BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating8BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating4x8BitPackedUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating4x8BitPackedUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating4x8BitPackedSignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating4x8BitPackedSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating4x8BitPackedMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating4x8BitPackedMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating16BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating16BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating16BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating16BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating16BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating16BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating32BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating32BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating32BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating32BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating32BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating32BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating64BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating64BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating64BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating64BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating64BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating64BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.storageTexelBufferOffsetAlignmentBytes, NULL, settings, "VkDeviceSize", "storageTexelBufferOffsetAlignmentBytes", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.storageTexelBufferOffsetSingleTexelAlignment, NULL, settings, "VkBool32", "storageTexelBufferOffsetSingleTexelAlignment", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.uniformTexelBufferOffsetAlignmentBytes, NULL, settings, "VkDeviceSize", "uniformTexelBufferOffsetAlignmentBytes", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.uniformTexelBufferOffsetSingleTexelAlignment, NULL, settings, "VkBool32", "uniformTexelBufferOffsetSingleTexelAlignment", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.maxBufferSize, NULL, settings, "VkDeviceSize", "maxBufferSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineCreationFeedback(const VkPipelineCreationFeedback& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkPipelineCreationFeedbackFlags>(object.flags, NULL, settings, "VkPipelineCreationFeedbackFlags", "flags", false, false, indents + 1, dump_json_VkPipelineCreationFeedbackFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.duration, NULL, settings, "uint64_t", "duration", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineCreationFeedbackCreateInfo(const VkPipelineCreationFeedbackCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const VkPipelineCreationFeedback>(object.pPipelineCreationFeedback, settings, "VkPipelineCreationFeedback*", "pPipelineCreationFeedback", true, false, indents + 1, dump_json_VkPipelineCreationFeedback);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.pipelineStageCreationFeedbackCount, NULL, settings, "uint32_t", "pipelineStageCreationFeedbackCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPipelineCreationFeedback>(object.pPipelineStageCreationFeedbacks, object.pipelineStageCreationFeedbackCount, settings, "VkPipelineCreationFeedback*", "VkPipelineCreationFeedback", "pPipelineStageCreationFeedbacks", true, false, indents + 1, dump_json_VkPipelineCreationFeedback); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderTerminateInvocationFeatures(const VkPhysicalDeviceShaderTerminateInvocationFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderTerminateInvocation, NULL, settings, "VkBool32", "shaderTerminateInvocation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceToolProperties(const VkPhysicalDeviceToolProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.name, NULL, settings, "char[VK_MAX_EXTENSION_NAME_SIZE]", "name", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.version, NULL, settings, "char[VK_MAX_EXTENSION_NAME_SIZE]", "version", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const VkToolPurposeFlags>(object.purposes, NULL, settings, "VkToolPurposeFlags", "purposes", false, false, indents + 1, dump_json_VkToolPurposeFlags);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.description, NULL, settings, "char[VK_MAX_DESCRIPTION_SIZE]", "description", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.layer, NULL, settings, "char[VK_MAX_EXTENSION_NAME_SIZE]", "layer", false, false, indents + 1, dump_json_cstring);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderDemoteToHelperInvocationFeatures(const VkPhysicalDeviceShaderDemoteToHelperInvocationFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDemoteToHelperInvocation, NULL, settings, "VkBool32", "shaderDemoteToHelperInvocation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePrivateDataFeatures(const VkPhysicalDevicePrivateDataFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.privateData, NULL, settings, "VkBool32", "privateData", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDevicePrivateDataCreateInfo(const VkDevicePrivateDataCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.privateDataSlotRequestCount, NULL, settings, "uint32_t", "privateDataSlotRequestCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPrivateDataSlotCreateInfo(const VkPrivateDataSlotCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPrivateDataSlotCreateFlags>(object.flags, NULL, settings, "VkPrivateDataSlotCreateFlags", "flags", false, false, indents + 1, dump_json_VkPrivateDataSlotCreateFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePipelineCreationCacheControlFeatures(const VkPhysicalDevicePipelineCreationCacheControlFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.pipelineCreationCacheControl, NULL, settings, "VkBool32", "pipelineCreationCacheControl", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryBarrier2(const VkMemoryBarrier2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineStageFlags2>(object.srcStageMask, NULL, settings, "VkPipelineStageFlags2", "srcStageMask", false, false, indents + 1, dump_json_VkPipelineStageFlags2);
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags2>(object.srcAccessMask, NULL, settings, "VkAccessFlags2", "srcAccessMask", false, false, indents + 1, dump_json_VkAccessFlags2);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineStageFlags2>(object.dstStageMask, NULL, settings, "VkPipelineStageFlags2", "dstStageMask", false, false, indents + 1, dump_json_VkPipelineStageFlags2);
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags2>(object.dstAccessMask, NULL, settings, "VkAccessFlags2", "dstAccessMask", false, false, indents + 1, dump_json_VkAccessFlags2);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBufferMemoryBarrier2(const VkBufferMemoryBarrier2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineStageFlags2>(object.srcStageMask, NULL, settings, "VkPipelineStageFlags2", "srcStageMask", false, false, indents + 1, dump_json_VkPipelineStageFlags2);
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags2>(object.srcAccessMask, NULL, settings, "VkAccessFlags2", "srcAccessMask", false, false, indents + 1, dump_json_VkAccessFlags2);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineStageFlags2>(object.dstStageMask, NULL, settings, "VkPipelineStageFlags2", "dstStageMask", false, false, indents + 1, dump_json_VkPipelineStageFlags2);
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags2>(object.dstAccessMask, NULL, settings, "VkAccessFlags2", "dstAccessMask", false, false, indents + 1, dump_json_VkAccessFlags2);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.srcQueueFamilyIndex, NULL, settings, "uint32_t", "srcQueueFamilyIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dstQueueFamilyIndex, NULL, settings, "uint32_t", "dstQueueFamilyIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.offset, NULL, settings, "VkDeviceSize", "offset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageMemoryBarrier2(const VkImageMemoryBarrier2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineStageFlags2>(object.srcStageMask, NULL, settings, "VkPipelineStageFlags2", "srcStageMask", false, false, indents + 1, dump_json_VkPipelineStageFlags2);
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags2>(object.srcAccessMask, NULL, settings, "VkAccessFlags2", "srcAccessMask", false, false, indents + 1, dump_json_VkAccessFlags2);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineStageFlags2>(object.dstStageMask, NULL, settings, "VkPipelineStageFlags2", "dstStageMask", false, false, indents + 1, dump_json_VkPipelineStageFlags2);
    settings.stream() << ",\n";
    dump_json_value<const VkAccessFlags2>(object.dstAccessMask, NULL, settings, "VkAccessFlags2", "dstAccessMask", false, false, indents + 1, dump_json_VkAccessFlags2);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.oldLayout, NULL, settings, "VkImageLayout", "oldLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.newLayout, NULL, settings, "VkImageLayout", "newLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.srcQueueFamilyIndex, NULL, settings, "uint32_t", "srcQueueFamilyIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dstQueueFamilyIndex, NULL, settings, "uint32_t", "dstQueueFamilyIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.image, NULL, settings, "VkImage", "image", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceRange>(object.subresourceRange, NULL, settings, "VkImageSubresourceRange", "subresourceRange", true, false, indents + 1, dump_json_VkImageSubresourceRange);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDependencyInfo(const VkDependencyInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDependencyFlags>(object.dependencyFlags, NULL, settings, "VkDependencyFlags", "dependencyFlags", false, false, indents + 1, dump_json_VkDependencyFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryBarrierCount, NULL, settings, "uint32_t", "memoryBarrierCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkMemoryBarrier2>(object.pMemoryBarriers, object.memoryBarrierCount, settings, "const VkMemoryBarrier2*", "const VkMemoryBarrier2", "pMemoryBarriers", true, false, indents + 1, dump_json_VkMemoryBarrier2); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.bufferMemoryBarrierCount, NULL, settings, "uint32_t", "bufferMemoryBarrierCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkBufferMemoryBarrier2>(object.pBufferMemoryBarriers, object.bufferMemoryBarrierCount, settings, "const VkBufferMemoryBarrier2*", "const VkBufferMemoryBarrier2", "pBufferMemoryBarriers", true, false, indents + 1, dump_json_VkBufferMemoryBarrier2); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.imageMemoryBarrierCount, NULL, settings, "uint32_t", "imageMemoryBarrierCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkImageMemoryBarrier2>(object.pImageMemoryBarriers, object.imageMemoryBarrierCount, settings, "const VkImageMemoryBarrier2*", "const VkImageMemoryBarrier2", "pImageMemoryBarriers", true, false, indents + 1, dump_json_VkImageMemoryBarrier2); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSemaphoreSubmitInfo(const VkSemaphoreSubmitInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphore>(object.semaphore, NULL, settings, "VkSemaphore", "semaphore", false, false, indents + 1, dump_json_VkSemaphore);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.value, NULL, settings, "uint64_t", "value", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineStageFlags2>(object.stageMask, NULL, settings, "VkPipelineStageFlags2", "stageMask", false, false, indents + 1, dump_json_VkPipelineStageFlags2);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceIndex, NULL, settings, "uint32_t", "deviceIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCommandBufferSubmitInfo(const VkCommandBufferSubmitInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkCommandBuffer>(object.commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, indents + 1, dump_json_VkCommandBuffer);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceMask, NULL, settings, "uint32_t", "deviceMask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSubmitInfo2(const VkSubmitInfo2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSubmitFlags>(object.flags, NULL, settings, "VkSubmitFlags", "flags", false, false, indents + 1, dump_json_VkSubmitFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.waitSemaphoreInfoCount, NULL, settings, "uint32_t", "waitSemaphoreInfoCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSemaphoreSubmitInfo>(object.pWaitSemaphoreInfos, object.waitSemaphoreInfoCount, settings, "const VkSemaphoreSubmitInfo*", "const VkSemaphoreSubmitInfo", "pWaitSemaphoreInfos", true, false, indents + 1, dump_json_VkSemaphoreSubmitInfo); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.commandBufferInfoCount, NULL, settings, "uint32_t", "commandBufferInfoCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkCommandBufferSubmitInfo>(object.pCommandBufferInfos, object.commandBufferInfoCount, settings, "const VkCommandBufferSubmitInfo*", "const VkCommandBufferSubmitInfo", "pCommandBufferInfos", true, false, indents + 1, dump_json_VkCommandBufferSubmitInfo); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.signalSemaphoreInfoCount, NULL, settings, "uint32_t", "signalSemaphoreInfoCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSemaphoreSubmitInfo>(object.pSignalSemaphoreInfos, object.signalSemaphoreInfoCount, settings, "const VkSemaphoreSubmitInfo*", "const VkSemaphoreSubmitInfo", "pSignalSemaphoreInfos", true, false, indents + 1, dump_json_VkSemaphoreSubmitInfo); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSynchronization2Features(const VkPhysicalDeviceSynchronization2Features& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.synchronization2, NULL, settings, "VkBool32", "synchronization2", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceZeroInitializeWorkgroupMemoryFeatures(const VkPhysicalDeviceZeroInitializeWorkgroupMemoryFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderZeroInitializeWorkgroupMemory, NULL, settings, "VkBool32", "shaderZeroInitializeWorkgroupMemory", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImageRobustnessFeatures(const VkPhysicalDeviceImageRobustnessFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.robustImageAccess, NULL, settings, "VkBool32", "robustImageAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBufferCopy2(const VkBufferCopy2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.srcOffset, NULL, settings, "VkDeviceSize", "srcOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.dstOffset, NULL, settings, "VkDeviceSize", "dstOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyBufferInfo2(const VkCopyBufferInfo2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.srcBuffer, NULL, settings, "VkBuffer", "srcBuffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.dstBuffer, NULL, settings, "VkBuffer", "dstBuffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.regionCount, NULL, settings, "uint32_t", "regionCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkBufferCopy2>(object.pRegions, object.regionCount, settings, "const VkBufferCopy2*", "const VkBufferCopy2", "pRegions", true, false, indents + 1, dump_json_VkBufferCopy2); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageCopy2(const VkImageCopy2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceLayers>(object.srcSubresource, NULL, settings, "VkImageSubresourceLayers", "srcSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset3D>(object.srcOffset, NULL, settings, "VkOffset3D", "srcOffset", true, false, indents + 1, dump_json_VkOffset3D);
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceLayers>(object.dstSubresource, NULL, settings, "VkImageSubresourceLayers", "dstSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset3D>(object.dstOffset, NULL, settings, "VkOffset3D", "dstOffset", true, false, indents + 1, dump_json_VkOffset3D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent3D>(object.extent, NULL, settings, "VkExtent3D", "extent", true, false, indents + 1, dump_json_VkExtent3D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyImageInfo2(const VkCopyImageInfo2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.srcImage, NULL, settings, "VkImage", "srcImage", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.srcImageLayout, NULL, settings, "VkImageLayout", "srcImageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.dstImage, NULL, settings, "VkImage", "dstImage", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.dstImageLayout, NULL, settings, "VkImageLayout", "dstImageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.regionCount, NULL, settings, "uint32_t", "regionCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkImageCopy2>(object.pRegions, object.regionCount, settings, "const VkImageCopy2*", "const VkImageCopy2", "pRegions", true, false, indents + 1, dump_json_VkImageCopy2); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBufferImageCopy2(const VkBufferImageCopy2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.bufferOffset, NULL, settings, "VkDeviceSize", "bufferOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.bufferRowLength, NULL, settings, "uint32_t", "bufferRowLength", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.bufferImageHeight, NULL, settings, "uint32_t", "bufferImageHeight", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceLayers>(object.imageSubresource, NULL, settings, "VkImageSubresourceLayers", "imageSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset3D>(object.imageOffset, NULL, settings, "VkOffset3D", "imageOffset", true, false, indents + 1, dump_json_VkOffset3D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent3D>(object.imageExtent, NULL, settings, "VkExtent3D", "imageExtent", true, false, indents + 1, dump_json_VkExtent3D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyBufferToImageInfo2(const VkCopyBufferToImageInfo2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.srcBuffer, NULL, settings, "VkBuffer", "srcBuffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.dstImage, NULL, settings, "VkImage", "dstImage", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.dstImageLayout, NULL, settings, "VkImageLayout", "dstImageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.regionCount, NULL, settings, "uint32_t", "regionCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkBufferImageCopy2>(object.pRegions, object.regionCount, settings, "const VkBufferImageCopy2*", "const VkBufferImageCopy2", "pRegions", true, false, indents + 1, dump_json_VkBufferImageCopy2); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyImageToBufferInfo2(const VkCopyImageToBufferInfo2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.srcImage, NULL, settings, "VkImage", "srcImage", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.srcImageLayout, NULL, settings, "VkImageLayout", "srcImageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.dstBuffer, NULL, settings, "VkBuffer", "dstBuffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.regionCount, NULL, settings, "uint32_t", "regionCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkBufferImageCopy2>(object.pRegions, object.regionCount, settings, "const VkBufferImageCopy2*", "const VkBufferImageCopy2", "pRegions", true, false, indents + 1, dump_json_VkBufferImageCopy2); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageBlit2(const VkImageBlit2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceLayers>(object.srcSubresource, NULL, settings, "VkImageSubresourceLayers", "srcSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_array<const VkOffset3D>(object.srcOffsets, 2, settings, "VkOffset3D[2]", "VkOffset3D", "srcOffsets", true, false, indents + 1, dump_json_VkOffset3D); // IQA
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceLayers>(object.dstSubresource, NULL, settings, "VkImageSubresourceLayers", "dstSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_array<const VkOffset3D>(object.dstOffsets, 2, settings, "VkOffset3D[2]", "VkOffset3D", "dstOffsets", true, false, indents + 1, dump_json_VkOffset3D); // IQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBlitImageInfo2(const VkBlitImageInfo2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.srcImage, NULL, settings, "VkImage", "srcImage", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.srcImageLayout, NULL, settings, "VkImageLayout", "srcImageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.dstImage, NULL, settings, "VkImage", "dstImage", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.dstImageLayout, NULL, settings, "VkImageLayout", "dstImageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.regionCount, NULL, settings, "uint32_t", "regionCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkImageBlit2>(object.pRegions, object.regionCount, settings, "const VkImageBlit2*", "const VkImageBlit2", "pRegions", true, false, indents + 1, dump_json_VkImageBlit2); // JQA
    settings.stream() << ",\n";
    dump_json_value<const VkFilter>(object.filter, NULL, settings, "VkFilter", "filter", false, false, indents + 1, dump_json_VkFilter);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageResolve2(const VkImageResolve2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceLayers>(object.srcSubresource, NULL, settings, "VkImageSubresourceLayers", "srcSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset3D>(object.srcOffset, NULL, settings, "VkOffset3D", "srcOffset", true, false, indents + 1, dump_json_VkOffset3D);
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceLayers>(object.dstSubresource, NULL, settings, "VkImageSubresourceLayers", "dstSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset3D>(object.dstOffset, NULL, settings, "VkOffset3D", "dstOffset", true, false, indents + 1, dump_json_VkOffset3D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent3D>(object.extent, NULL, settings, "VkExtent3D", "extent", true, false, indents + 1, dump_json_VkExtent3D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkResolveImageInfo2(const VkResolveImageInfo2& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.srcImage, NULL, settings, "VkImage", "srcImage", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.srcImageLayout, NULL, settings, "VkImageLayout", "srcImageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.dstImage, NULL, settings, "VkImage", "dstImage", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.dstImageLayout, NULL, settings, "VkImageLayout", "dstImageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.regionCount, NULL, settings, "uint32_t", "regionCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkImageResolve2>(object.pRegions, object.regionCount, settings, "const VkImageResolve2*", "const VkImageResolve2", "pRegions", true, false, indents + 1, dump_json_VkImageResolve2); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSubgroupSizeControlFeatures(const VkPhysicalDeviceSubgroupSizeControlFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.subgroupSizeControl, NULL, settings, "VkBool32", "subgroupSizeControl", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.computeFullSubgroups, NULL, settings, "VkBool32", "computeFullSubgroups", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSubgroupSizeControlProperties(const VkPhysicalDeviceSubgroupSizeControlProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minSubgroupSize, NULL, settings, "uint32_t", "minSubgroupSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxSubgroupSize, NULL, settings, "uint32_t", "maxSubgroupSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxComputeWorkgroupSubgroups, NULL, settings, "uint32_t", "maxComputeWorkgroupSubgroups", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlags>(object.requiredSubgroupSizeStages, NULL, settings, "VkShaderStageFlags", "requiredSubgroupSizeStages", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineShaderStageRequiredSubgroupSizeCreateInfo(const VkPipelineShaderStageRequiredSubgroupSizeCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.requiredSubgroupSize, NULL, settings, "uint32_t", "requiredSubgroupSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceInlineUniformBlockFeatures(const VkPhysicalDeviceInlineUniformBlockFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.inlineUniformBlock, NULL, settings, "VkBool32", "inlineUniformBlock", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingInlineUniformBlockUpdateAfterBind, NULL, settings, "VkBool32", "descriptorBindingInlineUniformBlockUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceInlineUniformBlockProperties(const VkPhysicalDeviceInlineUniformBlockProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxInlineUniformBlockSize, NULL, settings, "uint32_t", "maxInlineUniformBlockSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorInlineUniformBlocks, NULL, settings, "uint32_t", "maxPerStageDescriptorInlineUniformBlocks", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUpdateAfterBindInlineUniformBlocks, NULL, settings, "uint32_t", "maxPerStageDescriptorUpdateAfterBindInlineUniformBlocks", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetInlineUniformBlocks, NULL, settings, "uint32_t", "maxDescriptorSetInlineUniformBlocks", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindInlineUniformBlocks, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindInlineUniformBlocks", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkWriteDescriptorSetInlineUniformBlock(const VkWriteDescriptorSetInlineUniformBlock& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dataSize, NULL, settings, "uint32_t", "dataSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pData, NULL, settings, "const void*", "pData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorPoolInlineUniformBlockCreateInfo(const VkDescriptorPoolInlineUniformBlockCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxInlineUniformBlockBindings, NULL, settings, "uint32_t", "maxInlineUniformBlockBindings", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceTextureCompressionASTCHDRFeatures(const VkPhysicalDeviceTextureCompressionASTCHDRFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.textureCompressionASTC_HDR, NULL, settings, "VkBool32", "textureCompressionASTC_HDR", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderingAttachmentInfo(const VkRenderingAttachmentInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageView>(object.imageView, NULL, settings, "VkImageView", "imageView", false, false, indents + 1, dump_json_VkImageView);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.imageLayout, NULL, settings, "VkImageLayout", "imageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkResolveModeFlagBits>(object.resolveMode, NULL, settings, "VkResolveModeFlagBits", "resolveMode", false, false, indents + 1, dump_json_VkResolveModeFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const VkImageView>(object.resolveImageView, NULL, settings, "VkImageView", "resolveImageView", false, false, indents + 1, dump_json_VkImageView);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.resolveImageLayout, NULL, settings, "VkImageLayout", "resolveImageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkAttachmentLoadOp>(object.loadOp, NULL, settings, "VkAttachmentLoadOp", "loadOp", false, false, indents + 1, dump_json_VkAttachmentLoadOp);
    settings.stream() << ",\n";
    dump_json_value<const VkAttachmentStoreOp>(object.storeOp, NULL, settings, "VkAttachmentStoreOp", "storeOp", false, false, indents + 1, dump_json_VkAttachmentStoreOp);
    settings.stream() << ",\n";
    dump_json_value<const VkClearValue>(object.clearValue, NULL, settings, "VkClearValue", "clearValue", false, true, indents + 1, dump_json_VkClearValue);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderingInfo(const VkRenderingInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkRenderingFlags>(object.flags, NULL, settings, "VkRenderingFlags", "flags", false, false, indents + 1, dump_json_VkRenderingFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkRect2D>(object.renderArea, NULL, settings, "VkRect2D", "renderArea", true, false, indents + 1, dump_json_VkRect2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.layerCount, NULL, settings, "uint32_t", "layerCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.viewMask, NULL, settings, "uint32_t", "viewMask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.colorAttachmentCount, NULL, settings, "uint32_t", "colorAttachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkRenderingAttachmentInfo>(object.pColorAttachments, object.colorAttachmentCount, settings, "const VkRenderingAttachmentInfo*", "const VkRenderingAttachmentInfo", "pColorAttachments", true, false, indents + 1, dump_json_VkRenderingAttachmentInfo); // JQA
    settings.stream() << ",\n";
    dump_json_pointer<const VkRenderingAttachmentInfo>(object.pDepthAttachment, settings, "const VkRenderingAttachmentInfo*", "pDepthAttachment", true, false, indents + 1, dump_json_VkRenderingAttachmentInfo);
    settings.stream() << ",\n";
    dump_json_pointer<const VkRenderingAttachmentInfo>(object.pStencilAttachment, settings, "const VkRenderingAttachmentInfo*", "pStencilAttachment", true, false, indents + 1, dump_json_VkRenderingAttachmentInfo);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineRenderingCreateInfo(const VkPipelineRenderingCreateInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.viewMask, NULL, settings, "uint32_t", "viewMask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.colorAttachmentCount, NULL, settings, "uint32_t", "colorAttachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    if(!ApiDumpInstance::current().getIsGPLPreRasterOrFragmentShader())
    dump_json_array<const VkFormat>(object.pColorAttachmentFormats, object.colorAttachmentCount, settings, "const VkFormat*", "const VkFormat", "pColorAttachmentFormats", false, false, indents + 1, dump_json_VkFormat); // JQA
    else
        dump_json_UNUSED(settings, "const VkFormat*", "pColorAttachmentFormats", indents + 1);
    settings.stream() << ",\n";
    if(!ApiDumpInstance::current().getIsGPLPreRasterOrFragmentShader())
    dump_json_value<const VkFormat>(object.depthAttachmentFormat, NULL, settings, "VkFormat", "depthAttachmentFormat", false, false, indents + 1, dump_json_VkFormat);
    else
        dump_json_UNUSED(settings, "VkFormat", "depthAttachmentFormat", indents + 1);
    settings.stream() << ",\n";
    if(!ApiDumpInstance::current().getIsGPLPreRasterOrFragmentShader())
    dump_json_value<const VkFormat>(object.stencilAttachmentFormat, NULL, settings, "VkFormat", "stencilAttachmentFormat", false, false, indents + 1, dump_json_VkFormat);
    else
        dump_json_UNUSED(settings, "VkFormat", "stencilAttachmentFormat", indents + 1);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDynamicRenderingFeatures(const VkPhysicalDeviceDynamicRenderingFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.dynamicRendering, NULL, settings, "VkBool32", "dynamicRendering", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCommandBufferInheritanceRenderingInfo(const VkCommandBufferInheritanceRenderingInfo& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkRenderingFlags>(object.flags, NULL, settings, "VkRenderingFlags", "flags", false, false, indents + 1, dump_json_VkRenderingFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.viewMask, NULL, settings, "uint32_t", "viewMask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.colorAttachmentCount, NULL, settings, "uint32_t", "colorAttachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkFormat>(object.pColorAttachmentFormats, object.colorAttachmentCount, settings, "const VkFormat*", "const VkFormat", "pColorAttachmentFormats", false, false, indents + 1, dump_json_VkFormat); // JQA
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.depthAttachmentFormat, NULL, settings, "VkFormat", "depthAttachmentFormat", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.stencilAttachmentFormat, NULL, settings, "VkFormat", "stencilAttachmentFormat", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlagBits>(object.rasterizationSamples, NULL, settings, "VkSampleCountFlagBits", "rasterizationSamples", false, false, indents + 1, dump_json_VkSampleCountFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderIntegerDotProductFeatures(const VkPhysicalDeviceShaderIntegerDotProductFeatures& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderIntegerDotProduct, NULL, settings, "VkBool32", "shaderIntegerDotProduct", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderIntegerDotProductProperties(const VkPhysicalDeviceShaderIntegerDotProductProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct8BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct8BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct8BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct8BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct8BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProduct8BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct4x8BitPackedUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct4x8BitPackedUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct4x8BitPackedSignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct4x8BitPackedSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct4x8BitPackedMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProduct4x8BitPackedMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct16BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct16BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct16BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct16BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct16BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProduct16BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct32BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct32BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct32BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct32BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct32BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProduct32BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct64BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct64BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct64BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProduct64BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProduct64BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProduct64BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating8BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating8BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating8BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating8BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating8BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating8BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating4x8BitPackedUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating4x8BitPackedUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating4x8BitPackedSignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating4x8BitPackedSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating4x8BitPackedMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating4x8BitPackedMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating16BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating16BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating16BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating16BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating16BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating16BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating32BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating32BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating32BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating32BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating32BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating32BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating64BitUnsignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating64BitUnsignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating64BitSignedAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating64BitSignedAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.integerDotProductAccumulatingSaturating64BitMixedSignednessAccelerated, NULL, settings, "VkBool32", "integerDotProductAccumulatingSaturating64BitMixedSignednessAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceTexelBufferAlignmentProperties(const VkPhysicalDeviceTexelBufferAlignmentProperties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.storageTexelBufferOffsetAlignmentBytes, NULL, settings, "VkDeviceSize", "storageTexelBufferOffsetAlignmentBytes", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.storageTexelBufferOffsetSingleTexelAlignment, NULL, settings, "VkBool32", "storageTexelBufferOffsetSingleTexelAlignment", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.uniformTexelBufferOffsetAlignmentBytes, NULL, settings, "VkDeviceSize", "uniformTexelBufferOffsetAlignmentBytes", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.uniformTexelBufferOffsetSingleTexelAlignment, NULL, settings, "VkBool32", "uniformTexelBufferOffsetSingleTexelAlignment", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkFormatProperties3(const VkFormatProperties3& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFormatFeatureFlags2>(object.linearTilingFeatures, NULL, settings, "VkFormatFeatureFlags2", "linearTilingFeatures", false, false, indents + 1, dump_json_VkFormatFeatureFlags2);
    settings.stream() << ",\n";
    dump_json_value<const VkFormatFeatureFlags2>(object.optimalTilingFeatures, NULL, settings, "VkFormatFeatureFlags2", "optimalTilingFeatures", false, false, indents + 1, dump_json_VkFormatFeatureFlags2);
    settings.stream() << ",\n";
    dump_json_value<const VkFormatFeatureFlags2>(object.bufferFeatures, NULL, settings, "VkFormatFeatureFlags2", "bufferFeatures", false, false, indents + 1, dump_json_VkFormatFeatureFlags2);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMaintenance4Features(const VkPhysicalDeviceMaintenance4Features& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.maintenance4, NULL, settings, "VkBool32", "maintenance4", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMaintenance4Properties(const VkPhysicalDeviceMaintenance4Properties& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.maxBufferSize, NULL, settings, "VkDeviceSize", "maxBufferSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceBufferMemoryRequirements(const VkDeviceBufferMemoryRequirements& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const VkBufferCreateInfo>(object.pCreateInfo, settings, "const VkBufferCreateInfo*", "pCreateInfo", true, false, indents + 1, dump_json_VkBufferCreateInfo);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceImageMemoryRequirements(const VkDeviceImageMemoryRequirements& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const VkImageCreateInfo>(object.pCreateInfo, settings, "const VkImageCreateInfo*", "pCreateInfo", true, false, indents + 1, dump_json_VkImageCreateInfo);
    settings.stream() << ",\n";
    dump_json_value<const VkImageAspectFlagBits>(object.planeAspect, NULL, settings, "VkImageAspectFlagBits", "planeAspect", false, false, indents + 1, dump_json_VkImageAspectFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSurfaceCapabilitiesKHR(const VkSurfaceCapabilitiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.minImageCount, NULL, settings, "uint32_t", "minImageCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxImageCount, NULL, settings, "uint32_t", "maxImageCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.currentExtent, NULL, settings, "VkExtent2D", "currentExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.minImageExtent, NULL, settings, "VkExtent2D", "minImageExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxImageExtent, NULL, settings, "VkExtent2D", "maxImageExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxImageArrayLayers, NULL, settings, "uint32_t", "maxImageArrayLayers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceTransformFlagsKHR>(object.supportedTransforms, NULL, settings, "VkSurfaceTransformFlagsKHR", "supportedTransforms", false, false, indents + 1, dump_json_VkSurfaceTransformFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceTransformFlagBitsKHR>(object.currentTransform, NULL, settings, "VkSurfaceTransformFlagBitsKHR", "currentTransform", false, false, indents + 1, dump_json_VkSurfaceTransformFlagBitsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkCompositeAlphaFlagsKHR>(object.supportedCompositeAlpha, NULL, settings, "VkCompositeAlphaFlagsKHR", "supportedCompositeAlpha", false, false, indents + 1, dump_json_VkCompositeAlphaFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkImageUsageFlags>(object.supportedUsageFlags, NULL, settings, "VkImageUsageFlags", "supportedUsageFlags", false, false, indents + 1, dump_json_VkImageUsageFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSurfaceFormatKHR(const VkSurfaceFormatKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkColorSpaceKHR>(object.colorSpace, NULL, settings, "VkColorSpaceKHR", "colorSpace", false, false, indents + 1, dump_json_VkColorSpaceKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSwapchainCreateInfoKHR(const VkSwapchainCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSwapchainCreateFlagsKHR>(object.flags, NULL, settings, "VkSwapchainCreateFlagsKHR", "flags", false, false, indents + 1, dump_json_VkSwapchainCreateFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceKHR>(object.surface, NULL, settings, "VkSurfaceKHR", "surface", false, false, indents + 1, dump_json_VkSurfaceKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minImageCount, NULL, settings, "uint32_t", "minImageCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.imageFormat, NULL, settings, "VkFormat", "imageFormat", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkColorSpaceKHR>(object.imageColorSpace, NULL, settings, "VkColorSpaceKHR", "imageColorSpace", false, false, indents + 1, dump_json_VkColorSpaceKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.imageExtent, NULL, settings, "VkExtent2D", "imageExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.imageArrayLayers, NULL, settings, "uint32_t", "imageArrayLayers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkImageUsageFlags>(object.imageUsage, NULL, settings, "VkImageUsageFlags", "imageUsage", false, false, indents + 1, dump_json_VkImageUsageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSharingMode>(object.imageSharingMode, NULL, settings, "VkSharingMode", "imageSharingMode", false, false, indents + 1, dump_json_VkSharingMode);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queueFamilyIndexCount, NULL, settings, "uint32_t", "queueFamilyIndexCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    if(object.imageSharingMode == VK_SHARING_MODE_CONCURRENT)
    dump_json_array<const uint32_t>(object.pQueueFamilyIndices, object.queueFamilyIndexCount, settings, "const uint32_t*", "const uint32_t", "pQueueFamilyIndices", false, false, indents + 1, dump_json_uint32_t); // JQA
    else
        dump_json_UNUSED(settings, "const uint32_t*", "pQueueFamilyIndices", indents + 1);
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceTransformFlagBitsKHR>(object.preTransform, NULL, settings, "VkSurfaceTransformFlagBitsKHR", "preTransform", false, false, indents + 1, dump_json_VkSurfaceTransformFlagBitsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkCompositeAlphaFlagBitsKHR>(object.compositeAlpha, NULL, settings, "VkCompositeAlphaFlagBitsKHR", "compositeAlpha", false, false, indents + 1, dump_json_VkCompositeAlphaFlagBitsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkPresentModeKHR>(object.presentMode, NULL, settings, "VkPresentModeKHR", "presentMode", false, false, indents + 1, dump_json_VkPresentModeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.clipped, NULL, settings, "VkBool32", "clipped", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkSwapchainKHR>(object.oldSwapchain, NULL, settings, "VkSwapchainKHR", "oldSwapchain", false, false, indents + 1, dump_json_VkSwapchainKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPresentInfoKHR(const VkPresentInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.waitSemaphoreCount, NULL, settings, "uint32_t", "waitSemaphoreCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSemaphore>(object.pWaitSemaphores, object.waitSemaphoreCount, settings, "const VkSemaphore*", "const VkSemaphore", "pWaitSemaphores", false, false, indents + 1, dump_json_VkSemaphore); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.swapchainCount, NULL, settings, "uint32_t", "swapchainCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSwapchainKHR>(object.pSwapchains, object.swapchainCount, settings, "const VkSwapchainKHR*", "const VkSwapchainKHR", "pSwapchains", false, false, indents + 1, dump_json_VkSwapchainKHR); // JQA
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pImageIndices, object.swapchainCount, settings, "const uint32_t*", "const uint32_t", "pImageIndices", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << ",\n";
    dump_json_array<const VkResult>(object.pResults, object.swapchainCount, settings, "VkResult*", "VkResult", "pResults", false, false, indents + 1, dump_json_VkResult); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageSwapchainCreateInfoKHR(const VkImageSwapchainCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSwapchainKHR>(object.swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, indents + 1, dump_json_VkSwapchainKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindImageMemorySwapchainInfoKHR(const VkBindImageMemorySwapchainInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSwapchainKHR>(object.swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, indents + 1, dump_json_VkSwapchainKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.imageIndex, NULL, settings, "uint32_t", "imageIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAcquireNextImageInfoKHR(const VkAcquireNextImageInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSwapchainKHR>(object.swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, indents + 1, dump_json_VkSwapchainKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.timeout, NULL, settings, "uint64_t", "timeout", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphore>(object.semaphore, NULL, settings, "VkSemaphore", "semaphore", false, false, indents + 1, dump_json_VkSemaphore);
    settings.stream() << ",\n";
    dump_json_value<const VkFence>(object.fence, NULL, settings, "VkFence", "fence", false, false, indents + 1, dump_json_VkFence);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceMask, NULL, settings, "uint32_t", "deviceMask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceGroupPresentCapabilitiesKHR(const VkDeviceGroupPresentCapabilitiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.presentMask, 32, settings, "uint32_t[VK_MAX_DEVICE_GROUP_SIZE]", "uint32_t", "presentMask", false, false, indents + 1, dump_json_uint32_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceGroupPresentModeFlagsKHR>(object.modes, NULL, settings, "VkDeviceGroupPresentModeFlagsKHR", "modes", false, false, indents + 1, dump_json_VkDeviceGroupPresentModeFlagsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceGroupPresentInfoKHR(const VkDeviceGroupPresentInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.swapchainCount, NULL, settings, "uint32_t", "swapchainCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pDeviceMasks, object.swapchainCount, settings, "const uint32_t*", "const uint32_t", "pDeviceMasks", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceGroupPresentModeFlagBitsKHR>(object.mode, NULL, settings, "VkDeviceGroupPresentModeFlagBitsKHR", "mode", false, false, indents + 1, dump_json_VkDeviceGroupPresentModeFlagBitsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceGroupSwapchainCreateInfoKHR(const VkDeviceGroupSwapchainCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceGroupPresentModeFlagsKHR>(object.modes, NULL, settings, "VkDeviceGroupPresentModeFlagsKHR", "modes", false, false, indents + 1, dump_json_VkDeviceGroupPresentModeFlagsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplayModeParametersKHR(const VkDisplayModeParametersKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkExtent2D>(object.visibleRegion, NULL, settings, "VkExtent2D", "visibleRegion", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.refreshRate, NULL, settings, "uint32_t", "refreshRate", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplayModeCreateInfoKHR(const VkDisplayModeCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDisplayModeCreateFlagsKHR>(object.flags, NULL, settings, "VkDisplayModeCreateFlagsKHR", "flags", false, false, indents + 1, dump_json_VkDisplayModeCreateFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDisplayModeParametersKHR>(object.parameters, NULL, settings, "VkDisplayModeParametersKHR", "parameters", true, false, indents + 1, dump_json_VkDisplayModeParametersKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplayModePropertiesKHR(const VkDisplayModePropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDisplayModeKHR>(object.displayMode, NULL, settings, "VkDisplayModeKHR", "displayMode", false, false, indents + 1, dump_json_VkDisplayModeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDisplayModeParametersKHR>(object.parameters, NULL, settings, "VkDisplayModeParametersKHR", "parameters", true, false, indents + 1, dump_json_VkDisplayModeParametersKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplayPlaneCapabilitiesKHR(const VkDisplayPlaneCapabilitiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDisplayPlaneAlphaFlagsKHR>(object.supportedAlpha, NULL, settings, "VkDisplayPlaneAlphaFlagsKHR", "supportedAlpha", false, false, indents + 1, dump_json_VkDisplayPlaneAlphaFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset2D>(object.minSrcPosition, NULL, settings, "VkOffset2D", "minSrcPosition", true, false, indents + 1, dump_json_VkOffset2D);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset2D>(object.maxSrcPosition, NULL, settings, "VkOffset2D", "maxSrcPosition", true, false, indents + 1, dump_json_VkOffset2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.minSrcExtent, NULL, settings, "VkExtent2D", "minSrcExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxSrcExtent, NULL, settings, "VkExtent2D", "maxSrcExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset2D>(object.minDstPosition, NULL, settings, "VkOffset2D", "minDstPosition", true, false, indents + 1, dump_json_VkOffset2D);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset2D>(object.maxDstPosition, NULL, settings, "VkOffset2D", "maxDstPosition", true, false, indents + 1, dump_json_VkOffset2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.minDstExtent, NULL, settings, "VkExtent2D", "minDstExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxDstExtent, NULL, settings, "VkExtent2D", "maxDstExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplayPlanePropertiesKHR(const VkDisplayPlanePropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDisplayKHR>(object.currentDisplay, NULL, settings, "VkDisplayKHR", "currentDisplay", false, false, indents + 1, dump_json_VkDisplayKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.currentStackIndex, NULL, settings, "uint32_t", "currentStackIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplayPropertiesKHR(const VkDisplayPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDisplayKHR>(object.display, NULL, settings, "VkDisplayKHR", "display", false, false, indents + 1, dump_json_VkDisplayKHR);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.displayName, NULL, settings, "const char*", "displayName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.physicalDimensions, NULL, settings, "VkExtent2D", "physicalDimensions", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.physicalResolution, NULL, settings, "VkExtent2D", "physicalResolution", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceTransformFlagsKHR>(object.supportedTransforms, NULL, settings, "VkSurfaceTransformFlagsKHR", "supportedTransforms", false, false, indents + 1, dump_json_VkSurfaceTransformFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.planeReorderPossible, NULL, settings, "VkBool32", "planeReorderPossible", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.persistentContent, NULL, settings, "VkBool32", "persistentContent", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplaySurfaceCreateInfoKHR(const VkDisplaySurfaceCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDisplaySurfaceCreateFlagsKHR>(object.flags, NULL, settings, "VkDisplaySurfaceCreateFlagsKHR", "flags", false, false, indents + 1, dump_json_VkDisplaySurfaceCreateFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDisplayModeKHR>(object.displayMode, NULL, settings, "VkDisplayModeKHR", "displayMode", false, false, indents + 1, dump_json_VkDisplayModeKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.planeIndex, NULL, settings, "uint32_t", "planeIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.planeStackIndex, NULL, settings, "uint32_t", "planeStackIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceTransformFlagBitsKHR>(object.transform, NULL, settings, "VkSurfaceTransformFlagBitsKHR", "transform", false, false, indents + 1, dump_json_VkSurfaceTransformFlagBitsKHR);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.globalAlpha, NULL, settings, "float", "globalAlpha", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const VkDisplayPlaneAlphaFlagBitsKHR>(object.alphaMode, NULL, settings, "VkDisplayPlaneAlphaFlagBitsKHR", "alphaMode", false, false, indents + 1, dump_json_VkDisplayPlaneAlphaFlagBitsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.imageExtent, NULL, settings, "VkExtent2D", "imageExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplayPresentInfoKHR(const VkDisplayPresentInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkRect2D>(object.srcRect, NULL, settings, "VkRect2D", "srcRect", true, false, indents + 1, dump_json_VkRect2D);
    settings.stream() << ",\n";
    dump_json_value<const VkRect2D>(object.dstRect, NULL, settings, "VkRect2D", "dstRect", true, false, indents + 1, dump_json_VkRect2D);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.persistent, NULL, settings, "VkBool32", "persistent", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_XLIB_KHR)
void dump_json_VkXlibSurfaceCreateInfoKHR(const VkXlibSurfaceCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkXlibSurfaceCreateFlagsKHR>(object.flags, NULL, settings, "VkXlibSurfaceCreateFlagsKHR", "flags", false, false, indents + 1, dump_json_VkXlibSurfaceCreateFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const Display*>(object.dpy, NULL, settings, "Display*", "dpy", false, false, indents + 1, dump_json_Display);
    settings.stream() << ",\n";
    dump_json_value<const Window>(object.window, NULL, settings, "Window", "window", false, false, indents + 1, dump_json_Window);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_XLIB_KHR
#if defined(VK_USE_PLATFORM_XCB_KHR)
void dump_json_VkXcbSurfaceCreateInfoKHR(const VkXcbSurfaceCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkXcbSurfaceCreateFlagsKHR>(object.flags, NULL, settings, "VkXcbSurfaceCreateFlagsKHR", "flags", false, false, indents + 1, dump_json_VkXcbSurfaceCreateFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const xcb_connection_t*>(object.connection, NULL, settings, "xcb_connection_t*", "connection", false, false, indents + 1, dump_json_xcb_connection_t);
    settings.stream() << ",\n";
    dump_json_value<const xcb_window_t>(object.window, NULL, settings, "xcb_window_t", "window", false, false, indents + 1, dump_json_xcb_window_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_XCB_KHR
#if defined(VK_USE_PLATFORM_WAYLAND_KHR)
void dump_json_VkWaylandSurfaceCreateInfoKHR(const VkWaylandSurfaceCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkWaylandSurfaceCreateFlagsKHR>(object.flags, NULL, settings, "VkWaylandSurfaceCreateFlagsKHR", "flags", false, false, indents + 1, dump_json_VkWaylandSurfaceCreateFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const wl_display*>(object.display, NULL, settings, "struct wl_display*", "display", false, false, indents + 1, dump_json_wl_display);
    settings.stream() << ",\n";
    dump_json_value<const wl_surface*>(object.surface, NULL, settings, "struct wl_surface*", "surface", false, false, indents + 1, dump_json_wl_surface);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WAYLAND_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_VkAndroidSurfaceCreateInfoKHR(const VkAndroidSurfaceCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAndroidSurfaceCreateFlagsKHR>(object.flags, NULL, settings, "VkAndroidSurfaceCreateFlagsKHR", "flags", false, false, indents + 1, dump_json_VkAndroidSurfaceCreateFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const ANativeWindow*>(object.window, NULL, settings, "struct ANativeWindow*", "window", false, false, indents + 1, dump_json_ANativeWindow);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkWin32SurfaceCreateInfoKHR(const VkWin32SurfaceCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkWin32SurfaceCreateFlagsKHR>(object.flags, NULL, settings, "VkWin32SurfaceCreateFlagsKHR", "flags", false, false, indents + 1, dump_json_VkWin32SurfaceCreateFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const HINSTANCE>(object.hinstance, NULL, settings, "HINSTANCE", "hinstance", false, false, indents + 1, dump_json_HINSTANCE);
    settings.stream() << ",\n";
    dump_json_value<const HWND>(object.hwnd, NULL, settings, "HWND", "hwnd", false, false, indents + 1, dump_json_HWND);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
void dump_json_VkDebugReportCallbackCreateInfoEXT(const VkDebugReportCallbackCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDebugReportFlagsEXT>(object.flags, NULL, settings, "VkDebugReportFlagsEXT", "flags", false, false, indents + 1, dump_json_VkDebugReportFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const PFN_vkDebugReportCallbackEXT>(object.pfnCallback, NULL, settings, "PFN_vkDebugReportCallbackEXT", "pfnCallback", false, false, indents + 1, dump_json_PFN_vkDebugReportCallbackEXT);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pUserData, NULL, settings, "void*", "pUserData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineRasterizationStateRasterizationOrderAMD(const VkPipelineRasterizationStateRasterizationOrderAMD& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkRasterizationOrderAMD>(object.rasterizationOrder, NULL, settings, "VkRasterizationOrderAMD", "rasterizationOrder", false, false, indents + 1, dump_json_VkRasterizationOrderAMD);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDebugMarkerObjectNameInfoEXT(const VkDebugMarkerObjectNameInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDebugReportObjectTypeEXT>(object.objectType, NULL, settings, "VkDebugReportObjectTypeEXT", "objectType", false, false, indents + 1, dump_json_VkDebugReportObjectTypeEXT);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.object, NULL, settings, "uint64_t", "object", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.pObjectName, NULL, settings, "const char*", "pObjectName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDebugMarkerObjectTagInfoEXT(const VkDebugMarkerObjectTagInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDebugReportObjectTypeEXT>(object.objectType, NULL, settings, "VkDebugReportObjectTypeEXT", "objectType", false, false, indents + 1, dump_json_VkDebugReportObjectTypeEXT);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.object, NULL, settings, "uint64_t", "object", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.tagName, NULL, settings, "uint64_t", "tagName", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.tagSize, NULL, settings, "size_t", "tagSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pTag, NULL, settings, "const void*", "pTag", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDebugMarkerMarkerInfoEXT(const VkDebugMarkerMarkerInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.pMarkerName, NULL, settings, "const char*", "pMarkerName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_array<const float>(object.color, 4, settings, "float[4]", "float", "color", false, false, indents + 1, dump_json_float); // IQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkQueueFamilyQueryResultStatusPropertiesKHR(const VkQueueFamilyQueryResultStatusPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.queryResultStatusSupport, NULL, settings, "VkBool32", "queryResultStatusSupport", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkQueueFamilyVideoPropertiesKHR(const VkQueueFamilyVideoPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoCodecOperationFlagsKHR>(object.videoCodecOperations, NULL, settings, "VkVideoCodecOperationFlagsKHR", "videoCodecOperations", false, false, indents + 1, dump_json_VkVideoCodecOperationFlagsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoProfileInfoKHR(const VkVideoProfileInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoCodecOperationFlagBitsKHR>(object.videoCodecOperation, NULL, settings, "VkVideoCodecOperationFlagBitsKHR", "videoCodecOperation", false, false, indents + 1, dump_json_VkVideoCodecOperationFlagBitsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoChromaSubsamplingFlagsKHR>(object.chromaSubsampling, NULL, settings, "VkVideoChromaSubsamplingFlagsKHR", "chromaSubsampling", false, false, indents + 1, dump_json_VkVideoChromaSubsamplingFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoComponentBitDepthFlagsKHR>(object.lumaBitDepth, NULL, settings, "VkVideoComponentBitDepthFlagsKHR", "lumaBitDepth", false, false, indents + 1, dump_json_VkVideoComponentBitDepthFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoComponentBitDepthFlagsKHR>(object.chromaBitDepth, NULL, settings, "VkVideoComponentBitDepthFlagsKHR", "chromaBitDepth", false, false, indents + 1, dump_json_VkVideoComponentBitDepthFlagsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoProfileListInfoKHR(const VkVideoProfileListInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.profileCount, NULL, settings, "uint32_t", "profileCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkVideoProfileInfoKHR>(object.pProfiles, object.profileCount, settings, "const VkVideoProfileInfoKHR*", "const VkVideoProfileInfoKHR", "pProfiles", true, false, indents + 1, dump_json_VkVideoProfileInfoKHR); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoCapabilitiesKHR(const VkVideoCapabilitiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoCapabilityFlagsKHR>(object.flags, NULL, settings, "VkVideoCapabilityFlagsKHR", "flags", false, false, indents + 1, dump_json_VkVideoCapabilityFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.minBitstreamBufferOffsetAlignment, NULL, settings, "VkDeviceSize", "minBitstreamBufferOffsetAlignment", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.minBitstreamBufferSizeAlignment, NULL, settings, "VkDeviceSize", "minBitstreamBufferSizeAlignment", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.pictureAccessGranularity, NULL, settings, "VkExtent2D", "pictureAccessGranularity", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.minCodedExtent, NULL, settings, "VkExtent2D", "minCodedExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxCodedExtent, NULL, settings, "VkExtent2D", "maxCodedExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDpbSlots, NULL, settings, "uint32_t", "maxDpbSlots", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxActiveReferencePictures, NULL, settings, "uint32_t", "maxActiveReferencePictures", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkExtensionProperties>(object.stdHeaderVersion, NULL, settings, "VkExtensionProperties", "stdHeaderVersion", true, false, indents + 1, dump_json_VkExtensionProperties);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceVideoFormatInfoKHR(const VkPhysicalDeviceVideoFormatInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageUsageFlags>(object.imageUsage, NULL, settings, "VkImageUsageFlags", "imageUsage", false, false, indents + 1, dump_json_VkImageUsageFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoFormatPropertiesKHR(const VkVideoFormatPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentMapping>(object.componentMapping, NULL, settings, "VkComponentMapping", "componentMapping", true, false, indents + 1, dump_json_VkComponentMapping);
    settings.stream() << ",\n";
    dump_json_value<const VkImageCreateFlags>(object.imageCreateFlags, NULL, settings, "VkImageCreateFlags", "imageCreateFlags", false, false, indents + 1, dump_json_VkImageCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkImageType>(object.imageType, NULL, settings, "VkImageType", "imageType", false, false, indents + 1, dump_json_VkImageType);
    settings.stream() << ",\n";
    dump_json_value<const VkImageTiling>(object.imageTiling, NULL, settings, "VkImageTiling", "imageTiling", false, false, indents + 1, dump_json_VkImageTiling);
    settings.stream() << ",\n";
    dump_json_value<const VkImageUsageFlags>(object.imageUsageFlags, NULL, settings, "VkImageUsageFlags", "imageUsageFlags", false, false, indents + 1, dump_json_VkImageUsageFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoPictureResourceInfoKHR(const VkVideoPictureResourceInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkOffset2D>(object.codedOffset, NULL, settings, "VkOffset2D", "codedOffset", true, false, indents + 1, dump_json_VkOffset2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.codedExtent, NULL, settings, "VkExtent2D", "codedExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.baseArrayLayer, NULL, settings, "uint32_t", "baseArrayLayer", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkImageView>(object.imageViewBinding, NULL, settings, "VkImageView", "imageViewBinding", false, false, indents + 1, dump_json_VkImageView);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoReferenceSlotInfoKHR(const VkVideoReferenceSlotInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.slotIndex, NULL, settings, "int32_t", "slotIndex", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_pointer<const VkVideoPictureResourceInfoKHR>(object.pPictureResource, settings, "const VkVideoPictureResourceInfoKHR*", "pPictureResource", true, false, indents + 1, dump_json_VkVideoPictureResourceInfoKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoSessionMemoryRequirementsKHR(const VkVideoSessionMemoryRequirementsKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryBindIndex, NULL, settings, "uint32_t", "memoryBindIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkMemoryRequirements>(object.memoryRequirements, NULL, settings, "VkMemoryRequirements", "memoryRequirements", true, false, indents + 1, dump_json_VkMemoryRequirements);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindVideoSessionMemoryInfoKHR(const VkBindVideoSessionMemoryInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryBindIndex, NULL, settings, "uint32_t", "memoryBindIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.memoryOffset, NULL, settings, "VkDeviceSize", "memoryOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.memorySize, NULL, settings, "VkDeviceSize", "memorySize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoSessionCreateInfoKHR(const VkVideoSessionCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queueFamilyIndex, NULL, settings, "uint32_t", "queueFamilyIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoSessionCreateFlagsKHR>(object.flags, NULL, settings, "VkVideoSessionCreateFlagsKHR", "flags", false, false, indents + 1, dump_json_VkVideoSessionCreateFlagsKHR);
    settings.stream() << ",\n";
    dump_json_pointer<const VkVideoProfileInfoKHR>(object.pVideoProfile, settings, "const VkVideoProfileInfoKHR*", "pVideoProfile", true, false, indents + 1, dump_json_VkVideoProfileInfoKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.pictureFormat, NULL, settings, "VkFormat", "pictureFormat", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxCodedExtent, NULL, settings, "VkExtent2D", "maxCodedExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.referencePictureFormat, NULL, settings, "VkFormat", "referencePictureFormat", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDpbSlots, NULL, settings, "uint32_t", "maxDpbSlots", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxActiveReferencePictures, NULL, settings, "uint32_t", "maxActiveReferencePictures", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_pointer<const VkExtensionProperties>(object.pStdHeaderVersion, settings, "const VkExtensionProperties*", "pStdHeaderVersion", true, false, indents + 1, dump_json_VkExtensionProperties);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoSessionParametersCreateInfoKHR(const VkVideoSessionParametersCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoSessionParametersCreateFlagsKHR>(object.flags, NULL, settings, "VkVideoSessionParametersCreateFlagsKHR", "flags", false, false, indents + 1, dump_json_VkVideoSessionParametersCreateFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoSessionParametersKHR>(object.videoSessionParametersTemplate, NULL, settings, "VkVideoSessionParametersKHR", "videoSessionParametersTemplate", false, false, indents + 1, dump_json_VkVideoSessionParametersKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoSessionKHR>(object.videoSession, NULL, settings, "VkVideoSessionKHR", "videoSession", false, false, indents + 1, dump_json_VkVideoSessionKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoSessionParametersUpdateInfoKHR(const VkVideoSessionParametersUpdateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.updateSequenceCount, NULL, settings, "uint32_t", "updateSequenceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoBeginCodingInfoKHR(const VkVideoBeginCodingInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoBeginCodingFlagsKHR>(object.flags, NULL, settings, "VkVideoBeginCodingFlagsKHR", "flags", false, false, indents + 1, dump_json_VkVideoBeginCodingFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoSessionKHR>(object.videoSession, NULL, settings, "VkVideoSessionKHR", "videoSession", false, false, indents + 1, dump_json_VkVideoSessionKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoSessionParametersKHR>(object.videoSessionParameters, NULL, settings, "VkVideoSessionParametersKHR", "videoSessionParameters", false, false, indents + 1, dump_json_VkVideoSessionParametersKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.referenceSlotCount, NULL, settings, "uint32_t", "referenceSlotCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkVideoReferenceSlotInfoKHR>(object.pReferenceSlots, object.referenceSlotCount, settings, "const VkVideoReferenceSlotInfoKHR*", "const VkVideoReferenceSlotInfoKHR", "pReferenceSlots", true, false, indents + 1, dump_json_VkVideoReferenceSlotInfoKHR); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEndCodingInfoKHR(const VkVideoEndCodingInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEndCodingFlagsKHR>(object.flags, NULL, settings, "VkVideoEndCodingFlagsKHR", "flags", false, false, indents + 1, dump_json_VkVideoEndCodingFlagsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoCodingControlInfoKHR(const VkVideoCodingControlInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoCodingControlFlagsKHR>(object.flags, NULL, settings, "VkVideoCodingControlFlagsKHR", "flags", false, false, indents + 1, dump_json_VkVideoCodingControlFlagsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeCapabilitiesKHR(const VkVideoDecodeCapabilitiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoDecodeCapabilityFlagsKHR>(object.flags, NULL, settings, "VkVideoDecodeCapabilityFlagsKHR", "flags", false, false, indents + 1, dump_json_VkVideoDecodeCapabilityFlagsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeUsageInfoKHR(const VkVideoDecodeUsageInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoDecodeUsageFlagsKHR>(object.videoUsageHints, NULL, settings, "VkVideoDecodeUsageFlagsKHR", "videoUsageHints", false, false, indents + 1, dump_json_VkVideoDecodeUsageFlagsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeInfoKHR(const VkVideoDecodeInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoDecodeFlagsKHR>(object.flags, NULL, settings, "VkVideoDecodeFlagsKHR", "flags", false, false, indents + 1, dump_json_VkVideoDecodeFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.srcBuffer, NULL, settings, "VkBuffer", "srcBuffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.srcBufferOffset, NULL, settings, "VkDeviceSize", "srcBufferOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.srcBufferRange, NULL, settings, "VkDeviceSize", "srcBufferRange", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoPictureResourceInfoKHR>(object.dstPictureResource, NULL, settings, "VkVideoPictureResourceInfoKHR", "dstPictureResource", true, false, indents + 1, dump_json_VkVideoPictureResourceInfoKHR);
    settings.stream() << ",\n";
    dump_json_pointer<const VkVideoReferenceSlotInfoKHR>(object.pSetupReferenceSlot, settings, "const VkVideoReferenceSlotInfoKHR*", "pSetupReferenceSlot", true, false, indents + 1, dump_json_VkVideoReferenceSlotInfoKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.referenceSlotCount, NULL, settings, "uint32_t", "referenceSlotCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkVideoReferenceSlotInfoKHR>(object.pReferenceSlots, object.referenceSlotCount, settings, "const VkVideoReferenceSlotInfoKHR*", "const VkVideoReferenceSlotInfoKHR", "pReferenceSlots", true, false, indents + 1, dump_json_VkVideoReferenceSlotInfoKHR); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDedicatedAllocationImageCreateInfoNV(const VkDedicatedAllocationImageCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.dedicatedAllocation, NULL, settings, "VkBool32", "dedicatedAllocation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDedicatedAllocationBufferCreateInfoNV(const VkDedicatedAllocationBufferCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.dedicatedAllocation, NULL, settings, "VkBool32", "dedicatedAllocation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDedicatedAllocationMemoryAllocateInfoNV(const VkDedicatedAllocationMemoryAllocateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.image, NULL, settings, "VkImage", "image", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceTransformFeedbackFeaturesEXT(const VkPhysicalDeviceTransformFeedbackFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.transformFeedback, NULL, settings, "VkBool32", "transformFeedback", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.geometryStreams, NULL, settings, "VkBool32", "geometryStreams", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceTransformFeedbackPropertiesEXT(const VkPhysicalDeviceTransformFeedbackPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTransformFeedbackStreams, NULL, settings, "uint32_t", "maxTransformFeedbackStreams", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTransformFeedbackBuffers, NULL, settings, "uint32_t", "maxTransformFeedbackBuffers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.maxTransformFeedbackBufferSize, NULL, settings, "VkDeviceSize", "maxTransformFeedbackBufferSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTransformFeedbackStreamDataSize, NULL, settings, "uint32_t", "maxTransformFeedbackStreamDataSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTransformFeedbackBufferDataSize, NULL, settings, "uint32_t", "maxTransformFeedbackBufferDataSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTransformFeedbackBufferDataStride, NULL, settings, "uint32_t", "maxTransformFeedbackBufferDataStride", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.transformFeedbackQueries, NULL, settings, "VkBool32", "transformFeedbackQueries", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.transformFeedbackStreamsLinesTriangles, NULL, settings, "VkBool32", "transformFeedbackStreamsLinesTriangles", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.transformFeedbackRasterizationStreamSelect, NULL, settings, "VkBool32", "transformFeedbackRasterizationStreamSelect", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.transformFeedbackDraw, NULL, settings, "VkBool32", "transformFeedbackDraw", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineRasterizationStateStreamCreateInfoEXT(const VkPipelineRasterizationStateStreamCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineRasterizationStateStreamCreateFlagsEXT>(object.flags, NULL, settings, "VkPipelineRasterizationStateStreamCreateFlagsEXT", "flags", false, false, indents + 1, dump_json_VkPipelineRasterizationStateStreamCreateFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.rasterizationStream, NULL, settings, "uint32_t", "rasterizationStream", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCuModuleCreateInfoNVX(const VkCuModuleCreateInfoNVX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.dataSize, NULL, settings, "size_t", "dataSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pData, NULL, settings, "const void*", "pData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCuFunctionCreateInfoNVX(const VkCuFunctionCreateInfoNVX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkCuModuleNVX>(object.module, NULL, settings, "VkCuModuleNVX", "module", false, false, indents + 1, dump_json_VkCuModuleNVX);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.pName, NULL, settings, "const char*", "pName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCuLaunchInfoNVX(const VkCuLaunchInfoNVX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkCuFunctionNVX>(object.function, NULL, settings, "VkCuFunctionNVX", "function", false, false, indents + 1, dump_json_VkCuFunctionNVX);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.gridDimX, NULL, settings, "uint32_t", "gridDimX", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.gridDimY, NULL, settings, "uint32_t", "gridDimY", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.gridDimZ, NULL, settings, "uint32_t", "gridDimZ", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.blockDimX, NULL, settings, "uint32_t", "blockDimX", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.blockDimY, NULL, settings, "uint32_t", "blockDimY", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.blockDimZ, NULL, settings, "uint32_t", "blockDimZ", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.sharedMemBytes, NULL, settings, "uint32_t", "sharedMemBytes", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.paramCount, NULL, settings, "size_t", "paramCount", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_array<const void*>(object.pParams, object.paramCount, settings, "const void* const *", "const void* const ", "pParams", false, false, indents + 1, dump_json_void); // JQA
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.extraCount, NULL, settings, "size_t", "extraCount", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_array<const void*>(object.pExtras, object.extraCount, settings, "const void* const *", "const void* const ", "pExtras", false, false, indents + 1, dump_json_void); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageViewHandleInfoNVX(const VkImageViewHandleInfoNVX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageView>(object.imageView, NULL, settings, "VkImageView", "imageView", false, false, indents + 1, dump_json_VkImageView);
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorType>(object.descriptorType, NULL, settings, "VkDescriptorType", "descriptorType", false, false, indents + 1, dump_json_VkDescriptorType);
    settings.stream() << ",\n";
    dump_json_value<const VkSampler>(object.sampler, NULL, settings, "VkSampler", "sampler", false, false, indents + 1, dump_json_VkSampler);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageViewAddressPropertiesNVX(const VkImageViewAddressPropertiesNVX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddress>(object.deviceAddress, NULL, settings, "VkDeviceAddress", "deviceAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264CapabilitiesKHR(const VkVideoEncodeH264CapabilitiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH264CapabilityFlagsKHR>(object.flags, NULL, settings, "VkVideoEncodeH264CapabilityFlagsKHR", "flags", false, false, indents + 1, dump_json_VkVideoEncodeH264CapabilityFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const StdVideoH264LevelIdc>(object.maxLevelIdc, NULL, settings, "StdVideoH264LevelIdc", "maxLevelIdc", false, false, indents + 1, dump_json_StdVideoH264LevelIdc);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxSliceCount, NULL, settings, "uint32_t", "maxSliceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPPictureL0ReferenceCount, NULL, settings, "uint32_t", "maxPPictureL0ReferenceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxBPictureL0ReferenceCount, NULL, settings, "uint32_t", "maxBPictureL0ReferenceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxL1ReferenceCount, NULL, settings, "uint32_t", "maxL1ReferenceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTemporalLayerCount, NULL, settings, "uint32_t", "maxTemporalLayerCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.expectDyadicTemporalLayerPattern, NULL, settings, "VkBool32", "expectDyadicTemporalLayerPattern", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.minQp, NULL, settings, "int32_t", "minQp", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.maxQp, NULL, settings, "int32_t", "maxQp", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.prefersGopRemainingFrames, NULL, settings, "VkBool32", "prefersGopRemainingFrames", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.requiresGopRemainingFrames, NULL, settings, "VkBool32", "requiresGopRemainingFrames", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH264StdFlagsKHR>(object.stdSyntaxFlags, NULL, settings, "VkVideoEncodeH264StdFlagsKHR", "stdSyntaxFlags", false, false, indents + 1, dump_json_VkVideoEncodeH264StdFlagsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264QpKHR(const VkVideoEncodeH264QpKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const int32_t>(object.qpI, NULL, settings, "int32_t", "qpI", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.qpP, NULL, settings, "int32_t", "qpP", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.qpB, NULL, settings, "int32_t", "qpB", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264QualityLevelPropertiesKHR(const VkVideoEncodeH264QualityLevelPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH264RateControlFlagsKHR>(object.preferredRateControlFlags, NULL, settings, "VkVideoEncodeH264RateControlFlagsKHR", "preferredRateControlFlags", false, false, indents + 1, dump_json_VkVideoEncodeH264RateControlFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.preferredGopFrameCount, NULL, settings, "uint32_t", "preferredGopFrameCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.preferredIdrPeriod, NULL, settings, "uint32_t", "preferredIdrPeriod", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.preferredConsecutiveBFrameCount, NULL, settings, "uint32_t", "preferredConsecutiveBFrameCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.preferredTemporalLayerCount, NULL, settings, "uint32_t", "preferredTemporalLayerCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH264QpKHR>(object.preferredConstantQp, NULL, settings, "VkVideoEncodeH264QpKHR", "preferredConstantQp", true, false, indents + 1, dump_json_VkVideoEncodeH264QpKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.preferredMaxL0ReferenceCount, NULL, settings, "uint32_t", "preferredMaxL0ReferenceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.preferredMaxL1ReferenceCount, NULL, settings, "uint32_t", "preferredMaxL1ReferenceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.preferredStdEntropyCodingModeFlag, NULL, settings, "VkBool32", "preferredStdEntropyCodingModeFlag", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264SessionCreateInfoKHR(const VkVideoEncodeH264SessionCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.useMaxLevelIdc, NULL, settings, "VkBool32", "useMaxLevelIdc", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const StdVideoH264LevelIdc>(object.maxLevelIdc, NULL, settings, "StdVideoH264LevelIdc", "maxLevelIdc", false, false, indents + 1, dump_json_StdVideoH264LevelIdc);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264SessionParametersAddInfoKHR(const VkVideoEncodeH264SessionParametersAddInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stdSPSCount, NULL, settings, "uint32_t", "stdSPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const StdVideoH264SequenceParameterSet>(object.pStdSPSs, object.stdSPSCount, settings, "const StdVideoH264SequenceParameterSet*", "const StdVideoH264SequenceParameterSet", "pStdSPSs", false, false, indents + 1, dump_json_StdVideoH264SequenceParameterSet); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stdPPSCount, NULL, settings, "uint32_t", "stdPPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const StdVideoH264PictureParameterSet>(object.pStdPPSs, object.stdPPSCount, settings, "const StdVideoH264PictureParameterSet*", "const StdVideoH264PictureParameterSet", "pStdPPSs", false, false, indents + 1, dump_json_StdVideoH264PictureParameterSet); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264SessionParametersCreateInfoKHR(const VkVideoEncodeH264SessionParametersCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxStdSPSCount, NULL, settings, "uint32_t", "maxStdSPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxStdPPSCount, NULL, settings, "uint32_t", "maxStdPPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_pointer<const VkVideoEncodeH264SessionParametersAddInfoKHR>(object.pParametersAddInfo, settings, "const VkVideoEncodeH264SessionParametersAddInfoKHR*", "pParametersAddInfo", true, false, indents + 1, dump_json_VkVideoEncodeH264SessionParametersAddInfoKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264SessionParametersGetInfoKHR(const VkVideoEncodeH264SessionParametersGetInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.writeStdSPS, NULL, settings, "VkBool32", "writeStdSPS", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.writeStdPPS, NULL, settings, "VkBool32", "writeStdPPS", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stdSPSId, NULL, settings, "uint32_t", "stdSPSId", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stdPPSId, NULL, settings, "uint32_t", "stdPPSId", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264SessionParametersFeedbackInfoKHR(const VkVideoEncodeH264SessionParametersFeedbackInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.hasStdSPSOverrides, NULL, settings, "VkBool32", "hasStdSPSOverrides", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.hasStdPPSOverrides, NULL, settings, "VkBool32", "hasStdPPSOverrides", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264NaluSliceInfoKHR(const VkVideoEncodeH264NaluSliceInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.constantQp, NULL, settings, "int32_t", "constantQp", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_pointer<const StdVideoEncodeH264SliceHeader>(object.pStdSliceHeader, settings, "const StdVideoEncodeH264SliceHeader*", "pStdSliceHeader", false, false, indents + 1, dump_json_StdVideoEncodeH264SliceHeader);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264PictureInfoKHR(const VkVideoEncodeH264PictureInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.naluSliceEntryCount, NULL, settings, "uint32_t", "naluSliceEntryCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkVideoEncodeH264NaluSliceInfoKHR>(object.pNaluSliceEntries, object.naluSliceEntryCount, settings, "const VkVideoEncodeH264NaluSliceInfoKHR*", "const VkVideoEncodeH264NaluSliceInfoKHR", "pNaluSliceEntries", true, false, indents + 1, dump_json_VkVideoEncodeH264NaluSliceInfoKHR); // JQA
    settings.stream() << ",\n";
    dump_json_pointer<const StdVideoEncodeH264PictureInfo>(object.pStdPictureInfo, settings, "const StdVideoEncodeH264PictureInfo*", "pStdPictureInfo", false, false, indents + 1, dump_json_StdVideoEncodeH264PictureInfo);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.generatePrefixNalu, NULL, settings, "VkBool32", "generatePrefixNalu", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264DpbSlotInfoKHR(const VkVideoEncodeH264DpbSlotInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const StdVideoEncodeH264ReferenceInfo>(object.pStdReferenceInfo, settings, "const StdVideoEncodeH264ReferenceInfo*", "pStdReferenceInfo", false, false, indents + 1, dump_json_StdVideoEncodeH264ReferenceInfo);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264ProfileInfoKHR(const VkVideoEncodeH264ProfileInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const StdVideoH264ProfileIdc>(object.stdProfileIdc, NULL, settings, "StdVideoH264ProfileIdc", "stdProfileIdc", false, false, indents + 1, dump_json_StdVideoH264ProfileIdc);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264RateControlInfoKHR(const VkVideoEncodeH264RateControlInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH264RateControlFlagsKHR>(object.flags, NULL, settings, "VkVideoEncodeH264RateControlFlagsKHR", "flags", false, false, indents + 1, dump_json_VkVideoEncodeH264RateControlFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.gopFrameCount, NULL, settings, "uint32_t", "gopFrameCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.idrPeriod, NULL, settings, "uint32_t", "idrPeriod", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.consecutiveBFrameCount, NULL, settings, "uint32_t", "consecutiveBFrameCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.temporalLayerCount, NULL, settings, "uint32_t", "temporalLayerCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264FrameSizeKHR(const VkVideoEncodeH264FrameSizeKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.frameISize, NULL, settings, "uint32_t", "frameISize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.framePSize, NULL, settings, "uint32_t", "framePSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.frameBSize, NULL, settings, "uint32_t", "frameBSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264RateControlLayerInfoKHR(const VkVideoEncodeH264RateControlLayerInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.useMinQp, NULL, settings, "VkBool32", "useMinQp", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH264QpKHR>(object.minQp, NULL, settings, "VkVideoEncodeH264QpKHR", "minQp", true, false, indents + 1, dump_json_VkVideoEncodeH264QpKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.useMaxQp, NULL, settings, "VkBool32", "useMaxQp", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH264QpKHR>(object.maxQp, NULL, settings, "VkVideoEncodeH264QpKHR", "maxQp", true, false, indents + 1, dump_json_VkVideoEncodeH264QpKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.useMaxFrameSize, NULL, settings, "VkBool32", "useMaxFrameSize", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH264FrameSizeKHR>(object.maxFrameSize, NULL, settings, "VkVideoEncodeH264FrameSizeKHR", "maxFrameSize", true, false, indents + 1, dump_json_VkVideoEncodeH264FrameSizeKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH264GopRemainingFrameInfoKHR(const VkVideoEncodeH264GopRemainingFrameInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.useGopRemainingFrames, NULL, settings, "VkBool32", "useGopRemainingFrames", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.gopRemainingI, NULL, settings, "uint32_t", "gopRemainingI", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.gopRemainingP, NULL, settings, "uint32_t", "gopRemainingP", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.gopRemainingB, NULL, settings, "uint32_t", "gopRemainingB", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265CapabilitiesKHR(const VkVideoEncodeH265CapabilitiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH265CapabilityFlagsKHR>(object.flags, NULL, settings, "VkVideoEncodeH265CapabilityFlagsKHR", "flags", false, false, indents + 1, dump_json_VkVideoEncodeH265CapabilityFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const StdVideoH265LevelIdc>(object.maxLevelIdc, NULL, settings, "StdVideoH265LevelIdc", "maxLevelIdc", false, false, indents + 1, dump_json_StdVideoH265LevelIdc);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxSliceSegmentCount, NULL, settings, "uint32_t", "maxSliceSegmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxTiles, NULL, settings, "VkExtent2D", "maxTiles", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH265CtbSizeFlagsKHR>(object.ctbSizes, NULL, settings, "VkVideoEncodeH265CtbSizeFlagsKHR", "ctbSizes", false, false, indents + 1, dump_json_VkVideoEncodeH265CtbSizeFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH265TransformBlockSizeFlagsKHR>(object.transformBlockSizes, NULL, settings, "VkVideoEncodeH265TransformBlockSizeFlagsKHR", "transformBlockSizes", false, false, indents + 1, dump_json_VkVideoEncodeH265TransformBlockSizeFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPPictureL0ReferenceCount, NULL, settings, "uint32_t", "maxPPictureL0ReferenceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxBPictureL0ReferenceCount, NULL, settings, "uint32_t", "maxBPictureL0ReferenceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxL1ReferenceCount, NULL, settings, "uint32_t", "maxL1ReferenceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxSubLayerCount, NULL, settings, "uint32_t", "maxSubLayerCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.expectDyadicTemporalSubLayerPattern, NULL, settings, "VkBool32", "expectDyadicTemporalSubLayerPattern", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.minQp, NULL, settings, "int32_t", "minQp", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.maxQp, NULL, settings, "int32_t", "maxQp", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.prefersGopRemainingFrames, NULL, settings, "VkBool32", "prefersGopRemainingFrames", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.requiresGopRemainingFrames, NULL, settings, "VkBool32", "requiresGopRemainingFrames", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH265StdFlagsKHR>(object.stdSyntaxFlags, NULL, settings, "VkVideoEncodeH265StdFlagsKHR", "stdSyntaxFlags", false, false, indents + 1, dump_json_VkVideoEncodeH265StdFlagsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265SessionCreateInfoKHR(const VkVideoEncodeH265SessionCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.useMaxLevelIdc, NULL, settings, "VkBool32", "useMaxLevelIdc", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const StdVideoH265LevelIdc>(object.maxLevelIdc, NULL, settings, "StdVideoH265LevelIdc", "maxLevelIdc", false, false, indents + 1, dump_json_StdVideoH265LevelIdc);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265QpKHR(const VkVideoEncodeH265QpKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const int32_t>(object.qpI, NULL, settings, "int32_t", "qpI", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.qpP, NULL, settings, "int32_t", "qpP", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.qpB, NULL, settings, "int32_t", "qpB", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265QualityLevelPropertiesKHR(const VkVideoEncodeH265QualityLevelPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH265RateControlFlagsKHR>(object.preferredRateControlFlags, NULL, settings, "VkVideoEncodeH265RateControlFlagsKHR", "preferredRateControlFlags", false, false, indents + 1, dump_json_VkVideoEncodeH265RateControlFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.preferredGopFrameCount, NULL, settings, "uint32_t", "preferredGopFrameCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.preferredIdrPeriod, NULL, settings, "uint32_t", "preferredIdrPeriod", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.preferredConsecutiveBFrameCount, NULL, settings, "uint32_t", "preferredConsecutiveBFrameCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.preferredSubLayerCount, NULL, settings, "uint32_t", "preferredSubLayerCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH265QpKHR>(object.preferredConstantQp, NULL, settings, "VkVideoEncodeH265QpKHR", "preferredConstantQp", true, false, indents + 1, dump_json_VkVideoEncodeH265QpKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.preferredMaxL0ReferenceCount, NULL, settings, "uint32_t", "preferredMaxL0ReferenceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.preferredMaxL1ReferenceCount, NULL, settings, "uint32_t", "preferredMaxL1ReferenceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265SessionParametersAddInfoKHR(const VkVideoEncodeH265SessionParametersAddInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stdVPSCount, NULL, settings, "uint32_t", "stdVPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const StdVideoH265VideoParameterSet>(object.pStdVPSs, object.stdVPSCount, settings, "const StdVideoH265VideoParameterSet*", "const StdVideoH265VideoParameterSet", "pStdVPSs", false, false, indents + 1, dump_json_StdVideoH265VideoParameterSet); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stdSPSCount, NULL, settings, "uint32_t", "stdSPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const StdVideoH265SequenceParameterSet>(object.pStdSPSs, object.stdSPSCount, settings, "const StdVideoH265SequenceParameterSet*", "const StdVideoH265SequenceParameterSet", "pStdSPSs", false, false, indents + 1, dump_json_StdVideoH265SequenceParameterSet); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stdPPSCount, NULL, settings, "uint32_t", "stdPPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const StdVideoH265PictureParameterSet>(object.pStdPPSs, object.stdPPSCount, settings, "const StdVideoH265PictureParameterSet*", "const StdVideoH265PictureParameterSet", "pStdPPSs", false, false, indents + 1, dump_json_StdVideoH265PictureParameterSet); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265SessionParametersCreateInfoKHR(const VkVideoEncodeH265SessionParametersCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxStdVPSCount, NULL, settings, "uint32_t", "maxStdVPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxStdSPSCount, NULL, settings, "uint32_t", "maxStdSPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxStdPPSCount, NULL, settings, "uint32_t", "maxStdPPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_pointer<const VkVideoEncodeH265SessionParametersAddInfoKHR>(object.pParametersAddInfo, settings, "const VkVideoEncodeH265SessionParametersAddInfoKHR*", "pParametersAddInfo", true, false, indents + 1, dump_json_VkVideoEncodeH265SessionParametersAddInfoKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265SessionParametersGetInfoKHR(const VkVideoEncodeH265SessionParametersGetInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.writeStdVPS, NULL, settings, "VkBool32", "writeStdVPS", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.writeStdSPS, NULL, settings, "VkBool32", "writeStdSPS", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.writeStdPPS, NULL, settings, "VkBool32", "writeStdPPS", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stdVPSId, NULL, settings, "uint32_t", "stdVPSId", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stdSPSId, NULL, settings, "uint32_t", "stdSPSId", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stdPPSId, NULL, settings, "uint32_t", "stdPPSId", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265SessionParametersFeedbackInfoKHR(const VkVideoEncodeH265SessionParametersFeedbackInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.hasStdVPSOverrides, NULL, settings, "VkBool32", "hasStdVPSOverrides", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.hasStdSPSOverrides, NULL, settings, "VkBool32", "hasStdSPSOverrides", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.hasStdPPSOverrides, NULL, settings, "VkBool32", "hasStdPPSOverrides", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265NaluSliceSegmentInfoKHR(const VkVideoEncodeH265NaluSliceSegmentInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.constantQp, NULL, settings, "int32_t", "constantQp", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_pointer<const StdVideoEncodeH265SliceSegmentHeader>(object.pStdSliceSegmentHeader, settings, "const StdVideoEncodeH265SliceSegmentHeader*", "pStdSliceSegmentHeader", false, false, indents + 1, dump_json_StdVideoEncodeH265SliceSegmentHeader);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265PictureInfoKHR(const VkVideoEncodeH265PictureInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.naluSliceSegmentEntryCount, NULL, settings, "uint32_t", "naluSliceSegmentEntryCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkVideoEncodeH265NaluSliceSegmentInfoKHR>(object.pNaluSliceSegmentEntries, object.naluSliceSegmentEntryCount, settings, "const VkVideoEncodeH265NaluSliceSegmentInfoKHR*", "const VkVideoEncodeH265NaluSliceSegmentInfoKHR", "pNaluSliceSegmentEntries", true, false, indents + 1, dump_json_VkVideoEncodeH265NaluSliceSegmentInfoKHR); // JQA
    settings.stream() << ",\n";
    dump_json_pointer<const StdVideoEncodeH265PictureInfo>(object.pStdPictureInfo, settings, "const StdVideoEncodeH265PictureInfo*", "pStdPictureInfo", false, false, indents + 1, dump_json_StdVideoEncodeH265PictureInfo);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265DpbSlotInfoKHR(const VkVideoEncodeH265DpbSlotInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const StdVideoEncodeH265ReferenceInfo>(object.pStdReferenceInfo, settings, "const StdVideoEncodeH265ReferenceInfo*", "pStdReferenceInfo", false, false, indents + 1, dump_json_StdVideoEncodeH265ReferenceInfo);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265ProfileInfoKHR(const VkVideoEncodeH265ProfileInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const StdVideoH265ProfileIdc>(object.stdProfileIdc, NULL, settings, "StdVideoH265ProfileIdc", "stdProfileIdc", false, false, indents + 1, dump_json_StdVideoH265ProfileIdc);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265RateControlInfoKHR(const VkVideoEncodeH265RateControlInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH265RateControlFlagsKHR>(object.flags, NULL, settings, "VkVideoEncodeH265RateControlFlagsKHR", "flags", false, false, indents + 1, dump_json_VkVideoEncodeH265RateControlFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.gopFrameCount, NULL, settings, "uint32_t", "gopFrameCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.idrPeriod, NULL, settings, "uint32_t", "idrPeriod", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.consecutiveBFrameCount, NULL, settings, "uint32_t", "consecutiveBFrameCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.subLayerCount, NULL, settings, "uint32_t", "subLayerCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265FrameSizeKHR(const VkVideoEncodeH265FrameSizeKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.frameISize, NULL, settings, "uint32_t", "frameISize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.framePSize, NULL, settings, "uint32_t", "framePSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.frameBSize, NULL, settings, "uint32_t", "frameBSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265RateControlLayerInfoKHR(const VkVideoEncodeH265RateControlLayerInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.useMinQp, NULL, settings, "VkBool32", "useMinQp", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH265QpKHR>(object.minQp, NULL, settings, "VkVideoEncodeH265QpKHR", "minQp", true, false, indents + 1, dump_json_VkVideoEncodeH265QpKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.useMaxQp, NULL, settings, "VkBool32", "useMaxQp", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH265QpKHR>(object.maxQp, NULL, settings, "VkVideoEncodeH265QpKHR", "maxQp", true, false, indents + 1, dump_json_VkVideoEncodeH265QpKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.useMaxFrameSize, NULL, settings, "VkBool32", "useMaxFrameSize", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeH265FrameSizeKHR>(object.maxFrameSize, NULL, settings, "VkVideoEncodeH265FrameSizeKHR", "maxFrameSize", true, false, indents + 1, dump_json_VkVideoEncodeH265FrameSizeKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeH265GopRemainingFrameInfoKHR(const VkVideoEncodeH265GopRemainingFrameInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.useGopRemainingFrames, NULL, settings, "VkBool32", "useGopRemainingFrames", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.gopRemainingI, NULL, settings, "uint32_t", "gopRemainingI", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.gopRemainingP, NULL, settings, "uint32_t", "gopRemainingP", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.gopRemainingB, NULL, settings, "uint32_t", "gopRemainingB", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeH264ProfileInfoKHR(const VkVideoDecodeH264ProfileInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const StdVideoH264ProfileIdc>(object.stdProfileIdc, NULL, settings, "StdVideoH264ProfileIdc", "stdProfileIdc", false, false, indents + 1, dump_json_StdVideoH264ProfileIdc);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoDecodeH264PictureLayoutFlagBitsKHR>(object.pictureLayout, NULL, settings, "VkVideoDecodeH264PictureLayoutFlagBitsKHR", "pictureLayout", false, false, indents + 1, dump_json_VkVideoDecodeH264PictureLayoutFlagBitsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeH264CapabilitiesKHR(const VkVideoDecodeH264CapabilitiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const StdVideoH264LevelIdc>(object.maxLevelIdc, NULL, settings, "StdVideoH264LevelIdc", "maxLevelIdc", false, false, indents + 1, dump_json_StdVideoH264LevelIdc);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset2D>(object.fieldOffsetGranularity, NULL, settings, "VkOffset2D", "fieldOffsetGranularity", true, false, indents + 1, dump_json_VkOffset2D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeH264SessionParametersAddInfoKHR(const VkVideoDecodeH264SessionParametersAddInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stdSPSCount, NULL, settings, "uint32_t", "stdSPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const StdVideoH264SequenceParameterSet>(object.pStdSPSs, object.stdSPSCount, settings, "const StdVideoH264SequenceParameterSet*", "const StdVideoH264SequenceParameterSet", "pStdSPSs", false, false, indents + 1, dump_json_StdVideoH264SequenceParameterSet); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stdPPSCount, NULL, settings, "uint32_t", "stdPPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const StdVideoH264PictureParameterSet>(object.pStdPPSs, object.stdPPSCount, settings, "const StdVideoH264PictureParameterSet*", "const StdVideoH264PictureParameterSet", "pStdPPSs", false, false, indents + 1, dump_json_StdVideoH264PictureParameterSet); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeH264SessionParametersCreateInfoKHR(const VkVideoDecodeH264SessionParametersCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxStdSPSCount, NULL, settings, "uint32_t", "maxStdSPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxStdPPSCount, NULL, settings, "uint32_t", "maxStdPPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_pointer<const VkVideoDecodeH264SessionParametersAddInfoKHR>(object.pParametersAddInfo, settings, "const VkVideoDecodeH264SessionParametersAddInfoKHR*", "pParametersAddInfo", true, false, indents + 1, dump_json_VkVideoDecodeH264SessionParametersAddInfoKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeH264PictureInfoKHR(const VkVideoDecodeH264PictureInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const StdVideoDecodeH264PictureInfo>(object.pStdPictureInfo, settings, "const StdVideoDecodeH264PictureInfo*", "pStdPictureInfo", false, false, indents + 1, dump_json_StdVideoDecodeH264PictureInfo);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.sliceCount, NULL, settings, "uint32_t", "sliceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pSliceOffsets, object.sliceCount, settings, "const uint32_t*", "const uint32_t", "pSliceOffsets", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeH264DpbSlotInfoKHR(const VkVideoDecodeH264DpbSlotInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const StdVideoDecodeH264ReferenceInfo>(object.pStdReferenceInfo, settings, "const StdVideoDecodeH264ReferenceInfo*", "pStdReferenceInfo", false, false, indents + 1, dump_json_StdVideoDecodeH264ReferenceInfo);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkTextureLODGatherFormatPropertiesAMD(const VkTextureLODGatherFormatPropertiesAMD& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.supportsTextureGatherLODBiasAMD, NULL, settings, "VkBool32", "supportsTextureGatherLODBiasAMD", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkShaderResourceUsageAMD(const VkShaderResourceUsageAMD& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.numUsedVgprs, NULL, settings, "uint32_t", "numUsedVgprs", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.numUsedSgprs, NULL, settings, "uint32_t", "numUsedSgprs", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.ldsSizePerLocalWorkGroup, NULL, settings, "uint32_t", "ldsSizePerLocalWorkGroup", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.ldsUsageSizeInBytes, NULL, settings, "size_t", "ldsUsageSizeInBytes", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.scratchMemUsageInBytes, NULL, settings, "size_t", "scratchMemUsageInBytes", false, false, indents + 1, dump_json_size_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkShaderStatisticsInfoAMD(const VkShaderStatisticsInfoAMD& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkShaderStageFlags>(object.shaderStageMask, NULL, settings, "VkShaderStageFlags", "shaderStageMask", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderResourceUsageAMD>(object.resourceUsage, NULL, settings, "VkShaderResourceUsageAMD", "resourceUsage", true, false, indents + 1, dump_json_VkShaderResourceUsageAMD);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.numPhysicalVgprs, NULL, settings, "uint32_t", "numPhysicalVgprs", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.numPhysicalSgprs, NULL, settings, "uint32_t", "numPhysicalSgprs", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.numAvailableVgprs, NULL, settings, "uint32_t", "numAvailableVgprs", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.numAvailableSgprs, NULL, settings, "uint32_t", "numAvailableSgprs", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.computeWorkGroupSize, 3, settings, "uint32_t[3]", "uint32_t", "computeWorkGroupSize", false, false, indents + 1, dump_json_uint32_t); // IQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderingFragmentShadingRateAttachmentInfoKHR(const VkRenderingFragmentShadingRateAttachmentInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageView>(object.imageView, NULL, settings, "VkImageView", "imageView", false, false, indents + 1, dump_json_VkImageView);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.imageLayout, NULL, settings, "VkImageLayout", "imageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.shadingRateAttachmentTexelSize, NULL, settings, "VkExtent2D", "shadingRateAttachmentTexelSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderingFragmentDensityMapAttachmentInfoEXT(const VkRenderingFragmentDensityMapAttachmentInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageView>(object.imageView, NULL, settings, "VkImageView", "imageView", false, false, indents + 1, dump_json_VkImageView);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.imageLayout, NULL, settings, "VkImageLayout", "imageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAttachmentSampleCountInfoAMD(const VkAttachmentSampleCountInfoAMD& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.colorAttachmentCount, NULL, settings, "uint32_t", "colorAttachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSampleCountFlagBits>(object.pColorAttachmentSamples, object.colorAttachmentCount, settings, "const VkSampleCountFlagBits*", "const VkSampleCountFlagBits", "pColorAttachmentSamples", false, false, indents + 1, dump_json_VkSampleCountFlagBits); // JQA
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlagBits>(object.depthStencilAttachmentSamples, NULL, settings, "VkSampleCountFlagBits", "depthStencilAttachmentSamples", false, false, indents + 1, dump_json_VkSampleCountFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMultiviewPerViewAttributesInfoNVX(const VkMultiviewPerViewAttributesInfoNVX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.perViewAttributes, NULL, settings, "VkBool32", "perViewAttributes", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.perViewAttributesPositionXOnly, NULL, settings, "VkBool32", "perViewAttributesPositionXOnly", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_GGP)
void dump_json_VkStreamDescriptorSurfaceCreateInfoGGP(const VkStreamDescriptorSurfaceCreateInfoGGP& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkStreamDescriptorSurfaceCreateFlagsGGP>(object.flags, NULL, settings, "VkStreamDescriptorSurfaceCreateFlagsGGP", "flags", false, false, indents + 1, dump_json_VkStreamDescriptorSurfaceCreateFlagsGGP);
    settings.stream() << ",\n";
    dump_json_value<const GgpStreamDescriptor>(object.streamDescriptor, NULL, settings, "GgpStreamDescriptor", "streamDescriptor", false, false, indents + 1, dump_json_GgpStreamDescriptor);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_GGP
void dump_json_VkPhysicalDeviceCornerSampledImageFeaturesNV(const VkPhysicalDeviceCornerSampledImageFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.cornerSampledImage, NULL, settings, "VkBool32", "cornerSampledImage", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExternalImageFormatPropertiesNV(const VkExternalImageFormatPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkImageFormatProperties>(object.imageFormatProperties, NULL, settings, "VkImageFormatProperties", "imageFormatProperties", true, false, indents + 1, dump_json_VkImageFormatProperties);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryFeatureFlagsNV>(object.externalMemoryFeatures, NULL, settings, "VkExternalMemoryFeatureFlagsNV", "externalMemoryFeatures", false, false, indents + 1, dump_json_VkExternalMemoryFeatureFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlagsNV>(object.exportFromImportedHandleTypes, NULL, settings, "VkExternalMemoryHandleTypeFlagsNV", "exportFromImportedHandleTypes", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlagsNV>(object.compatibleHandleTypes, NULL, settings, "VkExternalMemoryHandleTypeFlagsNV", "compatibleHandleTypes", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlagsNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExternalMemoryImageCreateInfoNV(const VkExternalMemoryImageCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlagsNV>(object.handleTypes, NULL, settings, "VkExternalMemoryHandleTypeFlagsNV", "handleTypes", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlagsNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExportMemoryAllocateInfoNV(const VkExportMemoryAllocateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlagsNV>(object.handleTypes, NULL, settings, "VkExternalMemoryHandleTypeFlagsNV", "handleTypes", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlagsNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkImportMemoryWin32HandleInfoNV(const VkImportMemoryWin32HandleInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlagsNV>(object.handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagsNV", "handleType", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const HANDLE>(object.handle, NULL, settings, "HANDLE", "handle", false, false, indents + 1, dump_json_HANDLE);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkExportMemoryWin32HandleInfoNV(const VkExportMemoryWin32HandleInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const SECURITY_ATTRIBUTES*>(object.pAttributes, NULL, settings, "const SECURITY_ATTRIBUTES*", "pAttributes", false, false, indents + 1, dump_json_SECURITY_ATTRIBUTES);
    settings.stream() << ",\n";
    dump_json_value<const DWORD>(object.dwAccess, NULL, settings, "DWORD", "dwAccess", false, false, indents + 1, dump_json_DWORD);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkWin32KeyedMutexAcquireReleaseInfoNV(const VkWin32KeyedMutexAcquireReleaseInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.acquireCount, NULL, settings, "uint32_t", "acquireCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDeviceMemory>(object.pAcquireSyncs, object.acquireCount, settings, "const VkDeviceMemory*", "const VkDeviceMemory", "pAcquireSyncs", false, false, indents + 1, dump_json_VkDeviceMemory); // JQA
    settings.stream() << ",\n";
    dump_json_array<const uint64_t>(object.pAcquireKeys, object.acquireCount, settings, "const uint64_t*", "const uint64_t", "pAcquireKeys", false, false, indents + 1, dump_json_uint64_t); // JQA
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pAcquireTimeoutMilliseconds, object.acquireCount, settings, "const uint32_t*", "const uint32_t", "pAcquireTimeoutMilliseconds", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.releaseCount, NULL, settings, "uint32_t", "releaseCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDeviceMemory>(object.pReleaseSyncs, object.releaseCount, settings, "const VkDeviceMemory*", "const VkDeviceMemory", "pReleaseSyncs", false, false, indents + 1, dump_json_VkDeviceMemory); // JQA
    settings.stream() << ",\n";
    dump_json_array<const uint64_t>(object.pReleaseKeys, object.releaseCount, settings, "const uint64_t*", "const uint64_t", "pReleaseKeys", false, false, indents + 1, dump_json_uint64_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
void dump_json_VkValidationFlagsEXT(const VkValidationFlagsEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.disabledValidationCheckCount, NULL, settings, "uint32_t", "disabledValidationCheckCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkValidationCheckEXT>(object.pDisabledValidationChecks, object.disabledValidationCheckCount, settings, "const VkValidationCheckEXT*", "const VkValidationCheckEXT", "pDisabledValidationChecks", false, false, indents + 1, dump_json_VkValidationCheckEXT); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_VI_NN)
void dump_json_VkViSurfaceCreateInfoNN(const VkViSurfaceCreateInfoNN& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkViSurfaceCreateFlagsNN>(object.flags, NULL, settings, "VkViSurfaceCreateFlagsNN", "flags", false, false, indents + 1, dump_json_VkViSurfaceCreateFlagsNN);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.window, NULL, settings, "void*", "window", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_VI_NN
void dump_json_VkImageViewASTCDecodeModeEXT(const VkImageViewASTCDecodeModeEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.decodeMode, NULL, settings, "VkFormat", "decodeMode", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceASTCDecodeFeaturesEXT(const VkPhysicalDeviceASTCDecodeFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.decodeModeSharedExponent, NULL, settings, "VkBool32", "decodeModeSharedExponent", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePipelineRobustnessFeaturesEXT(const VkPhysicalDevicePipelineRobustnessFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.pipelineRobustness, NULL, settings, "VkBool32", "pipelineRobustness", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePipelineRobustnessPropertiesEXT(const VkPhysicalDevicePipelineRobustnessPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineRobustnessBufferBehaviorEXT>(object.defaultRobustnessStorageBuffers, NULL, settings, "VkPipelineRobustnessBufferBehaviorEXT", "defaultRobustnessStorageBuffers", false, false, indents + 1, dump_json_VkPipelineRobustnessBufferBehaviorEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineRobustnessBufferBehaviorEXT>(object.defaultRobustnessUniformBuffers, NULL, settings, "VkPipelineRobustnessBufferBehaviorEXT", "defaultRobustnessUniformBuffers", false, false, indents + 1, dump_json_VkPipelineRobustnessBufferBehaviorEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineRobustnessBufferBehaviorEXT>(object.defaultRobustnessVertexInputs, NULL, settings, "VkPipelineRobustnessBufferBehaviorEXT", "defaultRobustnessVertexInputs", false, false, indents + 1, dump_json_VkPipelineRobustnessBufferBehaviorEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineRobustnessImageBehaviorEXT>(object.defaultRobustnessImages, NULL, settings, "VkPipelineRobustnessImageBehaviorEXT", "defaultRobustnessImages", false, false, indents + 1, dump_json_VkPipelineRobustnessImageBehaviorEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineRobustnessCreateInfoEXT(const VkPipelineRobustnessCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineRobustnessBufferBehaviorEXT>(object.storageBuffers, NULL, settings, "VkPipelineRobustnessBufferBehaviorEXT", "storageBuffers", false, false, indents + 1, dump_json_VkPipelineRobustnessBufferBehaviorEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineRobustnessBufferBehaviorEXT>(object.uniformBuffers, NULL, settings, "VkPipelineRobustnessBufferBehaviorEXT", "uniformBuffers", false, false, indents + 1, dump_json_VkPipelineRobustnessBufferBehaviorEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineRobustnessBufferBehaviorEXT>(object.vertexInputs, NULL, settings, "VkPipelineRobustnessBufferBehaviorEXT", "vertexInputs", false, false, indents + 1, dump_json_VkPipelineRobustnessBufferBehaviorEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineRobustnessImageBehaviorEXT>(object.images, NULL, settings, "VkPipelineRobustnessImageBehaviorEXT", "images", false, false, indents + 1, dump_json_VkPipelineRobustnessImageBehaviorEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkImportMemoryWin32HandleInfoKHR(const VkImportMemoryWin32HandleInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const HANDLE>(object.handle, NULL, settings, "HANDLE", "handle", false, false, indents + 1, dump_json_HANDLE);
    settings.stream() << ",\n";
    dump_json_value<const LPCWSTR>(object.name, NULL, settings, "LPCWSTR", "name", false, false, indents + 1, dump_json_LPCWSTR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkExportMemoryWin32HandleInfoKHR(const VkExportMemoryWin32HandleInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const SECURITY_ATTRIBUTES*>(object.pAttributes, NULL, settings, "const SECURITY_ATTRIBUTES*", "pAttributes", false, false, indents + 1, dump_json_SECURITY_ATTRIBUTES);
    settings.stream() << ",\n";
    dump_json_value<const DWORD>(object.dwAccess, NULL, settings, "DWORD", "dwAccess", false, false, indents + 1, dump_json_DWORD);
    settings.stream() << ",\n";
    dump_json_value<const LPCWSTR>(object.name, NULL, settings, "LPCWSTR", "name", false, false, indents + 1, dump_json_LPCWSTR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkMemoryWin32HandlePropertiesKHR(const VkMemoryWin32HandlePropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryTypeBits, NULL, settings, "uint32_t", "memoryTypeBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkMemoryGetWin32HandleInfoKHR(const VkMemoryGetWin32HandleInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
void dump_json_VkImportMemoryFdInfoKHR(const VkImportMemoryFdInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const int>(object.fd, NULL, settings, "int", "fd", false, false, indents + 1, dump_json_int);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryFdPropertiesKHR(const VkMemoryFdPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryTypeBits, NULL, settings, "uint32_t", "memoryTypeBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryGetFdInfoKHR(const VkMemoryGetFdInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkWin32KeyedMutexAcquireReleaseInfoKHR(const VkWin32KeyedMutexAcquireReleaseInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.acquireCount, NULL, settings, "uint32_t", "acquireCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDeviceMemory>(object.pAcquireSyncs, object.acquireCount, settings, "const VkDeviceMemory*", "const VkDeviceMemory", "pAcquireSyncs", false, false, indents + 1, dump_json_VkDeviceMemory); // JQA
    settings.stream() << ",\n";
    dump_json_array<const uint64_t>(object.pAcquireKeys, object.acquireCount, settings, "const uint64_t*", "const uint64_t", "pAcquireKeys", false, false, indents + 1, dump_json_uint64_t); // JQA
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pAcquireTimeouts, object.acquireCount, settings, "const uint32_t*", "const uint32_t", "pAcquireTimeouts", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.releaseCount, NULL, settings, "uint32_t", "releaseCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDeviceMemory>(object.pReleaseSyncs, object.releaseCount, settings, "const VkDeviceMemory*", "const VkDeviceMemory", "pReleaseSyncs", false, false, indents + 1, dump_json_VkDeviceMemory); // JQA
    settings.stream() << ",\n";
    dump_json_array<const uint64_t>(object.pReleaseKeys, object.releaseCount, settings, "const uint64_t*", "const uint64_t", "pReleaseKeys", false, false, indents + 1, dump_json_uint64_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkImportSemaphoreWin32HandleInfoKHR(const VkImportSemaphoreWin32HandleInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphore>(object.semaphore, NULL, settings, "VkSemaphore", "semaphore", false, false, indents + 1, dump_json_VkSemaphore);
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphoreImportFlags>(object.flags, NULL, settings, "VkSemaphoreImportFlags", "flags", false, false, indents + 1, dump_json_VkSemaphoreImportFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalSemaphoreHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalSemaphoreHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalSemaphoreHandleTypeFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const HANDLE>(object.handle, NULL, settings, "HANDLE", "handle", false, false, indents + 1, dump_json_HANDLE);
    settings.stream() << ",\n";
    dump_json_value<const LPCWSTR>(object.name, NULL, settings, "LPCWSTR", "name", false, false, indents + 1, dump_json_LPCWSTR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkExportSemaphoreWin32HandleInfoKHR(const VkExportSemaphoreWin32HandleInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const SECURITY_ATTRIBUTES*>(object.pAttributes, NULL, settings, "const SECURITY_ATTRIBUTES*", "pAttributes", false, false, indents + 1, dump_json_SECURITY_ATTRIBUTES);
    settings.stream() << ",\n";
    dump_json_value<const DWORD>(object.dwAccess, NULL, settings, "DWORD", "dwAccess", false, false, indents + 1, dump_json_DWORD);
    settings.stream() << ",\n";
    dump_json_value<const LPCWSTR>(object.name, NULL, settings, "LPCWSTR", "name", false, false, indents + 1, dump_json_LPCWSTR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkD3D12FenceSubmitInfoKHR(const VkD3D12FenceSubmitInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.waitSemaphoreValuesCount, NULL, settings, "uint32_t", "waitSemaphoreValuesCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint64_t>(object.pWaitSemaphoreValues, object.waitSemaphoreValuesCount, settings, "const uint64_t*", "const uint64_t", "pWaitSemaphoreValues", false, false, indents + 1, dump_json_uint64_t); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.signalSemaphoreValuesCount, NULL, settings, "uint32_t", "signalSemaphoreValuesCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint64_t>(object.pSignalSemaphoreValues, object.signalSemaphoreValuesCount, settings, "const uint64_t*", "const uint64_t", "pSignalSemaphoreValues", false, false, indents + 1, dump_json_uint64_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkSemaphoreGetWin32HandleInfoKHR(const VkSemaphoreGetWin32HandleInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphore>(object.semaphore, NULL, settings, "VkSemaphore", "semaphore", false, false, indents + 1, dump_json_VkSemaphore);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalSemaphoreHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalSemaphoreHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalSemaphoreHandleTypeFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
void dump_json_VkImportSemaphoreFdInfoKHR(const VkImportSemaphoreFdInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphore>(object.semaphore, NULL, settings, "VkSemaphore", "semaphore", false, false, indents + 1, dump_json_VkSemaphore);
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphoreImportFlags>(object.flags, NULL, settings, "VkSemaphoreImportFlags", "flags", false, false, indents + 1, dump_json_VkSemaphoreImportFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalSemaphoreHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalSemaphoreHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalSemaphoreHandleTypeFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const int>(object.fd, NULL, settings, "int", "fd", false, false, indents + 1, dump_json_int);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSemaphoreGetFdInfoKHR(const VkSemaphoreGetFdInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphore>(object.semaphore, NULL, settings, "VkSemaphore", "semaphore", false, false, indents + 1, dump_json_VkSemaphore);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalSemaphoreHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalSemaphoreHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalSemaphoreHandleTypeFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePushDescriptorPropertiesKHR(const VkPhysicalDevicePushDescriptorPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPushDescriptors, NULL, settings, "uint32_t", "maxPushDescriptors", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkConditionalRenderingBeginInfoEXT(const VkConditionalRenderingBeginInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.offset, NULL, settings, "VkDeviceSize", "offset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkConditionalRenderingFlagsEXT>(object.flags, NULL, settings, "VkConditionalRenderingFlagsEXT", "flags", false, false, indents + 1, dump_json_VkConditionalRenderingFlagsEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceConditionalRenderingFeaturesEXT(const VkPhysicalDeviceConditionalRenderingFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.conditionalRendering, NULL, settings, "VkBool32", "conditionalRendering", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.inheritedConditionalRendering, NULL, settings, "VkBool32", "inheritedConditionalRendering", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCommandBufferInheritanceConditionalRenderingInfoEXT(const VkCommandBufferInheritanceConditionalRenderingInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.conditionalRenderingEnable, NULL, settings, "VkBool32", "conditionalRenderingEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRectLayerKHR(const VkRectLayerKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkOffset2D>(object.offset, NULL, settings, "VkOffset2D", "offset", true, false, indents + 1, dump_json_VkOffset2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.extent, NULL, settings, "VkExtent2D", "extent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.layer, NULL, settings, "uint32_t", "layer", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPresentRegionKHR(const VkPresentRegionKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.rectangleCount, NULL, settings, "uint32_t", "rectangleCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkRectLayerKHR>(object.pRectangles, object.rectangleCount, settings, "const VkRectLayerKHR*", "const VkRectLayerKHR", "pRectangles", true, false, indents + 1, dump_json_VkRectLayerKHR); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPresentRegionsKHR(const VkPresentRegionsKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.swapchainCount, NULL, settings, "uint32_t", "swapchainCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPresentRegionKHR>(object.pRegions, object.swapchainCount, settings, "const VkPresentRegionKHR*", "const VkPresentRegionKHR", "pRegions", true, false, indents + 1, dump_json_VkPresentRegionKHR); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkViewportWScalingNV(const VkViewportWScalingNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const float>(object.xcoeff, NULL, settings, "float", "xcoeff", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.ycoeff, NULL, settings, "float", "ycoeff", false, false, indents + 1, dump_json_float);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineViewportWScalingStateCreateInfoNV(const VkPipelineViewportWScalingStateCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.viewportWScalingEnable, NULL, settings, "VkBool32", "viewportWScalingEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.viewportCount, NULL, settings, "uint32_t", "viewportCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkViewportWScalingNV>(object.pViewportWScalings, object.viewportCount, settings, "const VkViewportWScalingNV*", "const VkViewportWScalingNV", "pViewportWScalings", true, false, indents + 1, dump_json_VkViewportWScalingNV); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSurfaceCapabilities2EXT(const VkSurfaceCapabilities2EXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minImageCount, NULL, settings, "uint32_t", "minImageCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxImageCount, NULL, settings, "uint32_t", "maxImageCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.currentExtent, NULL, settings, "VkExtent2D", "currentExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.minImageExtent, NULL, settings, "VkExtent2D", "minImageExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxImageExtent, NULL, settings, "VkExtent2D", "maxImageExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxImageArrayLayers, NULL, settings, "uint32_t", "maxImageArrayLayers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceTransformFlagsKHR>(object.supportedTransforms, NULL, settings, "VkSurfaceTransformFlagsKHR", "supportedTransforms", false, false, indents + 1, dump_json_VkSurfaceTransformFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceTransformFlagBitsKHR>(object.currentTransform, NULL, settings, "VkSurfaceTransformFlagBitsKHR", "currentTransform", false, false, indents + 1, dump_json_VkSurfaceTransformFlagBitsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkCompositeAlphaFlagsKHR>(object.supportedCompositeAlpha, NULL, settings, "VkCompositeAlphaFlagsKHR", "supportedCompositeAlpha", false, false, indents + 1, dump_json_VkCompositeAlphaFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkImageUsageFlags>(object.supportedUsageFlags, NULL, settings, "VkImageUsageFlags", "supportedUsageFlags", false, false, indents + 1, dump_json_VkImageUsageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceCounterFlagsEXT>(object.supportedSurfaceCounters, NULL, settings, "VkSurfaceCounterFlagsEXT", "supportedSurfaceCounters", false, false, indents + 1, dump_json_VkSurfaceCounterFlagsEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplayPowerInfoEXT(const VkDisplayPowerInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDisplayPowerStateEXT>(object.powerState, NULL, settings, "VkDisplayPowerStateEXT", "powerState", false, false, indents + 1, dump_json_VkDisplayPowerStateEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceEventInfoEXT(const VkDeviceEventInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceEventTypeEXT>(object.deviceEvent, NULL, settings, "VkDeviceEventTypeEXT", "deviceEvent", false, false, indents + 1, dump_json_VkDeviceEventTypeEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplayEventInfoEXT(const VkDisplayEventInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDisplayEventTypeEXT>(object.displayEvent, NULL, settings, "VkDisplayEventTypeEXT", "displayEvent", false, false, indents + 1, dump_json_VkDisplayEventTypeEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSwapchainCounterCreateInfoEXT(const VkSwapchainCounterCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceCounterFlagsEXT>(object.surfaceCounters, NULL, settings, "VkSurfaceCounterFlagsEXT", "surfaceCounters", false, false, indents + 1, dump_json_VkSurfaceCounterFlagsEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRefreshCycleDurationGOOGLE(const VkRefreshCycleDurationGOOGLE& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint64_t>(object.refreshDuration, NULL, settings, "uint64_t", "refreshDuration", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPastPresentationTimingGOOGLE(const VkPastPresentationTimingGOOGLE& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.presentID, NULL, settings, "uint32_t", "presentID", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.desiredPresentTime, NULL, settings, "uint64_t", "desiredPresentTime", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.actualPresentTime, NULL, settings, "uint64_t", "actualPresentTime", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.earliestPresentTime, NULL, settings, "uint64_t", "earliestPresentTime", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.presentMargin, NULL, settings, "uint64_t", "presentMargin", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPresentTimeGOOGLE(const VkPresentTimeGOOGLE& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.presentID, NULL, settings, "uint32_t", "presentID", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.desiredPresentTime, NULL, settings, "uint64_t", "desiredPresentTime", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPresentTimesInfoGOOGLE(const VkPresentTimesInfoGOOGLE& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.swapchainCount, NULL, settings, "uint32_t", "swapchainCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPresentTimeGOOGLE>(object.pTimes, object.swapchainCount, settings, "const VkPresentTimeGOOGLE*", "const VkPresentTimeGOOGLE", "pTimes", true, false, indents + 1, dump_json_VkPresentTimeGOOGLE); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMultiviewPerViewAttributesPropertiesNVX(const VkPhysicalDeviceMultiviewPerViewAttributesPropertiesNVX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.perViewPositionAllComponents, NULL, settings, "VkBool32", "perViewPositionAllComponents", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkViewportSwizzleNV(const VkViewportSwizzleNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkViewportCoordinateSwizzleNV>(object.x, NULL, settings, "VkViewportCoordinateSwizzleNV", "x", false, false, indents + 1, dump_json_VkViewportCoordinateSwizzleNV);
    settings.stream() << ",\n";
    dump_json_value<const VkViewportCoordinateSwizzleNV>(object.y, NULL, settings, "VkViewportCoordinateSwizzleNV", "y", false, false, indents + 1, dump_json_VkViewportCoordinateSwizzleNV);
    settings.stream() << ",\n";
    dump_json_value<const VkViewportCoordinateSwizzleNV>(object.z, NULL, settings, "VkViewportCoordinateSwizzleNV", "z", false, false, indents + 1, dump_json_VkViewportCoordinateSwizzleNV);
    settings.stream() << ",\n";
    dump_json_value<const VkViewportCoordinateSwizzleNV>(object.w, NULL, settings, "VkViewportCoordinateSwizzleNV", "w", false, false, indents + 1, dump_json_VkViewportCoordinateSwizzleNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineViewportSwizzleStateCreateInfoNV(const VkPipelineViewportSwizzleStateCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineViewportSwizzleStateCreateFlagsNV>(object.flags, NULL, settings, "VkPipelineViewportSwizzleStateCreateFlagsNV", "flags", false, false, indents + 1, dump_json_VkPipelineViewportSwizzleStateCreateFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.viewportCount, NULL, settings, "uint32_t", "viewportCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkViewportSwizzleNV>(object.pViewportSwizzles, object.viewportCount, settings, "const VkViewportSwizzleNV*", "const VkViewportSwizzleNV", "pViewportSwizzles", true, false, indents + 1, dump_json_VkViewportSwizzleNV); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDiscardRectanglePropertiesEXT(const VkPhysicalDeviceDiscardRectanglePropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDiscardRectangles, NULL, settings, "uint32_t", "maxDiscardRectangles", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineDiscardRectangleStateCreateInfoEXT(const VkPipelineDiscardRectangleStateCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineDiscardRectangleStateCreateFlagsEXT>(object.flags, NULL, settings, "VkPipelineDiscardRectangleStateCreateFlagsEXT", "flags", false, false, indents + 1, dump_json_VkPipelineDiscardRectangleStateCreateFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkDiscardRectangleModeEXT>(object.discardRectangleMode, NULL, settings, "VkDiscardRectangleModeEXT", "discardRectangleMode", false, false, indents + 1, dump_json_VkDiscardRectangleModeEXT);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.discardRectangleCount, NULL, settings, "uint32_t", "discardRectangleCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkRect2D>(object.pDiscardRectangles, object.discardRectangleCount, settings, "const VkRect2D*", "const VkRect2D", "pDiscardRectangles", true, false, indents + 1, dump_json_VkRect2D); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceConservativeRasterizationPropertiesEXT(const VkPhysicalDeviceConservativeRasterizationPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const float>(object.primitiveOverestimationSize, NULL, settings, "float", "primitiveOverestimationSize", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.maxExtraPrimitiveOverestimationSize, NULL, settings, "float", "maxExtraPrimitiveOverestimationSize", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.extraPrimitiveOverestimationSizeGranularity, NULL, settings, "float", "extraPrimitiveOverestimationSizeGranularity", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.primitiveUnderestimation, NULL, settings, "VkBool32", "primitiveUnderestimation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.conservativePointAndLineRasterization, NULL, settings, "VkBool32", "conservativePointAndLineRasterization", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.degenerateTrianglesRasterized, NULL, settings, "VkBool32", "degenerateTrianglesRasterized", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.degenerateLinesRasterized, NULL, settings, "VkBool32", "degenerateLinesRasterized", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fullyCoveredFragmentShaderInputVariable, NULL, settings, "VkBool32", "fullyCoveredFragmentShaderInputVariable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.conservativeRasterizationPostDepthCoverage, NULL, settings, "VkBool32", "conservativeRasterizationPostDepthCoverage", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineRasterizationConservativeStateCreateInfoEXT(const VkPipelineRasterizationConservativeStateCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineRasterizationConservativeStateCreateFlagsEXT>(object.flags, NULL, settings, "VkPipelineRasterizationConservativeStateCreateFlagsEXT", "flags", false, false, indents + 1, dump_json_VkPipelineRasterizationConservativeStateCreateFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkConservativeRasterizationModeEXT>(object.conservativeRasterizationMode, NULL, settings, "VkConservativeRasterizationModeEXT", "conservativeRasterizationMode", false, false, indents + 1, dump_json_VkConservativeRasterizationModeEXT);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.extraPrimitiveOverestimationSize, NULL, settings, "float", "extraPrimitiveOverestimationSize", false, false, indents + 1, dump_json_float);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDepthClipEnableFeaturesEXT(const VkPhysicalDeviceDepthClipEnableFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthClipEnable, NULL, settings, "VkBool32", "depthClipEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineRasterizationDepthClipStateCreateInfoEXT(const VkPipelineRasterizationDepthClipStateCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineRasterizationDepthClipStateCreateFlagsEXT>(object.flags, NULL, settings, "VkPipelineRasterizationDepthClipStateCreateFlagsEXT", "flags", false, false, indents + 1, dump_json_VkPipelineRasterizationDepthClipStateCreateFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthClipEnable, NULL, settings, "VkBool32", "depthClipEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkXYColorEXT(const VkXYColorEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const float>(object.x, NULL, settings, "float", "x", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.y, NULL, settings, "float", "y", false, false, indents + 1, dump_json_float);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkHdrMetadataEXT(const VkHdrMetadataEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkXYColorEXT>(object.displayPrimaryRed, NULL, settings, "VkXYColorEXT", "displayPrimaryRed", true, false, indents + 1, dump_json_VkXYColorEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkXYColorEXT>(object.displayPrimaryGreen, NULL, settings, "VkXYColorEXT", "displayPrimaryGreen", true, false, indents + 1, dump_json_VkXYColorEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkXYColorEXT>(object.displayPrimaryBlue, NULL, settings, "VkXYColorEXT", "displayPrimaryBlue", true, false, indents + 1, dump_json_VkXYColorEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkXYColorEXT>(object.whitePoint, NULL, settings, "VkXYColorEXT", "whitePoint", true, false, indents + 1, dump_json_VkXYColorEXT);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.maxLuminance, NULL, settings, "float", "maxLuminance", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.minLuminance, NULL, settings, "float", "minLuminance", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.maxContentLightLevel, NULL, settings, "float", "maxContentLightLevel", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.maxFrameAverageLightLevel, NULL, settings, "float", "maxFrameAverageLightLevel", false, false, indents + 1, dump_json_float);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRelaxedLineRasterizationFeaturesIMG(const VkPhysicalDeviceRelaxedLineRasterizationFeaturesIMG& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.relaxedLineRasterization, NULL, settings, "VkBool32", "relaxedLineRasterization", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSharedPresentSurfaceCapabilitiesKHR(const VkSharedPresentSurfaceCapabilitiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageUsageFlags>(object.sharedPresentSupportedUsageFlags, NULL, settings, "VkImageUsageFlags", "sharedPresentSupportedUsageFlags", false, false, indents + 1, dump_json_VkImageUsageFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkImportFenceWin32HandleInfoKHR(const VkImportFenceWin32HandleInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFence>(object.fence, NULL, settings, "VkFence", "fence", false, false, indents + 1, dump_json_VkFence);
    settings.stream() << ",\n";
    dump_json_value<const VkFenceImportFlags>(object.flags, NULL, settings, "VkFenceImportFlags", "flags", false, false, indents + 1, dump_json_VkFenceImportFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalFenceHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalFenceHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalFenceHandleTypeFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const HANDLE>(object.handle, NULL, settings, "HANDLE", "handle", false, false, indents + 1, dump_json_HANDLE);
    settings.stream() << ",\n";
    dump_json_value<const LPCWSTR>(object.name, NULL, settings, "LPCWSTR", "name", false, false, indents + 1, dump_json_LPCWSTR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkExportFenceWin32HandleInfoKHR(const VkExportFenceWin32HandleInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const SECURITY_ATTRIBUTES*>(object.pAttributes, NULL, settings, "const SECURITY_ATTRIBUTES*", "pAttributes", false, false, indents + 1, dump_json_SECURITY_ATTRIBUTES);
    settings.stream() << ",\n";
    dump_json_value<const DWORD>(object.dwAccess, NULL, settings, "DWORD", "dwAccess", false, false, indents + 1, dump_json_DWORD);
    settings.stream() << ",\n";
    dump_json_value<const LPCWSTR>(object.name, NULL, settings, "LPCWSTR", "name", false, false, indents + 1, dump_json_LPCWSTR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkFenceGetWin32HandleInfoKHR(const VkFenceGetWin32HandleInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFence>(object.fence, NULL, settings, "VkFence", "fence", false, false, indents + 1, dump_json_VkFence);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalFenceHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalFenceHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalFenceHandleTypeFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
void dump_json_VkImportFenceFdInfoKHR(const VkImportFenceFdInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFence>(object.fence, NULL, settings, "VkFence", "fence", false, false, indents + 1, dump_json_VkFence);
    settings.stream() << ",\n";
    dump_json_value<const VkFenceImportFlags>(object.flags, NULL, settings, "VkFenceImportFlags", "flags", false, false, indents + 1, dump_json_VkFenceImportFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalFenceHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalFenceHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalFenceHandleTypeFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const int>(object.fd, NULL, settings, "int", "fd", false, false, indents + 1, dump_json_int);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkFenceGetFdInfoKHR(const VkFenceGetFdInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFence>(object.fence, NULL, settings, "VkFence", "fence", false, false, indents + 1, dump_json_VkFence);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalFenceHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalFenceHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalFenceHandleTypeFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePerformanceQueryFeaturesKHR(const VkPhysicalDevicePerformanceQueryFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.performanceCounterQueryPools, NULL, settings, "VkBool32", "performanceCounterQueryPools", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.performanceCounterMultipleQueryPools, NULL, settings, "VkBool32", "performanceCounterMultipleQueryPools", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePerformanceQueryPropertiesKHR(const VkPhysicalDevicePerformanceQueryPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.allowCommandBufferQueryCopies, NULL, settings, "VkBool32", "allowCommandBufferQueryCopies", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPerformanceCounterKHR(const VkPerformanceCounterKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPerformanceCounterUnitKHR>(object.unit, NULL, settings, "VkPerformanceCounterUnitKHR", "unit", false, false, indents + 1, dump_json_VkPerformanceCounterUnitKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkPerformanceCounterScopeKHR>(object.scope, NULL, settings, "VkPerformanceCounterScopeKHR", "scope", false, false, indents + 1, dump_json_VkPerformanceCounterScopeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkPerformanceCounterStorageKHR>(object.storage, NULL, settings, "VkPerformanceCounterStorageKHR", "storage", false, false, indents + 1, dump_json_VkPerformanceCounterStorageKHR);
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.uuid, 16, settings, "uint8_t[VK_UUID_SIZE]", "uint8_t", "uuid", false, false, indents + 1, dump_json_uint8_t); // IQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPerformanceCounterDescriptionKHR(const VkPerformanceCounterDescriptionKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPerformanceCounterDescriptionFlagsKHR>(object.flags, NULL, settings, "VkPerformanceCounterDescriptionFlagsKHR", "flags", false, false, indents + 1, dump_json_VkPerformanceCounterDescriptionFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.name, NULL, settings, "char[VK_MAX_DESCRIPTION_SIZE]", "name", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.category, NULL, settings, "char[VK_MAX_DESCRIPTION_SIZE]", "category", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.description, NULL, settings, "char[VK_MAX_DESCRIPTION_SIZE]", "description", false, false, indents + 1, dump_json_cstring);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkQueryPoolPerformanceCreateInfoKHR(const VkQueryPoolPerformanceCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queueFamilyIndex, NULL, settings, "uint32_t", "queueFamilyIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.counterIndexCount, NULL, settings, "uint32_t", "counterIndexCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pCounterIndices, object.counterIndexCount, settings, "const uint32_t*", "const uint32_t", "pCounterIndices", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAcquireProfilingLockInfoKHR(const VkAcquireProfilingLockInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAcquireProfilingLockFlagsKHR>(object.flags, NULL, settings, "VkAcquireProfilingLockFlagsKHR", "flags", false, false, indents + 1, dump_json_VkAcquireProfilingLockFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.timeout, NULL, settings, "uint64_t", "timeout", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPerformanceQuerySubmitInfoKHR(const VkPerformanceQuerySubmitInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.counterPassIndex, NULL, settings, "uint32_t", "counterPassIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSurfaceInfo2KHR(const VkPhysicalDeviceSurfaceInfo2KHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceKHR>(object.surface, NULL, settings, "VkSurfaceKHR", "surface", false, false, indents + 1, dump_json_VkSurfaceKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSurfaceCapabilities2KHR(const VkSurfaceCapabilities2KHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceCapabilitiesKHR>(object.surfaceCapabilities, NULL, settings, "VkSurfaceCapabilitiesKHR", "surfaceCapabilities", true, false, indents + 1, dump_json_VkSurfaceCapabilitiesKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSurfaceFormat2KHR(const VkSurfaceFormat2KHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceFormatKHR>(object.surfaceFormat, NULL, settings, "VkSurfaceFormatKHR", "surfaceFormat", true, false, indents + 1, dump_json_VkSurfaceFormatKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplayProperties2KHR(const VkDisplayProperties2KHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDisplayPropertiesKHR>(object.displayProperties, NULL, settings, "VkDisplayPropertiesKHR", "displayProperties", true, false, indents + 1, dump_json_VkDisplayPropertiesKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplayPlaneProperties2KHR(const VkDisplayPlaneProperties2KHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDisplayPlanePropertiesKHR>(object.displayPlaneProperties, NULL, settings, "VkDisplayPlanePropertiesKHR", "displayPlaneProperties", true, false, indents + 1, dump_json_VkDisplayPlanePropertiesKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplayModeProperties2KHR(const VkDisplayModeProperties2KHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDisplayModePropertiesKHR>(object.displayModeProperties, NULL, settings, "VkDisplayModePropertiesKHR", "displayModeProperties", true, false, indents + 1, dump_json_VkDisplayModePropertiesKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplayPlaneInfo2KHR(const VkDisplayPlaneInfo2KHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDisplayModeKHR>(object.mode, NULL, settings, "VkDisplayModeKHR", "mode", false, false, indents + 1, dump_json_VkDisplayModeKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.planeIndex, NULL, settings, "uint32_t", "planeIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplayPlaneCapabilities2KHR(const VkDisplayPlaneCapabilities2KHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDisplayPlaneCapabilitiesKHR>(object.capabilities, NULL, settings, "VkDisplayPlaneCapabilitiesKHR", "capabilities", true, false, indents + 1, dump_json_VkDisplayPlaneCapabilitiesKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_IOS_MVK)
void dump_json_VkIOSSurfaceCreateInfoMVK(const VkIOSSurfaceCreateInfoMVK& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkIOSSurfaceCreateFlagsMVK>(object.flags, NULL, settings, "VkIOSSurfaceCreateFlagsMVK", "flags", false, false, indents + 1, dump_json_VkIOSSurfaceCreateFlagsMVK);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pView, NULL, settings, "const void*", "pView", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_IOS_MVK
#if defined(VK_USE_PLATFORM_MACOS_MVK)
void dump_json_VkMacOSSurfaceCreateInfoMVK(const VkMacOSSurfaceCreateInfoMVK& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkMacOSSurfaceCreateFlagsMVK>(object.flags, NULL, settings, "VkMacOSSurfaceCreateFlagsMVK", "flags", false, false, indents + 1, dump_json_VkMacOSSurfaceCreateFlagsMVK);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pView, NULL, settings, "const void*", "pView", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_MACOS_MVK
void dump_json_VkDebugUtilsLabelEXT(const VkDebugUtilsLabelEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.pLabelName, NULL, settings, "const char*", "pLabelName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_array<const float>(object.color, 4, settings, "float[4]", "float", "color", false, false, indents + 1, dump_json_float); // IQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDebugUtilsObjectNameInfoEXT(const VkDebugUtilsObjectNameInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkObjectType>(object.objectType, NULL, settings, "VkObjectType", "objectType", false, false, indents + 1, dump_json_VkObjectType);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.objectHandle, NULL, settings, "uint64_t", "objectHandle", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.pObjectName, NULL, settings, "const char*", "pObjectName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDebugUtilsMessengerCallbackDataEXT(const VkDebugUtilsMessengerCallbackDataEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDebugUtilsMessengerCallbackDataFlagsEXT>(object.flags, NULL, settings, "VkDebugUtilsMessengerCallbackDataFlagsEXT", "flags", false, false, indents + 1, dump_json_VkDebugUtilsMessengerCallbackDataFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.pMessageIdName, NULL, settings, "const char*", "pMessageIdName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.messageIdNumber, NULL, settings, "int32_t", "messageIdNumber", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.pMessage, NULL, settings, "const char*", "pMessage", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queueLabelCount, NULL, settings, "uint32_t", "queueLabelCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDebugUtilsLabelEXT>(object.pQueueLabels, object.queueLabelCount, settings, "const VkDebugUtilsLabelEXT*", "const VkDebugUtilsLabelEXT", "pQueueLabels", true, false, indents + 1, dump_json_VkDebugUtilsLabelEXT); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.cmdBufLabelCount, NULL, settings, "uint32_t", "cmdBufLabelCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDebugUtilsLabelEXT>(object.pCmdBufLabels, object.cmdBufLabelCount, settings, "const VkDebugUtilsLabelEXT*", "const VkDebugUtilsLabelEXT", "pCmdBufLabels", true, false, indents + 1, dump_json_VkDebugUtilsLabelEXT); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.objectCount, NULL, settings, "uint32_t", "objectCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDebugUtilsObjectNameInfoEXT>(object.pObjects, object.objectCount, settings, "const VkDebugUtilsObjectNameInfoEXT*", "const VkDebugUtilsObjectNameInfoEXT", "pObjects", true, false, indents + 1, dump_json_VkDebugUtilsObjectNameInfoEXT); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDebugUtilsMessengerCreateInfoEXT(const VkDebugUtilsMessengerCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDebugUtilsMessengerCreateFlagsEXT>(object.flags, NULL, settings, "VkDebugUtilsMessengerCreateFlagsEXT", "flags", false, false, indents + 1, dump_json_VkDebugUtilsMessengerCreateFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkDebugUtilsMessageSeverityFlagsEXT>(object.messageSeverity, NULL, settings, "VkDebugUtilsMessageSeverityFlagsEXT", "messageSeverity", false, false, indents + 1, dump_json_VkDebugUtilsMessageSeverityFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkDebugUtilsMessageTypeFlagsEXT>(object.messageType, NULL, settings, "VkDebugUtilsMessageTypeFlagsEXT", "messageType", false, false, indents + 1, dump_json_VkDebugUtilsMessageTypeFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const PFN_vkDebugUtilsMessengerCallbackEXT>(object.pfnUserCallback, NULL, settings, "PFN_vkDebugUtilsMessengerCallbackEXT", "pfnUserCallback", false, false, indents + 1, dump_json_PFN_vkDebugUtilsMessengerCallbackEXT);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pUserData, NULL, settings, "void*", "pUserData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDebugUtilsObjectTagInfoEXT(const VkDebugUtilsObjectTagInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkObjectType>(object.objectType, NULL, settings, "VkObjectType", "objectType", false, false, indents + 1, dump_json_VkObjectType);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.objectHandle, NULL, settings, "uint64_t", "objectHandle", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.tagName, NULL, settings, "uint64_t", "tagName", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.tagSize, NULL, settings, "size_t", "tagSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pTag, NULL, settings, "const void*", "pTag", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_VkAndroidHardwareBufferUsageANDROID(const VkAndroidHardwareBufferUsageANDROID& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.androidHardwareBufferUsage, NULL, settings, "uint64_t", "androidHardwareBufferUsage", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_VkAndroidHardwareBufferPropertiesANDROID(const VkAndroidHardwareBufferPropertiesANDROID& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.allocationSize, NULL, settings, "VkDeviceSize", "allocationSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryTypeBits, NULL, settings, "uint32_t", "memoryTypeBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_VkAndroidHardwareBufferFormatPropertiesANDROID(const VkAndroidHardwareBufferFormatPropertiesANDROID& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.externalFormat, NULL, settings, "uint64_t", "externalFormat", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const VkFormatFeatureFlags>(object.formatFeatures, NULL, settings, "VkFormatFeatureFlags", "formatFeatures", false, false, indents + 1, dump_json_VkFormatFeatureFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentMapping>(object.samplerYcbcrConversionComponents, NULL, settings, "VkComponentMapping", "samplerYcbcrConversionComponents", true, false, indents + 1, dump_json_VkComponentMapping);
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerYcbcrModelConversion>(object.suggestedYcbcrModel, NULL, settings, "VkSamplerYcbcrModelConversion", "suggestedYcbcrModel", false, false, indents + 1, dump_json_VkSamplerYcbcrModelConversion);
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerYcbcrRange>(object.suggestedYcbcrRange, NULL, settings, "VkSamplerYcbcrRange", "suggestedYcbcrRange", false, false, indents + 1, dump_json_VkSamplerYcbcrRange);
    settings.stream() << ",\n";
    dump_json_value<const VkChromaLocation>(object.suggestedXChromaOffset, NULL, settings, "VkChromaLocation", "suggestedXChromaOffset", false, false, indents + 1, dump_json_VkChromaLocation);
    settings.stream() << ",\n";
    dump_json_value<const VkChromaLocation>(object.suggestedYChromaOffset, NULL, settings, "VkChromaLocation", "suggestedYChromaOffset", false, false, indents + 1, dump_json_VkChromaLocation);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_VkImportAndroidHardwareBufferInfoANDROID(const VkImportAndroidHardwareBufferInfoANDROID& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const AHardwareBuffer*>(object.buffer, NULL, settings, "struct AHardwareBuffer*", "buffer", false, false, indents + 1, dump_json_AHardwareBuffer);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_VkMemoryGetAndroidHardwareBufferInfoANDROID(const VkMemoryGetAndroidHardwareBufferInfoANDROID& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_VkExternalFormatANDROID(const VkExternalFormatANDROID& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.externalFormat, NULL, settings, "uint64_t", "externalFormat", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_VkAndroidHardwareBufferFormatProperties2ANDROID(const VkAndroidHardwareBufferFormatProperties2ANDROID& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.externalFormat, NULL, settings, "uint64_t", "externalFormat", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const VkFormatFeatureFlags2>(object.formatFeatures, NULL, settings, "VkFormatFeatureFlags2", "formatFeatures", false, false, indents + 1, dump_json_VkFormatFeatureFlags2);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentMapping>(object.samplerYcbcrConversionComponents, NULL, settings, "VkComponentMapping", "samplerYcbcrConversionComponents", true, false, indents + 1, dump_json_VkComponentMapping);
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerYcbcrModelConversion>(object.suggestedYcbcrModel, NULL, settings, "VkSamplerYcbcrModelConversion", "suggestedYcbcrModel", false, false, indents + 1, dump_json_VkSamplerYcbcrModelConversion);
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerYcbcrRange>(object.suggestedYcbcrRange, NULL, settings, "VkSamplerYcbcrRange", "suggestedYcbcrRange", false, false, indents + 1, dump_json_VkSamplerYcbcrRange);
    settings.stream() << ",\n";
    dump_json_value<const VkChromaLocation>(object.suggestedXChromaOffset, NULL, settings, "VkChromaLocation", "suggestedXChromaOffset", false, false, indents + 1, dump_json_VkChromaLocation);
    settings.stream() << ",\n";
    dump_json_value<const VkChromaLocation>(object.suggestedYChromaOffset, NULL, settings, "VkChromaLocation", "suggestedYChromaOffset", false, false, indents + 1, dump_json_VkChromaLocation);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_VkPhysicalDeviceShaderEnqueueFeaturesAMDX(const VkPhysicalDeviceShaderEnqueueFeaturesAMDX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderEnqueue, NULL, settings, "VkBool32", "shaderEnqueue", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_VkPhysicalDeviceShaderEnqueuePropertiesAMDX(const VkPhysicalDeviceShaderEnqueuePropertiesAMDX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxExecutionGraphDepth, NULL, settings, "uint32_t", "maxExecutionGraphDepth", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxExecutionGraphShaderOutputNodes, NULL, settings, "uint32_t", "maxExecutionGraphShaderOutputNodes", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxExecutionGraphShaderPayloadSize, NULL, settings, "uint32_t", "maxExecutionGraphShaderPayloadSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxExecutionGraphShaderPayloadCount, NULL, settings, "uint32_t", "maxExecutionGraphShaderPayloadCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.executionGraphDispatchAddressAlignment, NULL, settings, "uint32_t", "executionGraphDispatchAddressAlignment", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_VkExecutionGraphPipelineScratchSizeAMDX(const VkExecutionGraphPipelineScratchSizeAMDX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_ENABLE_BETA_EXTENSIONS
void dump_json_VkPipelineLibraryCreateInfoKHR(const VkPipelineLibraryCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.libraryCount, NULL, settings, "uint32_t", "libraryCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPipeline>(object.pLibraries, object.libraryCount, settings, "const VkPipeline*", "const VkPipeline", "pLibraries", false, false, indents + 1, dump_json_VkPipeline); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_VkExecutionGraphPipelineCreateInfoAMDX(const VkExecutionGraphPipelineCreateInfoAMDX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineCreateFlags>(object.flags, NULL, settings, "VkPipelineCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stageCount, NULL, settings, "uint32_t", "stageCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPipelineShaderStageCreateInfo>(object.pStages, object.stageCount, settings, "const VkPipelineShaderStageCreateInfo*", "const VkPipelineShaderStageCreateInfo", "pStages", true, false, indents + 1, dump_json_VkPipelineShaderStageCreateInfo); // JQA
    settings.stream() << ",\n";
    dump_json_pointer<const VkPipelineLibraryCreateInfoKHR>(object.pLibraryInfo, settings, "const VkPipelineLibraryCreateInfoKHR*", "pLibraryInfo", true, false, indents + 1, dump_json_VkPipelineLibraryCreateInfoKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineLayout>(object.layout, NULL, settings, "VkPipelineLayout", "layout", false, false, indents + 1, dump_json_VkPipelineLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkPipeline>(object.basePipelineHandle, NULL, settings, "VkPipeline", "basePipelineHandle", false, false, indents + 1, dump_json_VkPipeline);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.basePipelineIndex, NULL, settings, "int32_t", "basePipelineIndex", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_VkDispatchGraphInfoAMDX(const VkDispatchGraphInfoAMDX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.nodeIndex, NULL, settings, "uint32_t", "nodeIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.payloadCount, NULL, settings, "uint32_t", "payloadCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstAMDX>(object.payloads, NULL, settings, "VkDeviceOrHostAddressConstAMDX", "payloads", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstAMDX);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.payloadStride, NULL, settings, "uint64_t", "payloadStride", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_VkDispatchGraphCountInfoAMDX(const VkDispatchGraphCountInfoAMDX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.count, NULL, settings, "uint32_t", "count", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstAMDX>(object.infos, NULL, settings, "VkDeviceOrHostAddressConstAMDX", "infos", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstAMDX);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.stride, NULL, settings, "uint64_t", "stride", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_VkPipelineShaderStageNodeCreateInfoAMDX(const VkPipelineShaderStageNodeCreateInfoAMDX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.pName, NULL, settings, "const char*", "pName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.index, NULL, settings, "uint32_t", "index", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_ENABLE_BETA_EXTENSIONS
void dump_json_VkSampleLocationEXT(const VkSampleLocationEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const float>(object.x, NULL, settings, "float", "x", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.y, NULL, settings, "float", "y", false, false, indents + 1, dump_json_float);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSampleLocationsInfoEXT(const VkSampleLocationsInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlagBits>(object.sampleLocationsPerPixel, NULL, settings, "VkSampleCountFlagBits", "sampleLocationsPerPixel", false, false, indents + 1, dump_json_VkSampleCountFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.sampleLocationGridSize, NULL, settings, "VkExtent2D", "sampleLocationGridSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.sampleLocationsCount, NULL, settings, "uint32_t", "sampleLocationsCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSampleLocationEXT>(object.pSampleLocations, object.sampleLocationsCount, settings, "const VkSampleLocationEXT*", "const VkSampleLocationEXT", "pSampleLocations", true, false, indents + 1, dump_json_VkSampleLocationEXT); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAttachmentSampleLocationsEXT(const VkAttachmentSampleLocationsEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.attachmentIndex, NULL, settings, "uint32_t", "attachmentIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleLocationsInfoEXT>(object.sampleLocationsInfo, NULL, settings, "VkSampleLocationsInfoEXT", "sampleLocationsInfo", true, false, indents + 1, dump_json_VkSampleLocationsInfoEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSubpassSampleLocationsEXT(const VkSubpassSampleLocationsEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.subpassIndex, NULL, settings, "uint32_t", "subpassIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleLocationsInfoEXT>(object.sampleLocationsInfo, NULL, settings, "VkSampleLocationsInfoEXT", "sampleLocationsInfo", true, false, indents + 1, dump_json_VkSampleLocationsInfoEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassSampleLocationsBeginInfoEXT(const VkRenderPassSampleLocationsBeginInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.attachmentInitialSampleLocationsCount, NULL, settings, "uint32_t", "attachmentInitialSampleLocationsCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkAttachmentSampleLocationsEXT>(object.pAttachmentInitialSampleLocations, object.attachmentInitialSampleLocationsCount, settings, "const VkAttachmentSampleLocationsEXT*", "const VkAttachmentSampleLocationsEXT", "pAttachmentInitialSampleLocations", true, false, indents + 1, dump_json_VkAttachmentSampleLocationsEXT); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.postSubpassSampleLocationsCount, NULL, settings, "uint32_t", "postSubpassSampleLocationsCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSubpassSampleLocationsEXT>(object.pPostSubpassSampleLocations, object.postSubpassSampleLocationsCount, settings, "const VkSubpassSampleLocationsEXT*", "const VkSubpassSampleLocationsEXT", "pPostSubpassSampleLocations", true, false, indents + 1, dump_json_VkSubpassSampleLocationsEXT); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineSampleLocationsStateCreateInfoEXT(const VkPipelineSampleLocationsStateCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sampleLocationsEnable, NULL, settings, "VkBool32", "sampleLocationsEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleLocationsInfoEXT>(object.sampleLocationsInfo, NULL, settings, "VkSampleLocationsInfoEXT", "sampleLocationsInfo", true, false, indents + 1, dump_json_VkSampleLocationsInfoEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSampleLocationsPropertiesEXT(const VkPhysicalDeviceSampleLocationsPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlags>(object.sampleLocationSampleCounts, NULL, settings, "VkSampleCountFlags", "sampleLocationSampleCounts", false, false, indents + 1, dump_json_VkSampleCountFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxSampleLocationGridSize, NULL, settings, "VkExtent2D", "maxSampleLocationGridSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_array<const float>(object.sampleLocationCoordinateRange, 2, settings, "float[2]", "float", "sampleLocationCoordinateRange", false, false, indents + 1, dump_json_float); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.sampleLocationSubPixelBits, NULL, settings, "uint32_t", "sampleLocationSubPixelBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.variableSampleLocations, NULL, settings, "VkBool32", "variableSampleLocations", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMultisamplePropertiesEXT(const VkMultisamplePropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxSampleLocationGridSize, NULL, settings, "VkExtent2D", "maxSampleLocationGridSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceBlendOperationAdvancedFeaturesEXT(const VkPhysicalDeviceBlendOperationAdvancedFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.advancedBlendCoherentOperations, NULL, settings, "VkBool32", "advancedBlendCoherentOperations", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceBlendOperationAdvancedPropertiesEXT(const VkPhysicalDeviceBlendOperationAdvancedPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.advancedBlendMaxColorAttachments, NULL, settings, "uint32_t", "advancedBlendMaxColorAttachments", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.advancedBlendIndependentBlend, NULL, settings, "VkBool32", "advancedBlendIndependentBlend", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.advancedBlendNonPremultipliedSrcColor, NULL, settings, "VkBool32", "advancedBlendNonPremultipliedSrcColor", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.advancedBlendNonPremultipliedDstColor, NULL, settings, "VkBool32", "advancedBlendNonPremultipliedDstColor", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.advancedBlendCorrelatedOverlap, NULL, settings, "VkBool32", "advancedBlendCorrelatedOverlap", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.advancedBlendAllOperations, NULL, settings, "VkBool32", "advancedBlendAllOperations", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineColorBlendAdvancedStateCreateInfoEXT(const VkPipelineColorBlendAdvancedStateCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.srcPremultiplied, NULL, settings, "VkBool32", "srcPremultiplied", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.dstPremultiplied, NULL, settings, "VkBool32", "dstPremultiplied", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBlendOverlapEXT>(object.blendOverlap, NULL, settings, "VkBlendOverlapEXT", "blendOverlap", false, false, indents + 1, dump_json_VkBlendOverlapEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineCoverageToColorStateCreateInfoNV(const VkPipelineCoverageToColorStateCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineCoverageToColorStateCreateFlagsNV>(object.flags, NULL, settings, "VkPipelineCoverageToColorStateCreateFlagsNV", "flags", false, false, indents + 1, dump_json_VkPipelineCoverageToColorStateCreateFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.coverageToColorEnable, NULL, settings, "VkBool32", "coverageToColorEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.coverageToColorLocation, NULL, settings, "uint32_t", "coverageToColorLocation", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureBuildRangeInfoKHR(const VkAccelerationStructureBuildRangeInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.primitiveCount, NULL, settings, "uint32_t", "primitiveCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.primitiveOffset, NULL, settings, "uint32_t", "primitiveOffset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.firstVertex, NULL, settings, "uint32_t", "firstVertex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.transformOffset, NULL, settings, "uint32_t", "transformOffset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAabbPositionsKHR(const VkAabbPositionsKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const float>(object.minX, NULL, settings, "float", "minX", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.minY, NULL, settings, "float", "minY", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.minZ, NULL, settings, "float", "minZ", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.maxX, NULL, settings, "float", "maxX", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.maxY, NULL, settings, "float", "maxY", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.maxZ, NULL, settings, "float", "maxZ", false, false, indents + 1, dump_json_float);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureGeometryTrianglesDataKHR(const VkAccelerationStructureGeometryTrianglesDataKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.vertexFormat, NULL, settings, "VkFormat", "vertexFormat", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstKHR>(object.vertexData, NULL, settings, "VkDeviceOrHostAddressConstKHR", "vertexData", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.vertexStride, NULL, settings, "VkDeviceSize", "vertexStride", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxVertex, NULL, settings, "uint32_t", "maxVertex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkIndexType>(object.indexType, NULL, settings, "VkIndexType", "indexType", false, false, indents + 1, dump_json_VkIndexType);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstKHR>(object.indexData, NULL, settings, "VkDeviceOrHostAddressConstKHR", "indexData", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstKHR>(object.transformData, NULL, settings, "VkDeviceOrHostAddressConstKHR", "transformData", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkTransformMatrixKHR(const VkTransformMatrixKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureGeometryAabbsDataKHR(const VkAccelerationStructureGeometryAabbsDataKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstKHR>(object.data, NULL, settings, "VkDeviceOrHostAddressConstKHR", "data", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.stride, NULL, settings, "VkDeviceSize", "stride", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureGeometryInstancesDataKHR(const VkAccelerationStructureGeometryInstancesDataKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.arrayOfPointers, NULL, settings, "VkBool32", "arrayOfPointers", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstKHR>(object.data, NULL, settings, "VkDeviceOrHostAddressConstKHR", "data", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureGeometryKHR(const VkAccelerationStructureGeometryKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkGeometryTypeKHR>(object.geometryType, NULL, settings, "VkGeometryTypeKHR", "geometryType", false, false, indents + 1, dump_json_VkGeometryTypeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureGeometryDataKHR>(object.geometry, NULL, settings, "VkAccelerationStructureGeometryDataKHR", "geometry", false, true, indents + 1, dump_json_VkAccelerationStructureGeometryDataKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkGeometryFlagsKHR>(object.flags, NULL, settings, "VkGeometryFlagsKHR", "flags", false, false, indents + 1, dump_json_VkGeometryFlagsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureBuildGeometryInfoKHR(const VkAccelerationStructureBuildGeometryInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureTypeKHR>(object.type, NULL, settings, "VkAccelerationStructureTypeKHR", "type", false, false, indents + 1, dump_json_VkAccelerationStructureTypeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkBuildAccelerationStructureFlagsKHR>(object.flags, NULL, settings, "VkBuildAccelerationStructureFlagsKHR", "flags", false, false, indents + 1, dump_json_VkBuildAccelerationStructureFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkBuildAccelerationStructureModeKHR>(object.mode, NULL, settings, "VkBuildAccelerationStructureModeKHR", "mode", false, false, indents + 1, dump_json_VkBuildAccelerationStructureModeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureKHR>(object.srcAccelerationStructure, NULL, settings, "VkAccelerationStructureKHR", "srcAccelerationStructure", false, false, indents + 1, dump_json_VkAccelerationStructureKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureKHR>(object.dstAccelerationStructure, NULL, settings, "VkAccelerationStructureKHR", "dstAccelerationStructure", false, false, indents + 1, dump_json_VkAccelerationStructureKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.geometryCount, NULL, settings, "uint32_t", "geometryCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkAccelerationStructureGeometryKHR>(object.pGeometries, object.geometryCount, settings, "const VkAccelerationStructureGeometryKHR*", "const VkAccelerationStructureGeometryKHR", "pGeometries", true, false, indents + 1, dump_json_VkAccelerationStructureGeometryKHR); // JQA
    settings.stream() << ",\n";
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressKHR>(object.scratchData, NULL, settings, "VkDeviceOrHostAddressKHR", "scratchData", false, true, indents + 1, dump_json_VkDeviceOrHostAddressKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureInstanceKHR(const VkAccelerationStructureInstanceKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkTransformMatrixKHR>(object.transform, NULL, settings, "VkTransformMatrixKHR", "transform", true, false, indents + 1, dump_json_VkTransformMatrixKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.instanceCustomIndex, NULL, settings, "uint32_t instanceCustomIndex:24", "instanceCustomIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.mask, NULL, settings, "uint32_t mask:8", "mask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.instanceShaderBindingTableRecordOffset, NULL, settings, "uint32_t instanceShaderBindingTableRecordOffset:24", "instanceShaderBindingTableRecordOffset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkGeometryInstanceFlagsKHR>(object.flags, NULL, settings, "VkGeometryInstanceFlagsKHR flags:8", "flags", false, false, indents + 1, dump_json_VkGeometryInstanceFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.accelerationStructureReference, NULL, settings, "uint64_t", "accelerationStructureReference", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureCreateInfoKHR(const VkAccelerationStructureCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureCreateFlagsKHR>(object.createFlags, NULL, settings, "VkAccelerationStructureCreateFlagsKHR", "createFlags", false, false, indents + 1, dump_json_VkAccelerationStructureCreateFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.offset, NULL, settings, "VkDeviceSize", "offset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureTypeKHR>(object.type, NULL, settings, "VkAccelerationStructureTypeKHR", "type", false, false, indents + 1, dump_json_VkAccelerationStructureTypeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddress>(object.deviceAddress, NULL, settings, "VkDeviceAddress", "deviceAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkWriteDescriptorSetAccelerationStructureKHR(const VkWriteDescriptorSetAccelerationStructureKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.accelerationStructureCount, NULL, settings, "uint32_t", "accelerationStructureCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkAccelerationStructureKHR>(object.pAccelerationStructures, object.accelerationStructureCount, settings, "const VkAccelerationStructureKHR*", "const VkAccelerationStructureKHR", "pAccelerationStructures", false, false, indents + 1, dump_json_VkAccelerationStructureKHR); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceAccelerationStructureFeaturesKHR(const VkPhysicalDeviceAccelerationStructureFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.accelerationStructure, NULL, settings, "VkBool32", "accelerationStructure", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.accelerationStructureCaptureReplay, NULL, settings, "VkBool32", "accelerationStructureCaptureReplay", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.accelerationStructureIndirectBuild, NULL, settings, "VkBool32", "accelerationStructureIndirectBuild", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.accelerationStructureHostCommands, NULL, settings, "VkBool32", "accelerationStructureHostCommands", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBindingAccelerationStructureUpdateAfterBind, NULL, settings, "VkBool32", "descriptorBindingAccelerationStructureUpdateAfterBind", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceAccelerationStructurePropertiesKHR(const VkPhysicalDeviceAccelerationStructurePropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.maxGeometryCount, NULL, settings, "uint64_t", "maxGeometryCount", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.maxInstanceCount, NULL, settings, "uint64_t", "maxInstanceCount", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.maxPrimitiveCount, NULL, settings, "uint64_t", "maxPrimitiveCount", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorAccelerationStructures, NULL, settings, "uint32_t", "maxPerStageDescriptorAccelerationStructures", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPerStageDescriptorUpdateAfterBindAccelerationStructures, NULL, settings, "uint32_t", "maxPerStageDescriptorUpdateAfterBindAccelerationStructures", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetAccelerationStructures, NULL, settings, "uint32_t", "maxDescriptorSetAccelerationStructures", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindAccelerationStructures, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindAccelerationStructures", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minAccelerationStructureScratchOffsetAlignment, NULL, settings, "uint32_t", "minAccelerationStructureScratchOffsetAlignment", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureDeviceAddressInfoKHR(const VkAccelerationStructureDeviceAddressInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureKHR>(object.accelerationStructure, NULL, settings, "VkAccelerationStructureKHR", "accelerationStructure", false, false, indents + 1, dump_json_VkAccelerationStructureKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureVersionInfoKHR(const VkAccelerationStructureVersionInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.pVersionData, 2*VK_UUID_SIZE, settings, "const uint8_t*", "const uint8_t", "pVersionData", false, false, indents + 1, dump_json_uint8_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyAccelerationStructureToMemoryInfoKHR(const VkCopyAccelerationStructureToMemoryInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureKHR>(object.src, NULL, settings, "VkAccelerationStructureKHR", "src", false, false, indents + 1, dump_json_VkAccelerationStructureKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressKHR>(object.dst, NULL, settings, "VkDeviceOrHostAddressKHR", "dst", false, true, indents + 1, dump_json_VkDeviceOrHostAddressKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkCopyAccelerationStructureModeKHR>(object.mode, NULL, settings, "VkCopyAccelerationStructureModeKHR", "mode", false, false, indents + 1, dump_json_VkCopyAccelerationStructureModeKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyMemoryToAccelerationStructureInfoKHR(const VkCopyMemoryToAccelerationStructureInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstKHR>(object.src, NULL, settings, "VkDeviceOrHostAddressConstKHR", "src", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureKHR>(object.dst, NULL, settings, "VkAccelerationStructureKHR", "dst", false, false, indents + 1, dump_json_VkAccelerationStructureKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkCopyAccelerationStructureModeKHR>(object.mode, NULL, settings, "VkCopyAccelerationStructureModeKHR", "mode", false, false, indents + 1, dump_json_VkCopyAccelerationStructureModeKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyAccelerationStructureInfoKHR(const VkCopyAccelerationStructureInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureKHR>(object.src, NULL, settings, "VkAccelerationStructureKHR", "src", false, false, indents + 1, dump_json_VkAccelerationStructureKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureKHR>(object.dst, NULL, settings, "VkAccelerationStructureKHR", "dst", false, false, indents + 1, dump_json_VkAccelerationStructureKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkCopyAccelerationStructureModeKHR>(object.mode, NULL, settings, "VkCopyAccelerationStructureModeKHR", "mode", false, false, indents + 1, dump_json_VkCopyAccelerationStructureModeKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureBuildSizesInfoKHR(const VkAccelerationStructureBuildSizesInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.accelerationStructureSize, NULL, settings, "VkDeviceSize", "accelerationStructureSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.updateScratchSize, NULL, settings, "VkDeviceSize", "updateScratchSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.buildScratchSize, NULL, settings, "VkDeviceSize", "buildScratchSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineCoverageModulationStateCreateInfoNV(const VkPipelineCoverageModulationStateCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineCoverageModulationStateCreateFlagsNV>(object.flags, NULL, settings, "VkPipelineCoverageModulationStateCreateFlagsNV", "flags", false, false, indents + 1, dump_json_VkPipelineCoverageModulationStateCreateFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const VkCoverageModulationModeNV>(object.coverageModulationMode, NULL, settings, "VkCoverageModulationModeNV", "coverageModulationMode", false, false, indents + 1, dump_json_VkCoverageModulationModeNV);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.coverageModulationTableEnable, NULL, settings, "VkBool32", "coverageModulationTableEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.coverageModulationTableCount, NULL, settings, "uint32_t", "coverageModulationTableCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const float>(object.pCoverageModulationTable, object.coverageModulationTableCount, settings, "const float*", "const float", "pCoverageModulationTable", false, false, indents + 1, dump_json_float); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderSMBuiltinsPropertiesNV(const VkPhysicalDeviceShaderSMBuiltinsPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shaderSMCount, NULL, settings, "uint32_t", "shaderSMCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shaderWarpsPerSM, NULL, settings, "uint32_t", "shaderWarpsPerSM", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderSMBuiltinsFeaturesNV(const VkPhysicalDeviceShaderSMBuiltinsFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSMBuiltins, NULL, settings, "VkBool32", "shaderSMBuiltins", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDrmFormatModifierPropertiesEXT(const VkDrmFormatModifierPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint64_t>(object.drmFormatModifier, NULL, settings, "uint64_t", "drmFormatModifier", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.drmFormatModifierPlaneCount, NULL, settings, "uint32_t", "drmFormatModifierPlaneCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkFormatFeatureFlags>(object.drmFormatModifierTilingFeatures, NULL, settings, "VkFormatFeatureFlags", "drmFormatModifierTilingFeatures", false, false, indents + 1, dump_json_VkFormatFeatureFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDrmFormatModifierPropertiesListEXT(const VkDrmFormatModifierPropertiesListEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.drmFormatModifierCount, NULL, settings, "uint32_t", "drmFormatModifierCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDrmFormatModifierPropertiesEXT>(object.pDrmFormatModifierProperties, object.drmFormatModifierCount, settings, "VkDrmFormatModifierPropertiesEXT*", "VkDrmFormatModifierPropertiesEXT", "pDrmFormatModifierProperties", true, false, indents + 1, dump_json_VkDrmFormatModifierPropertiesEXT); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImageDrmFormatModifierInfoEXT(const VkPhysicalDeviceImageDrmFormatModifierInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.drmFormatModifier, NULL, settings, "uint64_t", "drmFormatModifier", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const VkSharingMode>(object.sharingMode, NULL, settings, "VkSharingMode", "sharingMode", false, false, indents + 1, dump_json_VkSharingMode);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queueFamilyIndexCount, NULL, settings, "uint32_t", "queueFamilyIndexCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pQueueFamilyIndices, object.queueFamilyIndexCount, settings, "const uint32_t*", "const uint32_t", "pQueueFamilyIndices", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageDrmFormatModifierListCreateInfoEXT(const VkImageDrmFormatModifierListCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.drmFormatModifierCount, NULL, settings, "uint32_t", "drmFormatModifierCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint64_t>(object.pDrmFormatModifiers, object.drmFormatModifierCount, settings, "const uint64_t*", "const uint64_t", "pDrmFormatModifiers", false, false, indents + 1, dump_json_uint64_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageDrmFormatModifierExplicitCreateInfoEXT(const VkImageDrmFormatModifierExplicitCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.drmFormatModifier, NULL, settings, "uint64_t", "drmFormatModifier", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.drmFormatModifierPlaneCount, NULL, settings, "uint32_t", "drmFormatModifierPlaneCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSubresourceLayout>(object.pPlaneLayouts, object.drmFormatModifierPlaneCount, settings, "const VkSubresourceLayout*", "const VkSubresourceLayout", "pPlaneLayouts", true, false, indents + 1, dump_json_VkSubresourceLayout); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageDrmFormatModifierPropertiesEXT(const VkImageDrmFormatModifierPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.drmFormatModifier, NULL, settings, "uint64_t", "drmFormatModifier", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDrmFormatModifierProperties2EXT(const VkDrmFormatModifierProperties2EXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint64_t>(object.drmFormatModifier, NULL, settings, "uint64_t", "drmFormatModifier", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.drmFormatModifierPlaneCount, NULL, settings, "uint32_t", "drmFormatModifierPlaneCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkFormatFeatureFlags2>(object.drmFormatModifierTilingFeatures, NULL, settings, "VkFormatFeatureFlags2", "drmFormatModifierTilingFeatures", false, false, indents + 1, dump_json_VkFormatFeatureFlags2);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDrmFormatModifierPropertiesList2EXT(const VkDrmFormatModifierPropertiesList2EXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.drmFormatModifierCount, NULL, settings, "uint32_t", "drmFormatModifierCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDrmFormatModifierProperties2EXT>(object.pDrmFormatModifierProperties, object.drmFormatModifierCount, settings, "VkDrmFormatModifierProperties2EXT*", "VkDrmFormatModifierProperties2EXT", "pDrmFormatModifierProperties", true, false, indents + 1, dump_json_VkDrmFormatModifierProperties2EXT); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkValidationCacheCreateInfoEXT(const VkValidationCacheCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkValidationCacheCreateFlagsEXT>(object.flags, NULL, settings, "VkValidationCacheCreateFlagsEXT", "flags", false, false, indents + 1, dump_json_VkValidationCacheCreateFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.initialDataSize, NULL, settings, "size_t", "initialDataSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pInitialData, NULL, settings, "const void*", "pInitialData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkShaderModuleValidationCacheCreateInfoEXT(const VkShaderModuleValidationCacheCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkValidationCacheEXT>(object.validationCache, NULL, settings, "VkValidationCacheEXT", "validationCache", false, false, indents + 1, dump_json_VkValidationCacheEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_VkPhysicalDevicePortabilitySubsetFeaturesKHR(const VkPhysicalDevicePortabilitySubsetFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.constantAlphaColorBlendFactors, NULL, settings, "VkBool32", "constantAlphaColorBlendFactors", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.events, NULL, settings, "VkBool32", "events", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.imageViewFormatReinterpretation, NULL, settings, "VkBool32", "imageViewFormatReinterpretation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.imageViewFormatSwizzle, NULL, settings, "VkBool32", "imageViewFormatSwizzle", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.imageView2DOn3DImage, NULL, settings, "VkBool32", "imageView2DOn3DImage", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multisampleArrayImage, NULL, settings, "VkBool32", "multisampleArrayImage", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.mutableComparisonSamplers, NULL, settings, "VkBool32", "mutableComparisonSamplers", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.pointPolygons, NULL, settings, "VkBool32", "pointPolygons", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.samplerMipLodBias, NULL, settings, "VkBool32", "samplerMipLodBias", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.separateStencilMaskRef, NULL, settings, "VkBool32", "separateStencilMaskRef", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSampleRateInterpolationFunctions, NULL, settings, "VkBool32", "shaderSampleRateInterpolationFunctions", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.tessellationIsolines, NULL, settings, "VkBool32", "tessellationIsolines", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.tessellationPointMode, NULL, settings, "VkBool32", "tessellationPointMode", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.triangleFans, NULL, settings, "VkBool32", "triangleFans", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.vertexAttributeAccessBeyondStride, NULL, settings, "VkBool32", "vertexAttributeAccessBeyondStride", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_VkPhysicalDevicePortabilitySubsetPropertiesKHR(const VkPhysicalDevicePortabilitySubsetPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minVertexInputBindingStrideAlignment, NULL, settings, "uint32_t", "minVertexInputBindingStrideAlignment", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_ENABLE_BETA_EXTENSIONS
void dump_json_VkShadingRatePaletteNV(const VkShadingRatePaletteNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.shadingRatePaletteEntryCount, NULL, settings, "uint32_t", "shadingRatePaletteEntryCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkShadingRatePaletteEntryNV>(object.pShadingRatePaletteEntries, object.shadingRatePaletteEntryCount, settings, "const VkShadingRatePaletteEntryNV*", "const VkShadingRatePaletteEntryNV", "pShadingRatePaletteEntries", false, false, indents + 1, dump_json_VkShadingRatePaletteEntryNV); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineViewportShadingRateImageStateCreateInfoNV(const VkPipelineViewportShadingRateImageStateCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shadingRateImageEnable, NULL, settings, "VkBool32", "shadingRateImageEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.viewportCount, NULL, settings, "uint32_t", "viewportCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkShadingRatePaletteNV>(object.pShadingRatePalettes, object.viewportCount, settings, "const VkShadingRatePaletteNV*", "const VkShadingRatePaletteNV", "pShadingRatePalettes", true, false, indents + 1, dump_json_VkShadingRatePaletteNV); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShadingRateImageFeaturesNV(const VkPhysicalDeviceShadingRateImageFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shadingRateImage, NULL, settings, "VkBool32", "shadingRateImage", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shadingRateCoarseSampleOrder, NULL, settings, "VkBool32", "shadingRateCoarseSampleOrder", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShadingRateImagePropertiesNV(const VkPhysicalDeviceShadingRateImagePropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.shadingRateTexelSize, NULL, settings, "VkExtent2D", "shadingRateTexelSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shadingRatePaletteSize, NULL, settings, "uint32_t", "shadingRatePaletteSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shadingRateMaxCoarseSamples, NULL, settings, "uint32_t", "shadingRateMaxCoarseSamples", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCoarseSampleLocationNV(const VkCoarseSampleLocationNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.pixelX, NULL, settings, "uint32_t", "pixelX", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.pixelY, NULL, settings, "uint32_t", "pixelY", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.sample, NULL, settings, "uint32_t", "sample", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCoarseSampleOrderCustomNV(const VkCoarseSampleOrderCustomNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkShadingRatePaletteEntryNV>(object.shadingRate, NULL, settings, "VkShadingRatePaletteEntryNV", "shadingRate", false, false, indents + 1, dump_json_VkShadingRatePaletteEntryNV);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.sampleCount, NULL, settings, "uint32_t", "sampleCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.sampleLocationCount, NULL, settings, "uint32_t", "sampleLocationCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkCoarseSampleLocationNV>(object.pSampleLocations, object.sampleLocationCount, settings, "const VkCoarseSampleLocationNV*", "const VkCoarseSampleLocationNV", "pSampleLocations", true, false, indents + 1, dump_json_VkCoarseSampleLocationNV); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineViewportCoarseSampleOrderStateCreateInfoNV(const VkPipelineViewportCoarseSampleOrderStateCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkCoarseSampleOrderTypeNV>(object.sampleOrderType, NULL, settings, "VkCoarseSampleOrderTypeNV", "sampleOrderType", false, false, indents + 1, dump_json_VkCoarseSampleOrderTypeNV);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.customSampleOrderCount, NULL, settings, "uint32_t", "customSampleOrderCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkCoarseSampleOrderCustomNV>(object.pCustomSampleOrders, object.customSampleOrderCount, settings, "const VkCoarseSampleOrderCustomNV*", "const VkCoarseSampleOrderCustomNV", "pCustomSampleOrders", true, false, indents + 1, dump_json_VkCoarseSampleOrderCustomNV); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRayTracingShaderGroupCreateInfoNV(const VkRayTracingShaderGroupCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkRayTracingShaderGroupTypeKHR>(object.type, NULL, settings, "VkRayTracingShaderGroupTypeKHR", "type", false, false, indents + 1, dump_json_VkRayTracingShaderGroupTypeKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.generalShader, NULL, settings, "uint32_t", "generalShader", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.closestHitShader, NULL, settings, "uint32_t", "closestHitShader", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.anyHitShader, NULL, settings, "uint32_t", "anyHitShader", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.intersectionShader, NULL, settings, "uint32_t", "intersectionShader", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRayTracingPipelineCreateInfoNV(const VkRayTracingPipelineCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineCreateFlags>(object.flags, NULL, settings, "VkPipelineCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stageCount, NULL, settings, "uint32_t", "stageCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPipelineShaderStageCreateInfo>(object.pStages, object.stageCount, settings, "const VkPipelineShaderStageCreateInfo*", "const VkPipelineShaderStageCreateInfo", "pStages", true, false, indents + 1, dump_json_VkPipelineShaderStageCreateInfo); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.groupCount, NULL, settings, "uint32_t", "groupCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkRayTracingShaderGroupCreateInfoNV>(object.pGroups, object.groupCount, settings, "const VkRayTracingShaderGroupCreateInfoNV*", "const VkRayTracingShaderGroupCreateInfoNV", "pGroups", true, false, indents + 1, dump_json_VkRayTracingShaderGroupCreateInfoNV); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxRecursionDepth, NULL, settings, "uint32_t", "maxRecursionDepth", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineLayout>(object.layout, NULL, settings, "VkPipelineLayout", "layout", false, false, indents + 1, dump_json_VkPipelineLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkPipeline>(object.basePipelineHandle, NULL, settings, "VkPipeline", "basePipelineHandle", false, false, indents + 1, dump_json_VkPipeline);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.basePipelineIndex, NULL, settings, "int32_t", "basePipelineIndex", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkGeometryTrianglesNV(const VkGeometryTrianglesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.vertexData, NULL, settings, "VkBuffer", "vertexData", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.vertexOffset, NULL, settings, "VkDeviceSize", "vertexOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.vertexCount, NULL, settings, "uint32_t", "vertexCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.vertexStride, NULL, settings, "VkDeviceSize", "vertexStride", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.vertexFormat, NULL, settings, "VkFormat", "vertexFormat", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.indexData, NULL, settings, "VkBuffer", "indexData", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.indexOffset, NULL, settings, "VkDeviceSize", "indexOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.indexCount, NULL, settings, "uint32_t", "indexCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkIndexType>(object.indexType, NULL, settings, "VkIndexType", "indexType", false, false, indents + 1, dump_json_VkIndexType);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.transformData, NULL, settings, "VkBuffer", "transformData", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.transformOffset, NULL, settings, "VkDeviceSize", "transformOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkGeometryAABBNV(const VkGeometryAABBNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.aabbData, NULL, settings, "VkBuffer", "aabbData", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.numAABBs, NULL, settings, "uint32_t", "numAABBs", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stride, NULL, settings, "uint32_t", "stride", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.offset, NULL, settings, "VkDeviceSize", "offset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkGeometryDataNV(const VkGeometryDataNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkGeometryTrianglesNV>(object.triangles, NULL, settings, "VkGeometryTrianglesNV", "triangles", true, false, indents + 1, dump_json_VkGeometryTrianglesNV);
    settings.stream() << ",\n";
    dump_json_value<const VkGeometryAABBNV>(object.aabbs, NULL, settings, "VkGeometryAABBNV", "aabbs", true, false, indents + 1, dump_json_VkGeometryAABBNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkGeometryNV(const VkGeometryNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkGeometryTypeKHR>(object.geometryType, NULL, settings, "VkGeometryTypeKHR", "geometryType", false, false, indents + 1, dump_json_VkGeometryTypeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkGeometryDataNV>(object.geometry, NULL, settings, "VkGeometryDataNV", "geometry", true, false, indents + 1, dump_json_VkGeometryDataNV);
    settings.stream() << ",\n";
    dump_json_value<const VkGeometryFlagsKHR>(object.flags, NULL, settings, "VkGeometryFlagsKHR", "flags", false, false, indents + 1, dump_json_VkGeometryFlagsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureInfoNV(const VkAccelerationStructureInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureTypeNV>(object.type, NULL, settings, "VkAccelerationStructureTypeNV", "type", false, false, indents + 1, dump_json_VkAccelerationStructureTypeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkBuildAccelerationStructureFlagsNV>(object.flags, NULL, settings, "VkBuildAccelerationStructureFlagsNV", "flags", false, false, indents + 1, dump_json_VkBuildAccelerationStructureFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.instanceCount, NULL, settings, "uint32_t", "instanceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.geometryCount, NULL, settings, "uint32_t", "geometryCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkGeometryNV>(object.pGeometries, object.geometryCount, settings, "const VkGeometryNV*", "const VkGeometryNV", "pGeometries", true, false, indents + 1, dump_json_VkGeometryNV); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureCreateInfoNV(const VkAccelerationStructureCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.compactedSize, NULL, settings, "VkDeviceSize", "compactedSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureInfoNV>(object.info, NULL, settings, "VkAccelerationStructureInfoNV", "info", true, false, indents + 1, dump_json_VkAccelerationStructureInfoNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindAccelerationStructureMemoryInfoNV(const VkBindAccelerationStructureMemoryInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureNV>(object.accelerationStructure, NULL, settings, "VkAccelerationStructureNV", "accelerationStructure", false, false, indents + 1, dump_json_VkAccelerationStructureNV);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.memoryOffset, NULL, settings, "VkDeviceSize", "memoryOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceIndexCount, NULL, settings, "uint32_t", "deviceIndexCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pDeviceIndices, object.deviceIndexCount, settings, "const uint32_t*", "const uint32_t", "pDeviceIndices", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkWriteDescriptorSetAccelerationStructureNV(const VkWriteDescriptorSetAccelerationStructureNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.accelerationStructureCount, NULL, settings, "uint32_t", "accelerationStructureCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkAccelerationStructureNV>(object.pAccelerationStructures, object.accelerationStructureCount, settings, "const VkAccelerationStructureNV*", "const VkAccelerationStructureNV", "pAccelerationStructures", false, false, indents + 1, dump_json_VkAccelerationStructureNV); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureMemoryRequirementsInfoNV(const VkAccelerationStructureMemoryRequirementsInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureMemoryRequirementsTypeNV>(object.type, NULL, settings, "VkAccelerationStructureMemoryRequirementsTypeNV", "type", false, false, indents + 1, dump_json_VkAccelerationStructureMemoryRequirementsTypeNV);
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureNV>(object.accelerationStructure, NULL, settings, "VkAccelerationStructureNV", "accelerationStructure", false, false, indents + 1, dump_json_VkAccelerationStructureNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRayTracingPropertiesNV(const VkPhysicalDeviceRayTracingPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shaderGroupHandleSize, NULL, settings, "uint32_t", "shaderGroupHandleSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxRecursionDepth, NULL, settings, "uint32_t", "maxRecursionDepth", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxShaderGroupStride, NULL, settings, "uint32_t", "maxShaderGroupStride", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shaderGroupBaseAlignment, NULL, settings, "uint32_t", "shaderGroupBaseAlignment", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.maxGeometryCount, NULL, settings, "uint64_t", "maxGeometryCount", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.maxInstanceCount, NULL, settings, "uint64_t", "maxInstanceCount", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.maxTriangleCount, NULL, settings, "uint64_t", "maxTriangleCount", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetAccelerationStructures, NULL, settings, "uint32_t", "maxDescriptorSetAccelerationStructures", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRepresentativeFragmentTestFeaturesNV(const VkPhysicalDeviceRepresentativeFragmentTestFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.representativeFragmentTest, NULL, settings, "VkBool32", "representativeFragmentTest", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineRepresentativeFragmentTestStateCreateInfoNV(const VkPipelineRepresentativeFragmentTestStateCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.representativeFragmentTestEnable, NULL, settings, "VkBool32", "representativeFragmentTestEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImageViewImageFormatInfoEXT(const VkPhysicalDeviceImageViewImageFormatInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageViewType>(object.imageViewType, NULL, settings, "VkImageViewType", "imageViewType", false, false, indents + 1, dump_json_VkImageViewType);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkFilterCubicImageViewImageFormatPropertiesEXT(const VkFilterCubicImageViewImageFormatPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.filterCubic, NULL, settings, "VkBool32", "filterCubic", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.filterCubicMinmax, NULL, settings, "VkBool32", "filterCubicMinmax", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceQueueGlobalPriorityCreateInfoKHR(const VkDeviceQueueGlobalPriorityCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkQueueGlobalPriorityKHR>(object.globalPriority, NULL, settings, "VkQueueGlobalPriorityKHR", "globalPriority", false, false, indents + 1, dump_json_VkQueueGlobalPriorityKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImportMemoryHostPointerInfoEXT(const VkImportMemoryHostPointerInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pHostPointer, NULL, settings, "void*", "pHostPointer", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryHostPointerPropertiesEXT(const VkMemoryHostPointerPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryTypeBits, NULL, settings, "uint32_t", "memoryTypeBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceExternalMemoryHostPropertiesEXT(const VkPhysicalDeviceExternalMemoryHostPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.minImportedHostPointerAlignment, NULL, settings, "VkDeviceSize", "minImportedHostPointerAlignment", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderClockFeaturesKHR(const VkPhysicalDeviceShaderClockFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSubgroupClock, NULL, settings, "VkBool32", "shaderSubgroupClock", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderDeviceClock, NULL, settings, "VkBool32", "shaderDeviceClock", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineCompilerControlCreateInfoAMD(const VkPipelineCompilerControlCreateInfoAMD& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineCompilerControlFlagsAMD>(object.compilerControlFlags, NULL, settings, "VkPipelineCompilerControlFlagsAMD", "compilerControlFlags", false, false, indents + 1, dump_json_VkPipelineCompilerControlFlagsAMD);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCalibratedTimestampInfoKHR(const VkCalibratedTimestampInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkTimeDomainKHR>(object.timeDomain, NULL, settings, "VkTimeDomainKHR", "timeDomain", false, false, indents + 1, dump_json_VkTimeDomainKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderCorePropertiesAMD(const VkPhysicalDeviceShaderCorePropertiesAMD& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shaderEngineCount, NULL, settings, "uint32_t", "shaderEngineCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shaderArraysPerEngineCount, NULL, settings, "uint32_t", "shaderArraysPerEngineCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.computeUnitsPerShaderArray, NULL, settings, "uint32_t", "computeUnitsPerShaderArray", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.simdPerComputeUnit, NULL, settings, "uint32_t", "simdPerComputeUnit", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.wavefrontsPerSimd, NULL, settings, "uint32_t", "wavefrontsPerSimd", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.wavefrontSize, NULL, settings, "uint32_t", "wavefrontSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.sgprsPerSimd, NULL, settings, "uint32_t", "sgprsPerSimd", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minSgprAllocation, NULL, settings, "uint32_t", "minSgprAllocation", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxSgprAllocation, NULL, settings, "uint32_t", "maxSgprAllocation", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.sgprAllocationGranularity, NULL, settings, "uint32_t", "sgprAllocationGranularity", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.vgprsPerSimd, NULL, settings, "uint32_t", "vgprsPerSimd", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minVgprAllocation, NULL, settings, "uint32_t", "minVgprAllocation", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxVgprAllocation, NULL, settings, "uint32_t", "maxVgprAllocation", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.vgprAllocationGranularity, NULL, settings, "uint32_t", "vgprAllocationGranularity", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeH265ProfileInfoKHR(const VkVideoDecodeH265ProfileInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const StdVideoH265ProfileIdc>(object.stdProfileIdc, NULL, settings, "StdVideoH265ProfileIdc", "stdProfileIdc", false, false, indents + 1, dump_json_StdVideoH265ProfileIdc);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeH265CapabilitiesKHR(const VkVideoDecodeH265CapabilitiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const StdVideoH265LevelIdc>(object.maxLevelIdc, NULL, settings, "StdVideoH265LevelIdc", "maxLevelIdc", false, false, indents + 1, dump_json_StdVideoH265LevelIdc);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeH265SessionParametersAddInfoKHR(const VkVideoDecodeH265SessionParametersAddInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stdVPSCount, NULL, settings, "uint32_t", "stdVPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const StdVideoH265VideoParameterSet>(object.pStdVPSs, object.stdVPSCount, settings, "const StdVideoH265VideoParameterSet*", "const StdVideoH265VideoParameterSet", "pStdVPSs", false, false, indents + 1, dump_json_StdVideoH265VideoParameterSet); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stdSPSCount, NULL, settings, "uint32_t", "stdSPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const StdVideoH265SequenceParameterSet>(object.pStdSPSs, object.stdSPSCount, settings, "const StdVideoH265SequenceParameterSet*", "const StdVideoH265SequenceParameterSet", "pStdSPSs", false, false, indents + 1, dump_json_StdVideoH265SequenceParameterSet); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stdPPSCount, NULL, settings, "uint32_t", "stdPPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const StdVideoH265PictureParameterSet>(object.pStdPPSs, object.stdPPSCount, settings, "const StdVideoH265PictureParameterSet*", "const StdVideoH265PictureParameterSet", "pStdPPSs", false, false, indents + 1, dump_json_StdVideoH265PictureParameterSet); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeH265SessionParametersCreateInfoKHR(const VkVideoDecodeH265SessionParametersCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxStdVPSCount, NULL, settings, "uint32_t", "maxStdVPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxStdSPSCount, NULL, settings, "uint32_t", "maxStdSPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxStdPPSCount, NULL, settings, "uint32_t", "maxStdPPSCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_pointer<const VkVideoDecodeH265SessionParametersAddInfoKHR>(object.pParametersAddInfo, settings, "const VkVideoDecodeH265SessionParametersAddInfoKHR*", "pParametersAddInfo", true, false, indents + 1, dump_json_VkVideoDecodeH265SessionParametersAddInfoKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeH265PictureInfoKHR(const VkVideoDecodeH265PictureInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const StdVideoDecodeH265PictureInfo>(object.pStdPictureInfo, settings, "const StdVideoDecodeH265PictureInfo*", "pStdPictureInfo", false, false, indents + 1, dump_json_StdVideoDecodeH265PictureInfo);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.sliceSegmentCount, NULL, settings, "uint32_t", "sliceSegmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pSliceSegmentOffsets, object.sliceSegmentCount, settings, "const uint32_t*", "const uint32_t", "pSliceSegmentOffsets", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeH265DpbSlotInfoKHR(const VkVideoDecodeH265DpbSlotInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const StdVideoDecodeH265ReferenceInfo>(object.pStdReferenceInfo, settings, "const StdVideoDecodeH265ReferenceInfo*", "pStdReferenceInfo", false, false, indents + 1, dump_json_StdVideoDecodeH265ReferenceInfo);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceGlobalPriorityQueryFeaturesKHR(const VkPhysicalDeviceGlobalPriorityQueryFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.globalPriorityQuery, NULL, settings, "VkBool32", "globalPriorityQuery", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkQueueFamilyGlobalPriorityPropertiesKHR(const VkQueueFamilyGlobalPriorityPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.priorityCount, NULL, settings, "uint32_t", "priorityCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkQueueGlobalPriorityKHR>(object.priorities, object.priorityCount, settings, "VkQueueGlobalPriorityKHR[VK_MAX_GLOBAL_PRIORITY_SIZE_KHR]", "VkQueueGlobalPriorityKHR", "priorities", false, false, indents + 1, dump_json_VkQueueGlobalPriorityKHR); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceMemoryOverallocationCreateInfoAMD(const VkDeviceMemoryOverallocationCreateInfoAMD& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkMemoryOverallocationBehaviorAMD>(object.overallocationBehavior, NULL, settings, "VkMemoryOverallocationBehaviorAMD", "overallocationBehavior", false, false, indents + 1, dump_json_VkMemoryOverallocationBehaviorAMD);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceVertexAttributeDivisorPropertiesEXT(const VkPhysicalDeviceVertexAttributeDivisorPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxVertexAttribDivisor, NULL, settings, "uint32_t", "maxVertexAttribDivisor", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVertexInputBindingDivisorDescriptionKHR(const VkVertexInputBindingDivisorDescriptionKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.binding, NULL, settings, "uint32_t", "binding", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.divisor, NULL, settings, "uint32_t", "divisor", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineVertexInputDivisorStateCreateInfoKHR(const VkPipelineVertexInputDivisorStateCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.vertexBindingDivisorCount, NULL, settings, "uint32_t", "vertexBindingDivisorCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkVertexInputBindingDivisorDescriptionKHR>(object.pVertexBindingDivisors, object.vertexBindingDivisorCount, settings, "const VkVertexInputBindingDivisorDescriptionKHR*", "const VkVertexInputBindingDivisorDescriptionKHR", "pVertexBindingDivisors", true, false, indents + 1, dump_json_VkVertexInputBindingDivisorDescriptionKHR); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceVertexAttributeDivisorFeaturesKHR(const VkPhysicalDeviceVertexAttributeDivisorFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.vertexAttributeInstanceRateDivisor, NULL, settings, "VkBool32", "vertexAttributeInstanceRateDivisor", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.vertexAttributeInstanceRateZeroDivisor, NULL, settings, "VkBool32", "vertexAttributeInstanceRateZeroDivisor", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_GGP)
void dump_json_VkPresentFrameTokenGGP(const VkPresentFrameTokenGGP& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const GgpFrameToken>(object.frameToken, NULL, settings, "GgpFrameToken", "frameToken", false, false, indents + 1, dump_json_GgpFrameToken);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_GGP
void dump_json_VkPhysicalDeviceComputeShaderDerivativesFeaturesNV(const VkPhysicalDeviceComputeShaderDerivativesFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.computeDerivativeGroupQuads, NULL, settings, "VkBool32", "computeDerivativeGroupQuads", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.computeDerivativeGroupLinear, NULL, settings, "VkBool32", "computeDerivativeGroupLinear", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMeshShaderFeaturesNV(const VkPhysicalDeviceMeshShaderFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.taskShader, NULL, settings, "VkBool32", "taskShader", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.meshShader, NULL, settings, "VkBool32", "meshShader", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMeshShaderPropertiesNV(const VkPhysicalDeviceMeshShaderPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDrawMeshTasksCount, NULL, settings, "uint32_t", "maxDrawMeshTasksCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTaskWorkGroupInvocations, NULL, settings, "uint32_t", "maxTaskWorkGroupInvocations", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.maxTaskWorkGroupSize, 3, settings, "uint32_t[3]", "uint32_t", "maxTaskWorkGroupSize", false, false, indents + 1, dump_json_uint32_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTaskTotalMemorySize, NULL, settings, "uint32_t", "maxTaskTotalMemorySize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTaskOutputCount, NULL, settings, "uint32_t", "maxTaskOutputCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshWorkGroupInvocations, NULL, settings, "uint32_t", "maxMeshWorkGroupInvocations", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.maxMeshWorkGroupSize, 3, settings, "uint32_t[3]", "uint32_t", "maxMeshWorkGroupSize", false, false, indents + 1, dump_json_uint32_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshTotalMemorySize, NULL, settings, "uint32_t", "maxMeshTotalMemorySize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshOutputVertices, NULL, settings, "uint32_t", "maxMeshOutputVertices", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshOutputPrimitives, NULL, settings, "uint32_t", "maxMeshOutputPrimitives", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshMultiviewViewCount, NULL, settings, "uint32_t", "maxMeshMultiviewViewCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.meshOutputPerVertexGranularity, NULL, settings, "uint32_t", "meshOutputPerVertexGranularity", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.meshOutputPerPrimitiveGranularity, NULL, settings, "uint32_t", "meshOutputPerPrimitiveGranularity", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDrawMeshTasksIndirectCommandNV(const VkDrawMeshTasksIndirectCommandNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.taskCount, NULL, settings, "uint32_t", "taskCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.firstTask, NULL, settings, "uint32_t", "firstTask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFragmentShaderBarycentricFeaturesKHR(const VkPhysicalDeviceFragmentShaderBarycentricFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentShaderBarycentric, NULL, settings, "VkBool32", "fragmentShaderBarycentric", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderImageFootprintFeaturesNV(const VkPhysicalDeviceShaderImageFootprintFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.imageFootprint, NULL, settings, "VkBool32", "imageFootprint", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineViewportExclusiveScissorStateCreateInfoNV(const VkPipelineViewportExclusiveScissorStateCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.exclusiveScissorCount, NULL, settings, "uint32_t", "exclusiveScissorCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkRect2D>(object.pExclusiveScissors, object.exclusiveScissorCount, settings, "const VkRect2D*", "const VkRect2D", "pExclusiveScissors", true, false, indents + 1, dump_json_VkRect2D); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceExclusiveScissorFeaturesNV(const VkPhysicalDeviceExclusiveScissorFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.exclusiveScissor, NULL, settings, "VkBool32", "exclusiveScissor", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkQueueFamilyCheckpointPropertiesNV(const VkQueueFamilyCheckpointPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineStageFlags>(object.checkpointExecutionStageMask, NULL, settings, "VkPipelineStageFlags", "checkpointExecutionStageMask", false, false, indents + 1, dump_json_VkPipelineStageFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCheckpointDataNV(const VkCheckpointDataNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineStageFlagBits>(object.stage, NULL, settings, "VkPipelineStageFlagBits", "stage", false, false, indents + 1, dump_json_VkPipelineStageFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pCheckpointMarker, NULL, settings, "void*", "pCheckpointMarker", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderIntegerFunctions2FeaturesINTEL(const VkPhysicalDeviceShaderIntegerFunctions2FeaturesINTEL& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderIntegerFunctions2, NULL, settings, "VkBool32", "shaderIntegerFunctions2", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPerformanceValueINTEL(const VkPerformanceValueINTEL& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkPerformanceValueTypeINTEL>(object.type, NULL, settings, "VkPerformanceValueTypeINTEL", "type", false, false, indents + 1, dump_json_VkPerformanceValueTypeINTEL);
    settings.stream() << ",\n";
    dump_json_value<const VkPerformanceValueDataINTEL>(object.data, NULL, settings, "VkPerformanceValueDataINTEL", "data", false, true, indents + 1, dump_json_VkPerformanceValueDataINTEL);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkInitializePerformanceApiInfoINTEL(const VkInitializePerformanceApiInfoINTEL& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pUserData, NULL, settings, "void*", "pUserData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkQueryPoolPerformanceQueryCreateInfoINTEL(const VkQueryPoolPerformanceQueryCreateInfoINTEL& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkQueryPoolSamplingModeINTEL>(object.performanceCountersSampling, NULL, settings, "VkQueryPoolSamplingModeINTEL", "performanceCountersSampling", false, false, indents + 1, dump_json_VkQueryPoolSamplingModeINTEL);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPerformanceMarkerInfoINTEL(const VkPerformanceMarkerInfoINTEL& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.marker, NULL, settings, "uint64_t", "marker", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPerformanceStreamMarkerInfoINTEL(const VkPerformanceStreamMarkerInfoINTEL& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.marker, NULL, settings, "uint32_t", "marker", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPerformanceOverrideInfoINTEL(const VkPerformanceOverrideInfoINTEL& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPerformanceOverrideTypeINTEL>(object.type, NULL, settings, "VkPerformanceOverrideTypeINTEL", "type", false, false, indents + 1, dump_json_VkPerformanceOverrideTypeINTEL);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.enable, NULL, settings, "VkBool32", "enable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.parameter, NULL, settings, "uint64_t", "parameter", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPerformanceConfigurationAcquireInfoINTEL(const VkPerformanceConfigurationAcquireInfoINTEL& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPerformanceConfigurationTypeINTEL>(object.type, NULL, settings, "VkPerformanceConfigurationTypeINTEL", "type", false, false, indents + 1, dump_json_VkPerformanceConfigurationTypeINTEL);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePCIBusInfoPropertiesEXT(const VkPhysicalDevicePCIBusInfoPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.pciDomain, NULL, settings, "uint32_t", "pciDomain", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.pciBus, NULL, settings, "uint32_t", "pciBus", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.pciDevice, NULL, settings, "uint32_t", "pciDevice", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.pciFunction, NULL, settings, "uint32_t", "pciFunction", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDisplayNativeHdrSurfaceCapabilitiesAMD(const VkDisplayNativeHdrSurfaceCapabilitiesAMD& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.localDimmingSupport, NULL, settings, "VkBool32", "localDimmingSupport", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSwapchainDisplayNativeHdrCreateInfoAMD(const VkSwapchainDisplayNativeHdrCreateInfoAMD& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.localDimmingEnable, NULL, settings, "VkBool32", "localDimmingEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkImagePipeSurfaceCreateInfoFUCHSIA(const VkImagePipeSurfaceCreateInfoFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImagePipeSurfaceCreateFlagsFUCHSIA>(object.flags, NULL, settings, "VkImagePipeSurfaceCreateFlagsFUCHSIA", "flags", false, false, indents + 1, dump_json_VkImagePipeSurfaceCreateFlagsFUCHSIA);
    settings.stream() << ",\n";
    dump_json_value<const zx_handle_t>(object.imagePipeHandle, NULL, settings, "zx_handle_t", "imagePipeHandle", false, false, indents + 1, dump_json_zx_handle_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkMetalSurfaceCreateInfoEXT(const VkMetalSurfaceCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkMetalSurfaceCreateFlagsEXT>(object.flags, NULL, settings, "VkMetalSurfaceCreateFlagsEXT", "flags", false, false, indents + 1, dump_json_VkMetalSurfaceCreateFlagsEXT);
    settings.stream() << ",\n";
    dump_json_pointer<const CAMetalLayer>(object.pLayer, settings, "const CAMetalLayer*", "pLayer", false, false, indents + 1, dump_json_CAMetalLayer);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_METAL_EXT
void dump_json_VkPhysicalDeviceFragmentDensityMapFeaturesEXT(const VkPhysicalDeviceFragmentDensityMapFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentDensityMap, NULL, settings, "VkBool32", "fragmentDensityMap", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentDensityMapDynamic, NULL, settings, "VkBool32", "fragmentDensityMapDynamic", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentDensityMapNonSubsampledImages, NULL, settings, "VkBool32", "fragmentDensityMapNonSubsampledImages", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFragmentDensityMapPropertiesEXT(const VkPhysicalDeviceFragmentDensityMapPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.minFragmentDensityTexelSize, NULL, settings, "VkExtent2D", "minFragmentDensityTexelSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxFragmentDensityTexelSize, NULL, settings, "VkExtent2D", "maxFragmentDensityTexelSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentDensityInvocations, NULL, settings, "VkBool32", "fragmentDensityInvocations", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassFragmentDensityMapCreateInfoEXT(const VkRenderPassFragmentDensityMapCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAttachmentReference>(object.fragmentDensityMapAttachment, NULL, settings, "VkAttachmentReference", "fragmentDensityMapAttachment", true, false, indents + 1, dump_json_VkAttachmentReference);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkFragmentShadingRateAttachmentInfoKHR(const VkFragmentShadingRateAttachmentInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const VkAttachmentReference2>(object.pFragmentShadingRateAttachment, settings, "const VkAttachmentReference2*", "pFragmentShadingRateAttachment", true, false, indents + 1, dump_json_VkAttachmentReference2);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.shadingRateAttachmentTexelSize, NULL, settings, "VkExtent2D", "shadingRateAttachmentTexelSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineFragmentShadingRateStateCreateInfoKHR(const VkPipelineFragmentShadingRateStateCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.fragmentSize, NULL, settings, "VkExtent2D", "fragmentSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_array<const VkFragmentShadingRateCombinerOpKHR>(object.combinerOps, 2, settings, "VkFragmentShadingRateCombinerOpKHR[2]", "VkFragmentShadingRateCombinerOpKHR", "combinerOps", false, false, indents + 1, dump_json_VkFragmentShadingRateCombinerOpKHR); // IQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFragmentShadingRateFeaturesKHR(const VkPhysicalDeviceFragmentShadingRateFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.pipelineFragmentShadingRate, NULL, settings, "VkBool32", "pipelineFragmentShadingRate", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.primitiveFragmentShadingRate, NULL, settings, "VkBool32", "primitiveFragmentShadingRate", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.attachmentFragmentShadingRate, NULL, settings, "VkBool32", "attachmentFragmentShadingRate", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFragmentShadingRatePropertiesKHR(const VkPhysicalDeviceFragmentShadingRatePropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.minFragmentShadingRateAttachmentTexelSize, NULL, settings, "VkExtent2D", "minFragmentShadingRateAttachmentTexelSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxFragmentShadingRateAttachmentTexelSize, NULL, settings, "VkExtent2D", "maxFragmentShadingRateAttachmentTexelSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxFragmentShadingRateAttachmentTexelSizeAspectRatio, NULL, settings, "uint32_t", "maxFragmentShadingRateAttachmentTexelSizeAspectRatio", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.primitiveFragmentShadingRateWithMultipleViewports, NULL, settings, "VkBool32", "primitiveFragmentShadingRateWithMultipleViewports", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.layeredShadingRateAttachments, NULL, settings, "VkBool32", "layeredShadingRateAttachments", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentShadingRateNonTrivialCombinerOps, NULL, settings, "VkBool32", "fragmentShadingRateNonTrivialCombinerOps", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxFragmentSize, NULL, settings, "VkExtent2D", "maxFragmentSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxFragmentSizeAspectRatio, NULL, settings, "uint32_t", "maxFragmentSizeAspectRatio", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxFragmentShadingRateCoverageSamples, NULL, settings, "uint32_t", "maxFragmentShadingRateCoverageSamples", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlagBits>(object.maxFragmentShadingRateRasterizationSamples, NULL, settings, "VkSampleCountFlagBits", "maxFragmentShadingRateRasterizationSamples", false, false, indents + 1, dump_json_VkSampleCountFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentShadingRateWithShaderDepthStencilWrites, NULL, settings, "VkBool32", "fragmentShadingRateWithShaderDepthStencilWrites", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentShadingRateWithSampleMask, NULL, settings, "VkBool32", "fragmentShadingRateWithSampleMask", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentShadingRateWithShaderSampleMask, NULL, settings, "VkBool32", "fragmentShadingRateWithShaderSampleMask", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentShadingRateWithConservativeRasterization, NULL, settings, "VkBool32", "fragmentShadingRateWithConservativeRasterization", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentShadingRateWithFragmentShaderInterlock, NULL, settings, "VkBool32", "fragmentShadingRateWithFragmentShaderInterlock", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentShadingRateWithCustomSampleLocations, NULL, settings, "VkBool32", "fragmentShadingRateWithCustomSampleLocations", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentShadingRateStrictMultiplyCombiner, NULL, settings, "VkBool32", "fragmentShadingRateStrictMultiplyCombiner", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFragmentShadingRateKHR(const VkPhysicalDeviceFragmentShadingRateKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlags>(object.sampleCounts, NULL, settings, "VkSampleCountFlags", "sampleCounts", false, false, indents + 1, dump_json_VkSampleCountFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.fragmentSize, NULL, settings, "VkExtent2D", "fragmentSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderCoreProperties2AMD(const VkPhysicalDeviceShaderCoreProperties2AMD& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkShaderCorePropertiesFlagsAMD>(object.shaderCoreFeatures, NULL, settings, "VkShaderCorePropertiesFlagsAMD", "shaderCoreFeatures", false, false, indents + 1, dump_json_VkShaderCorePropertiesFlagsAMD);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.activeComputeUnitCount, NULL, settings, "uint32_t", "activeComputeUnitCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceCoherentMemoryFeaturesAMD(const VkPhysicalDeviceCoherentMemoryFeaturesAMD& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.deviceCoherentMemory, NULL, settings, "VkBool32", "deviceCoherentMemory", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDynamicRenderingLocalReadFeaturesKHR(const VkPhysicalDeviceDynamicRenderingLocalReadFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.dynamicRenderingLocalRead, NULL, settings, "VkBool32", "dynamicRenderingLocalRead", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderingAttachmentLocationInfoKHR(const VkRenderingAttachmentLocationInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.colorAttachmentCount, NULL, settings, "uint32_t", "colorAttachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pColorAttachmentLocations, object.colorAttachmentCount, settings, "const uint32_t*", "const uint32_t", "pColorAttachmentLocations", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderingInputAttachmentIndexInfoKHR(const VkRenderingInputAttachmentIndexInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.colorAttachmentCount, NULL, settings, "uint32_t", "colorAttachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pColorAttachmentInputIndices, object.colorAttachmentCount, settings, "const uint32_t*", "const uint32_t", "pColorAttachmentInputIndices", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << ",\n";
    dump_json_pointer<const uint32_t>(object.pDepthInputAttachmentIndex, settings, "const uint32_t*", "pDepthInputAttachmentIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_pointer<const uint32_t>(object.pStencilInputAttachmentIndex, settings, "const uint32_t*", "pStencilInputAttachmentIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderImageAtomicInt64FeaturesEXT(const VkPhysicalDeviceShaderImageAtomicInt64FeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderImageInt64Atomics, NULL, settings, "VkBool32", "shaderImageInt64Atomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sparseImageInt64Atomics, NULL, settings, "VkBool32", "sparseImageInt64Atomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderQuadControlFeaturesKHR(const VkPhysicalDeviceShaderQuadControlFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderQuadControl, NULL, settings, "VkBool32", "shaderQuadControl", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMemoryBudgetPropertiesEXT(const VkPhysicalDeviceMemoryBudgetPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_array<const VkDeviceSize>(object.heapBudget, ApiDumpInstance::current().getMemoryHeapCount(), settings, "VkDeviceSize[VK_MAX_MEMORY_HEAPS]", "VkDeviceSize", "heapBudget", false, false, indents + 1, dump_json_VkDeviceSize); // JQA
    settings.stream() << ",\n";
    dump_json_array<const VkDeviceSize>(object.heapUsage, ApiDumpInstance::current().getMemoryHeapCount(), settings, "VkDeviceSize[VK_MAX_MEMORY_HEAPS]", "VkDeviceSize", "heapUsage", false, false, indents + 1, dump_json_VkDeviceSize); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMemoryPriorityFeaturesEXT(const VkPhysicalDeviceMemoryPriorityFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.memoryPriority, NULL, settings, "VkBool32", "memoryPriority", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryPriorityAllocateInfoEXT(const VkMemoryPriorityAllocateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const float>(object.priority, NULL, settings, "float", "priority", false, false, indents + 1, dump_json_float);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSurfaceProtectedCapabilitiesKHR(const VkSurfaceProtectedCapabilitiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.supportsProtected, NULL, settings, "VkBool32", "supportsProtected", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDedicatedAllocationImageAliasingFeaturesNV(const VkPhysicalDeviceDedicatedAllocationImageAliasingFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.dedicatedAllocationImageAliasing, NULL, settings, "VkBool32", "dedicatedAllocationImageAliasing", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceBufferDeviceAddressFeaturesEXT(const VkPhysicalDeviceBufferDeviceAddressFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.bufferDeviceAddress, NULL, settings, "VkBool32", "bufferDeviceAddress", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.bufferDeviceAddressCaptureReplay, NULL, settings, "VkBool32", "bufferDeviceAddressCaptureReplay", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.bufferDeviceAddressMultiDevice, NULL, settings, "VkBool32", "bufferDeviceAddressMultiDevice", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBufferDeviceAddressCreateInfoEXT(const VkBufferDeviceAddressCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddress>(object.deviceAddress, NULL, settings, "VkDeviceAddress", "deviceAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkValidationFeaturesEXT(const VkValidationFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.enabledValidationFeatureCount, NULL, settings, "uint32_t", "enabledValidationFeatureCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkValidationFeatureEnableEXT>(object.pEnabledValidationFeatures, object.enabledValidationFeatureCount, settings, "const VkValidationFeatureEnableEXT*", "const VkValidationFeatureEnableEXT", "pEnabledValidationFeatures", false, false, indents + 1, dump_json_VkValidationFeatureEnableEXT); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.disabledValidationFeatureCount, NULL, settings, "uint32_t", "disabledValidationFeatureCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkValidationFeatureDisableEXT>(object.pDisabledValidationFeatures, object.disabledValidationFeatureCount, settings, "const VkValidationFeatureDisableEXT*", "const VkValidationFeatureDisableEXT", "pDisabledValidationFeatures", false, false, indents + 1, dump_json_VkValidationFeatureDisableEXT); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePresentWaitFeaturesKHR(const VkPhysicalDevicePresentWaitFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.presentWait, NULL, settings, "VkBool32", "presentWait", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCooperativeMatrixPropertiesNV(const VkCooperativeMatrixPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.MSize, NULL, settings, "uint32_t", "MSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.NSize, NULL, settings, "uint32_t", "NSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.KSize, NULL, settings, "uint32_t", "KSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentTypeNV>(object.AType, NULL, settings, "VkComponentTypeNV", "AType", false, false, indents + 1, dump_json_VkComponentTypeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentTypeNV>(object.BType, NULL, settings, "VkComponentTypeNV", "BType", false, false, indents + 1, dump_json_VkComponentTypeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentTypeNV>(object.CType, NULL, settings, "VkComponentTypeNV", "CType", false, false, indents + 1, dump_json_VkComponentTypeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentTypeNV>(object.DType, NULL, settings, "VkComponentTypeNV", "DType", false, false, indents + 1, dump_json_VkComponentTypeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkScopeNV>(object.scope, NULL, settings, "VkScopeNV", "scope", false, false, indents + 1, dump_json_VkScopeKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceCooperativeMatrixFeaturesNV(const VkPhysicalDeviceCooperativeMatrixFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.cooperativeMatrix, NULL, settings, "VkBool32", "cooperativeMatrix", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.cooperativeMatrixRobustBufferAccess, NULL, settings, "VkBool32", "cooperativeMatrixRobustBufferAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceCooperativeMatrixPropertiesNV(const VkPhysicalDeviceCooperativeMatrixPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlags>(object.cooperativeMatrixSupportedStages, NULL, settings, "VkShaderStageFlags", "cooperativeMatrixSupportedStages", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceCoverageReductionModeFeaturesNV(const VkPhysicalDeviceCoverageReductionModeFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.coverageReductionMode, NULL, settings, "VkBool32", "coverageReductionMode", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineCoverageReductionStateCreateInfoNV(const VkPipelineCoverageReductionStateCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineCoverageReductionStateCreateFlagsNV>(object.flags, NULL, settings, "VkPipelineCoverageReductionStateCreateFlagsNV", "flags", false, false, indents + 1, dump_json_VkPipelineCoverageReductionStateCreateFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const VkCoverageReductionModeNV>(object.coverageReductionMode, NULL, settings, "VkCoverageReductionModeNV", "coverageReductionMode", false, false, indents + 1, dump_json_VkCoverageReductionModeNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkFramebufferMixedSamplesCombinationNV(const VkFramebufferMixedSamplesCombinationNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkCoverageReductionModeNV>(object.coverageReductionMode, NULL, settings, "VkCoverageReductionModeNV", "coverageReductionMode", false, false, indents + 1, dump_json_VkCoverageReductionModeNV);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlagBits>(object.rasterizationSamples, NULL, settings, "VkSampleCountFlagBits", "rasterizationSamples", false, false, indents + 1, dump_json_VkSampleCountFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlags>(object.depthStencilSamples, NULL, settings, "VkSampleCountFlags", "depthStencilSamples", false, false, indents + 1, dump_json_VkSampleCountFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlags>(object.colorSamples, NULL, settings, "VkSampleCountFlags", "colorSamples", false, false, indents + 1, dump_json_VkSampleCountFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFragmentShaderInterlockFeaturesEXT(const VkPhysicalDeviceFragmentShaderInterlockFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentShaderSampleInterlock, NULL, settings, "VkBool32", "fragmentShaderSampleInterlock", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentShaderPixelInterlock, NULL, settings, "VkBool32", "fragmentShaderPixelInterlock", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentShaderShadingRateInterlock, NULL, settings, "VkBool32", "fragmentShaderShadingRateInterlock", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceYcbcrImageArraysFeaturesEXT(const VkPhysicalDeviceYcbcrImageArraysFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.ycbcrImageArrays, NULL, settings, "VkBool32", "ycbcrImageArrays", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceProvokingVertexFeaturesEXT(const VkPhysicalDeviceProvokingVertexFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.provokingVertexLast, NULL, settings, "VkBool32", "provokingVertexLast", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.transformFeedbackPreservesProvokingVertex, NULL, settings, "VkBool32", "transformFeedbackPreservesProvokingVertex", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceProvokingVertexPropertiesEXT(const VkPhysicalDeviceProvokingVertexPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.provokingVertexModePerPipeline, NULL, settings, "VkBool32", "provokingVertexModePerPipeline", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.transformFeedbackPreservesTriangleFanProvokingVertex, NULL, settings, "VkBool32", "transformFeedbackPreservesTriangleFanProvokingVertex", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineRasterizationProvokingVertexStateCreateInfoEXT(const VkPipelineRasterizationProvokingVertexStateCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkProvokingVertexModeEXT>(object.provokingVertexMode, NULL, settings, "VkProvokingVertexModeEXT", "provokingVertexMode", false, false, indents + 1, dump_json_VkProvokingVertexModeEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkSurfaceFullScreenExclusiveInfoEXT(const VkSurfaceFullScreenExclusiveInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFullScreenExclusiveEXT>(object.fullScreenExclusive, NULL, settings, "VkFullScreenExclusiveEXT", "fullScreenExclusive", false, false, indents + 1, dump_json_VkFullScreenExclusiveEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkSurfaceCapabilitiesFullScreenExclusiveEXT(const VkSurfaceCapabilitiesFullScreenExclusiveEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fullScreenExclusiveSupported, NULL, settings, "VkBool32", "fullScreenExclusiveSupported", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_VkSurfaceFullScreenExclusiveWin32InfoEXT(const VkSurfaceFullScreenExclusiveWin32InfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const HMONITOR>(object.hmonitor, NULL, settings, "HMONITOR", "hmonitor", false, false, indents + 1, dump_json_HMONITOR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_WIN32_KHR
void dump_json_VkHeadlessSurfaceCreateInfoEXT(const VkHeadlessSurfaceCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkHeadlessSurfaceCreateFlagsEXT>(object.flags, NULL, settings, "VkHeadlessSurfaceCreateFlagsEXT", "flags", false, false, indents + 1, dump_json_VkHeadlessSurfaceCreateFlagsEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceLineRasterizationFeaturesKHR(const VkPhysicalDeviceLineRasterizationFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rectangularLines, NULL, settings, "VkBool32", "rectangularLines", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.bresenhamLines, NULL, settings, "VkBool32", "bresenhamLines", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.smoothLines, NULL, settings, "VkBool32", "smoothLines", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.stippledRectangularLines, NULL, settings, "VkBool32", "stippledRectangularLines", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.stippledBresenhamLines, NULL, settings, "VkBool32", "stippledBresenhamLines", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.stippledSmoothLines, NULL, settings, "VkBool32", "stippledSmoothLines", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceLineRasterizationPropertiesKHR(const VkPhysicalDeviceLineRasterizationPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.lineSubPixelPrecisionBits, NULL, settings, "uint32_t", "lineSubPixelPrecisionBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineRasterizationLineStateCreateInfoKHR(const VkPipelineRasterizationLineStateCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkLineRasterizationModeKHR>(object.lineRasterizationMode, NULL, settings, "VkLineRasterizationModeKHR", "lineRasterizationMode", false, false, indents + 1, dump_json_VkLineRasterizationModeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.stippledLineEnable, NULL, settings, "VkBool32", "stippledLineEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.lineStippleFactor, NULL, settings, "uint32_t", "lineStippleFactor", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint16_t>(object.lineStipplePattern, NULL, settings, "uint16_t", "lineStipplePattern", false, false, indents + 1, dump_json_uint16_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderAtomicFloatFeaturesEXT(const VkPhysicalDeviceShaderAtomicFloatFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderBufferFloat32Atomics, NULL, settings, "VkBool32", "shaderBufferFloat32Atomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderBufferFloat32AtomicAdd, NULL, settings, "VkBool32", "shaderBufferFloat32AtomicAdd", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderBufferFloat64Atomics, NULL, settings, "VkBool32", "shaderBufferFloat64Atomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderBufferFloat64AtomicAdd, NULL, settings, "VkBool32", "shaderBufferFloat64AtomicAdd", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSharedFloat32Atomics, NULL, settings, "VkBool32", "shaderSharedFloat32Atomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSharedFloat32AtomicAdd, NULL, settings, "VkBool32", "shaderSharedFloat32AtomicAdd", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSharedFloat64Atomics, NULL, settings, "VkBool32", "shaderSharedFloat64Atomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSharedFloat64AtomicAdd, NULL, settings, "VkBool32", "shaderSharedFloat64AtomicAdd", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderImageFloat32Atomics, NULL, settings, "VkBool32", "shaderImageFloat32Atomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderImageFloat32AtomicAdd, NULL, settings, "VkBool32", "shaderImageFloat32AtomicAdd", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sparseImageFloat32Atomics, NULL, settings, "VkBool32", "sparseImageFloat32Atomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sparseImageFloat32AtomicAdd, NULL, settings, "VkBool32", "sparseImageFloat32AtomicAdd", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceIndexTypeUint8FeaturesKHR(const VkPhysicalDeviceIndexTypeUint8FeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.indexTypeUint8, NULL, settings, "VkBool32", "indexTypeUint8", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceExtendedDynamicStateFeaturesEXT(const VkPhysicalDeviceExtendedDynamicStateFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState, NULL, settings, "VkBool32", "extendedDynamicState", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePipelineExecutablePropertiesFeaturesKHR(const VkPhysicalDevicePipelineExecutablePropertiesFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.pipelineExecutableInfo, NULL, settings, "VkBool32", "pipelineExecutableInfo", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineInfoKHR(const VkPipelineInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipeline>(object.pipeline, NULL, settings, "VkPipeline", "pipeline", false, false, indents + 1, dump_json_VkPipeline);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineExecutablePropertiesKHR(const VkPipelineExecutablePropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlags>(object.stages, NULL, settings, "VkShaderStageFlags", "stages", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.name, NULL, settings, "char[VK_MAX_DESCRIPTION_SIZE]", "name", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.description, NULL, settings, "char[VK_MAX_DESCRIPTION_SIZE]", "description", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.subgroupSize, NULL, settings, "uint32_t", "subgroupSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineExecutableInfoKHR(const VkPipelineExecutableInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipeline>(object.pipeline, NULL, settings, "VkPipeline", "pipeline", false, false, indents + 1, dump_json_VkPipeline);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.executableIndex, NULL, settings, "uint32_t", "executableIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineExecutableStatisticKHR(const VkPipelineExecutableStatisticKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.name, NULL, settings, "char[VK_MAX_DESCRIPTION_SIZE]", "name", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.description, NULL, settings, "char[VK_MAX_DESCRIPTION_SIZE]", "description", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineExecutableStatisticFormatKHR>(object.format, NULL, settings, "VkPipelineExecutableStatisticFormatKHR", "format", false, false, indents + 1, dump_json_VkPipelineExecutableStatisticFormatKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineExecutableStatisticValueKHR>(object.value, NULL, settings, "VkPipelineExecutableStatisticValueKHR", "value", false, true, indents + 1, dump_json_VkPipelineExecutableStatisticValueKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineExecutableInternalRepresentationKHR(const VkPipelineExecutableInternalRepresentationKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.name, NULL, settings, "char[VK_MAX_DESCRIPTION_SIZE]", "name", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.description, NULL, settings, "char[VK_MAX_DESCRIPTION_SIZE]", "description", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.isText, NULL, settings, "VkBool32", "isText", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.dataSize, NULL, settings, "size_t", "dataSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pData, NULL, settings, "void*", "pData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceHostImageCopyFeaturesEXT(const VkPhysicalDeviceHostImageCopyFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.hostImageCopy, NULL, settings, "VkBool32", "hostImageCopy", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceHostImageCopyPropertiesEXT(const VkPhysicalDeviceHostImageCopyPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.copySrcLayoutCount, NULL, settings, "uint32_t", "copySrcLayoutCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkImageLayout>(object.pCopySrcLayouts, object.copySrcLayoutCount, settings, "VkImageLayout*", "VkImageLayout", "pCopySrcLayouts", false, false, indents + 1, dump_json_VkImageLayout); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.copyDstLayoutCount, NULL, settings, "uint32_t", "copyDstLayoutCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkImageLayout>(object.pCopyDstLayouts, object.copyDstLayoutCount, settings, "VkImageLayout*", "VkImageLayout", "pCopyDstLayouts", false, false, indents + 1, dump_json_VkImageLayout); // JQA
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.optimalTilingLayoutUUID, 16, settings, "uint8_t[VK_UUID_SIZE]", "uint8_t", "optimalTilingLayoutUUID", false, false, indents + 1, dump_json_uint8_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.identicalMemoryTypeRequirements, NULL, settings, "VkBool32", "identicalMemoryTypeRequirements", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryToImageCopyEXT(const VkMemoryToImageCopyEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pHostPointer, NULL, settings, "const void*", "pHostPointer", false, false, indents + 1, dump_json_void);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryRowLength, NULL, settings, "uint32_t", "memoryRowLength", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryImageHeight, NULL, settings, "uint32_t", "memoryImageHeight", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceLayers>(object.imageSubresource, NULL, settings, "VkImageSubresourceLayers", "imageSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset3D>(object.imageOffset, NULL, settings, "VkOffset3D", "imageOffset", true, false, indents + 1, dump_json_VkOffset3D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent3D>(object.imageExtent, NULL, settings, "VkExtent3D", "imageExtent", true, false, indents + 1, dump_json_VkExtent3D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageToMemoryCopyEXT(const VkImageToMemoryCopyEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pHostPointer, NULL, settings, "void*", "pHostPointer", false, false, indents + 1, dump_json_void);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryRowLength, NULL, settings, "uint32_t", "memoryRowLength", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryImageHeight, NULL, settings, "uint32_t", "memoryImageHeight", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceLayers>(object.imageSubresource, NULL, settings, "VkImageSubresourceLayers", "imageSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset3D>(object.imageOffset, NULL, settings, "VkOffset3D", "imageOffset", true, false, indents + 1, dump_json_VkOffset3D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent3D>(object.imageExtent, NULL, settings, "VkExtent3D", "imageExtent", true, false, indents + 1, dump_json_VkExtent3D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyMemoryToImageInfoEXT(const VkCopyMemoryToImageInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkHostImageCopyFlagsEXT>(object.flags, NULL, settings, "VkHostImageCopyFlagsEXT", "flags", false, false, indents + 1, dump_json_VkHostImageCopyFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.dstImage, NULL, settings, "VkImage", "dstImage", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.dstImageLayout, NULL, settings, "VkImageLayout", "dstImageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.regionCount, NULL, settings, "uint32_t", "regionCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkMemoryToImageCopyEXT>(object.pRegions, object.regionCount, settings, "const VkMemoryToImageCopyEXT*", "const VkMemoryToImageCopyEXT", "pRegions", true, false, indents + 1, dump_json_VkMemoryToImageCopyEXT); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyImageToMemoryInfoEXT(const VkCopyImageToMemoryInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkHostImageCopyFlagsEXT>(object.flags, NULL, settings, "VkHostImageCopyFlagsEXT", "flags", false, false, indents + 1, dump_json_VkHostImageCopyFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.srcImage, NULL, settings, "VkImage", "srcImage", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.srcImageLayout, NULL, settings, "VkImageLayout", "srcImageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.regionCount, NULL, settings, "uint32_t", "regionCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkImageToMemoryCopyEXT>(object.pRegions, object.regionCount, settings, "const VkImageToMemoryCopyEXT*", "const VkImageToMemoryCopyEXT", "pRegions", true, false, indents + 1, dump_json_VkImageToMemoryCopyEXT); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyImageToImageInfoEXT(const VkCopyImageToImageInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkHostImageCopyFlagsEXT>(object.flags, NULL, settings, "VkHostImageCopyFlagsEXT", "flags", false, false, indents + 1, dump_json_VkHostImageCopyFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.srcImage, NULL, settings, "VkImage", "srcImage", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.srcImageLayout, NULL, settings, "VkImageLayout", "srcImageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.dstImage, NULL, settings, "VkImage", "dstImage", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.dstImageLayout, NULL, settings, "VkImageLayout", "dstImageLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.regionCount, NULL, settings, "uint32_t", "regionCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkImageCopy2>(object.pRegions, object.regionCount, settings, "const VkImageCopy2*", "const VkImageCopy2", "pRegions", true, false, indents + 1, dump_json_VkImageCopy2); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkHostImageLayoutTransitionInfoEXT(const VkHostImageLayoutTransitionInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.image, NULL, settings, "VkImage", "image", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.oldLayout, NULL, settings, "VkImageLayout", "oldLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkImageLayout>(object.newLayout, NULL, settings, "VkImageLayout", "newLayout", false, false, indents + 1, dump_json_VkImageLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceRange>(object.subresourceRange, NULL, settings, "VkImageSubresourceRange", "subresourceRange", true, false, indents + 1, dump_json_VkImageSubresourceRange);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSubresourceHostMemcpySizeEXT(const VkSubresourceHostMemcpySizeEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkHostImageCopyDevicePerformanceQueryEXT(const VkHostImageCopyDevicePerformanceQueryEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.optimalDeviceAccess, NULL, settings, "VkBool32", "optimalDeviceAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.identicalMemoryLayout, NULL, settings, "VkBool32", "identicalMemoryLayout", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSubresourceLayout2KHR(const VkSubresourceLayout2KHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSubresourceLayout>(object.subresourceLayout, NULL, settings, "VkSubresourceLayout", "subresourceLayout", true, false, indents + 1, dump_json_VkSubresourceLayout);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageSubresource2KHR(const VkImageSubresource2KHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresource>(object.imageSubresource, NULL, settings, "VkImageSubresource", "imageSubresource", true, false, indents + 1, dump_json_VkImageSubresource);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryMapInfoKHR(const VkMemoryMapInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkMemoryMapFlags>(object.flags, NULL, settings, "VkMemoryMapFlags", "flags", false, false, indents + 1, dump_json_VkMemoryMapFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.offset, NULL, settings, "VkDeviceSize", "offset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryUnmapInfoKHR(const VkMemoryUnmapInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkMemoryUnmapFlagsKHR>(object.flags, NULL, settings, "VkMemoryUnmapFlagsKHR", "flags", false, false, indents + 1, dump_json_VkMemoryUnmapFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMapMemoryPlacedFeaturesEXT(const VkPhysicalDeviceMapMemoryPlacedFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.memoryMapPlaced, NULL, settings, "VkBool32", "memoryMapPlaced", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.memoryMapRangePlaced, NULL, settings, "VkBool32", "memoryMapRangePlaced", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.memoryUnmapReserve, NULL, settings, "VkBool32", "memoryUnmapReserve", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMapMemoryPlacedPropertiesEXT(const VkPhysicalDeviceMapMemoryPlacedPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.minPlacedMemoryMapAlignment, NULL, settings, "VkDeviceSize", "minPlacedMemoryMapAlignment", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryMapPlacedInfoEXT(const VkMemoryMapPlacedInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pPlacedAddress, NULL, settings, "void*", "pPlacedAddress", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderAtomicFloat2FeaturesEXT(const VkPhysicalDeviceShaderAtomicFloat2FeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderBufferFloat16Atomics, NULL, settings, "VkBool32", "shaderBufferFloat16Atomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderBufferFloat16AtomicAdd, NULL, settings, "VkBool32", "shaderBufferFloat16AtomicAdd", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderBufferFloat16AtomicMinMax, NULL, settings, "VkBool32", "shaderBufferFloat16AtomicMinMax", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderBufferFloat32AtomicMinMax, NULL, settings, "VkBool32", "shaderBufferFloat32AtomicMinMax", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderBufferFloat64AtomicMinMax, NULL, settings, "VkBool32", "shaderBufferFloat64AtomicMinMax", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSharedFloat16Atomics, NULL, settings, "VkBool32", "shaderSharedFloat16Atomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSharedFloat16AtomicAdd, NULL, settings, "VkBool32", "shaderSharedFloat16AtomicAdd", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSharedFloat16AtomicMinMax, NULL, settings, "VkBool32", "shaderSharedFloat16AtomicMinMax", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSharedFloat32AtomicMinMax, NULL, settings, "VkBool32", "shaderSharedFloat32AtomicMinMax", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSharedFloat64AtomicMinMax, NULL, settings, "VkBool32", "shaderSharedFloat64AtomicMinMax", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderImageFloat32AtomicMinMax, NULL, settings, "VkBool32", "shaderImageFloat32AtomicMinMax", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sparseImageFloat32AtomicMinMax, NULL, settings, "VkBool32", "sparseImageFloat32AtomicMinMax", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSurfacePresentModeEXT(const VkSurfacePresentModeEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPresentModeKHR>(object.presentMode, NULL, settings, "VkPresentModeKHR", "presentMode", false, false, indents + 1, dump_json_VkPresentModeKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSurfacePresentScalingCapabilitiesEXT(const VkSurfacePresentScalingCapabilitiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPresentScalingFlagsEXT>(object.supportedPresentScaling, NULL, settings, "VkPresentScalingFlagsEXT", "supportedPresentScaling", false, false, indents + 1, dump_json_VkPresentScalingFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkPresentGravityFlagsEXT>(object.supportedPresentGravityX, NULL, settings, "VkPresentGravityFlagsEXT", "supportedPresentGravityX", false, false, indents + 1, dump_json_VkPresentGravityFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkPresentGravityFlagsEXT>(object.supportedPresentGravityY, NULL, settings, "VkPresentGravityFlagsEXT", "supportedPresentGravityY", false, false, indents + 1, dump_json_VkPresentGravityFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.minScaledImageExtent, NULL, settings, "VkExtent2D", "minScaledImageExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxScaledImageExtent, NULL, settings, "VkExtent2D", "maxScaledImageExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSurfacePresentModeCompatibilityEXT(const VkSurfacePresentModeCompatibilityEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.presentModeCount, NULL, settings, "uint32_t", "presentModeCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPresentModeKHR>(object.pPresentModes, object.presentModeCount, settings, "VkPresentModeKHR*", "VkPresentModeKHR", "pPresentModes", false, false, indents + 1, dump_json_VkPresentModeKHR); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSwapchainMaintenance1FeaturesEXT(const VkPhysicalDeviceSwapchainMaintenance1FeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.swapchainMaintenance1, NULL, settings, "VkBool32", "swapchainMaintenance1", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSwapchainPresentFenceInfoEXT(const VkSwapchainPresentFenceInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.swapchainCount, NULL, settings, "uint32_t", "swapchainCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkFence>(object.pFences, object.swapchainCount, settings, "const VkFence*", "const VkFence", "pFences", false, false, indents + 1, dump_json_VkFence); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSwapchainPresentModesCreateInfoEXT(const VkSwapchainPresentModesCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.presentModeCount, NULL, settings, "uint32_t", "presentModeCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPresentModeKHR>(object.pPresentModes, object.presentModeCount, settings, "const VkPresentModeKHR*", "const VkPresentModeKHR", "pPresentModes", false, false, indents + 1, dump_json_VkPresentModeKHR); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSwapchainPresentModeInfoEXT(const VkSwapchainPresentModeInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.swapchainCount, NULL, settings, "uint32_t", "swapchainCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPresentModeKHR>(object.pPresentModes, object.swapchainCount, settings, "const VkPresentModeKHR*", "const VkPresentModeKHR", "pPresentModes", false, false, indents + 1, dump_json_VkPresentModeKHR); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSwapchainPresentScalingCreateInfoEXT(const VkSwapchainPresentScalingCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPresentScalingFlagsEXT>(object.scalingBehavior, NULL, settings, "VkPresentScalingFlagsEXT", "scalingBehavior", false, false, indents + 1, dump_json_VkPresentScalingFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkPresentGravityFlagsEXT>(object.presentGravityX, NULL, settings, "VkPresentGravityFlagsEXT", "presentGravityX", false, false, indents + 1, dump_json_VkPresentGravityFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkPresentGravityFlagsEXT>(object.presentGravityY, NULL, settings, "VkPresentGravityFlagsEXT", "presentGravityY", false, false, indents + 1, dump_json_VkPresentGravityFlagsEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkReleaseSwapchainImagesInfoEXT(const VkReleaseSwapchainImagesInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSwapchainKHR>(object.swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, indents + 1, dump_json_VkSwapchainKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.imageIndexCount, NULL, settings, "uint32_t", "imageIndexCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pImageIndices, object.imageIndexCount, settings, "const uint32_t*", "const uint32_t", "pImageIndices", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDeviceGeneratedCommandsPropertiesNV(const VkPhysicalDeviceDeviceGeneratedCommandsPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxGraphicsShaderGroupCount, NULL, settings, "uint32_t", "maxGraphicsShaderGroupCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxIndirectSequenceCount, NULL, settings, "uint32_t", "maxIndirectSequenceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxIndirectCommandsTokenCount, NULL, settings, "uint32_t", "maxIndirectCommandsTokenCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxIndirectCommandsStreamCount, NULL, settings, "uint32_t", "maxIndirectCommandsStreamCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxIndirectCommandsTokenOffset, NULL, settings, "uint32_t", "maxIndirectCommandsTokenOffset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxIndirectCommandsStreamStride, NULL, settings, "uint32_t", "maxIndirectCommandsStreamStride", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minSequencesCountBufferOffsetAlignment, NULL, settings, "uint32_t", "minSequencesCountBufferOffsetAlignment", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minSequencesIndexBufferOffsetAlignment, NULL, settings, "uint32_t", "minSequencesIndexBufferOffsetAlignment", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minIndirectCommandsBufferOffsetAlignment, NULL, settings, "uint32_t", "minIndirectCommandsBufferOffsetAlignment", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDeviceGeneratedCommandsFeaturesNV(const VkPhysicalDeviceDeviceGeneratedCommandsFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.deviceGeneratedCommands, NULL, settings, "VkBool32", "deviceGeneratedCommands", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkGraphicsShaderGroupCreateInfoNV(const VkGraphicsShaderGroupCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stageCount, NULL, settings, "uint32_t", "stageCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPipelineShaderStageCreateInfo>(object.pStages, object.stageCount, settings, "const VkPipelineShaderStageCreateInfo*", "const VkPipelineShaderStageCreateInfo", "pStages", true, false, indents + 1, dump_json_VkPipelineShaderStageCreateInfo); // JQA
    settings.stream() << ",\n";
    dump_json_pointer<const VkPipelineVertexInputStateCreateInfo>(object.pVertexInputState, settings, "const VkPipelineVertexInputStateCreateInfo*", "pVertexInputState", true, false, indents + 1, dump_json_VkPipelineVertexInputStateCreateInfo);
    settings.stream() << ",\n";
    dump_json_pointer<const VkPipelineTessellationStateCreateInfo>(object.pTessellationState, settings, "const VkPipelineTessellationStateCreateInfo*", "pTessellationState", true, false, indents + 1, dump_json_VkPipelineTessellationStateCreateInfo);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkGraphicsPipelineShaderGroupsCreateInfoNV(const VkGraphicsPipelineShaderGroupsCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.groupCount, NULL, settings, "uint32_t", "groupCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkGraphicsShaderGroupCreateInfoNV>(object.pGroups, object.groupCount, settings, "const VkGraphicsShaderGroupCreateInfoNV*", "const VkGraphicsShaderGroupCreateInfoNV", "pGroups", true, false, indents + 1, dump_json_VkGraphicsShaderGroupCreateInfoNV); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.pipelineCount, NULL, settings, "uint32_t", "pipelineCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPipeline>(object.pPipelines, object.pipelineCount, settings, "const VkPipeline*", "const VkPipeline", "pPipelines", false, false, indents + 1, dump_json_VkPipeline); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindShaderGroupIndirectCommandNV(const VkBindShaderGroupIndirectCommandNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.groupIndex, NULL, settings, "uint32_t", "groupIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindIndexBufferIndirectCommandNV(const VkBindIndexBufferIndirectCommandNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceAddress>(object.bufferAddress, NULL, settings, "VkDeviceAddress", "bufferAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.size, NULL, settings, "uint32_t", "size", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkIndexType>(object.indexType, NULL, settings, "VkIndexType", "indexType", false, false, indents + 1, dump_json_VkIndexType);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindVertexBufferIndirectCommandNV(const VkBindVertexBufferIndirectCommandNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceAddress>(object.bufferAddress, NULL, settings, "VkDeviceAddress", "bufferAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.size, NULL, settings, "uint32_t", "size", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stride, NULL, settings, "uint32_t", "stride", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSetStateFlagsIndirectCommandNV(const VkSetStateFlagsIndirectCommandNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.data, NULL, settings, "uint32_t", "data", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkIndirectCommandsStreamNV(const VkIndirectCommandsStreamNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.offset, NULL, settings, "VkDeviceSize", "offset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkIndirectCommandsLayoutTokenNV(const VkIndirectCommandsLayoutTokenNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkIndirectCommandsTokenTypeNV>(object.tokenType, NULL, settings, "VkIndirectCommandsTokenTypeNV", "tokenType", false, false, indents + 1, dump_json_VkIndirectCommandsTokenTypeNV);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stream, NULL, settings, "uint32_t", "stream", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.offset, NULL, settings, "uint32_t", "offset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.vertexBindingUnit, NULL, settings, "uint32_t", "vertexBindingUnit", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.vertexDynamicStride, NULL, settings, "VkBool32", "vertexDynamicStride", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineLayout>(object.pushconstantPipelineLayout, NULL, settings, "VkPipelineLayout", "pushconstantPipelineLayout", false, false, indents + 1, dump_json_VkPipelineLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlags>(object.pushconstantShaderStageFlags, NULL, settings, "VkShaderStageFlags", "pushconstantShaderStageFlags", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.pushconstantOffset, NULL, settings, "uint32_t", "pushconstantOffset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.pushconstantSize, NULL, settings, "uint32_t", "pushconstantSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkIndirectStateFlagsNV>(object.indirectStateFlags, NULL, settings, "VkIndirectStateFlagsNV", "indirectStateFlags", false, false, indents + 1, dump_json_VkIndirectStateFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.indexTypeCount, NULL, settings, "uint32_t", "indexTypeCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkIndexType>(object.pIndexTypes, object.indexTypeCount, settings, "const VkIndexType*", "const VkIndexType", "pIndexTypes", false, false, indents + 1, dump_json_VkIndexType); // JQA
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pIndexTypeValues, object.indexTypeCount, settings, "const uint32_t*", "const uint32_t", "pIndexTypeValues", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkIndirectCommandsLayoutCreateInfoNV(const VkIndirectCommandsLayoutCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkIndirectCommandsLayoutUsageFlagsNV>(object.flags, NULL, settings, "VkIndirectCommandsLayoutUsageFlagsNV", "flags", false, false, indents + 1, dump_json_VkIndirectCommandsLayoutUsageFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineBindPoint>(object.pipelineBindPoint, NULL, settings, "VkPipelineBindPoint", "pipelineBindPoint", false, false, indents + 1, dump_json_VkPipelineBindPoint);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.tokenCount, NULL, settings, "uint32_t", "tokenCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkIndirectCommandsLayoutTokenNV>(object.pTokens, object.tokenCount, settings, "const VkIndirectCommandsLayoutTokenNV*", "const VkIndirectCommandsLayoutTokenNV", "pTokens", true, false, indents + 1, dump_json_VkIndirectCommandsLayoutTokenNV); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.streamCount, NULL, settings, "uint32_t", "streamCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pStreamStrides, object.streamCount, settings, "const uint32_t*", "const uint32_t", "pStreamStrides", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkGeneratedCommandsInfoNV(const VkGeneratedCommandsInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineBindPoint>(object.pipelineBindPoint, NULL, settings, "VkPipelineBindPoint", "pipelineBindPoint", false, false, indents + 1, dump_json_VkPipelineBindPoint);
    settings.stream() << ",\n";
    dump_json_value<const VkPipeline>(object.pipeline, NULL, settings, "VkPipeline", "pipeline", false, false, indents + 1, dump_json_VkPipeline);
    settings.stream() << ",\n";
    dump_json_value<const VkIndirectCommandsLayoutNV>(object.indirectCommandsLayout, NULL, settings, "VkIndirectCommandsLayoutNV", "indirectCommandsLayout", false, false, indents + 1, dump_json_VkIndirectCommandsLayoutNV);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.streamCount, NULL, settings, "uint32_t", "streamCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkIndirectCommandsStreamNV>(object.pStreams, object.streamCount, settings, "const VkIndirectCommandsStreamNV*", "const VkIndirectCommandsStreamNV", "pStreams", true, false, indents + 1, dump_json_VkIndirectCommandsStreamNV); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.sequencesCount, NULL, settings, "uint32_t", "sequencesCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.preprocessBuffer, NULL, settings, "VkBuffer", "preprocessBuffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.preprocessOffset, NULL, settings, "VkDeviceSize", "preprocessOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.preprocessSize, NULL, settings, "VkDeviceSize", "preprocessSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.sequencesCountBuffer, NULL, settings, "VkBuffer", "sequencesCountBuffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.sequencesCountOffset, NULL, settings, "VkDeviceSize", "sequencesCountOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.sequencesIndexBuffer, NULL, settings, "VkBuffer", "sequencesIndexBuffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.sequencesIndexOffset, NULL, settings, "VkDeviceSize", "sequencesIndexOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkGeneratedCommandsMemoryRequirementsInfoNV(const VkGeneratedCommandsMemoryRequirementsInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineBindPoint>(object.pipelineBindPoint, NULL, settings, "VkPipelineBindPoint", "pipelineBindPoint", false, false, indents + 1, dump_json_VkPipelineBindPoint);
    settings.stream() << ",\n";
    dump_json_value<const VkPipeline>(object.pipeline, NULL, settings, "VkPipeline", "pipeline", false, false, indents + 1, dump_json_VkPipeline);
    settings.stream() << ",\n";
    dump_json_value<const VkIndirectCommandsLayoutNV>(object.indirectCommandsLayout, NULL, settings, "VkIndirectCommandsLayoutNV", "indirectCommandsLayout", false, false, indents + 1, dump_json_VkIndirectCommandsLayoutNV);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxSequencesCount, NULL, settings, "uint32_t", "maxSequencesCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceInheritedViewportScissorFeaturesNV(const VkPhysicalDeviceInheritedViewportScissorFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.inheritedViewportScissor2D, NULL, settings, "VkBool32", "inheritedViewportScissor2D", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCommandBufferInheritanceViewportScissorInfoNV(const VkCommandBufferInheritanceViewportScissorInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.viewportScissor2D, NULL, settings, "VkBool32", "viewportScissor2D", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.viewportDepthCount, NULL, settings, "uint32_t", "viewportDepthCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_pointer<const VkViewport>(object.pViewportDepths, settings, "const VkViewport*", "pViewportDepths", true, false, indents + 1, dump_json_VkViewport);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceTexelBufferAlignmentFeaturesEXT(const VkPhysicalDeviceTexelBufferAlignmentFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.texelBufferAlignment, NULL, settings, "VkBool32", "texelBufferAlignment", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassTransformBeginInfoQCOM(const VkRenderPassTransformBeginInfoQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceTransformFlagBitsKHR>(object.transform, NULL, settings, "VkSurfaceTransformFlagBitsKHR", "transform", false, false, indents + 1, dump_json_VkSurfaceTransformFlagBitsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCommandBufferInheritanceRenderPassTransformInfoQCOM(const VkCommandBufferInheritanceRenderPassTransformInfoQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceTransformFlagBitsKHR>(object.transform, NULL, settings, "VkSurfaceTransformFlagBitsKHR", "transform", false, false, indents + 1, dump_json_VkSurfaceTransformFlagBitsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkRect2D>(object.renderArea, NULL, settings, "VkRect2D", "renderArea", true, false, indents + 1, dump_json_VkRect2D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDepthBiasControlFeaturesEXT(const VkPhysicalDeviceDepthBiasControlFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthBiasControl, NULL, settings, "VkBool32", "depthBiasControl", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.leastRepresentableValueForceUnormRepresentation, NULL, settings, "VkBool32", "leastRepresentableValueForceUnormRepresentation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.floatRepresentation, NULL, settings, "VkBool32", "floatRepresentation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthBiasExact, NULL, settings, "VkBool32", "depthBiasExact", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDepthBiasInfoEXT(const VkDepthBiasInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const float>(object.depthBiasConstantFactor, NULL, settings, "float", "depthBiasConstantFactor", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.depthBiasClamp, NULL, settings, "float", "depthBiasClamp", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.depthBiasSlopeFactor, NULL, settings, "float", "depthBiasSlopeFactor", false, false, indents + 1, dump_json_float);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDepthBiasRepresentationInfoEXT(const VkDepthBiasRepresentationInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDepthBiasRepresentationEXT>(object.depthBiasRepresentation, NULL, settings, "VkDepthBiasRepresentationEXT", "depthBiasRepresentation", false, false, indents + 1, dump_json_VkDepthBiasRepresentationEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthBiasExact, NULL, settings, "VkBool32", "depthBiasExact", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDeviceMemoryReportFeaturesEXT(const VkPhysicalDeviceDeviceMemoryReportFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.deviceMemoryReport, NULL, settings, "VkBool32", "deviceMemoryReport", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceMemoryReportCallbackDataEXT(const VkDeviceMemoryReportCallbackDataEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemoryReportFlagsEXT>(object.flags, NULL, settings, "VkDeviceMemoryReportFlagsEXT", "flags", false, false, indents + 1, dump_json_VkDeviceMemoryReportFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemoryReportEventTypeEXT>(object.type, NULL, settings, "VkDeviceMemoryReportEventTypeEXT", "type", false, false, indents + 1, dump_json_VkDeviceMemoryReportEventTypeEXT);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.memoryObjectId, NULL, settings, "uint64_t", "memoryObjectId", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkObjectType>(object.objectType, NULL, settings, "VkObjectType", "objectType", false, false, indents + 1, dump_json_VkObjectType);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.objectHandle, NULL, settings, "uint64_t", "objectHandle", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.heapIndex, NULL, settings, "uint32_t", "heapIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceDeviceMemoryReportCreateInfoEXT(const VkDeviceDeviceMemoryReportCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemoryReportFlagsEXT>(object.flags, NULL, settings, "VkDeviceMemoryReportFlagsEXT", "flags", false, false, indents + 1, dump_json_VkDeviceMemoryReportFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const PFN_vkDeviceMemoryReportCallbackEXT>(object.pfnUserCallback, NULL, settings, "PFN_vkDeviceMemoryReportCallbackEXT", "pfnUserCallback", false, false, indents + 1, dump_json_PFN_vkDeviceMemoryReportCallbackEXT);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pUserData, NULL, settings, "void*", "pUserData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRobustness2FeaturesEXT(const VkPhysicalDeviceRobustness2FeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.robustBufferAccess2, NULL, settings, "VkBool32", "robustBufferAccess2", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.robustImageAccess2, NULL, settings, "VkBool32", "robustImageAccess2", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.nullDescriptor, NULL, settings, "VkBool32", "nullDescriptor", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRobustness2PropertiesEXT(const VkPhysicalDeviceRobustness2PropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.robustStorageBufferAccessSizeAlignment, NULL, settings, "VkDeviceSize", "robustStorageBufferAccessSizeAlignment", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.robustUniformBufferAccessSizeAlignment, NULL, settings, "VkDeviceSize", "robustUniformBufferAccessSizeAlignment", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSamplerCustomBorderColorCreateInfoEXT(const VkSamplerCustomBorderColorCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkClearColorValue>(object.customBorderColor, NULL, settings, "VkClearColorValue", "customBorderColor", false, true, indents + 1, dump_json_VkClearColorValue);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceCustomBorderColorPropertiesEXT(const VkPhysicalDeviceCustomBorderColorPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxCustomBorderColorSamplers, NULL, settings, "uint32_t", "maxCustomBorderColorSamplers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceCustomBorderColorFeaturesEXT(const VkPhysicalDeviceCustomBorderColorFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.customBorderColors, NULL, settings, "VkBool32", "customBorderColors", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.customBorderColorWithoutFormat, NULL, settings, "VkBool32", "customBorderColorWithoutFormat", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePresentBarrierFeaturesNV(const VkPhysicalDevicePresentBarrierFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.presentBarrier, NULL, settings, "VkBool32", "presentBarrier", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSurfaceCapabilitiesPresentBarrierNV(const VkSurfaceCapabilitiesPresentBarrierNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.presentBarrierSupported, NULL, settings, "VkBool32", "presentBarrierSupported", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSwapchainPresentBarrierCreateInfoNV(const VkSwapchainPresentBarrierCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.presentBarrierEnable, NULL, settings, "VkBool32", "presentBarrierEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPresentIdKHR(const VkPresentIdKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.swapchainCount, NULL, settings, "uint32_t", "swapchainCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint64_t>(object.pPresentIds, object.swapchainCount, settings, "const uint64_t*", "const uint64_t", "pPresentIds", false, false, indents + 1, dump_json_uint64_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePresentIdFeaturesKHR(const VkPhysicalDevicePresentIdFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.presentId, NULL, settings, "VkBool32", "presentId", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeInfoKHR(const VkVideoEncodeInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeFlagsKHR>(object.flags, NULL, settings, "VkVideoEncodeFlagsKHR", "flags", false, false, indents + 1, dump_json_VkVideoEncodeFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.dstBuffer, NULL, settings, "VkBuffer", "dstBuffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.dstBufferOffset, NULL, settings, "VkDeviceSize", "dstBufferOffset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.dstBufferRange, NULL, settings, "VkDeviceSize", "dstBufferRange", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoPictureResourceInfoKHR>(object.srcPictureResource, NULL, settings, "VkVideoPictureResourceInfoKHR", "srcPictureResource", true, false, indents + 1, dump_json_VkVideoPictureResourceInfoKHR);
    settings.stream() << ",\n";
    dump_json_pointer<const VkVideoReferenceSlotInfoKHR>(object.pSetupReferenceSlot, settings, "const VkVideoReferenceSlotInfoKHR*", "pSetupReferenceSlot", true, false, indents + 1, dump_json_VkVideoReferenceSlotInfoKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.referenceSlotCount, NULL, settings, "uint32_t", "referenceSlotCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkVideoReferenceSlotInfoKHR>(object.pReferenceSlots, object.referenceSlotCount, settings, "const VkVideoReferenceSlotInfoKHR*", "const VkVideoReferenceSlotInfoKHR", "pReferenceSlots", true, false, indents + 1, dump_json_VkVideoReferenceSlotInfoKHR); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.precedingExternallyEncodedBytes, NULL, settings, "uint32_t", "precedingExternallyEncodedBytes", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeCapabilitiesKHR(const VkVideoEncodeCapabilitiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeCapabilityFlagsKHR>(object.flags, NULL, settings, "VkVideoEncodeCapabilityFlagsKHR", "flags", false, false, indents + 1, dump_json_VkVideoEncodeCapabilityFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeRateControlModeFlagsKHR>(object.rateControlModes, NULL, settings, "VkVideoEncodeRateControlModeFlagsKHR", "rateControlModes", false, false, indents + 1, dump_json_VkVideoEncodeRateControlModeFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxRateControlLayers, NULL, settings, "uint32_t", "maxRateControlLayers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.maxBitrate, NULL, settings, "uint64_t", "maxBitrate", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxQualityLevels, NULL, settings, "uint32_t", "maxQualityLevels", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.encodeInputPictureGranularity, NULL, settings, "VkExtent2D", "encodeInputPictureGranularity", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeFeedbackFlagsKHR>(object.supportedEncodeFeedbackFlags, NULL, settings, "VkVideoEncodeFeedbackFlagsKHR", "supportedEncodeFeedbackFlags", false, false, indents + 1, dump_json_VkVideoEncodeFeedbackFlagsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkQueryPoolVideoEncodeFeedbackCreateInfoKHR(const VkQueryPoolVideoEncodeFeedbackCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeFeedbackFlagsKHR>(object.encodeFeedbackFlags, NULL, settings, "VkVideoEncodeFeedbackFlagsKHR", "encodeFeedbackFlags", false, false, indents + 1, dump_json_VkVideoEncodeFeedbackFlagsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeUsageInfoKHR(const VkVideoEncodeUsageInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeUsageFlagsKHR>(object.videoUsageHints, NULL, settings, "VkVideoEncodeUsageFlagsKHR", "videoUsageHints", false, false, indents + 1, dump_json_VkVideoEncodeUsageFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeContentFlagsKHR>(object.videoContentHints, NULL, settings, "VkVideoEncodeContentFlagsKHR", "videoContentHints", false, false, indents + 1, dump_json_VkVideoEncodeContentFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeTuningModeKHR>(object.tuningMode, NULL, settings, "VkVideoEncodeTuningModeKHR", "tuningMode", false, false, indents + 1, dump_json_VkVideoEncodeTuningModeKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeRateControlLayerInfoKHR(const VkVideoEncodeRateControlLayerInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.averageBitrate, NULL, settings, "uint64_t", "averageBitrate", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.maxBitrate, NULL, settings, "uint64_t", "maxBitrate", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.frameRateNumerator, NULL, settings, "uint32_t", "frameRateNumerator", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.frameRateDenominator, NULL, settings, "uint32_t", "frameRateDenominator", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeRateControlInfoKHR(const VkVideoEncodeRateControlInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeRateControlFlagsKHR>(object.flags, NULL, settings, "VkVideoEncodeRateControlFlagsKHR", "flags", false, false, indents + 1, dump_json_VkVideoEncodeRateControlFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeRateControlModeFlagBitsKHR>(object.rateControlMode, NULL, settings, "VkVideoEncodeRateControlModeFlagBitsKHR", "rateControlMode", false, false, indents + 1, dump_json_VkVideoEncodeRateControlModeFlagBitsKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.layerCount, NULL, settings, "uint32_t", "layerCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkVideoEncodeRateControlLayerInfoKHR>(object.pLayers, object.layerCount, settings, "const VkVideoEncodeRateControlLayerInfoKHR*", "const VkVideoEncodeRateControlLayerInfoKHR", "pLayers", true, false, indents + 1, dump_json_VkVideoEncodeRateControlLayerInfoKHR); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.virtualBufferSizeInMs, NULL, settings, "uint32_t", "virtualBufferSizeInMs", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.initialVirtualBufferSizeInMs, NULL, settings, "uint32_t", "initialVirtualBufferSizeInMs", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceVideoEncodeQualityLevelInfoKHR(const VkPhysicalDeviceVideoEncodeQualityLevelInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const VkVideoProfileInfoKHR>(object.pVideoProfile, settings, "const VkVideoProfileInfoKHR*", "pVideoProfile", true, false, indents + 1, dump_json_VkVideoProfileInfoKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.qualityLevel, NULL, settings, "uint32_t", "qualityLevel", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeQualityLevelPropertiesKHR(const VkVideoEncodeQualityLevelPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoEncodeRateControlModeFlagBitsKHR>(object.preferredRateControlMode, NULL, settings, "VkVideoEncodeRateControlModeFlagBitsKHR", "preferredRateControlMode", false, false, indents + 1, dump_json_VkVideoEncodeRateControlModeFlagBitsKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.preferredRateControlLayerCount, NULL, settings, "uint32_t", "preferredRateControlLayerCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeQualityLevelInfoKHR(const VkVideoEncodeQualityLevelInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.qualityLevel, NULL, settings, "uint32_t", "qualityLevel", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeSessionParametersGetInfoKHR(const VkVideoEncodeSessionParametersGetInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkVideoSessionParametersKHR>(object.videoSessionParameters, NULL, settings, "VkVideoSessionParametersKHR", "videoSessionParameters", false, false, indents + 1, dump_json_VkVideoSessionParametersKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoEncodeSessionParametersFeedbackInfoKHR(const VkVideoEncodeSessionParametersFeedbackInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.hasOverrides, NULL, settings, "VkBool32", "hasOverrides", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDiagnosticsConfigFeaturesNV(const VkPhysicalDeviceDiagnosticsConfigFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.diagnosticsConfig, NULL, settings, "VkBool32", "diagnosticsConfig", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceDiagnosticsConfigCreateInfoNV(const VkDeviceDiagnosticsConfigCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceDiagnosticsConfigFlagsNV>(object.flags, NULL, settings, "VkDeviceDiagnosticsConfigFlagsNV", "flags", false, false, indents + 1, dump_json_VkDeviceDiagnosticsConfigFlagsNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCudaModuleCreateInfoNV(const VkCudaModuleCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.dataSize, NULL, settings, "size_t", "dataSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pData, NULL, settings, "const void*", "pData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCudaFunctionCreateInfoNV(const VkCudaFunctionCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkCudaModuleNV>(object.module, NULL, settings, "VkCudaModuleNV", "module", false, false, indents + 1, dump_json_VkCudaModuleNV);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.pName, NULL, settings, "const char*", "pName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCudaLaunchInfoNV(const VkCudaLaunchInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkCudaFunctionNV>(object.function, NULL, settings, "VkCudaFunctionNV", "function", false, false, indents + 1, dump_json_VkCudaFunctionNV);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.gridDimX, NULL, settings, "uint32_t", "gridDimX", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.gridDimY, NULL, settings, "uint32_t", "gridDimY", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.gridDimZ, NULL, settings, "uint32_t", "gridDimZ", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.blockDimX, NULL, settings, "uint32_t", "blockDimX", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.blockDimY, NULL, settings, "uint32_t", "blockDimY", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.blockDimZ, NULL, settings, "uint32_t", "blockDimZ", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.sharedMemBytes, NULL, settings, "uint32_t", "sharedMemBytes", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.paramCount, NULL, settings, "size_t", "paramCount", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_array<const void*>(object.pParams, object.paramCount, settings, "const void* const *", "const void* const ", "pParams", false, false, indents + 1, dump_json_void); // JQA
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.extraCount, NULL, settings, "size_t", "extraCount", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_array<const void*>(object.pExtras, object.extraCount, settings, "const void* const *", "const void* const ", "pExtras", false, false, indents + 1, dump_json_void); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceCudaKernelLaunchFeaturesNV(const VkPhysicalDeviceCudaKernelLaunchFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.cudaKernelLaunchFeatures, NULL, settings, "VkBool32", "cudaKernelLaunchFeatures", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceCudaKernelLaunchPropertiesNV(const VkPhysicalDeviceCudaKernelLaunchPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.computeCapabilityMinor, NULL, settings, "uint32_t", "computeCapabilityMinor", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.computeCapabilityMajor, NULL, settings, "uint32_t", "computeCapabilityMajor", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkQueryLowLatencySupportNV(const VkQueryLowLatencySupportNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pQueriedLowLatencyData, NULL, settings, "void*", "pQueriedLowLatencyData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkExportMetalObjectCreateInfoEXT(const VkExportMetalObjectCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExportMetalObjectTypeFlagBitsEXT>(object.exportObjectType, NULL, settings, "VkExportMetalObjectTypeFlagBitsEXT", "exportObjectType", false, false, indents + 1, dump_json_VkExportMetalObjectTypeFlagBitsEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkExportMetalObjectsInfoEXT(const VkExportMetalObjectsInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkExportMetalDeviceInfoEXT(const VkExportMetalDeviceInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const MTLDevice_id>(object.mtlDevice, NULL, settings, "MTLDevice_id", "mtlDevice", false, false, indents + 1, dump_json_MTLDevice_id);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkExportMetalCommandQueueInfoEXT(const VkExportMetalCommandQueueInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkQueue>(object.queue, NULL, settings, "VkQueue", "queue", false, false, indents + 1, dump_json_VkQueue);
    settings.stream() << ",\n";
    dump_json_value<const MTLCommandQueue_id>(object.mtlCommandQueue, NULL, settings, "MTLCommandQueue_id", "mtlCommandQueue", false, false, indents + 1, dump_json_MTLCommandQueue_id);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkExportMetalBufferInfoEXT(const VkExportMetalBufferInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << ",\n";
    dump_json_value<const MTLBuffer_id>(object.mtlBuffer, NULL, settings, "MTLBuffer_id", "mtlBuffer", false, false, indents + 1, dump_json_MTLBuffer_id);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkImportMetalBufferInfoEXT(const VkImportMetalBufferInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const MTLBuffer_id>(object.mtlBuffer, NULL, settings, "MTLBuffer_id", "mtlBuffer", false, false, indents + 1, dump_json_MTLBuffer_id);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkExportMetalTextureInfoEXT(const VkExportMetalTextureInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.image, NULL, settings, "VkImage", "image", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const VkImageView>(object.imageView, NULL, settings, "VkImageView", "imageView", false, false, indents + 1, dump_json_VkImageView);
    settings.stream() << ",\n";
    dump_json_value<const VkBufferView>(object.bufferView, NULL, settings, "VkBufferView", "bufferView", false, false, indents + 1, dump_json_VkBufferView);
    settings.stream() << ",\n";
    dump_json_value<const VkImageAspectFlagBits>(object.plane, NULL, settings, "VkImageAspectFlagBits", "plane", false, false, indents + 1, dump_json_VkImageAspectFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const MTLTexture_id>(object.mtlTexture, NULL, settings, "MTLTexture_id", "mtlTexture", false, false, indents + 1, dump_json_MTLTexture_id);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkImportMetalTextureInfoEXT(const VkImportMetalTextureInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageAspectFlagBits>(object.plane, NULL, settings, "VkImageAspectFlagBits", "plane", false, false, indents + 1, dump_json_VkImageAspectFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const MTLTexture_id>(object.mtlTexture, NULL, settings, "MTLTexture_id", "mtlTexture", false, false, indents + 1, dump_json_MTLTexture_id);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkExportMetalIOSurfaceInfoEXT(const VkExportMetalIOSurfaceInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.image, NULL, settings, "VkImage", "image", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << ",\n";
    dump_json_value<const IOSurfaceRef>(object.ioSurface, NULL, settings, "IOSurfaceRef", "ioSurface", false, false, indents + 1, dump_json_IOSurfaceRef);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkImportMetalIOSurfaceInfoEXT(const VkImportMetalIOSurfaceInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const IOSurfaceRef>(object.ioSurface, NULL, settings, "IOSurfaceRef", "ioSurface", false, false, indents + 1, dump_json_IOSurfaceRef);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkExportMetalSharedEventInfoEXT(const VkExportMetalSharedEventInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphore>(object.semaphore, NULL, settings, "VkSemaphore", "semaphore", false, false, indents + 1, dump_json_VkSemaphore);
    settings.stream() << ",\n";
    dump_json_value<const VkEvent>(object.event, NULL, settings, "VkEvent", "event", false, false, indents + 1, dump_json_VkEvent);
    settings.stream() << ",\n";
    dump_json_value<const MTLSharedEvent_id>(object.mtlSharedEvent, NULL, settings, "MTLSharedEvent_id", "mtlSharedEvent", false, false, indents + 1, dump_json_MTLSharedEvent_id);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_VkImportMetalSharedEventInfoEXT(const VkImportMetalSharedEventInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const MTLSharedEvent_id>(object.mtlSharedEvent, NULL, settings, "MTLSharedEvent_id", "mtlSharedEvent", false, false, indents + 1, dump_json_MTLSharedEvent_id);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_METAL_EXT
void dump_json_VkQueueFamilyCheckpointProperties2NV(const VkQueueFamilyCheckpointProperties2NV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineStageFlags2>(object.checkpointExecutionStageMask, NULL, settings, "VkPipelineStageFlags2", "checkpointExecutionStageMask", false, false, indents + 1, dump_json_VkPipelineStageFlags2);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCheckpointData2NV(const VkCheckpointData2NV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineStageFlags2>(object.stage, NULL, settings, "VkPipelineStageFlags2", "stage", false, false, indents + 1, dump_json_VkPipelineStageFlags2);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pCheckpointMarker, NULL, settings, "void*", "pCheckpointMarker", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDescriptorBufferPropertiesEXT(const VkPhysicalDeviceDescriptorBufferPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.combinedImageSamplerDescriptorSingleArray, NULL, settings, "VkBool32", "combinedImageSamplerDescriptorSingleArray", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.bufferlessPushDescriptors, NULL, settings, "VkBool32", "bufferlessPushDescriptors", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.allowSamplerImageViewPostSubmitCreation, NULL, settings, "VkBool32", "allowSamplerImageViewPostSubmitCreation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.descriptorBufferOffsetAlignment, NULL, settings, "VkDeviceSize", "descriptorBufferOffsetAlignment", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorBufferBindings, NULL, settings, "uint32_t", "maxDescriptorBufferBindings", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxResourceDescriptorBufferBindings, NULL, settings, "uint32_t", "maxResourceDescriptorBufferBindings", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxSamplerDescriptorBufferBindings, NULL, settings, "uint32_t", "maxSamplerDescriptorBufferBindings", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxEmbeddedImmutableSamplerBindings, NULL, settings, "uint32_t", "maxEmbeddedImmutableSamplerBindings", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxEmbeddedImmutableSamplers, NULL, settings, "uint32_t", "maxEmbeddedImmutableSamplers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.bufferCaptureReplayDescriptorDataSize, NULL, settings, "size_t", "bufferCaptureReplayDescriptorDataSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.imageCaptureReplayDescriptorDataSize, NULL, settings, "size_t", "imageCaptureReplayDescriptorDataSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.imageViewCaptureReplayDescriptorDataSize, NULL, settings, "size_t", "imageViewCaptureReplayDescriptorDataSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.samplerCaptureReplayDescriptorDataSize, NULL, settings, "size_t", "samplerCaptureReplayDescriptorDataSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.accelerationStructureCaptureReplayDescriptorDataSize, NULL, settings, "size_t", "accelerationStructureCaptureReplayDescriptorDataSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.samplerDescriptorSize, NULL, settings, "size_t", "samplerDescriptorSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.combinedImageSamplerDescriptorSize, NULL, settings, "size_t", "combinedImageSamplerDescriptorSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.sampledImageDescriptorSize, NULL, settings, "size_t", "sampledImageDescriptorSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.storageImageDescriptorSize, NULL, settings, "size_t", "storageImageDescriptorSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.uniformTexelBufferDescriptorSize, NULL, settings, "size_t", "uniformTexelBufferDescriptorSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.robustUniformTexelBufferDescriptorSize, NULL, settings, "size_t", "robustUniformTexelBufferDescriptorSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.storageTexelBufferDescriptorSize, NULL, settings, "size_t", "storageTexelBufferDescriptorSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.robustStorageTexelBufferDescriptorSize, NULL, settings, "size_t", "robustStorageTexelBufferDescriptorSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.uniformBufferDescriptorSize, NULL, settings, "size_t", "uniformBufferDescriptorSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.robustUniformBufferDescriptorSize, NULL, settings, "size_t", "robustUniformBufferDescriptorSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.storageBufferDescriptorSize, NULL, settings, "size_t", "storageBufferDescriptorSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.robustStorageBufferDescriptorSize, NULL, settings, "size_t", "robustStorageBufferDescriptorSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.inputAttachmentDescriptorSize, NULL, settings, "size_t", "inputAttachmentDescriptorSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.accelerationStructureDescriptorSize, NULL, settings, "size_t", "accelerationStructureDescriptorSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.maxSamplerDescriptorBufferRange, NULL, settings, "VkDeviceSize", "maxSamplerDescriptorBufferRange", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.maxResourceDescriptorBufferRange, NULL, settings, "VkDeviceSize", "maxResourceDescriptorBufferRange", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.samplerDescriptorBufferAddressSpaceSize, NULL, settings, "VkDeviceSize", "samplerDescriptorBufferAddressSpaceSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.resourceDescriptorBufferAddressSpaceSize, NULL, settings, "VkDeviceSize", "resourceDescriptorBufferAddressSpaceSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.descriptorBufferAddressSpaceSize, NULL, settings, "VkDeviceSize", "descriptorBufferAddressSpaceSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDescriptorBufferDensityMapPropertiesEXT(const VkPhysicalDeviceDescriptorBufferDensityMapPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.combinedImageSamplerDensityMapDescriptorSize, NULL, settings, "size_t", "combinedImageSamplerDensityMapDescriptorSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDescriptorBufferFeaturesEXT(const VkPhysicalDeviceDescriptorBufferFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBuffer, NULL, settings, "VkBool32", "descriptorBuffer", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBufferCaptureReplay, NULL, settings, "VkBool32", "descriptorBufferCaptureReplay", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBufferImageLayoutIgnored, NULL, settings, "VkBool32", "descriptorBufferImageLayoutIgnored", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorBufferPushDescriptors, NULL, settings, "VkBool32", "descriptorBufferPushDescriptors", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorAddressInfoEXT(const VkDescriptorAddressInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddress>(object.address, NULL, settings, "VkDeviceAddress", "address", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.range, NULL, settings, "VkDeviceSize", "range", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorBufferBindingInfoEXT(const VkDescriptorBufferBindingInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddress>(object.address, NULL, settings, "VkDeviceAddress", "address", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const VkBufferUsageFlags>(object.usage, NULL, settings, "VkBufferUsageFlags", "usage", false, false, indents + 1, dump_json_VkBufferUsageFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorBufferBindingPushDescriptorBufferHandleEXT(const VkDescriptorBufferBindingPushDescriptorBufferHandleEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorGetInfoEXT(const VkDescriptorGetInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    ApiDumpInstance::current().setDescriptorType(object.type);
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorType>(object.type, NULL, settings, "VkDescriptorType", "type", false, false, indents + 1, dump_json_VkDescriptorType);
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorDataEXT>(object.data, NULL, settings, "VkDescriptorDataEXT", "data", false, true, indents + 1, dump_json_VkDescriptorDataEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBufferCaptureDescriptorDataInfoEXT(const VkBufferCaptureDescriptorDataInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageCaptureDescriptorDataInfoEXT(const VkImageCaptureDescriptorDataInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImage>(object.image, NULL, settings, "VkImage", "image", false, false, indents + 1, dump_json_VkImage);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageViewCaptureDescriptorDataInfoEXT(const VkImageViewCaptureDescriptorDataInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageView>(object.imageView, NULL, settings, "VkImageView", "imageView", false, false, indents + 1, dump_json_VkImageView);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSamplerCaptureDescriptorDataInfoEXT(const VkSamplerCaptureDescriptorDataInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSampler>(object.sampler, NULL, settings, "VkSampler", "sampler", false, false, indents + 1, dump_json_VkSampler);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkOpaqueCaptureDescriptorDataCreateInfoEXT(const VkOpaqueCaptureDescriptorDataCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.opaqueCaptureDescriptorData, NULL, settings, "const void*", "opaqueCaptureDescriptorData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureCaptureDescriptorDataInfoEXT(const VkAccelerationStructureCaptureDescriptorDataInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureKHR>(object.accelerationStructure, NULL, settings, "VkAccelerationStructureKHR", "accelerationStructure", false, false, indents + 1, dump_json_VkAccelerationStructureKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureNV>(object.accelerationStructureNV, NULL, settings, "VkAccelerationStructureNV", "accelerationStructureNV", false, false, indents + 1, dump_json_VkAccelerationStructureNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceGraphicsPipelineLibraryFeaturesEXT(const VkPhysicalDeviceGraphicsPipelineLibraryFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.graphicsPipelineLibrary, NULL, settings, "VkBool32", "graphicsPipelineLibrary", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceGraphicsPipelineLibraryPropertiesEXT(const VkPhysicalDeviceGraphicsPipelineLibraryPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.graphicsPipelineLibraryFastLinking, NULL, settings, "VkBool32", "graphicsPipelineLibraryFastLinking", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.graphicsPipelineLibraryIndependentInterpolationDecoration, NULL, settings, "VkBool32", "graphicsPipelineLibraryIndependentInterpolationDecoration", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkGraphicsPipelineLibraryCreateInfoEXT(const VkGraphicsPipelineLibraryCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkGraphicsPipelineLibraryFlagsEXT>(object.flags, NULL, settings, "VkGraphicsPipelineLibraryFlagsEXT", "flags", false, false, indents + 1, dump_json_VkGraphicsPipelineLibraryFlagsEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderEarlyAndLateFragmentTestsFeaturesAMD(const VkPhysicalDeviceShaderEarlyAndLateFragmentTestsFeaturesAMD& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderEarlyAndLateFragmentTests, NULL, settings, "VkBool32", "shaderEarlyAndLateFragmentTests", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFragmentShaderBarycentricPropertiesKHR(const VkPhysicalDeviceFragmentShaderBarycentricPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.triStripVertexOrderIndependentOfProvokingVertex, NULL, settings, "VkBool32", "triStripVertexOrderIndependentOfProvokingVertex", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderSubgroupUniformControlFlowFeaturesKHR(const VkPhysicalDeviceShaderSubgroupUniformControlFlowFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSubgroupUniformControlFlow, NULL, settings, "VkBool32", "shaderSubgroupUniformControlFlow", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFragmentShadingRateEnumsFeaturesNV(const VkPhysicalDeviceFragmentShadingRateEnumsFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentShadingRateEnums, NULL, settings, "VkBool32", "fragmentShadingRateEnums", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.supersampleFragmentShadingRates, NULL, settings, "VkBool32", "supersampleFragmentShadingRates", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.noInvocationFragmentShadingRates, NULL, settings, "VkBool32", "noInvocationFragmentShadingRates", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFragmentShadingRateEnumsPropertiesNV(const VkPhysicalDeviceFragmentShadingRateEnumsPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlagBits>(object.maxFragmentShadingRateInvocationCount, NULL, settings, "VkSampleCountFlagBits", "maxFragmentShadingRateInvocationCount", false, false, indents + 1, dump_json_VkSampleCountFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineFragmentShadingRateEnumStateCreateInfoNV(const VkPipelineFragmentShadingRateEnumStateCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFragmentShadingRateTypeNV>(object.shadingRateType, NULL, settings, "VkFragmentShadingRateTypeNV", "shadingRateType", false, false, indents + 1, dump_json_VkFragmentShadingRateTypeNV);
    settings.stream() << ",\n";
    dump_json_value<const VkFragmentShadingRateNV>(object.shadingRate, NULL, settings, "VkFragmentShadingRateNV", "shadingRate", false, false, indents + 1, dump_json_VkFragmentShadingRateNV);
    settings.stream() << ",\n";
    dump_json_array<const VkFragmentShadingRateCombinerOpKHR>(object.combinerOps, 2, settings, "VkFragmentShadingRateCombinerOpKHR[2]", "VkFragmentShadingRateCombinerOpKHR", "combinerOps", false, false, indents + 1, dump_json_VkFragmentShadingRateCombinerOpKHR); // IQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureGeometryMotionTrianglesDataNV(const VkAccelerationStructureGeometryMotionTrianglesDataNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstKHR>(object.vertexData, NULL, settings, "VkDeviceOrHostAddressConstKHR", "vertexData", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureMotionInfoNV(const VkAccelerationStructureMotionInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxInstances, NULL, settings, "uint32_t", "maxInstances", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureMotionInfoFlagsNV>(object.flags, NULL, settings, "VkAccelerationStructureMotionInfoFlagsNV", "flags", false, false, indents + 1, dump_json_VkAccelerationStructureMotionInfoFlagsNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureMatrixMotionInstanceNV(const VkAccelerationStructureMatrixMotionInstanceNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkTransformMatrixKHR>(object.transformT0, NULL, settings, "VkTransformMatrixKHR", "transformT0", true, false, indents + 1, dump_json_VkTransformMatrixKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkTransformMatrixKHR>(object.transformT1, NULL, settings, "VkTransformMatrixKHR", "transformT1", true, false, indents + 1, dump_json_VkTransformMatrixKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.instanceCustomIndex, NULL, settings, "uint32_t instanceCustomIndex:24", "instanceCustomIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.mask, NULL, settings, "uint32_t mask:8", "mask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.instanceShaderBindingTableRecordOffset, NULL, settings, "uint32_t instanceShaderBindingTableRecordOffset:24", "instanceShaderBindingTableRecordOffset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkGeometryInstanceFlagsKHR>(object.flags, NULL, settings, "VkGeometryInstanceFlagsKHR flags:8", "flags", false, false, indents + 1, dump_json_VkGeometryInstanceFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.accelerationStructureReference, NULL, settings, "uint64_t", "accelerationStructureReference", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSRTDataNV(const VkSRTDataNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const float>(object.sx, NULL, settings, "float", "sx", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.a, NULL, settings, "float", "a", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.b, NULL, settings, "float", "b", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.pvx, NULL, settings, "float", "pvx", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.sy, NULL, settings, "float", "sy", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.c, NULL, settings, "float", "c", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.pvy, NULL, settings, "float", "pvy", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.sz, NULL, settings, "float", "sz", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.pvz, NULL, settings, "float", "pvz", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.qx, NULL, settings, "float", "qx", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.qy, NULL, settings, "float", "qy", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.qz, NULL, settings, "float", "qz", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.qw, NULL, settings, "float", "qw", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.tx, NULL, settings, "float", "tx", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.ty, NULL, settings, "float", "ty", false, false, indents + 1, dump_json_float);
    settings.stream() << ",\n";
    dump_json_value<const float>(object.tz, NULL, settings, "float", "tz", false, false, indents + 1, dump_json_float);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureSRTMotionInstanceNV(const VkAccelerationStructureSRTMotionInstanceNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkSRTDataNV>(object.transformT0, NULL, settings, "VkSRTDataNV", "transformT0", true, false, indents + 1, dump_json_VkSRTDataNV);
    settings.stream() << ",\n";
    dump_json_value<const VkSRTDataNV>(object.transformT1, NULL, settings, "VkSRTDataNV", "transformT1", true, false, indents + 1, dump_json_VkSRTDataNV);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.instanceCustomIndex, NULL, settings, "uint32_t instanceCustomIndex:24", "instanceCustomIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.mask, NULL, settings, "uint32_t mask:8", "mask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.instanceShaderBindingTableRecordOffset, NULL, settings, "uint32_t instanceShaderBindingTableRecordOffset:24", "instanceShaderBindingTableRecordOffset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkGeometryInstanceFlagsKHR>(object.flags, NULL, settings, "VkGeometryInstanceFlagsKHR flags:8", "flags", false, false, indents + 1, dump_json_VkGeometryInstanceFlagsKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.accelerationStructureReference, NULL, settings, "uint64_t", "accelerationStructureReference", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureMotionInstanceNV(const VkAccelerationStructureMotionInstanceNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkAccelerationStructureMotionInstanceTypeNV>(object.type, NULL, settings, "VkAccelerationStructureMotionInstanceTypeNV", "type", false, false, indents + 1, dump_json_VkAccelerationStructureMotionInstanceTypeNV);
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureMotionInstanceFlagsNV>(object.flags, NULL, settings, "VkAccelerationStructureMotionInstanceFlagsNV", "flags", false, false, indents + 1, dump_json_VkAccelerationStructureMotionInstanceFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const VkAccelerationStructureMotionInstanceDataNV>(object.data, NULL, settings, "VkAccelerationStructureMotionInstanceDataNV", "data", false, true, indents + 1, dump_json_VkAccelerationStructureMotionInstanceDataNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRayTracingMotionBlurFeaturesNV(const VkPhysicalDeviceRayTracingMotionBlurFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rayTracingMotionBlur, NULL, settings, "VkBool32", "rayTracingMotionBlur", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rayTracingMotionBlurPipelineTraceRaysIndirect, NULL, settings, "VkBool32", "rayTracingMotionBlurPipelineTraceRaysIndirect", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMeshShaderFeaturesEXT(const VkPhysicalDeviceMeshShaderFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.taskShader, NULL, settings, "VkBool32", "taskShader", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.meshShader, NULL, settings, "VkBool32", "meshShader", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multiviewMeshShader, NULL, settings, "VkBool32", "multiviewMeshShader", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.primitiveFragmentShadingRateMeshShader, NULL, settings, "VkBool32", "primitiveFragmentShadingRateMeshShader", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.meshShaderQueries, NULL, settings, "VkBool32", "meshShaderQueries", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMeshShaderPropertiesEXT(const VkPhysicalDeviceMeshShaderPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTaskWorkGroupTotalCount, NULL, settings, "uint32_t", "maxTaskWorkGroupTotalCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.maxTaskWorkGroupCount, 3, settings, "uint32_t[3]", "uint32_t", "maxTaskWorkGroupCount", false, false, indents + 1, dump_json_uint32_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTaskWorkGroupInvocations, NULL, settings, "uint32_t", "maxTaskWorkGroupInvocations", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.maxTaskWorkGroupSize, 3, settings, "uint32_t[3]", "uint32_t", "maxTaskWorkGroupSize", false, false, indents + 1, dump_json_uint32_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTaskPayloadSize, NULL, settings, "uint32_t", "maxTaskPayloadSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTaskSharedMemorySize, NULL, settings, "uint32_t", "maxTaskSharedMemorySize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxTaskPayloadAndSharedMemorySize, NULL, settings, "uint32_t", "maxTaskPayloadAndSharedMemorySize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshWorkGroupTotalCount, NULL, settings, "uint32_t", "maxMeshWorkGroupTotalCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.maxMeshWorkGroupCount, 3, settings, "uint32_t[3]", "uint32_t", "maxMeshWorkGroupCount", false, false, indents + 1, dump_json_uint32_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshWorkGroupInvocations, NULL, settings, "uint32_t", "maxMeshWorkGroupInvocations", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.maxMeshWorkGroupSize, 3, settings, "uint32_t[3]", "uint32_t", "maxMeshWorkGroupSize", false, false, indents + 1, dump_json_uint32_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshSharedMemorySize, NULL, settings, "uint32_t", "maxMeshSharedMemorySize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshPayloadAndSharedMemorySize, NULL, settings, "uint32_t", "maxMeshPayloadAndSharedMemorySize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshOutputMemorySize, NULL, settings, "uint32_t", "maxMeshOutputMemorySize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshPayloadAndOutputMemorySize, NULL, settings, "uint32_t", "maxMeshPayloadAndOutputMemorySize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshOutputComponents, NULL, settings, "uint32_t", "maxMeshOutputComponents", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshOutputVertices, NULL, settings, "uint32_t", "maxMeshOutputVertices", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshOutputPrimitives, NULL, settings, "uint32_t", "maxMeshOutputPrimitives", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshOutputLayers, NULL, settings, "uint32_t", "maxMeshOutputLayers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMeshMultiviewViewCount, NULL, settings, "uint32_t", "maxMeshMultiviewViewCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.meshOutputPerVertexGranularity, NULL, settings, "uint32_t", "meshOutputPerVertexGranularity", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.meshOutputPerPrimitiveGranularity, NULL, settings, "uint32_t", "meshOutputPerPrimitiveGranularity", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPreferredTaskWorkGroupInvocations, NULL, settings, "uint32_t", "maxPreferredTaskWorkGroupInvocations", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPreferredMeshWorkGroupInvocations, NULL, settings, "uint32_t", "maxPreferredMeshWorkGroupInvocations", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.prefersLocalInvocationVertexOutput, NULL, settings, "VkBool32", "prefersLocalInvocationVertexOutput", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.prefersLocalInvocationPrimitiveOutput, NULL, settings, "VkBool32", "prefersLocalInvocationPrimitiveOutput", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.prefersCompactVertexOutput, NULL, settings, "VkBool32", "prefersCompactVertexOutput", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.prefersCompactPrimitiveOutput, NULL, settings, "VkBool32", "prefersCompactPrimitiveOutput", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDrawMeshTasksIndirectCommandEXT(const VkDrawMeshTasksIndirectCommandEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.groupCountX, NULL, settings, "uint32_t", "groupCountX", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.groupCountY, NULL, settings, "uint32_t", "groupCountY", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.groupCountZ, NULL, settings, "uint32_t", "groupCountZ", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceYcbcr2Plane444FormatsFeaturesEXT(const VkPhysicalDeviceYcbcr2Plane444FormatsFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.ycbcr2plane444Formats, NULL, settings, "VkBool32", "ycbcr2plane444Formats", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFragmentDensityMap2FeaturesEXT(const VkPhysicalDeviceFragmentDensityMap2FeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentDensityMapDeferred, NULL, settings, "VkBool32", "fragmentDensityMapDeferred", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFragmentDensityMap2PropertiesEXT(const VkPhysicalDeviceFragmentDensityMap2PropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.subsampledLoads, NULL, settings, "VkBool32", "subsampledLoads", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.subsampledCoarseReconstructionEarlyAccess, NULL, settings, "VkBool32", "subsampledCoarseReconstructionEarlyAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxSubsampledArrayLayers, NULL, settings, "uint32_t", "maxSubsampledArrayLayers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetSubsampledSamplers, NULL, settings, "uint32_t", "maxDescriptorSetSubsampledSamplers", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyCommandTransformInfoQCOM(const VkCopyCommandTransformInfoQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSurfaceTransformFlagBitsKHR>(object.transform, NULL, settings, "VkSurfaceTransformFlagBitsKHR", "transform", false, false, indents + 1, dump_json_VkSurfaceTransformFlagBitsKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceWorkgroupMemoryExplicitLayoutFeaturesKHR(const VkPhysicalDeviceWorkgroupMemoryExplicitLayoutFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.workgroupMemoryExplicitLayout, NULL, settings, "VkBool32", "workgroupMemoryExplicitLayout", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.workgroupMemoryExplicitLayoutScalarBlockLayout, NULL, settings, "VkBool32", "workgroupMemoryExplicitLayoutScalarBlockLayout", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.workgroupMemoryExplicitLayout8BitAccess, NULL, settings, "VkBool32", "workgroupMemoryExplicitLayout8BitAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.workgroupMemoryExplicitLayout16BitAccess, NULL, settings, "VkBool32", "workgroupMemoryExplicitLayout16BitAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImageCompressionControlFeaturesEXT(const VkPhysicalDeviceImageCompressionControlFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.imageCompressionControl, NULL, settings, "VkBool32", "imageCompressionControl", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageCompressionControlEXT(const VkImageCompressionControlEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageCompressionFlagsEXT>(object.flags, NULL, settings, "VkImageCompressionFlagsEXT", "flags", false, false, indents + 1, dump_json_VkImageCompressionFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.compressionControlPlaneCount, NULL, settings, "uint32_t", "compressionControlPlaneCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkImageCompressionFixedRateFlagsEXT>(object.pFixedRateFlags, object.compressionControlPlaneCount, settings, "VkImageCompressionFixedRateFlagsEXT*", "VkImageCompressionFixedRateFlagsEXT", "pFixedRateFlags", false, false, indents + 1, dump_json_VkImageCompressionFixedRateFlagsEXT); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageCompressionPropertiesEXT(const VkImageCompressionPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageCompressionFlagsEXT>(object.imageCompressionFlags, NULL, settings, "VkImageCompressionFlagsEXT", "imageCompressionFlags", false, false, indents + 1, dump_json_VkImageCompressionFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkImageCompressionFixedRateFlagsEXT>(object.imageCompressionFixedRateFlags, NULL, settings, "VkImageCompressionFixedRateFlagsEXT", "imageCompressionFixedRateFlags", false, false, indents + 1, dump_json_VkImageCompressionFixedRateFlagsEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceAttachmentFeedbackLoopLayoutFeaturesEXT(const VkPhysicalDeviceAttachmentFeedbackLoopLayoutFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.attachmentFeedbackLoopLayout, NULL, settings, "VkBool32", "attachmentFeedbackLoopLayout", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevice4444FormatsFeaturesEXT(const VkPhysicalDevice4444FormatsFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.formatA4R4G4B4, NULL, settings, "VkBool32", "formatA4R4G4B4", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.formatA4B4G4R4, NULL, settings, "VkBool32", "formatA4B4G4R4", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFaultFeaturesEXT(const VkPhysicalDeviceFaultFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.deviceFault, NULL, settings, "VkBool32", "deviceFault", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.deviceFaultVendorBinary, NULL, settings, "VkBool32", "deviceFaultVendorBinary", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceFaultCountsEXT(const VkDeviceFaultCountsEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.addressInfoCount, NULL, settings, "uint32_t", "addressInfoCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.vendorInfoCount, NULL, settings, "uint32_t", "vendorInfoCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.vendorBinarySize, NULL, settings, "VkDeviceSize", "vendorBinarySize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceFaultAddressInfoEXT(const VkDeviceFaultAddressInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceFaultAddressTypeEXT>(object.addressType, NULL, settings, "VkDeviceFaultAddressTypeEXT", "addressType", false, false, indents + 1, dump_json_VkDeviceFaultAddressTypeEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddress>(object.reportedAddress, NULL, settings, "VkDeviceAddress", "reportedAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.addressPrecision, NULL, settings, "VkDeviceSize", "addressPrecision", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceFaultVendorInfoEXT(const VkDeviceFaultVendorInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const char*>(object.description, NULL, settings, "char[VK_MAX_DESCRIPTION_SIZE]", "description", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.vendorFaultCode, NULL, settings, "uint64_t", "vendorFaultCode", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.vendorFaultData, NULL, settings, "uint64_t", "vendorFaultData", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceFaultInfoEXT(const VkDeviceFaultInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.description, NULL, settings, "char[VK_MAX_DESCRIPTION_SIZE]", "description", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_pointer<const VkDeviceFaultAddressInfoEXT>(object.pAddressInfos, settings, "VkDeviceFaultAddressInfoEXT*", "pAddressInfos", true, false, indents + 1, dump_json_VkDeviceFaultAddressInfoEXT);
    settings.stream() << ",\n";
    dump_json_pointer<const VkDeviceFaultVendorInfoEXT>(object.pVendorInfos, settings, "VkDeviceFaultVendorInfoEXT*", "pVendorInfos", true, false, indents + 1, dump_json_VkDeviceFaultVendorInfoEXT);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pVendorBinaryData, NULL, settings, "void*", "pVendorBinaryData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceFaultVendorBinaryHeaderVersionOneEXT(const VkDeviceFaultVendorBinaryHeaderVersionOneEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.headerSize, NULL, settings, "uint32_t", "headerSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceFaultVendorBinaryHeaderVersionEXT>(object.headerVersion, NULL, settings, "VkDeviceFaultVendorBinaryHeaderVersionEXT", "headerVersion", false, false, indents + 1, dump_json_VkDeviceFaultVendorBinaryHeaderVersionEXT);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.vendorID, NULL, settings, "uint32_t", "vendorID", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceID, NULL, settings, "uint32_t", "deviceID", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.driverVersion, NULL, settings, "uint32_t", "driverVersion", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.pipelineCacheUUID, 16, settings, "uint8_t[VK_UUID_SIZE]", "uint8_t", "pipelineCacheUUID", false, false, indents + 1, dump_json_uint8_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.applicationNameOffset, NULL, settings, "uint32_t", "applicationNameOffset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.applicationVersion, NULL, settings, "uint32_t", "applicationVersion", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.engineNameOffset, NULL, settings, "uint32_t", "engineNameOffset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.engineVersion, NULL, settings, "uint32_t", "engineVersion", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.apiVersion, NULL, settings, "uint32_t", "apiVersion", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRasterizationOrderAttachmentAccessFeaturesEXT(const VkPhysicalDeviceRasterizationOrderAttachmentAccessFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rasterizationOrderColorAttachmentAccess, NULL, settings, "VkBool32", "rasterizationOrderColorAttachmentAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rasterizationOrderDepthAttachmentAccess, NULL, settings, "VkBool32", "rasterizationOrderDepthAttachmentAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rasterizationOrderStencilAttachmentAccess, NULL, settings, "VkBool32", "rasterizationOrderStencilAttachmentAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRGBA10X6FormatsFeaturesEXT(const VkPhysicalDeviceRGBA10X6FormatsFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.formatRgba10x6WithoutYCbCrSampler, NULL, settings, "VkBool32", "formatRgba10x6WithoutYCbCrSampler", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_DIRECTFB_EXT)
void dump_json_VkDirectFBSurfaceCreateInfoEXT(const VkDirectFBSurfaceCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDirectFBSurfaceCreateFlagsEXT>(object.flags, NULL, settings, "VkDirectFBSurfaceCreateFlagsEXT", "flags", false, false, indents + 1, dump_json_VkDirectFBSurfaceCreateFlagsEXT);
    settings.stream() << ",\n";
    dump_json_pointer<const IDirectFB>(object.dfb, settings, "IDirectFB*", "dfb", false, false, indents + 1, dump_json_IDirectFB);
    settings.stream() << ",\n";
    dump_json_pointer<const IDirectFBSurface>(object.surface, settings, "IDirectFBSurface*", "surface", false, false, indents + 1, dump_json_IDirectFBSurface);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_DIRECTFB_EXT
void dump_json_VkRayTracingShaderGroupCreateInfoKHR(const VkRayTracingShaderGroupCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkRayTracingShaderGroupTypeKHR>(object.type, NULL, settings, "VkRayTracingShaderGroupTypeKHR", "type", false, false, indents + 1, dump_json_VkRayTracingShaderGroupTypeKHR);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.generalShader, NULL, settings, "uint32_t", "generalShader", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.closestHitShader, NULL, settings, "uint32_t", "closestHitShader", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.anyHitShader, NULL, settings, "uint32_t", "anyHitShader", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.intersectionShader, NULL, settings, "uint32_t", "intersectionShader", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pShaderGroupCaptureReplayHandle, NULL, settings, "const void*", "pShaderGroupCaptureReplayHandle", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRayTracingPipelineInterfaceCreateInfoKHR(const VkRayTracingPipelineInterfaceCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPipelineRayPayloadSize, NULL, settings, "uint32_t", "maxPipelineRayPayloadSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPipelineRayHitAttributeSize, NULL, settings, "uint32_t", "maxPipelineRayHitAttributeSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRayTracingPipelineCreateInfoKHR(const VkRayTracingPipelineCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineCreateFlags>(object.flags, NULL, settings, "VkPipelineCreateFlags", "flags", false, false, indents + 1, dump_json_VkPipelineCreateFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stageCount, NULL, settings, "uint32_t", "stageCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPipelineShaderStageCreateInfo>(object.pStages, object.stageCount, settings, "const VkPipelineShaderStageCreateInfo*", "const VkPipelineShaderStageCreateInfo", "pStages", true, false, indents + 1, dump_json_VkPipelineShaderStageCreateInfo); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.groupCount, NULL, settings, "uint32_t", "groupCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkRayTracingShaderGroupCreateInfoKHR>(object.pGroups, object.groupCount, settings, "const VkRayTracingShaderGroupCreateInfoKHR*", "const VkRayTracingShaderGroupCreateInfoKHR", "pGroups", true, false, indents + 1, dump_json_VkRayTracingShaderGroupCreateInfoKHR); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxPipelineRayRecursionDepth, NULL, settings, "uint32_t", "maxPipelineRayRecursionDepth", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_pointer<const VkPipelineLibraryCreateInfoKHR>(object.pLibraryInfo, settings, "const VkPipelineLibraryCreateInfoKHR*", "pLibraryInfo", true, false, indents + 1, dump_json_VkPipelineLibraryCreateInfoKHR);
    settings.stream() << ",\n";
    dump_json_pointer<const VkRayTracingPipelineInterfaceCreateInfoKHR>(object.pLibraryInterface, settings, "const VkRayTracingPipelineInterfaceCreateInfoKHR*", "pLibraryInterface", true, false, indents + 1, dump_json_VkRayTracingPipelineInterfaceCreateInfoKHR);
    settings.stream() << ",\n";
    dump_json_pointer<const VkPipelineDynamicStateCreateInfo>(object.pDynamicState, settings, "const VkPipelineDynamicStateCreateInfo*", "pDynamicState", true, false, indents + 1, dump_json_VkPipelineDynamicStateCreateInfo);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineLayout>(object.layout, NULL, settings, "VkPipelineLayout", "layout", false, false, indents + 1, dump_json_VkPipelineLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkPipeline>(object.basePipelineHandle, NULL, settings, "VkPipeline", "basePipelineHandle", false, false, indents + 1, dump_json_VkPipeline);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.basePipelineIndex, NULL, settings, "int32_t", "basePipelineIndex", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRayTracingPipelineFeaturesKHR(const VkPhysicalDeviceRayTracingPipelineFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rayTracingPipeline, NULL, settings, "VkBool32", "rayTracingPipeline", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rayTracingPipelineShaderGroupHandleCaptureReplay, NULL, settings, "VkBool32", "rayTracingPipelineShaderGroupHandleCaptureReplay", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rayTracingPipelineShaderGroupHandleCaptureReplayMixed, NULL, settings, "VkBool32", "rayTracingPipelineShaderGroupHandleCaptureReplayMixed", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rayTracingPipelineTraceRaysIndirect, NULL, settings, "VkBool32", "rayTracingPipelineTraceRaysIndirect", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rayTraversalPrimitiveCulling, NULL, settings, "VkBool32", "rayTraversalPrimitiveCulling", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRayTracingPipelinePropertiesKHR(const VkPhysicalDeviceRayTracingPipelinePropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shaderGroupHandleSize, NULL, settings, "uint32_t", "shaderGroupHandleSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxRayRecursionDepth, NULL, settings, "uint32_t", "maxRayRecursionDepth", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxShaderGroupStride, NULL, settings, "uint32_t", "maxShaderGroupStride", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shaderGroupBaseAlignment, NULL, settings, "uint32_t", "shaderGroupBaseAlignment", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shaderGroupHandleCaptureReplaySize, NULL, settings, "uint32_t", "shaderGroupHandleCaptureReplaySize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxRayDispatchInvocationCount, NULL, settings, "uint32_t", "maxRayDispatchInvocationCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shaderGroupHandleAlignment, NULL, settings, "uint32_t", "shaderGroupHandleAlignment", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxRayHitAttributeSize, NULL, settings, "uint32_t", "maxRayHitAttributeSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkStridedDeviceAddressRegionKHR(const VkStridedDeviceAddressRegionKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceAddress>(object.deviceAddress, NULL, settings, "VkDeviceAddress", "deviceAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.stride, NULL, settings, "VkDeviceSize", "stride", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkTraceRaysIndirectCommandKHR(const VkTraceRaysIndirectCommandKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.width, NULL, settings, "uint32_t", "width", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.height, NULL, settings, "uint32_t", "height", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.depth, NULL, settings, "uint32_t", "depth", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRayQueryFeaturesKHR(const VkPhysicalDeviceRayQueryFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rayQuery, NULL, settings, "VkBool32", "rayQuery", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMutableDescriptorTypeFeaturesEXT(const VkPhysicalDeviceMutableDescriptorTypeFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.mutableDescriptorType, NULL, settings, "VkBool32", "mutableDescriptorType", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMutableDescriptorTypeListEXT(const VkMutableDescriptorTypeListEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.descriptorTypeCount, NULL, settings, "uint32_t", "descriptorTypeCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDescriptorType>(object.pDescriptorTypes, object.descriptorTypeCount, settings, "const VkDescriptorType*", "const VkDescriptorType", "pDescriptorTypes", false, false, indents + 1, dump_json_VkDescriptorType); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMutableDescriptorTypeCreateInfoEXT(const VkMutableDescriptorTypeCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.mutableDescriptorTypeListCount, NULL, settings, "uint32_t", "mutableDescriptorTypeListCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkMutableDescriptorTypeListEXT>(object.pMutableDescriptorTypeLists, object.mutableDescriptorTypeListCount, settings, "const VkMutableDescriptorTypeListEXT*", "const VkMutableDescriptorTypeListEXT", "pMutableDescriptorTypeLists", true, false, indents + 1, dump_json_VkMutableDescriptorTypeListEXT); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceVertexInputDynamicStateFeaturesEXT(const VkPhysicalDeviceVertexInputDynamicStateFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.vertexInputDynamicState, NULL, settings, "VkBool32", "vertexInputDynamicState", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVertexInputBindingDescription2EXT(const VkVertexInputBindingDescription2EXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.binding, NULL, settings, "uint32_t", "binding", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stride, NULL, settings, "uint32_t", "stride", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkVertexInputRate>(object.inputRate, NULL, settings, "VkVertexInputRate", "inputRate", false, false, indents + 1, dump_json_VkVertexInputRate);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.divisor, NULL, settings, "uint32_t", "divisor", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVertexInputAttributeDescription2EXT(const VkVertexInputAttributeDescription2EXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.location, NULL, settings, "uint32_t", "location", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.binding, NULL, settings, "uint32_t", "binding", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.offset, NULL, settings, "uint32_t", "offset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDrmPropertiesEXT(const VkPhysicalDeviceDrmPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.hasPrimary, NULL, settings, "VkBool32", "hasPrimary", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.hasRender, NULL, settings, "VkBool32", "hasRender", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const int64_t>(object.primaryMajor, NULL, settings, "int64_t", "primaryMajor", false, false, indents + 1, dump_json_int64_t);
    settings.stream() << ",\n";
    dump_json_value<const int64_t>(object.primaryMinor, NULL, settings, "int64_t", "primaryMinor", false, false, indents + 1, dump_json_int64_t);
    settings.stream() << ",\n";
    dump_json_value<const int64_t>(object.renderMajor, NULL, settings, "int64_t", "renderMajor", false, false, indents + 1, dump_json_int64_t);
    settings.stream() << ",\n";
    dump_json_value<const int64_t>(object.renderMinor, NULL, settings, "int64_t", "renderMinor", false, false, indents + 1, dump_json_int64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceAddressBindingReportFeaturesEXT(const VkPhysicalDeviceAddressBindingReportFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.reportAddressBinding, NULL, settings, "VkBool32", "reportAddressBinding", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceAddressBindingCallbackDataEXT(const VkDeviceAddressBindingCallbackDataEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddressBindingFlagsEXT>(object.flags, NULL, settings, "VkDeviceAddressBindingFlagsEXT", "flags", false, false, indents + 1, dump_json_VkDeviceAddressBindingFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddress>(object.baseAddress, NULL, settings, "VkDeviceAddress", "baseAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddressBindingTypeEXT>(object.bindingType, NULL, settings, "VkDeviceAddressBindingTypeEXT", "bindingType", false, false, indents + 1, dump_json_VkDeviceAddressBindingTypeEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDepthClipControlFeaturesEXT(const VkPhysicalDeviceDepthClipControlFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthClipControl, NULL, settings, "VkBool32", "depthClipControl", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineViewportDepthClipControlCreateInfoEXT(const VkPipelineViewportDepthClipControlCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.negativeOneToOne, NULL, settings, "VkBool32", "negativeOneToOne", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePrimitiveTopologyListRestartFeaturesEXT(const VkPhysicalDevicePrimitiveTopologyListRestartFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.primitiveTopologyListRestart, NULL, settings, "VkBool32", "primitiveTopologyListRestart", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.primitiveTopologyPatchListRestart, NULL, settings, "VkBool32", "primitiveTopologyPatchListRestart", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkImportMemoryZirconHandleInfoFUCHSIA(const VkImportMemoryZirconHandleInfoFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const zx_handle_t>(object.handle, NULL, settings, "zx_handle_t", "handle", false, false, indents + 1, dump_json_zx_handle_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkMemoryZirconHandlePropertiesFUCHSIA(const VkMemoryZirconHandlePropertiesFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryTypeBits, NULL, settings, "uint32_t", "memoryTypeBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkMemoryGetZirconHandleInfoFUCHSIA(const VkMemoryGetZirconHandleInfoFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkImportSemaphoreZirconHandleInfoFUCHSIA(const VkImportSemaphoreZirconHandleInfoFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphore>(object.semaphore, NULL, settings, "VkSemaphore", "semaphore", false, false, indents + 1, dump_json_VkSemaphore);
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphoreImportFlags>(object.flags, NULL, settings, "VkSemaphoreImportFlags", "flags", false, false, indents + 1, dump_json_VkSemaphoreImportFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalSemaphoreHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalSemaphoreHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalSemaphoreHandleTypeFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const zx_handle_t>(object.zirconHandle, NULL, settings, "zx_handle_t", "zirconHandle", false, false, indents + 1, dump_json_zx_handle_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkSemaphoreGetZirconHandleInfoFUCHSIA(const VkSemaphoreGetZirconHandleInfoFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphore>(object.semaphore, NULL, settings, "VkSemaphore", "semaphore", false, false, indents + 1, dump_json_VkSemaphore);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalSemaphoreHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalSemaphoreHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalSemaphoreHandleTypeFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkBufferCollectionCreateInfoFUCHSIA(const VkBufferCollectionCreateInfoFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const zx_handle_t>(object.collectionToken, NULL, settings, "zx_handle_t", "collectionToken", false, false, indents + 1, dump_json_zx_handle_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkImportMemoryBufferCollectionFUCHSIA(const VkImportMemoryBufferCollectionFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBufferCollectionFUCHSIA>(object.collection, NULL, settings, "VkBufferCollectionFUCHSIA", "collection", false, false, indents + 1, dump_json_VkBufferCollectionFUCHSIA);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.index, NULL, settings, "uint32_t", "index", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkBufferCollectionImageCreateInfoFUCHSIA(const VkBufferCollectionImageCreateInfoFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBufferCollectionFUCHSIA>(object.collection, NULL, settings, "VkBufferCollectionFUCHSIA", "collection", false, false, indents + 1, dump_json_VkBufferCollectionFUCHSIA);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.index, NULL, settings, "uint32_t", "index", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkBufferCollectionConstraintsInfoFUCHSIA(const VkBufferCollectionConstraintsInfoFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minBufferCount, NULL, settings, "uint32_t", "minBufferCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxBufferCount, NULL, settings, "uint32_t", "maxBufferCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minBufferCountForCamping, NULL, settings, "uint32_t", "minBufferCountForCamping", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minBufferCountForDedicatedSlack, NULL, settings, "uint32_t", "minBufferCountForDedicatedSlack", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minBufferCountForSharedSlack, NULL, settings, "uint32_t", "minBufferCountForSharedSlack", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkBufferConstraintsInfoFUCHSIA(const VkBufferConstraintsInfoFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBufferCreateInfo>(object.createInfo, NULL, settings, "VkBufferCreateInfo", "createInfo", true, false, indents + 1, dump_json_VkBufferCreateInfo);
    settings.stream() << ",\n";
    dump_json_value<const VkFormatFeatureFlags>(object.requiredFormatFeatures, NULL, settings, "VkFormatFeatureFlags", "requiredFormatFeatures", false, false, indents + 1, dump_json_VkFormatFeatureFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkBufferCollectionConstraintsInfoFUCHSIA>(object.bufferCollectionConstraints, NULL, settings, "VkBufferCollectionConstraintsInfoFUCHSIA", "bufferCollectionConstraints", true, false, indents + 1, dump_json_VkBufferCollectionConstraintsInfoFUCHSIA);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkBufferCollectionBufferCreateInfoFUCHSIA(const VkBufferCollectionBufferCreateInfoFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBufferCollectionFUCHSIA>(object.collection, NULL, settings, "VkBufferCollectionFUCHSIA", "collection", false, false, indents + 1, dump_json_VkBufferCollectionFUCHSIA);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.index, NULL, settings, "uint32_t", "index", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkSysmemColorSpaceFUCHSIA(const VkSysmemColorSpaceFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.colorSpace, NULL, settings, "uint32_t", "colorSpace", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkBufferCollectionPropertiesFUCHSIA(const VkBufferCollectionPropertiesFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryTypeBits, NULL, settings, "uint32_t", "memoryTypeBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.bufferCount, NULL, settings, "uint32_t", "bufferCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.createInfoIndex, NULL, settings, "uint32_t", "createInfoIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.sysmemPixelFormat, NULL, settings, "uint64_t", "sysmemPixelFormat", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const VkFormatFeatureFlags>(object.formatFeatures, NULL, settings, "VkFormatFeatureFlags", "formatFeatures", false, false, indents + 1, dump_json_VkFormatFeatureFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkSysmemColorSpaceFUCHSIA>(object.sysmemColorSpaceIndex, NULL, settings, "VkSysmemColorSpaceFUCHSIA", "sysmemColorSpaceIndex", true, false, indents + 1, dump_json_VkSysmemColorSpaceFUCHSIA);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentMapping>(object.samplerYcbcrConversionComponents, NULL, settings, "VkComponentMapping", "samplerYcbcrConversionComponents", true, false, indents + 1, dump_json_VkComponentMapping);
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerYcbcrModelConversion>(object.suggestedYcbcrModel, NULL, settings, "VkSamplerYcbcrModelConversion", "suggestedYcbcrModel", false, false, indents + 1, dump_json_VkSamplerYcbcrModelConversion);
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerYcbcrRange>(object.suggestedYcbcrRange, NULL, settings, "VkSamplerYcbcrRange", "suggestedYcbcrRange", false, false, indents + 1, dump_json_VkSamplerYcbcrRange);
    settings.stream() << ",\n";
    dump_json_value<const VkChromaLocation>(object.suggestedXChromaOffset, NULL, settings, "VkChromaLocation", "suggestedXChromaOffset", false, false, indents + 1, dump_json_VkChromaLocation);
    settings.stream() << ",\n";
    dump_json_value<const VkChromaLocation>(object.suggestedYChromaOffset, NULL, settings, "VkChromaLocation", "suggestedYChromaOffset", false, false, indents + 1, dump_json_VkChromaLocation);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkImageFormatConstraintsInfoFUCHSIA(const VkImageFormatConstraintsInfoFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkImageCreateInfo>(object.imageCreateInfo, NULL, settings, "VkImageCreateInfo", "imageCreateInfo", true, false, indents + 1, dump_json_VkImageCreateInfo);
    settings.stream() << ",\n";
    dump_json_value<const VkFormatFeatureFlags>(object.requiredFormatFeatures, NULL, settings, "VkFormatFeatureFlags", "requiredFormatFeatures", false, false, indents + 1, dump_json_VkFormatFeatureFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkImageFormatConstraintsFlagsFUCHSIA>(object.flags, NULL, settings, "VkImageFormatConstraintsFlagsFUCHSIA", "flags", false, false, indents + 1, dump_json_VkImageFormatConstraintsFlagsFUCHSIA);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.sysmemPixelFormat, NULL, settings, "uint64_t", "sysmemPixelFormat", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.colorSpaceCount, NULL, settings, "uint32_t", "colorSpaceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSysmemColorSpaceFUCHSIA>(object.pColorSpaces, object.colorSpaceCount, settings, "const VkSysmemColorSpaceFUCHSIA*", "const VkSysmemColorSpaceFUCHSIA", "pColorSpaces", true, false, indents + 1, dump_json_VkSysmemColorSpaceFUCHSIA); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_VkImageConstraintsInfoFUCHSIA(const VkImageConstraintsInfoFUCHSIA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.formatConstraintsCount, NULL, settings, "uint32_t", "formatConstraintsCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkImageFormatConstraintsInfoFUCHSIA>(object.pFormatConstraints, object.formatConstraintsCount, settings, "const VkImageFormatConstraintsInfoFUCHSIA*", "const VkImageFormatConstraintsInfoFUCHSIA", "pFormatConstraints", true, false, indents + 1, dump_json_VkImageFormatConstraintsInfoFUCHSIA); // JQA
    settings.stream() << ",\n";
    dump_json_value<const VkBufferCollectionConstraintsInfoFUCHSIA>(object.bufferCollectionConstraints, NULL, settings, "VkBufferCollectionConstraintsInfoFUCHSIA", "bufferCollectionConstraints", true, false, indents + 1, dump_json_VkBufferCollectionConstraintsInfoFUCHSIA);
    settings.stream() << ",\n";
    dump_json_value<const VkImageConstraintsInfoFlagsFUCHSIA>(object.flags, NULL, settings, "VkImageConstraintsInfoFlagsFUCHSIA", "flags", false, false, indents + 1, dump_json_VkImageConstraintsInfoFlagsFUCHSIA);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_FUCHSIA
void dump_json_VkSubpassShadingPipelineCreateInfoHUAWEI(const VkSubpassShadingPipelineCreateInfoHUAWEI& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkRenderPass>(object.renderPass, NULL, settings, "VkRenderPass", "renderPass", false, false, indents + 1, dump_json_VkRenderPass);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.subpass, NULL, settings, "uint32_t", "subpass", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSubpassShadingFeaturesHUAWEI(const VkPhysicalDeviceSubpassShadingFeaturesHUAWEI& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.subpassShading, NULL, settings, "VkBool32", "subpassShading", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSubpassShadingPropertiesHUAWEI(const VkPhysicalDeviceSubpassShadingPropertiesHUAWEI& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxSubpassShadingWorkgroupSizeAspectRatio, NULL, settings, "uint32_t", "maxSubpassShadingWorkgroupSizeAspectRatio", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceInvocationMaskFeaturesHUAWEI(const VkPhysicalDeviceInvocationMaskFeaturesHUAWEI& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.invocationMask, NULL, settings, "VkBool32", "invocationMask", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMemoryGetRemoteAddressInfoNV(const VkMemoryGetRemoteAddressInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceMemory>(object.memory, NULL, settings, "VkDeviceMemory", "memory", false, false, indents + 1, dump_json_VkDeviceMemory);
    settings.stream() << ",\n";
    dump_json_value<const VkExternalMemoryHandleTypeFlagBits>(object.handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagBits", "handleType", false, false, indents + 1, dump_json_VkExternalMemoryHandleTypeFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceExternalMemoryRDMAFeaturesNV(const VkPhysicalDeviceExternalMemoryRDMAFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.externalMemoryRDMA, NULL, settings, "VkBool32", "externalMemoryRDMA", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelinePropertiesIdentifierEXT(const VkPipelinePropertiesIdentifierEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.pipelineIdentifier, 16, settings, "uint8_t[VK_UUID_SIZE]", "uint8_t", "pipelineIdentifier", false, false, indents + 1, dump_json_uint8_t); // IQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePipelinePropertiesFeaturesEXT(const VkPhysicalDevicePipelinePropertiesFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.pipelinePropertiesIdentifier, NULL, settings, "VkBool32", "pipelinePropertiesIdentifier", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFrameBoundaryFeaturesEXT(const VkPhysicalDeviceFrameBoundaryFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.frameBoundary, NULL, settings, "VkBool32", "frameBoundary", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkFrameBoundaryEXT(const VkFrameBoundaryEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFrameBoundaryFlagsEXT>(object.flags, NULL, settings, "VkFrameBoundaryFlagsEXT", "flags", false, false, indents + 1, dump_json_VkFrameBoundaryFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.frameID, NULL, settings, "uint64_t", "frameID", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.imageCount, NULL, settings, "uint32_t", "imageCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkImage>(object.pImages, object.imageCount, settings, "const VkImage*", "const VkImage", "pImages", false, false, indents + 1, dump_json_VkImage); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.bufferCount, NULL, settings, "uint32_t", "bufferCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkBuffer>(object.pBuffers, object.bufferCount, settings, "const VkBuffer*", "const VkBuffer", "pBuffers", false, false, indents + 1, dump_json_VkBuffer); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.tagName, NULL, settings, "uint64_t", "tagName", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.tagSize, NULL, settings, "size_t", "tagSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pTag, NULL, settings, "const void*", "pTag", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMultisampledRenderToSingleSampledFeaturesEXT(const VkPhysicalDeviceMultisampledRenderToSingleSampledFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multisampledRenderToSingleSampled, NULL, settings, "VkBool32", "multisampledRenderToSingleSampled", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSubpassResolvePerformanceQueryEXT(const VkSubpassResolvePerformanceQueryEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.optimal, NULL, settings, "VkBool32", "optimal", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMultisampledRenderToSingleSampledInfoEXT(const VkMultisampledRenderToSingleSampledInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multisampledRenderToSingleSampledEnable, NULL, settings, "VkBool32", "multisampledRenderToSingleSampledEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkSampleCountFlagBits>(object.rasterizationSamples, NULL, settings, "VkSampleCountFlagBits", "rasterizationSamples", false, false, indents + 1, dump_json_VkSampleCountFlagBits);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceExtendedDynamicState2FeaturesEXT(const VkPhysicalDeviceExtendedDynamicState2FeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState2, NULL, settings, "VkBool32", "extendedDynamicState2", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState2LogicOp, NULL, settings, "VkBool32", "extendedDynamicState2LogicOp", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState2PatchControlPoints, NULL, settings, "VkBool32", "extendedDynamicState2PatchControlPoints", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
void dump_json_VkScreenSurfaceCreateInfoQNX(const VkScreenSurfaceCreateInfoQNX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkScreenSurfaceCreateFlagsQNX>(object.flags, NULL, settings, "VkScreenSurfaceCreateFlagsQNX", "flags", false, false, indents + 1, dump_json_VkScreenSurfaceCreateFlagsQNX);
    settings.stream() << ",\n";
    dump_json_value<const _screen_context*>(object.context, NULL, settings, "struct _screen_context*", "context", false, false, indents + 1, dump_json__screen_context);
    settings.stream() << ",\n";
    dump_json_value<const _screen_window*>(object.window, NULL, settings, "struct _screen_window*", "window", false, false, indents + 1, dump_json__screen_window);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_SCREEN_QNX
void dump_json_VkPhysicalDeviceColorWriteEnableFeaturesEXT(const VkPhysicalDeviceColorWriteEnableFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.colorWriteEnable, NULL, settings, "VkBool32", "colorWriteEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineColorWriteCreateInfoEXT(const VkPipelineColorWriteCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.attachmentCount, NULL, settings, "uint32_t", "attachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkBool32>(object.pColorWriteEnables, object.attachmentCount, settings, "const VkBool32*", "const VkBool32", "pColorWriteEnables", false, false, indents + 1, dump_json_VkBool32); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePrimitivesGeneratedQueryFeaturesEXT(const VkPhysicalDevicePrimitivesGeneratedQueryFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.primitivesGeneratedQuery, NULL, settings, "VkBool32", "primitivesGeneratedQuery", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.primitivesGeneratedQueryWithRasterizerDiscard, NULL, settings, "VkBool32", "primitivesGeneratedQueryWithRasterizerDiscard", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.primitivesGeneratedQueryWithNonZeroStreams, NULL, settings, "VkBool32", "primitivesGeneratedQueryWithNonZeroStreams", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRayTracingMaintenance1FeaturesKHR(const VkPhysicalDeviceRayTracingMaintenance1FeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rayTracingMaintenance1, NULL, settings, "VkBool32", "rayTracingMaintenance1", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rayTracingPipelineTraceRaysIndirect2, NULL, settings, "VkBool32", "rayTracingPipelineTraceRaysIndirect2", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkTraceRaysIndirectCommand2KHR(const VkTraceRaysIndirectCommand2KHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceAddress>(object.raygenShaderRecordAddress, NULL, settings, "VkDeviceAddress", "raygenShaderRecordAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.raygenShaderRecordSize, NULL, settings, "VkDeviceSize", "raygenShaderRecordSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddress>(object.missShaderBindingTableAddress, NULL, settings, "VkDeviceAddress", "missShaderBindingTableAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.missShaderBindingTableSize, NULL, settings, "VkDeviceSize", "missShaderBindingTableSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.missShaderBindingTableStride, NULL, settings, "VkDeviceSize", "missShaderBindingTableStride", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddress>(object.hitShaderBindingTableAddress, NULL, settings, "VkDeviceAddress", "hitShaderBindingTableAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.hitShaderBindingTableSize, NULL, settings, "VkDeviceSize", "hitShaderBindingTableSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.hitShaderBindingTableStride, NULL, settings, "VkDeviceSize", "hitShaderBindingTableStride", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddress>(object.callableShaderBindingTableAddress, NULL, settings, "VkDeviceAddress", "callableShaderBindingTableAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.callableShaderBindingTableSize, NULL, settings, "VkDeviceSize", "callableShaderBindingTableSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.callableShaderBindingTableStride, NULL, settings, "VkDeviceSize", "callableShaderBindingTableStride", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.width, NULL, settings, "uint32_t", "width", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.height, NULL, settings, "uint32_t", "height", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.depth, NULL, settings, "uint32_t", "depth", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImageViewMinLodFeaturesEXT(const VkPhysicalDeviceImageViewMinLodFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.minLod, NULL, settings, "VkBool32", "minLod", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageViewMinLodCreateInfoEXT(const VkImageViewMinLodCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const float>(object.minLod, NULL, settings, "float", "minLod", false, false, indents + 1, dump_json_float);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMultiDrawFeaturesEXT(const VkPhysicalDeviceMultiDrawFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multiDraw, NULL, settings, "VkBool32", "multiDraw", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMultiDrawPropertiesEXT(const VkPhysicalDeviceMultiDrawPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxMultiDrawCount, NULL, settings, "uint32_t", "maxMultiDrawCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMultiDrawInfoEXT(const VkMultiDrawInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.firstVertex, NULL, settings, "uint32_t", "firstVertex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.vertexCount, NULL, settings, "uint32_t", "vertexCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMultiDrawIndexedInfoEXT(const VkMultiDrawIndexedInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.firstIndex, NULL, settings, "uint32_t", "firstIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.indexCount, NULL, settings, "uint32_t", "indexCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const int32_t>(object.vertexOffset, NULL, settings, "int32_t", "vertexOffset", false, false, indents + 1, dump_json_int32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImage2DViewOf3DFeaturesEXT(const VkPhysicalDeviceImage2DViewOf3DFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.image2DViewOf3D, NULL, settings, "VkBool32", "image2DViewOf3D", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.sampler2DViewOf3D, NULL, settings, "VkBool32", "sampler2DViewOf3D", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderTileImageFeaturesEXT(const VkPhysicalDeviceShaderTileImageFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderTileImageColorReadAccess, NULL, settings, "VkBool32", "shaderTileImageColorReadAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderTileImageDepthReadAccess, NULL, settings, "VkBool32", "shaderTileImageDepthReadAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderTileImageStencilReadAccess, NULL, settings, "VkBool32", "shaderTileImageStencilReadAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderTileImagePropertiesEXT(const VkPhysicalDeviceShaderTileImagePropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderTileImageCoherentReadAccelerated, NULL, settings, "VkBool32", "shaderTileImageCoherentReadAccelerated", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderTileImageReadSampleFromPixelRateInvocation, NULL, settings, "VkBool32", "shaderTileImageReadSampleFromPixelRateInvocation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderTileImageReadFromHelperInvocation, NULL, settings, "VkBool32", "shaderTileImageReadFromHelperInvocation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMicromapUsageEXT(const VkMicromapUsageEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.count, NULL, settings, "uint32_t", "count", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.subdivisionLevel, NULL, settings, "uint32_t", "subdivisionLevel", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.format, NULL, settings, "uint32_t", "format", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMicromapBuildInfoEXT(const VkMicromapBuildInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkMicromapTypeEXT>(object.type, NULL, settings, "VkMicromapTypeEXT", "type", false, false, indents + 1, dump_json_VkMicromapTypeEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkBuildMicromapFlagsEXT>(object.flags, NULL, settings, "VkBuildMicromapFlagsEXT", "flags", false, false, indents + 1, dump_json_VkBuildMicromapFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkBuildMicromapModeEXT>(object.mode, NULL, settings, "VkBuildMicromapModeEXT", "mode", false, false, indents + 1, dump_json_VkBuildMicromapModeEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkMicromapEXT>(object.dstMicromap, NULL, settings, "VkMicromapEXT", "dstMicromap", false, false, indents + 1, dump_json_VkMicromapEXT);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.usageCountsCount, NULL, settings, "uint32_t", "usageCountsCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkMicromapUsageEXT>(object.pUsageCounts, object.usageCountsCount, settings, "const VkMicromapUsageEXT*", "const VkMicromapUsageEXT", "pUsageCounts", true, false, indents + 1, dump_json_VkMicromapUsageEXT); // JQA
    settings.stream() << ",\n";
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstKHR>(object.data, NULL, settings, "VkDeviceOrHostAddressConstKHR", "data", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressKHR>(object.scratchData, NULL, settings, "VkDeviceOrHostAddressKHR", "scratchData", false, true, indents + 1, dump_json_VkDeviceOrHostAddressKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstKHR>(object.triangleArray, NULL, settings, "VkDeviceOrHostAddressConstKHR", "triangleArray", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.triangleArrayStride, NULL, settings, "VkDeviceSize", "triangleArrayStride", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMicromapCreateInfoEXT(const VkMicromapCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkMicromapCreateFlagsEXT>(object.createFlags, NULL, settings, "VkMicromapCreateFlagsEXT", "createFlags", false, false, indents + 1, dump_json_VkMicromapCreateFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkBuffer>(object.buffer, NULL, settings, "VkBuffer", "buffer", false, false, indents + 1, dump_json_VkBuffer);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.offset, NULL, settings, "VkDeviceSize", "offset", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkMicromapTypeEXT>(object.type, NULL, settings, "VkMicromapTypeEXT", "type", false, false, indents + 1, dump_json_VkMicromapTypeEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddress>(object.deviceAddress, NULL, settings, "VkDeviceAddress", "deviceAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceOpacityMicromapFeaturesEXT(const VkPhysicalDeviceOpacityMicromapFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.micromap, NULL, settings, "VkBool32", "micromap", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.micromapCaptureReplay, NULL, settings, "VkBool32", "micromapCaptureReplay", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.micromapHostCommands, NULL, settings, "VkBool32", "micromapHostCommands", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceOpacityMicromapPropertiesEXT(const VkPhysicalDeviceOpacityMicromapPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxOpacity2StateSubdivisionLevel, NULL, settings, "uint32_t", "maxOpacity2StateSubdivisionLevel", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxOpacity4StateSubdivisionLevel, NULL, settings, "uint32_t", "maxOpacity4StateSubdivisionLevel", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMicromapVersionInfoEXT(const VkMicromapVersionInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.pVersionData, 2*VK_UUID_SIZE, settings, "const uint8_t*", "const uint8_t", "pVersionData", false, false, indents + 1, dump_json_uint8_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyMicromapToMemoryInfoEXT(const VkCopyMicromapToMemoryInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkMicromapEXT>(object.src, NULL, settings, "VkMicromapEXT", "src", false, false, indents + 1, dump_json_VkMicromapEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressKHR>(object.dst, NULL, settings, "VkDeviceOrHostAddressKHR", "dst", false, true, indents + 1, dump_json_VkDeviceOrHostAddressKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkCopyMicromapModeEXT>(object.mode, NULL, settings, "VkCopyMicromapModeEXT", "mode", false, false, indents + 1, dump_json_VkCopyMicromapModeEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyMemoryToMicromapInfoEXT(const VkCopyMemoryToMicromapInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstKHR>(object.src, NULL, settings, "VkDeviceOrHostAddressConstKHR", "src", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkMicromapEXT>(object.dst, NULL, settings, "VkMicromapEXT", "dst", false, false, indents + 1, dump_json_VkMicromapEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkCopyMicromapModeEXT>(object.mode, NULL, settings, "VkCopyMicromapModeEXT", "mode", false, false, indents + 1, dump_json_VkCopyMicromapModeEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyMicromapInfoEXT(const VkCopyMicromapInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkMicromapEXT>(object.src, NULL, settings, "VkMicromapEXT", "src", false, false, indents + 1, dump_json_VkMicromapEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkMicromapEXT>(object.dst, NULL, settings, "VkMicromapEXT", "dst", false, false, indents + 1, dump_json_VkMicromapEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkCopyMicromapModeEXT>(object.mode, NULL, settings, "VkCopyMicromapModeEXT", "mode", false, false, indents + 1, dump_json_VkCopyMicromapModeEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMicromapBuildSizesInfoEXT(const VkMicromapBuildSizesInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.micromapSize, NULL, settings, "VkDeviceSize", "micromapSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.buildScratchSize, NULL, settings, "VkDeviceSize", "buildScratchSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.discardable, NULL, settings, "VkBool32", "discardable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureTrianglesOpacityMicromapEXT(const VkAccelerationStructureTrianglesOpacityMicromapEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkIndexType>(object.indexType, NULL, settings, "VkIndexType", "indexType", false, false, indents + 1, dump_json_VkIndexType);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstKHR>(object.indexBuffer, NULL, settings, "VkDeviceOrHostAddressConstKHR", "indexBuffer", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.indexStride, NULL, settings, "VkDeviceSize", "indexStride", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.baseTriangle, NULL, settings, "uint32_t", "baseTriangle", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.usageCountsCount, NULL, settings, "uint32_t", "usageCountsCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkMicromapUsageEXT>(object.pUsageCounts, object.usageCountsCount, settings, "const VkMicromapUsageEXT*", "const VkMicromapUsageEXT", "pUsageCounts", true, false, indents + 1, dump_json_VkMicromapUsageEXT); // JQA
    settings.stream() << ",\n";
    settings.stream() << ",\n";
    dump_json_value<const VkMicromapEXT>(object.micromap, NULL, settings, "VkMicromapEXT", "micromap", false, false, indents + 1, dump_json_VkMicromapEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMicromapTriangleEXT(const VkMicromapTriangleEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.dataOffset, NULL, settings, "uint32_t", "dataOffset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint16_t>(object.subdivisionLevel, NULL, settings, "uint16_t", "subdivisionLevel", false, false, indents + 1, dump_json_uint16_t);
    settings.stream() << ",\n";
    dump_json_value<const uint16_t>(object.format, NULL, settings, "uint16_t", "format", false, false, indents + 1, dump_json_uint16_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_VkPhysicalDeviceDisplacementMicromapFeaturesNV(const VkPhysicalDeviceDisplacementMicromapFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.displacementMicromap, NULL, settings, "VkBool32", "displacementMicromap", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_VkPhysicalDeviceDisplacementMicromapPropertiesNV(const VkPhysicalDeviceDisplacementMicromapPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDisplacementMicromapSubdivisionLevel, NULL, settings, "uint32_t", "maxDisplacementMicromapSubdivisionLevel", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_VkAccelerationStructureTrianglesDisplacementMicromapNV(const VkAccelerationStructureTrianglesDisplacementMicromapNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.displacementBiasAndScaleFormat, NULL, settings, "VkFormat", "displacementBiasAndScaleFormat", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.displacementVectorFormat, NULL, settings, "VkFormat", "displacementVectorFormat", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstKHR>(object.displacementBiasAndScaleBuffer, NULL, settings, "VkDeviceOrHostAddressConstKHR", "displacementBiasAndScaleBuffer", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.displacementBiasAndScaleStride, NULL, settings, "VkDeviceSize", "displacementBiasAndScaleStride", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstKHR>(object.displacementVectorBuffer, NULL, settings, "VkDeviceOrHostAddressConstKHR", "displacementVectorBuffer", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.displacementVectorStride, NULL, settings, "VkDeviceSize", "displacementVectorStride", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstKHR>(object.displacedMicromapPrimitiveFlags, NULL, settings, "VkDeviceOrHostAddressConstKHR", "displacedMicromapPrimitiveFlags", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.displacedMicromapPrimitiveFlagsStride, NULL, settings, "VkDeviceSize", "displacedMicromapPrimitiveFlagsStride", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkIndexType>(object.indexType, NULL, settings, "VkIndexType", "indexType", false, false, indents + 1, dump_json_VkIndexType);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceOrHostAddressConstKHR>(object.indexBuffer, NULL, settings, "VkDeviceOrHostAddressConstKHR", "indexBuffer", false, true, indents + 1, dump_json_VkDeviceOrHostAddressConstKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.indexStride, NULL, settings, "VkDeviceSize", "indexStride", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.baseTriangle, NULL, settings, "uint32_t", "baseTriangle", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.usageCountsCount, NULL, settings, "uint32_t", "usageCountsCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkMicromapUsageEXT>(object.pUsageCounts, object.usageCountsCount, settings, "const VkMicromapUsageEXT*", "const VkMicromapUsageEXT", "pUsageCounts", true, false, indents + 1, dump_json_VkMicromapUsageEXT); // JQA
    settings.stream() << ",\n";
    settings.stream() << ",\n";
    dump_json_value<const VkMicromapEXT>(object.micromap, NULL, settings, "VkMicromapEXT", "micromap", false, false, indents + 1, dump_json_VkMicromapEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_ENABLE_BETA_EXTENSIONS
void dump_json_VkPhysicalDeviceClusterCullingShaderFeaturesHUAWEI(const VkPhysicalDeviceClusterCullingShaderFeaturesHUAWEI& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*pNext", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.clustercullingShader, NULL, settings, "VkBool32", "clustercullingShader", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multiviewClusterCullingShader, NULL, settings, "VkBool32", "multiviewClusterCullingShader", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceClusterCullingShaderPropertiesHUAWEI(const VkPhysicalDeviceClusterCullingShaderPropertiesHUAWEI& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.maxWorkGroupCount, 3, settings, "uint32_t[3]", "uint32_t", "maxWorkGroupCount", false, false, indents + 1, dump_json_uint32_t); // IQA
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.maxWorkGroupSize, 3, settings, "uint32_t[3]", "uint32_t", "maxWorkGroupSize", false, false, indents + 1, dump_json_uint32_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxOutputClusterCount, NULL, settings, "uint32_t", "maxOutputClusterCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.indirectBufferOffsetAlignment, NULL, settings, "VkDeviceSize", "indirectBufferOffsetAlignment", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceClusterCullingShaderVrsFeaturesHUAWEI(const VkPhysicalDeviceClusterCullingShaderVrsFeaturesHUAWEI& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*pNext", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.clusterShadingRate, NULL, settings, "VkBool32", "clusterShadingRate", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceBorderColorSwizzleFeaturesEXT(const VkPhysicalDeviceBorderColorSwizzleFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.borderColorSwizzle, NULL, settings, "VkBool32", "borderColorSwizzle", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.borderColorSwizzleFromImage, NULL, settings, "VkBool32", "borderColorSwizzleFromImage", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSamplerBorderColorComponentMappingCreateInfoEXT(const VkSamplerBorderColorComponentMappingCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkComponentMapping>(object.components, NULL, settings, "VkComponentMapping", "components", true, false, indents + 1, dump_json_VkComponentMapping);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.srgb, NULL, settings, "VkBool32", "srgb", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePageableDeviceLocalMemoryFeaturesEXT(const VkPhysicalDevicePageableDeviceLocalMemoryFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.pageableDeviceLocalMemory, NULL, settings, "VkBool32", "pageableDeviceLocalMemory", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderCorePropertiesARM(const VkPhysicalDeviceShaderCorePropertiesARM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.pixelRate, NULL, settings, "uint32_t", "pixelRate", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.texelRate, NULL, settings, "uint32_t", "texelRate", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.fmaRate, NULL, settings, "uint32_t", "fmaRate", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderSubgroupRotateFeaturesKHR(const VkPhysicalDeviceShaderSubgroupRotateFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSubgroupRotate, NULL, settings, "VkBool32", "shaderSubgroupRotate", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderSubgroupRotateClustered, NULL, settings, "VkBool32", "shaderSubgroupRotateClustered", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceQueueShaderCoreControlCreateInfoARM(const VkDeviceQueueShaderCoreControlCreateInfoARM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shaderCoreCount, NULL, settings, "uint32_t", "shaderCoreCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSchedulingControlsFeaturesARM(const VkPhysicalDeviceSchedulingControlsFeaturesARM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.schedulingControls, NULL, settings, "VkBool32", "schedulingControls", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSchedulingControlsPropertiesARM(const VkPhysicalDeviceSchedulingControlsPropertiesARM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPhysicalDeviceSchedulingControlsFlagsARM>(object.schedulingControlsFlags, NULL, settings, "VkPhysicalDeviceSchedulingControlsFlagsARM", "schedulingControlsFlags", false, false, indents + 1, dump_json_VkPhysicalDeviceSchedulingControlsFlagsARM);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImageSlicedViewOf3DFeaturesEXT(const VkPhysicalDeviceImageSlicedViewOf3DFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.imageSlicedViewOf3D, NULL, settings, "VkBool32", "imageSlicedViewOf3D", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageViewSlicedCreateInfoEXT(const VkImageViewSlicedCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.sliceOffset, NULL, settings, "uint32_t", "sliceOffset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.sliceCount, NULL, settings, "uint32_t", "sliceCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDescriptorSetHostMappingFeaturesVALVE(const VkPhysicalDeviceDescriptorSetHostMappingFeaturesVALVE& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorSetHostMapping, NULL, settings, "VkBool32", "descriptorSetHostMapping", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorSetBindingReferenceVALVE(const VkDescriptorSetBindingReferenceVALVE& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorSetLayout>(object.descriptorSetLayout, NULL, settings, "VkDescriptorSetLayout", "descriptorSetLayout", false, false, indents + 1, dump_json_VkDescriptorSetLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.binding, NULL, settings, "uint32_t", "binding", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorSetLayoutHostMappingInfoVALVE(const VkDescriptorSetLayoutHostMappingInfoVALVE& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.descriptorOffset, NULL, settings, "size_t", "descriptorOffset", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.descriptorSize, NULL, settings, "uint32_t", "descriptorSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDepthClampZeroOneFeaturesEXT(const VkPhysicalDeviceDepthClampZeroOneFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthClampZeroOne, NULL, settings, "VkBool32", "depthClampZeroOne", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceNonSeamlessCubeMapFeaturesEXT(const VkPhysicalDeviceNonSeamlessCubeMapFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.nonSeamlessCubeMap, NULL, settings, "VkBool32", "nonSeamlessCubeMap", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRenderPassStripedFeaturesARM(const VkPhysicalDeviceRenderPassStripedFeaturesARM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.renderPassStriped, NULL, settings, "VkBool32", "renderPassStriped", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRenderPassStripedPropertiesARM(const VkPhysicalDeviceRenderPassStripedPropertiesARM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.renderPassStripeGranularity, NULL, settings, "VkExtent2D", "renderPassStripeGranularity", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxRenderPassStripes, NULL, settings, "uint32_t", "maxRenderPassStripes", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassStripeInfoARM(const VkRenderPassStripeInfoARM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkRect2D>(object.stripeArea, NULL, settings, "VkRect2D", "stripeArea", true, false, indents + 1, dump_json_VkRect2D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassStripeBeginInfoARM(const VkRenderPassStripeBeginInfoARM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stripeInfoCount, NULL, settings, "uint32_t", "stripeInfoCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkRenderPassStripeInfoARM>(object.pStripeInfos, object.stripeInfoCount, settings, "const VkRenderPassStripeInfoARM*", "const VkRenderPassStripeInfoARM", "pStripeInfos", true, false, indents + 1, dump_json_VkRenderPassStripeInfoARM); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassStripeSubmitInfoARM(const VkRenderPassStripeSubmitInfoARM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.stripeSemaphoreInfoCount, NULL, settings, "uint32_t", "stripeSemaphoreInfoCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkSemaphoreSubmitInfo>(object.pStripeSemaphoreInfos, object.stripeSemaphoreInfoCount, settings, "const VkSemaphoreSubmitInfo*", "const VkSemaphoreSubmitInfo", "pStripeSemaphoreInfos", true, false, indents + 1, dump_json_VkSemaphoreSubmitInfo); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFragmentDensityMapOffsetFeaturesQCOM(const VkPhysicalDeviceFragmentDensityMapOffsetFeaturesQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentDensityMapOffset, NULL, settings, "VkBool32", "fragmentDensityMapOffset", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceFragmentDensityMapOffsetPropertiesQCOM(const VkPhysicalDeviceFragmentDensityMapOffsetPropertiesQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.fragmentDensityOffsetGranularity, NULL, settings, "VkExtent2D", "fragmentDensityOffsetGranularity", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSubpassFragmentDensityMapOffsetEndInfoQCOM(const VkSubpassFragmentDensityMapOffsetEndInfoQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.fragmentDensityOffsetCount, NULL, settings, "uint32_t", "fragmentDensityOffsetCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkOffset2D>(object.pFragmentDensityOffsets, object.fragmentDensityOffsetCount, settings, "const VkOffset2D*", "const VkOffset2D", "pFragmentDensityOffsets", true, false, indents + 1, dump_json_VkOffset2D); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyMemoryIndirectCommandNV(const VkCopyMemoryIndirectCommandNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceAddress>(object.srcAddress, NULL, settings, "VkDeviceAddress", "srcAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddress>(object.dstAddress, NULL, settings, "VkDeviceAddress", "dstAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCopyMemoryToImageIndirectCommandNV(const VkCopyMemoryToImageIndirectCommandNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceAddress>(object.srcAddress, NULL, settings, "VkDeviceAddress", "srcAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.bufferRowLength, NULL, settings, "uint32_t", "bufferRowLength", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.bufferImageHeight, NULL, settings, "uint32_t", "bufferImageHeight", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkImageSubresourceLayers>(object.imageSubresource, NULL, settings, "VkImageSubresourceLayers", "imageSubresource", true, false, indents + 1, dump_json_VkImageSubresourceLayers);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset3D>(object.imageOffset, NULL, settings, "VkOffset3D", "imageOffset", true, false, indents + 1, dump_json_VkOffset3D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent3D>(object.imageExtent, NULL, settings, "VkExtent3D", "imageExtent", true, false, indents + 1, dump_json_VkExtent3D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceCopyMemoryIndirectFeaturesNV(const VkPhysicalDeviceCopyMemoryIndirectFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.indirectCopy, NULL, settings, "VkBool32", "indirectCopy", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceCopyMemoryIndirectPropertiesNV(const VkPhysicalDeviceCopyMemoryIndirectPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkQueueFlags>(object.supportedQueues, NULL, settings, "VkQueueFlags", "supportedQueues", false, false, indents + 1, dump_json_VkQueueFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDecompressMemoryRegionNV(const VkDecompressMemoryRegionNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceAddress>(object.srcAddress, NULL, settings, "VkDeviceAddress", "srcAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddress>(object.dstAddress, NULL, settings, "VkDeviceAddress", "dstAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.compressedSize, NULL, settings, "VkDeviceSize", "compressedSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.decompressedSize, NULL, settings, "VkDeviceSize", "decompressedSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkMemoryDecompressionMethodFlagsNV>(object.decompressionMethod, NULL, settings, "VkMemoryDecompressionMethodFlagsNV", "decompressionMethod", false, false, indents + 1, dump_json_VkMemoryDecompressionMethodFlagsNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMemoryDecompressionFeaturesNV(const VkPhysicalDeviceMemoryDecompressionFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.memoryDecompression, NULL, settings, "VkBool32", "memoryDecompression", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMemoryDecompressionPropertiesNV(const VkPhysicalDeviceMemoryDecompressionPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkMemoryDecompressionMethodFlagsNV>(object.decompressionMethods, NULL, settings, "VkMemoryDecompressionMethodFlagsNV", "decompressionMethods", false, false, indents + 1, dump_json_VkMemoryDecompressionMethodFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.maxDecompressionIndirectCount, NULL, settings, "uint64_t", "maxDecompressionIndirectCount", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDeviceGeneratedCommandsComputeFeaturesNV(const VkPhysicalDeviceDeviceGeneratedCommandsComputeFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.deviceGeneratedCompute, NULL, settings, "VkBool32", "deviceGeneratedCompute", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.deviceGeneratedComputePipelines, NULL, settings, "VkBool32", "deviceGeneratedComputePipelines", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.deviceGeneratedComputeCaptureReplay, NULL, settings, "VkBool32", "deviceGeneratedComputeCaptureReplay", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkComputePipelineIndirectBufferInfoNV(const VkComputePipelineIndirectBufferInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddress>(object.deviceAddress, NULL, settings, "VkDeviceAddress", "deviceAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.size, NULL, settings, "VkDeviceSize", "size", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceAddress>(object.pipelineDeviceAddressCaptureReplay, NULL, settings, "VkDeviceAddress", "pipelineDeviceAddressCaptureReplay", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineIndirectDeviceAddressInfoNV(const VkPipelineIndirectDeviceAddressInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineBindPoint>(object.pipelineBindPoint, NULL, settings, "VkPipelineBindPoint", "pipelineBindPoint", false, false, indents + 1, dump_json_VkPipelineBindPoint);
    settings.stream() << ",\n";
    dump_json_value<const VkPipeline>(object.pipeline, NULL, settings, "VkPipeline", "pipeline", false, false, indents + 1, dump_json_VkPipeline);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindPipelineIndirectCommandNV(const VkBindPipelineIndirectCommandNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceAddress>(object.pipelineAddress, NULL, settings, "VkDeviceAddress", "pipelineAddress", false, false, indents + 1, dump_json_VkDeviceAddress);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceLinearColorAttachmentFeaturesNV(const VkPhysicalDeviceLinearColorAttachmentFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.linearColorAttachment, NULL, settings, "VkBool32", "linearColorAttachment", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderMaximalReconvergenceFeaturesKHR(const VkPhysicalDeviceShaderMaximalReconvergenceFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderMaximalReconvergence, NULL, settings, "VkBool32", "shaderMaximalReconvergence", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImageCompressionControlSwapchainFeaturesEXT(const VkPhysicalDeviceImageCompressionControlSwapchainFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.imageCompressionControlSwapchain, NULL, settings, "VkBool32", "imageCompressionControlSwapchain", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageViewSampleWeightCreateInfoQCOM(const VkImageViewSampleWeightCreateInfoQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkOffset2D>(object.filterCenter, NULL, settings, "VkOffset2D", "filterCenter", true, false, indents + 1, dump_json_VkOffset2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.filterSize, NULL, settings, "VkExtent2D", "filterSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.numPhases, NULL, settings, "uint32_t", "numPhases", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImageProcessingFeaturesQCOM(const VkPhysicalDeviceImageProcessingFeaturesQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.textureSampleWeighted, NULL, settings, "VkBool32", "textureSampleWeighted", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.textureBoxFilter, NULL, settings, "VkBool32", "textureBoxFilter", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.textureBlockMatch, NULL, settings, "VkBool32", "textureBlockMatch", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImageProcessingPropertiesQCOM(const VkPhysicalDeviceImageProcessingPropertiesQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxWeightFilterPhases, NULL, settings, "uint32_t", "maxWeightFilterPhases", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxWeightFilterDimension, NULL, settings, "VkExtent2D", "maxWeightFilterDimension", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxBlockMatchRegion, NULL, settings, "VkExtent2D", "maxBlockMatchRegion", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxBoxFilterBlockSize, NULL, settings, "VkExtent2D", "maxBoxFilterBlockSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceNestedCommandBufferFeaturesEXT(const VkPhysicalDeviceNestedCommandBufferFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.nestedCommandBuffer, NULL, settings, "VkBool32", "nestedCommandBuffer", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.nestedCommandBufferRendering, NULL, settings, "VkBool32", "nestedCommandBufferRendering", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.nestedCommandBufferSimultaneousUse, NULL, settings, "VkBool32", "nestedCommandBufferSimultaneousUse", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceNestedCommandBufferPropertiesEXT(const VkPhysicalDeviceNestedCommandBufferPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxCommandBufferNestingLevel, NULL, settings, "uint32_t", "maxCommandBufferNestingLevel", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkExternalMemoryAcquireUnmodifiedEXT(const VkExternalMemoryAcquireUnmodifiedEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.acquireUnmodifiedMemory, NULL, settings, "VkBool32", "acquireUnmodifiedMemory", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceExtendedDynamicState3FeaturesEXT(const VkPhysicalDeviceExtendedDynamicState3FeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3TessellationDomainOrigin, NULL, settings, "VkBool32", "extendedDynamicState3TessellationDomainOrigin", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3DepthClampEnable, NULL, settings, "VkBool32", "extendedDynamicState3DepthClampEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3PolygonMode, NULL, settings, "VkBool32", "extendedDynamicState3PolygonMode", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3RasterizationSamples, NULL, settings, "VkBool32", "extendedDynamicState3RasterizationSamples", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3SampleMask, NULL, settings, "VkBool32", "extendedDynamicState3SampleMask", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3AlphaToCoverageEnable, NULL, settings, "VkBool32", "extendedDynamicState3AlphaToCoverageEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3AlphaToOneEnable, NULL, settings, "VkBool32", "extendedDynamicState3AlphaToOneEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3LogicOpEnable, NULL, settings, "VkBool32", "extendedDynamicState3LogicOpEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3ColorBlendEnable, NULL, settings, "VkBool32", "extendedDynamicState3ColorBlendEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3ColorBlendEquation, NULL, settings, "VkBool32", "extendedDynamicState3ColorBlendEquation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3ColorWriteMask, NULL, settings, "VkBool32", "extendedDynamicState3ColorWriteMask", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3RasterizationStream, NULL, settings, "VkBool32", "extendedDynamicState3RasterizationStream", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3ConservativeRasterizationMode, NULL, settings, "VkBool32", "extendedDynamicState3ConservativeRasterizationMode", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3ExtraPrimitiveOverestimationSize, NULL, settings, "VkBool32", "extendedDynamicState3ExtraPrimitiveOverestimationSize", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3DepthClipEnable, NULL, settings, "VkBool32", "extendedDynamicState3DepthClipEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3SampleLocationsEnable, NULL, settings, "VkBool32", "extendedDynamicState3SampleLocationsEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3ColorBlendAdvanced, NULL, settings, "VkBool32", "extendedDynamicState3ColorBlendAdvanced", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3ProvokingVertexMode, NULL, settings, "VkBool32", "extendedDynamicState3ProvokingVertexMode", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3LineRasterizationMode, NULL, settings, "VkBool32", "extendedDynamicState3LineRasterizationMode", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3LineStippleEnable, NULL, settings, "VkBool32", "extendedDynamicState3LineStippleEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3DepthClipNegativeOneToOne, NULL, settings, "VkBool32", "extendedDynamicState3DepthClipNegativeOneToOne", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3ViewportWScalingEnable, NULL, settings, "VkBool32", "extendedDynamicState3ViewportWScalingEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3ViewportSwizzle, NULL, settings, "VkBool32", "extendedDynamicState3ViewportSwizzle", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3CoverageToColorEnable, NULL, settings, "VkBool32", "extendedDynamicState3CoverageToColorEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3CoverageToColorLocation, NULL, settings, "VkBool32", "extendedDynamicState3CoverageToColorLocation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3CoverageModulationMode, NULL, settings, "VkBool32", "extendedDynamicState3CoverageModulationMode", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3CoverageModulationTableEnable, NULL, settings, "VkBool32", "extendedDynamicState3CoverageModulationTableEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3CoverageModulationTable, NULL, settings, "VkBool32", "extendedDynamicState3CoverageModulationTable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3CoverageReductionMode, NULL, settings, "VkBool32", "extendedDynamicState3CoverageReductionMode", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3RepresentativeFragmentTestEnable, NULL, settings, "VkBool32", "extendedDynamicState3RepresentativeFragmentTestEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedDynamicState3ShadingRateImageEnable, NULL, settings, "VkBool32", "extendedDynamicState3ShadingRateImageEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceExtendedDynamicState3PropertiesEXT(const VkPhysicalDeviceExtendedDynamicState3PropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.dynamicPrimitiveTopologyUnrestricted, NULL, settings, "VkBool32", "dynamicPrimitiveTopologyUnrestricted", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkColorBlendEquationEXT(const VkColorBlendEquationEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkBlendFactor>(object.srcColorBlendFactor, NULL, settings, "VkBlendFactor", "srcColorBlendFactor", false, false, indents + 1, dump_json_VkBlendFactor);
    settings.stream() << ",\n";
    dump_json_value<const VkBlendFactor>(object.dstColorBlendFactor, NULL, settings, "VkBlendFactor", "dstColorBlendFactor", false, false, indents + 1, dump_json_VkBlendFactor);
    settings.stream() << ",\n";
    dump_json_value<const VkBlendOp>(object.colorBlendOp, NULL, settings, "VkBlendOp", "colorBlendOp", false, false, indents + 1, dump_json_VkBlendOp);
    settings.stream() << ",\n";
    dump_json_value<const VkBlendFactor>(object.srcAlphaBlendFactor, NULL, settings, "VkBlendFactor", "srcAlphaBlendFactor", false, false, indents + 1, dump_json_VkBlendFactor);
    settings.stream() << ",\n";
    dump_json_value<const VkBlendFactor>(object.dstAlphaBlendFactor, NULL, settings, "VkBlendFactor", "dstAlphaBlendFactor", false, false, indents + 1, dump_json_VkBlendFactor);
    settings.stream() << ",\n";
    dump_json_value<const VkBlendOp>(object.alphaBlendOp, NULL, settings, "VkBlendOp", "alphaBlendOp", false, false, indents + 1, dump_json_VkBlendOp);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkColorBlendAdvancedEXT(const VkColorBlendAdvancedEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkBlendOp>(object.advancedBlendOp, NULL, settings, "VkBlendOp", "advancedBlendOp", false, false, indents + 1, dump_json_VkBlendOp);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.srcPremultiplied, NULL, settings, "VkBool32", "srcPremultiplied", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.dstPremultiplied, NULL, settings, "VkBool32", "dstPremultiplied", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBlendOverlapEXT>(object.blendOverlap, NULL, settings, "VkBlendOverlapEXT", "blendOverlap", false, false, indents + 1, dump_json_VkBlendOverlapEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.clampResults, NULL, settings, "VkBool32", "clampResults", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceSubpassMergeFeedbackFeaturesEXT(const VkPhysicalDeviceSubpassMergeFeedbackFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.subpassMergeFeedback, NULL, settings, "VkBool32", "subpassMergeFeedback", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassCreationControlEXT(const VkRenderPassCreationControlEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.disallowMerging, NULL, settings, "VkBool32", "disallowMerging", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassCreationFeedbackInfoEXT(const VkRenderPassCreationFeedbackInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.postMergeSubpassCount, NULL, settings, "uint32_t", "postMergeSubpassCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassCreationFeedbackCreateInfoEXT(const VkRenderPassCreationFeedbackCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const VkRenderPassCreationFeedbackInfoEXT>(object.pRenderPassFeedback, settings, "VkRenderPassCreationFeedbackInfoEXT*", "pRenderPassFeedback", true, false, indents + 1, dump_json_VkRenderPassCreationFeedbackInfoEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassSubpassFeedbackInfoEXT(const VkRenderPassSubpassFeedbackInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkSubpassMergeStatusEXT>(object.subpassMergeStatus, NULL, settings, "VkSubpassMergeStatusEXT", "subpassMergeStatus", false, false, indents + 1, dump_json_VkSubpassMergeStatusEXT);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.description, NULL, settings, "char[VK_MAX_DESCRIPTION_SIZE]", "description", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.postMergeIndex, NULL, settings, "uint32_t", "postMergeIndex", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderPassSubpassFeedbackCreateInfoEXT(const VkRenderPassSubpassFeedbackCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const VkRenderPassSubpassFeedbackInfoEXT>(object.pSubpassFeedback, settings, "VkRenderPassSubpassFeedbackInfoEXT*", "pSubpassFeedback", true, false, indents + 1, dump_json_VkRenderPassSubpassFeedbackInfoEXT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDirectDriverLoadingInfoLUNARG(const VkDirectDriverLoadingInfoLUNARG& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDirectDriverLoadingFlagsLUNARG>(object.flags, NULL, settings, "VkDirectDriverLoadingFlagsLUNARG", "flags", false, false, indents + 1, dump_json_VkDirectDriverLoadingFlagsLUNARG);
    settings.stream() << ",\n";
    dump_json_value<const PFN_vkGetInstanceProcAddrLUNARG>(object.pfnGetInstanceProcAddr, NULL, settings, "PFN_vkGetInstanceProcAddrLUNARG", "pfnGetInstanceProcAddr", false, false, indents + 1, dump_json_PFN_vkGetInstanceProcAddrLUNARG);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDirectDriverLoadingListLUNARG(const VkDirectDriverLoadingListLUNARG& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDirectDriverLoadingModeLUNARG>(object.mode, NULL, settings, "VkDirectDriverLoadingModeLUNARG", "mode", false, false, indents + 1, dump_json_VkDirectDriverLoadingModeLUNARG);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.driverCount, NULL, settings, "uint32_t", "driverCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDirectDriverLoadingInfoLUNARG>(object.pDrivers, object.driverCount, settings, "const VkDirectDriverLoadingInfoLUNARG*", "const VkDirectDriverLoadingInfoLUNARG", "pDrivers", true, false, indents + 1, dump_json_VkDirectDriverLoadingInfoLUNARG); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderModuleIdentifierFeaturesEXT(const VkPhysicalDeviceShaderModuleIdentifierFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderModuleIdentifier, NULL, settings, "VkBool32", "shaderModuleIdentifier", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderModuleIdentifierPropertiesEXT(const VkPhysicalDeviceShaderModuleIdentifierPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.shaderModuleIdentifierAlgorithmUUID, 16, settings, "uint8_t[VK_UUID_SIZE]", "uint8_t", "shaderModuleIdentifierAlgorithmUUID", false, false, indents + 1, dump_json_uint8_t); // IQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineShaderStageModuleIdentifierCreateInfoEXT(const VkPipelineShaderStageModuleIdentifierCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.identifierSize, NULL, settings, "uint32_t", "identifierSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.pIdentifier, object.identifierSize, settings, "const uint8_t*", "const uint8_t", "pIdentifier", false, false, indents + 1, dump_json_uint8_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkShaderModuleIdentifierEXT(const VkShaderModuleIdentifierEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.identifierSize, NULL, settings, "uint32_t", "identifierSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.identifier, 32, settings, "uint8_t[VK_MAX_SHADER_MODULE_IDENTIFIER_SIZE_EXT]", "uint8_t", "identifier", false, false, indents + 1, dump_json_uint8_t); // IQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceOpticalFlowFeaturesNV(const VkPhysicalDeviceOpticalFlowFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.opticalFlow, NULL, settings, "VkBool32", "opticalFlow", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceOpticalFlowPropertiesNV(const VkPhysicalDeviceOpticalFlowPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkOpticalFlowGridSizeFlagsNV>(object.supportedOutputGridSizes, NULL, settings, "VkOpticalFlowGridSizeFlagsNV", "supportedOutputGridSizes", false, false, indents + 1, dump_json_VkOpticalFlowGridSizeFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const VkOpticalFlowGridSizeFlagsNV>(object.supportedHintGridSizes, NULL, settings, "VkOpticalFlowGridSizeFlagsNV", "supportedHintGridSizes", false, false, indents + 1, dump_json_VkOpticalFlowGridSizeFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.hintSupported, NULL, settings, "VkBool32", "hintSupported", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.costSupported, NULL, settings, "VkBool32", "costSupported", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.bidirectionalFlowSupported, NULL, settings, "VkBool32", "bidirectionalFlowSupported", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.globalFlowSupported, NULL, settings, "VkBool32", "globalFlowSupported", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minWidth, NULL, settings, "uint32_t", "minWidth", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minHeight, NULL, settings, "uint32_t", "minHeight", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxWidth, NULL, settings, "uint32_t", "maxWidth", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxHeight, NULL, settings, "uint32_t", "maxHeight", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxNumRegionsOfInterest, NULL, settings, "uint32_t", "maxNumRegionsOfInterest", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkOpticalFlowImageFormatInfoNV(const VkOpticalFlowImageFormatInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkOpticalFlowUsageFlagsNV>(object.usage, NULL, settings, "VkOpticalFlowUsageFlagsNV", "usage", false, false, indents + 1, dump_json_VkOpticalFlowUsageFlagsNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkOpticalFlowImageFormatPropertiesNV(const VkOpticalFlowImageFormatPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkOpticalFlowSessionCreateInfoNV(const VkOpticalFlowSessionCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.width, NULL, settings, "uint32_t", "width", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.height, NULL, settings, "uint32_t", "height", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.imageFormat, NULL, settings, "VkFormat", "imageFormat", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.flowVectorFormat, NULL, settings, "VkFormat", "flowVectorFormat", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.costFormat, NULL, settings, "VkFormat", "costFormat", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkOpticalFlowGridSizeFlagsNV>(object.outputGridSize, NULL, settings, "VkOpticalFlowGridSizeFlagsNV", "outputGridSize", false, false, indents + 1, dump_json_VkOpticalFlowGridSizeFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const VkOpticalFlowGridSizeFlagsNV>(object.hintGridSize, NULL, settings, "VkOpticalFlowGridSizeFlagsNV", "hintGridSize", false, false, indents + 1, dump_json_VkOpticalFlowGridSizeFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const VkOpticalFlowPerformanceLevelNV>(object.performanceLevel, NULL, settings, "VkOpticalFlowPerformanceLevelNV", "performanceLevel", false, false, indents + 1, dump_json_VkOpticalFlowPerformanceLevelNV);
    settings.stream() << ",\n";
    dump_json_value<const VkOpticalFlowSessionCreateFlagsNV>(object.flags, NULL, settings, "VkOpticalFlowSessionCreateFlagsNV", "flags", false, false, indents + 1, dump_json_VkOpticalFlowSessionCreateFlagsNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkOpticalFlowSessionCreatePrivateDataInfoNV(const VkOpticalFlowSessionCreatePrivateDataInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.id, NULL, settings, "uint32_t", "id", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.size, NULL, settings, "uint32_t", "size", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pPrivateData, NULL, settings, "const void*", "pPrivateData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkOpticalFlowExecuteInfoNV(const VkOpticalFlowExecuteInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkOpticalFlowExecuteFlagsNV>(object.flags, NULL, settings, "VkOpticalFlowExecuteFlagsNV", "flags", false, false, indents + 1, dump_json_VkOpticalFlowExecuteFlagsNV);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.regionCount, NULL, settings, "uint32_t", "regionCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkRect2D>(object.pRegions, object.regionCount, settings, "const VkRect2D*", "const VkRect2D", "pRegions", true, false, indents + 1, dump_json_VkRect2D); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceLegacyDitheringFeaturesEXT(const VkPhysicalDeviceLegacyDitheringFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.legacyDithering, NULL, settings, "VkBool32", "legacyDithering", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePipelineProtectedAccessFeaturesEXT(const VkPhysicalDevicePipelineProtectedAccessFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.pipelineProtectedAccess, NULL, settings, "VkBool32", "pipelineProtectedAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_VkPhysicalDeviceExternalFormatResolveFeaturesANDROID(const VkPhysicalDeviceExternalFormatResolveFeaturesANDROID& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.externalFormatResolve, NULL, settings, "VkBool32", "externalFormatResolve", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_VkPhysicalDeviceExternalFormatResolvePropertiesANDROID(const VkPhysicalDeviceExternalFormatResolvePropertiesANDROID& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.nullColorAttachmentWithExternalFormatResolve, NULL, settings, "VkBool32", "nullColorAttachmentWithExternalFormatResolve", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkChromaLocation>(object.externalFormatResolveChromaOffsetX, NULL, settings, "VkChromaLocation", "externalFormatResolveChromaOffsetX", false, false, indents + 1, dump_json_VkChromaLocation);
    settings.stream() << ",\n";
    dump_json_value<const VkChromaLocation>(object.externalFormatResolveChromaOffsetY, NULL, settings, "VkChromaLocation", "externalFormatResolveChromaOffsetY", false, false, indents + 1, dump_json_VkChromaLocation);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_VkAndroidHardwareBufferFormatResolvePropertiesANDROID(const VkAndroidHardwareBufferFormatResolvePropertiesANDROID& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.colorAttachmentFormat, NULL, settings, "VkFormat", "colorAttachmentFormat", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_ANDROID_KHR
void dump_json_VkPhysicalDeviceMaintenance5FeaturesKHR(const VkPhysicalDeviceMaintenance5FeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.maintenance5, NULL, settings, "VkBool32", "maintenance5", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMaintenance5PropertiesKHR(const VkPhysicalDeviceMaintenance5PropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.earlyFragmentMultisampleCoverageAfterSampleCounting, NULL, settings, "VkBool32", "earlyFragmentMultisampleCoverageAfterSampleCounting", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.earlyFragmentSampleMaskTestBeforeSampleCounting, NULL, settings, "VkBool32", "earlyFragmentSampleMaskTestBeforeSampleCounting", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.depthStencilSwizzleOneSupport, NULL, settings, "VkBool32", "depthStencilSwizzleOneSupport", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.polygonModePointSize, NULL, settings, "VkBool32", "polygonModePointSize", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.nonStrictSinglePixelWideLinesUseParallelogram, NULL, settings, "VkBool32", "nonStrictSinglePixelWideLinesUseParallelogram", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.nonStrictWideLinesUseParallelogram, NULL, settings, "VkBool32", "nonStrictWideLinesUseParallelogram", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkRenderingAreaInfoKHR(const VkRenderingAreaInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.viewMask, NULL, settings, "uint32_t", "viewMask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.colorAttachmentCount, NULL, settings, "uint32_t", "colorAttachmentCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkFormat>(object.pColorAttachmentFormats, object.colorAttachmentCount, settings, "const VkFormat*", "const VkFormat", "pColorAttachmentFormats", false, false, indents + 1, dump_json_VkFormat); // JQA
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.depthAttachmentFormat, NULL, settings, "VkFormat", "depthAttachmentFormat", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.stencilAttachmentFormat, NULL, settings, "VkFormat", "stencilAttachmentFormat", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceImageSubresourceInfoKHR(const VkDeviceImageSubresourceInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const VkImageCreateInfo>(object.pCreateInfo, settings, "const VkImageCreateInfo*", "pCreateInfo", true, false, indents + 1, dump_json_VkImageCreateInfo);
    settings.stream() << ",\n";
    dump_json_pointer<const VkImageSubresource2KHR>(object.pSubresource, settings, "const VkImageSubresource2KHR*", "pSubresource", true, false, indents + 1, dump_json_VkImageSubresource2KHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineCreateFlags2CreateInfoKHR(const VkPipelineCreateFlags2CreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineCreateFlags2KHR>(object.flags, NULL, settings, "VkPipelineCreateFlags2KHR", "flags", false, false, indents + 1, dump_json_VkPipelineCreateFlags2KHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBufferUsageFlags2CreateInfoKHR(const VkBufferUsageFlags2CreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBufferUsageFlags2KHR>(object.usage, NULL, settings, "VkBufferUsageFlags2KHR", "usage", false, false, indents + 1, dump_json_VkBufferUsageFlags2KHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRayTracingPositionFetchFeaturesKHR(const VkPhysicalDeviceRayTracingPositionFetchFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rayTracingPositionFetch, NULL, settings, "VkBool32", "rayTracingPositionFetch", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderObjectFeaturesEXT(const VkPhysicalDeviceShaderObjectFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderObject, NULL, settings, "VkBool32", "shaderObject", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderObjectPropertiesEXT(const VkPhysicalDeviceShaderObjectPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_array<const uint8_t>(object.shaderBinaryUUID, 16, settings, "uint8_t[VK_UUID_SIZE]", "uint8_t", "shaderBinaryUUID", false, false, indents + 1, dump_json_uint8_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shaderBinaryVersion, NULL, settings, "uint32_t", "shaderBinaryVersion", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkShaderCreateInfoEXT(const VkShaderCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkShaderCreateFlagsEXT>(object.flags, NULL, settings, "VkShaderCreateFlagsEXT", "flags", false, false, indents + 1, dump_json_VkShaderCreateFlagsEXT);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlagBits>(object.stage, NULL, settings, "VkShaderStageFlagBits", "stage", false, false, indents + 1, dump_json_VkShaderStageFlagBits);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlags>(object.nextStage, NULL, settings, "VkShaderStageFlags", "nextStage", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderCodeTypeEXT>(object.codeType, NULL, settings, "VkShaderCodeTypeEXT", "codeType", false, false, indents + 1, dump_json_VkShaderCodeTypeEXT);
    settings.stream() << ",\n";
    dump_json_value<const size_t>(object.codeSize, NULL, settings, "size_t", "codeSize", false, false, indents + 1, dump_json_size_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pCode, NULL, settings, "const void*", "pCode", false, false, indents + 1, dump_json_void);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.pName, NULL, settings, "const char*", "pName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.setLayoutCount, NULL, settings, "uint32_t", "setLayoutCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDescriptorSetLayout>(object.pSetLayouts, object.setLayoutCount, settings, "const VkDescriptorSetLayout*", "const VkDescriptorSetLayout", "pSetLayouts", false, false, indents + 1, dump_json_VkDescriptorSetLayout); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.pushConstantRangeCount, NULL, settings, "uint32_t", "pushConstantRangeCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPushConstantRange>(object.pPushConstantRanges, object.pushConstantRangeCount, settings, "const VkPushConstantRange*", "const VkPushConstantRange", "pPushConstantRanges", true, false, indents + 1, dump_json_VkPushConstantRange); // JQA
    settings.stream() << ",\n";
    dump_json_pointer<const VkSpecializationInfo>(object.pSpecializationInfo, settings, "const VkSpecializationInfo*", "pSpecializationInfo", true, false, indents + 1, dump_json_VkSpecializationInfo);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceTilePropertiesFeaturesQCOM(const VkPhysicalDeviceTilePropertiesFeaturesQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.tileProperties, NULL, settings, "VkBool32", "tileProperties", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkTilePropertiesQCOM(const VkTilePropertiesQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExtent3D>(object.tileSize, NULL, settings, "VkExtent3D", "tileSize", true, false, indents + 1, dump_json_VkExtent3D);
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.apronSize, NULL, settings, "VkExtent2D", "apronSize", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkOffset2D>(object.origin, NULL, settings, "VkOffset2D", "origin", true, false, indents + 1, dump_json_VkOffset2D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceAmigoProfilingFeaturesSEC(const VkPhysicalDeviceAmigoProfilingFeaturesSEC& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.amigoProfiling, NULL, settings, "VkBool32", "amigoProfiling", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAmigoProfilingSubmitInfoSEC(const VkAmigoProfilingSubmitInfoSEC& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.firstDrawTimestamp, NULL, settings, "uint64_t", "firstDrawTimestamp", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.swapBufferTimestamp, NULL, settings, "uint64_t", "swapBufferTimestamp", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMultiviewPerViewViewportsFeaturesQCOM(const VkPhysicalDeviceMultiviewPerViewViewportsFeaturesQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multiviewPerViewViewports, NULL, settings, "VkBool32", "multiviewPerViewViewports", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRayTracingInvocationReorderPropertiesNV(const VkPhysicalDeviceRayTracingInvocationReorderPropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkRayTracingInvocationReorderModeNV>(object.rayTracingInvocationReorderReorderingHint, NULL, settings, "VkRayTracingInvocationReorderModeNV", "rayTracingInvocationReorderReorderingHint", false, false, indents + 1, dump_json_VkRayTracingInvocationReorderModeNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRayTracingInvocationReorderFeaturesNV(const VkPhysicalDeviceRayTracingInvocationReorderFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rayTracingInvocationReorder, NULL, settings, "VkBool32", "rayTracingInvocationReorder", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceExtendedSparseAddressSpaceFeaturesNV(const VkPhysicalDeviceExtendedSparseAddressSpaceFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.extendedSparseAddressSpace, NULL, settings, "VkBool32", "extendedSparseAddressSpace", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceExtendedSparseAddressSpacePropertiesNV(const VkPhysicalDeviceExtendedSparseAddressSpacePropertiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.extendedSparseAddressSpaceSize, NULL, settings, "VkDeviceSize", "extendedSparseAddressSpaceSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const VkImageUsageFlags>(object.extendedSparseImageUsageFlags, NULL, settings, "VkImageUsageFlags", "extendedSparseImageUsageFlags", false, false, indents + 1, dump_json_VkImageUsageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkBufferUsageFlags>(object.extendedSparseBufferUsageFlags, NULL, settings, "VkBufferUsageFlags", "extendedSparseBufferUsageFlags", false, false, indents + 1, dump_json_VkBufferUsageFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceLegacyVertexAttributesFeaturesEXT(const VkPhysicalDeviceLegacyVertexAttributesFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.legacyVertexAttributes, NULL, settings, "VkBool32", "legacyVertexAttributes", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceLegacyVertexAttributesPropertiesEXT(const VkPhysicalDeviceLegacyVertexAttributesPropertiesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.nativeUnalignedPerformance, NULL, settings, "VkBool32", "nativeUnalignedPerformance", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkLayerSettingEXT(const VkLayerSettingEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const char*>(object.pLayerName, NULL, settings, "const char*", "pLayerName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.pSettingName, NULL, settings, "const char*", "pSettingName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << ",\n";
    dump_json_value<const VkLayerSettingTypeEXT>(object.type, NULL, settings, "VkLayerSettingTypeEXT", "type", false, false, indents + 1, dump_json_VkLayerSettingTypeEXT);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.valueCount, NULL, settings, "uint32_t", "valueCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pValues, NULL, settings, "const void*", "pValues", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkLayerSettingsCreateInfoEXT(const VkLayerSettingsCreateInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.settingCount, NULL, settings, "uint32_t", "settingCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkLayerSettingEXT>(object.pSettings, object.settingCount, settings, "const VkLayerSettingEXT*", "const VkLayerSettingEXT", "pSettings", true, false, indents + 1, dump_json_VkLayerSettingEXT); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderCoreBuiltinsFeaturesARM(const VkPhysicalDeviceShaderCoreBuiltinsFeaturesARM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderCoreBuiltins, NULL, settings, "VkBool32", "shaderCoreBuiltins", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderCoreBuiltinsPropertiesARM(const VkPhysicalDeviceShaderCoreBuiltinsPropertiesARM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.shaderCoreMask, NULL, settings, "uint64_t", "shaderCoreMask", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shaderCoreCount, NULL, settings, "uint32_t", "shaderCoreCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.shaderWarpsPerCore, NULL, settings, "uint32_t", "shaderWarpsPerCore", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePipelineLibraryGroupHandlesFeaturesEXT(const VkPhysicalDevicePipelineLibraryGroupHandlesFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.pipelineLibraryGroupHandles, NULL, settings, "VkBool32", "pipelineLibraryGroupHandles", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDynamicRenderingUnusedAttachmentsFeaturesEXT(const VkPhysicalDeviceDynamicRenderingUnusedAttachmentsFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.dynamicRenderingUnusedAttachments, NULL, settings, "VkBool32", "dynamicRenderingUnusedAttachments", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkLatencySleepModeInfoNV(const VkLatencySleepModeInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.lowLatencyMode, NULL, settings, "VkBool32", "lowLatencyMode", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.lowLatencyBoost, NULL, settings, "VkBool32", "lowLatencyBoost", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.minimumIntervalUs, NULL, settings, "uint32_t", "minimumIntervalUs", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkLatencySleepInfoNV(const VkLatencySleepInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkSemaphore>(object.signalSemaphore, NULL, settings, "VkSemaphore", "signalSemaphore", false, false, indents + 1, dump_json_VkSemaphore);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.value, NULL, settings, "uint64_t", "value", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSetLatencyMarkerInfoNV(const VkSetLatencyMarkerInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.presentID, NULL, settings, "uint64_t", "presentID", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const VkLatencyMarkerNV>(object.marker, NULL, settings, "VkLatencyMarkerNV", "marker", false, false, indents + 1, dump_json_VkLatencyMarkerNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkLatencyTimingsFrameReportNV(const VkLatencyTimingsFrameReportNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.presentID, NULL, settings, "uint64_t", "presentID", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.inputSampleTimeUs, NULL, settings, "uint64_t", "inputSampleTimeUs", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.simStartTimeUs, NULL, settings, "uint64_t", "simStartTimeUs", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.simEndTimeUs, NULL, settings, "uint64_t", "simEndTimeUs", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.renderSubmitStartTimeUs, NULL, settings, "uint64_t", "renderSubmitStartTimeUs", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.renderSubmitEndTimeUs, NULL, settings, "uint64_t", "renderSubmitEndTimeUs", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.presentStartTimeUs, NULL, settings, "uint64_t", "presentStartTimeUs", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.presentEndTimeUs, NULL, settings, "uint64_t", "presentEndTimeUs", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.driverStartTimeUs, NULL, settings, "uint64_t", "driverStartTimeUs", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.driverEndTimeUs, NULL, settings, "uint64_t", "driverEndTimeUs", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.osRenderQueueStartTimeUs, NULL, settings, "uint64_t", "osRenderQueueStartTimeUs", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.osRenderQueueEndTimeUs, NULL, settings, "uint64_t", "osRenderQueueEndTimeUs", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.gpuRenderStartTimeUs, NULL, settings, "uint64_t", "gpuRenderStartTimeUs", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.gpuRenderEndTimeUs, NULL, settings, "uint64_t", "gpuRenderEndTimeUs", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkGetLatencyMarkerInfoNV(const VkGetLatencyMarkerInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.timingCount, NULL, settings, "uint32_t", "timingCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkLatencyTimingsFrameReportNV>(object.pTimings, object.timingCount, settings, "VkLatencyTimingsFrameReportNV*", "VkLatencyTimingsFrameReportNV", "pTimings", true, false, indents + 1, dump_json_VkLatencyTimingsFrameReportNV); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkLatencySubmissionPresentIdNV(const VkLatencySubmissionPresentIdNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.presentID, NULL, settings, "uint64_t", "presentID", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSwapchainLatencyCreateInfoNV(const VkSwapchainLatencyCreateInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.latencyModeEnable, NULL, settings, "VkBool32", "latencyModeEnable", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkOutOfBandQueueTypeInfoNV(const VkOutOfBandQueueTypeInfoNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkOutOfBandQueueTypeNV>(object.queueType, NULL, settings, "VkOutOfBandQueueTypeNV", "queueType", false, false, indents + 1, dump_json_VkOutOfBandQueueTypeNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkLatencySurfaceCapabilitiesNV(const VkLatencySurfaceCapabilitiesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.presentModeCount, NULL, settings, "uint32_t", "presentModeCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPresentModeKHR>(object.pPresentModes, object.presentModeCount, settings, "VkPresentModeKHR*", "VkPresentModeKHR", "pPresentModes", false, false, indents + 1, dump_json_VkPresentModeKHR); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkCooperativeMatrixPropertiesKHR(const VkCooperativeMatrixPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.MSize, NULL, settings, "uint32_t", "MSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.NSize, NULL, settings, "uint32_t", "NSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.KSize, NULL, settings, "uint32_t", "KSize", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentTypeKHR>(object.AType, NULL, settings, "VkComponentTypeKHR", "AType", false, false, indents + 1, dump_json_VkComponentTypeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentTypeKHR>(object.BType, NULL, settings, "VkComponentTypeKHR", "BType", false, false, indents + 1, dump_json_VkComponentTypeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentTypeKHR>(object.CType, NULL, settings, "VkComponentTypeKHR", "CType", false, false, indents + 1, dump_json_VkComponentTypeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentTypeKHR>(object.ResultType, NULL, settings, "VkComponentTypeKHR", "ResultType", false, false, indents + 1, dump_json_VkComponentTypeKHR);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.saturatingAccumulation, NULL, settings, "VkBool32", "saturatingAccumulation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkScopeKHR>(object.scope, NULL, settings, "VkScopeKHR", "scope", false, false, indents + 1, dump_json_VkScopeKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceCooperativeMatrixFeaturesKHR(const VkPhysicalDeviceCooperativeMatrixFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.cooperativeMatrix, NULL, settings, "VkBool32", "cooperativeMatrix", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.cooperativeMatrixRobustBufferAccess, NULL, settings, "VkBool32", "cooperativeMatrixRobustBufferAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceCooperativeMatrixPropertiesKHR(const VkPhysicalDeviceCooperativeMatrixPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlags>(object.cooperativeMatrixSupportedStages, NULL, settings, "VkShaderStageFlags", "cooperativeMatrixSupportedStages", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMultiviewPerViewRenderAreasFeaturesQCOM(const VkPhysicalDeviceMultiviewPerViewRenderAreasFeaturesQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.multiviewPerViewRenderAreas, NULL, settings, "VkBool32", "multiviewPerViewRenderAreas", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkMultiviewPerViewRenderAreasRenderPassBeginInfoQCOM(const VkMultiviewPerViewRenderAreasRenderPassBeginInfoQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.perViewRenderAreaCount, NULL, settings, "uint32_t", "perViewRenderAreaCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkRect2D>(object.pPerViewRenderAreas, object.perViewRenderAreaCount, settings, "const VkRect2D*", "const VkRect2D", "pPerViewRenderAreas", true, false, indents + 1, dump_json_VkRect2D); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeAV1ProfileInfoKHR(const VkVideoDecodeAV1ProfileInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const StdVideoAV1Profile>(object.stdProfile, NULL, settings, "StdVideoAV1Profile", "stdProfile", false, false, indents + 1, dump_json_StdVideoAV1Profile);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.filmGrainSupport, NULL, settings, "VkBool32", "filmGrainSupport", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeAV1CapabilitiesKHR(const VkVideoDecodeAV1CapabilitiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const StdVideoAV1Level>(object.maxLevel, NULL, settings, "StdVideoAV1Level", "maxLevel", false, false, indents + 1, dump_json_StdVideoAV1Level);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeAV1SessionParametersCreateInfoKHR(const VkVideoDecodeAV1SessionParametersCreateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const StdVideoAV1SequenceHeader>(object.pStdSequenceHeader, settings, "const StdVideoAV1SequenceHeader*", "pStdSequenceHeader", false, false, indents + 1, dump_json_StdVideoAV1SequenceHeader);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeAV1PictureInfoKHR(const VkVideoDecodeAV1PictureInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const StdVideoDecodeAV1PictureInfo>(object.pStdPictureInfo, settings, "const StdVideoDecodeAV1PictureInfo*", "pStdPictureInfo", false, false, indents + 1, dump_json_StdVideoDecodeAV1PictureInfo);
    settings.stream() << ",\n";
    dump_json_array<const int32_t>(object.referenceNameSlotIndices, 7, settings, "int32_t[VK_MAX_VIDEO_AV1_REFERENCES_PER_FRAME_KHR]", "int32_t", "referenceNameSlotIndices", false, false, indents + 1, dump_json_int32_t); // IQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.frameHeaderOffset, NULL, settings, "uint32_t", "frameHeaderOffset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.tileCount, NULL, settings, "uint32_t", "tileCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pTileOffsets, object.tileCount, settings, "const uint32_t*", "const uint32_t", "pTileOffsets", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pTileSizes, object.tileCount, settings, "const uint32_t*", "const uint32_t", "pTileSizes", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoDecodeAV1DpbSlotInfoKHR(const VkVideoDecodeAV1DpbSlotInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const StdVideoDecodeAV1ReferenceInfo>(object.pStdReferenceInfo, settings, "const StdVideoDecodeAV1ReferenceInfo*", "pStdReferenceInfo", false, false, indents + 1, dump_json_StdVideoDecodeAV1ReferenceInfo);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceVideoMaintenance1FeaturesKHR(const VkPhysicalDeviceVideoMaintenance1FeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.videoMaintenance1, NULL, settings, "VkBool32", "videoMaintenance1", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkVideoInlineQueryInfoKHR(const VkVideoInlineQueryInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkQueryPool>(object.queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, indents + 1, dump_json_VkQueryPool);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.firstQuery, NULL, settings, "uint32_t", "firstQuery", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.queryCount, NULL, settings, "uint32_t", "queryCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDevicePerStageDescriptorSetFeaturesNV(const VkPhysicalDevicePerStageDescriptorSetFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.perStageDescriptorSet, NULL, settings, "VkBool32", "perStageDescriptorSet", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.dynamicPipelineLayout, NULL, settings, "VkBool32", "dynamicPipelineLayout", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImageProcessing2FeaturesQCOM(const VkPhysicalDeviceImageProcessing2FeaturesQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.textureBlockMatch2, NULL, settings, "VkBool32", "textureBlockMatch2", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImageProcessing2PropertiesQCOM(const VkPhysicalDeviceImageProcessing2PropertiesQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.maxBlockMatchWindow, NULL, settings, "VkExtent2D", "maxBlockMatchWindow", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSamplerBlockMatchWindowCreateInfoQCOM(const VkSamplerBlockMatchWindowCreateInfoQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkExtent2D>(object.windowExtent, NULL, settings, "VkExtent2D", "windowExtent", true, false, indents + 1, dump_json_VkExtent2D);
    settings.stream() << ",\n";
    dump_json_value<const VkBlockMatchWindowCompareModeQCOM>(object.windowCompareMode, NULL, settings, "VkBlockMatchWindowCompareModeQCOM", "windowCompareMode", false, false, indents + 1, dump_json_VkBlockMatchWindowCompareModeQCOM);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceCubicWeightsFeaturesQCOM(const VkPhysicalDeviceCubicWeightsFeaturesQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.selectableCubicWeights, NULL, settings, "VkBool32", "selectableCubicWeights", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSamplerCubicWeightsCreateInfoQCOM(const VkSamplerCubicWeightsCreateInfoQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkCubicFilterWeightsQCOM>(object.cubicWeights, NULL, settings, "VkCubicFilterWeightsQCOM", "cubicWeights", false, false, indents + 1, dump_json_VkCubicFilterWeightsQCOM);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBlitImageCubicWeightsInfoQCOM(const VkBlitImageCubicWeightsInfoQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkCubicFilterWeightsQCOM>(object.cubicWeights, NULL, settings, "VkCubicFilterWeightsQCOM", "cubicWeights", false, false, indents + 1, dump_json_VkCubicFilterWeightsQCOM);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceYcbcrDegammaFeaturesQCOM(const VkPhysicalDeviceYcbcrDegammaFeaturesQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.ycbcrDegamma, NULL, settings, "VkBool32", "ycbcrDegamma", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSamplerYcbcrConversionYcbcrDegammaCreateInfoQCOM(const VkSamplerYcbcrConversionYcbcrDegammaCreateInfoQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.enableYDegamma, NULL, settings, "VkBool32", "enableYDegamma", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.enableCbCrDegamma, NULL, settings, "VkBool32", "enableCbCrDegamma", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceCubicClampFeaturesQCOM(const VkPhysicalDeviceCubicClampFeaturesQCOM& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.cubicRangeClamp, NULL, settings, "VkBool32", "cubicRangeClamp", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceAttachmentFeedbackLoopDynamicStateFeaturesEXT(const VkPhysicalDeviceAttachmentFeedbackLoopDynamicStateFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.attachmentFeedbackLoopDynamicState, NULL, settings, "VkBool32", "attachmentFeedbackLoopDynamicState", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceVertexAttributeDivisorPropertiesKHR(const VkPhysicalDeviceVertexAttributeDivisorPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxVertexAttribDivisor, NULL, settings, "uint32_t", "maxVertexAttribDivisor", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.supportsNonZeroFirstInstance, NULL, settings, "VkBool32", "supportsNonZeroFirstInstance", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderFloatControls2FeaturesKHR(const VkPhysicalDeviceShaderFloatControls2FeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderFloatControls2, NULL, settings, "VkBool32", "shaderFloatControls2", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
void dump_json_VkScreenBufferPropertiesQNX(const VkScreenBufferPropertiesQNX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDeviceSize>(object.allocationSize, NULL, settings, "VkDeviceSize", "allocationSize", false, false, indents + 1, dump_json_VkDeviceSize);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.memoryTypeBits, NULL, settings, "uint32_t", "memoryTypeBits", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_SCREEN_QNX
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
void dump_json_VkScreenBufferFormatPropertiesQNX(const VkScreenBufferFormatPropertiesQNX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkFormat>(object.format, NULL, settings, "VkFormat", "format", false, false, indents + 1, dump_json_VkFormat);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.externalFormat, NULL, settings, "uint64_t", "externalFormat", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.screenUsage, NULL, settings, "uint64_t", "screenUsage", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << ",\n";
    dump_json_value<const VkFormatFeatureFlags>(object.formatFeatures, NULL, settings, "VkFormatFeatureFlags", "formatFeatures", false, false, indents + 1, dump_json_VkFormatFeatureFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkComponentMapping>(object.samplerYcbcrConversionComponents, NULL, settings, "VkComponentMapping", "samplerYcbcrConversionComponents", true, false, indents + 1, dump_json_VkComponentMapping);
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerYcbcrModelConversion>(object.suggestedYcbcrModel, NULL, settings, "VkSamplerYcbcrModelConversion", "suggestedYcbcrModel", false, false, indents + 1, dump_json_VkSamplerYcbcrModelConversion);
    settings.stream() << ",\n";
    dump_json_value<const VkSamplerYcbcrRange>(object.suggestedYcbcrRange, NULL, settings, "VkSamplerYcbcrRange", "suggestedYcbcrRange", false, false, indents + 1, dump_json_VkSamplerYcbcrRange);
    settings.stream() << ",\n";
    dump_json_value<const VkChromaLocation>(object.suggestedXChromaOffset, NULL, settings, "VkChromaLocation", "suggestedXChromaOffset", false, false, indents + 1, dump_json_VkChromaLocation);
    settings.stream() << ",\n";
    dump_json_value<const VkChromaLocation>(object.suggestedYChromaOffset, NULL, settings, "VkChromaLocation", "suggestedYChromaOffset", false, false, indents + 1, dump_json_VkChromaLocation);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_SCREEN_QNX
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
void dump_json_VkImportScreenBufferInfoQNX(const VkImportScreenBufferInfoQNX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const _screen_buffer*>(object.buffer, NULL, settings, "struct _screen_buffer*", "buffer", false, false, indents + 1, dump_json__screen_buffer);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_SCREEN_QNX
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
void dump_json_VkExternalFormatQNX(const VkExternalFormatQNX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint64_t>(object.externalFormat, NULL, settings, "uint64_t", "externalFormat", false, false, indents + 1, dump_json_uint64_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_SCREEN_QNX
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
void dump_json_VkPhysicalDeviceExternalMemoryScreenBufferFeaturesQNX(const VkPhysicalDeviceExternalMemoryScreenBufferFeaturesQNX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.screenBufferImport, NULL, settings, "VkBool32", "screenBufferImport", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_USE_PLATFORM_SCREEN_QNX
void dump_json_VkPhysicalDeviceLayeredDriverPropertiesMSFT(const VkPhysicalDeviceLayeredDriverPropertiesMSFT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkLayeredDriverUnderlyingApiMSFT>(object.underlyingAPI, NULL, settings, "VkLayeredDriverUnderlyingApiMSFT", "underlyingAPI", false, false, indents + 1, dump_json_VkLayeredDriverUnderlyingApiMSFT);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderExpectAssumeFeaturesKHR(const VkPhysicalDeviceShaderExpectAssumeFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderExpectAssume, NULL, settings, "VkBool32", "shaderExpectAssume", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMaintenance6FeaturesKHR(const VkPhysicalDeviceMaintenance6FeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.maintenance6, NULL, settings, "VkBool32", "maintenance6", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMaintenance6PropertiesKHR(const VkPhysicalDeviceMaintenance6PropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.blockTexelViewCompatibleMultipleLayers, NULL, settings, "VkBool32", "blockTexelViewCompatibleMultipleLayers", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxCombinedImageSamplerDescriptorCount, NULL, settings, "uint32_t", "maxCombinedImageSamplerDescriptorCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.fragmentShadingRateClampCombinerInputs, NULL, settings, "VkBool32", "fragmentShadingRateClampCombinerInputs", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindMemoryStatusKHR(const VkBindMemoryStatusKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_pointer<const VkResult>(object.pResult, settings, "VkResult*", "pResult", false, false, indents + 1, dump_json_VkResult);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindDescriptorSetsInfoKHR(const VkBindDescriptorSetsInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlags>(object.stageFlags, NULL, settings, "VkShaderStageFlags", "stageFlags", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineLayout>(object.layout, NULL, settings, "VkPipelineLayout", "layout", false, false, indents + 1, dump_json_VkPipelineLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.firstSet, NULL, settings, "uint32_t", "firstSet", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.descriptorSetCount, NULL, settings, "uint32_t", "descriptorSetCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkDescriptorSet>(object.pDescriptorSets, object.descriptorSetCount, settings, "const VkDescriptorSet*", "const VkDescriptorSet", "pDescriptorSets", false, false, indents + 1, dump_json_VkDescriptorSet); // JQA
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.dynamicOffsetCount, NULL, settings, "uint32_t", "dynamicOffsetCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pDynamicOffsets, object.dynamicOffsetCount, settings, "const uint32_t*", "const uint32_t", "pDynamicOffsets", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPushConstantsInfoKHR(const VkPushConstantsInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineLayout>(object.layout, NULL, settings, "VkPipelineLayout", "layout", false, false, indents + 1, dump_json_VkPipelineLayout);
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlags>(object.stageFlags, NULL, settings, "VkShaderStageFlags", "stageFlags", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.offset, NULL, settings, "uint32_t", "offset", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.size, NULL, settings, "uint32_t", "size", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pValues, NULL, settings, "const void*", "pValues", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPushDescriptorSetInfoKHR(const VkPushDescriptorSetInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlags>(object.stageFlags, NULL, settings, "VkShaderStageFlags", "stageFlags", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineLayout>(object.layout, NULL, settings, "VkPipelineLayout", "layout", false, false, indents + 1, dump_json_VkPipelineLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.set, NULL, settings, "uint32_t", "set", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.descriptorWriteCount, NULL, settings, "uint32_t", "descriptorWriteCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkWriteDescriptorSet>(object.pDescriptorWrites, object.descriptorWriteCount, settings, "const VkWriteDescriptorSet*", "const VkWriteDescriptorSet", "pDescriptorWrites", true, false, indents + 1, dump_json_VkWriteDescriptorSet); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPushDescriptorSetWithTemplateInfoKHR(const VkPushDescriptorSetWithTemplateInfoKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkDescriptorUpdateTemplate>(object.descriptorUpdateTemplate, NULL, settings, "VkDescriptorUpdateTemplate", "descriptorUpdateTemplate", false, false, indents + 1, dump_json_VkDescriptorUpdateTemplate);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineLayout>(object.layout, NULL, settings, "VkPipelineLayout", "layout", false, false, indents + 1, dump_json_VkPipelineLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.set, NULL, settings, "uint32_t", "set", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const void*>(object.pData, NULL, settings, "const void*", "pData", false, false, indents + 1, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkSetDescriptorBufferOffsetsInfoEXT(const VkSetDescriptorBufferOffsetsInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlags>(object.stageFlags, NULL, settings, "VkShaderStageFlags", "stageFlags", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineLayout>(object.layout, NULL, settings, "VkPipelineLayout", "layout", false, false, indents + 1, dump_json_VkPipelineLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.firstSet, NULL, settings, "uint32_t", "firstSet", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.setCount, NULL, settings, "uint32_t", "setCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const uint32_t>(object.pBufferIndices, object.setCount, settings, "const uint32_t*", "const uint32_t", "pBufferIndices", false, false, indents + 1, dump_json_uint32_t); // JQA
    settings.stream() << ",\n";
    dump_json_array<const VkDeviceSize>(object.pOffsets, object.setCount, settings, "const VkDeviceSize*", "const VkDeviceSize", "pOffsets", false, false, indents + 1, dump_json_VkDeviceSize); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkBindDescriptorBufferEmbeddedSamplersInfoEXT(const VkBindDescriptorBufferEmbeddedSamplersInfoEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkShaderStageFlags>(object.stageFlags, NULL, settings, "VkShaderStageFlags", "stageFlags", false, false, indents + 1, dump_json_VkShaderStageFlags);
    settings.stream() << ",\n";
    dump_json_value<const VkPipelineLayout>(object.layout, NULL, settings, "VkPipelineLayout", "layout", false, false, indents + 1, dump_json_VkPipelineLayout);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.set, NULL, settings, "uint32_t", "set", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceDescriptorPoolOverallocationFeaturesNV(const VkPhysicalDeviceDescriptorPoolOverallocationFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.descriptorPoolOverallocation, NULL, settings, "VkBool32", "descriptorPoolOverallocation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRawAccessChainsFeaturesNV(const VkPhysicalDeviceRawAccessChainsFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderRawAccessChains, NULL, settings, "VkBool32", "shaderRawAccessChains", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderRelaxedExtendedInstructionFeaturesKHR(const VkPhysicalDeviceShaderRelaxedExtendedInstructionFeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderRelaxedExtendedInstruction, NULL, settings, "VkBool32", "shaderRelaxedExtendedInstruction", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMaintenance7FeaturesKHR(const VkPhysicalDeviceMaintenance7FeaturesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.maintenance7, NULL, settings, "VkBool32", "maintenance7", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceMaintenance7PropertiesKHR(const VkPhysicalDeviceMaintenance7PropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.robustFragmentShadingRateAttachmentAccess, NULL, settings, "VkBool32", "robustFragmentShadingRateAttachmentAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.separateDepthStencilAttachmentAccess, NULL, settings, "VkBool32", "separateDepthStencilAttachmentAccess", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetTotalUniformBuffersDynamic, NULL, settings, "uint32_t", "maxDescriptorSetTotalUniformBuffersDynamic", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetTotalStorageBuffersDynamic, NULL, settings, "uint32_t", "maxDescriptorSetTotalStorageBuffersDynamic", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetTotalBuffersDynamic, NULL, settings, "uint32_t", "maxDescriptorSetTotalBuffersDynamic", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindTotalUniformBuffersDynamic, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindTotalUniformBuffersDynamic", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindTotalStorageBuffersDynamic, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindTotalStorageBuffersDynamic", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maxDescriptorSetUpdateAfterBindTotalBuffersDynamic, NULL, settings, "uint32_t", "maxDescriptorSetUpdateAfterBindTotalBuffersDynamic", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceLayeredApiPropertiesKHR(const VkPhysicalDeviceLayeredApiPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.vendorID, NULL, settings, "uint32_t", "vendorID", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.deviceID, NULL, settings, "uint32_t", "deviceID", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_value<const VkPhysicalDeviceLayeredApiKHR>(object.layeredAPI, NULL, settings, "VkPhysicalDeviceLayeredApiKHR", "layeredAPI", false, false, indents + 1, dump_json_VkPhysicalDeviceLayeredApiKHR);
    settings.stream() << ",\n";
    dump_json_value<const char*>(object.deviceName, NULL, settings, "char[VK_MAX_PHYSICAL_DEVICE_NAME_SIZE]", "deviceName", false, false, indents + 1, dump_json_cstring);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceLayeredApiPropertiesListKHR(const VkPhysicalDeviceLayeredApiPropertiesListKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.layeredApiCount, NULL, settings, "uint32_t", "layeredApiCount", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << ",\n";
    dump_json_array<const VkPhysicalDeviceLayeredApiPropertiesKHR>(object.pLayeredApis, object.layeredApiCount, settings, "VkPhysicalDeviceLayeredApiPropertiesKHR*", "VkPhysicalDeviceLayeredApiPropertiesKHR", "pLayeredApis", true, false, indents + 1, dump_json_VkPhysicalDeviceLayeredApiPropertiesKHR); // JQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceLayeredApiVulkanPropertiesKHR(const VkPhysicalDeviceLayeredApiVulkanPropertiesKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkPhysicalDeviceProperties2>(object.properties, NULL, settings, "VkPhysicalDeviceProperties2", "properties", true, false, indents + 1, dump_json_VkPhysicalDeviceProperties2);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderAtomicFloat16VectorFeaturesNV(const VkPhysicalDeviceShaderAtomicFloat16VectorFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderFloat16VectorAtomics, NULL, settings, "VkBool32", "shaderFloat16VectorAtomics", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceShaderReplicatedCompositesFeaturesEXT(const VkPhysicalDeviceShaderReplicatedCompositesFeaturesEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.shaderReplicatedComposites, NULL, settings, "VkBool32", "shaderReplicatedComposites", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceRayTracingValidationFeaturesNV(const VkPhysicalDeviceRayTracingValidationFeaturesNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.rayTracingValidation, NULL, settings, "VkBool32", "rayTracingValidation", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImageAlignmentControlFeaturesMESA(const VkPhysicalDeviceImageAlignmentControlFeaturesMESA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const VkBool32>(object.imageAlignmentControl, NULL, settings, "VkBool32", "imageAlignmentControl", false, false, indents + 1, dump_json_VkBool32);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPhysicalDeviceImageAlignmentControlPropertiesMESA(const VkPhysicalDeviceImageAlignmentControlPropertiesMESA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.supportedImageAlignmentMask, NULL, settings, "uint32_t", "supportedImageAlignmentMask", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkImageAlignmentControlCreateInfoMESA(const VkImageAlignmentControlCreateInfoMESA& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkStructureType>(object.sType, NULL, settings, "VkStructureType", "sType", false, false, indents + 1, dump_json_VkStructureType);
    settings.stream() << ",\n";
    if(object.pNext != nullptr){
        dump_json_pNext_trampoline(object.pNext, settings, indents + 1);
    } else {
        dump_json_value<const void*>(object.pNext, object.pNext, settings, "const void*", "pNext", false, false, indents + 1, dump_json_void);
    }
    settings.stream() << ",\n";
    dump_json_value<const uint32_t>(object.maximumRequestedAlignment, NULL, settings, "uint32_t", "maximumRequestedAlignment", false, false, indents + 1, dump_json_uint32_t);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}

//========================== Union Implementations ==========================//

void dump_json_VkClearColorValue(const VkClearColorValue& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_array<const float>(object.float32, 4, settings, "float[4]", "float", "float32", false, false, indents + 2, dump_json_float); // OQA
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_array<const int32_t>(object.int32, 4, settings, "int32_t[4]", "int32_t", "int32", false, false, indents + 2, dump_json_int32_t); // OQA
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_array<const uint32_t>(object.uint32, 4, settings, "uint32_t[4]", "uint32_t", "uint32", false, false, indents + 2, dump_json_uint32_t); // OQA
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkClearValue(const VkClearValue& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkClearColorValue>(object.color, NULL, settings, "VkClearColorValue", "color", false, true, indents + 2, dump_json_VkClearColorValue);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const VkClearDepthStencilValue>(object.depthStencil, NULL, settings, "VkClearDepthStencilValue", "depthStencil", true, false, indents + 2, dump_json_VkClearDepthStencilValue);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPerformanceCounterResultKHR(const VkPerformanceCounterResultKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const int32_t>(object.int32, NULL, settings, "int32_t", "int32", false, false, indents + 2, dump_json_int32_t);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const int64_t>(object.int64, NULL, settings, "int64_t", "int64", false, false, indents + 2, dump_json_int64_t);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const uint32_t>(object.uint32, NULL, settings, "uint32_t", "uint32", false, false, indents + 2, dump_json_uint32_t);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const uint64_t>(object.uint64, NULL, settings, "uint64_t", "uint64", false, false, indents + 2, dump_json_uint64_t);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const float>(object.float32, NULL, settings, "float", "float32", false, false, indents + 2, dump_json_float);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const double>(object.float64, NULL, settings, "double", "float64", false, false, indents + 2, dump_json_double);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_VkDeviceOrHostAddressConstAMDX(const VkDeviceOrHostAddressConstAMDX& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceAddress>(object.deviceAddress, NULL, settings, "VkDeviceAddress", "deviceAddress", false, false, indents + 2, dump_json_VkDeviceAddress);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const void*>(object.hostAddress, NULL, settings, "const void*", "hostAddress", false, false, indents + 2, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
#endif // VK_ENABLE_BETA_EXTENSIONS
void dump_json_VkDeviceOrHostAddressKHR(const VkDeviceOrHostAddressKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceAddress>(object.deviceAddress, NULL, settings, "VkDeviceAddress", "deviceAddress", false, false, indents + 2, dump_json_VkDeviceAddress);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const void*>(object.hostAddress, NULL, settings, "void*", "hostAddress", false, false, indents + 2, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDeviceOrHostAddressConstKHR(const VkDeviceOrHostAddressConstKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkDeviceAddress>(object.deviceAddress, NULL, settings, "VkDeviceAddress", "deviceAddress", false, false, indents + 2, dump_json_VkDeviceAddress);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const void*>(object.hostAddress, NULL, settings, "const void*", "hostAddress", false, false, indents + 2, dump_json_void);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureGeometryDataKHR(const VkAccelerationStructureGeometryDataKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkAccelerationStructureGeometryTrianglesDataKHR>(object.triangles, NULL, settings, "VkAccelerationStructureGeometryTrianglesDataKHR", "triangles", true, false, indents + 2, dump_json_VkAccelerationStructureGeometryTrianglesDataKHR);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const VkAccelerationStructureGeometryAabbsDataKHR>(object.aabbs, NULL, settings, "VkAccelerationStructureGeometryAabbsDataKHR", "aabbs", true, false, indents + 2, dump_json_VkAccelerationStructureGeometryAabbsDataKHR);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const VkAccelerationStructureGeometryInstancesDataKHR>(object.instances, NULL, settings, "VkAccelerationStructureGeometryInstancesDataKHR", "instances", true, false, indents + 2, dump_json_VkAccelerationStructureGeometryInstancesDataKHR);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPerformanceValueDataINTEL(const VkPerformanceValueDataINTEL& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const uint32_t>(object.value32, NULL, settings, "uint32_t", "value32", false, false, indents + 2, dump_json_uint32_t);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const uint64_t>(object.value64, NULL, settings, "uint64_t", "value64", false, false, indents + 2, dump_json_uint64_t);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const float>(object.valueFloat, NULL, settings, "float", "valueFloat", false, false, indents + 2, dump_json_float);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const VkBool32>(object.valueBool, NULL, settings, "VkBool32", "valueBool", false, false, indents + 2, dump_json_VkBool32);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const char*>(object.valueString, NULL, settings, "const char*", "valueString", false, false, indents + 2, dump_json_cstring);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkPipelineExecutableStatisticValueKHR(const VkPipelineExecutableStatisticValueKHR& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkBool32>(object.b32, NULL, settings, "VkBool32", "b32", false, false, indents + 2, dump_json_VkBool32);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const int64_t>(object.i64, NULL, settings, "int64_t", "i64", false, false, indents + 2, dump_json_int64_t);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const uint64_t>(object.u64, NULL, settings, "uint64_t", "u64", false, false, indents + 2, dump_json_uint64_t);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const double>(object.f64, NULL, settings, "double", "f64", false, false, indents + 2, dump_json_double);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkDescriptorDataEXT(const VkDescriptorDataEXT& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    if(ApiDumpInstance::current().getDescriptorType() == VK_DESCRIPTOR_TYPE_SAMPLER)
    dump_json_pointer<const VkSampler>(object.pSampler, settings, "const VkSampler*", "pSampler", false, false, indents + 2, dump_json_VkSampler);
    if(ApiDumpInstance::current().getDescriptorType() == VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER)
    dump_json_pointer<const VkDescriptorImageInfo>(object.pCombinedImageSampler, settings, "const VkDescriptorImageInfo*", "pCombinedImageSampler", true, false, indents + 2, dump_json_VkDescriptorImageInfo);
    if(ApiDumpInstance::current().getDescriptorType() == VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT)
    dump_json_pointer<const VkDescriptorImageInfo>(object.pInputAttachmentImage, settings, "const VkDescriptorImageInfo*", "pInputAttachmentImage", true, false, indents + 2, dump_json_VkDescriptorImageInfo);
    if(ApiDumpInstance::current().getDescriptorType() == VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE)
    dump_json_pointer<const VkDescriptorImageInfo>(object.pSampledImage, settings, "const VkDescriptorImageInfo*", "pSampledImage", true, false, indents + 2, dump_json_VkDescriptorImageInfo);
    if(ApiDumpInstance::current().getDescriptorType() == VK_DESCRIPTOR_TYPE_STORAGE_IMAGE)
    dump_json_pointer<const VkDescriptorImageInfo>(object.pStorageImage, settings, "const VkDescriptorImageInfo*", "pStorageImage", true, false, indents + 2, dump_json_VkDescriptorImageInfo);
    if(ApiDumpInstance::current().getDescriptorType() == VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER)
    dump_json_pointer<const VkDescriptorAddressInfoEXT>(object.pUniformTexelBuffer, settings, "const VkDescriptorAddressInfoEXT*", "pUniformTexelBuffer", true, false, indents + 2, dump_json_VkDescriptorAddressInfoEXT);
    if(ApiDumpInstance::current().getDescriptorType() == VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER)
    dump_json_pointer<const VkDescriptorAddressInfoEXT>(object.pStorageTexelBuffer, settings, "const VkDescriptorAddressInfoEXT*", "pStorageTexelBuffer", true, false, indents + 2, dump_json_VkDescriptorAddressInfoEXT);
    if(ApiDumpInstance::current().getDescriptorType() == VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER)
    dump_json_pointer<const VkDescriptorAddressInfoEXT>(object.pUniformBuffer, settings, "const VkDescriptorAddressInfoEXT*", "pUniformBuffer", true, false, indents + 2, dump_json_VkDescriptorAddressInfoEXT);
    if(ApiDumpInstance::current().getDescriptorType() == VK_DESCRIPTOR_TYPE_STORAGE_BUFFER)
    dump_json_pointer<const VkDescriptorAddressInfoEXT>(object.pStorageBuffer, settings, "const VkDescriptorAddressInfoEXT*", "pStorageBuffer", true, false, indents + 2, dump_json_VkDescriptorAddressInfoEXT);
    if(ApiDumpInstance::current().getDescriptorType() == VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR)
    dump_json_value<const VkDeviceAddress>(object.accelerationStructure, NULL, settings, "VkDeviceAddress", "accelerationStructure", false, false, indents + 2, dump_json_VkDeviceAddress);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}
void dump_json_VkAccelerationStructureMotionInstanceDataNV(const VkAccelerationStructureMotionInstanceDataNV& object, const ApiDumpSettings& settings, int indents)
{
    settings.stream() << settings.indentation(indents) << "[\n";
    dump_json_value<const VkAccelerationStructureInstanceKHR>(object.staticInstance, NULL, settings, "VkAccelerationStructureInstanceKHR", "staticInstance", true, false, indents + 2, dump_json_VkAccelerationStructureInstanceKHR);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const VkAccelerationStructureMatrixMotionInstanceNV>(object.matrixMotionInstance, NULL, settings, "VkAccelerationStructureMatrixMotionInstanceNV", "matrixMotionInstance", true, false, indents + 2, dump_json_VkAccelerationStructureMatrixMotionInstanceNV);
    settings.stream() << ",\n"; // Only need commas when more than one field is printed
    dump_json_value<const VkAccelerationStructureSRTMotionInstanceNV>(object.srtMotionInstance, NULL, settings, "VkAccelerationStructureSRTMotionInstanceNV", "srtMotionInstance", true, false, indents + 2, dump_json_VkAccelerationStructureSRTMotionInstanceNV);
    settings.stream() << "\n" << settings.indentation(indents) << "]";
}

//======================== pNext Chain Implementation =======================//

void dump_json_pNext_trampoline(const void* object, const ApiDumpSettings& settings, int indents)
{
    switch((int64_t) (static_cast<const VkBaseInStructure*>(object)->sType)) {
    case 44:
        dump_json_pNext<const VkBufferMemoryBarrier>(static_cast<const VkBufferMemoryBarrier*>(object), settings, "VkBufferMemoryBarrier", indents, dump_json_VkBufferMemoryBarrier);
        break;
    case 45:
        dump_json_pNext<const VkImageMemoryBarrier>(static_cast<const VkImageMemoryBarrier*>(object), settings, "VkImageMemoryBarrier", indents, dump_json_VkImageMemoryBarrier);
        break;
    case 46:
        dump_json_pNext<const VkMemoryBarrier>(static_cast<const VkMemoryBarrier*>(object), settings, "VkMemoryBarrier", indents, dump_json_VkMemoryBarrier);
        break;
    case 0:
        dump_json_pNext<const VkApplicationInfo>(static_cast<const VkApplicationInfo*>(object), settings, "VkApplicationInfo", indents, dump_json_VkApplicationInfo);
        break;
    case 1:
        dump_json_pNext<const VkInstanceCreateInfo>(static_cast<const VkInstanceCreateInfo*>(object), settings, "VkInstanceCreateInfo", indents, dump_json_VkInstanceCreateInfo);
        break;
    case 2:
        dump_json_pNext<const VkDeviceQueueCreateInfo>(static_cast<const VkDeviceQueueCreateInfo*>(object), settings, "VkDeviceQueueCreateInfo", indents, dump_json_VkDeviceQueueCreateInfo);
        break;
    case 3:
        dump_json_pNext<const VkDeviceCreateInfo>(static_cast<const VkDeviceCreateInfo*>(object), settings, "VkDeviceCreateInfo", indents, dump_json_VkDeviceCreateInfo);
        break;
    case 4:
        dump_json_pNext<const VkSubmitInfo>(static_cast<const VkSubmitInfo*>(object), settings, "VkSubmitInfo", indents, dump_json_VkSubmitInfo);
        break;
    case 6:
        dump_json_pNext<const VkMappedMemoryRange>(static_cast<const VkMappedMemoryRange*>(object), settings, "VkMappedMemoryRange", indents, dump_json_VkMappedMemoryRange);
        break;
    case 5:
        dump_json_pNext<const VkMemoryAllocateInfo>(static_cast<const VkMemoryAllocateInfo*>(object), settings, "VkMemoryAllocateInfo", indents, dump_json_VkMemoryAllocateInfo);
        break;
    case 7:
        dump_json_pNext<const VkBindSparseInfo>(static_cast<const VkBindSparseInfo*>(object), settings, "VkBindSparseInfo", indents, dump_json_VkBindSparseInfo);
        break;
    case 8:
        dump_json_pNext<const VkFenceCreateInfo>(static_cast<const VkFenceCreateInfo*>(object), settings, "VkFenceCreateInfo", indents, dump_json_VkFenceCreateInfo);
        break;
    case 9:
        dump_json_pNext<const VkSemaphoreCreateInfo>(static_cast<const VkSemaphoreCreateInfo*>(object), settings, "VkSemaphoreCreateInfo", indents, dump_json_VkSemaphoreCreateInfo);
        break;
    case 10:
        dump_json_pNext<const VkEventCreateInfo>(static_cast<const VkEventCreateInfo*>(object), settings, "VkEventCreateInfo", indents, dump_json_VkEventCreateInfo);
        break;
    case 11:
        dump_json_pNext<const VkQueryPoolCreateInfo>(static_cast<const VkQueryPoolCreateInfo*>(object), settings, "VkQueryPoolCreateInfo", indents, dump_json_VkQueryPoolCreateInfo);
        break;
    case 12:
        dump_json_pNext<const VkBufferCreateInfo>(static_cast<const VkBufferCreateInfo*>(object), settings, "VkBufferCreateInfo", indents, dump_json_VkBufferCreateInfo);
        break;
    case 13:
        dump_json_pNext<const VkBufferViewCreateInfo>(static_cast<const VkBufferViewCreateInfo*>(object), settings, "VkBufferViewCreateInfo", indents, dump_json_VkBufferViewCreateInfo);
        break;
    case 14:
        dump_json_pNext<const VkImageCreateInfo>(static_cast<const VkImageCreateInfo*>(object), settings, "VkImageCreateInfo", indents, dump_json_VkImageCreateInfo);
        break;
    case 15:
        dump_json_pNext<const VkImageViewCreateInfo>(static_cast<const VkImageViewCreateInfo*>(object), settings, "VkImageViewCreateInfo", indents, dump_json_VkImageViewCreateInfo);
        break;
    case 16:
        dump_json_pNext<const VkShaderModuleCreateInfo>(static_cast<const VkShaderModuleCreateInfo*>(object), settings, "VkShaderModuleCreateInfo", indents, dump_json_VkShaderModuleCreateInfo);
        break;
    case 17:
        dump_json_pNext<const VkPipelineCacheCreateInfo>(static_cast<const VkPipelineCacheCreateInfo*>(object), settings, "VkPipelineCacheCreateInfo", indents, dump_json_VkPipelineCacheCreateInfo);
        break;
    case 18:
        dump_json_pNext<const VkPipelineShaderStageCreateInfo>(static_cast<const VkPipelineShaderStageCreateInfo*>(object), settings, "VkPipelineShaderStageCreateInfo", indents, dump_json_VkPipelineShaderStageCreateInfo);
        break;
    case 29:
        dump_json_pNext<const VkComputePipelineCreateInfo>(static_cast<const VkComputePipelineCreateInfo*>(object), settings, "VkComputePipelineCreateInfo", indents, dump_json_VkComputePipelineCreateInfo);
        break;
    case 19:
        dump_json_pNext<const VkPipelineVertexInputStateCreateInfo>(static_cast<const VkPipelineVertexInputStateCreateInfo*>(object), settings, "VkPipelineVertexInputStateCreateInfo", indents, dump_json_VkPipelineVertexInputStateCreateInfo);
        break;
    case 20:
        dump_json_pNext<const VkPipelineInputAssemblyStateCreateInfo>(static_cast<const VkPipelineInputAssemblyStateCreateInfo*>(object), settings, "VkPipelineInputAssemblyStateCreateInfo", indents, dump_json_VkPipelineInputAssemblyStateCreateInfo);
        break;
    case 21:
        dump_json_pNext<const VkPipelineTessellationStateCreateInfo>(static_cast<const VkPipelineTessellationStateCreateInfo*>(object), settings, "VkPipelineTessellationStateCreateInfo", indents, dump_json_VkPipelineTessellationStateCreateInfo);
        break;
    case 22:
        dump_json_pNext<const VkPipelineViewportStateCreateInfo>(static_cast<const VkPipelineViewportStateCreateInfo*>(object), settings, "VkPipelineViewportStateCreateInfo", indents, dump_json_VkPipelineViewportStateCreateInfo);
        break;
    case 23:
        dump_json_pNext<const VkPipelineRasterizationStateCreateInfo>(static_cast<const VkPipelineRasterizationStateCreateInfo*>(object), settings, "VkPipelineRasterizationStateCreateInfo", indents, dump_json_VkPipelineRasterizationStateCreateInfo);
        break;
    case 24:
        dump_json_pNext<const VkPipelineMultisampleStateCreateInfo>(static_cast<const VkPipelineMultisampleStateCreateInfo*>(object), settings, "VkPipelineMultisampleStateCreateInfo", indents, dump_json_VkPipelineMultisampleStateCreateInfo);
        break;
    case 25:
        dump_json_pNext<const VkPipelineDepthStencilStateCreateInfo>(static_cast<const VkPipelineDepthStencilStateCreateInfo*>(object), settings, "VkPipelineDepthStencilStateCreateInfo", indents, dump_json_VkPipelineDepthStencilStateCreateInfo);
        break;
    case 26:
        dump_json_pNext<const VkPipelineColorBlendStateCreateInfo>(static_cast<const VkPipelineColorBlendStateCreateInfo*>(object), settings, "VkPipelineColorBlendStateCreateInfo", indents, dump_json_VkPipelineColorBlendStateCreateInfo);
        break;
    case 27:
        dump_json_pNext<const VkPipelineDynamicStateCreateInfo>(static_cast<const VkPipelineDynamicStateCreateInfo*>(object), settings, "VkPipelineDynamicStateCreateInfo", indents, dump_json_VkPipelineDynamicStateCreateInfo);
        break;
    case 28:
        dump_json_pNext<const VkGraphicsPipelineCreateInfo>(static_cast<const VkGraphicsPipelineCreateInfo*>(object), settings, "VkGraphicsPipelineCreateInfo", indents, dump_json_VkGraphicsPipelineCreateInfo);
        break;
    case 30:
        dump_json_pNext<const VkPipelineLayoutCreateInfo>(static_cast<const VkPipelineLayoutCreateInfo*>(object), settings, "VkPipelineLayoutCreateInfo", indents, dump_json_VkPipelineLayoutCreateInfo);
        break;
    case 31:
        dump_json_pNext<const VkSamplerCreateInfo>(static_cast<const VkSamplerCreateInfo*>(object), settings, "VkSamplerCreateInfo", indents, dump_json_VkSamplerCreateInfo);
        break;
    case 36:
        dump_json_pNext<const VkCopyDescriptorSet>(static_cast<const VkCopyDescriptorSet*>(object), settings, "VkCopyDescriptorSet", indents, dump_json_VkCopyDescriptorSet);
        break;
    case 33:
        dump_json_pNext<const VkDescriptorPoolCreateInfo>(static_cast<const VkDescriptorPoolCreateInfo*>(object), settings, "VkDescriptorPoolCreateInfo", indents, dump_json_VkDescriptorPoolCreateInfo);
        break;
    case 34:
        dump_json_pNext<const VkDescriptorSetAllocateInfo>(static_cast<const VkDescriptorSetAllocateInfo*>(object), settings, "VkDescriptorSetAllocateInfo", indents, dump_json_VkDescriptorSetAllocateInfo);
        break;
    case 32:
        dump_json_pNext<const VkDescriptorSetLayoutCreateInfo>(static_cast<const VkDescriptorSetLayoutCreateInfo*>(object), settings, "VkDescriptorSetLayoutCreateInfo", indents, dump_json_VkDescriptorSetLayoutCreateInfo);
        break;
    case 35:
        dump_json_pNext<const VkWriteDescriptorSet>(static_cast<const VkWriteDescriptorSet*>(object), settings, "VkWriteDescriptorSet", indents, dump_json_VkWriteDescriptorSet);
        break;
    case 37:
        dump_json_pNext<const VkFramebufferCreateInfo>(static_cast<const VkFramebufferCreateInfo*>(object), settings, "VkFramebufferCreateInfo", indents, dump_json_VkFramebufferCreateInfo);
        break;
    case 38:
        dump_json_pNext<const VkRenderPassCreateInfo>(static_cast<const VkRenderPassCreateInfo*>(object), settings, "VkRenderPassCreateInfo", indents, dump_json_VkRenderPassCreateInfo);
        break;
    case 39:
        dump_json_pNext<const VkCommandPoolCreateInfo>(static_cast<const VkCommandPoolCreateInfo*>(object), settings, "VkCommandPoolCreateInfo", indents, dump_json_VkCommandPoolCreateInfo);
        break;
    case 40:
        dump_json_pNext<const VkCommandBufferAllocateInfo>(static_cast<const VkCommandBufferAllocateInfo*>(object), settings, "VkCommandBufferAllocateInfo", indents, dump_json_VkCommandBufferAllocateInfo);
        break;
    case 41:
        dump_json_pNext<const VkCommandBufferInheritanceInfo>(static_cast<const VkCommandBufferInheritanceInfo*>(object), settings, "VkCommandBufferInheritanceInfo", indents, dump_json_VkCommandBufferInheritanceInfo);
        break;
    case 42:
        dump_json_pNext<const VkCommandBufferBeginInfo>(static_cast<const VkCommandBufferBeginInfo*>(object), settings, "VkCommandBufferBeginInfo", indents, dump_json_VkCommandBufferBeginInfo);
        break;
    case 43:
        dump_json_pNext<const VkRenderPassBeginInfo>(static_cast<const VkRenderPassBeginInfo*>(object), settings, "VkRenderPassBeginInfo", indents, dump_json_VkRenderPassBeginInfo);
        break;
    case 1000094000:
        dump_json_pNext<const VkPhysicalDeviceSubgroupProperties>(static_cast<const VkPhysicalDeviceSubgroupProperties*>(object), settings, "VkPhysicalDeviceSubgroupProperties", indents, dump_json_VkPhysicalDeviceSubgroupProperties);
        break;
    case 1000157000:
        dump_json_pNext<const VkBindBufferMemoryInfo>(static_cast<const VkBindBufferMemoryInfo*>(object), settings, "VkBindBufferMemoryInfo", indents, dump_json_VkBindBufferMemoryInfo);
        break;
    case 1000157001:
        dump_json_pNext<const VkBindImageMemoryInfo>(static_cast<const VkBindImageMemoryInfo*>(object), settings, "VkBindImageMemoryInfo", indents, dump_json_VkBindImageMemoryInfo);
        break;
    case 1000083000:
        dump_json_pNext<const VkPhysicalDevice16BitStorageFeatures>(static_cast<const VkPhysicalDevice16BitStorageFeatures*>(object), settings, "VkPhysicalDevice16BitStorageFeatures", indents, dump_json_VkPhysicalDevice16BitStorageFeatures);
        break;
    case 1000127000:
        dump_json_pNext<const VkMemoryDedicatedRequirements>(static_cast<const VkMemoryDedicatedRequirements*>(object), settings, "VkMemoryDedicatedRequirements", indents, dump_json_VkMemoryDedicatedRequirements);
        break;
    case 1000127001:
        dump_json_pNext<const VkMemoryDedicatedAllocateInfo>(static_cast<const VkMemoryDedicatedAllocateInfo*>(object), settings, "VkMemoryDedicatedAllocateInfo", indents, dump_json_VkMemoryDedicatedAllocateInfo);
        break;
    case 1000060000:
        dump_json_pNext<const VkMemoryAllocateFlagsInfo>(static_cast<const VkMemoryAllocateFlagsInfo*>(object), settings, "VkMemoryAllocateFlagsInfo", indents, dump_json_VkMemoryAllocateFlagsInfo);
        break;
    case 1000060003:
        dump_json_pNext<const VkDeviceGroupRenderPassBeginInfo>(static_cast<const VkDeviceGroupRenderPassBeginInfo*>(object), settings, "VkDeviceGroupRenderPassBeginInfo", indents, dump_json_VkDeviceGroupRenderPassBeginInfo);
        break;
    case 1000060004:
        dump_json_pNext<const VkDeviceGroupCommandBufferBeginInfo>(static_cast<const VkDeviceGroupCommandBufferBeginInfo*>(object), settings, "VkDeviceGroupCommandBufferBeginInfo", indents, dump_json_VkDeviceGroupCommandBufferBeginInfo);
        break;
    case 1000060005:
        dump_json_pNext<const VkDeviceGroupSubmitInfo>(static_cast<const VkDeviceGroupSubmitInfo*>(object), settings, "VkDeviceGroupSubmitInfo", indents, dump_json_VkDeviceGroupSubmitInfo);
        break;
    case 1000060006:
        dump_json_pNext<const VkDeviceGroupBindSparseInfo>(static_cast<const VkDeviceGroupBindSparseInfo*>(object), settings, "VkDeviceGroupBindSparseInfo", indents, dump_json_VkDeviceGroupBindSparseInfo);
        break;
    case 1000060013:
        dump_json_pNext<const VkBindBufferMemoryDeviceGroupInfo>(static_cast<const VkBindBufferMemoryDeviceGroupInfo*>(object), settings, "VkBindBufferMemoryDeviceGroupInfo", indents, dump_json_VkBindBufferMemoryDeviceGroupInfo);
        break;
    case 1000060014:
        dump_json_pNext<const VkBindImageMemoryDeviceGroupInfo>(static_cast<const VkBindImageMemoryDeviceGroupInfo*>(object), settings, "VkBindImageMemoryDeviceGroupInfo", indents, dump_json_VkBindImageMemoryDeviceGroupInfo);
        break;
    case 1000070000:
        dump_json_pNext<const VkPhysicalDeviceGroupProperties>(static_cast<const VkPhysicalDeviceGroupProperties*>(object), settings, "VkPhysicalDeviceGroupProperties", indents, dump_json_VkPhysicalDeviceGroupProperties);
        break;
    case 1000070001:
        dump_json_pNext<const VkDeviceGroupDeviceCreateInfo>(static_cast<const VkDeviceGroupDeviceCreateInfo*>(object), settings, "VkDeviceGroupDeviceCreateInfo", indents, dump_json_VkDeviceGroupDeviceCreateInfo);
        break;
    case 1000146000:
        dump_json_pNext<const VkBufferMemoryRequirementsInfo2>(static_cast<const VkBufferMemoryRequirementsInfo2*>(object), settings, "VkBufferMemoryRequirementsInfo2", indents, dump_json_VkBufferMemoryRequirementsInfo2);
        break;
    case 1000146001:
        dump_json_pNext<const VkImageMemoryRequirementsInfo2>(static_cast<const VkImageMemoryRequirementsInfo2*>(object), settings, "VkImageMemoryRequirementsInfo2", indents, dump_json_VkImageMemoryRequirementsInfo2);
        break;
    case 1000146002:
        dump_json_pNext<const VkImageSparseMemoryRequirementsInfo2>(static_cast<const VkImageSparseMemoryRequirementsInfo2*>(object), settings, "VkImageSparseMemoryRequirementsInfo2", indents, dump_json_VkImageSparseMemoryRequirementsInfo2);
        break;
    case 1000146003:
        dump_json_pNext<const VkMemoryRequirements2>(static_cast<const VkMemoryRequirements2*>(object), settings, "VkMemoryRequirements2", indents, dump_json_VkMemoryRequirements2);
        break;
    case 1000146004:
        dump_json_pNext<const VkSparseImageMemoryRequirements2>(static_cast<const VkSparseImageMemoryRequirements2*>(object), settings, "VkSparseImageMemoryRequirements2", indents, dump_json_VkSparseImageMemoryRequirements2);
        break;
    case 1000059000:
        dump_json_pNext<const VkPhysicalDeviceFeatures2>(static_cast<const VkPhysicalDeviceFeatures2*>(object), settings, "VkPhysicalDeviceFeatures2", indents, dump_json_VkPhysicalDeviceFeatures2);
        break;
    case 1000059001:
        dump_json_pNext<const VkPhysicalDeviceProperties2>(static_cast<const VkPhysicalDeviceProperties2*>(object), settings, "VkPhysicalDeviceProperties2", indents, dump_json_VkPhysicalDeviceProperties2);
        break;
    case 1000059002:
        dump_json_pNext<const VkFormatProperties2>(static_cast<const VkFormatProperties2*>(object), settings, "VkFormatProperties2", indents, dump_json_VkFormatProperties2);
        break;
    case 1000059003:
        dump_json_pNext<const VkImageFormatProperties2>(static_cast<const VkImageFormatProperties2*>(object), settings, "VkImageFormatProperties2", indents, dump_json_VkImageFormatProperties2);
        break;
    case 1000059004:
        dump_json_pNext<const VkPhysicalDeviceImageFormatInfo2>(static_cast<const VkPhysicalDeviceImageFormatInfo2*>(object), settings, "VkPhysicalDeviceImageFormatInfo2", indents, dump_json_VkPhysicalDeviceImageFormatInfo2);
        break;
    case 1000059005:
        dump_json_pNext<const VkQueueFamilyProperties2>(static_cast<const VkQueueFamilyProperties2*>(object), settings, "VkQueueFamilyProperties2", indents, dump_json_VkQueueFamilyProperties2);
        break;
    case 1000059006:
        dump_json_pNext<const VkPhysicalDeviceMemoryProperties2>(static_cast<const VkPhysicalDeviceMemoryProperties2*>(object), settings, "VkPhysicalDeviceMemoryProperties2", indents, dump_json_VkPhysicalDeviceMemoryProperties2);
        break;
    case 1000059007:
        dump_json_pNext<const VkSparseImageFormatProperties2>(static_cast<const VkSparseImageFormatProperties2*>(object), settings, "VkSparseImageFormatProperties2", indents, dump_json_VkSparseImageFormatProperties2);
        break;
    case 1000059008:
        dump_json_pNext<const VkPhysicalDeviceSparseImageFormatInfo2>(static_cast<const VkPhysicalDeviceSparseImageFormatInfo2*>(object), settings, "VkPhysicalDeviceSparseImageFormatInfo2", indents, dump_json_VkPhysicalDeviceSparseImageFormatInfo2);
        break;
    case 1000117000:
        dump_json_pNext<const VkPhysicalDevicePointClippingProperties>(static_cast<const VkPhysicalDevicePointClippingProperties*>(object), settings, "VkPhysicalDevicePointClippingProperties", indents, dump_json_VkPhysicalDevicePointClippingProperties);
        break;
    case 1000117001:
        dump_json_pNext<const VkRenderPassInputAttachmentAspectCreateInfo>(static_cast<const VkRenderPassInputAttachmentAspectCreateInfo*>(object), settings, "VkRenderPassInputAttachmentAspectCreateInfo", indents, dump_json_VkRenderPassInputAttachmentAspectCreateInfo);
        break;
    case 1000117002:
        dump_json_pNext<const VkImageViewUsageCreateInfo>(static_cast<const VkImageViewUsageCreateInfo*>(object), settings, "VkImageViewUsageCreateInfo", indents, dump_json_VkImageViewUsageCreateInfo);
        break;
    case 1000117003:
        dump_json_pNext<const VkPipelineTessellationDomainOriginStateCreateInfo>(static_cast<const VkPipelineTessellationDomainOriginStateCreateInfo*>(object), settings, "VkPipelineTessellationDomainOriginStateCreateInfo", indents, dump_json_VkPipelineTessellationDomainOriginStateCreateInfo);
        break;
    case 1000053000:
        dump_json_pNext<const VkRenderPassMultiviewCreateInfo>(static_cast<const VkRenderPassMultiviewCreateInfo*>(object), settings, "VkRenderPassMultiviewCreateInfo", indents, dump_json_VkRenderPassMultiviewCreateInfo);
        break;
    case 1000053001:
        dump_json_pNext<const VkPhysicalDeviceMultiviewFeatures>(static_cast<const VkPhysicalDeviceMultiviewFeatures*>(object), settings, "VkPhysicalDeviceMultiviewFeatures", indents, dump_json_VkPhysicalDeviceMultiviewFeatures);
        break;
    case 1000053002:
        dump_json_pNext<const VkPhysicalDeviceMultiviewProperties>(static_cast<const VkPhysicalDeviceMultiviewProperties*>(object), settings, "VkPhysicalDeviceMultiviewProperties", indents, dump_json_VkPhysicalDeviceMultiviewProperties);
        break;
    case 1000120000:
        dump_json_pNext<const VkPhysicalDeviceVariablePointersFeatures>(static_cast<const VkPhysicalDeviceVariablePointersFeatures*>(object), settings, "VkPhysicalDeviceVariablePointersFeatures", indents, dump_json_VkPhysicalDeviceVariablePointersFeatures);
        break;
    case 1000145001:
        dump_json_pNext<const VkPhysicalDeviceProtectedMemoryFeatures>(static_cast<const VkPhysicalDeviceProtectedMemoryFeatures*>(object), settings, "VkPhysicalDeviceProtectedMemoryFeatures", indents, dump_json_VkPhysicalDeviceProtectedMemoryFeatures);
        break;
    case 1000145002:
        dump_json_pNext<const VkPhysicalDeviceProtectedMemoryProperties>(static_cast<const VkPhysicalDeviceProtectedMemoryProperties*>(object), settings, "VkPhysicalDeviceProtectedMemoryProperties", indents, dump_json_VkPhysicalDeviceProtectedMemoryProperties);
        break;
    case 1000145003:
        dump_json_pNext<const VkDeviceQueueInfo2>(static_cast<const VkDeviceQueueInfo2*>(object), settings, "VkDeviceQueueInfo2", indents, dump_json_VkDeviceQueueInfo2);
        break;
    case 1000145000:
        dump_json_pNext<const VkProtectedSubmitInfo>(static_cast<const VkProtectedSubmitInfo*>(object), settings, "VkProtectedSubmitInfo", indents, dump_json_VkProtectedSubmitInfo);
        break;
    case 1000156000:
        dump_json_pNext<const VkSamplerYcbcrConversionCreateInfo>(static_cast<const VkSamplerYcbcrConversionCreateInfo*>(object), settings, "VkSamplerYcbcrConversionCreateInfo", indents, dump_json_VkSamplerYcbcrConversionCreateInfo);
        break;
    case 1000156001:
        dump_json_pNext<const VkSamplerYcbcrConversionInfo>(static_cast<const VkSamplerYcbcrConversionInfo*>(object), settings, "VkSamplerYcbcrConversionInfo", indents, dump_json_VkSamplerYcbcrConversionInfo);
        break;
    case 1000156002:
        dump_json_pNext<const VkBindImagePlaneMemoryInfo>(static_cast<const VkBindImagePlaneMemoryInfo*>(object), settings, "VkBindImagePlaneMemoryInfo", indents, dump_json_VkBindImagePlaneMemoryInfo);
        break;
    case 1000156003:
        dump_json_pNext<const VkImagePlaneMemoryRequirementsInfo>(static_cast<const VkImagePlaneMemoryRequirementsInfo*>(object), settings, "VkImagePlaneMemoryRequirementsInfo", indents, dump_json_VkImagePlaneMemoryRequirementsInfo);
        break;
    case 1000156004:
        dump_json_pNext<const VkPhysicalDeviceSamplerYcbcrConversionFeatures>(static_cast<const VkPhysicalDeviceSamplerYcbcrConversionFeatures*>(object), settings, "VkPhysicalDeviceSamplerYcbcrConversionFeatures", indents, dump_json_VkPhysicalDeviceSamplerYcbcrConversionFeatures);
        break;
    case 1000156005:
        dump_json_pNext<const VkSamplerYcbcrConversionImageFormatProperties>(static_cast<const VkSamplerYcbcrConversionImageFormatProperties*>(object), settings, "VkSamplerYcbcrConversionImageFormatProperties", indents, dump_json_VkSamplerYcbcrConversionImageFormatProperties);
        break;
    case 1000085000:
        dump_json_pNext<const VkDescriptorUpdateTemplateCreateInfo>(static_cast<const VkDescriptorUpdateTemplateCreateInfo*>(object), settings, "VkDescriptorUpdateTemplateCreateInfo", indents, dump_json_VkDescriptorUpdateTemplateCreateInfo);
        break;
    case 1000071000:
        dump_json_pNext<const VkPhysicalDeviceExternalImageFormatInfo>(static_cast<const VkPhysicalDeviceExternalImageFormatInfo*>(object), settings, "VkPhysicalDeviceExternalImageFormatInfo", indents, dump_json_VkPhysicalDeviceExternalImageFormatInfo);
        break;
    case 1000071001:
        dump_json_pNext<const VkExternalImageFormatProperties>(static_cast<const VkExternalImageFormatProperties*>(object), settings, "VkExternalImageFormatProperties", indents, dump_json_VkExternalImageFormatProperties);
        break;
    case 1000071002:
        dump_json_pNext<const VkPhysicalDeviceExternalBufferInfo>(static_cast<const VkPhysicalDeviceExternalBufferInfo*>(object), settings, "VkPhysicalDeviceExternalBufferInfo", indents, dump_json_VkPhysicalDeviceExternalBufferInfo);
        break;
    case 1000071003:
        dump_json_pNext<const VkExternalBufferProperties>(static_cast<const VkExternalBufferProperties*>(object), settings, "VkExternalBufferProperties", indents, dump_json_VkExternalBufferProperties);
        break;
    case 1000071004:
        dump_json_pNext<const VkPhysicalDeviceIDProperties>(static_cast<const VkPhysicalDeviceIDProperties*>(object), settings, "VkPhysicalDeviceIDProperties", indents, dump_json_VkPhysicalDeviceIDProperties);
        break;
    case 1000072001:
        dump_json_pNext<const VkExternalMemoryImageCreateInfo>(static_cast<const VkExternalMemoryImageCreateInfo*>(object), settings, "VkExternalMemoryImageCreateInfo", indents, dump_json_VkExternalMemoryImageCreateInfo);
        break;
    case 1000072000:
        dump_json_pNext<const VkExternalMemoryBufferCreateInfo>(static_cast<const VkExternalMemoryBufferCreateInfo*>(object), settings, "VkExternalMemoryBufferCreateInfo", indents, dump_json_VkExternalMemoryBufferCreateInfo);
        break;
    case 1000072002:
        dump_json_pNext<const VkExportMemoryAllocateInfo>(static_cast<const VkExportMemoryAllocateInfo*>(object), settings, "VkExportMemoryAllocateInfo", indents, dump_json_VkExportMemoryAllocateInfo);
        break;
    case 1000112000:
        dump_json_pNext<const VkPhysicalDeviceExternalFenceInfo>(static_cast<const VkPhysicalDeviceExternalFenceInfo*>(object), settings, "VkPhysicalDeviceExternalFenceInfo", indents, dump_json_VkPhysicalDeviceExternalFenceInfo);
        break;
    case 1000112001:
        dump_json_pNext<const VkExternalFenceProperties>(static_cast<const VkExternalFenceProperties*>(object), settings, "VkExternalFenceProperties", indents, dump_json_VkExternalFenceProperties);
        break;
    case 1000113000:
        dump_json_pNext<const VkExportFenceCreateInfo>(static_cast<const VkExportFenceCreateInfo*>(object), settings, "VkExportFenceCreateInfo", indents, dump_json_VkExportFenceCreateInfo);
        break;
    case 1000077000:
        dump_json_pNext<const VkExportSemaphoreCreateInfo>(static_cast<const VkExportSemaphoreCreateInfo*>(object), settings, "VkExportSemaphoreCreateInfo", indents, dump_json_VkExportSemaphoreCreateInfo);
        break;
    case 1000076000:
        dump_json_pNext<const VkPhysicalDeviceExternalSemaphoreInfo>(static_cast<const VkPhysicalDeviceExternalSemaphoreInfo*>(object), settings, "VkPhysicalDeviceExternalSemaphoreInfo", indents, dump_json_VkPhysicalDeviceExternalSemaphoreInfo);
        break;
    case 1000076001:
        dump_json_pNext<const VkExternalSemaphoreProperties>(static_cast<const VkExternalSemaphoreProperties*>(object), settings, "VkExternalSemaphoreProperties", indents, dump_json_VkExternalSemaphoreProperties);
        break;
    case 1000168000:
        dump_json_pNext<const VkPhysicalDeviceMaintenance3Properties>(static_cast<const VkPhysicalDeviceMaintenance3Properties*>(object), settings, "VkPhysicalDeviceMaintenance3Properties", indents, dump_json_VkPhysicalDeviceMaintenance3Properties);
        break;
    case 1000168001:
        dump_json_pNext<const VkDescriptorSetLayoutSupport>(static_cast<const VkDescriptorSetLayoutSupport*>(object), settings, "VkDescriptorSetLayoutSupport", indents, dump_json_VkDescriptorSetLayoutSupport);
        break;
    case 1000063000:
        dump_json_pNext<const VkPhysicalDeviceShaderDrawParametersFeatures>(static_cast<const VkPhysicalDeviceShaderDrawParametersFeatures*>(object), settings, "VkPhysicalDeviceShaderDrawParametersFeatures", indents, dump_json_VkPhysicalDeviceShaderDrawParametersFeatures);
        break;
    case 49:
        dump_json_pNext<const VkPhysicalDeviceVulkan11Features>(static_cast<const VkPhysicalDeviceVulkan11Features*>(object), settings, "VkPhysicalDeviceVulkan11Features", indents, dump_json_VkPhysicalDeviceVulkan11Features);
        break;
    case 50:
        dump_json_pNext<const VkPhysicalDeviceVulkan11Properties>(static_cast<const VkPhysicalDeviceVulkan11Properties*>(object), settings, "VkPhysicalDeviceVulkan11Properties", indents, dump_json_VkPhysicalDeviceVulkan11Properties);
        break;
    case 51:
        dump_json_pNext<const VkPhysicalDeviceVulkan12Features>(static_cast<const VkPhysicalDeviceVulkan12Features*>(object), settings, "VkPhysicalDeviceVulkan12Features", indents, dump_json_VkPhysicalDeviceVulkan12Features);
        break;
    case 52:
        dump_json_pNext<const VkPhysicalDeviceVulkan12Properties>(static_cast<const VkPhysicalDeviceVulkan12Properties*>(object), settings, "VkPhysicalDeviceVulkan12Properties", indents, dump_json_VkPhysicalDeviceVulkan12Properties);
        break;
    case 1000147000:
        dump_json_pNext<const VkImageFormatListCreateInfo>(static_cast<const VkImageFormatListCreateInfo*>(object), settings, "VkImageFormatListCreateInfo", indents, dump_json_VkImageFormatListCreateInfo);
        break;
    case 1000109000:
        dump_json_pNext<const VkAttachmentDescription2>(static_cast<const VkAttachmentDescription2*>(object), settings, "VkAttachmentDescription2", indents, dump_json_VkAttachmentDescription2);
        break;
    case 1000109001:
        dump_json_pNext<const VkAttachmentReference2>(static_cast<const VkAttachmentReference2*>(object), settings, "VkAttachmentReference2", indents, dump_json_VkAttachmentReference2);
        break;
    case 1000109002:
        dump_json_pNext<const VkSubpassDescription2>(static_cast<const VkSubpassDescription2*>(object), settings, "VkSubpassDescription2", indents, dump_json_VkSubpassDescription2);
        break;
    case 1000109003:
        dump_json_pNext<const VkSubpassDependency2>(static_cast<const VkSubpassDependency2*>(object), settings, "VkSubpassDependency2", indents, dump_json_VkSubpassDependency2);
        break;
    case 1000109004:
        dump_json_pNext<const VkRenderPassCreateInfo2>(static_cast<const VkRenderPassCreateInfo2*>(object), settings, "VkRenderPassCreateInfo2", indents, dump_json_VkRenderPassCreateInfo2);
        break;
    case 1000109005:
        dump_json_pNext<const VkSubpassBeginInfo>(static_cast<const VkSubpassBeginInfo*>(object), settings, "VkSubpassBeginInfo", indents, dump_json_VkSubpassBeginInfo);
        break;
    case 1000109006:
        dump_json_pNext<const VkSubpassEndInfo>(static_cast<const VkSubpassEndInfo*>(object), settings, "VkSubpassEndInfo", indents, dump_json_VkSubpassEndInfo);
        break;
    case 1000177000:
        dump_json_pNext<const VkPhysicalDevice8BitStorageFeatures>(static_cast<const VkPhysicalDevice8BitStorageFeatures*>(object), settings, "VkPhysicalDevice8BitStorageFeatures", indents, dump_json_VkPhysicalDevice8BitStorageFeatures);
        break;
    case 1000196000:
        dump_json_pNext<const VkPhysicalDeviceDriverProperties>(static_cast<const VkPhysicalDeviceDriverProperties*>(object), settings, "VkPhysicalDeviceDriverProperties", indents, dump_json_VkPhysicalDeviceDriverProperties);
        break;
    case 1000180000:
        dump_json_pNext<const VkPhysicalDeviceShaderAtomicInt64Features>(static_cast<const VkPhysicalDeviceShaderAtomicInt64Features*>(object), settings, "VkPhysicalDeviceShaderAtomicInt64Features", indents, dump_json_VkPhysicalDeviceShaderAtomicInt64Features);
        break;
    case 1000082000:
        dump_json_pNext<const VkPhysicalDeviceShaderFloat16Int8Features>(static_cast<const VkPhysicalDeviceShaderFloat16Int8Features*>(object), settings, "VkPhysicalDeviceShaderFloat16Int8Features", indents, dump_json_VkPhysicalDeviceShaderFloat16Int8Features);
        break;
    case 1000197000:
        dump_json_pNext<const VkPhysicalDeviceFloatControlsProperties>(static_cast<const VkPhysicalDeviceFloatControlsProperties*>(object), settings, "VkPhysicalDeviceFloatControlsProperties", indents, dump_json_VkPhysicalDeviceFloatControlsProperties);
        break;
    case 1000161000:
        dump_json_pNext<const VkDescriptorSetLayoutBindingFlagsCreateInfo>(static_cast<const VkDescriptorSetLayoutBindingFlagsCreateInfo*>(object), settings, "VkDescriptorSetLayoutBindingFlagsCreateInfo", indents, dump_json_VkDescriptorSetLayoutBindingFlagsCreateInfo);
        break;
    case 1000161001:
        dump_json_pNext<const VkPhysicalDeviceDescriptorIndexingFeatures>(static_cast<const VkPhysicalDeviceDescriptorIndexingFeatures*>(object), settings, "VkPhysicalDeviceDescriptorIndexingFeatures", indents, dump_json_VkPhysicalDeviceDescriptorIndexingFeatures);
        break;
    case 1000161002:
        dump_json_pNext<const VkPhysicalDeviceDescriptorIndexingProperties>(static_cast<const VkPhysicalDeviceDescriptorIndexingProperties*>(object), settings, "VkPhysicalDeviceDescriptorIndexingProperties", indents, dump_json_VkPhysicalDeviceDescriptorIndexingProperties);
        break;
    case 1000161003:
        dump_json_pNext<const VkDescriptorSetVariableDescriptorCountAllocateInfo>(static_cast<const VkDescriptorSetVariableDescriptorCountAllocateInfo*>(object), settings, "VkDescriptorSetVariableDescriptorCountAllocateInfo", indents, dump_json_VkDescriptorSetVariableDescriptorCountAllocateInfo);
        break;
    case 1000161004:
        dump_json_pNext<const VkDescriptorSetVariableDescriptorCountLayoutSupport>(static_cast<const VkDescriptorSetVariableDescriptorCountLayoutSupport*>(object), settings, "VkDescriptorSetVariableDescriptorCountLayoutSupport", indents, dump_json_VkDescriptorSetVariableDescriptorCountLayoutSupport);
        break;
    case 1000199001:
        dump_json_pNext<const VkSubpassDescriptionDepthStencilResolve>(static_cast<const VkSubpassDescriptionDepthStencilResolve*>(object), settings, "VkSubpassDescriptionDepthStencilResolve", indents, dump_json_VkSubpassDescriptionDepthStencilResolve);
        break;
    case 1000199000:
        dump_json_pNext<const VkPhysicalDeviceDepthStencilResolveProperties>(static_cast<const VkPhysicalDeviceDepthStencilResolveProperties*>(object), settings, "VkPhysicalDeviceDepthStencilResolveProperties", indents, dump_json_VkPhysicalDeviceDepthStencilResolveProperties);
        break;
    case 1000221000:
        dump_json_pNext<const VkPhysicalDeviceScalarBlockLayoutFeatures>(static_cast<const VkPhysicalDeviceScalarBlockLayoutFeatures*>(object), settings, "VkPhysicalDeviceScalarBlockLayoutFeatures", indents, dump_json_VkPhysicalDeviceScalarBlockLayoutFeatures);
        break;
    case 1000246000:
        dump_json_pNext<const VkImageStencilUsageCreateInfo>(static_cast<const VkImageStencilUsageCreateInfo*>(object), settings, "VkImageStencilUsageCreateInfo", indents, dump_json_VkImageStencilUsageCreateInfo);
        break;
    case 1000130001:
        dump_json_pNext<const VkSamplerReductionModeCreateInfo>(static_cast<const VkSamplerReductionModeCreateInfo*>(object), settings, "VkSamplerReductionModeCreateInfo", indents, dump_json_VkSamplerReductionModeCreateInfo);
        break;
    case 1000130000:
        dump_json_pNext<const VkPhysicalDeviceSamplerFilterMinmaxProperties>(static_cast<const VkPhysicalDeviceSamplerFilterMinmaxProperties*>(object), settings, "VkPhysicalDeviceSamplerFilterMinmaxProperties", indents, dump_json_VkPhysicalDeviceSamplerFilterMinmaxProperties);
        break;
    case 1000211000:
        dump_json_pNext<const VkPhysicalDeviceVulkanMemoryModelFeatures>(static_cast<const VkPhysicalDeviceVulkanMemoryModelFeatures*>(object), settings, "VkPhysicalDeviceVulkanMemoryModelFeatures", indents, dump_json_VkPhysicalDeviceVulkanMemoryModelFeatures);
        break;
    case 1000108000:
        dump_json_pNext<const VkPhysicalDeviceImagelessFramebufferFeatures>(static_cast<const VkPhysicalDeviceImagelessFramebufferFeatures*>(object), settings, "VkPhysicalDeviceImagelessFramebufferFeatures", indents, dump_json_VkPhysicalDeviceImagelessFramebufferFeatures);
        break;
    case 1000108002:
        dump_json_pNext<const VkFramebufferAttachmentImageInfo>(static_cast<const VkFramebufferAttachmentImageInfo*>(object), settings, "VkFramebufferAttachmentImageInfo", indents, dump_json_VkFramebufferAttachmentImageInfo);
        break;
    case 1000108001:
        dump_json_pNext<const VkFramebufferAttachmentsCreateInfo>(static_cast<const VkFramebufferAttachmentsCreateInfo*>(object), settings, "VkFramebufferAttachmentsCreateInfo", indents, dump_json_VkFramebufferAttachmentsCreateInfo);
        break;
    case 1000108003:
        dump_json_pNext<const VkRenderPassAttachmentBeginInfo>(static_cast<const VkRenderPassAttachmentBeginInfo*>(object), settings, "VkRenderPassAttachmentBeginInfo", indents, dump_json_VkRenderPassAttachmentBeginInfo);
        break;
    case 1000253000:
        dump_json_pNext<const VkPhysicalDeviceUniformBufferStandardLayoutFeatures>(static_cast<const VkPhysicalDeviceUniformBufferStandardLayoutFeatures*>(object), settings, "VkPhysicalDeviceUniformBufferStandardLayoutFeatures", indents, dump_json_VkPhysicalDeviceUniformBufferStandardLayoutFeatures);
        break;
    case 1000175000:
        dump_json_pNext<const VkPhysicalDeviceShaderSubgroupExtendedTypesFeatures>(static_cast<const VkPhysicalDeviceShaderSubgroupExtendedTypesFeatures*>(object), settings, "VkPhysicalDeviceShaderSubgroupExtendedTypesFeatures", indents, dump_json_VkPhysicalDeviceShaderSubgroupExtendedTypesFeatures);
        break;
    case 1000241000:
        dump_json_pNext<const VkPhysicalDeviceSeparateDepthStencilLayoutsFeatures>(static_cast<const VkPhysicalDeviceSeparateDepthStencilLayoutsFeatures*>(object), settings, "VkPhysicalDeviceSeparateDepthStencilLayoutsFeatures", indents, dump_json_VkPhysicalDeviceSeparateDepthStencilLayoutsFeatures);
        break;
    case 1000241001:
        dump_json_pNext<const VkAttachmentReferenceStencilLayout>(static_cast<const VkAttachmentReferenceStencilLayout*>(object), settings, "VkAttachmentReferenceStencilLayout", indents, dump_json_VkAttachmentReferenceStencilLayout);
        break;
    case 1000241002:
        dump_json_pNext<const VkAttachmentDescriptionStencilLayout>(static_cast<const VkAttachmentDescriptionStencilLayout*>(object), settings, "VkAttachmentDescriptionStencilLayout", indents, dump_json_VkAttachmentDescriptionStencilLayout);
        break;
    case 1000261000:
        dump_json_pNext<const VkPhysicalDeviceHostQueryResetFeatures>(static_cast<const VkPhysicalDeviceHostQueryResetFeatures*>(object), settings, "VkPhysicalDeviceHostQueryResetFeatures", indents, dump_json_VkPhysicalDeviceHostQueryResetFeatures);
        break;
    case 1000207000:
        dump_json_pNext<const VkPhysicalDeviceTimelineSemaphoreFeatures>(static_cast<const VkPhysicalDeviceTimelineSemaphoreFeatures*>(object), settings, "VkPhysicalDeviceTimelineSemaphoreFeatures", indents, dump_json_VkPhysicalDeviceTimelineSemaphoreFeatures);
        break;
    case 1000207001:
        dump_json_pNext<const VkPhysicalDeviceTimelineSemaphoreProperties>(static_cast<const VkPhysicalDeviceTimelineSemaphoreProperties*>(object), settings, "VkPhysicalDeviceTimelineSemaphoreProperties", indents, dump_json_VkPhysicalDeviceTimelineSemaphoreProperties);
        break;
    case 1000207002:
        dump_json_pNext<const VkSemaphoreTypeCreateInfo>(static_cast<const VkSemaphoreTypeCreateInfo*>(object), settings, "VkSemaphoreTypeCreateInfo", indents, dump_json_VkSemaphoreTypeCreateInfo);
        break;
    case 1000207003:
        dump_json_pNext<const VkTimelineSemaphoreSubmitInfo>(static_cast<const VkTimelineSemaphoreSubmitInfo*>(object), settings, "VkTimelineSemaphoreSubmitInfo", indents, dump_json_VkTimelineSemaphoreSubmitInfo);
        break;
    case 1000207004:
        dump_json_pNext<const VkSemaphoreWaitInfo>(static_cast<const VkSemaphoreWaitInfo*>(object), settings, "VkSemaphoreWaitInfo", indents, dump_json_VkSemaphoreWaitInfo);
        break;
    case 1000207005:
        dump_json_pNext<const VkSemaphoreSignalInfo>(static_cast<const VkSemaphoreSignalInfo*>(object), settings, "VkSemaphoreSignalInfo", indents, dump_json_VkSemaphoreSignalInfo);
        break;
    case 1000257000:
        dump_json_pNext<const VkPhysicalDeviceBufferDeviceAddressFeatures>(static_cast<const VkPhysicalDeviceBufferDeviceAddressFeatures*>(object), settings, "VkPhysicalDeviceBufferDeviceAddressFeatures", indents, dump_json_VkPhysicalDeviceBufferDeviceAddressFeatures);
        break;
    case 1000244001:
        dump_json_pNext<const VkBufferDeviceAddressInfo>(static_cast<const VkBufferDeviceAddressInfo*>(object), settings, "VkBufferDeviceAddressInfo", indents, dump_json_VkBufferDeviceAddressInfo);
        break;
    case 1000257002:
        dump_json_pNext<const VkBufferOpaqueCaptureAddressCreateInfo>(static_cast<const VkBufferOpaqueCaptureAddressCreateInfo*>(object), settings, "VkBufferOpaqueCaptureAddressCreateInfo", indents, dump_json_VkBufferOpaqueCaptureAddressCreateInfo);
        break;
    case 1000257003:
        dump_json_pNext<const VkMemoryOpaqueCaptureAddressAllocateInfo>(static_cast<const VkMemoryOpaqueCaptureAddressAllocateInfo*>(object), settings, "VkMemoryOpaqueCaptureAddressAllocateInfo", indents, dump_json_VkMemoryOpaqueCaptureAddressAllocateInfo);
        break;
    case 1000257004:
        dump_json_pNext<const VkDeviceMemoryOpaqueCaptureAddressInfo>(static_cast<const VkDeviceMemoryOpaqueCaptureAddressInfo*>(object), settings, "VkDeviceMemoryOpaqueCaptureAddressInfo", indents, dump_json_VkDeviceMemoryOpaqueCaptureAddressInfo);
        break;
    case 53:
        dump_json_pNext<const VkPhysicalDeviceVulkan13Features>(static_cast<const VkPhysicalDeviceVulkan13Features*>(object), settings, "VkPhysicalDeviceVulkan13Features", indents, dump_json_VkPhysicalDeviceVulkan13Features);
        break;
    case 54:
        dump_json_pNext<const VkPhysicalDeviceVulkan13Properties>(static_cast<const VkPhysicalDeviceVulkan13Properties*>(object), settings, "VkPhysicalDeviceVulkan13Properties", indents, dump_json_VkPhysicalDeviceVulkan13Properties);
        break;
    case 1000192000:
        dump_json_pNext<const VkPipelineCreationFeedbackCreateInfo>(static_cast<const VkPipelineCreationFeedbackCreateInfo*>(object), settings, "VkPipelineCreationFeedbackCreateInfo", indents, dump_json_VkPipelineCreationFeedbackCreateInfo);
        break;
    case 1000215000:
        dump_json_pNext<const VkPhysicalDeviceShaderTerminateInvocationFeatures>(static_cast<const VkPhysicalDeviceShaderTerminateInvocationFeatures*>(object), settings, "VkPhysicalDeviceShaderTerminateInvocationFeatures", indents, dump_json_VkPhysicalDeviceShaderTerminateInvocationFeatures);
        break;
    case 1000245000:
        dump_json_pNext<const VkPhysicalDeviceToolProperties>(static_cast<const VkPhysicalDeviceToolProperties*>(object), settings, "VkPhysicalDeviceToolProperties", indents, dump_json_VkPhysicalDeviceToolProperties);
        break;
    case 1000276000:
        dump_json_pNext<const VkPhysicalDeviceShaderDemoteToHelperInvocationFeatures>(static_cast<const VkPhysicalDeviceShaderDemoteToHelperInvocationFeatures*>(object), settings, "VkPhysicalDeviceShaderDemoteToHelperInvocationFeatures", indents, dump_json_VkPhysicalDeviceShaderDemoteToHelperInvocationFeatures);
        break;
    case 1000295000:
        dump_json_pNext<const VkPhysicalDevicePrivateDataFeatures>(static_cast<const VkPhysicalDevicePrivateDataFeatures*>(object), settings, "VkPhysicalDevicePrivateDataFeatures", indents, dump_json_VkPhysicalDevicePrivateDataFeatures);
        break;
    case 1000295001:
        dump_json_pNext<const VkDevicePrivateDataCreateInfo>(static_cast<const VkDevicePrivateDataCreateInfo*>(object), settings, "VkDevicePrivateDataCreateInfo", indents, dump_json_VkDevicePrivateDataCreateInfo);
        break;
    case 1000295002:
        dump_json_pNext<const VkPrivateDataSlotCreateInfo>(static_cast<const VkPrivateDataSlotCreateInfo*>(object), settings, "VkPrivateDataSlotCreateInfo", indents, dump_json_VkPrivateDataSlotCreateInfo);
        break;
    case 1000297000:
        dump_json_pNext<const VkPhysicalDevicePipelineCreationCacheControlFeatures>(static_cast<const VkPhysicalDevicePipelineCreationCacheControlFeatures*>(object), settings, "VkPhysicalDevicePipelineCreationCacheControlFeatures", indents, dump_json_VkPhysicalDevicePipelineCreationCacheControlFeatures);
        break;
    case 1000314000:
        dump_json_pNext<const VkMemoryBarrier2>(static_cast<const VkMemoryBarrier2*>(object), settings, "VkMemoryBarrier2", indents, dump_json_VkMemoryBarrier2);
        break;
    case 1000314001:
        dump_json_pNext<const VkBufferMemoryBarrier2>(static_cast<const VkBufferMemoryBarrier2*>(object), settings, "VkBufferMemoryBarrier2", indents, dump_json_VkBufferMemoryBarrier2);
        break;
    case 1000314002:
        dump_json_pNext<const VkImageMemoryBarrier2>(static_cast<const VkImageMemoryBarrier2*>(object), settings, "VkImageMemoryBarrier2", indents, dump_json_VkImageMemoryBarrier2);
        break;
    case 1000314003:
        dump_json_pNext<const VkDependencyInfo>(static_cast<const VkDependencyInfo*>(object), settings, "VkDependencyInfo", indents, dump_json_VkDependencyInfo);
        break;
    case 1000314005:
        dump_json_pNext<const VkSemaphoreSubmitInfo>(static_cast<const VkSemaphoreSubmitInfo*>(object), settings, "VkSemaphoreSubmitInfo", indents, dump_json_VkSemaphoreSubmitInfo);
        break;
    case 1000314006:
        dump_json_pNext<const VkCommandBufferSubmitInfo>(static_cast<const VkCommandBufferSubmitInfo*>(object), settings, "VkCommandBufferSubmitInfo", indents, dump_json_VkCommandBufferSubmitInfo);
        break;
    case 1000314004:
        dump_json_pNext<const VkSubmitInfo2>(static_cast<const VkSubmitInfo2*>(object), settings, "VkSubmitInfo2", indents, dump_json_VkSubmitInfo2);
        break;
    case 1000314007:
        dump_json_pNext<const VkPhysicalDeviceSynchronization2Features>(static_cast<const VkPhysicalDeviceSynchronization2Features*>(object), settings, "VkPhysicalDeviceSynchronization2Features", indents, dump_json_VkPhysicalDeviceSynchronization2Features);
        break;
    case 1000325000:
        dump_json_pNext<const VkPhysicalDeviceZeroInitializeWorkgroupMemoryFeatures>(static_cast<const VkPhysicalDeviceZeroInitializeWorkgroupMemoryFeatures*>(object), settings, "VkPhysicalDeviceZeroInitializeWorkgroupMemoryFeatures", indents, dump_json_VkPhysicalDeviceZeroInitializeWorkgroupMemoryFeatures);
        break;
    case 1000335000:
        dump_json_pNext<const VkPhysicalDeviceImageRobustnessFeatures>(static_cast<const VkPhysicalDeviceImageRobustnessFeatures*>(object), settings, "VkPhysicalDeviceImageRobustnessFeatures", indents, dump_json_VkPhysicalDeviceImageRobustnessFeatures);
        break;
    case 1000337006:
        dump_json_pNext<const VkBufferCopy2>(static_cast<const VkBufferCopy2*>(object), settings, "VkBufferCopy2", indents, dump_json_VkBufferCopy2);
        break;
    case 1000337000:
        dump_json_pNext<const VkCopyBufferInfo2>(static_cast<const VkCopyBufferInfo2*>(object), settings, "VkCopyBufferInfo2", indents, dump_json_VkCopyBufferInfo2);
        break;
    case 1000337007:
        dump_json_pNext<const VkImageCopy2>(static_cast<const VkImageCopy2*>(object), settings, "VkImageCopy2", indents, dump_json_VkImageCopy2);
        break;
    case 1000337001:
        dump_json_pNext<const VkCopyImageInfo2>(static_cast<const VkCopyImageInfo2*>(object), settings, "VkCopyImageInfo2", indents, dump_json_VkCopyImageInfo2);
        break;
    case 1000337009:
        dump_json_pNext<const VkBufferImageCopy2>(static_cast<const VkBufferImageCopy2*>(object), settings, "VkBufferImageCopy2", indents, dump_json_VkBufferImageCopy2);
        break;
    case 1000337002:
        dump_json_pNext<const VkCopyBufferToImageInfo2>(static_cast<const VkCopyBufferToImageInfo2*>(object), settings, "VkCopyBufferToImageInfo2", indents, dump_json_VkCopyBufferToImageInfo2);
        break;
    case 1000337003:
        dump_json_pNext<const VkCopyImageToBufferInfo2>(static_cast<const VkCopyImageToBufferInfo2*>(object), settings, "VkCopyImageToBufferInfo2", indents, dump_json_VkCopyImageToBufferInfo2);
        break;
    case 1000337008:
        dump_json_pNext<const VkImageBlit2>(static_cast<const VkImageBlit2*>(object), settings, "VkImageBlit2", indents, dump_json_VkImageBlit2);
        break;
    case 1000337004:
        dump_json_pNext<const VkBlitImageInfo2>(static_cast<const VkBlitImageInfo2*>(object), settings, "VkBlitImageInfo2", indents, dump_json_VkBlitImageInfo2);
        break;
    case 1000337010:
        dump_json_pNext<const VkImageResolve2>(static_cast<const VkImageResolve2*>(object), settings, "VkImageResolve2", indents, dump_json_VkImageResolve2);
        break;
    case 1000337005:
        dump_json_pNext<const VkResolveImageInfo2>(static_cast<const VkResolveImageInfo2*>(object), settings, "VkResolveImageInfo2", indents, dump_json_VkResolveImageInfo2);
        break;
    case 1000225002:
        dump_json_pNext<const VkPhysicalDeviceSubgroupSizeControlFeatures>(static_cast<const VkPhysicalDeviceSubgroupSizeControlFeatures*>(object), settings, "VkPhysicalDeviceSubgroupSizeControlFeatures", indents, dump_json_VkPhysicalDeviceSubgroupSizeControlFeatures);
        break;
    case 1000225000:
        dump_json_pNext<const VkPhysicalDeviceSubgroupSizeControlProperties>(static_cast<const VkPhysicalDeviceSubgroupSizeControlProperties*>(object), settings, "VkPhysicalDeviceSubgroupSizeControlProperties", indents, dump_json_VkPhysicalDeviceSubgroupSizeControlProperties);
        break;
    case 1000225001:
        dump_json_pNext<const VkPipelineShaderStageRequiredSubgroupSizeCreateInfo>(static_cast<const VkPipelineShaderStageRequiredSubgroupSizeCreateInfo*>(object), settings, "VkPipelineShaderStageRequiredSubgroupSizeCreateInfo", indents, dump_json_VkPipelineShaderStageRequiredSubgroupSizeCreateInfo);
        break;
    case 1000138000:
        dump_json_pNext<const VkPhysicalDeviceInlineUniformBlockFeatures>(static_cast<const VkPhysicalDeviceInlineUniformBlockFeatures*>(object), settings, "VkPhysicalDeviceInlineUniformBlockFeatures", indents, dump_json_VkPhysicalDeviceInlineUniformBlockFeatures);
        break;
    case 1000138001:
        dump_json_pNext<const VkPhysicalDeviceInlineUniformBlockProperties>(static_cast<const VkPhysicalDeviceInlineUniformBlockProperties*>(object), settings, "VkPhysicalDeviceInlineUniformBlockProperties", indents, dump_json_VkPhysicalDeviceInlineUniformBlockProperties);
        break;
    case 1000138002:
        dump_json_pNext<const VkWriteDescriptorSetInlineUniformBlock>(static_cast<const VkWriteDescriptorSetInlineUniformBlock*>(object), settings, "VkWriteDescriptorSetInlineUniformBlock", indents, dump_json_VkWriteDescriptorSetInlineUniformBlock);
        break;
    case 1000138003:
        dump_json_pNext<const VkDescriptorPoolInlineUniformBlockCreateInfo>(static_cast<const VkDescriptorPoolInlineUniformBlockCreateInfo*>(object), settings, "VkDescriptorPoolInlineUniformBlockCreateInfo", indents, dump_json_VkDescriptorPoolInlineUniformBlockCreateInfo);
        break;
    case 1000066000:
        dump_json_pNext<const VkPhysicalDeviceTextureCompressionASTCHDRFeatures>(static_cast<const VkPhysicalDeviceTextureCompressionASTCHDRFeatures*>(object), settings, "VkPhysicalDeviceTextureCompressionASTCHDRFeatures", indents, dump_json_VkPhysicalDeviceTextureCompressionASTCHDRFeatures);
        break;
    case 1000044001:
        dump_json_pNext<const VkRenderingAttachmentInfo>(static_cast<const VkRenderingAttachmentInfo*>(object), settings, "VkRenderingAttachmentInfo", indents, dump_json_VkRenderingAttachmentInfo);
        break;
    case 1000044000:
        dump_json_pNext<const VkRenderingInfo>(static_cast<const VkRenderingInfo*>(object), settings, "VkRenderingInfo", indents, dump_json_VkRenderingInfo);
        break;
    case 1000044002:
        dump_json_pNext<const VkPipelineRenderingCreateInfo>(static_cast<const VkPipelineRenderingCreateInfo*>(object), settings, "VkPipelineRenderingCreateInfo", indents, dump_json_VkPipelineRenderingCreateInfo);
        break;
    case 1000044003:
        dump_json_pNext<const VkPhysicalDeviceDynamicRenderingFeatures>(static_cast<const VkPhysicalDeviceDynamicRenderingFeatures*>(object), settings, "VkPhysicalDeviceDynamicRenderingFeatures", indents, dump_json_VkPhysicalDeviceDynamicRenderingFeatures);
        break;
    case 1000044004:
        dump_json_pNext<const VkCommandBufferInheritanceRenderingInfo>(static_cast<const VkCommandBufferInheritanceRenderingInfo*>(object), settings, "VkCommandBufferInheritanceRenderingInfo", indents, dump_json_VkCommandBufferInheritanceRenderingInfo);
        break;
    case 1000280000:
        dump_json_pNext<const VkPhysicalDeviceShaderIntegerDotProductFeatures>(static_cast<const VkPhysicalDeviceShaderIntegerDotProductFeatures*>(object), settings, "VkPhysicalDeviceShaderIntegerDotProductFeatures", indents, dump_json_VkPhysicalDeviceShaderIntegerDotProductFeatures);
        break;
    case 1000280001:
        dump_json_pNext<const VkPhysicalDeviceShaderIntegerDotProductProperties>(static_cast<const VkPhysicalDeviceShaderIntegerDotProductProperties*>(object), settings, "VkPhysicalDeviceShaderIntegerDotProductProperties", indents, dump_json_VkPhysicalDeviceShaderIntegerDotProductProperties);
        break;
    case 1000281001:
        dump_json_pNext<const VkPhysicalDeviceTexelBufferAlignmentProperties>(static_cast<const VkPhysicalDeviceTexelBufferAlignmentProperties*>(object), settings, "VkPhysicalDeviceTexelBufferAlignmentProperties", indents, dump_json_VkPhysicalDeviceTexelBufferAlignmentProperties);
        break;
    case 1000360000:
        dump_json_pNext<const VkFormatProperties3>(static_cast<const VkFormatProperties3*>(object), settings, "VkFormatProperties3", indents, dump_json_VkFormatProperties3);
        break;
    case 1000413000:
        dump_json_pNext<const VkPhysicalDeviceMaintenance4Features>(static_cast<const VkPhysicalDeviceMaintenance4Features*>(object), settings, "VkPhysicalDeviceMaintenance4Features", indents, dump_json_VkPhysicalDeviceMaintenance4Features);
        break;
    case 1000413001:
        dump_json_pNext<const VkPhysicalDeviceMaintenance4Properties>(static_cast<const VkPhysicalDeviceMaintenance4Properties*>(object), settings, "VkPhysicalDeviceMaintenance4Properties", indents, dump_json_VkPhysicalDeviceMaintenance4Properties);
        break;
    case 1000413002:
        dump_json_pNext<const VkDeviceBufferMemoryRequirements>(static_cast<const VkDeviceBufferMemoryRequirements*>(object), settings, "VkDeviceBufferMemoryRequirements", indents, dump_json_VkDeviceBufferMemoryRequirements);
        break;
    case 1000413003:
        dump_json_pNext<const VkDeviceImageMemoryRequirements>(static_cast<const VkDeviceImageMemoryRequirements*>(object), settings, "VkDeviceImageMemoryRequirements", indents, dump_json_VkDeviceImageMemoryRequirements);
        break;
    case 1000001000:
        dump_json_pNext<const VkSwapchainCreateInfoKHR>(static_cast<const VkSwapchainCreateInfoKHR*>(object), settings, "VkSwapchainCreateInfoKHR", indents, dump_json_VkSwapchainCreateInfoKHR);
        break;
    case 1000001001:
        dump_json_pNext<const VkPresentInfoKHR>(static_cast<const VkPresentInfoKHR*>(object), settings, "VkPresentInfoKHR", indents, dump_json_VkPresentInfoKHR);
        break;
    case 1000060008:
        dump_json_pNext<const VkImageSwapchainCreateInfoKHR>(static_cast<const VkImageSwapchainCreateInfoKHR*>(object), settings, "VkImageSwapchainCreateInfoKHR", indents, dump_json_VkImageSwapchainCreateInfoKHR);
        break;
    case 1000060009:
        dump_json_pNext<const VkBindImageMemorySwapchainInfoKHR>(static_cast<const VkBindImageMemorySwapchainInfoKHR*>(object), settings, "VkBindImageMemorySwapchainInfoKHR", indents, dump_json_VkBindImageMemorySwapchainInfoKHR);
        break;
    case 1000060010:
        dump_json_pNext<const VkAcquireNextImageInfoKHR>(static_cast<const VkAcquireNextImageInfoKHR*>(object), settings, "VkAcquireNextImageInfoKHR", indents, dump_json_VkAcquireNextImageInfoKHR);
        break;
    case 1000060007:
        dump_json_pNext<const VkDeviceGroupPresentCapabilitiesKHR>(static_cast<const VkDeviceGroupPresentCapabilitiesKHR*>(object), settings, "VkDeviceGroupPresentCapabilitiesKHR", indents, dump_json_VkDeviceGroupPresentCapabilitiesKHR);
        break;
    case 1000060011:
        dump_json_pNext<const VkDeviceGroupPresentInfoKHR>(static_cast<const VkDeviceGroupPresentInfoKHR*>(object), settings, "VkDeviceGroupPresentInfoKHR", indents, dump_json_VkDeviceGroupPresentInfoKHR);
        break;
    case 1000060012:
        dump_json_pNext<const VkDeviceGroupSwapchainCreateInfoKHR>(static_cast<const VkDeviceGroupSwapchainCreateInfoKHR*>(object), settings, "VkDeviceGroupSwapchainCreateInfoKHR", indents, dump_json_VkDeviceGroupSwapchainCreateInfoKHR);
        break;
    case 1000002000:
        dump_json_pNext<const VkDisplayModeCreateInfoKHR>(static_cast<const VkDisplayModeCreateInfoKHR*>(object), settings, "VkDisplayModeCreateInfoKHR", indents, dump_json_VkDisplayModeCreateInfoKHR);
        break;
    case 1000002001:
        dump_json_pNext<const VkDisplaySurfaceCreateInfoKHR>(static_cast<const VkDisplaySurfaceCreateInfoKHR*>(object), settings, "VkDisplaySurfaceCreateInfoKHR", indents, dump_json_VkDisplaySurfaceCreateInfoKHR);
        break;
    case 1000003000:
        dump_json_pNext<const VkDisplayPresentInfoKHR>(static_cast<const VkDisplayPresentInfoKHR*>(object), settings, "VkDisplayPresentInfoKHR", indents, dump_json_VkDisplayPresentInfoKHR);
        break;
#if defined(VK_USE_PLATFORM_XLIB_KHR)
    case 1000004000:
        dump_json_pNext<const VkXlibSurfaceCreateInfoKHR>(static_cast<const VkXlibSurfaceCreateInfoKHR*>(object), settings, "VkXlibSurfaceCreateInfoKHR", indents, dump_json_VkXlibSurfaceCreateInfoKHR);
        break;
#endif // VK_USE_PLATFORM_XLIB_KHR
#if defined(VK_USE_PLATFORM_XCB_KHR)
    case 1000005000:
        dump_json_pNext<const VkXcbSurfaceCreateInfoKHR>(static_cast<const VkXcbSurfaceCreateInfoKHR*>(object), settings, "VkXcbSurfaceCreateInfoKHR", indents, dump_json_VkXcbSurfaceCreateInfoKHR);
        break;
#endif // VK_USE_PLATFORM_XCB_KHR
#if defined(VK_USE_PLATFORM_WAYLAND_KHR)
    case 1000006000:
        dump_json_pNext<const VkWaylandSurfaceCreateInfoKHR>(static_cast<const VkWaylandSurfaceCreateInfoKHR*>(object), settings, "VkWaylandSurfaceCreateInfoKHR", indents, dump_json_VkWaylandSurfaceCreateInfoKHR);
        break;
#endif // VK_USE_PLATFORM_WAYLAND_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
    case 1000008000:
        dump_json_pNext<const VkAndroidSurfaceCreateInfoKHR>(static_cast<const VkAndroidSurfaceCreateInfoKHR*>(object), settings, "VkAndroidSurfaceCreateInfoKHR", indents, dump_json_VkAndroidSurfaceCreateInfoKHR);
        break;
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000009000:
        dump_json_pNext<const VkWin32SurfaceCreateInfoKHR>(static_cast<const VkWin32SurfaceCreateInfoKHR*>(object), settings, "VkWin32SurfaceCreateInfoKHR", indents, dump_json_VkWin32SurfaceCreateInfoKHR);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
    case 1000011000:
        dump_json_pNext<const VkDebugReportCallbackCreateInfoEXT>(static_cast<const VkDebugReportCallbackCreateInfoEXT*>(object), settings, "VkDebugReportCallbackCreateInfoEXT", indents, dump_json_VkDebugReportCallbackCreateInfoEXT);
        break;
    case 1000018000:
        dump_json_pNext<const VkPipelineRasterizationStateRasterizationOrderAMD>(static_cast<const VkPipelineRasterizationStateRasterizationOrderAMD*>(object), settings, "VkPipelineRasterizationStateRasterizationOrderAMD", indents, dump_json_VkPipelineRasterizationStateRasterizationOrderAMD);
        break;
    case 1000022000:
        dump_json_pNext<const VkDebugMarkerObjectNameInfoEXT>(static_cast<const VkDebugMarkerObjectNameInfoEXT*>(object), settings, "VkDebugMarkerObjectNameInfoEXT", indents, dump_json_VkDebugMarkerObjectNameInfoEXT);
        break;
    case 1000022001:
        dump_json_pNext<const VkDebugMarkerObjectTagInfoEXT>(static_cast<const VkDebugMarkerObjectTagInfoEXT*>(object), settings, "VkDebugMarkerObjectTagInfoEXT", indents, dump_json_VkDebugMarkerObjectTagInfoEXT);
        break;
    case 1000022002:
        dump_json_pNext<const VkDebugMarkerMarkerInfoEXT>(static_cast<const VkDebugMarkerMarkerInfoEXT*>(object), settings, "VkDebugMarkerMarkerInfoEXT", indents, dump_json_VkDebugMarkerMarkerInfoEXT);
        break;
    case 1000023016:
        dump_json_pNext<const VkQueueFamilyQueryResultStatusPropertiesKHR>(static_cast<const VkQueueFamilyQueryResultStatusPropertiesKHR*>(object), settings, "VkQueueFamilyQueryResultStatusPropertiesKHR", indents, dump_json_VkQueueFamilyQueryResultStatusPropertiesKHR);
        break;
    case 1000023012:
        dump_json_pNext<const VkQueueFamilyVideoPropertiesKHR>(static_cast<const VkQueueFamilyVideoPropertiesKHR*>(object), settings, "VkQueueFamilyVideoPropertiesKHR", indents, dump_json_VkQueueFamilyVideoPropertiesKHR);
        break;
    case 1000023000:
        dump_json_pNext<const VkVideoProfileInfoKHR>(static_cast<const VkVideoProfileInfoKHR*>(object), settings, "VkVideoProfileInfoKHR", indents, dump_json_VkVideoProfileInfoKHR);
        break;
    case 1000023013:
        dump_json_pNext<const VkVideoProfileListInfoKHR>(static_cast<const VkVideoProfileListInfoKHR*>(object), settings, "VkVideoProfileListInfoKHR", indents, dump_json_VkVideoProfileListInfoKHR);
        break;
    case 1000023001:
        dump_json_pNext<const VkVideoCapabilitiesKHR>(static_cast<const VkVideoCapabilitiesKHR*>(object), settings, "VkVideoCapabilitiesKHR", indents, dump_json_VkVideoCapabilitiesKHR);
        break;
    case 1000023014:
        dump_json_pNext<const VkPhysicalDeviceVideoFormatInfoKHR>(static_cast<const VkPhysicalDeviceVideoFormatInfoKHR*>(object), settings, "VkPhysicalDeviceVideoFormatInfoKHR", indents, dump_json_VkPhysicalDeviceVideoFormatInfoKHR);
        break;
    case 1000023015:
        dump_json_pNext<const VkVideoFormatPropertiesKHR>(static_cast<const VkVideoFormatPropertiesKHR*>(object), settings, "VkVideoFormatPropertiesKHR", indents, dump_json_VkVideoFormatPropertiesKHR);
        break;
    case 1000023002:
        dump_json_pNext<const VkVideoPictureResourceInfoKHR>(static_cast<const VkVideoPictureResourceInfoKHR*>(object), settings, "VkVideoPictureResourceInfoKHR", indents, dump_json_VkVideoPictureResourceInfoKHR);
        break;
    case 1000023011:
        dump_json_pNext<const VkVideoReferenceSlotInfoKHR>(static_cast<const VkVideoReferenceSlotInfoKHR*>(object), settings, "VkVideoReferenceSlotInfoKHR", indents, dump_json_VkVideoReferenceSlotInfoKHR);
        break;
    case 1000023003:
        dump_json_pNext<const VkVideoSessionMemoryRequirementsKHR>(static_cast<const VkVideoSessionMemoryRequirementsKHR*>(object), settings, "VkVideoSessionMemoryRequirementsKHR", indents, dump_json_VkVideoSessionMemoryRequirementsKHR);
        break;
    case 1000023004:
        dump_json_pNext<const VkBindVideoSessionMemoryInfoKHR>(static_cast<const VkBindVideoSessionMemoryInfoKHR*>(object), settings, "VkBindVideoSessionMemoryInfoKHR", indents, dump_json_VkBindVideoSessionMemoryInfoKHR);
        break;
    case 1000023005:
        dump_json_pNext<const VkVideoSessionCreateInfoKHR>(static_cast<const VkVideoSessionCreateInfoKHR*>(object), settings, "VkVideoSessionCreateInfoKHR", indents, dump_json_VkVideoSessionCreateInfoKHR);
        break;
    case 1000023006:
        dump_json_pNext<const VkVideoSessionParametersCreateInfoKHR>(static_cast<const VkVideoSessionParametersCreateInfoKHR*>(object), settings, "VkVideoSessionParametersCreateInfoKHR", indents, dump_json_VkVideoSessionParametersCreateInfoKHR);
        break;
    case 1000023007:
        dump_json_pNext<const VkVideoSessionParametersUpdateInfoKHR>(static_cast<const VkVideoSessionParametersUpdateInfoKHR*>(object), settings, "VkVideoSessionParametersUpdateInfoKHR", indents, dump_json_VkVideoSessionParametersUpdateInfoKHR);
        break;
    case 1000023008:
        dump_json_pNext<const VkVideoBeginCodingInfoKHR>(static_cast<const VkVideoBeginCodingInfoKHR*>(object), settings, "VkVideoBeginCodingInfoKHR", indents, dump_json_VkVideoBeginCodingInfoKHR);
        break;
    case 1000023009:
        dump_json_pNext<const VkVideoEndCodingInfoKHR>(static_cast<const VkVideoEndCodingInfoKHR*>(object), settings, "VkVideoEndCodingInfoKHR", indents, dump_json_VkVideoEndCodingInfoKHR);
        break;
    case 1000023010:
        dump_json_pNext<const VkVideoCodingControlInfoKHR>(static_cast<const VkVideoCodingControlInfoKHR*>(object), settings, "VkVideoCodingControlInfoKHR", indents, dump_json_VkVideoCodingControlInfoKHR);
        break;
    case 1000024001:
        dump_json_pNext<const VkVideoDecodeCapabilitiesKHR>(static_cast<const VkVideoDecodeCapabilitiesKHR*>(object), settings, "VkVideoDecodeCapabilitiesKHR", indents, dump_json_VkVideoDecodeCapabilitiesKHR);
        break;
    case 1000024002:
        dump_json_pNext<const VkVideoDecodeUsageInfoKHR>(static_cast<const VkVideoDecodeUsageInfoKHR*>(object), settings, "VkVideoDecodeUsageInfoKHR", indents, dump_json_VkVideoDecodeUsageInfoKHR);
        break;
    case 1000024000:
        dump_json_pNext<const VkVideoDecodeInfoKHR>(static_cast<const VkVideoDecodeInfoKHR*>(object), settings, "VkVideoDecodeInfoKHR", indents, dump_json_VkVideoDecodeInfoKHR);
        break;
    case 1000026000:
        dump_json_pNext<const VkDedicatedAllocationImageCreateInfoNV>(static_cast<const VkDedicatedAllocationImageCreateInfoNV*>(object), settings, "VkDedicatedAllocationImageCreateInfoNV", indents, dump_json_VkDedicatedAllocationImageCreateInfoNV);
        break;
    case 1000026001:
        dump_json_pNext<const VkDedicatedAllocationBufferCreateInfoNV>(static_cast<const VkDedicatedAllocationBufferCreateInfoNV*>(object), settings, "VkDedicatedAllocationBufferCreateInfoNV", indents, dump_json_VkDedicatedAllocationBufferCreateInfoNV);
        break;
    case 1000026002:
        dump_json_pNext<const VkDedicatedAllocationMemoryAllocateInfoNV>(static_cast<const VkDedicatedAllocationMemoryAllocateInfoNV*>(object), settings, "VkDedicatedAllocationMemoryAllocateInfoNV", indents, dump_json_VkDedicatedAllocationMemoryAllocateInfoNV);
        break;
    case 1000028000:
        dump_json_pNext<const VkPhysicalDeviceTransformFeedbackFeaturesEXT>(static_cast<const VkPhysicalDeviceTransformFeedbackFeaturesEXT*>(object), settings, "VkPhysicalDeviceTransformFeedbackFeaturesEXT", indents, dump_json_VkPhysicalDeviceTransformFeedbackFeaturesEXT);
        break;
    case 1000028001:
        dump_json_pNext<const VkPhysicalDeviceTransformFeedbackPropertiesEXT>(static_cast<const VkPhysicalDeviceTransformFeedbackPropertiesEXT*>(object), settings, "VkPhysicalDeviceTransformFeedbackPropertiesEXT", indents, dump_json_VkPhysicalDeviceTransformFeedbackPropertiesEXT);
        break;
    case 1000028002:
        dump_json_pNext<const VkPipelineRasterizationStateStreamCreateInfoEXT>(static_cast<const VkPipelineRasterizationStateStreamCreateInfoEXT*>(object), settings, "VkPipelineRasterizationStateStreamCreateInfoEXT", indents, dump_json_VkPipelineRasterizationStateStreamCreateInfoEXT);
        break;
    case 1000029000:
        dump_json_pNext<const VkCuModuleCreateInfoNVX>(static_cast<const VkCuModuleCreateInfoNVX*>(object), settings, "VkCuModuleCreateInfoNVX", indents, dump_json_VkCuModuleCreateInfoNVX);
        break;
    case 1000029001:
        dump_json_pNext<const VkCuFunctionCreateInfoNVX>(static_cast<const VkCuFunctionCreateInfoNVX*>(object), settings, "VkCuFunctionCreateInfoNVX", indents, dump_json_VkCuFunctionCreateInfoNVX);
        break;
    case 1000029002:
        dump_json_pNext<const VkCuLaunchInfoNVX>(static_cast<const VkCuLaunchInfoNVX*>(object), settings, "VkCuLaunchInfoNVX", indents, dump_json_VkCuLaunchInfoNVX);
        break;
    case 1000030000:
        dump_json_pNext<const VkImageViewHandleInfoNVX>(static_cast<const VkImageViewHandleInfoNVX*>(object), settings, "VkImageViewHandleInfoNVX", indents, dump_json_VkImageViewHandleInfoNVX);
        break;
    case 1000030001:
        dump_json_pNext<const VkImageViewAddressPropertiesNVX>(static_cast<const VkImageViewAddressPropertiesNVX*>(object), settings, "VkImageViewAddressPropertiesNVX", indents, dump_json_VkImageViewAddressPropertiesNVX);
        break;
    case 1000038000:
        dump_json_pNext<const VkVideoEncodeH264CapabilitiesKHR>(static_cast<const VkVideoEncodeH264CapabilitiesKHR*>(object), settings, "VkVideoEncodeH264CapabilitiesKHR", indents, dump_json_VkVideoEncodeH264CapabilitiesKHR);
        break;
    case 1000038011:
        dump_json_pNext<const VkVideoEncodeH264QualityLevelPropertiesKHR>(static_cast<const VkVideoEncodeH264QualityLevelPropertiesKHR*>(object), settings, "VkVideoEncodeH264QualityLevelPropertiesKHR", indents, dump_json_VkVideoEncodeH264QualityLevelPropertiesKHR);
        break;
    case 1000038010:
        dump_json_pNext<const VkVideoEncodeH264SessionCreateInfoKHR>(static_cast<const VkVideoEncodeH264SessionCreateInfoKHR*>(object), settings, "VkVideoEncodeH264SessionCreateInfoKHR", indents, dump_json_VkVideoEncodeH264SessionCreateInfoKHR);
        break;
    case 1000038002:
        dump_json_pNext<const VkVideoEncodeH264SessionParametersAddInfoKHR>(static_cast<const VkVideoEncodeH264SessionParametersAddInfoKHR*>(object), settings, "VkVideoEncodeH264SessionParametersAddInfoKHR", indents, dump_json_VkVideoEncodeH264SessionParametersAddInfoKHR);
        break;
    case 1000038001:
        dump_json_pNext<const VkVideoEncodeH264SessionParametersCreateInfoKHR>(static_cast<const VkVideoEncodeH264SessionParametersCreateInfoKHR*>(object), settings, "VkVideoEncodeH264SessionParametersCreateInfoKHR", indents, dump_json_VkVideoEncodeH264SessionParametersCreateInfoKHR);
        break;
    case 1000038012:
        dump_json_pNext<const VkVideoEncodeH264SessionParametersGetInfoKHR>(static_cast<const VkVideoEncodeH264SessionParametersGetInfoKHR*>(object), settings, "VkVideoEncodeH264SessionParametersGetInfoKHR", indents, dump_json_VkVideoEncodeH264SessionParametersGetInfoKHR);
        break;
    case 1000038013:
        dump_json_pNext<const VkVideoEncodeH264SessionParametersFeedbackInfoKHR>(static_cast<const VkVideoEncodeH264SessionParametersFeedbackInfoKHR*>(object), settings, "VkVideoEncodeH264SessionParametersFeedbackInfoKHR", indents, dump_json_VkVideoEncodeH264SessionParametersFeedbackInfoKHR);
        break;
    case 1000038005:
        dump_json_pNext<const VkVideoEncodeH264NaluSliceInfoKHR>(static_cast<const VkVideoEncodeH264NaluSliceInfoKHR*>(object), settings, "VkVideoEncodeH264NaluSliceInfoKHR", indents, dump_json_VkVideoEncodeH264NaluSliceInfoKHR);
        break;
    case 1000038003:
        dump_json_pNext<const VkVideoEncodeH264PictureInfoKHR>(static_cast<const VkVideoEncodeH264PictureInfoKHR*>(object), settings, "VkVideoEncodeH264PictureInfoKHR", indents, dump_json_VkVideoEncodeH264PictureInfoKHR);
        break;
    case 1000038004:
        dump_json_pNext<const VkVideoEncodeH264DpbSlotInfoKHR>(static_cast<const VkVideoEncodeH264DpbSlotInfoKHR*>(object), settings, "VkVideoEncodeH264DpbSlotInfoKHR", indents, dump_json_VkVideoEncodeH264DpbSlotInfoKHR);
        break;
    case 1000038007:
        dump_json_pNext<const VkVideoEncodeH264ProfileInfoKHR>(static_cast<const VkVideoEncodeH264ProfileInfoKHR*>(object), settings, "VkVideoEncodeH264ProfileInfoKHR", indents, dump_json_VkVideoEncodeH264ProfileInfoKHR);
        break;
    case 1000038008:
        dump_json_pNext<const VkVideoEncodeH264RateControlInfoKHR>(static_cast<const VkVideoEncodeH264RateControlInfoKHR*>(object), settings, "VkVideoEncodeH264RateControlInfoKHR", indents, dump_json_VkVideoEncodeH264RateControlInfoKHR);
        break;
    case 1000038009:
        dump_json_pNext<const VkVideoEncodeH264RateControlLayerInfoKHR>(static_cast<const VkVideoEncodeH264RateControlLayerInfoKHR*>(object), settings, "VkVideoEncodeH264RateControlLayerInfoKHR", indents, dump_json_VkVideoEncodeH264RateControlLayerInfoKHR);
        break;
    case 1000038006:
        dump_json_pNext<const VkVideoEncodeH264GopRemainingFrameInfoKHR>(static_cast<const VkVideoEncodeH264GopRemainingFrameInfoKHR*>(object), settings, "VkVideoEncodeH264GopRemainingFrameInfoKHR", indents, dump_json_VkVideoEncodeH264GopRemainingFrameInfoKHR);
        break;
    case 1000039000:
        dump_json_pNext<const VkVideoEncodeH265CapabilitiesKHR>(static_cast<const VkVideoEncodeH265CapabilitiesKHR*>(object), settings, "VkVideoEncodeH265CapabilitiesKHR", indents, dump_json_VkVideoEncodeH265CapabilitiesKHR);
        break;
    case 1000039011:
        dump_json_pNext<const VkVideoEncodeH265SessionCreateInfoKHR>(static_cast<const VkVideoEncodeH265SessionCreateInfoKHR*>(object), settings, "VkVideoEncodeH265SessionCreateInfoKHR", indents, dump_json_VkVideoEncodeH265SessionCreateInfoKHR);
        break;
    case 1000039012:
        dump_json_pNext<const VkVideoEncodeH265QualityLevelPropertiesKHR>(static_cast<const VkVideoEncodeH265QualityLevelPropertiesKHR*>(object), settings, "VkVideoEncodeH265QualityLevelPropertiesKHR", indents, dump_json_VkVideoEncodeH265QualityLevelPropertiesKHR);
        break;
    case 1000039002:
        dump_json_pNext<const VkVideoEncodeH265SessionParametersAddInfoKHR>(static_cast<const VkVideoEncodeH265SessionParametersAddInfoKHR*>(object), settings, "VkVideoEncodeH265SessionParametersAddInfoKHR", indents, dump_json_VkVideoEncodeH265SessionParametersAddInfoKHR);
        break;
    case 1000039001:
        dump_json_pNext<const VkVideoEncodeH265SessionParametersCreateInfoKHR>(static_cast<const VkVideoEncodeH265SessionParametersCreateInfoKHR*>(object), settings, "VkVideoEncodeH265SessionParametersCreateInfoKHR", indents, dump_json_VkVideoEncodeH265SessionParametersCreateInfoKHR);
        break;
    case 1000039013:
        dump_json_pNext<const VkVideoEncodeH265SessionParametersGetInfoKHR>(static_cast<const VkVideoEncodeH265SessionParametersGetInfoKHR*>(object), settings, "VkVideoEncodeH265SessionParametersGetInfoKHR", indents, dump_json_VkVideoEncodeH265SessionParametersGetInfoKHR);
        break;
    case 1000039014:
        dump_json_pNext<const VkVideoEncodeH265SessionParametersFeedbackInfoKHR>(static_cast<const VkVideoEncodeH265SessionParametersFeedbackInfoKHR*>(object), settings, "VkVideoEncodeH265SessionParametersFeedbackInfoKHR", indents, dump_json_VkVideoEncodeH265SessionParametersFeedbackInfoKHR);
        break;
    case 1000039005:
        dump_json_pNext<const VkVideoEncodeH265NaluSliceSegmentInfoKHR>(static_cast<const VkVideoEncodeH265NaluSliceSegmentInfoKHR*>(object), settings, "VkVideoEncodeH265NaluSliceSegmentInfoKHR", indents, dump_json_VkVideoEncodeH265NaluSliceSegmentInfoKHR);
        break;
    case 1000039003:
        dump_json_pNext<const VkVideoEncodeH265PictureInfoKHR>(static_cast<const VkVideoEncodeH265PictureInfoKHR*>(object), settings, "VkVideoEncodeH265PictureInfoKHR", indents, dump_json_VkVideoEncodeH265PictureInfoKHR);
        break;
    case 1000039004:
        dump_json_pNext<const VkVideoEncodeH265DpbSlotInfoKHR>(static_cast<const VkVideoEncodeH265DpbSlotInfoKHR*>(object), settings, "VkVideoEncodeH265DpbSlotInfoKHR", indents, dump_json_VkVideoEncodeH265DpbSlotInfoKHR);
        break;
    case 1000039007:
        dump_json_pNext<const VkVideoEncodeH265ProfileInfoKHR>(static_cast<const VkVideoEncodeH265ProfileInfoKHR*>(object), settings, "VkVideoEncodeH265ProfileInfoKHR", indents, dump_json_VkVideoEncodeH265ProfileInfoKHR);
        break;
    case 1000039009:
        dump_json_pNext<const VkVideoEncodeH265RateControlInfoKHR>(static_cast<const VkVideoEncodeH265RateControlInfoKHR*>(object), settings, "VkVideoEncodeH265RateControlInfoKHR", indents, dump_json_VkVideoEncodeH265RateControlInfoKHR);
        break;
    case 1000039010:
        dump_json_pNext<const VkVideoEncodeH265RateControlLayerInfoKHR>(static_cast<const VkVideoEncodeH265RateControlLayerInfoKHR*>(object), settings, "VkVideoEncodeH265RateControlLayerInfoKHR", indents, dump_json_VkVideoEncodeH265RateControlLayerInfoKHR);
        break;
    case 1000039006:
        dump_json_pNext<const VkVideoEncodeH265GopRemainingFrameInfoKHR>(static_cast<const VkVideoEncodeH265GopRemainingFrameInfoKHR*>(object), settings, "VkVideoEncodeH265GopRemainingFrameInfoKHR", indents, dump_json_VkVideoEncodeH265GopRemainingFrameInfoKHR);
        break;
    case 1000040003:
        dump_json_pNext<const VkVideoDecodeH264ProfileInfoKHR>(static_cast<const VkVideoDecodeH264ProfileInfoKHR*>(object), settings, "VkVideoDecodeH264ProfileInfoKHR", indents, dump_json_VkVideoDecodeH264ProfileInfoKHR);
        break;
    case 1000040000:
        dump_json_pNext<const VkVideoDecodeH264CapabilitiesKHR>(static_cast<const VkVideoDecodeH264CapabilitiesKHR*>(object), settings, "VkVideoDecodeH264CapabilitiesKHR", indents, dump_json_VkVideoDecodeH264CapabilitiesKHR);
        break;
    case 1000040005:
        dump_json_pNext<const VkVideoDecodeH264SessionParametersAddInfoKHR>(static_cast<const VkVideoDecodeH264SessionParametersAddInfoKHR*>(object), settings, "VkVideoDecodeH264SessionParametersAddInfoKHR", indents, dump_json_VkVideoDecodeH264SessionParametersAddInfoKHR);
        break;
    case 1000040004:
        dump_json_pNext<const VkVideoDecodeH264SessionParametersCreateInfoKHR>(static_cast<const VkVideoDecodeH264SessionParametersCreateInfoKHR*>(object), settings, "VkVideoDecodeH264SessionParametersCreateInfoKHR", indents, dump_json_VkVideoDecodeH264SessionParametersCreateInfoKHR);
        break;
    case 1000040001:
        dump_json_pNext<const VkVideoDecodeH264PictureInfoKHR>(static_cast<const VkVideoDecodeH264PictureInfoKHR*>(object), settings, "VkVideoDecodeH264PictureInfoKHR", indents, dump_json_VkVideoDecodeH264PictureInfoKHR);
        break;
    case 1000040006:
        dump_json_pNext<const VkVideoDecodeH264DpbSlotInfoKHR>(static_cast<const VkVideoDecodeH264DpbSlotInfoKHR*>(object), settings, "VkVideoDecodeH264DpbSlotInfoKHR", indents, dump_json_VkVideoDecodeH264DpbSlotInfoKHR);
        break;
    case 1000041000:
        dump_json_pNext<const VkTextureLODGatherFormatPropertiesAMD>(static_cast<const VkTextureLODGatherFormatPropertiesAMD*>(object), settings, "VkTextureLODGatherFormatPropertiesAMD", indents, dump_json_VkTextureLODGatherFormatPropertiesAMD);
        break;
    case 1000044006:
        dump_json_pNext<const VkRenderingFragmentShadingRateAttachmentInfoKHR>(static_cast<const VkRenderingFragmentShadingRateAttachmentInfoKHR*>(object), settings, "VkRenderingFragmentShadingRateAttachmentInfoKHR", indents, dump_json_VkRenderingFragmentShadingRateAttachmentInfoKHR);
        break;
    case 1000044007:
        dump_json_pNext<const VkRenderingFragmentDensityMapAttachmentInfoEXT>(static_cast<const VkRenderingFragmentDensityMapAttachmentInfoEXT*>(object), settings, "VkRenderingFragmentDensityMapAttachmentInfoEXT", indents, dump_json_VkRenderingFragmentDensityMapAttachmentInfoEXT);
        break;
    case 1000044008:
        dump_json_pNext<const VkAttachmentSampleCountInfoAMD>(static_cast<const VkAttachmentSampleCountInfoAMD*>(object), settings, "VkAttachmentSampleCountInfoAMD", indents, dump_json_VkAttachmentSampleCountInfoAMD);
        break;
    case 1000044009:
        dump_json_pNext<const VkMultiviewPerViewAttributesInfoNVX>(static_cast<const VkMultiviewPerViewAttributesInfoNVX*>(object), settings, "VkMultiviewPerViewAttributesInfoNVX", indents, dump_json_VkMultiviewPerViewAttributesInfoNVX);
        break;
#if defined(VK_USE_PLATFORM_GGP)
    case 1000049000:
        dump_json_pNext<const VkStreamDescriptorSurfaceCreateInfoGGP>(static_cast<const VkStreamDescriptorSurfaceCreateInfoGGP*>(object), settings, "VkStreamDescriptorSurfaceCreateInfoGGP", indents, dump_json_VkStreamDescriptorSurfaceCreateInfoGGP);
        break;
#endif // VK_USE_PLATFORM_GGP
    case 1000050000:
        dump_json_pNext<const VkPhysicalDeviceCornerSampledImageFeaturesNV>(static_cast<const VkPhysicalDeviceCornerSampledImageFeaturesNV*>(object), settings, "VkPhysicalDeviceCornerSampledImageFeaturesNV", indents, dump_json_VkPhysicalDeviceCornerSampledImageFeaturesNV);
        break;
    case 1000056000:
        dump_json_pNext<const VkExternalMemoryImageCreateInfoNV>(static_cast<const VkExternalMemoryImageCreateInfoNV*>(object), settings, "VkExternalMemoryImageCreateInfoNV", indents, dump_json_VkExternalMemoryImageCreateInfoNV);
        break;
    case 1000056001:
        dump_json_pNext<const VkExportMemoryAllocateInfoNV>(static_cast<const VkExportMemoryAllocateInfoNV*>(object), settings, "VkExportMemoryAllocateInfoNV", indents, dump_json_VkExportMemoryAllocateInfoNV);
        break;
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000057000:
        dump_json_pNext<const VkImportMemoryWin32HandleInfoNV>(static_cast<const VkImportMemoryWin32HandleInfoNV*>(object), settings, "VkImportMemoryWin32HandleInfoNV", indents, dump_json_VkImportMemoryWin32HandleInfoNV);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000057001:
        dump_json_pNext<const VkExportMemoryWin32HandleInfoNV>(static_cast<const VkExportMemoryWin32HandleInfoNV*>(object), settings, "VkExportMemoryWin32HandleInfoNV", indents, dump_json_VkExportMemoryWin32HandleInfoNV);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000058000:
        dump_json_pNext<const VkWin32KeyedMutexAcquireReleaseInfoNV>(static_cast<const VkWin32KeyedMutexAcquireReleaseInfoNV*>(object), settings, "VkWin32KeyedMutexAcquireReleaseInfoNV", indents, dump_json_VkWin32KeyedMutexAcquireReleaseInfoNV);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
    case 1000061000:
        dump_json_pNext<const VkValidationFlagsEXT>(static_cast<const VkValidationFlagsEXT*>(object), settings, "VkValidationFlagsEXT", indents, dump_json_VkValidationFlagsEXT);
        break;
#if defined(VK_USE_PLATFORM_VI_NN)
    case 1000062000:
        dump_json_pNext<const VkViSurfaceCreateInfoNN>(static_cast<const VkViSurfaceCreateInfoNN*>(object), settings, "VkViSurfaceCreateInfoNN", indents, dump_json_VkViSurfaceCreateInfoNN);
        break;
#endif // VK_USE_PLATFORM_VI_NN
    case 1000067000:
        dump_json_pNext<const VkImageViewASTCDecodeModeEXT>(static_cast<const VkImageViewASTCDecodeModeEXT*>(object), settings, "VkImageViewASTCDecodeModeEXT", indents, dump_json_VkImageViewASTCDecodeModeEXT);
        break;
    case 1000067001:
        dump_json_pNext<const VkPhysicalDeviceASTCDecodeFeaturesEXT>(static_cast<const VkPhysicalDeviceASTCDecodeFeaturesEXT*>(object), settings, "VkPhysicalDeviceASTCDecodeFeaturesEXT", indents, dump_json_VkPhysicalDeviceASTCDecodeFeaturesEXT);
        break;
    case 1000068001:
        dump_json_pNext<const VkPhysicalDevicePipelineRobustnessFeaturesEXT>(static_cast<const VkPhysicalDevicePipelineRobustnessFeaturesEXT*>(object), settings, "VkPhysicalDevicePipelineRobustnessFeaturesEXT", indents, dump_json_VkPhysicalDevicePipelineRobustnessFeaturesEXT);
        break;
    case 1000068002:
        dump_json_pNext<const VkPhysicalDevicePipelineRobustnessPropertiesEXT>(static_cast<const VkPhysicalDevicePipelineRobustnessPropertiesEXT*>(object), settings, "VkPhysicalDevicePipelineRobustnessPropertiesEXT", indents, dump_json_VkPhysicalDevicePipelineRobustnessPropertiesEXT);
        break;
    case 1000068000:
        dump_json_pNext<const VkPipelineRobustnessCreateInfoEXT>(static_cast<const VkPipelineRobustnessCreateInfoEXT*>(object), settings, "VkPipelineRobustnessCreateInfoEXT", indents, dump_json_VkPipelineRobustnessCreateInfoEXT);
        break;
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000073000:
        dump_json_pNext<const VkImportMemoryWin32HandleInfoKHR>(static_cast<const VkImportMemoryWin32HandleInfoKHR*>(object), settings, "VkImportMemoryWin32HandleInfoKHR", indents, dump_json_VkImportMemoryWin32HandleInfoKHR);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000073001:
        dump_json_pNext<const VkExportMemoryWin32HandleInfoKHR>(static_cast<const VkExportMemoryWin32HandleInfoKHR*>(object), settings, "VkExportMemoryWin32HandleInfoKHR", indents, dump_json_VkExportMemoryWin32HandleInfoKHR);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000073002:
        dump_json_pNext<const VkMemoryWin32HandlePropertiesKHR>(static_cast<const VkMemoryWin32HandlePropertiesKHR*>(object), settings, "VkMemoryWin32HandlePropertiesKHR", indents, dump_json_VkMemoryWin32HandlePropertiesKHR);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000073003:
        dump_json_pNext<const VkMemoryGetWin32HandleInfoKHR>(static_cast<const VkMemoryGetWin32HandleInfoKHR*>(object), settings, "VkMemoryGetWin32HandleInfoKHR", indents, dump_json_VkMemoryGetWin32HandleInfoKHR);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
    case 1000074000:
        dump_json_pNext<const VkImportMemoryFdInfoKHR>(static_cast<const VkImportMemoryFdInfoKHR*>(object), settings, "VkImportMemoryFdInfoKHR", indents, dump_json_VkImportMemoryFdInfoKHR);
        break;
    case 1000074001:
        dump_json_pNext<const VkMemoryFdPropertiesKHR>(static_cast<const VkMemoryFdPropertiesKHR*>(object), settings, "VkMemoryFdPropertiesKHR", indents, dump_json_VkMemoryFdPropertiesKHR);
        break;
    case 1000074002:
        dump_json_pNext<const VkMemoryGetFdInfoKHR>(static_cast<const VkMemoryGetFdInfoKHR*>(object), settings, "VkMemoryGetFdInfoKHR", indents, dump_json_VkMemoryGetFdInfoKHR);
        break;
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000075000:
        dump_json_pNext<const VkWin32KeyedMutexAcquireReleaseInfoKHR>(static_cast<const VkWin32KeyedMutexAcquireReleaseInfoKHR*>(object), settings, "VkWin32KeyedMutexAcquireReleaseInfoKHR", indents, dump_json_VkWin32KeyedMutexAcquireReleaseInfoKHR);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000078000:
        dump_json_pNext<const VkImportSemaphoreWin32HandleInfoKHR>(static_cast<const VkImportSemaphoreWin32HandleInfoKHR*>(object), settings, "VkImportSemaphoreWin32HandleInfoKHR", indents, dump_json_VkImportSemaphoreWin32HandleInfoKHR);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000078001:
        dump_json_pNext<const VkExportSemaphoreWin32HandleInfoKHR>(static_cast<const VkExportSemaphoreWin32HandleInfoKHR*>(object), settings, "VkExportSemaphoreWin32HandleInfoKHR", indents, dump_json_VkExportSemaphoreWin32HandleInfoKHR);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000078002:
        dump_json_pNext<const VkD3D12FenceSubmitInfoKHR>(static_cast<const VkD3D12FenceSubmitInfoKHR*>(object), settings, "VkD3D12FenceSubmitInfoKHR", indents, dump_json_VkD3D12FenceSubmitInfoKHR);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000078003:
        dump_json_pNext<const VkSemaphoreGetWin32HandleInfoKHR>(static_cast<const VkSemaphoreGetWin32HandleInfoKHR*>(object), settings, "VkSemaphoreGetWin32HandleInfoKHR", indents, dump_json_VkSemaphoreGetWin32HandleInfoKHR);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
    case 1000079000:
        dump_json_pNext<const VkImportSemaphoreFdInfoKHR>(static_cast<const VkImportSemaphoreFdInfoKHR*>(object), settings, "VkImportSemaphoreFdInfoKHR", indents, dump_json_VkImportSemaphoreFdInfoKHR);
        break;
    case 1000079001:
        dump_json_pNext<const VkSemaphoreGetFdInfoKHR>(static_cast<const VkSemaphoreGetFdInfoKHR*>(object), settings, "VkSemaphoreGetFdInfoKHR", indents, dump_json_VkSemaphoreGetFdInfoKHR);
        break;
    case 1000080000:
        dump_json_pNext<const VkPhysicalDevicePushDescriptorPropertiesKHR>(static_cast<const VkPhysicalDevicePushDescriptorPropertiesKHR*>(object), settings, "VkPhysicalDevicePushDescriptorPropertiesKHR", indents, dump_json_VkPhysicalDevicePushDescriptorPropertiesKHR);
        break;
    case 1000081002:
        dump_json_pNext<const VkConditionalRenderingBeginInfoEXT>(static_cast<const VkConditionalRenderingBeginInfoEXT*>(object), settings, "VkConditionalRenderingBeginInfoEXT", indents, dump_json_VkConditionalRenderingBeginInfoEXT);
        break;
    case 1000081001:
        dump_json_pNext<const VkPhysicalDeviceConditionalRenderingFeaturesEXT>(static_cast<const VkPhysicalDeviceConditionalRenderingFeaturesEXT*>(object), settings, "VkPhysicalDeviceConditionalRenderingFeaturesEXT", indents, dump_json_VkPhysicalDeviceConditionalRenderingFeaturesEXT);
        break;
    case 1000081000:
        dump_json_pNext<const VkCommandBufferInheritanceConditionalRenderingInfoEXT>(static_cast<const VkCommandBufferInheritanceConditionalRenderingInfoEXT*>(object), settings, "VkCommandBufferInheritanceConditionalRenderingInfoEXT", indents, dump_json_VkCommandBufferInheritanceConditionalRenderingInfoEXT);
        break;
    case 1000084000:
        dump_json_pNext<const VkPresentRegionsKHR>(static_cast<const VkPresentRegionsKHR*>(object), settings, "VkPresentRegionsKHR", indents, dump_json_VkPresentRegionsKHR);
        break;
    case 1000087000:
        dump_json_pNext<const VkPipelineViewportWScalingStateCreateInfoNV>(static_cast<const VkPipelineViewportWScalingStateCreateInfoNV*>(object), settings, "VkPipelineViewportWScalingStateCreateInfoNV", indents, dump_json_VkPipelineViewportWScalingStateCreateInfoNV);
        break;
    case 1000090000:
        dump_json_pNext<const VkSurfaceCapabilities2EXT>(static_cast<const VkSurfaceCapabilities2EXT*>(object), settings, "VkSurfaceCapabilities2EXT", indents, dump_json_VkSurfaceCapabilities2EXT);
        break;
    case 1000091000:
        dump_json_pNext<const VkDisplayPowerInfoEXT>(static_cast<const VkDisplayPowerInfoEXT*>(object), settings, "VkDisplayPowerInfoEXT", indents, dump_json_VkDisplayPowerInfoEXT);
        break;
    case 1000091001:
        dump_json_pNext<const VkDeviceEventInfoEXT>(static_cast<const VkDeviceEventInfoEXT*>(object), settings, "VkDeviceEventInfoEXT", indents, dump_json_VkDeviceEventInfoEXT);
        break;
    case 1000091002:
        dump_json_pNext<const VkDisplayEventInfoEXT>(static_cast<const VkDisplayEventInfoEXT*>(object), settings, "VkDisplayEventInfoEXT", indents, dump_json_VkDisplayEventInfoEXT);
        break;
    case 1000091003:
        dump_json_pNext<const VkSwapchainCounterCreateInfoEXT>(static_cast<const VkSwapchainCounterCreateInfoEXT*>(object), settings, "VkSwapchainCounterCreateInfoEXT", indents, dump_json_VkSwapchainCounterCreateInfoEXT);
        break;
    case 1000092000:
        dump_json_pNext<const VkPresentTimesInfoGOOGLE>(static_cast<const VkPresentTimesInfoGOOGLE*>(object), settings, "VkPresentTimesInfoGOOGLE", indents, dump_json_VkPresentTimesInfoGOOGLE);
        break;
    case 1000097000:
        dump_json_pNext<const VkPhysicalDeviceMultiviewPerViewAttributesPropertiesNVX>(static_cast<const VkPhysicalDeviceMultiviewPerViewAttributesPropertiesNVX*>(object), settings, "VkPhysicalDeviceMultiviewPerViewAttributesPropertiesNVX", indents, dump_json_VkPhysicalDeviceMultiviewPerViewAttributesPropertiesNVX);
        break;
    case 1000098000:
        dump_json_pNext<const VkPipelineViewportSwizzleStateCreateInfoNV>(static_cast<const VkPipelineViewportSwizzleStateCreateInfoNV*>(object), settings, "VkPipelineViewportSwizzleStateCreateInfoNV", indents, dump_json_VkPipelineViewportSwizzleStateCreateInfoNV);
        break;
    case 1000099000:
        dump_json_pNext<const VkPhysicalDeviceDiscardRectanglePropertiesEXT>(static_cast<const VkPhysicalDeviceDiscardRectanglePropertiesEXT*>(object), settings, "VkPhysicalDeviceDiscardRectanglePropertiesEXT", indents, dump_json_VkPhysicalDeviceDiscardRectanglePropertiesEXT);
        break;
    case 1000099001:
        dump_json_pNext<const VkPipelineDiscardRectangleStateCreateInfoEXT>(static_cast<const VkPipelineDiscardRectangleStateCreateInfoEXT*>(object), settings, "VkPipelineDiscardRectangleStateCreateInfoEXT", indents, dump_json_VkPipelineDiscardRectangleStateCreateInfoEXT);
        break;
    case 1000101000:
        dump_json_pNext<const VkPhysicalDeviceConservativeRasterizationPropertiesEXT>(static_cast<const VkPhysicalDeviceConservativeRasterizationPropertiesEXT*>(object), settings, "VkPhysicalDeviceConservativeRasterizationPropertiesEXT", indents, dump_json_VkPhysicalDeviceConservativeRasterizationPropertiesEXT);
        break;
    case 1000101001:
        dump_json_pNext<const VkPipelineRasterizationConservativeStateCreateInfoEXT>(static_cast<const VkPipelineRasterizationConservativeStateCreateInfoEXT*>(object), settings, "VkPipelineRasterizationConservativeStateCreateInfoEXT", indents, dump_json_VkPipelineRasterizationConservativeStateCreateInfoEXT);
        break;
    case 1000102000:
        dump_json_pNext<const VkPhysicalDeviceDepthClipEnableFeaturesEXT>(static_cast<const VkPhysicalDeviceDepthClipEnableFeaturesEXT*>(object), settings, "VkPhysicalDeviceDepthClipEnableFeaturesEXT", indents, dump_json_VkPhysicalDeviceDepthClipEnableFeaturesEXT);
        break;
    case 1000102001:
        dump_json_pNext<const VkPipelineRasterizationDepthClipStateCreateInfoEXT>(static_cast<const VkPipelineRasterizationDepthClipStateCreateInfoEXT*>(object), settings, "VkPipelineRasterizationDepthClipStateCreateInfoEXT", indents, dump_json_VkPipelineRasterizationDepthClipStateCreateInfoEXT);
        break;
    case 1000105000:
        dump_json_pNext<const VkHdrMetadataEXT>(static_cast<const VkHdrMetadataEXT*>(object), settings, "VkHdrMetadataEXT", indents, dump_json_VkHdrMetadataEXT);
        break;
    case 1000110000:
        dump_json_pNext<const VkPhysicalDeviceRelaxedLineRasterizationFeaturesIMG>(static_cast<const VkPhysicalDeviceRelaxedLineRasterizationFeaturesIMG*>(object), settings, "VkPhysicalDeviceRelaxedLineRasterizationFeaturesIMG", indents, dump_json_VkPhysicalDeviceRelaxedLineRasterizationFeaturesIMG);
        break;
    case 1000111000:
        dump_json_pNext<const VkSharedPresentSurfaceCapabilitiesKHR>(static_cast<const VkSharedPresentSurfaceCapabilitiesKHR*>(object), settings, "VkSharedPresentSurfaceCapabilitiesKHR", indents, dump_json_VkSharedPresentSurfaceCapabilitiesKHR);
        break;
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000114000:
        dump_json_pNext<const VkImportFenceWin32HandleInfoKHR>(static_cast<const VkImportFenceWin32HandleInfoKHR*>(object), settings, "VkImportFenceWin32HandleInfoKHR", indents, dump_json_VkImportFenceWin32HandleInfoKHR);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000114001:
        dump_json_pNext<const VkExportFenceWin32HandleInfoKHR>(static_cast<const VkExportFenceWin32HandleInfoKHR*>(object), settings, "VkExportFenceWin32HandleInfoKHR", indents, dump_json_VkExportFenceWin32HandleInfoKHR);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000114002:
        dump_json_pNext<const VkFenceGetWin32HandleInfoKHR>(static_cast<const VkFenceGetWin32HandleInfoKHR*>(object), settings, "VkFenceGetWin32HandleInfoKHR", indents, dump_json_VkFenceGetWin32HandleInfoKHR);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
    case 1000115000:
        dump_json_pNext<const VkImportFenceFdInfoKHR>(static_cast<const VkImportFenceFdInfoKHR*>(object), settings, "VkImportFenceFdInfoKHR", indents, dump_json_VkImportFenceFdInfoKHR);
        break;
    case 1000115001:
        dump_json_pNext<const VkFenceGetFdInfoKHR>(static_cast<const VkFenceGetFdInfoKHR*>(object), settings, "VkFenceGetFdInfoKHR", indents, dump_json_VkFenceGetFdInfoKHR);
        break;
    case 1000116000:
        dump_json_pNext<const VkPhysicalDevicePerformanceQueryFeaturesKHR>(static_cast<const VkPhysicalDevicePerformanceQueryFeaturesKHR*>(object), settings, "VkPhysicalDevicePerformanceQueryFeaturesKHR", indents, dump_json_VkPhysicalDevicePerformanceQueryFeaturesKHR);
        break;
    case 1000116001:
        dump_json_pNext<const VkPhysicalDevicePerformanceQueryPropertiesKHR>(static_cast<const VkPhysicalDevicePerformanceQueryPropertiesKHR*>(object), settings, "VkPhysicalDevicePerformanceQueryPropertiesKHR", indents, dump_json_VkPhysicalDevicePerformanceQueryPropertiesKHR);
        break;
    case 1000116005:
        dump_json_pNext<const VkPerformanceCounterKHR>(static_cast<const VkPerformanceCounterKHR*>(object), settings, "VkPerformanceCounterKHR", indents, dump_json_VkPerformanceCounterKHR);
        break;
    case 1000116006:
        dump_json_pNext<const VkPerformanceCounterDescriptionKHR>(static_cast<const VkPerformanceCounterDescriptionKHR*>(object), settings, "VkPerformanceCounterDescriptionKHR", indents, dump_json_VkPerformanceCounterDescriptionKHR);
        break;
    case 1000116002:
        dump_json_pNext<const VkQueryPoolPerformanceCreateInfoKHR>(static_cast<const VkQueryPoolPerformanceCreateInfoKHR*>(object), settings, "VkQueryPoolPerformanceCreateInfoKHR", indents, dump_json_VkQueryPoolPerformanceCreateInfoKHR);
        break;
    case 1000116004:
        dump_json_pNext<const VkAcquireProfilingLockInfoKHR>(static_cast<const VkAcquireProfilingLockInfoKHR*>(object), settings, "VkAcquireProfilingLockInfoKHR", indents, dump_json_VkAcquireProfilingLockInfoKHR);
        break;
    case 1000116003:
        dump_json_pNext<const VkPerformanceQuerySubmitInfoKHR>(static_cast<const VkPerformanceQuerySubmitInfoKHR*>(object), settings, "VkPerformanceQuerySubmitInfoKHR", indents, dump_json_VkPerformanceQuerySubmitInfoKHR);
        break;
    case 1000119000:
        dump_json_pNext<const VkPhysicalDeviceSurfaceInfo2KHR>(static_cast<const VkPhysicalDeviceSurfaceInfo2KHR*>(object), settings, "VkPhysicalDeviceSurfaceInfo2KHR", indents, dump_json_VkPhysicalDeviceSurfaceInfo2KHR);
        break;
    case 1000119001:
        dump_json_pNext<const VkSurfaceCapabilities2KHR>(static_cast<const VkSurfaceCapabilities2KHR*>(object), settings, "VkSurfaceCapabilities2KHR", indents, dump_json_VkSurfaceCapabilities2KHR);
        break;
    case 1000119002:
        dump_json_pNext<const VkSurfaceFormat2KHR>(static_cast<const VkSurfaceFormat2KHR*>(object), settings, "VkSurfaceFormat2KHR", indents, dump_json_VkSurfaceFormat2KHR);
        break;
    case 1000121000:
        dump_json_pNext<const VkDisplayProperties2KHR>(static_cast<const VkDisplayProperties2KHR*>(object), settings, "VkDisplayProperties2KHR", indents, dump_json_VkDisplayProperties2KHR);
        break;
    case 1000121001:
        dump_json_pNext<const VkDisplayPlaneProperties2KHR>(static_cast<const VkDisplayPlaneProperties2KHR*>(object), settings, "VkDisplayPlaneProperties2KHR", indents, dump_json_VkDisplayPlaneProperties2KHR);
        break;
    case 1000121002:
        dump_json_pNext<const VkDisplayModeProperties2KHR>(static_cast<const VkDisplayModeProperties2KHR*>(object), settings, "VkDisplayModeProperties2KHR", indents, dump_json_VkDisplayModeProperties2KHR);
        break;
    case 1000121003:
        dump_json_pNext<const VkDisplayPlaneInfo2KHR>(static_cast<const VkDisplayPlaneInfo2KHR*>(object), settings, "VkDisplayPlaneInfo2KHR", indents, dump_json_VkDisplayPlaneInfo2KHR);
        break;
    case 1000121004:
        dump_json_pNext<const VkDisplayPlaneCapabilities2KHR>(static_cast<const VkDisplayPlaneCapabilities2KHR*>(object), settings, "VkDisplayPlaneCapabilities2KHR", indents, dump_json_VkDisplayPlaneCapabilities2KHR);
        break;
#if defined(VK_USE_PLATFORM_IOS_MVK)
    case 1000122000:
        dump_json_pNext<const VkIOSSurfaceCreateInfoMVK>(static_cast<const VkIOSSurfaceCreateInfoMVK*>(object), settings, "VkIOSSurfaceCreateInfoMVK", indents, dump_json_VkIOSSurfaceCreateInfoMVK);
        break;
#endif // VK_USE_PLATFORM_IOS_MVK
#if defined(VK_USE_PLATFORM_MACOS_MVK)
    case 1000123000:
        dump_json_pNext<const VkMacOSSurfaceCreateInfoMVK>(static_cast<const VkMacOSSurfaceCreateInfoMVK*>(object), settings, "VkMacOSSurfaceCreateInfoMVK", indents, dump_json_VkMacOSSurfaceCreateInfoMVK);
        break;
#endif // VK_USE_PLATFORM_MACOS_MVK
    case 1000128002:
        dump_json_pNext<const VkDebugUtilsLabelEXT>(static_cast<const VkDebugUtilsLabelEXT*>(object), settings, "VkDebugUtilsLabelEXT", indents, dump_json_VkDebugUtilsLabelEXT);
        break;
    case 1000128000:
        dump_json_pNext<const VkDebugUtilsObjectNameInfoEXT>(static_cast<const VkDebugUtilsObjectNameInfoEXT*>(object), settings, "VkDebugUtilsObjectNameInfoEXT", indents, dump_json_VkDebugUtilsObjectNameInfoEXT);
        break;
    case 1000128003:
        dump_json_pNext<const VkDebugUtilsMessengerCallbackDataEXT>(static_cast<const VkDebugUtilsMessengerCallbackDataEXT*>(object), settings, "VkDebugUtilsMessengerCallbackDataEXT", indents, dump_json_VkDebugUtilsMessengerCallbackDataEXT);
        break;
    case 1000128004:
        dump_json_pNext<const VkDebugUtilsMessengerCreateInfoEXT>(static_cast<const VkDebugUtilsMessengerCreateInfoEXT*>(object), settings, "VkDebugUtilsMessengerCreateInfoEXT", indents, dump_json_VkDebugUtilsMessengerCreateInfoEXT);
        break;
    case 1000128001:
        dump_json_pNext<const VkDebugUtilsObjectTagInfoEXT>(static_cast<const VkDebugUtilsObjectTagInfoEXT*>(object), settings, "VkDebugUtilsObjectTagInfoEXT", indents, dump_json_VkDebugUtilsObjectTagInfoEXT);
        break;
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
    case 1000129000:
        dump_json_pNext<const VkAndroidHardwareBufferUsageANDROID>(static_cast<const VkAndroidHardwareBufferUsageANDROID*>(object), settings, "VkAndroidHardwareBufferUsageANDROID", indents, dump_json_VkAndroidHardwareBufferUsageANDROID);
        break;
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
    case 1000129001:
        dump_json_pNext<const VkAndroidHardwareBufferPropertiesANDROID>(static_cast<const VkAndroidHardwareBufferPropertiesANDROID*>(object), settings, "VkAndroidHardwareBufferPropertiesANDROID", indents, dump_json_VkAndroidHardwareBufferPropertiesANDROID);
        break;
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
    case 1000129002:
        dump_json_pNext<const VkAndroidHardwareBufferFormatPropertiesANDROID>(static_cast<const VkAndroidHardwareBufferFormatPropertiesANDROID*>(object), settings, "VkAndroidHardwareBufferFormatPropertiesANDROID", indents, dump_json_VkAndroidHardwareBufferFormatPropertiesANDROID);
        break;
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
    case 1000129003:
        dump_json_pNext<const VkImportAndroidHardwareBufferInfoANDROID>(static_cast<const VkImportAndroidHardwareBufferInfoANDROID*>(object), settings, "VkImportAndroidHardwareBufferInfoANDROID", indents, dump_json_VkImportAndroidHardwareBufferInfoANDROID);
        break;
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
    case 1000129004:
        dump_json_pNext<const VkMemoryGetAndroidHardwareBufferInfoANDROID>(static_cast<const VkMemoryGetAndroidHardwareBufferInfoANDROID*>(object), settings, "VkMemoryGetAndroidHardwareBufferInfoANDROID", indents, dump_json_VkMemoryGetAndroidHardwareBufferInfoANDROID);
        break;
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
    case 1000129005:
        dump_json_pNext<const VkExternalFormatANDROID>(static_cast<const VkExternalFormatANDROID*>(object), settings, "VkExternalFormatANDROID", indents, dump_json_VkExternalFormatANDROID);
        break;
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
    case 1000129006:
        dump_json_pNext<const VkAndroidHardwareBufferFormatProperties2ANDROID>(static_cast<const VkAndroidHardwareBufferFormatProperties2ANDROID*>(object), settings, "VkAndroidHardwareBufferFormatProperties2ANDROID", indents, dump_json_VkAndroidHardwareBufferFormatProperties2ANDROID);
        break;
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_ENABLE_BETA_EXTENSIONS)
    case 1000134000:
        dump_json_pNext<const VkPhysicalDeviceShaderEnqueueFeaturesAMDX>(static_cast<const VkPhysicalDeviceShaderEnqueueFeaturesAMDX*>(object), settings, "VkPhysicalDeviceShaderEnqueueFeaturesAMDX", indents, dump_json_VkPhysicalDeviceShaderEnqueueFeaturesAMDX);
        break;
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
    case 1000134001:
        dump_json_pNext<const VkPhysicalDeviceShaderEnqueuePropertiesAMDX>(static_cast<const VkPhysicalDeviceShaderEnqueuePropertiesAMDX*>(object), settings, "VkPhysicalDeviceShaderEnqueuePropertiesAMDX", indents, dump_json_VkPhysicalDeviceShaderEnqueuePropertiesAMDX);
        break;
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
    case 1000134002:
        dump_json_pNext<const VkExecutionGraphPipelineScratchSizeAMDX>(static_cast<const VkExecutionGraphPipelineScratchSizeAMDX*>(object), settings, "VkExecutionGraphPipelineScratchSizeAMDX", indents, dump_json_VkExecutionGraphPipelineScratchSizeAMDX);
        break;
#endif // VK_ENABLE_BETA_EXTENSIONS
    case 1000290000:
        dump_json_pNext<const VkPipelineLibraryCreateInfoKHR>(static_cast<const VkPipelineLibraryCreateInfoKHR*>(object), settings, "VkPipelineLibraryCreateInfoKHR", indents, dump_json_VkPipelineLibraryCreateInfoKHR);
        break;
#if defined(VK_ENABLE_BETA_EXTENSIONS)
    case 1000134003:
        dump_json_pNext<const VkExecutionGraphPipelineCreateInfoAMDX>(static_cast<const VkExecutionGraphPipelineCreateInfoAMDX*>(object), settings, "VkExecutionGraphPipelineCreateInfoAMDX", indents, dump_json_VkExecutionGraphPipelineCreateInfoAMDX);
        break;
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
    case 1000134004:
        dump_json_pNext<const VkPipelineShaderStageNodeCreateInfoAMDX>(static_cast<const VkPipelineShaderStageNodeCreateInfoAMDX*>(object), settings, "VkPipelineShaderStageNodeCreateInfoAMDX", indents, dump_json_VkPipelineShaderStageNodeCreateInfoAMDX);
        break;
#endif // VK_ENABLE_BETA_EXTENSIONS
    case 1000143000:
        dump_json_pNext<const VkSampleLocationsInfoEXT>(static_cast<const VkSampleLocationsInfoEXT*>(object), settings, "VkSampleLocationsInfoEXT", indents, dump_json_VkSampleLocationsInfoEXT);
        break;
    case 1000143001:
        dump_json_pNext<const VkRenderPassSampleLocationsBeginInfoEXT>(static_cast<const VkRenderPassSampleLocationsBeginInfoEXT*>(object), settings, "VkRenderPassSampleLocationsBeginInfoEXT", indents, dump_json_VkRenderPassSampleLocationsBeginInfoEXT);
        break;
    case 1000143002:
        dump_json_pNext<const VkPipelineSampleLocationsStateCreateInfoEXT>(static_cast<const VkPipelineSampleLocationsStateCreateInfoEXT*>(object), settings, "VkPipelineSampleLocationsStateCreateInfoEXT", indents, dump_json_VkPipelineSampleLocationsStateCreateInfoEXT);
        break;
    case 1000143003:
        dump_json_pNext<const VkPhysicalDeviceSampleLocationsPropertiesEXT>(static_cast<const VkPhysicalDeviceSampleLocationsPropertiesEXT*>(object), settings, "VkPhysicalDeviceSampleLocationsPropertiesEXT", indents, dump_json_VkPhysicalDeviceSampleLocationsPropertiesEXT);
        break;
    case 1000143004:
        dump_json_pNext<const VkMultisamplePropertiesEXT>(static_cast<const VkMultisamplePropertiesEXT*>(object), settings, "VkMultisamplePropertiesEXT", indents, dump_json_VkMultisamplePropertiesEXT);
        break;
    case 1000148000:
        dump_json_pNext<const VkPhysicalDeviceBlendOperationAdvancedFeaturesEXT>(static_cast<const VkPhysicalDeviceBlendOperationAdvancedFeaturesEXT*>(object), settings, "VkPhysicalDeviceBlendOperationAdvancedFeaturesEXT", indents, dump_json_VkPhysicalDeviceBlendOperationAdvancedFeaturesEXT);
        break;
    case 1000148001:
        dump_json_pNext<const VkPhysicalDeviceBlendOperationAdvancedPropertiesEXT>(static_cast<const VkPhysicalDeviceBlendOperationAdvancedPropertiesEXT*>(object), settings, "VkPhysicalDeviceBlendOperationAdvancedPropertiesEXT", indents, dump_json_VkPhysicalDeviceBlendOperationAdvancedPropertiesEXT);
        break;
    case 1000148002:
        dump_json_pNext<const VkPipelineColorBlendAdvancedStateCreateInfoEXT>(static_cast<const VkPipelineColorBlendAdvancedStateCreateInfoEXT*>(object), settings, "VkPipelineColorBlendAdvancedStateCreateInfoEXT", indents, dump_json_VkPipelineColorBlendAdvancedStateCreateInfoEXT);
        break;
    case 1000149000:
        dump_json_pNext<const VkPipelineCoverageToColorStateCreateInfoNV>(static_cast<const VkPipelineCoverageToColorStateCreateInfoNV*>(object), settings, "VkPipelineCoverageToColorStateCreateInfoNV", indents, dump_json_VkPipelineCoverageToColorStateCreateInfoNV);
        break;
    case 1000150005:
        dump_json_pNext<const VkAccelerationStructureGeometryTrianglesDataKHR>(static_cast<const VkAccelerationStructureGeometryTrianglesDataKHR*>(object), settings, "VkAccelerationStructureGeometryTrianglesDataKHR", indents, dump_json_VkAccelerationStructureGeometryTrianglesDataKHR);
        break;
    case 1000150003:
        dump_json_pNext<const VkAccelerationStructureGeometryAabbsDataKHR>(static_cast<const VkAccelerationStructureGeometryAabbsDataKHR*>(object), settings, "VkAccelerationStructureGeometryAabbsDataKHR", indents, dump_json_VkAccelerationStructureGeometryAabbsDataKHR);
        break;
    case 1000150004:
        dump_json_pNext<const VkAccelerationStructureGeometryInstancesDataKHR>(static_cast<const VkAccelerationStructureGeometryInstancesDataKHR*>(object), settings, "VkAccelerationStructureGeometryInstancesDataKHR", indents, dump_json_VkAccelerationStructureGeometryInstancesDataKHR);
        break;
    case 1000150006:
        dump_json_pNext<const VkAccelerationStructureGeometryKHR>(static_cast<const VkAccelerationStructureGeometryKHR*>(object), settings, "VkAccelerationStructureGeometryKHR", indents, dump_json_VkAccelerationStructureGeometryKHR);
        break;
    case 1000150000:
        dump_json_pNext<const VkAccelerationStructureBuildGeometryInfoKHR>(static_cast<const VkAccelerationStructureBuildGeometryInfoKHR*>(object), settings, "VkAccelerationStructureBuildGeometryInfoKHR", indents, dump_json_VkAccelerationStructureBuildGeometryInfoKHR);
        break;
    case 1000150017:
        dump_json_pNext<const VkAccelerationStructureCreateInfoKHR>(static_cast<const VkAccelerationStructureCreateInfoKHR*>(object), settings, "VkAccelerationStructureCreateInfoKHR", indents, dump_json_VkAccelerationStructureCreateInfoKHR);
        break;
    case 1000150007:
        dump_json_pNext<const VkWriteDescriptorSetAccelerationStructureKHR>(static_cast<const VkWriteDescriptorSetAccelerationStructureKHR*>(object), settings, "VkWriteDescriptorSetAccelerationStructureKHR", indents, dump_json_VkWriteDescriptorSetAccelerationStructureKHR);
        break;
    case 1000150013:
        dump_json_pNext<const VkPhysicalDeviceAccelerationStructureFeaturesKHR>(static_cast<const VkPhysicalDeviceAccelerationStructureFeaturesKHR*>(object), settings, "VkPhysicalDeviceAccelerationStructureFeaturesKHR", indents, dump_json_VkPhysicalDeviceAccelerationStructureFeaturesKHR);
        break;
    case 1000150014:
        dump_json_pNext<const VkPhysicalDeviceAccelerationStructurePropertiesKHR>(static_cast<const VkPhysicalDeviceAccelerationStructurePropertiesKHR*>(object), settings, "VkPhysicalDeviceAccelerationStructurePropertiesKHR", indents, dump_json_VkPhysicalDeviceAccelerationStructurePropertiesKHR);
        break;
    case 1000150002:
        dump_json_pNext<const VkAccelerationStructureDeviceAddressInfoKHR>(static_cast<const VkAccelerationStructureDeviceAddressInfoKHR*>(object), settings, "VkAccelerationStructureDeviceAddressInfoKHR", indents, dump_json_VkAccelerationStructureDeviceAddressInfoKHR);
        break;
    case 1000150009:
        dump_json_pNext<const VkAccelerationStructureVersionInfoKHR>(static_cast<const VkAccelerationStructureVersionInfoKHR*>(object), settings, "VkAccelerationStructureVersionInfoKHR", indents, dump_json_VkAccelerationStructureVersionInfoKHR);
        break;
    case 1000150011:
        dump_json_pNext<const VkCopyAccelerationStructureToMemoryInfoKHR>(static_cast<const VkCopyAccelerationStructureToMemoryInfoKHR*>(object), settings, "VkCopyAccelerationStructureToMemoryInfoKHR", indents, dump_json_VkCopyAccelerationStructureToMemoryInfoKHR);
        break;
    case 1000150012:
        dump_json_pNext<const VkCopyMemoryToAccelerationStructureInfoKHR>(static_cast<const VkCopyMemoryToAccelerationStructureInfoKHR*>(object), settings, "VkCopyMemoryToAccelerationStructureInfoKHR", indents, dump_json_VkCopyMemoryToAccelerationStructureInfoKHR);
        break;
    case 1000150010:
        dump_json_pNext<const VkCopyAccelerationStructureInfoKHR>(static_cast<const VkCopyAccelerationStructureInfoKHR*>(object), settings, "VkCopyAccelerationStructureInfoKHR", indents, dump_json_VkCopyAccelerationStructureInfoKHR);
        break;
    case 1000150020:
        dump_json_pNext<const VkAccelerationStructureBuildSizesInfoKHR>(static_cast<const VkAccelerationStructureBuildSizesInfoKHR*>(object), settings, "VkAccelerationStructureBuildSizesInfoKHR", indents, dump_json_VkAccelerationStructureBuildSizesInfoKHR);
        break;
    case 1000152000:
        dump_json_pNext<const VkPipelineCoverageModulationStateCreateInfoNV>(static_cast<const VkPipelineCoverageModulationStateCreateInfoNV*>(object), settings, "VkPipelineCoverageModulationStateCreateInfoNV", indents, dump_json_VkPipelineCoverageModulationStateCreateInfoNV);
        break;
    case 1000154001:
        dump_json_pNext<const VkPhysicalDeviceShaderSMBuiltinsPropertiesNV>(static_cast<const VkPhysicalDeviceShaderSMBuiltinsPropertiesNV*>(object), settings, "VkPhysicalDeviceShaderSMBuiltinsPropertiesNV", indents, dump_json_VkPhysicalDeviceShaderSMBuiltinsPropertiesNV);
        break;
    case 1000154000:
        dump_json_pNext<const VkPhysicalDeviceShaderSMBuiltinsFeaturesNV>(static_cast<const VkPhysicalDeviceShaderSMBuiltinsFeaturesNV*>(object), settings, "VkPhysicalDeviceShaderSMBuiltinsFeaturesNV", indents, dump_json_VkPhysicalDeviceShaderSMBuiltinsFeaturesNV);
        break;
    case 1000158000:
        dump_json_pNext<const VkDrmFormatModifierPropertiesListEXT>(static_cast<const VkDrmFormatModifierPropertiesListEXT*>(object), settings, "VkDrmFormatModifierPropertiesListEXT", indents, dump_json_VkDrmFormatModifierPropertiesListEXT);
        break;
    case 1000158002:
        dump_json_pNext<const VkPhysicalDeviceImageDrmFormatModifierInfoEXT>(static_cast<const VkPhysicalDeviceImageDrmFormatModifierInfoEXT*>(object), settings, "VkPhysicalDeviceImageDrmFormatModifierInfoEXT", indents, dump_json_VkPhysicalDeviceImageDrmFormatModifierInfoEXT);
        break;
    case 1000158003:
        dump_json_pNext<const VkImageDrmFormatModifierListCreateInfoEXT>(static_cast<const VkImageDrmFormatModifierListCreateInfoEXT*>(object), settings, "VkImageDrmFormatModifierListCreateInfoEXT", indents, dump_json_VkImageDrmFormatModifierListCreateInfoEXT);
        break;
    case 1000158004:
        dump_json_pNext<const VkImageDrmFormatModifierExplicitCreateInfoEXT>(static_cast<const VkImageDrmFormatModifierExplicitCreateInfoEXT*>(object), settings, "VkImageDrmFormatModifierExplicitCreateInfoEXT", indents, dump_json_VkImageDrmFormatModifierExplicitCreateInfoEXT);
        break;
    case 1000158005:
        dump_json_pNext<const VkImageDrmFormatModifierPropertiesEXT>(static_cast<const VkImageDrmFormatModifierPropertiesEXT*>(object), settings, "VkImageDrmFormatModifierPropertiesEXT", indents, dump_json_VkImageDrmFormatModifierPropertiesEXT);
        break;
    case 1000158006:
        dump_json_pNext<const VkDrmFormatModifierPropertiesList2EXT>(static_cast<const VkDrmFormatModifierPropertiesList2EXT*>(object), settings, "VkDrmFormatModifierPropertiesList2EXT", indents, dump_json_VkDrmFormatModifierPropertiesList2EXT);
        break;
    case 1000160000:
        dump_json_pNext<const VkValidationCacheCreateInfoEXT>(static_cast<const VkValidationCacheCreateInfoEXT*>(object), settings, "VkValidationCacheCreateInfoEXT", indents, dump_json_VkValidationCacheCreateInfoEXT);
        break;
    case 1000160001:
        dump_json_pNext<const VkShaderModuleValidationCacheCreateInfoEXT>(static_cast<const VkShaderModuleValidationCacheCreateInfoEXT*>(object), settings, "VkShaderModuleValidationCacheCreateInfoEXT", indents, dump_json_VkShaderModuleValidationCacheCreateInfoEXT);
        break;
#if defined(VK_ENABLE_BETA_EXTENSIONS)
    case 1000163000:
        dump_json_pNext<const VkPhysicalDevicePortabilitySubsetFeaturesKHR>(static_cast<const VkPhysicalDevicePortabilitySubsetFeaturesKHR*>(object), settings, "VkPhysicalDevicePortabilitySubsetFeaturesKHR", indents, dump_json_VkPhysicalDevicePortabilitySubsetFeaturesKHR);
        break;
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
    case 1000163001:
        dump_json_pNext<const VkPhysicalDevicePortabilitySubsetPropertiesKHR>(static_cast<const VkPhysicalDevicePortabilitySubsetPropertiesKHR*>(object), settings, "VkPhysicalDevicePortabilitySubsetPropertiesKHR", indents, dump_json_VkPhysicalDevicePortabilitySubsetPropertiesKHR);
        break;
#endif // VK_ENABLE_BETA_EXTENSIONS
    case 1000164000:
        dump_json_pNext<const VkPipelineViewportShadingRateImageStateCreateInfoNV>(static_cast<const VkPipelineViewportShadingRateImageStateCreateInfoNV*>(object), settings, "VkPipelineViewportShadingRateImageStateCreateInfoNV", indents, dump_json_VkPipelineViewportShadingRateImageStateCreateInfoNV);
        break;
    case 1000164001:
        dump_json_pNext<const VkPhysicalDeviceShadingRateImageFeaturesNV>(static_cast<const VkPhysicalDeviceShadingRateImageFeaturesNV*>(object), settings, "VkPhysicalDeviceShadingRateImageFeaturesNV", indents, dump_json_VkPhysicalDeviceShadingRateImageFeaturesNV);
        break;
    case 1000164002:
        dump_json_pNext<const VkPhysicalDeviceShadingRateImagePropertiesNV>(static_cast<const VkPhysicalDeviceShadingRateImagePropertiesNV*>(object), settings, "VkPhysicalDeviceShadingRateImagePropertiesNV", indents, dump_json_VkPhysicalDeviceShadingRateImagePropertiesNV);
        break;
    case 1000164005:
        dump_json_pNext<const VkPipelineViewportCoarseSampleOrderStateCreateInfoNV>(static_cast<const VkPipelineViewportCoarseSampleOrderStateCreateInfoNV*>(object), settings, "VkPipelineViewportCoarseSampleOrderStateCreateInfoNV", indents, dump_json_VkPipelineViewportCoarseSampleOrderStateCreateInfoNV);
        break;
    case 1000165011:
        dump_json_pNext<const VkRayTracingShaderGroupCreateInfoNV>(static_cast<const VkRayTracingShaderGroupCreateInfoNV*>(object), settings, "VkRayTracingShaderGroupCreateInfoNV", indents, dump_json_VkRayTracingShaderGroupCreateInfoNV);
        break;
    case 1000165000:
        dump_json_pNext<const VkRayTracingPipelineCreateInfoNV>(static_cast<const VkRayTracingPipelineCreateInfoNV*>(object), settings, "VkRayTracingPipelineCreateInfoNV", indents, dump_json_VkRayTracingPipelineCreateInfoNV);
        break;
    case 1000165004:
        dump_json_pNext<const VkGeometryTrianglesNV>(static_cast<const VkGeometryTrianglesNV*>(object), settings, "VkGeometryTrianglesNV", indents, dump_json_VkGeometryTrianglesNV);
        break;
    case 1000165005:
        dump_json_pNext<const VkGeometryAABBNV>(static_cast<const VkGeometryAABBNV*>(object), settings, "VkGeometryAABBNV", indents, dump_json_VkGeometryAABBNV);
        break;
    case 1000165003:
        dump_json_pNext<const VkGeometryNV>(static_cast<const VkGeometryNV*>(object), settings, "VkGeometryNV", indents, dump_json_VkGeometryNV);
        break;
    case 1000165012:
        dump_json_pNext<const VkAccelerationStructureInfoNV>(static_cast<const VkAccelerationStructureInfoNV*>(object), settings, "VkAccelerationStructureInfoNV", indents, dump_json_VkAccelerationStructureInfoNV);
        break;
    case 1000165001:
        dump_json_pNext<const VkAccelerationStructureCreateInfoNV>(static_cast<const VkAccelerationStructureCreateInfoNV*>(object), settings, "VkAccelerationStructureCreateInfoNV", indents, dump_json_VkAccelerationStructureCreateInfoNV);
        break;
    case 1000165006:
        dump_json_pNext<const VkBindAccelerationStructureMemoryInfoNV>(static_cast<const VkBindAccelerationStructureMemoryInfoNV*>(object), settings, "VkBindAccelerationStructureMemoryInfoNV", indents, dump_json_VkBindAccelerationStructureMemoryInfoNV);
        break;
    case 1000165007:
        dump_json_pNext<const VkWriteDescriptorSetAccelerationStructureNV>(static_cast<const VkWriteDescriptorSetAccelerationStructureNV*>(object), settings, "VkWriteDescriptorSetAccelerationStructureNV", indents, dump_json_VkWriteDescriptorSetAccelerationStructureNV);
        break;
    case 1000165008:
        dump_json_pNext<const VkAccelerationStructureMemoryRequirementsInfoNV>(static_cast<const VkAccelerationStructureMemoryRequirementsInfoNV*>(object), settings, "VkAccelerationStructureMemoryRequirementsInfoNV", indents, dump_json_VkAccelerationStructureMemoryRequirementsInfoNV);
        break;
    case 1000165009:
        dump_json_pNext<const VkPhysicalDeviceRayTracingPropertiesNV>(static_cast<const VkPhysicalDeviceRayTracingPropertiesNV*>(object), settings, "VkPhysicalDeviceRayTracingPropertiesNV", indents, dump_json_VkPhysicalDeviceRayTracingPropertiesNV);
        break;
    case 1000166000:
        dump_json_pNext<const VkPhysicalDeviceRepresentativeFragmentTestFeaturesNV>(static_cast<const VkPhysicalDeviceRepresentativeFragmentTestFeaturesNV*>(object), settings, "VkPhysicalDeviceRepresentativeFragmentTestFeaturesNV", indents, dump_json_VkPhysicalDeviceRepresentativeFragmentTestFeaturesNV);
        break;
    case 1000166001:
        dump_json_pNext<const VkPipelineRepresentativeFragmentTestStateCreateInfoNV>(static_cast<const VkPipelineRepresentativeFragmentTestStateCreateInfoNV*>(object), settings, "VkPipelineRepresentativeFragmentTestStateCreateInfoNV", indents, dump_json_VkPipelineRepresentativeFragmentTestStateCreateInfoNV);
        break;
    case 1000170000:
        dump_json_pNext<const VkPhysicalDeviceImageViewImageFormatInfoEXT>(static_cast<const VkPhysicalDeviceImageViewImageFormatInfoEXT*>(object), settings, "VkPhysicalDeviceImageViewImageFormatInfoEXT", indents, dump_json_VkPhysicalDeviceImageViewImageFormatInfoEXT);
        break;
    case 1000170001:
        dump_json_pNext<const VkFilterCubicImageViewImageFormatPropertiesEXT>(static_cast<const VkFilterCubicImageViewImageFormatPropertiesEXT*>(object), settings, "VkFilterCubicImageViewImageFormatPropertiesEXT", indents, dump_json_VkFilterCubicImageViewImageFormatPropertiesEXT);
        break;
    case 1000174000:
        dump_json_pNext<const VkDeviceQueueGlobalPriorityCreateInfoKHR>(static_cast<const VkDeviceQueueGlobalPriorityCreateInfoKHR*>(object), settings, "VkDeviceQueueGlobalPriorityCreateInfoKHR", indents, dump_json_VkDeviceQueueGlobalPriorityCreateInfoKHR);
        break;
    case 1000178000:
        dump_json_pNext<const VkImportMemoryHostPointerInfoEXT>(static_cast<const VkImportMemoryHostPointerInfoEXT*>(object), settings, "VkImportMemoryHostPointerInfoEXT", indents, dump_json_VkImportMemoryHostPointerInfoEXT);
        break;
    case 1000178001:
        dump_json_pNext<const VkMemoryHostPointerPropertiesEXT>(static_cast<const VkMemoryHostPointerPropertiesEXT*>(object), settings, "VkMemoryHostPointerPropertiesEXT", indents, dump_json_VkMemoryHostPointerPropertiesEXT);
        break;
    case 1000178002:
        dump_json_pNext<const VkPhysicalDeviceExternalMemoryHostPropertiesEXT>(static_cast<const VkPhysicalDeviceExternalMemoryHostPropertiesEXT*>(object), settings, "VkPhysicalDeviceExternalMemoryHostPropertiesEXT", indents, dump_json_VkPhysicalDeviceExternalMemoryHostPropertiesEXT);
        break;
    case 1000181000:
        dump_json_pNext<const VkPhysicalDeviceShaderClockFeaturesKHR>(static_cast<const VkPhysicalDeviceShaderClockFeaturesKHR*>(object), settings, "VkPhysicalDeviceShaderClockFeaturesKHR", indents, dump_json_VkPhysicalDeviceShaderClockFeaturesKHR);
        break;
    case 1000183000:
        dump_json_pNext<const VkPipelineCompilerControlCreateInfoAMD>(static_cast<const VkPipelineCompilerControlCreateInfoAMD*>(object), settings, "VkPipelineCompilerControlCreateInfoAMD", indents, dump_json_VkPipelineCompilerControlCreateInfoAMD);
        break;
    case 1000184000:
        dump_json_pNext<const VkCalibratedTimestampInfoKHR>(static_cast<const VkCalibratedTimestampInfoKHR*>(object), settings, "VkCalibratedTimestampInfoKHR", indents, dump_json_VkCalibratedTimestampInfoKHR);
        break;
    case 1000185000:
        dump_json_pNext<const VkPhysicalDeviceShaderCorePropertiesAMD>(static_cast<const VkPhysicalDeviceShaderCorePropertiesAMD*>(object), settings, "VkPhysicalDeviceShaderCorePropertiesAMD", indents, dump_json_VkPhysicalDeviceShaderCorePropertiesAMD);
        break;
    case 1000187003:
        dump_json_pNext<const VkVideoDecodeH265ProfileInfoKHR>(static_cast<const VkVideoDecodeH265ProfileInfoKHR*>(object), settings, "VkVideoDecodeH265ProfileInfoKHR", indents, dump_json_VkVideoDecodeH265ProfileInfoKHR);
        break;
    case 1000187000:
        dump_json_pNext<const VkVideoDecodeH265CapabilitiesKHR>(static_cast<const VkVideoDecodeH265CapabilitiesKHR*>(object), settings, "VkVideoDecodeH265CapabilitiesKHR", indents, dump_json_VkVideoDecodeH265CapabilitiesKHR);
        break;
    case 1000187002:
        dump_json_pNext<const VkVideoDecodeH265SessionParametersAddInfoKHR>(static_cast<const VkVideoDecodeH265SessionParametersAddInfoKHR*>(object), settings, "VkVideoDecodeH265SessionParametersAddInfoKHR", indents, dump_json_VkVideoDecodeH265SessionParametersAddInfoKHR);
        break;
    case 1000187001:
        dump_json_pNext<const VkVideoDecodeH265SessionParametersCreateInfoKHR>(static_cast<const VkVideoDecodeH265SessionParametersCreateInfoKHR*>(object), settings, "VkVideoDecodeH265SessionParametersCreateInfoKHR", indents, dump_json_VkVideoDecodeH265SessionParametersCreateInfoKHR);
        break;
    case 1000187004:
        dump_json_pNext<const VkVideoDecodeH265PictureInfoKHR>(static_cast<const VkVideoDecodeH265PictureInfoKHR*>(object), settings, "VkVideoDecodeH265PictureInfoKHR", indents, dump_json_VkVideoDecodeH265PictureInfoKHR);
        break;
    case 1000187005:
        dump_json_pNext<const VkVideoDecodeH265DpbSlotInfoKHR>(static_cast<const VkVideoDecodeH265DpbSlotInfoKHR*>(object), settings, "VkVideoDecodeH265DpbSlotInfoKHR", indents, dump_json_VkVideoDecodeH265DpbSlotInfoKHR);
        break;
    case 1000388000:
        dump_json_pNext<const VkPhysicalDeviceGlobalPriorityQueryFeaturesKHR>(static_cast<const VkPhysicalDeviceGlobalPriorityQueryFeaturesKHR*>(object), settings, "VkPhysicalDeviceGlobalPriorityQueryFeaturesKHR", indents, dump_json_VkPhysicalDeviceGlobalPriorityQueryFeaturesKHR);
        break;
    case 1000388001:
        dump_json_pNext<const VkQueueFamilyGlobalPriorityPropertiesKHR>(static_cast<const VkQueueFamilyGlobalPriorityPropertiesKHR*>(object), settings, "VkQueueFamilyGlobalPriorityPropertiesKHR", indents, dump_json_VkQueueFamilyGlobalPriorityPropertiesKHR);
        break;
    case 1000189000:
        dump_json_pNext<const VkDeviceMemoryOverallocationCreateInfoAMD>(static_cast<const VkDeviceMemoryOverallocationCreateInfoAMD*>(object), settings, "VkDeviceMemoryOverallocationCreateInfoAMD", indents, dump_json_VkDeviceMemoryOverallocationCreateInfoAMD);
        break;
    case 1000190000:
        dump_json_pNext<const VkPhysicalDeviceVertexAttributeDivisorPropertiesEXT>(static_cast<const VkPhysicalDeviceVertexAttributeDivisorPropertiesEXT*>(object), settings, "VkPhysicalDeviceVertexAttributeDivisorPropertiesEXT", indents, dump_json_VkPhysicalDeviceVertexAttributeDivisorPropertiesEXT);
        break;
    case 1000190001:
        dump_json_pNext<const VkPipelineVertexInputDivisorStateCreateInfoKHR>(static_cast<const VkPipelineVertexInputDivisorStateCreateInfoKHR*>(object), settings, "VkPipelineVertexInputDivisorStateCreateInfoKHR", indents, dump_json_VkPipelineVertexInputDivisorStateCreateInfoKHR);
        break;
    case 1000190002:
        dump_json_pNext<const VkPhysicalDeviceVertexAttributeDivisorFeaturesKHR>(static_cast<const VkPhysicalDeviceVertexAttributeDivisorFeaturesKHR*>(object), settings, "VkPhysicalDeviceVertexAttributeDivisorFeaturesKHR", indents, dump_json_VkPhysicalDeviceVertexAttributeDivisorFeaturesKHR);
        break;
#if defined(VK_USE_PLATFORM_GGP)
    case 1000191000:
        dump_json_pNext<const VkPresentFrameTokenGGP>(static_cast<const VkPresentFrameTokenGGP*>(object), settings, "VkPresentFrameTokenGGP", indents, dump_json_VkPresentFrameTokenGGP);
        break;
#endif // VK_USE_PLATFORM_GGP
    case 1000201000:
        dump_json_pNext<const VkPhysicalDeviceComputeShaderDerivativesFeaturesNV>(static_cast<const VkPhysicalDeviceComputeShaderDerivativesFeaturesNV*>(object), settings, "VkPhysicalDeviceComputeShaderDerivativesFeaturesNV", indents, dump_json_VkPhysicalDeviceComputeShaderDerivativesFeaturesNV);
        break;
    case 1000202000:
        dump_json_pNext<const VkPhysicalDeviceMeshShaderFeaturesNV>(static_cast<const VkPhysicalDeviceMeshShaderFeaturesNV*>(object), settings, "VkPhysicalDeviceMeshShaderFeaturesNV", indents, dump_json_VkPhysicalDeviceMeshShaderFeaturesNV);
        break;
    case 1000202001:
        dump_json_pNext<const VkPhysicalDeviceMeshShaderPropertiesNV>(static_cast<const VkPhysicalDeviceMeshShaderPropertiesNV*>(object), settings, "VkPhysicalDeviceMeshShaderPropertiesNV", indents, dump_json_VkPhysicalDeviceMeshShaderPropertiesNV);
        break;
    case 1000203000:
        dump_json_pNext<const VkPhysicalDeviceFragmentShaderBarycentricFeaturesKHR>(static_cast<const VkPhysicalDeviceFragmentShaderBarycentricFeaturesKHR*>(object), settings, "VkPhysicalDeviceFragmentShaderBarycentricFeaturesKHR", indents, dump_json_VkPhysicalDeviceFragmentShaderBarycentricFeaturesKHR);
        break;
    case 1000204000:
        dump_json_pNext<const VkPhysicalDeviceShaderImageFootprintFeaturesNV>(static_cast<const VkPhysicalDeviceShaderImageFootprintFeaturesNV*>(object), settings, "VkPhysicalDeviceShaderImageFootprintFeaturesNV", indents, dump_json_VkPhysicalDeviceShaderImageFootprintFeaturesNV);
        break;
    case 1000205000:
        dump_json_pNext<const VkPipelineViewportExclusiveScissorStateCreateInfoNV>(static_cast<const VkPipelineViewportExclusiveScissorStateCreateInfoNV*>(object), settings, "VkPipelineViewportExclusiveScissorStateCreateInfoNV", indents, dump_json_VkPipelineViewportExclusiveScissorStateCreateInfoNV);
        break;
    case 1000205002:
        dump_json_pNext<const VkPhysicalDeviceExclusiveScissorFeaturesNV>(static_cast<const VkPhysicalDeviceExclusiveScissorFeaturesNV*>(object), settings, "VkPhysicalDeviceExclusiveScissorFeaturesNV", indents, dump_json_VkPhysicalDeviceExclusiveScissorFeaturesNV);
        break;
    case 1000206001:
        dump_json_pNext<const VkQueueFamilyCheckpointPropertiesNV>(static_cast<const VkQueueFamilyCheckpointPropertiesNV*>(object), settings, "VkQueueFamilyCheckpointPropertiesNV", indents, dump_json_VkQueueFamilyCheckpointPropertiesNV);
        break;
    case 1000206000:
        dump_json_pNext<const VkCheckpointDataNV>(static_cast<const VkCheckpointDataNV*>(object), settings, "VkCheckpointDataNV", indents, dump_json_VkCheckpointDataNV);
        break;
    case 1000209000:
        dump_json_pNext<const VkPhysicalDeviceShaderIntegerFunctions2FeaturesINTEL>(static_cast<const VkPhysicalDeviceShaderIntegerFunctions2FeaturesINTEL*>(object), settings, "VkPhysicalDeviceShaderIntegerFunctions2FeaturesINTEL", indents, dump_json_VkPhysicalDeviceShaderIntegerFunctions2FeaturesINTEL);
        break;
    case 1000210001:
        dump_json_pNext<const VkInitializePerformanceApiInfoINTEL>(static_cast<const VkInitializePerformanceApiInfoINTEL*>(object), settings, "VkInitializePerformanceApiInfoINTEL", indents, dump_json_VkInitializePerformanceApiInfoINTEL);
        break;
    case 1000210000:
        dump_json_pNext<const VkQueryPoolPerformanceQueryCreateInfoINTEL>(static_cast<const VkQueryPoolPerformanceQueryCreateInfoINTEL*>(object), settings, "VkQueryPoolPerformanceQueryCreateInfoINTEL", indents, dump_json_VkQueryPoolPerformanceQueryCreateInfoINTEL);
        break;
    case 1000210002:
        dump_json_pNext<const VkPerformanceMarkerInfoINTEL>(static_cast<const VkPerformanceMarkerInfoINTEL*>(object), settings, "VkPerformanceMarkerInfoINTEL", indents, dump_json_VkPerformanceMarkerInfoINTEL);
        break;
    case 1000210003:
        dump_json_pNext<const VkPerformanceStreamMarkerInfoINTEL>(static_cast<const VkPerformanceStreamMarkerInfoINTEL*>(object), settings, "VkPerformanceStreamMarkerInfoINTEL", indents, dump_json_VkPerformanceStreamMarkerInfoINTEL);
        break;
    case 1000210004:
        dump_json_pNext<const VkPerformanceOverrideInfoINTEL>(static_cast<const VkPerformanceOverrideInfoINTEL*>(object), settings, "VkPerformanceOverrideInfoINTEL", indents, dump_json_VkPerformanceOverrideInfoINTEL);
        break;
    case 1000210005:
        dump_json_pNext<const VkPerformanceConfigurationAcquireInfoINTEL>(static_cast<const VkPerformanceConfigurationAcquireInfoINTEL*>(object), settings, "VkPerformanceConfigurationAcquireInfoINTEL", indents, dump_json_VkPerformanceConfigurationAcquireInfoINTEL);
        break;
    case 1000212000:
        dump_json_pNext<const VkPhysicalDevicePCIBusInfoPropertiesEXT>(static_cast<const VkPhysicalDevicePCIBusInfoPropertiesEXT*>(object), settings, "VkPhysicalDevicePCIBusInfoPropertiesEXT", indents, dump_json_VkPhysicalDevicePCIBusInfoPropertiesEXT);
        break;
    case 1000213000:
        dump_json_pNext<const VkDisplayNativeHdrSurfaceCapabilitiesAMD>(static_cast<const VkDisplayNativeHdrSurfaceCapabilitiesAMD*>(object), settings, "VkDisplayNativeHdrSurfaceCapabilitiesAMD", indents, dump_json_VkDisplayNativeHdrSurfaceCapabilitiesAMD);
        break;
    case 1000213001:
        dump_json_pNext<const VkSwapchainDisplayNativeHdrCreateInfoAMD>(static_cast<const VkSwapchainDisplayNativeHdrCreateInfoAMD*>(object), settings, "VkSwapchainDisplayNativeHdrCreateInfoAMD", indents, dump_json_VkSwapchainDisplayNativeHdrCreateInfoAMD);
        break;
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000214000:
        dump_json_pNext<const VkImagePipeSurfaceCreateInfoFUCHSIA>(static_cast<const VkImagePipeSurfaceCreateInfoFUCHSIA*>(object), settings, "VkImagePipeSurfaceCreateInfoFUCHSIA", indents, dump_json_VkImagePipeSurfaceCreateInfoFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_METAL_EXT)
    case 1000217000:
        dump_json_pNext<const VkMetalSurfaceCreateInfoEXT>(static_cast<const VkMetalSurfaceCreateInfoEXT*>(object), settings, "VkMetalSurfaceCreateInfoEXT", indents, dump_json_VkMetalSurfaceCreateInfoEXT);
        break;
#endif // VK_USE_PLATFORM_METAL_EXT
    case 1000218000:
        dump_json_pNext<const VkPhysicalDeviceFragmentDensityMapFeaturesEXT>(static_cast<const VkPhysicalDeviceFragmentDensityMapFeaturesEXT*>(object), settings, "VkPhysicalDeviceFragmentDensityMapFeaturesEXT", indents, dump_json_VkPhysicalDeviceFragmentDensityMapFeaturesEXT);
        break;
    case 1000218001:
        dump_json_pNext<const VkPhysicalDeviceFragmentDensityMapPropertiesEXT>(static_cast<const VkPhysicalDeviceFragmentDensityMapPropertiesEXT*>(object), settings, "VkPhysicalDeviceFragmentDensityMapPropertiesEXT", indents, dump_json_VkPhysicalDeviceFragmentDensityMapPropertiesEXT);
        break;
    case 1000218002:
        dump_json_pNext<const VkRenderPassFragmentDensityMapCreateInfoEXT>(static_cast<const VkRenderPassFragmentDensityMapCreateInfoEXT*>(object), settings, "VkRenderPassFragmentDensityMapCreateInfoEXT", indents, dump_json_VkRenderPassFragmentDensityMapCreateInfoEXT);
        break;
    case 1000226000:
        dump_json_pNext<const VkFragmentShadingRateAttachmentInfoKHR>(static_cast<const VkFragmentShadingRateAttachmentInfoKHR*>(object), settings, "VkFragmentShadingRateAttachmentInfoKHR", indents, dump_json_VkFragmentShadingRateAttachmentInfoKHR);
        break;
    case 1000226001:
        dump_json_pNext<const VkPipelineFragmentShadingRateStateCreateInfoKHR>(static_cast<const VkPipelineFragmentShadingRateStateCreateInfoKHR*>(object), settings, "VkPipelineFragmentShadingRateStateCreateInfoKHR", indents, dump_json_VkPipelineFragmentShadingRateStateCreateInfoKHR);
        break;
    case 1000226003:
        dump_json_pNext<const VkPhysicalDeviceFragmentShadingRateFeaturesKHR>(static_cast<const VkPhysicalDeviceFragmentShadingRateFeaturesKHR*>(object), settings, "VkPhysicalDeviceFragmentShadingRateFeaturesKHR", indents, dump_json_VkPhysicalDeviceFragmentShadingRateFeaturesKHR);
        break;
    case 1000226002:
        dump_json_pNext<const VkPhysicalDeviceFragmentShadingRatePropertiesKHR>(static_cast<const VkPhysicalDeviceFragmentShadingRatePropertiesKHR*>(object), settings, "VkPhysicalDeviceFragmentShadingRatePropertiesKHR", indents, dump_json_VkPhysicalDeviceFragmentShadingRatePropertiesKHR);
        break;
    case 1000226004:
        dump_json_pNext<const VkPhysicalDeviceFragmentShadingRateKHR>(static_cast<const VkPhysicalDeviceFragmentShadingRateKHR*>(object), settings, "VkPhysicalDeviceFragmentShadingRateKHR", indents, dump_json_VkPhysicalDeviceFragmentShadingRateKHR);
        break;
    case 1000227000:
        dump_json_pNext<const VkPhysicalDeviceShaderCoreProperties2AMD>(static_cast<const VkPhysicalDeviceShaderCoreProperties2AMD*>(object), settings, "VkPhysicalDeviceShaderCoreProperties2AMD", indents, dump_json_VkPhysicalDeviceShaderCoreProperties2AMD);
        break;
    case 1000229000:
        dump_json_pNext<const VkPhysicalDeviceCoherentMemoryFeaturesAMD>(static_cast<const VkPhysicalDeviceCoherentMemoryFeaturesAMD*>(object), settings, "VkPhysicalDeviceCoherentMemoryFeaturesAMD", indents, dump_json_VkPhysicalDeviceCoherentMemoryFeaturesAMD);
        break;
    case 1000232000:
        dump_json_pNext<const VkPhysicalDeviceDynamicRenderingLocalReadFeaturesKHR>(static_cast<const VkPhysicalDeviceDynamicRenderingLocalReadFeaturesKHR*>(object), settings, "VkPhysicalDeviceDynamicRenderingLocalReadFeaturesKHR", indents, dump_json_VkPhysicalDeviceDynamicRenderingLocalReadFeaturesKHR);
        break;
    case 1000232001:
        dump_json_pNext<const VkRenderingAttachmentLocationInfoKHR>(static_cast<const VkRenderingAttachmentLocationInfoKHR*>(object), settings, "VkRenderingAttachmentLocationInfoKHR", indents, dump_json_VkRenderingAttachmentLocationInfoKHR);
        break;
    case 1000232002:
        dump_json_pNext<const VkRenderingInputAttachmentIndexInfoKHR>(static_cast<const VkRenderingInputAttachmentIndexInfoKHR*>(object), settings, "VkRenderingInputAttachmentIndexInfoKHR", indents, dump_json_VkRenderingInputAttachmentIndexInfoKHR);
        break;
    case 1000234000:
        dump_json_pNext<const VkPhysicalDeviceShaderImageAtomicInt64FeaturesEXT>(static_cast<const VkPhysicalDeviceShaderImageAtomicInt64FeaturesEXT*>(object), settings, "VkPhysicalDeviceShaderImageAtomicInt64FeaturesEXT", indents, dump_json_VkPhysicalDeviceShaderImageAtomicInt64FeaturesEXT);
        break;
    case 1000235000:
        dump_json_pNext<const VkPhysicalDeviceShaderQuadControlFeaturesKHR>(static_cast<const VkPhysicalDeviceShaderQuadControlFeaturesKHR*>(object), settings, "VkPhysicalDeviceShaderQuadControlFeaturesKHR", indents, dump_json_VkPhysicalDeviceShaderQuadControlFeaturesKHR);
        break;
    case 1000237000:
        dump_json_pNext<const VkPhysicalDeviceMemoryBudgetPropertiesEXT>(static_cast<const VkPhysicalDeviceMemoryBudgetPropertiesEXT*>(object), settings, "VkPhysicalDeviceMemoryBudgetPropertiesEXT", indents, dump_json_VkPhysicalDeviceMemoryBudgetPropertiesEXT);
        break;
    case 1000238000:
        dump_json_pNext<const VkPhysicalDeviceMemoryPriorityFeaturesEXT>(static_cast<const VkPhysicalDeviceMemoryPriorityFeaturesEXT*>(object), settings, "VkPhysicalDeviceMemoryPriorityFeaturesEXT", indents, dump_json_VkPhysicalDeviceMemoryPriorityFeaturesEXT);
        break;
    case 1000238001:
        dump_json_pNext<const VkMemoryPriorityAllocateInfoEXT>(static_cast<const VkMemoryPriorityAllocateInfoEXT*>(object), settings, "VkMemoryPriorityAllocateInfoEXT", indents, dump_json_VkMemoryPriorityAllocateInfoEXT);
        break;
    case 1000239000:
        dump_json_pNext<const VkSurfaceProtectedCapabilitiesKHR>(static_cast<const VkSurfaceProtectedCapabilitiesKHR*>(object), settings, "VkSurfaceProtectedCapabilitiesKHR", indents, dump_json_VkSurfaceProtectedCapabilitiesKHR);
        break;
    case 1000240000:
        dump_json_pNext<const VkPhysicalDeviceDedicatedAllocationImageAliasingFeaturesNV>(static_cast<const VkPhysicalDeviceDedicatedAllocationImageAliasingFeaturesNV*>(object), settings, "VkPhysicalDeviceDedicatedAllocationImageAliasingFeaturesNV", indents, dump_json_VkPhysicalDeviceDedicatedAllocationImageAliasingFeaturesNV);
        break;
    case 1000244000:
        dump_json_pNext<const VkPhysicalDeviceBufferDeviceAddressFeaturesEXT>(static_cast<const VkPhysicalDeviceBufferDeviceAddressFeaturesEXT*>(object), settings, "VkPhysicalDeviceBufferDeviceAddressFeaturesEXT", indents, dump_json_VkPhysicalDeviceBufferDeviceAddressFeaturesEXT);
        break;
    case 1000244002:
        dump_json_pNext<const VkBufferDeviceAddressCreateInfoEXT>(static_cast<const VkBufferDeviceAddressCreateInfoEXT*>(object), settings, "VkBufferDeviceAddressCreateInfoEXT", indents, dump_json_VkBufferDeviceAddressCreateInfoEXT);
        break;
    case 1000247000:
        dump_json_pNext<const VkValidationFeaturesEXT>(static_cast<const VkValidationFeaturesEXT*>(object), settings, "VkValidationFeaturesEXT", indents, dump_json_VkValidationFeaturesEXT);
        break;
    case 1000248000:
        dump_json_pNext<const VkPhysicalDevicePresentWaitFeaturesKHR>(static_cast<const VkPhysicalDevicePresentWaitFeaturesKHR*>(object), settings, "VkPhysicalDevicePresentWaitFeaturesKHR", indents, dump_json_VkPhysicalDevicePresentWaitFeaturesKHR);
        break;
    case 1000249001:
        dump_json_pNext<const VkCooperativeMatrixPropertiesNV>(static_cast<const VkCooperativeMatrixPropertiesNV*>(object), settings, "VkCooperativeMatrixPropertiesNV", indents, dump_json_VkCooperativeMatrixPropertiesNV);
        break;
    case 1000249000:
        dump_json_pNext<const VkPhysicalDeviceCooperativeMatrixFeaturesNV>(static_cast<const VkPhysicalDeviceCooperativeMatrixFeaturesNV*>(object), settings, "VkPhysicalDeviceCooperativeMatrixFeaturesNV", indents, dump_json_VkPhysicalDeviceCooperativeMatrixFeaturesNV);
        break;
    case 1000249002:
        dump_json_pNext<const VkPhysicalDeviceCooperativeMatrixPropertiesNV>(static_cast<const VkPhysicalDeviceCooperativeMatrixPropertiesNV*>(object), settings, "VkPhysicalDeviceCooperativeMatrixPropertiesNV", indents, dump_json_VkPhysicalDeviceCooperativeMatrixPropertiesNV);
        break;
    case 1000250000:
        dump_json_pNext<const VkPhysicalDeviceCoverageReductionModeFeaturesNV>(static_cast<const VkPhysicalDeviceCoverageReductionModeFeaturesNV*>(object), settings, "VkPhysicalDeviceCoverageReductionModeFeaturesNV", indents, dump_json_VkPhysicalDeviceCoverageReductionModeFeaturesNV);
        break;
    case 1000250001:
        dump_json_pNext<const VkPipelineCoverageReductionStateCreateInfoNV>(static_cast<const VkPipelineCoverageReductionStateCreateInfoNV*>(object), settings, "VkPipelineCoverageReductionStateCreateInfoNV", indents, dump_json_VkPipelineCoverageReductionStateCreateInfoNV);
        break;
    case 1000250002:
        dump_json_pNext<const VkFramebufferMixedSamplesCombinationNV>(static_cast<const VkFramebufferMixedSamplesCombinationNV*>(object), settings, "VkFramebufferMixedSamplesCombinationNV", indents, dump_json_VkFramebufferMixedSamplesCombinationNV);
        break;
    case 1000251000:
        dump_json_pNext<const VkPhysicalDeviceFragmentShaderInterlockFeaturesEXT>(static_cast<const VkPhysicalDeviceFragmentShaderInterlockFeaturesEXT*>(object), settings, "VkPhysicalDeviceFragmentShaderInterlockFeaturesEXT", indents, dump_json_VkPhysicalDeviceFragmentShaderInterlockFeaturesEXT);
        break;
    case 1000252000:
        dump_json_pNext<const VkPhysicalDeviceYcbcrImageArraysFeaturesEXT>(static_cast<const VkPhysicalDeviceYcbcrImageArraysFeaturesEXT*>(object), settings, "VkPhysicalDeviceYcbcrImageArraysFeaturesEXT", indents, dump_json_VkPhysicalDeviceYcbcrImageArraysFeaturesEXT);
        break;
    case 1000254000:
        dump_json_pNext<const VkPhysicalDeviceProvokingVertexFeaturesEXT>(static_cast<const VkPhysicalDeviceProvokingVertexFeaturesEXT*>(object), settings, "VkPhysicalDeviceProvokingVertexFeaturesEXT", indents, dump_json_VkPhysicalDeviceProvokingVertexFeaturesEXT);
        break;
    case 1000254002:
        dump_json_pNext<const VkPhysicalDeviceProvokingVertexPropertiesEXT>(static_cast<const VkPhysicalDeviceProvokingVertexPropertiesEXT*>(object), settings, "VkPhysicalDeviceProvokingVertexPropertiesEXT", indents, dump_json_VkPhysicalDeviceProvokingVertexPropertiesEXT);
        break;
    case 1000254001:
        dump_json_pNext<const VkPipelineRasterizationProvokingVertexStateCreateInfoEXT>(static_cast<const VkPipelineRasterizationProvokingVertexStateCreateInfoEXT*>(object), settings, "VkPipelineRasterizationProvokingVertexStateCreateInfoEXT", indents, dump_json_VkPipelineRasterizationProvokingVertexStateCreateInfoEXT);
        break;
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000255000:
        dump_json_pNext<const VkSurfaceFullScreenExclusiveInfoEXT>(static_cast<const VkSurfaceFullScreenExclusiveInfoEXT*>(object), settings, "VkSurfaceFullScreenExclusiveInfoEXT", indents, dump_json_VkSurfaceFullScreenExclusiveInfoEXT);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000255002:
        dump_json_pNext<const VkSurfaceCapabilitiesFullScreenExclusiveEXT>(static_cast<const VkSurfaceCapabilitiesFullScreenExclusiveEXT*>(object), settings, "VkSurfaceCapabilitiesFullScreenExclusiveEXT", indents, dump_json_VkSurfaceCapabilitiesFullScreenExclusiveEXT);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
    case 1000255001:
        dump_json_pNext<const VkSurfaceFullScreenExclusiveWin32InfoEXT>(static_cast<const VkSurfaceFullScreenExclusiveWin32InfoEXT*>(object), settings, "VkSurfaceFullScreenExclusiveWin32InfoEXT", indents, dump_json_VkSurfaceFullScreenExclusiveWin32InfoEXT);
        break;
#endif // VK_USE_PLATFORM_WIN32_KHR
    case 1000256000:
        dump_json_pNext<const VkHeadlessSurfaceCreateInfoEXT>(static_cast<const VkHeadlessSurfaceCreateInfoEXT*>(object), settings, "VkHeadlessSurfaceCreateInfoEXT", indents, dump_json_VkHeadlessSurfaceCreateInfoEXT);
        break;
    case 1000259000:
        dump_json_pNext<const VkPhysicalDeviceLineRasterizationFeaturesKHR>(static_cast<const VkPhysicalDeviceLineRasterizationFeaturesKHR*>(object), settings, "VkPhysicalDeviceLineRasterizationFeaturesKHR", indents, dump_json_VkPhysicalDeviceLineRasterizationFeaturesKHR);
        break;
    case 1000259002:
        dump_json_pNext<const VkPhysicalDeviceLineRasterizationPropertiesKHR>(static_cast<const VkPhysicalDeviceLineRasterizationPropertiesKHR*>(object), settings, "VkPhysicalDeviceLineRasterizationPropertiesKHR", indents, dump_json_VkPhysicalDeviceLineRasterizationPropertiesKHR);
        break;
    case 1000259001:
        dump_json_pNext<const VkPipelineRasterizationLineStateCreateInfoKHR>(static_cast<const VkPipelineRasterizationLineStateCreateInfoKHR*>(object), settings, "VkPipelineRasterizationLineStateCreateInfoKHR", indents, dump_json_VkPipelineRasterizationLineStateCreateInfoKHR);
        break;
    case 1000260000:
        dump_json_pNext<const VkPhysicalDeviceShaderAtomicFloatFeaturesEXT>(static_cast<const VkPhysicalDeviceShaderAtomicFloatFeaturesEXT*>(object), settings, "VkPhysicalDeviceShaderAtomicFloatFeaturesEXT", indents, dump_json_VkPhysicalDeviceShaderAtomicFloatFeaturesEXT);
        break;
    case 1000265000:
        dump_json_pNext<const VkPhysicalDeviceIndexTypeUint8FeaturesKHR>(static_cast<const VkPhysicalDeviceIndexTypeUint8FeaturesKHR*>(object), settings, "VkPhysicalDeviceIndexTypeUint8FeaturesKHR", indents, dump_json_VkPhysicalDeviceIndexTypeUint8FeaturesKHR);
        break;
    case 1000267000:
        dump_json_pNext<const VkPhysicalDeviceExtendedDynamicStateFeaturesEXT>(static_cast<const VkPhysicalDeviceExtendedDynamicStateFeaturesEXT*>(object), settings, "VkPhysicalDeviceExtendedDynamicStateFeaturesEXT", indents, dump_json_VkPhysicalDeviceExtendedDynamicStateFeaturesEXT);
        break;
    case 1000269000:
        dump_json_pNext<const VkPhysicalDevicePipelineExecutablePropertiesFeaturesKHR>(static_cast<const VkPhysicalDevicePipelineExecutablePropertiesFeaturesKHR*>(object), settings, "VkPhysicalDevicePipelineExecutablePropertiesFeaturesKHR", indents, dump_json_VkPhysicalDevicePipelineExecutablePropertiesFeaturesKHR);
        break;
    case 1000269001:
        dump_json_pNext<const VkPipelineInfoKHR>(static_cast<const VkPipelineInfoKHR*>(object), settings, "VkPipelineInfoKHR", indents, dump_json_VkPipelineInfoKHR);
        break;
    case 1000269002:
        dump_json_pNext<const VkPipelineExecutablePropertiesKHR>(static_cast<const VkPipelineExecutablePropertiesKHR*>(object), settings, "VkPipelineExecutablePropertiesKHR", indents, dump_json_VkPipelineExecutablePropertiesKHR);
        break;
    case 1000269003:
        dump_json_pNext<const VkPipelineExecutableInfoKHR>(static_cast<const VkPipelineExecutableInfoKHR*>(object), settings, "VkPipelineExecutableInfoKHR", indents, dump_json_VkPipelineExecutableInfoKHR);
        break;
    case 1000269004:
        dump_json_pNext<const VkPipelineExecutableStatisticKHR>(static_cast<const VkPipelineExecutableStatisticKHR*>(object), settings, "VkPipelineExecutableStatisticKHR", indents, dump_json_VkPipelineExecutableStatisticKHR);
        break;
    case 1000269005:
        dump_json_pNext<const VkPipelineExecutableInternalRepresentationKHR>(static_cast<const VkPipelineExecutableInternalRepresentationKHR*>(object), settings, "VkPipelineExecutableInternalRepresentationKHR", indents, dump_json_VkPipelineExecutableInternalRepresentationKHR);
        break;
    case 1000270000:
        dump_json_pNext<const VkPhysicalDeviceHostImageCopyFeaturesEXT>(static_cast<const VkPhysicalDeviceHostImageCopyFeaturesEXT*>(object), settings, "VkPhysicalDeviceHostImageCopyFeaturesEXT", indents, dump_json_VkPhysicalDeviceHostImageCopyFeaturesEXT);
        break;
    case 1000270001:
        dump_json_pNext<const VkPhysicalDeviceHostImageCopyPropertiesEXT>(static_cast<const VkPhysicalDeviceHostImageCopyPropertiesEXT*>(object), settings, "VkPhysicalDeviceHostImageCopyPropertiesEXT", indents, dump_json_VkPhysicalDeviceHostImageCopyPropertiesEXT);
        break;
    case 1000270002:
        dump_json_pNext<const VkMemoryToImageCopyEXT>(static_cast<const VkMemoryToImageCopyEXT*>(object), settings, "VkMemoryToImageCopyEXT", indents, dump_json_VkMemoryToImageCopyEXT);
        break;
    case 1000270003:
        dump_json_pNext<const VkImageToMemoryCopyEXT>(static_cast<const VkImageToMemoryCopyEXT*>(object), settings, "VkImageToMemoryCopyEXT", indents, dump_json_VkImageToMemoryCopyEXT);
        break;
    case 1000270005:
        dump_json_pNext<const VkCopyMemoryToImageInfoEXT>(static_cast<const VkCopyMemoryToImageInfoEXT*>(object), settings, "VkCopyMemoryToImageInfoEXT", indents, dump_json_VkCopyMemoryToImageInfoEXT);
        break;
    case 1000270004:
        dump_json_pNext<const VkCopyImageToMemoryInfoEXT>(static_cast<const VkCopyImageToMemoryInfoEXT*>(object), settings, "VkCopyImageToMemoryInfoEXT", indents, dump_json_VkCopyImageToMemoryInfoEXT);
        break;
    case 1000270007:
        dump_json_pNext<const VkCopyImageToImageInfoEXT>(static_cast<const VkCopyImageToImageInfoEXT*>(object), settings, "VkCopyImageToImageInfoEXT", indents, dump_json_VkCopyImageToImageInfoEXT);
        break;
    case 1000270006:
        dump_json_pNext<const VkHostImageLayoutTransitionInfoEXT>(static_cast<const VkHostImageLayoutTransitionInfoEXT*>(object), settings, "VkHostImageLayoutTransitionInfoEXT", indents, dump_json_VkHostImageLayoutTransitionInfoEXT);
        break;
    case 1000270008:
        dump_json_pNext<const VkSubresourceHostMemcpySizeEXT>(static_cast<const VkSubresourceHostMemcpySizeEXT*>(object), settings, "VkSubresourceHostMemcpySizeEXT", indents, dump_json_VkSubresourceHostMemcpySizeEXT);
        break;
    case 1000270009:
        dump_json_pNext<const VkHostImageCopyDevicePerformanceQueryEXT>(static_cast<const VkHostImageCopyDevicePerformanceQueryEXT*>(object), settings, "VkHostImageCopyDevicePerformanceQueryEXT", indents, dump_json_VkHostImageCopyDevicePerformanceQueryEXT);
        break;
    case 1000338002:
        dump_json_pNext<const VkSubresourceLayout2KHR>(static_cast<const VkSubresourceLayout2KHR*>(object), settings, "VkSubresourceLayout2KHR", indents, dump_json_VkSubresourceLayout2KHR);
        break;
    case 1000338003:
        dump_json_pNext<const VkImageSubresource2KHR>(static_cast<const VkImageSubresource2KHR*>(object), settings, "VkImageSubresource2KHR", indents, dump_json_VkImageSubresource2KHR);
        break;
    case 1000271000:
        dump_json_pNext<const VkMemoryMapInfoKHR>(static_cast<const VkMemoryMapInfoKHR*>(object), settings, "VkMemoryMapInfoKHR", indents, dump_json_VkMemoryMapInfoKHR);
        break;
    case 1000271001:
        dump_json_pNext<const VkMemoryUnmapInfoKHR>(static_cast<const VkMemoryUnmapInfoKHR*>(object), settings, "VkMemoryUnmapInfoKHR", indents, dump_json_VkMemoryUnmapInfoKHR);
        break;
    case 1000272000:
        dump_json_pNext<const VkPhysicalDeviceMapMemoryPlacedFeaturesEXT>(static_cast<const VkPhysicalDeviceMapMemoryPlacedFeaturesEXT*>(object), settings, "VkPhysicalDeviceMapMemoryPlacedFeaturesEXT", indents, dump_json_VkPhysicalDeviceMapMemoryPlacedFeaturesEXT);
        break;
    case 1000272001:
        dump_json_pNext<const VkPhysicalDeviceMapMemoryPlacedPropertiesEXT>(static_cast<const VkPhysicalDeviceMapMemoryPlacedPropertiesEXT*>(object), settings, "VkPhysicalDeviceMapMemoryPlacedPropertiesEXT", indents, dump_json_VkPhysicalDeviceMapMemoryPlacedPropertiesEXT);
        break;
    case 1000272002:
        dump_json_pNext<const VkMemoryMapPlacedInfoEXT>(static_cast<const VkMemoryMapPlacedInfoEXT*>(object), settings, "VkMemoryMapPlacedInfoEXT", indents, dump_json_VkMemoryMapPlacedInfoEXT);
        break;
    case 1000273000:
        dump_json_pNext<const VkPhysicalDeviceShaderAtomicFloat2FeaturesEXT>(static_cast<const VkPhysicalDeviceShaderAtomicFloat2FeaturesEXT*>(object), settings, "VkPhysicalDeviceShaderAtomicFloat2FeaturesEXT", indents, dump_json_VkPhysicalDeviceShaderAtomicFloat2FeaturesEXT);
        break;
    case 1000274000:
        dump_json_pNext<const VkSurfacePresentModeEXT>(static_cast<const VkSurfacePresentModeEXT*>(object), settings, "VkSurfacePresentModeEXT", indents, dump_json_VkSurfacePresentModeEXT);
        break;
    case 1000274001:
        dump_json_pNext<const VkSurfacePresentScalingCapabilitiesEXT>(static_cast<const VkSurfacePresentScalingCapabilitiesEXT*>(object), settings, "VkSurfacePresentScalingCapabilitiesEXT", indents, dump_json_VkSurfacePresentScalingCapabilitiesEXT);
        break;
    case 1000274002:
        dump_json_pNext<const VkSurfacePresentModeCompatibilityEXT>(static_cast<const VkSurfacePresentModeCompatibilityEXT*>(object), settings, "VkSurfacePresentModeCompatibilityEXT", indents, dump_json_VkSurfacePresentModeCompatibilityEXT);
        break;
    case 1000275000:
        dump_json_pNext<const VkPhysicalDeviceSwapchainMaintenance1FeaturesEXT>(static_cast<const VkPhysicalDeviceSwapchainMaintenance1FeaturesEXT*>(object), settings, "VkPhysicalDeviceSwapchainMaintenance1FeaturesEXT", indents, dump_json_VkPhysicalDeviceSwapchainMaintenance1FeaturesEXT);
        break;
    case 1000275001:
        dump_json_pNext<const VkSwapchainPresentFenceInfoEXT>(static_cast<const VkSwapchainPresentFenceInfoEXT*>(object), settings, "VkSwapchainPresentFenceInfoEXT", indents, dump_json_VkSwapchainPresentFenceInfoEXT);
        break;
    case 1000275002:
        dump_json_pNext<const VkSwapchainPresentModesCreateInfoEXT>(static_cast<const VkSwapchainPresentModesCreateInfoEXT*>(object), settings, "VkSwapchainPresentModesCreateInfoEXT", indents, dump_json_VkSwapchainPresentModesCreateInfoEXT);
        break;
    case 1000275003:
        dump_json_pNext<const VkSwapchainPresentModeInfoEXT>(static_cast<const VkSwapchainPresentModeInfoEXT*>(object), settings, "VkSwapchainPresentModeInfoEXT", indents, dump_json_VkSwapchainPresentModeInfoEXT);
        break;
    case 1000275004:
        dump_json_pNext<const VkSwapchainPresentScalingCreateInfoEXT>(static_cast<const VkSwapchainPresentScalingCreateInfoEXT*>(object), settings, "VkSwapchainPresentScalingCreateInfoEXT", indents, dump_json_VkSwapchainPresentScalingCreateInfoEXT);
        break;
    case 1000275005:
        dump_json_pNext<const VkReleaseSwapchainImagesInfoEXT>(static_cast<const VkReleaseSwapchainImagesInfoEXT*>(object), settings, "VkReleaseSwapchainImagesInfoEXT", indents, dump_json_VkReleaseSwapchainImagesInfoEXT);
        break;
    case 1000277000:
        dump_json_pNext<const VkPhysicalDeviceDeviceGeneratedCommandsPropertiesNV>(static_cast<const VkPhysicalDeviceDeviceGeneratedCommandsPropertiesNV*>(object), settings, "VkPhysicalDeviceDeviceGeneratedCommandsPropertiesNV", indents, dump_json_VkPhysicalDeviceDeviceGeneratedCommandsPropertiesNV);
        break;
    case 1000277007:
        dump_json_pNext<const VkPhysicalDeviceDeviceGeneratedCommandsFeaturesNV>(static_cast<const VkPhysicalDeviceDeviceGeneratedCommandsFeaturesNV*>(object), settings, "VkPhysicalDeviceDeviceGeneratedCommandsFeaturesNV", indents, dump_json_VkPhysicalDeviceDeviceGeneratedCommandsFeaturesNV);
        break;
    case 1000277001:
        dump_json_pNext<const VkGraphicsShaderGroupCreateInfoNV>(static_cast<const VkGraphicsShaderGroupCreateInfoNV*>(object), settings, "VkGraphicsShaderGroupCreateInfoNV", indents, dump_json_VkGraphicsShaderGroupCreateInfoNV);
        break;
    case 1000277002:
        dump_json_pNext<const VkGraphicsPipelineShaderGroupsCreateInfoNV>(static_cast<const VkGraphicsPipelineShaderGroupsCreateInfoNV*>(object), settings, "VkGraphicsPipelineShaderGroupsCreateInfoNV", indents, dump_json_VkGraphicsPipelineShaderGroupsCreateInfoNV);
        break;
    case 1000277003:
        dump_json_pNext<const VkIndirectCommandsLayoutTokenNV>(static_cast<const VkIndirectCommandsLayoutTokenNV*>(object), settings, "VkIndirectCommandsLayoutTokenNV", indents, dump_json_VkIndirectCommandsLayoutTokenNV);
        break;
    case 1000277004:
        dump_json_pNext<const VkIndirectCommandsLayoutCreateInfoNV>(static_cast<const VkIndirectCommandsLayoutCreateInfoNV*>(object), settings, "VkIndirectCommandsLayoutCreateInfoNV", indents, dump_json_VkIndirectCommandsLayoutCreateInfoNV);
        break;
    case 1000277005:
        dump_json_pNext<const VkGeneratedCommandsInfoNV>(static_cast<const VkGeneratedCommandsInfoNV*>(object), settings, "VkGeneratedCommandsInfoNV", indents, dump_json_VkGeneratedCommandsInfoNV);
        break;
    case 1000277006:
        dump_json_pNext<const VkGeneratedCommandsMemoryRequirementsInfoNV>(static_cast<const VkGeneratedCommandsMemoryRequirementsInfoNV*>(object), settings, "VkGeneratedCommandsMemoryRequirementsInfoNV", indents, dump_json_VkGeneratedCommandsMemoryRequirementsInfoNV);
        break;
    case 1000278000:
        dump_json_pNext<const VkPhysicalDeviceInheritedViewportScissorFeaturesNV>(static_cast<const VkPhysicalDeviceInheritedViewportScissorFeaturesNV*>(object), settings, "VkPhysicalDeviceInheritedViewportScissorFeaturesNV", indents, dump_json_VkPhysicalDeviceInheritedViewportScissorFeaturesNV);
        break;
    case 1000278001:
        dump_json_pNext<const VkCommandBufferInheritanceViewportScissorInfoNV>(static_cast<const VkCommandBufferInheritanceViewportScissorInfoNV*>(object), settings, "VkCommandBufferInheritanceViewportScissorInfoNV", indents, dump_json_VkCommandBufferInheritanceViewportScissorInfoNV);
        break;
    case 1000281000:
        dump_json_pNext<const VkPhysicalDeviceTexelBufferAlignmentFeaturesEXT>(static_cast<const VkPhysicalDeviceTexelBufferAlignmentFeaturesEXT*>(object), settings, "VkPhysicalDeviceTexelBufferAlignmentFeaturesEXT", indents, dump_json_VkPhysicalDeviceTexelBufferAlignmentFeaturesEXT);
        break;
    case 1000282001:
        dump_json_pNext<const VkRenderPassTransformBeginInfoQCOM>(static_cast<const VkRenderPassTransformBeginInfoQCOM*>(object), settings, "VkRenderPassTransformBeginInfoQCOM", indents, dump_json_VkRenderPassTransformBeginInfoQCOM);
        break;
    case 1000282000:
        dump_json_pNext<const VkCommandBufferInheritanceRenderPassTransformInfoQCOM>(static_cast<const VkCommandBufferInheritanceRenderPassTransformInfoQCOM*>(object), settings, "VkCommandBufferInheritanceRenderPassTransformInfoQCOM", indents, dump_json_VkCommandBufferInheritanceRenderPassTransformInfoQCOM);
        break;
    case 1000283000:
        dump_json_pNext<const VkPhysicalDeviceDepthBiasControlFeaturesEXT>(static_cast<const VkPhysicalDeviceDepthBiasControlFeaturesEXT*>(object), settings, "VkPhysicalDeviceDepthBiasControlFeaturesEXT", indents, dump_json_VkPhysicalDeviceDepthBiasControlFeaturesEXT);
        break;
    case 1000283001:
        dump_json_pNext<const VkDepthBiasInfoEXT>(static_cast<const VkDepthBiasInfoEXT*>(object), settings, "VkDepthBiasInfoEXT", indents, dump_json_VkDepthBiasInfoEXT);
        break;
    case 1000283002:
        dump_json_pNext<const VkDepthBiasRepresentationInfoEXT>(static_cast<const VkDepthBiasRepresentationInfoEXT*>(object), settings, "VkDepthBiasRepresentationInfoEXT", indents, dump_json_VkDepthBiasRepresentationInfoEXT);
        break;
    case 1000284000:
        dump_json_pNext<const VkPhysicalDeviceDeviceMemoryReportFeaturesEXT>(static_cast<const VkPhysicalDeviceDeviceMemoryReportFeaturesEXT*>(object), settings, "VkPhysicalDeviceDeviceMemoryReportFeaturesEXT", indents, dump_json_VkPhysicalDeviceDeviceMemoryReportFeaturesEXT);
        break;
    case 1000284002:
        dump_json_pNext<const VkDeviceMemoryReportCallbackDataEXT>(static_cast<const VkDeviceMemoryReportCallbackDataEXT*>(object), settings, "VkDeviceMemoryReportCallbackDataEXT", indents, dump_json_VkDeviceMemoryReportCallbackDataEXT);
        break;
    case 1000284001:
        dump_json_pNext<const VkDeviceDeviceMemoryReportCreateInfoEXT>(static_cast<const VkDeviceDeviceMemoryReportCreateInfoEXT*>(object), settings, "VkDeviceDeviceMemoryReportCreateInfoEXT", indents, dump_json_VkDeviceDeviceMemoryReportCreateInfoEXT);
        break;
    case 1000286000:
        dump_json_pNext<const VkPhysicalDeviceRobustness2FeaturesEXT>(static_cast<const VkPhysicalDeviceRobustness2FeaturesEXT*>(object), settings, "VkPhysicalDeviceRobustness2FeaturesEXT", indents, dump_json_VkPhysicalDeviceRobustness2FeaturesEXT);
        break;
    case 1000286001:
        dump_json_pNext<const VkPhysicalDeviceRobustness2PropertiesEXT>(static_cast<const VkPhysicalDeviceRobustness2PropertiesEXT*>(object), settings, "VkPhysicalDeviceRobustness2PropertiesEXT", indents, dump_json_VkPhysicalDeviceRobustness2PropertiesEXT);
        break;
    case 1000287000:
        dump_json_pNext<const VkSamplerCustomBorderColorCreateInfoEXT>(static_cast<const VkSamplerCustomBorderColorCreateInfoEXT*>(object), settings, "VkSamplerCustomBorderColorCreateInfoEXT", indents, dump_json_VkSamplerCustomBorderColorCreateInfoEXT);
        break;
    case 1000287001:
        dump_json_pNext<const VkPhysicalDeviceCustomBorderColorPropertiesEXT>(static_cast<const VkPhysicalDeviceCustomBorderColorPropertiesEXT*>(object), settings, "VkPhysicalDeviceCustomBorderColorPropertiesEXT", indents, dump_json_VkPhysicalDeviceCustomBorderColorPropertiesEXT);
        break;
    case 1000287002:
        dump_json_pNext<const VkPhysicalDeviceCustomBorderColorFeaturesEXT>(static_cast<const VkPhysicalDeviceCustomBorderColorFeaturesEXT*>(object), settings, "VkPhysicalDeviceCustomBorderColorFeaturesEXT", indents, dump_json_VkPhysicalDeviceCustomBorderColorFeaturesEXT);
        break;
    case 1000292000:
        dump_json_pNext<const VkPhysicalDevicePresentBarrierFeaturesNV>(static_cast<const VkPhysicalDevicePresentBarrierFeaturesNV*>(object), settings, "VkPhysicalDevicePresentBarrierFeaturesNV", indents, dump_json_VkPhysicalDevicePresentBarrierFeaturesNV);
        break;
    case 1000292001:
        dump_json_pNext<const VkSurfaceCapabilitiesPresentBarrierNV>(static_cast<const VkSurfaceCapabilitiesPresentBarrierNV*>(object), settings, "VkSurfaceCapabilitiesPresentBarrierNV", indents, dump_json_VkSurfaceCapabilitiesPresentBarrierNV);
        break;
    case 1000292002:
        dump_json_pNext<const VkSwapchainPresentBarrierCreateInfoNV>(static_cast<const VkSwapchainPresentBarrierCreateInfoNV*>(object), settings, "VkSwapchainPresentBarrierCreateInfoNV", indents, dump_json_VkSwapchainPresentBarrierCreateInfoNV);
        break;
    case 1000294000:
        dump_json_pNext<const VkPresentIdKHR>(static_cast<const VkPresentIdKHR*>(object), settings, "VkPresentIdKHR", indents, dump_json_VkPresentIdKHR);
        break;
    case 1000294001:
        dump_json_pNext<const VkPhysicalDevicePresentIdFeaturesKHR>(static_cast<const VkPhysicalDevicePresentIdFeaturesKHR*>(object), settings, "VkPhysicalDevicePresentIdFeaturesKHR", indents, dump_json_VkPhysicalDevicePresentIdFeaturesKHR);
        break;
    case 1000299000:
        dump_json_pNext<const VkVideoEncodeInfoKHR>(static_cast<const VkVideoEncodeInfoKHR*>(object), settings, "VkVideoEncodeInfoKHR", indents, dump_json_VkVideoEncodeInfoKHR);
        break;
    case 1000299003:
        dump_json_pNext<const VkVideoEncodeCapabilitiesKHR>(static_cast<const VkVideoEncodeCapabilitiesKHR*>(object), settings, "VkVideoEncodeCapabilitiesKHR", indents, dump_json_VkVideoEncodeCapabilitiesKHR);
        break;
    case 1000299005:
        dump_json_pNext<const VkQueryPoolVideoEncodeFeedbackCreateInfoKHR>(static_cast<const VkQueryPoolVideoEncodeFeedbackCreateInfoKHR*>(object), settings, "VkQueryPoolVideoEncodeFeedbackCreateInfoKHR", indents, dump_json_VkQueryPoolVideoEncodeFeedbackCreateInfoKHR);
        break;
    case 1000299004:
        dump_json_pNext<const VkVideoEncodeUsageInfoKHR>(static_cast<const VkVideoEncodeUsageInfoKHR*>(object), settings, "VkVideoEncodeUsageInfoKHR", indents, dump_json_VkVideoEncodeUsageInfoKHR);
        break;
    case 1000299002:
        dump_json_pNext<const VkVideoEncodeRateControlLayerInfoKHR>(static_cast<const VkVideoEncodeRateControlLayerInfoKHR*>(object), settings, "VkVideoEncodeRateControlLayerInfoKHR", indents, dump_json_VkVideoEncodeRateControlLayerInfoKHR);
        break;
    case 1000299001:
        dump_json_pNext<const VkVideoEncodeRateControlInfoKHR>(static_cast<const VkVideoEncodeRateControlInfoKHR*>(object), settings, "VkVideoEncodeRateControlInfoKHR", indents, dump_json_VkVideoEncodeRateControlInfoKHR);
        break;
    case 1000299006:
        dump_json_pNext<const VkPhysicalDeviceVideoEncodeQualityLevelInfoKHR>(static_cast<const VkPhysicalDeviceVideoEncodeQualityLevelInfoKHR*>(object), settings, "VkPhysicalDeviceVideoEncodeQualityLevelInfoKHR", indents, dump_json_VkPhysicalDeviceVideoEncodeQualityLevelInfoKHR);
        break;
    case 1000299007:
        dump_json_pNext<const VkVideoEncodeQualityLevelPropertiesKHR>(static_cast<const VkVideoEncodeQualityLevelPropertiesKHR*>(object), settings, "VkVideoEncodeQualityLevelPropertiesKHR", indents, dump_json_VkVideoEncodeQualityLevelPropertiesKHR);
        break;
    case 1000299008:
        dump_json_pNext<const VkVideoEncodeQualityLevelInfoKHR>(static_cast<const VkVideoEncodeQualityLevelInfoKHR*>(object), settings, "VkVideoEncodeQualityLevelInfoKHR", indents, dump_json_VkVideoEncodeQualityLevelInfoKHR);
        break;
    case 1000299009:
        dump_json_pNext<const VkVideoEncodeSessionParametersGetInfoKHR>(static_cast<const VkVideoEncodeSessionParametersGetInfoKHR*>(object), settings, "VkVideoEncodeSessionParametersGetInfoKHR", indents, dump_json_VkVideoEncodeSessionParametersGetInfoKHR);
        break;
    case 1000299010:
        dump_json_pNext<const VkVideoEncodeSessionParametersFeedbackInfoKHR>(static_cast<const VkVideoEncodeSessionParametersFeedbackInfoKHR*>(object), settings, "VkVideoEncodeSessionParametersFeedbackInfoKHR", indents, dump_json_VkVideoEncodeSessionParametersFeedbackInfoKHR);
        break;
    case 1000300000:
        dump_json_pNext<const VkPhysicalDeviceDiagnosticsConfigFeaturesNV>(static_cast<const VkPhysicalDeviceDiagnosticsConfigFeaturesNV*>(object), settings, "VkPhysicalDeviceDiagnosticsConfigFeaturesNV", indents, dump_json_VkPhysicalDeviceDiagnosticsConfigFeaturesNV);
        break;
    case 1000300001:
        dump_json_pNext<const VkDeviceDiagnosticsConfigCreateInfoNV>(static_cast<const VkDeviceDiagnosticsConfigCreateInfoNV*>(object), settings, "VkDeviceDiagnosticsConfigCreateInfoNV", indents, dump_json_VkDeviceDiagnosticsConfigCreateInfoNV);
        break;
    case 1000307000:
        dump_json_pNext<const VkCudaModuleCreateInfoNV>(static_cast<const VkCudaModuleCreateInfoNV*>(object), settings, "VkCudaModuleCreateInfoNV", indents, dump_json_VkCudaModuleCreateInfoNV);
        break;
    case 1000307001:
        dump_json_pNext<const VkCudaFunctionCreateInfoNV>(static_cast<const VkCudaFunctionCreateInfoNV*>(object), settings, "VkCudaFunctionCreateInfoNV", indents, dump_json_VkCudaFunctionCreateInfoNV);
        break;
    case 1000307002:
        dump_json_pNext<const VkCudaLaunchInfoNV>(static_cast<const VkCudaLaunchInfoNV*>(object), settings, "VkCudaLaunchInfoNV", indents, dump_json_VkCudaLaunchInfoNV);
        break;
    case 1000307003:
        dump_json_pNext<const VkPhysicalDeviceCudaKernelLaunchFeaturesNV>(static_cast<const VkPhysicalDeviceCudaKernelLaunchFeaturesNV*>(object), settings, "VkPhysicalDeviceCudaKernelLaunchFeaturesNV", indents, dump_json_VkPhysicalDeviceCudaKernelLaunchFeaturesNV);
        break;
    case 1000307004:
        dump_json_pNext<const VkPhysicalDeviceCudaKernelLaunchPropertiesNV>(static_cast<const VkPhysicalDeviceCudaKernelLaunchPropertiesNV*>(object), settings, "VkPhysicalDeviceCudaKernelLaunchPropertiesNV", indents, dump_json_VkPhysicalDeviceCudaKernelLaunchPropertiesNV);
        break;
    case 1000310000:
        dump_json_pNext<const VkQueryLowLatencySupportNV>(static_cast<const VkQueryLowLatencySupportNV*>(object), settings, "VkQueryLowLatencySupportNV", indents, dump_json_VkQueryLowLatencySupportNV);
        break;
#if defined(VK_USE_PLATFORM_METAL_EXT)
    case 1000311000:
        dump_json_pNext<const VkExportMetalObjectCreateInfoEXT>(static_cast<const VkExportMetalObjectCreateInfoEXT*>(object), settings, "VkExportMetalObjectCreateInfoEXT", indents, dump_json_VkExportMetalObjectCreateInfoEXT);
        break;
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
    case 1000311001:
        dump_json_pNext<const VkExportMetalObjectsInfoEXT>(static_cast<const VkExportMetalObjectsInfoEXT*>(object), settings, "VkExportMetalObjectsInfoEXT", indents, dump_json_VkExportMetalObjectsInfoEXT);
        break;
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
    case 1000311002:
        dump_json_pNext<const VkExportMetalDeviceInfoEXT>(static_cast<const VkExportMetalDeviceInfoEXT*>(object), settings, "VkExportMetalDeviceInfoEXT", indents, dump_json_VkExportMetalDeviceInfoEXT);
        break;
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
    case 1000311003:
        dump_json_pNext<const VkExportMetalCommandQueueInfoEXT>(static_cast<const VkExportMetalCommandQueueInfoEXT*>(object), settings, "VkExportMetalCommandQueueInfoEXT", indents, dump_json_VkExportMetalCommandQueueInfoEXT);
        break;
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
    case 1000311004:
        dump_json_pNext<const VkExportMetalBufferInfoEXT>(static_cast<const VkExportMetalBufferInfoEXT*>(object), settings, "VkExportMetalBufferInfoEXT", indents, dump_json_VkExportMetalBufferInfoEXT);
        break;
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
    case 1000311005:
        dump_json_pNext<const VkImportMetalBufferInfoEXT>(static_cast<const VkImportMetalBufferInfoEXT*>(object), settings, "VkImportMetalBufferInfoEXT", indents, dump_json_VkImportMetalBufferInfoEXT);
        break;
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
    case 1000311006:
        dump_json_pNext<const VkExportMetalTextureInfoEXT>(static_cast<const VkExportMetalTextureInfoEXT*>(object), settings, "VkExportMetalTextureInfoEXT", indents, dump_json_VkExportMetalTextureInfoEXT);
        break;
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
    case 1000311007:
        dump_json_pNext<const VkImportMetalTextureInfoEXT>(static_cast<const VkImportMetalTextureInfoEXT*>(object), settings, "VkImportMetalTextureInfoEXT", indents, dump_json_VkImportMetalTextureInfoEXT);
        break;
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
    case 1000311008:
        dump_json_pNext<const VkExportMetalIOSurfaceInfoEXT>(static_cast<const VkExportMetalIOSurfaceInfoEXT*>(object), settings, "VkExportMetalIOSurfaceInfoEXT", indents, dump_json_VkExportMetalIOSurfaceInfoEXT);
        break;
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
    case 1000311009:
        dump_json_pNext<const VkImportMetalIOSurfaceInfoEXT>(static_cast<const VkImportMetalIOSurfaceInfoEXT*>(object), settings, "VkImportMetalIOSurfaceInfoEXT", indents, dump_json_VkImportMetalIOSurfaceInfoEXT);
        break;
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
    case 1000311010:
        dump_json_pNext<const VkExportMetalSharedEventInfoEXT>(static_cast<const VkExportMetalSharedEventInfoEXT*>(object), settings, "VkExportMetalSharedEventInfoEXT", indents, dump_json_VkExportMetalSharedEventInfoEXT);
        break;
#endif // VK_USE_PLATFORM_METAL_EXT
#if defined(VK_USE_PLATFORM_METAL_EXT)
    case 1000311011:
        dump_json_pNext<const VkImportMetalSharedEventInfoEXT>(static_cast<const VkImportMetalSharedEventInfoEXT*>(object), settings, "VkImportMetalSharedEventInfoEXT", indents, dump_json_VkImportMetalSharedEventInfoEXT);
        break;
#endif // VK_USE_PLATFORM_METAL_EXT
    case 1000314008:
        dump_json_pNext<const VkQueueFamilyCheckpointProperties2NV>(static_cast<const VkQueueFamilyCheckpointProperties2NV*>(object), settings, "VkQueueFamilyCheckpointProperties2NV", indents, dump_json_VkQueueFamilyCheckpointProperties2NV);
        break;
    case 1000314009:
        dump_json_pNext<const VkCheckpointData2NV>(static_cast<const VkCheckpointData2NV*>(object), settings, "VkCheckpointData2NV", indents, dump_json_VkCheckpointData2NV);
        break;
    case 1000316000:
        dump_json_pNext<const VkPhysicalDeviceDescriptorBufferPropertiesEXT>(static_cast<const VkPhysicalDeviceDescriptorBufferPropertiesEXT*>(object), settings, "VkPhysicalDeviceDescriptorBufferPropertiesEXT", indents, dump_json_VkPhysicalDeviceDescriptorBufferPropertiesEXT);
        break;
    case 1000316001:
        dump_json_pNext<const VkPhysicalDeviceDescriptorBufferDensityMapPropertiesEXT>(static_cast<const VkPhysicalDeviceDescriptorBufferDensityMapPropertiesEXT*>(object), settings, "VkPhysicalDeviceDescriptorBufferDensityMapPropertiesEXT", indents, dump_json_VkPhysicalDeviceDescriptorBufferDensityMapPropertiesEXT);
        break;
    case 1000316002:
        dump_json_pNext<const VkPhysicalDeviceDescriptorBufferFeaturesEXT>(static_cast<const VkPhysicalDeviceDescriptorBufferFeaturesEXT*>(object), settings, "VkPhysicalDeviceDescriptorBufferFeaturesEXT", indents, dump_json_VkPhysicalDeviceDescriptorBufferFeaturesEXT);
        break;
    case 1000316003:
        dump_json_pNext<const VkDescriptorAddressInfoEXT>(static_cast<const VkDescriptorAddressInfoEXT*>(object), settings, "VkDescriptorAddressInfoEXT", indents, dump_json_VkDescriptorAddressInfoEXT);
        break;
    case 1000316011:
        dump_json_pNext<const VkDescriptorBufferBindingInfoEXT>(static_cast<const VkDescriptorBufferBindingInfoEXT*>(object), settings, "VkDescriptorBufferBindingInfoEXT", indents, dump_json_VkDescriptorBufferBindingInfoEXT);
        break;
    case 1000316012:
        dump_json_pNext<const VkDescriptorBufferBindingPushDescriptorBufferHandleEXT>(static_cast<const VkDescriptorBufferBindingPushDescriptorBufferHandleEXT*>(object), settings, "VkDescriptorBufferBindingPushDescriptorBufferHandleEXT", indents, dump_json_VkDescriptorBufferBindingPushDescriptorBufferHandleEXT);
        break;
    case 1000316004:
        dump_json_pNext<const VkDescriptorGetInfoEXT>(static_cast<const VkDescriptorGetInfoEXT*>(object), settings, "VkDescriptorGetInfoEXT", indents, dump_json_VkDescriptorGetInfoEXT);
        break;
    case 1000316005:
        dump_json_pNext<const VkBufferCaptureDescriptorDataInfoEXT>(static_cast<const VkBufferCaptureDescriptorDataInfoEXT*>(object), settings, "VkBufferCaptureDescriptorDataInfoEXT", indents, dump_json_VkBufferCaptureDescriptorDataInfoEXT);
        break;
    case 1000316006:
        dump_json_pNext<const VkImageCaptureDescriptorDataInfoEXT>(static_cast<const VkImageCaptureDescriptorDataInfoEXT*>(object), settings, "VkImageCaptureDescriptorDataInfoEXT", indents, dump_json_VkImageCaptureDescriptorDataInfoEXT);
        break;
    case 1000316007:
        dump_json_pNext<const VkImageViewCaptureDescriptorDataInfoEXT>(static_cast<const VkImageViewCaptureDescriptorDataInfoEXT*>(object), settings, "VkImageViewCaptureDescriptorDataInfoEXT", indents, dump_json_VkImageViewCaptureDescriptorDataInfoEXT);
        break;
    case 1000316008:
        dump_json_pNext<const VkSamplerCaptureDescriptorDataInfoEXT>(static_cast<const VkSamplerCaptureDescriptorDataInfoEXT*>(object), settings, "VkSamplerCaptureDescriptorDataInfoEXT", indents, dump_json_VkSamplerCaptureDescriptorDataInfoEXT);
        break;
    case 1000316010:
        dump_json_pNext<const VkOpaqueCaptureDescriptorDataCreateInfoEXT>(static_cast<const VkOpaqueCaptureDescriptorDataCreateInfoEXT*>(object), settings, "VkOpaqueCaptureDescriptorDataCreateInfoEXT", indents, dump_json_VkOpaqueCaptureDescriptorDataCreateInfoEXT);
        break;
    case 1000316009:
        dump_json_pNext<const VkAccelerationStructureCaptureDescriptorDataInfoEXT>(static_cast<const VkAccelerationStructureCaptureDescriptorDataInfoEXT*>(object), settings, "VkAccelerationStructureCaptureDescriptorDataInfoEXT", indents, dump_json_VkAccelerationStructureCaptureDescriptorDataInfoEXT);
        break;
    case 1000320000:
        dump_json_pNext<const VkPhysicalDeviceGraphicsPipelineLibraryFeaturesEXT>(static_cast<const VkPhysicalDeviceGraphicsPipelineLibraryFeaturesEXT*>(object), settings, "VkPhysicalDeviceGraphicsPipelineLibraryFeaturesEXT", indents, dump_json_VkPhysicalDeviceGraphicsPipelineLibraryFeaturesEXT);
        break;
    case 1000320001:
        dump_json_pNext<const VkPhysicalDeviceGraphicsPipelineLibraryPropertiesEXT>(static_cast<const VkPhysicalDeviceGraphicsPipelineLibraryPropertiesEXT*>(object), settings, "VkPhysicalDeviceGraphicsPipelineLibraryPropertiesEXT", indents, dump_json_VkPhysicalDeviceGraphicsPipelineLibraryPropertiesEXT);
        break;
    case 1000320002:
        dump_json_pNext<const VkGraphicsPipelineLibraryCreateInfoEXT>(static_cast<const VkGraphicsPipelineLibraryCreateInfoEXT*>(object), settings, "VkGraphicsPipelineLibraryCreateInfoEXT", indents, dump_json_VkGraphicsPipelineLibraryCreateInfoEXT);
        break;
    case 1000321000:
        dump_json_pNext<const VkPhysicalDeviceShaderEarlyAndLateFragmentTestsFeaturesAMD>(static_cast<const VkPhysicalDeviceShaderEarlyAndLateFragmentTestsFeaturesAMD*>(object), settings, "VkPhysicalDeviceShaderEarlyAndLateFragmentTestsFeaturesAMD", indents, dump_json_VkPhysicalDeviceShaderEarlyAndLateFragmentTestsFeaturesAMD);
        break;
    case 1000322000:
        dump_json_pNext<const VkPhysicalDeviceFragmentShaderBarycentricPropertiesKHR>(static_cast<const VkPhysicalDeviceFragmentShaderBarycentricPropertiesKHR*>(object), settings, "VkPhysicalDeviceFragmentShaderBarycentricPropertiesKHR", indents, dump_json_VkPhysicalDeviceFragmentShaderBarycentricPropertiesKHR);
        break;
    case 1000323000:
        dump_json_pNext<const VkPhysicalDeviceShaderSubgroupUniformControlFlowFeaturesKHR>(static_cast<const VkPhysicalDeviceShaderSubgroupUniformControlFlowFeaturesKHR*>(object), settings, "VkPhysicalDeviceShaderSubgroupUniformControlFlowFeaturesKHR", indents, dump_json_VkPhysicalDeviceShaderSubgroupUniformControlFlowFeaturesKHR);
        break;
    case 1000326001:
        dump_json_pNext<const VkPhysicalDeviceFragmentShadingRateEnumsFeaturesNV>(static_cast<const VkPhysicalDeviceFragmentShadingRateEnumsFeaturesNV*>(object), settings, "VkPhysicalDeviceFragmentShadingRateEnumsFeaturesNV", indents, dump_json_VkPhysicalDeviceFragmentShadingRateEnumsFeaturesNV);
        break;
    case 1000326000:
        dump_json_pNext<const VkPhysicalDeviceFragmentShadingRateEnumsPropertiesNV>(static_cast<const VkPhysicalDeviceFragmentShadingRateEnumsPropertiesNV*>(object), settings, "VkPhysicalDeviceFragmentShadingRateEnumsPropertiesNV", indents, dump_json_VkPhysicalDeviceFragmentShadingRateEnumsPropertiesNV);
        break;
    case 1000326002:
        dump_json_pNext<const VkPipelineFragmentShadingRateEnumStateCreateInfoNV>(static_cast<const VkPipelineFragmentShadingRateEnumStateCreateInfoNV*>(object), settings, "VkPipelineFragmentShadingRateEnumStateCreateInfoNV", indents, dump_json_VkPipelineFragmentShadingRateEnumStateCreateInfoNV);
        break;
    case 1000327000:
        dump_json_pNext<const VkAccelerationStructureGeometryMotionTrianglesDataNV>(static_cast<const VkAccelerationStructureGeometryMotionTrianglesDataNV*>(object), settings, "VkAccelerationStructureGeometryMotionTrianglesDataNV", indents, dump_json_VkAccelerationStructureGeometryMotionTrianglesDataNV);
        break;
    case 1000327002:
        dump_json_pNext<const VkAccelerationStructureMotionInfoNV>(static_cast<const VkAccelerationStructureMotionInfoNV*>(object), settings, "VkAccelerationStructureMotionInfoNV", indents, dump_json_VkAccelerationStructureMotionInfoNV);
        break;
    case 1000327001:
        dump_json_pNext<const VkPhysicalDeviceRayTracingMotionBlurFeaturesNV>(static_cast<const VkPhysicalDeviceRayTracingMotionBlurFeaturesNV*>(object), settings, "VkPhysicalDeviceRayTracingMotionBlurFeaturesNV", indents, dump_json_VkPhysicalDeviceRayTracingMotionBlurFeaturesNV);
        break;
    case 1000328000:
        dump_json_pNext<const VkPhysicalDeviceMeshShaderFeaturesEXT>(static_cast<const VkPhysicalDeviceMeshShaderFeaturesEXT*>(object), settings, "VkPhysicalDeviceMeshShaderFeaturesEXT", indents, dump_json_VkPhysicalDeviceMeshShaderFeaturesEXT);
        break;
    case 1000328001:
        dump_json_pNext<const VkPhysicalDeviceMeshShaderPropertiesEXT>(static_cast<const VkPhysicalDeviceMeshShaderPropertiesEXT*>(object), settings, "VkPhysicalDeviceMeshShaderPropertiesEXT", indents, dump_json_VkPhysicalDeviceMeshShaderPropertiesEXT);
        break;
    case 1000330000:
        dump_json_pNext<const VkPhysicalDeviceYcbcr2Plane444FormatsFeaturesEXT>(static_cast<const VkPhysicalDeviceYcbcr2Plane444FormatsFeaturesEXT*>(object), settings, "VkPhysicalDeviceYcbcr2Plane444FormatsFeaturesEXT", indents, dump_json_VkPhysicalDeviceYcbcr2Plane444FormatsFeaturesEXT);
        break;
    case 1000332000:
        dump_json_pNext<const VkPhysicalDeviceFragmentDensityMap2FeaturesEXT>(static_cast<const VkPhysicalDeviceFragmentDensityMap2FeaturesEXT*>(object), settings, "VkPhysicalDeviceFragmentDensityMap2FeaturesEXT", indents, dump_json_VkPhysicalDeviceFragmentDensityMap2FeaturesEXT);
        break;
    case 1000332001:
        dump_json_pNext<const VkPhysicalDeviceFragmentDensityMap2PropertiesEXT>(static_cast<const VkPhysicalDeviceFragmentDensityMap2PropertiesEXT*>(object), settings, "VkPhysicalDeviceFragmentDensityMap2PropertiesEXT", indents, dump_json_VkPhysicalDeviceFragmentDensityMap2PropertiesEXT);
        break;
    case 1000333000:
        dump_json_pNext<const VkCopyCommandTransformInfoQCOM>(static_cast<const VkCopyCommandTransformInfoQCOM*>(object), settings, "VkCopyCommandTransformInfoQCOM", indents, dump_json_VkCopyCommandTransformInfoQCOM);
        break;
    case 1000336000:
        dump_json_pNext<const VkPhysicalDeviceWorkgroupMemoryExplicitLayoutFeaturesKHR>(static_cast<const VkPhysicalDeviceWorkgroupMemoryExplicitLayoutFeaturesKHR*>(object), settings, "VkPhysicalDeviceWorkgroupMemoryExplicitLayoutFeaturesKHR", indents, dump_json_VkPhysicalDeviceWorkgroupMemoryExplicitLayoutFeaturesKHR);
        break;
    case 1000338000:
        dump_json_pNext<const VkPhysicalDeviceImageCompressionControlFeaturesEXT>(static_cast<const VkPhysicalDeviceImageCompressionControlFeaturesEXT*>(object), settings, "VkPhysicalDeviceImageCompressionControlFeaturesEXT", indents, dump_json_VkPhysicalDeviceImageCompressionControlFeaturesEXT);
        break;
    case 1000338001:
        dump_json_pNext<const VkImageCompressionControlEXT>(static_cast<const VkImageCompressionControlEXT*>(object), settings, "VkImageCompressionControlEXT", indents, dump_json_VkImageCompressionControlEXT);
        break;
    case 1000338004:
        dump_json_pNext<const VkImageCompressionPropertiesEXT>(static_cast<const VkImageCompressionPropertiesEXT*>(object), settings, "VkImageCompressionPropertiesEXT", indents, dump_json_VkImageCompressionPropertiesEXT);
        break;
    case 1000339000:
        dump_json_pNext<const VkPhysicalDeviceAttachmentFeedbackLoopLayoutFeaturesEXT>(static_cast<const VkPhysicalDeviceAttachmentFeedbackLoopLayoutFeaturesEXT*>(object), settings, "VkPhysicalDeviceAttachmentFeedbackLoopLayoutFeaturesEXT", indents, dump_json_VkPhysicalDeviceAttachmentFeedbackLoopLayoutFeaturesEXT);
        break;
    case 1000340000:
        dump_json_pNext<const VkPhysicalDevice4444FormatsFeaturesEXT>(static_cast<const VkPhysicalDevice4444FormatsFeaturesEXT*>(object), settings, "VkPhysicalDevice4444FormatsFeaturesEXT", indents, dump_json_VkPhysicalDevice4444FormatsFeaturesEXT);
        break;
    case 1000341000:
        dump_json_pNext<const VkPhysicalDeviceFaultFeaturesEXT>(static_cast<const VkPhysicalDeviceFaultFeaturesEXT*>(object), settings, "VkPhysicalDeviceFaultFeaturesEXT", indents, dump_json_VkPhysicalDeviceFaultFeaturesEXT);
        break;
    case 1000341001:
        dump_json_pNext<const VkDeviceFaultCountsEXT>(static_cast<const VkDeviceFaultCountsEXT*>(object), settings, "VkDeviceFaultCountsEXT", indents, dump_json_VkDeviceFaultCountsEXT);
        break;
    case 1000341002:
        dump_json_pNext<const VkDeviceFaultInfoEXT>(static_cast<const VkDeviceFaultInfoEXT*>(object), settings, "VkDeviceFaultInfoEXT", indents, dump_json_VkDeviceFaultInfoEXT);
        break;
    case 1000342000:
        dump_json_pNext<const VkPhysicalDeviceRasterizationOrderAttachmentAccessFeaturesEXT>(static_cast<const VkPhysicalDeviceRasterizationOrderAttachmentAccessFeaturesEXT*>(object), settings, "VkPhysicalDeviceRasterizationOrderAttachmentAccessFeaturesEXT", indents, dump_json_VkPhysicalDeviceRasterizationOrderAttachmentAccessFeaturesEXT);
        break;
    case 1000344000:
        dump_json_pNext<const VkPhysicalDeviceRGBA10X6FormatsFeaturesEXT>(static_cast<const VkPhysicalDeviceRGBA10X6FormatsFeaturesEXT*>(object), settings, "VkPhysicalDeviceRGBA10X6FormatsFeaturesEXT", indents, dump_json_VkPhysicalDeviceRGBA10X6FormatsFeaturesEXT);
        break;
#if defined(VK_USE_PLATFORM_DIRECTFB_EXT)
    case 1000346000:
        dump_json_pNext<const VkDirectFBSurfaceCreateInfoEXT>(static_cast<const VkDirectFBSurfaceCreateInfoEXT*>(object), settings, "VkDirectFBSurfaceCreateInfoEXT", indents, dump_json_VkDirectFBSurfaceCreateInfoEXT);
        break;
#endif // VK_USE_PLATFORM_DIRECTFB_EXT
    case 1000150016:
        dump_json_pNext<const VkRayTracingShaderGroupCreateInfoKHR>(static_cast<const VkRayTracingShaderGroupCreateInfoKHR*>(object), settings, "VkRayTracingShaderGroupCreateInfoKHR", indents, dump_json_VkRayTracingShaderGroupCreateInfoKHR);
        break;
    case 1000150018:
        dump_json_pNext<const VkRayTracingPipelineInterfaceCreateInfoKHR>(static_cast<const VkRayTracingPipelineInterfaceCreateInfoKHR*>(object), settings, "VkRayTracingPipelineInterfaceCreateInfoKHR", indents, dump_json_VkRayTracingPipelineInterfaceCreateInfoKHR);
        break;
    case 1000150015:
        dump_json_pNext<const VkRayTracingPipelineCreateInfoKHR>(static_cast<const VkRayTracingPipelineCreateInfoKHR*>(object), settings, "VkRayTracingPipelineCreateInfoKHR", indents, dump_json_VkRayTracingPipelineCreateInfoKHR);
        break;
    case 1000347000:
        dump_json_pNext<const VkPhysicalDeviceRayTracingPipelineFeaturesKHR>(static_cast<const VkPhysicalDeviceRayTracingPipelineFeaturesKHR*>(object), settings, "VkPhysicalDeviceRayTracingPipelineFeaturesKHR", indents, dump_json_VkPhysicalDeviceRayTracingPipelineFeaturesKHR);
        break;
    case 1000347001:
        dump_json_pNext<const VkPhysicalDeviceRayTracingPipelinePropertiesKHR>(static_cast<const VkPhysicalDeviceRayTracingPipelinePropertiesKHR*>(object), settings, "VkPhysicalDeviceRayTracingPipelinePropertiesKHR", indents, dump_json_VkPhysicalDeviceRayTracingPipelinePropertiesKHR);
        break;
    case 1000348013:
        dump_json_pNext<const VkPhysicalDeviceRayQueryFeaturesKHR>(static_cast<const VkPhysicalDeviceRayQueryFeaturesKHR*>(object), settings, "VkPhysicalDeviceRayQueryFeaturesKHR", indents, dump_json_VkPhysicalDeviceRayQueryFeaturesKHR);
        break;
    case 1000351000:
        dump_json_pNext<const VkPhysicalDeviceMutableDescriptorTypeFeaturesEXT>(static_cast<const VkPhysicalDeviceMutableDescriptorTypeFeaturesEXT*>(object), settings, "VkPhysicalDeviceMutableDescriptorTypeFeaturesEXT", indents, dump_json_VkPhysicalDeviceMutableDescriptorTypeFeaturesEXT);
        break;
    case 1000351002:
        dump_json_pNext<const VkMutableDescriptorTypeCreateInfoEXT>(static_cast<const VkMutableDescriptorTypeCreateInfoEXT*>(object), settings, "VkMutableDescriptorTypeCreateInfoEXT", indents, dump_json_VkMutableDescriptorTypeCreateInfoEXT);
        break;
    case 1000352000:
        dump_json_pNext<const VkPhysicalDeviceVertexInputDynamicStateFeaturesEXT>(static_cast<const VkPhysicalDeviceVertexInputDynamicStateFeaturesEXT*>(object), settings, "VkPhysicalDeviceVertexInputDynamicStateFeaturesEXT", indents, dump_json_VkPhysicalDeviceVertexInputDynamicStateFeaturesEXT);
        break;
    case 1000352001:
        dump_json_pNext<const VkVertexInputBindingDescription2EXT>(static_cast<const VkVertexInputBindingDescription2EXT*>(object), settings, "VkVertexInputBindingDescription2EXT", indents, dump_json_VkVertexInputBindingDescription2EXT);
        break;
    case 1000352002:
        dump_json_pNext<const VkVertexInputAttributeDescription2EXT>(static_cast<const VkVertexInputAttributeDescription2EXT*>(object), settings, "VkVertexInputAttributeDescription2EXT", indents, dump_json_VkVertexInputAttributeDescription2EXT);
        break;
    case 1000353000:
        dump_json_pNext<const VkPhysicalDeviceDrmPropertiesEXT>(static_cast<const VkPhysicalDeviceDrmPropertiesEXT*>(object), settings, "VkPhysicalDeviceDrmPropertiesEXT", indents, dump_json_VkPhysicalDeviceDrmPropertiesEXT);
        break;
    case 1000354000:
        dump_json_pNext<const VkPhysicalDeviceAddressBindingReportFeaturesEXT>(static_cast<const VkPhysicalDeviceAddressBindingReportFeaturesEXT*>(object), settings, "VkPhysicalDeviceAddressBindingReportFeaturesEXT", indents, dump_json_VkPhysicalDeviceAddressBindingReportFeaturesEXT);
        break;
    case 1000354001:
        dump_json_pNext<const VkDeviceAddressBindingCallbackDataEXT>(static_cast<const VkDeviceAddressBindingCallbackDataEXT*>(object), settings, "VkDeviceAddressBindingCallbackDataEXT", indents, dump_json_VkDeviceAddressBindingCallbackDataEXT);
        break;
    case 1000355000:
        dump_json_pNext<const VkPhysicalDeviceDepthClipControlFeaturesEXT>(static_cast<const VkPhysicalDeviceDepthClipControlFeaturesEXT*>(object), settings, "VkPhysicalDeviceDepthClipControlFeaturesEXT", indents, dump_json_VkPhysicalDeviceDepthClipControlFeaturesEXT);
        break;
    case 1000355001:
        dump_json_pNext<const VkPipelineViewportDepthClipControlCreateInfoEXT>(static_cast<const VkPipelineViewportDepthClipControlCreateInfoEXT*>(object), settings, "VkPipelineViewportDepthClipControlCreateInfoEXT", indents, dump_json_VkPipelineViewportDepthClipControlCreateInfoEXT);
        break;
    case 1000356000:
        dump_json_pNext<const VkPhysicalDevicePrimitiveTopologyListRestartFeaturesEXT>(static_cast<const VkPhysicalDevicePrimitiveTopologyListRestartFeaturesEXT*>(object), settings, "VkPhysicalDevicePrimitiveTopologyListRestartFeaturesEXT", indents, dump_json_VkPhysicalDevicePrimitiveTopologyListRestartFeaturesEXT);
        break;
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000364000:
        dump_json_pNext<const VkImportMemoryZirconHandleInfoFUCHSIA>(static_cast<const VkImportMemoryZirconHandleInfoFUCHSIA*>(object), settings, "VkImportMemoryZirconHandleInfoFUCHSIA", indents, dump_json_VkImportMemoryZirconHandleInfoFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000364001:
        dump_json_pNext<const VkMemoryZirconHandlePropertiesFUCHSIA>(static_cast<const VkMemoryZirconHandlePropertiesFUCHSIA*>(object), settings, "VkMemoryZirconHandlePropertiesFUCHSIA", indents, dump_json_VkMemoryZirconHandlePropertiesFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000364002:
        dump_json_pNext<const VkMemoryGetZirconHandleInfoFUCHSIA>(static_cast<const VkMemoryGetZirconHandleInfoFUCHSIA*>(object), settings, "VkMemoryGetZirconHandleInfoFUCHSIA", indents, dump_json_VkMemoryGetZirconHandleInfoFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000365000:
        dump_json_pNext<const VkImportSemaphoreZirconHandleInfoFUCHSIA>(static_cast<const VkImportSemaphoreZirconHandleInfoFUCHSIA*>(object), settings, "VkImportSemaphoreZirconHandleInfoFUCHSIA", indents, dump_json_VkImportSemaphoreZirconHandleInfoFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000365001:
        dump_json_pNext<const VkSemaphoreGetZirconHandleInfoFUCHSIA>(static_cast<const VkSemaphoreGetZirconHandleInfoFUCHSIA*>(object), settings, "VkSemaphoreGetZirconHandleInfoFUCHSIA", indents, dump_json_VkSemaphoreGetZirconHandleInfoFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000366000:
        dump_json_pNext<const VkBufferCollectionCreateInfoFUCHSIA>(static_cast<const VkBufferCollectionCreateInfoFUCHSIA*>(object), settings, "VkBufferCollectionCreateInfoFUCHSIA", indents, dump_json_VkBufferCollectionCreateInfoFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000366001:
        dump_json_pNext<const VkImportMemoryBufferCollectionFUCHSIA>(static_cast<const VkImportMemoryBufferCollectionFUCHSIA*>(object), settings, "VkImportMemoryBufferCollectionFUCHSIA", indents, dump_json_VkImportMemoryBufferCollectionFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000366002:
        dump_json_pNext<const VkBufferCollectionImageCreateInfoFUCHSIA>(static_cast<const VkBufferCollectionImageCreateInfoFUCHSIA*>(object), settings, "VkBufferCollectionImageCreateInfoFUCHSIA", indents, dump_json_VkBufferCollectionImageCreateInfoFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000366009:
        dump_json_pNext<const VkBufferCollectionConstraintsInfoFUCHSIA>(static_cast<const VkBufferCollectionConstraintsInfoFUCHSIA*>(object), settings, "VkBufferCollectionConstraintsInfoFUCHSIA", indents, dump_json_VkBufferCollectionConstraintsInfoFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000366004:
        dump_json_pNext<const VkBufferConstraintsInfoFUCHSIA>(static_cast<const VkBufferConstraintsInfoFUCHSIA*>(object), settings, "VkBufferConstraintsInfoFUCHSIA", indents, dump_json_VkBufferConstraintsInfoFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000366005:
        dump_json_pNext<const VkBufferCollectionBufferCreateInfoFUCHSIA>(static_cast<const VkBufferCollectionBufferCreateInfoFUCHSIA*>(object), settings, "VkBufferCollectionBufferCreateInfoFUCHSIA", indents, dump_json_VkBufferCollectionBufferCreateInfoFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000366008:
        dump_json_pNext<const VkSysmemColorSpaceFUCHSIA>(static_cast<const VkSysmemColorSpaceFUCHSIA*>(object), settings, "VkSysmemColorSpaceFUCHSIA", indents, dump_json_VkSysmemColorSpaceFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000366003:
        dump_json_pNext<const VkBufferCollectionPropertiesFUCHSIA>(static_cast<const VkBufferCollectionPropertiesFUCHSIA*>(object), settings, "VkBufferCollectionPropertiesFUCHSIA", indents, dump_json_VkBufferCollectionPropertiesFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000366007:
        dump_json_pNext<const VkImageFormatConstraintsInfoFUCHSIA>(static_cast<const VkImageFormatConstraintsInfoFUCHSIA*>(object), settings, "VkImageFormatConstraintsInfoFUCHSIA", indents, dump_json_VkImageFormatConstraintsInfoFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
    case 1000366006:
        dump_json_pNext<const VkImageConstraintsInfoFUCHSIA>(static_cast<const VkImageConstraintsInfoFUCHSIA*>(object), settings, "VkImageConstraintsInfoFUCHSIA", indents, dump_json_VkImageConstraintsInfoFUCHSIA);
        break;
#endif // VK_USE_PLATFORM_FUCHSIA
    case 1000369000:
        dump_json_pNext<const VkSubpassShadingPipelineCreateInfoHUAWEI>(static_cast<const VkSubpassShadingPipelineCreateInfoHUAWEI*>(object), settings, "VkSubpassShadingPipelineCreateInfoHUAWEI", indents, dump_json_VkSubpassShadingPipelineCreateInfoHUAWEI);
        break;
    case 1000369001:
        dump_json_pNext<const VkPhysicalDeviceSubpassShadingFeaturesHUAWEI>(static_cast<const VkPhysicalDeviceSubpassShadingFeaturesHUAWEI*>(object), settings, "VkPhysicalDeviceSubpassShadingFeaturesHUAWEI", indents, dump_json_VkPhysicalDeviceSubpassShadingFeaturesHUAWEI);
        break;
    case 1000369002:
        dump_json_pNext<const VkPhysicalDeviceSubpassShadingPropertiesHUAWEI>(static_cast<const VkPhysicalDeviceSubpassShadingPropertiesHUAWEI*>(object), settings, "VkPhysicalDeviceSubpassShadingPropertiesHUAWEI", indents, dump_json_VkPhysicalDeviceSubpassShadingPropertiesHUAWEI);
        break;
    case 1000370000:
        dump_json_pNext<const VkPhysicalDeviceInvocationMaskFeaturesHUAWEI>(static_cast<const VkPhysicalDeviceInvocationMaskFeaturesHUAWEI*>(object), settings, "VkPhysicalDeviceInvocationMaskFeaturesHUAWEI", indents, dump_json_VkPhysicalDeviceInvocationMaskFeaturesHUAWEI);
        break;
    case 1000371000:
        dump_json_pNext<const VkMemoryGetRemoteAddressInfoNV>(static_cast<const VkMemoryGetRemoteAddressInfoNV*>(object), settings, "VkMemoryGetRemoteAddressInfoNV", indents, dump_json_VkMemoryGetRemoteAddressInfoNV);
        break;
    case 1000371001:
        dump_json_pNext<const VkPhysicalDeviceExternalMemoryRDMAFeaturesNV>(static_cast<const VkPhysicalDeviceExternalMemoryRDMAFeaturesNV*>(object), settings, "VkPhysicalDeviceExternalMemoryRDMAFeaturesNV", indents, dump_json_VkPhysicalDeviceExternalMemoryRDMAFeaturesNV);
        break;
    case 1000372000:
        dump_json_pNext<const VkPipelinePropertiesIdentifierEXT>(static_cast<const VkPipelinePropertiesIdentifierEXT*>(object), settings, "VkPipelinePropertiesIdentifierEXT", indents, dump_json_VkPipelinePropertiesIdentifierEXT);
        break;
    case 1000372001:
        dump_json_pNext<const VkPhysicalDevicePipelinePropertiesFeaturesEXT>(static_cast<const VkPhysicalDevicePipelinePropertiesFeaturesEXT*>(object), settings, "VkPhysicalDevicePipelinePropertiesFeaturesEXT", indents, dump_json_VkPhysicalDevicePipelinePropertiesFeaturesEXT);
        break;
    case 1000375000:
        dump_json_pNext<const VkPhysicalDeviceFrameBoundaryFeaturesEXT>(static_cast<const VkPhysicalDeviceFrameBoundaryFeaturesEXT*>(object), settings, "VkPhysicalDeviceFrameBoundaryFeaturesEXT", indents, dump_json_VkPhysicalDeviceFrameBoundaryFeaturesEXT);
        break;
    case 1000375001:
        dump_json_pNext<const VkFrameBoundaryEXT>(static_cast<const VkFrameBoundaryEXT*>(object), settings, "VkFrameBoundaryEXT", indents, dump_json_VkFrameBoundaryEXT);
        break;
    case 1000376000:
        dump_json_pNext<const VkPhysicalDeviceMultisampledRenderToSingleSampledFeaturesEXT>(static_cast<const VkPhysicalDeviceMultisampledRenderToSingleSampledFeaturesEXT*>(object), settings, "VkPhysicalDeviceMultisampledRenderToSingleSampledFeaturesEXT", indents, dump_json_VkPhysicalDeviceMultisampledRenderToSingleSampledFeaturesEXT);
        break;
    case 1000376001:
        dump_json_pNext<const VkSubpassResolvePerformanceQueryEXT>(static_cast<const VkSubpassResolvePerformanceQueryEXT*>(object), settings, "VkSubpassResolvePerformanceQueryEXT", indents, dump_json_VkSubpassResolvePerformanceQueryEXT);
        break;
    case 1000376002:
        dump_json_pNext<const VkMultisampledRenderToSingleSampledInfoEXT>(static_cast<const VkMultisampledRenderToSingleSampledInfoEXT*>(object), settings, "VkMultisampledRenderToSingleSampledInfoEXT", indents, dump_json_VkMultisampledRenderToSingleSampledInfoEXT);
        break;
    case 1000377000:
        dump_json_pNext<const VkPhysicalDeviceExtendedDynamicState2FeaturesEXT>(static_cast<const VkPhysicalDeviceExtendedDynamicState2FeaturesEXT*>(object), settings, "VkPhysicalDeviceExtendedDynamicState2FeaturesEXT", indents, dump_json_VkPhysicalDeviceExtendedDynamicState2FeaturesEXT);
        break;
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
    case 1000378000:
        dump_json_pNext<const VkScreenSurfaceCreateInfoQNX>(static_cast<const VkScreenSurfaceCreateInfoQNX*>(object), settings, "VkScreenSurfaceCreateInfoQNX", indents, dump_json_VkScreenSurfaceCreateInfoQNX);
        break;
#endif // VK_USE_PLATFORM_SCREEN_QNX
    case 1000381000:
        dump_json_pNext<const VkPhysicalDeviceColorWriteEnableFeaturesEXT>(static_cast<const VkPhysicalDeviceColorWriteEnableFeaturesEXT*>(object), settings, "VkPhysicalDeviceColorWriteEnableFeaturesEXT", indents, dump_json_VkPhysicalDeviceColorWriteEnableFeaturesEXT);
        break;
    case 1000381001:
        dump_json_pNext<const VkPipelineColorWriteCreateInfoEXT>(static_cast<const VkPipelineColorWriteCreateInfoEXT*>(object), settings, "VkPipelineColorWriteCreateInfoEXT", indents, dump_json_VkPipelineColorWriteCreateInfoEXT);
        break;
    case 1000382000:
        dump_json_pNext<const VkPhysicalDevicePrimitivesGeneratedQueryFeaturesEXT>(static_cast<const VkPhysicalDevicePrimitivesGeneratedQueryFeaturesEXT*>(object), settings, "VkPhysicalDevicePrimitivesGeneratedQueryFeaturesEXT", indents, dump_json_VkPhysicalDevicePrimitivesGeneratedQueryFeaturesEXT);
        break;
    case 1000386000:
        dump_json_pNext<const VkPhysicalDeviceRayTracingMaintenance1FeaturesKHR>(static_cast<const VkPhysicalDeviceRayTracingMaintenance1FeaturesKHR*>(object), settings, "VkPhysicalDeviceRayTracingMaintenance1FeaturesKHR", indents, dump_json_VkPhysicalDeviceRayTracingMaintenance1FeaturesKHR);
        break;
    case 1000391000:
        dump_json_pNext<const VkPhysicalDeviceImageViewMinLodFeaturesEXT>(static_cast<const VkPhysicalDeviceImageViewMinLodFeaturesEXT*>(object), settings, "VkPhysicalDeviceImageViewMinLodFeaturesEXT", indents, dump_json_VkPhysicalDeviceImageViewMinLodFeaturesEXT);
        break;
    case 1000391001:
        dump_json_pNext<const VkImageViewMinLodCreateInfoEXT>(static_cast<const VkImageViewMinLodCreateInfoEXT*>(object), settings, "VkImageViewMinLodCreateInfoEXT", indents, dump_json_VkImageViewMinLodCreateInfoEXT);
        break;
    case 1000392000:
        dump_json_pNext<const VkPhysicalDeviceMultiDrawFeaturesEXT>(static_cast<const VkPhysicalDeviceMultiDrawFeaturesEXT*>(object), settings, "VkPhysicalDeviceMultiDrawFeaturesEXT", indents, dump_json_VkPhysicalDeviceMultiDrawFeaturesEXT);
        break;
    case 1000392001:
        dump_json_pNext<const VkPhysicalDeviceMultiDrawPropertiesEXT>(static_cast<const VkPhysicalDeviceMultiDrawPropertiesEXT*>(object), settings, "VkPhysicalDeviceMultiDrawPropertiesEXT", indents, dump_json_VkPhysicalDeviceMultiDrawPropertiesEXT);
        break;
    case 1000393000:
        dump_json_pNext<const VkPhysicalDeviceImage2DViewOf3DFeaturesEXT>(static_cast<const VkPhysicalDeviceImage2DViewOf3DFeaturesEXT*>(object), settings, "VkPhysicalDeviceImage2DViewOf3DFeaturesEXT", indents, dump_json_VkPhysicalDeviceImage2DViewOf3DFeaturesEXT);
        break;
    case 1000395000:
        dump_json_pNext<const VkPhysicalDeviceShaderTileImageFeaturesEXT>(static_cast<const VkPhysicalDeviceShaderTileImageFeaturesEXT*>(object), settings, "VkPhysicalDeviceShaderTileImageFeaturesEXT", indents, dump_json_VkPhysicalDeviceShaderTileImageFeaturesEXT);
        break;
    case 1000395001:
        dump_json_pNext<const VkPhysicalDeviceShaderTileImagePropertiesEXT>(static_cast<const VkPhysicalDeviceShaderTileImagePropertiesEXT*>(object), settings, "VkPhysicalDeviceShaderTileImagePropertiesEXT", indents, dump_json_VkPhysicalDeviceShaderTileImagePropertiesEXT);
        break;
    case 1000396000:
        dump_json_pNext<const VkMicromapBuildInfoEXT>(static_cast<const VkMicromapBuildInfoEXT*>(object), settings, "VkMicromapBuildInfoEXT", indents, dump_json_VkMicromapBuildInfoEXT);
        break;
    case 1000396007:
        dump_json_pNext<const VkMicromapCreateInfoEXT>(static_cast<const VkMicromapCreateInfoEXT*>(object), settings, "VkMicromapCreateInfoEXT", indents, dump_json_VkMicromapCreateInfoEXT);
        break;
    case 1000396005:
        dump_json_pNext<const VkPhysicalDeviceOpacityMicromapFeaturesEXT>(static_cast<const VkPhysicalDeviceOpacityMicromapFeaturesEXT*>(object), settings, "VkPhysicalDeviceOpacityMicromapFeaturesEXT", indents, dump_json_VkPhysicalDeviceOpacityMicromapFeaturesEXT);
        break;
    case 1000396006:
        dump_json_pNext<const VkPhysicalDeviceOpacityMicromapPropertiesEXT>(static_cast<const VkPhysicalDeviceOpacityMicromapPropertiesEXT*>(object), settings, "VkPhysicalDeviceOpacityMicromapPropertiesEXT", indents, dump_json_VkPhysicalDeviceOpacityMicromapPropertiesEXT);
        break;
    case 1000396001:
        dump_json_pNext<const VkMicromapVersionInfoEXT>(static_cast<const VkMicromapVersionInfoEXT*>(object), settings, "VkMicromapVersionInfoEXT", indents, dump_json_VkMicromapVersionInfoEXT);
        break;
    case 1000396003:
        dump_json_pNext<const VkCopyMicromapToMemoryInfoEXT>(static_cast<const VkCopyMicromapToMemoryInfoEXT*>(object), settings, "VkCopyMicromapToMemoryInfoEXT", indents, dump_json_VkCopyMicromapToMemoryInfoEXT);
        break;
    case 1000396004:
        dump_json_pNext<const VkCopyMemoryToMicromapInfoEXT>(static_cast<const VkCopyMemoryToMicromapInfoEXT*>(object), settings, "VkCopyMemoryToMicromapInfoEXT", indents, dump_json_VkCopyMemoryToMicromapInfoEXT);
        break;
    case 1000396002:
        dump_json_pNext<const VkCopyMicromapInfoEXT>(static_cast<const VkCopyMicromapInfoEXT*>(object), settings, "VkCopyMicromapInfoEXT", indents, dump_json_VkCopyMicromapInfoEXT);
        break;
    case 1000396008:
        dump_json_pNext<const VkMicromapBuildSizesInfoEXT>(static_cast<const VkMicromapBuildSizesInfoEXT*>(object), settings, "VkMicromapBuildSizesInfoEXT", indents, dump_json_VkMicromapBuildSizesInfoEXT);
        break;
    case 1000396009:
        dump_json_pNext<const VkAccelerationStructureTrianglesOpacityMicromapEXT>(static_cast<const VkAccelerationStructureTrianglesOpacityMicromapEXT*>(object), settings, "VkAccelerationStructureTrianglesOpacityMicromapEXT", indents, dump_json_VkAccelerationStructureTrianglesOpacityMicromapEXT);
        break;
#if defined(VK_ENABLE_BETA_EXTENSIONS)
    case 1000397000:
        dump_json_pNext<const VkPhysicalDeviceDisplacementMicromapFeaturesNV>(static_cast<const VkPhysicalDeviceDisplacementMicromapFeaturesNV*>(object), settings, "VkPhysicalDeviceDisplacementMicromapFeaturesNV", indents, dump_json_VkPhysicalDeviceDisplacementMicromapFeaturesNV);
        break;
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
    case 1000397001:
        dump_json_pNext<const VkPhysicalDeviceDisplacementMicromapPropertiesNV>(static_cast<const VkPhysicalDeviceDisplacementMicromapPropertiesNV*>(object), settings, "VkPhysicalDeviceDisplacementMicromapPropertiesNV", indents, dump_json_VkPhysicalDeviceDisplacementMicromapPropertiesNV);
        break;
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
    case 1000397002:
        dump_json_pNext<const VkAccelerationStructureTrianglesDisplacementMicromapNV>(static_cast<const VkAccelerationStructureTrianglesDisplacementMicromapNV*>(object), settings, "VkAccelerationStructureTrianglesDisplacementMicromapNV", indents, dump_json_VkAccelerationStructureTrianglesDisplacementMicromapNV);
        break;
#endif // VK_ENABLE_BETA_EXTENSIONS
    case 1000404000:
        dump_json_pNext<const VkPhysicalDeviceClusterCullingShaderFeaturesHUAWEI>(static_cast<const VkPhysicalDeviceClusterCullingShaderFeaturesHUAWEI*>(object), settings, "VkPhysicalDeviceClusterCullingShaderFeaturesHUAWEI", indents, dump_json_VkPhysicalDeviceClusterCullingShaderFeaturesHUAWEI);
        break;
    case 1000404001:
        dump_json_pNext<const VkPhysicalDeviceClusterCullingShaderPropertiesHUAWEI>(static_cast<const VkPhysicalDeviceClusterCullingShaderPropertiesHUAWEI*>(object), settings, "VkPhysicalDeviceClusterCullingShaderPropertiesHUAWEI", indents, dump_json_VkPhysicalDeviceClusterCullingShaderPropertiesHUAWEI);
        break;
    case 1000404002:
        dump_json_pNext<const VkPhysicalDeviceClusterCullingShaderVrsFeaturesHUAWEI>(static_cast<const VkPhysicalDeviceClusterCullingShaderVrsFeaturesHUAWEI*>(object), settings, "VkPhysicalDeviceClusterCullingShaderVrsFeaturesHUAWEI", indents, dump_json_VkPhysicalDeviceClusterCullingShaderVrsFeaturesHUAWEI);
        break;
    case 1000411000:
        dump_json_pNext<const VkPhysicalDeviceBorderColorSwizzleFeaturesEXT>(static_cast<const VkPhysicalDeviceBorderColorSwizzleFeaturesEXT*>(object), settings, "VkPhysicalDeviceBorderColorSwizzleFeaturesEXT", indents, dump_json_VkPhysicalDeviceBorderColorSwizzleFeaturesEXT);
        break;
    case 1000411001:
        dump_json_pNext<const VkSamplerBorderColorComponentMappingCreateInfoEXT>(static_cast<const VkSamplerBorderColorComponentMappingCreateInfoEXT*>(object), settings, "VkSamplerBorderColorComponentMappingCreateInfoEXT", indents, dump_json_VkSamplerBorderColorComponentMappingCreateInfoEXT);
        break;
    case 1000412000:
        dump_json_pNext<const VkPhysicalDevicePageableDeviceLocalMemoryFeaturesEXT>(static_cast<const VkPhysicalDevicePageableDeviceLocalMemoryFeaturesEXT*>(object), settings, "VkPhysicalDevicePageableDeviceLocalMemoryFeaturesEXT", indents, dump_json_VkPhysicalDevicePageableDeviceLocalMemoryFeaturesEXT);
        break;
    case 1000415000:
        dump_json_pNext<const VkPhysicalDeviceShaderCorePropertiesARM>(static_cast<const VkPhysicalDeviceShaderCorePropertiesARM*>(object), settings, "VkPhysicalDeviceShaderCorePropertiesARM", indents, dump_json_VkPhysicalDeviceShaderCorePropertiesARM);
        break;
    case 1000416000:
        dump_json_pNext<const VkPhysicalDeviceShaderSubgroupRotateFeaturesKHR>(static_cast<const VkPhysicalDeviceShaderSubgroupRotateFeaturesKHR*>(object), settings, "VkPhysicalDeviceShaderSubgroupRotateFeaturesKHR", indents, dump_json_VkPhysicalDeviceShaderSubgroupRotateFeaturesKHR);
        break;
    case 1000417000:
        dump_json_pNext<const VkDeviceQueueShaderCoreControlCreateInfoARM>(static_cast<const VkDeviceQueueShaderCoreControlCreateInfoARM*>(object), settings, "VkDeviceQueueShaderCoreControlCreateInfoARM", indents, dump_json_VkDeviceQueueShaderCoreControlCreateInfoARM);
        break;
    case 1000417001:
        dump_json_pNext<const VkPhysicalDeviceSchedulingControlsFeaturesARM>(static_cast<const VkPhysicalDeviceSchedulingControlsFeaturesARM*>(object), settings, "VkPhysicalDeviceSchedulingControlsFeaturesARM", indents, dump_json_VkPhysicalDeviceSchedulingControlsFeaturesARM);
        break;
    case 1000417002:
        dump_json_pNext<const VkPhysicalDeviceSchedulingControlsPropertiesARM>(static_cast<const VkPhysicalDeviceSchedulingControlsPropertiesARM*>(object), settings, "VkPhysicalDeviceSchedulingControlsPropertiesARM", indents, dump_json_VkPhysicalDeviceSchedulingControlsPropertiesARM);
        break;
    case 1000418000:
        dump_json_pNext<const VkPhysicalDeviceImageSlicedViewOf3DFeaturesEXT>(static_cast<const VkPhysicalDeviceImageSlicedViewOf3DFeaturesEXT*>(object), settings, "VkPhysicalDeviceImageSlicedViewOf3DFeaturesEXT", indents, dump_json_VkPhysicalDeviceImageSlicedViewOf3DFeaturesEXT);
        break;
    case 1000418001:
        dump_json_pNext<const VkImageViewSlicedCreateInfoEXT>(static_cast<const VkImageViewSlicedCreateInfoEXT*>(object), settings, "VkImageViewSlicedCreateInfoEXT", indents, dump_json_VkImageViewSlicedCreateInfoEXT);
        break;
    case 1000420000:
        dump_json_pNext<const VkPhysicalDeviceDescriptorSetHostMappingFeaturesVALVE>(static_cast<const VkPhysicalDeviceDescriptorSetHostMappingFeaturesVALVE*>(object), settings, "VkPhysicalDeviceDescriptorSetHostMappingFeaturesVALVE", indents, dump_json_VkPhysicalDeviceDescriptorSetHostMappingFeaturesVALVE);
        break;
    case 1000420001:
        dump_json_pNext<const VkDescriptorSetBindingReferenceVALVE>(static_cast<const VkDescriptorSetBindingReferenceVALVE*>(object), settings, "VkDescriptorSetBindingReferenceVALVE", indents, dump_json_VkDescriptorSetBindingReferenceVALVE);
        break;
    case 1000420002:
        dump_json_pNext<const VkDescriptorSetLayoutHostMappingInfoVALVE>(static_cast<const VkDescriptorSetLayoutHostMappingInfoVALVE*>(object), settings, "VkDescriptorSetLayoutHostMappingInfoVALVE", indents, dump_json_VkDescriptorSetLayoutHostMappingInfoVALVE);
        break;
    case 1000421000:
        dump_json_pNext<const VkPhysicalDeviceDepthClampZeroOneFeaturesEXT>(static_cast<const VkPhysicalDeviceDepthClampZeroOneFeaturesEXT*>(object), settings, "VkPhysicalDeviceDepthClampZeroOneFeaturesEXT", indents, dump_json_VkPhysicalDeviceDepthClampZeroOneFeaturesEXT);
        break;
    case 1000422000:
        dump_json_pNext<const VkPhysicalDeviceNonSeamlessCubeMapFeaturesEXT>(static_cast<const VkPhysicalDeviceNonSeamlessCubeMapFeaturesEXT*>(object), settings, "VkPhysicalDeviceNonSeamlessCubeMapFeaturesEXT", indents, dump_json_VkPhysicalDeviceNonSeamlessCubeMapFeaturesEXT);
        break;
    case 1000424000:
        dump_json_pNext<const VkPhysicalDeviceRenderPassStripedFeaturesARM>(static_cast<const VkPhysicalDeviceRenderPassStripedFeaturesARM*>(object), settings, "VkPhysicalDeviceRenderPassStripedFeaturesARM", indents, dump_json_VkPhysicalDeviceRenderPassStripedFeaturesARM);
        break;
    case 1000424001:
        dump_json_pNext<const VkPhysicalDeviceRenderPassStripedPropertiesARM>(static_cast<const VkPhysicalDeviceRenderPassStripedPropertiesARM*>(object), settings, "VkPhysicalDeviceRenderPassStripedPropertiesARM", indents, dump_json_VkPhysicalDeviceRenderPassStripedPropertiesARM);
        break;
    case 1000424003:
        dump_json_pNext<const VkRenderPassStripeInfoARM>(static_cast<const VkRenderPassStripeInfoARM*>(object), settings, "VkRenderPassStripeInfoARM", indents, dump_json_VkRenderPassStripeInfoARM);
        break;
    case 1000424002:
        dump_json_pNext<const VkRenderPassStripeBeginInfoARM>(static_cast<const VkRenderPassStripeBeginInfoARM*>(object), settings, "VkRenderPassStripeBeginInfoARM", indents, dump_json_VkRenderPassStripeBeginInfoARM);
        break;
    case 1000424004:
        dump_json_pNext<const VkRenderPassStripeSubmitInfoARM>(static_cast<const VkRenderPassStripeSubmitInfoARM*>(object), settings, "VkRenderPassStripeSubmitInfoARM", indents, dump_json_VkRenderPassStripeSubmitInfoARM);
        break;
    case 1000425000:
        dump_json_pNext<const VkPhysicalDeviceFragmentDensityMapOffsetFeaturesQCOM>(static_cast<const VkPhysicalDeviceFragmentDensityMapOffsetFeaturesQCOM*>(object), settings, "VkPhysicalDeviceFragmentDensityMapOffsetFeaturesQCOM", indents, dump_json_VkPhysicalDeviceFragmentDensityMapOffsetFeaturesQCOM);
        break;
    case 1000425001:
        dump_json_pNext<const VkPhysicalDeviceFragmentDensityMapOffsetPropertiesQCOM>(static_cast<const VkPhysicalDeviceFragmentDensityMapOffsetPropertiesQCOM*>(object), settings, "VkPhysicalDeviceFragmentDensityMapOffsetPropertiesQCOM", indents, dump_json_VkPhysicalDeviceFragmentDensityMapOffsetPropertiesQCOM);
        break;
    case 1000425002:
        dump_json_pNext<const VkSubpassFragmentDensityMapOffsetEndInfoQCOM>(static_cast<const VkSubpassFragmentDensityMapOffsetEndInfoQCOM*>(object), settings, "VkSubpassFragmentDensityMapOffsetEndInfoQCOM", indents, dump_json_VkSubpassFragmentDensityMapOffsetEndInfoQCOM);
        break;
    case 1000426000:
        dump_json_pNext<const VkPhysicalDeviceCopyMemoryIndirectFeaturesNV>(static_cast<const VkPhysicalDeviceCopyMemoryIndirectFeaturesNV*>(object), settings, "VkPhysicalDeviceCopyMemoryIndirectFeaturesNV", indents, dump_json_VkPhysicalDeviceCopyMemoryIndirectFeaturesNV);
        break;
    case 1000426001:
        dump_json_pNext<const VkPhysicalDeviceCopyMemoryIndirectPropertiesNV>(static_cast<const VkPhysicalDeviceCopyMemoryIndirectPropertiesNV*>(object), settings, "VkPhysicalDeviceCopyMemoryIndirectPropertiesNV", indents, dump_json_VkPhysicalDeviceCopyMemoryIndirectPropertiesNV);
        break;
    case 1000427000:
        dump_json_pNext<const VkPhysicalDeviceMemoryDecompressionFeaturesNV>(static_cast<const VkPhysicalDeviceMemoryDecompressionFeaturesNV*>(object), settings, "VkPhysicalDeviceMemoryDecompressionFeaturesNV", indents, dump_json_VkPhysicalDeviceMemoryDecompressionFeaturesNV);
        break;
    case 1000427001:
        dump_json_pNext<const VkPhysicalDeviceMemoryDecompressionPropertiesNV>(static_cast<const VkPhysicalDeviceMemoryDecompressionPropertiesNV*>(object), settings, "VkPhysicalDeviceMemoryDecompressionPropertiesNV", indents, dump_json_VkPhysicalDeviceMemoryDecompressionPropertiesNV);
        break;
    case 1000428000:
        dump_json_pNext<const VkPhysicalDeviceDeviceGeneratedCommandsComputeFeaturesNV>(static_cast<const VkPhysicalDeviceDeviceGeneratedCommandsComputeFeaturesNV*>(object), settings, "VkPhysicalDeviceDeviceGeneratedCommandsComputeFeaturesNV", indents, dump_json_VkPhysicalDeviceDeviceGeneratedCommandsComputeFeaturesNV);
        break;
    case 1000428001:
        dump_json_pNext<const VkComputePipelineIndirectBufferInfoNV>(static_cast<const VkComputePipelineIndirectBufferInfoNV*>(object), settings, "VkComputePipelineIndirectBufferInfoNV", indents, dump_json_VkComputePipelineIndirectBufferInfoNV);
        break;
    case 1000428002:
        dump_json_pNext<const VkPipelineIndirectDeviceAddressInfoNV>(static_cast<const VkPipelineIndirectDeviceAddressInfoNV*>(object), settings, "VkPipelineIndirectDeviceAddressInfoNV", indents, dump_json_VkPipelineIndirectDeviceAddressInfoNV);
        break;
    case 1000430000:
        dump_json_pNext<const VkPhysicalDeviceLinearColorAttachmentFeaturesNV>(static_cast<const VkPhysicalDeviceLinearColorAttachmentFeaturesNV*>(object), settings, "VkPhysicalDeviceLinearColorAttachmentFeaturesNV", indents, dump_json_VkPhysicalDeviceLinearColorAttachmentFeaturesNV);
        break;
    case 1000434000:
        dump_json_pNext<const VkPhysicalDeviceShaderMaximalReconvergenceFeaturesKHR>(static_cast<const VkPhysicalDeviceShaderMaximalReconvergenceFeaturesKHR*>(object), settings, "VkPhysicalDeviceShaderMaximalReconvergenceFeaturesKHR", indents, dump_json_VkPhysicalDeviceShaderMaximalReconvergenceFeaturesKHR);
        break;
    case 1000437000:
        dump_json_pNext<const VkPhysicalDeviceImageCompressionControlSwapchainFeaturesEXT>(static_cast<const VkPhysicalDeviceImageCompressionControlSwapchainFeaturesEXT*>(object), settings, "VkPhysicalDeviceImageCompressionControlSwapchainFeaturesEXT", indents, dump_json_VkPhysicalDeviceImageCompressionControlSwapchainFeaturesEXT);
        break;
    case 1000440002:
        dump_json_pNext<const VkImageViewSampleWeightCreateInfoQCOM>(static_cast<const VkImageViewSampleWeightCreateInfoQCOM*>(object), settings, "VkImageViewSampleWeightCreateInfoQCOM", indents, dump_json_VkImageViewSampleWeightCreateInfoQCOM);
        break;
    case 1000440000:
        dump_json_pNext<const VkPhysicalDeviceImageProcessingFeaturesQCOM>(static_cast<const VkPhysicalDeviceImageProcessingFeaturesQCOM*>(object), settings, "VkPhysicalDeviceImageProcessingFeaturesQCOM", indents, dump_json_VkPhysicalDeviceImageProcessingFeaturesQCOM);
        break;
    case 1000440001:
        dump_json_pNext<const VkPhysicalDeviceImageProcessingPropertiesQCOM>(static_cast<const VkPhysicalDeviceImageProcessingPropertiesQCOM*>(object), settings, "VkPhysicalDeviceImageProcessingPropertiesQCOM", indents, dump_json_VkPhysicalDeviceImageProcessingPropertiesQCOM);
        break;
    case 1000451000:
        dump_json_pNext<const VkPhysicalDeviceNestedCommandBufferFeaturesEXT>(static_cast<const VkPhysicalDeviceNestedCommandBufferFeaturesEXT*>(object), settings, "VkPhysicalDeviceNestedCommandBufferFeaturesEXT", indents, dump_json_VkPhysicalDeviceNestedCommandBufferFeaturesEXT);
        break;
    case 1000451001:
        dump_json_pNext<const VkPhysicalDeviceNestedCommandBufferPropertiesEXT>(static_cast<const VkPhysicalDeviceNestedCommandBufferPropertiesEXT*>(object), settings, "VkPhysicalDeviceNestedCommandBufferPropertiesEXT", indents, dump_json_VkPhysicalDeviceNestedCommandBufferPropertiesEXT);
        break;
    case 1000453000:
        dump_json_pNext<const VkExternalMemoryAcquireUnmodifiedEXT>(static_cast<const VkExternalMemoryAcquireUnmodifiedEXT*>(object), settings, "VkExternalMemoryAcquireUnmodifiedEXT", indents, dump_json_VkExternalMemoryAcquireUnmodifiedEXT);
        break;
    case 1000455000:
        dump_json_pNext<const VkPhysicalDeviceExtendedDynamicState3FeaturesEXT>(static_cast<const VkPhysicalDeviceExtendedDynamicState3FeaturesEXT*>(object), settings, "VkPhysicalDeviceExtendedDynamicState3FeaturesEXT", indents, dump_json_VkPhysicalDeviceExtendedDynamicState3FeaturesEXT);
        break;
    case 1000455001:
        dump_json_pNext<const VkPhysicalDeviceExtendedDynamicState3PropertiesEXT>(static_cast<const VkPhysicalDeviceExtendedDynamicState3PropertiesEXT*>(object), settings, "VkPhysicalDeviceExtendedDynamicState3PropertiesEXT", indents, dump_json_VkPhysicalDeviceExtendedDynamicState3PropertiesEXT);
        break;
    case 1000458000:
        dump_json_pNext<const VkPhysicalDeviceSubpassMergeFeedbackFeaturesEXT>(static_cast<const VkPhysicalDeviceSubpassMergeFeedbackFeaturesEXT*>(object), settings, "VkPhysicalDeviceSubpassMergeFeedbackFeaturesEXT", indents, dump_json_VkPhysicalDeviceSubpassMergeFeedbackFeaturesEXT);
        break;
    case 1000458001:
        dump_json_pNext<const VkRenderPassCreationControlEXT>(static_cast<const VkRenderPassCreationControlEXT*>(object), settings, "VkRenderPassCreationControlEXT", indents, dump_json_VkRenderPassCreationControlEXT);
        break;
    case 1000458002:
        dump_json_pNext<const VkRenderPassCreationFeedbackCreateInfoEXT>(static_cast<const VkRenderPassCreationFeedbackCreateInfoEXT*>(object), settings, "VkRenderPassCreationFeedbackCreateInfoEXT", indents, dump_json_VkRenderPassCreationFeedbackCreateInfoEXT);
        break;
    case 1000458003:
        dump_json_pNext<const VkRenderPassSubpassFeedbackCreateInfoEXT>(static_cast<const VkRenderPassSubpassFeedbackCreateInfoEXT*>(object), settings, "VkRenderPassSubpassFeedbackCreateInfoEXT", indents, dump_json_VkRenderPassSubpassFeedbackCreateInfoEXT);
        break;
    case 1000459000:
        dump_json_pNext<const VkDirectDriverLoadingInfoLUNARG>(static_cast<const VkDirectDriverLoadingInfoLUNARG*>(object), settings, "VkDirectDriverLoadingInfoLUNARG", indents, dump_json_VkDirectDriverLoadingInfoLUNARG);
        break;
    case 1000459001:
        dump_json_pNext<const VkDirectDriverLoadingListLUNARG>(static_cast<const VkDirectDriverLoadingListLUNARG*>(object), settings, "VkDirectDriverLoadingListLUNARG", indents, dump_json_VkDirectDriverLoadingListLUNARG);
        break;
    case 1000462000:
        dump_json_pNext<const VkPhysicalDeviceShaderModuleIdentifierFeaturesEXT>(static_cast<const VkPhysicalDeviceShaderModuleIdentifierFeaturesEXT*>(object), settings, "VkPhysicalDeviceShaderModuleIdentifierFeaturesEXT", indents, dump_json_VkPhysicalDeviceShaderModuleIdentifierFeaturesEXT);
        break;
    case 1000462001:
        dump_json_pNext<const VkPhysicalDeviceShaderModuleIdentifierPropertiesEXT>(static_cast<const VkPhysicalDeviceShaderModuleIdentifierPropertiesEXT*>(object), settings, "VkPhysicalDeviceShaderModuleIdentifierPropertiesEXT", indents, dump_json_VkPhysicalDeviceShaderModuleIdentifierPropertiesEXT);
        break;
    case 1000462002:
        dump_json_pNext<const VkPipelineShaderStageModuleIdentifierCreateInfoEXT>(static_cast<const VkPipelineShaderStageModuleIdentifierCreateInfoEXT*>(object), settings, "VkPipelineShaderStageModuleIdentifierCreateInfoEXT", indents, dump_json_VkPipelineShaderStageModuleIdentifierCreateInfoEXT);
        break;
    case 1000462003:
        dump_json_pNext<const VkShaderModuleIdentifierEXT>(static_cast<const VkShaderModuleIdentifierEXT*>(object), settings, "VkShaderModuleIdentifierEXT", indents, dump_json_VkShaderModuleIdentifierEXT);
        break;
    case 1000464000:
        dump_json_pNext<const VkPhysicalDeviceOpticalFlowFeaturesNV>(static_cast<const VkPhysicalDeviceOpticalFlowFeaturesNV*>(object), settings, "VkPhysicalDeviceOpticalFlowFeaturesNV", indents, dump_json_VkPhysicalDeviceOpticalFlowFeaturesNV);
        break;
    case 1000464001:
        dump_json_pNext<const VkPhysicalDeviceOpticalFlowPropertiesNV>(static_cast<const VkPhysicalDeviceOpticalFlowPropertiesNV*>(object), settings, "VkPhysicalDeviceOpticalFlowPropertiesNV", indents, dump_json_VkPhysicalDeviceOpticalFlowPropertiesNV);
        break;
    case 1000464002:
        dump_json_pNext<const VkOpticalFlowImageFormatInfoNV>(static_cast<const VkOpticalFlowImageFormatInfoNV*>(object), settings, "VkOpticalFlowImageFormatInfoNV", indents, dump_json_VkOpticalFlowImageFormatInfoNV);
        break;
    case 1000464003:
        dump_json_pNext<const VkOpticalFlowImageFormatPropertiesNV>(static_cast<const VkOpticalFlowImageFormatPropertiesNV*>(object), settings, "VkOpticalFlowImageFormatPropertiesNV", indents, dump_json_VkOpticalFlowImageFormatPropertiesNV);
        break;
    case 1000464004:
        dump_json_pNext<const VkOpticalFlowSessionCreateInfoNV>(static_cast<const VkOpticalFlowSessionCreateInfoNV*>(object), settings, "VkOpticalFlowSessionCreateInfoNV", indents, dump_json_VkOpticalFlowSessionCreateInfoNV);
        break;
    case 1000464010:
        dump_json_pNext<const VkOpticalFlowSessionCreatePrivateDataInfoNV>(static_cast<const VkOpticalFlowSessionCreatePrivateDataInfoNV*>(object), settings, "VkOpticalFlowSessionCreatePrivateDataInfoNV", indents, dump_json_VkOpticalFlowSessionCreatePrivateDataInfoNV);
        break;
    case 1000464005:
        dump_json_pNext<const VkOpticalFlowExecuteInfoNV>(static_cast<const VkOpticalFlowExecuteInfoNV*>(object), settings, "VkOpticalFlowExecuteInfoNV", indents, dump_json_VkOpticalFlowExecuteInfoNV);
        break;
    case 1000465000:
        dump_json_pNext<const VkPhysicalDeviceLegacyDitheringFeaturesEXT>(static_cast<const VkPhysicalDeviceLegacyDitheringFeaturesEXT*>(object), settings, "VkPhysicalDeviceLegacyDitheringFeaturesEXT", indents, dump_json_VkPhysicalDeviceLegacyDitheringFeaturesEXT);
        break;
    case 1000466000:
        dump_json_pNext<const VkPhysicalDevicePipelineProtectedAccessFeaturesEXT>(static_cast<const VkPhysicalDevicePipelineProtectedAccessFeaturesEXT*>(object), settings, "VkPhysicalDevicePipelineProtectedAccessFeaturesEXT", indents, dump_json_VkPhysicalDevicePipelineProtectedAccessFeaturesEXT);
        break;
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
    case 1000468000:
        dump_json_pNext<const VkPhysicalDeviceExternalFormatResolveFeaturesANDROID>(static_cast<const VkPhysicalDeviceExternalFormatResolveFeaturesANDROID*>(object), settings, "VkPhysicalDeviceExternalFormatResolveFeaturesANDROID", indents, dump_json_VkPhysicalDeviceExternalFormatResolveFeaturesANDROID);
        break;
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
    case 1000468001:
        dump_json_pNext<const VkPhysicalDeviceExternalFormatResolvePropertiesANDROID>(static_cast<const VkPhysicalDeviceExternalFormatResolvePropertiesANDROID*>(object), settings, "VkPhysicalDeviceExternalFormatResolvePropertiesANDROID", indents, dump_json_VkPhysicalDeviceExternalFormatResolvePropertiesANDROID);
        break;
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
    case 1000468002:
        dump_json_pNext<const VkAndroidHardwareBufferFormatResolvePropertiesANDROID>(static_cast<const VkAndroidHardwareBufferFormatResolvePropertiesANDROID*>(object), settings, "VkAndroidHardwareBufferFormatResolvePropertiesANDROID", indents, dump_json_VkAndroidHardwareBufferFormatResolvePropertiesANDROID);
        break;
#endif // VK_USE_PLATFORM_ANDROID_KHR
    case 1000470000:
        dump_json_pNext<const VkPhysicalDeviceMaintenance5FeaturesKHR>(static_cast<const VkPhysicalDeviceMaintenance5FeaturesKHR*>(object), settings, "VkPhysicalDeviceMaintenance5FeaturesKHR", indents, dump_json_VkPhysicalDeviceMaintenance5FeaturesKHR);
        break;
    case 1000470001:
        dump_json_pNext<const VkPhysicalDeviceMaintenance5PropertiesKHR>(static_cast<const VkPhysicalDeviceMaintenance5PropertiesKHR*>(object), settings, "VkPhysicalDeviceMaintenance5PropertiesKHR", indents, dump_json_VkPhysicalDeviceMaintenance5PropertiesKHR);
        break;
    case 1000470003:
        dump_json_pNext<const VkRenderingAreaInfoKHR>(static_cast<const VkRenderingAreaInfoKHR*>(object), settings, "VkRenderingAreaInfoKHR", indents, dump_json_VkRenderingAreaInfoKHR);
        break;
    case 1000470004:
        dump_json_pNext<const VkDeviceImageSubresourceInfoKHR>(static_cast<const VkDeviceImageSubresourceInfoKHR*>(object), settings, "VkDeviceImageSubresourceInfoKHR", indents, dump_json_VkDeviceImageSubresourceInfoKHR);
        break;
    case 1000470005:
        dump_json_pNext<const VkPipelineCreateFlags2CreateInfoKHR>(static_cast<const VkPipelineCreateFlags2CreateInfoKHR*>(object), settings, "VkPipelineCreateFlags2CreateInfoKHR", indents, dump_json_VkPipelineCreateFlags2CreateInfoKHR);
        break;
    case 1000470006:
        dump_json_pNext<const VkBufferUsageFlags2CreateInfoKHR>(static_cast<const VkBufferUsageFlags2CreateInfoKHR*>(object), settings, "VkBufferUsageFlags2CreateInfoKHR", indents, dump_json_VkBufferUsageFlags2CreateInfoKHR);
        break;
    case 1000481000:
        dump_json_pNext<const VkPhysicalDeviceRayTracingPositionFetchFeaturesKHR>(static_cast<const VkPhysicalDeviceRayTracingPositionFetchFeaturesKHR*>(object), settings, "VkPhysicalDeviceRayTracingPositionFetchFeaturesKHR", indents, dump_json_VkPhysicalDeviceRayTracingPositionFetchFeaturesKHR);
        break;
    case 1000482000:
        dump_json_pNext<const VkPhysicalDeviceShaderObjectFeaturesEXT>(static_cast<const VkPhysicalDeviceShaderObjectFeaturesEXT*>(object), settings, "VkPhysicalDeviceShaderObjectFeaturesEXT", indents, dump_json_VkPhysicalDeviceShaderObjectFeaturesEXT);
        break;
    case 1000482001:
        dump_json_pNext<const VkPhysicalDeviceShaderObjectPropertiesEXT>(static_cast<const VkPhysicalDeviceShaderObjectPropertiesEXT*>(object), settings, "VkPhysicalDeviceShaderObjectPropertiesEXT", indents, dump_json_VkPhysicalDeviceShaderObjectPropertiesEXT);
        break;
    case 1000482002:
        dump_json_pNext<const VkShaderCreateInfoEXT>(static_cast<const VkShaderCreateInfoEXT*>(object), settings, "VkShaderCreateInfoEXT", indents, dump_json_VkShaderCreateInfoEXT);
        break;
    case 1000484000:
        dump_json_pNext<const VkPhysicalDeviceTilePropertiesFeaturesQCOM>(static_cast<const VkPhysicalDeviceTilePropertiesFeaturesQCOM*>(object), settings, "VkPhysicalDeviceTilePropertiesFeaturesQCOM", indents, dump_json_VkPhysicalDeviceTilePropertiesFeaturesQCOM);
        break;
    case 1000484001:
        dump_json_pNext<const VkTilePropertiesQCOM>(static_cast<const VkTilePropertiesQCOM*>(object), settings, "VkTilePropertiesQCOM", indents, dump_json_VkTilePropertiesQCOM);
        break;
    case 1000485000:
        dump_json_pNext<const VkPhysicalDeviceAmigoProfilingFeaturesSEC>(static_cast<const VkPhysicalDeviceAmigoProfilingFeaturesSEC*>(object), settings, "VkPhysicalDeviceAmigoProfilingFeaturesSEC", indents, dump_json_VkPhysicalDeviceAmigoProfilingFeaturesSEC);
        break;
    case 1000485001:
        dump_json_pNext<const VkAmigoProfilingSubmitInfoSEC>(static_cast<const VkAmigoProfilingSubmitInfoSEC*>(object), settings, "VkAmigoProfilingSubmitInfoSEC", indents, dump_json_VkAmigoProfilingSubmitInfoSEC);
        break;
    case 1000488000:
        dump_json_pNext<const VkPhysicalDeviceMultiviewPerViewViewportsFeaturesQCOM>(static_cast<const VkPhysicalDeviceMultiviewPerViewViewportsFeaturesQCOM*>(object), settings, "VkPhysicalDeviceMultiviewPerViewViewportsFeaturesQCOM", indents, dump_json_VkPhysicalDeviceMultiviewPerViewViewportsFeaturesQCOM);
        break;
    case 1000490001:
        dump_json_pNext<const VkPhysicalDeviceRayTracingInvocationReorderPropertiesNV>(static_cast<const VkPhysicalDeviceRayTracingInvocationReorderPropertiesNV*>(object), settings, "VkPhysicalDeviceRayTracingInvocationReorderPropertiesNV", indents, dump_json_VkPhysicalDeviceRayTracingInvocationReorderPropertiesNV);
        break;
    case 1000490000:
        dump_json_pNext<const VkPhysicalDeviceRayTracingInvocationReorderFeaturesNV>(static_cast<const VkPhysicalDeviceRayTracingInvocationReorderFeaturesNV*>(object), settings, "VkPhysicalDeviceRayTracingInvocationReorderFeaturesNV", indents, dump_json_VkPhysicalDeviceRayTracingInvocationReorderFeaturesNV);
        break;
    case 1000492000:
        dump_json_pNext<const VkPhysicalDeviceExtendedSparseAddressSpaceFeaturesNV>(static_cast<const VkPhysicalDeviceExtendedSparseAddressSpaceFeaturesNV*>(object), settings, "VkPhysicalDeviceExtendedSparseAddressSpaceFeaturesNV", indents, dump_json_VkPhysicalDeviceExtendedSparseAddressSpaceFeaturesNV);
        break;
    case 1000492001:
        dump_json_pNext<const VkPhysicalDeviceExtendedSparseAddressSpacePropertiesNV>(static_cast<const VkPhysicalDeviceExtendedSparseAddressSpacePropertiesNV*>(object), settings, "VkPhysicalDeviceExtendedSparseAddressSpacePropertiesNV", indents, dump_json_VkPhysicalDeviceExtendedSparseAddressSpacePropertiesNV);
        break;
    case 1000495000:
        dump_json_pNext<const VkPhysicalDeviceLegacyVertexAttributesFeaturesEXT>(static_cast<const VkPhysicalDeviceLegacyVertexAttributesFeaturesEXT*>(object), settings, "VkPhysicalDeviceLegacyVertexAttributesFeaturesEXT", indents, dump_json_VkPhysicalDeviceLegacyVertexAttributesFeaturesEXT);
        break;
    case 1000495001:
        dump_json_pNext<const VkPhysicalDeviceLegacyVertexAttributesPropertiesEXT>(static_cast<const VkPhysicalDeviceLegacyVertexAttributesPropertiesEXT*>(object), settings, "VkPhysicalDeviceLegacyVertexAttributesPropertiesEXT", indents, dump_json_VkPhysicalDeviceLegacyVertexAttributesPropertiesEXT);
        break;
    case 1000496000:
        dump_json_pNext<const VkLayerSettingsCreateInfoEXT>(static_cast<const VkLayerSettingsCreateInfoEXT*>(object), settings, "VkLayerSettingsCreateInfoEXT", indents, dump_json_VkLayerSettingsCreateInfoEXT);
        break;
    case 1000497000:
        dump_json_pNext<const VkPhysicalDeviceShaderCoreBuiltinsFeaturesARM>(static_cast<const VkPhysicalDeviceShaderCoreBuiltinsFeaturesARM*>(object), settings, "VkPhysicalDeviceShaderCoreBuiltinsFeaturesARM", indents, dump_json_VkPhysicalDeviceShaderCoreBuiltinsFeaturesARM);
        break;
    case 1000497001:
        dump_json_pNext<const VkPhysicalDeviceShaderCoreBuiltinsPropertiesARM>(static_cast<const VkPhysicalDeviceShaderCoreBuiltinsPropertiesARM*>(object), settings, "VkPhysicalDeviceShaderCoreBuiltinsPropertiesARM", indents, dump_json_VkPhysicalDeviceShaderCoreBuiltinsPropertiesARM);
        break;
    case 1000498000:
        dump_json_pNext<const VkPhysicalDevicePipelineLibraryGroupHandlesFeaturesEXT>(static_cast<const VkPhysicalDevicePipelineLibraryGroupHandlesFeaturesEXT*>(object), settings, "VkPhysicalDevicePipelineLibraryGroupHandlesFeaturesEXT", indents, dump_json_VkPhysicalDevicePipelineLibraryGroupHandlesFeaturesEXT);
        break;
    case 1000499000:
        dump_json_pNext<const VkPhysicalDeviceDynamicRenderingUnusedAttachmentsFeaturesEXT>(static_cast<const VkPhysicalDeviceDynamicRenderingUnusedAttachmentsFeaturesEXT*>(object), settings, "VkPhysicalDeviceDynamicRenderingUnusedAttachmentsFeaturesEXT", indents, dump_json_VkPhysicalDeviceDynamicRenderingUnusedAttachmentsFeaturesEXT);
        break;
    case 1000505000:
        dump_json_pNext<const VkLatencySleepModeInfoNV>(static_cast<const VkLatencySleepModeInfoNV*>(object), settings, "VkLatencySleepModeInfoNV", indents, dump_json_VkLatencySleepModeInfoNV);
        break;
    case 1000505001:
        dump_json_pNext<const VkLatencySleepInfoNV>(static_cast<const VkLatencySleepInfoNV*>(object), settings, "VkLatencySleepInfoNV", indents, dump_json_VkLatencySleepInfoNV);
        break;
    case 1000505002:
        dump_json_pNext<const VkSetLatencyMarkerInfoNV>(static_cast<const VkSetLatencyMarkerInfoNV*>(object), settings, "VkSetLatencyMarkerInfoNV", indents, dump_json_VkSetLatencyMarkerInfoNV);
        break;
    case 1000505004:
        dump_json_pNext<const VkLatencyTimingsFrameReportNV>(static_cast<const VkLatencyTimingsFrameReportNV*>(object), settings, "VkLatencyTimingsFrameReportNV", indents, dump_json_VkLatencyTimingsFrameReportNV);
        break;
    case 1000505003:
        dump_json_pNext<const VkGetLatencyMarkerInfoNV>(static_cast<const VkGetLatencyMarkerInfoNV*>(object), settings, "VkGetLatencyMarkerInfoNV", indents, dump_json_VkGetLatencyMarkerInfoNV);
        break;
    case 1000505005:
        dump_json_pNext<const VkLatencySubmissionPresentIdNV>(static_cast<const VkLatencySubmissionPresentIdNV*>(object), settings, "VkLatencySubmissionPresentIdNV", indents, dump_json_VkLatencySubmissionPresentIdNV);
        break;
    case 1000505007:
        dump_json_pNext<const VkSwapchainLatencyCreateInfoNV>(static_cast<const VkSwapchainLatencyCreateInfoNV*>(object), settings, "VkSwapchainLatencyCreateInfoNV", indents, dump_json_VkSwapchainLatencyCreateInfoNV);
        break;
    case 1000505006:
        dump_json_pNext<const VkOutOfBandQueueTypeInfoNV>(static_cast<const VkOutOfBandQueueTypeInfoNV*>(object), settings, "VkOutOfBandQueueTypeInfoNV", indents, dump_json_VkOutOfBandQueueTypeInfoNV);
        break;
    case 1000505008:
        dump_json_pNext<const VkLatencySurfaceCapabilitiesNV>(static_cast<const VkLatencySurfaceCapabilitiesNV*>(object), settings, "VkLatencySurfaceCapabilitiesNV", indents, dump_json_VkLatencySurfaceCapabilitiesNV);
        break;
    case 1000506001:
        dump_json_pNext<const VkCooperativeMatrixPropertiesKHR>(static_cast<const VkCooperativeMatrixPropertiesKHR*>(object), settings, "VkCooperativeMatrixPropertiesKHR", indents, dump_json_VkCooperativeMatrixPropertiesKHR);
        break;
    case 1000506000:
        dump_json_pNext<const VkPhysicalDeviceCooperativeMatrixFeaturesKHR>(static_cast<const VkPhysicalDeviceCooperativeMatrixFeaturesKHR*>(object), settings, "VkPhysicalDeviceCooperativeMatrixFeaturesKHR", indents, dump_json_VkPhysicalDeviceCooperativeMatrixFeaturesKHR);
        break;
    case 1000506002:
        dump_json_pNext<const VkPhysicalDeviceCooperativeMatrixPropertiesKHR>(static_cast<const VkPhysicalDeviceCooperativeMatrixPropertiesKHR*>(object), settings, "VkPhysicalDeviceCooperativeMatrixPropertiesKHR", indents, dump_json_VkPhysicalDeviceCooperativeMatrixPropertiesKHR);
        break;
    case 1000510000:
        dump_json_pNext<const VkPhysicalDeviceMultiviewPerViewRenderAreasFeaturesQCOM>(static_cast<const VkPhysicalDeviceMultiviewPerViewRenderAreasFeaturesQCOM*>(object), settings, "VkPhysicalDeviceMultiviewPerViewRenderAreasFeaturesQCOM", indents, dump_json_VkPhysicalDeviceMultiviewPerViewRenderAreasFeaturesQCOM);
        break;
    case 1000510001:
        dump_json_pNext<const VkMultiviewPerViewRenderAreasRenderPassBeginInfoQCOM>(static_cast<const VkMultiviewPerViewRenderAreasRenderPassBeginInfoQCOM*>(object), settings, "VkMultiviewPerViewRenderAreasRenderPassBeginInfoQCOM", indents, dump_json_VkMultiviewPerViewRenderAreasRenderPassBeginInfoQCOM);
        break;
    case 1000512003:
        dump_json_pNext<const VkVideoDecodeAV1ProfileInfoKHR>(static_cast<const VkVideoDecodeAV1ProfileInfoKHR*>(object), settings, "VkVideoDecodeAV1ProfileInfoKHR", indents, dump_json_VkVideoDecodeAV1ProfileInfoKHR);
        break;
    case 1000512000:
        dump_json_pNext<const VkVideoDecodeAV1CapabilitiesKHR>(static_cast<const VkVideoDecodeAV1CapabilitiesKHR*>(object), settings, "VkVideoDecodeAV1CapabilitiesKHR", indents, dump_json_VkVideoDecodeAV1CapabilitiesKHR);
        break;
    case 1000512004:
        dump_json_pNext<const VkVideoDecodeAV1SessionParametersCreateInfoKHR>(static_cast<const VkVideoDecodeAV1SessionParametersCreateInfoKHR*>(object), settings, "VkVideoDecodeAV1SessionParametersCreateInfoKHR", indents, dump_json_VkVideoDecodeAV1SessionParametersCreateInfoKHR);
        break;
    case 1000512001:
        dump_json_pNext<const VkVideoDecodeAV1PictureInfoKHR>(static_cast<const VkVideoDecodeAV1PictureInfoKHR*>(object), settings, "VkVideoDecodeAV1PictureInfoKHR", indents, dump_json_VkVideoDecodeAV1PictureInfoKHR);
        break;
    case 1000512005:
        dump_json_pNext<const VkVideoDecodeAV1DpbSlotInfoKHR>(static_cast<const VkVideoDecodeAV1DpbSlotInfoKHR*>(object), settings, "VkVideoDecodeAV1DpbSlotInfoKHR", indents, dump_json_VkVideoDecodeAV1DpbSlotInfoKHR);
        break;
    case 1000515000:
        dump_json_pNext<const VkPhysicalDeviceVideoMaintenance1FeaturesKHR>(static_cast<const VkPhysicalDeviceVideoMaintenance1FeaturesKHR*>(object), settings, "VkPhysicalDeviceVideoMaintenance1FeaturesKHR", indents, dump_json_VkPhysicalDeviceVideoMaintenance1FeaturesKHR);
        break;
    case 1000515001:
        dump_json_pNext<const VkVideoInlineQueryInfoKHR>(static_cast<const VkVideoInlineQueryInfoKHR*>(object), settings, "VkVideoInlineQueryInfoKHR", indents, dump_json_VkVideoInlineQueryInfoKHR);
        break;
    case 1000516000:
        dump_json_pNext<const VkPhysicalDevicePerStageDescriptorSetFeaturesNV>(static_cast<const VkPhysicalDevicePerStageDescriptorSetFeaturesNV*>(object), settings, "VkPhysicalDevicePerStageDescriptorSetFeaturesNV", indents, dump_json_VkPhysicalDevicePerStageDescriptorSetFeaturesNV);
        break;
    case 1000518000:
        dump_json_pNext<const VkPhysicalDeviceImageProcessing2FeaturesQCOM>(static_cast<const VkPhysicalDeviceImageProcessing2FeaturesQCOM*>(object), settings, "VkPhysicalDeviceImageProcessing2FeaturesQCOM", indents, dump_json_VkPhysicalDeviceImageProcessing2FeaturesQCOM);
        break;
    case 1000518001:
        dump_json_pNext<const VkPhysicalDeviceImageProcessing2PropertiesQCOM>(static_cast<const VkPhysicalDeviceImageProcessing2PropertiesQCOM*>(object), settings, "VkPhysicalDeviceImageProcessing2PropertiesQCOM", indents, dump_json_VkPhysicalDeviceImageProcessing2PropertiesQCOM);
        break;
    case 1000518002:
        dump_json_pNext<const VkSamplerBlockMatchWindowCreateInfoQCOM>(static_cast<const VkSamplerBlockMatchWindowCreateInfoQCOM*>(object), settings, "VkSamplerBlockMatchWindowCreateInfoQCOM", indents, dump_json_VkSamplerBlockMatchWindowCreateInfoQCOM);
        break;
    case 1000519001:
        dump_json_pNext<const VkPhysicalDeviceCubicWeightsFeaturesQCOM>(static_cast<const VkPhysicalDeviceCubicWeightsFeaturesQCOM*>(object), settings, "VkPhysicalDeviceCubicWeightsFeaturesQCOM", indents, dump_json_VkPhysicalDeviceCubicWeightsFeaturesQCOM);
        break;
    case 1000519000:
        dump_json_pNext<const VkSamplerCubicWeightsCreateInfoQCOM>(static_cast<const VkSamplerCubicWeightsCreateInfoQCOM*>(object), settings, "VkSamplerCubicWeightsCreateInfoQCOM", indents, dump_json_VkSamplerCubicWeightsCreateInfoQCOM);
        break;
    case 1000519002:
        dump_json_pNext<const VkBlitImageCubicWeightsInfoQCOM>(static_cast<const VkBlitImageCubicWeightsInfoQCOM*>(object), settings, "VkBlitImageCubicWeightsInfoQCOM", indents, dump_json_VkBlitImageCubicWeightsInfoQCOM);
        break;
    case 1000520000:
        dump_json_pNext<const VkPhysicalDeviceYcbcrDegammaFeaturesQCOM>(static_cast<const VkPhysicalDeviceYcbcrDegammaFeaturesQCOM*>(object), settings, "VkPhysicalDeviceYcbcrDegammaFeaturesQCOM", indents, dump_json_VkPhysicalDeviceYcbcrDegammaFeaturesQCOM);
        break;
    case 1000520001:
        dump_json_pNext<const VkSamplerYcbcrConversionYcbcrDegammaCreateInfoQCOM>(static_cast<const VkSamplerYcbcrConversionYcbcrDegammaCreateInfoQCOM*>(object), settings, "VkSamplerYcbcrConversionYcbcrDegammaCreateInfoQCOM", indents, dump_json_VkSamplerYcbcrConversionYcbcrDegammaCreateInfoQCOM);
        break;
    case 1000521000:
        dump_json_pNext<const VkPhysicalDeviceCubicClampFeaturesQCOM>(static_cast<const VkPhysicalDeviceCubicClampFeaturesQCOM*>(object), settings, "VkPhysicalDeviceCubicClampFeaturesQCOM", indents, dump_json_VkPhysicalDeviceCubicClampFeaturesQCOM);
        break;
    case 1000524000:
        dump_json_pNext<const VkPhysicalDeviceAttachmentFeedbackLoopDynamicStateFeaturesEXT>(static_cast<const VkPhysicalDeviceAttachmentFeedbackLoopDynamicStateFeaturesEXT*>(object), settings, "VkPhysicalDeviceAttachmentFeedbackLoopDynamicStateFeaturesEXT", indents, dump_json_VkPhysicalDeviceAttachmentFeedbackLoopDynamicStateFeaturesEXT);
        break;
    case 1000525000:
        dump_json_pNext<const VkPhysicalDeviceVertexAttributeDivisorPropertiesKHR>(static_cast<const VkPhysicalDeviceVertexAttributeDivisorPropertiesKHR*>(object), settings, "VkPhysicalDeviceVertexAttributeDivisorPropertiesKHR", indents, dump_json_VkPhysicalDeviceVertexAttributeDivisorPropertiesKHR);
        break;
    case 1000528000:
        dump_json_pNext<const VkPhysicalDeviceShaderFloatControls2FeaturesKHR>(static_cast<const VkPhysicalDeviceShaderFloatControls2FeaturesKHR*>(object), settings, "VkPhysicalDeviceShaderFloatControls2FeaturesKHR", indents, dump_json_VkPhysicalDeviceShaderFloatControls2FeaturesKHR);
        break;
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
    case 1000529000:
        dump_json_pNext<const VkScreenBufferPropertiesQNX>(static_cast<const VkScreenBufferPropertiesQNX*>(object), settings, "VkScreenBufferPropertiesQNX", indents, dump_json_VkScreenBufferPropertiesQNX);
        break;
#endif // VK_USE_PLATFORM_SCREEN_QNX
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
    case 1000529001:
        dump_json_pNext<const VkScreenBufferFormatPropertiesQNX>(static_cast<const VkScreenBufferFormatPropertiesQNX*>(object), settings, "VkScreenBufferFormatPropertiesQNX", indents, dump_json_VkScreenBufferFormatPropertiesQNX);
        break;
#endif // VK_USE_PLATFORM_SCREEN_QNX
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
    case 1000529002:
        dump_json_pNext<const VkImportScreenBufferInfoQNX>(static_cast<const VkImportScreenBufferInfoQNX*>(object), settings, "VkImportScreenBufferInfoQNX", indents, dump_json_VkImportScreenBufferInfoQNX);
        break;
#endif // VK_USE_PLATFORM_SCREEN_QNX
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
    case 1000529003:
        dump_json_pNext<const VkExternalFormatQNX>(static_cast<const VkExternalFormatQNX*>(object), settings, "VkExternalFormatQNX", indents, dump_json_VkExternalFormatQNX);
        break;
#endif // VK_USE_PLATFORM_SCREEN_QNX
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
    case 1000529004:
        dump_json_pNext<const VkPhysicalDeviceExternalMemoryScreenBufferFeaturesQNX>(static_cast<const VkPhysicalDeviceExternalMemoryScreenBufferFeaturesQNX*>(object), settings, "VkPhysicalDeviceExternalMemoryScreenBufferFeaturesQNX", indents, dump_json_VkPhysicalDeviceExternalMemoryScreenBufferFeaturesQNX);
        break;
#endif // VK_USE_PLATFORM_SCREEN_QNX
    case 1000530000:
        dump_json_pNext<const VkPhysicalDeviceLayeredDriverPropertiesMSFT>(static_cast<const VkPhysicalDeviceLayeredDriverPropertiesMSFT*>(object), settings, "VkPhysicalDeviceLayeredDriverPropertiesMSFT", indents, dump_json_VkPhysicalDeviceLayeredDriverPropertiesMSFT);
        break;
    case 1000544000:
        dump_json_pNext<const VkPhysicalDeviceShaderExpectAssumeFeaturesKHR>(static_cast<const VkPhysicalDeviceShaderExpectAssumeFeaturesKHR*>(object), settings, "VkPhysicalDeviceShaderExpectAssumeFeaturesKHR", indents, dump_json_VkPhysicalDeviceShaderExpectAssumeFeaturesKHR);
        break;
    case 1000545000:
        dump_json_pNext<const VkPhysicalDeviceMaintenance6FeaturesKHR>(static_cast<const VkPhysicalDeviceMaintenance6FeaturesKHR*>(object), settings, "VkPhysicalDeviceMaintenance6FeaturesKHR", indents, dump_json_VkPhysicalDeviceMaintenance6FeaturesKHR);
        break;
    case 1000545001:
        dump_json_pNext<const VkPhysicalDeviceMaintenance6PropertiesKHR>(static_cast<const VkPhysicalDeviceMaintenance6PropertiesKHR*>(object), settings, "VkPhysicalDeviceMaintenance6PropertiesKHR", indents, dump_json_VkPhysicalDeviceMaintenance6PropertiesKHR);
        break;
    case 1000545002:
        dump_json_pNext<const VkBindMemoryStatusKHR>(static_cast<const VkBindMemoryStatusKHR*>(object), settings, "VkBindMemoryStatusKHR", indents, dump_json_VkBindMemoryStatusKHR);
        break;
    case 1000545003:
        dump_json_pNext<const VkBindDescriptorSetsInfoKHR>(static_cast<const VkBindDescriptorSetsInfoKHR*>(object), settings, "VkBindDescriptorSetsInfoKHR", indents, dump_json_VkBindDescriptorSetsInfoKHR);
        break;
    case 1000545004:
        dump_json_pNext<const VkPushConstantsInfoKHR>(static_cast<const VkPushConstantsInfoKHR*>(object), settings, "VkPushConstantsInfoKHR", indents, dump_json_VkPushConstantsInfoKHR);
        break;
    case 1000545005:
        dump_json_pNext<const VkPushDescriptorSetInfoKHR>(static_cast<const VkPushDescriptorSetInfoKHR*>(object), settings, "VkPushDescriptorSetInfoKHR", indents, dump_json_VkPushDescriptorSetInfoKHR);
        break;
    case 1000545006:
        dump_json_pNext<const VkPushDescriptorSetWithTemplateInfoKHR>(static_cast<const VkPushDescriptorSetWithTemplateInfoKHR*>(object), settings, "VkPushDescriptorSetWithTemplateInfoKHR", indents, dump_json_VkPushDescriptorSetWithTemplateInfoKHR);
        break;
    case 1000545007:
        dump_json_pNext<const VkSetDescriptorBufferOffsetsInfoEXT>(static_cast<const VkSetDescriptorBufferOffsetsInfoEXT*>(object), settings, "VkSetDescriptorBufferOffsetsInfoEXT", indents, dump_json_VkSetDescriptorBufferOffsetsInfoEXT);
        break;
    case 1000545008:
        dump_json_pNext<const VkBindDescriptorBufferEmbeddedSamplersInfoEXT>(static_cast<const VkBindDescriptorBufferEmbeddedSamplersInfoEXT*>(object), settings, "VkBindDescriptorBufferEmbeddedSamplersInfoEXT", indents, dump_json_VkBindDescriptorBufferEmbeddedSamplersInfoEXT);
        break;
    case 1000546000:
        dump_json_pNext<const VkPhysicalDeviceDescriptorPoolOverallocationFeaturesNV>(static_cast<const VkPhysicalDeviceDescriptorPoolOverallocationFeaturesNV*>(object), settings, "VkPhysicalDeviceDescriptorPoolOverallocationFeaturesNV", indents, dump_json_VkPhysicalDeviceDescriptorPoolOverallocationFeaturesNV);
        break;
    case 1000555000:
        dump_json_pNext<const VkPhysicalDeviceRawAccessChainsFeaturesNV>(static_cast<const VkPhysicalDeviceRawAccessChainsFeaturesNV*>(object), settings, "VkPhysicalDeviceRawAccessChainsFeaturesNV", indents, dump_json_VkPhysicalDeviceRawAccessChainsFeaturesNV);
        break;
    case 1000558000:
        dump_json_pNext<const VkPhysicalDeviceShaderRelaxedExtendedInstructionFeaturesKHR>(static_cast<const VkPhysicalDeviceShaderRelaxedExtendedInstructionFeaturesKHR*>(object), settings, "VkPhysicalDeviceShaderRelaxedExtendedInstructionFeaturesKHR", indents, dump_json_VkPhysicalDeviceShaderRelaxedExtendedInstructionFeaturesKHR);
        break;
    case 1000562000:
        dump_json_pNext<const VkPhysicalDeviceMaintenance7FeaturesKHR>(static_cast<const VkPhysicalDeviceMaintenance7FeaturesKHR*>(object), settings, "VkPhysicalDeviceMaintenance7FeaturesKHR", indents, dump_json_VkPhysicalDeviceMaintenance7FeaturesKHR);
        break;
    case 1000562001:
        dump_json_pNext<const VkPhysicalDeviceMaintenance7PropertiesKHR>(static_cast<const VkPhysicalDeviceMaintenance7PropertiesKHR*>(object), settings, "VkPhysicalDeviceMaintenance7PropertiesKHR", indents, dump_json_VkPhysicalDeviceMaintenance7PropertiesKHR);
        break;
    case 1000562003:
        dump_json_pNext<const VkPhysicalDeviceLayeredApiPropertiesKHR>(static_cast<const VkPhysicalDeviceLayeredApiPropertiesKHR*>(object), settings, "VkPhysicalDeviceLayeredApiPropertiesKHR", indents, dump_json_VkPhysicalDeviceLayeredApiPropertiesKHR);
        break;
    case 1000562002:
        dump_json_pNext<const VkPhysicalDeviceLayeredApiPropertiesListKHR>(static_cast<const VkPhysicalDeviceLayeredApiPropertiesListKHR*>(object), settings, "VkPhysicalDeviceLayeredApiPropertiesListKHR", indents, dump_json_VkPhysicalDeviceLayeredApiPropertiesListKHR);
        break;
    case 1000562004:
        dump_json_pNext<const VkPhysicalDeviceLayeredApiVulkanPropertiesKHR>(static_cast<const VkPhysicalDeviceLayeredApiVulkanPropertiesKHR*>(object), settings, "VkPhysicalDeviceLayeredApiVulkanPropertiesKHR", indents, dump_json_VkPhysicalDeviceLayeredApiVulkanPropertiesKHR);
        break;
    case 1000563000:
        dump_json_pNext<const VkPhysicalDeviceShaderAtomicFloat16VectorFeaturesNV>(static_cast<const VkPhysicalDeviceShaderAtomicFloat16VectorFeaturesNV*>(object), settings, "VkPhysicalDeviceShaderAtomicFloat16VectorFeaturesNV", indents, dump_json_VkPhysicalDeviceShaderAtomicFloat16VectorFeaturesNV);
        break;
    case 1000564000:
        dump_json_pNext<const VkPhysicalDeviceShaderReplicatedCompositesFeaturesEXT>(static_cast<const VkPhysicalDeviceShaderReplicatedCompositesFeaturesEXT*>(object), settings, "VkPhysicalDeviceShaderReplicatedCompositesFeaturesEXT", indents, dump_json_VkPhysicalDeviceShaderReplicatedCompositesFeaturesEXT);
        break;
    case 1000568000:
        dump_json_pNext<const VkPhysicalDeviceRayTracingValidationFeaturesNV>(static_cast<const VkPhysicalDeviceRayTracingValidationFeaturesNV*>(object), settings, "VkPhysicalDeviceRayTracingValidationFeaturesNV", indents, dump_json_VkPhysicalDeviceRayTracingValidationFeaturesNV);
        break;
    case 1000575000:
        dump_json_pNext<const VkPhysicalDeviceImageAlignmentControlFeaturesMESA>(static_cast<const VkPhysicalDeviceImageAlignmentControlFeaturesMESA*>(object), settings, "VkPhysicalDeviceImageAlignmentControlFeaturesMESA", indents, dump_json_VkPhysicalDeviceImageAlignmentControlFeaturesMESA);
        break;
    case 1000575001:
        dump_json_pNext<const VkPhysicalDeviceImageAlignmentControlPropertiesMESA>(static_cast<const VkPhysicalDeviceImageAlignmentControlPropertiesMESA*>(object), settings, "VkPhysicalDeviceImageAlignmentControlPropertiesMESA", indents, dump_json_VkPhysicalDeviceImageAlignmentControlPropertiesMESA);
        break;
    case 1000575002:
        dump_json_pNext<const VkImageAlignmentControlCreateInfoMESA>(static_cast<const VkImageAlignmentControlCreateInfoMESA*>(object), settings, "VkImageAlignmentControlCreateInfoMESA", indents, dump_json_VkImageAlignmentControlCreateInfoMESA);
        break;
    case VK_STRUCTURE_TYPE_LOADER_INSTANCE_CREATE_INFO: // 47
    case VK_STRUCTURE_TYPE_LOADER_DEVICE_CREATE_INFO: // 48
        if(static_cast<const VkBaseInStructure*>(object)->pNext != nullptr){
            dump_json_pNext_trampoline(static_cast<const void*>(static_cast<const VkBaseInStructure*>(object)->pNext), settings, indents);
        } else {
            settings.stream() << settings.indentation(indents) << "{\n";
            settings.stream() << settings.indentation(indents + 1) << "\"type\" : \"const void*\",\n";
            settings.stream() << settings.indentation(indents + 1) << "\"name\" : \"pNext\",\n";
            settings.stream() << settings.indentation(indents + 1) << "\"value\" : \"NULL\"\n";
            settings.stream() << settings.indentation(indents) << "}";
        }
        break;
    default:
        settings.stream() << settings.indentation(indents) << "{\n";
        settings.stream() << settings.indentation(indents + 1) << "\"type\" : \"const void*\",\n";
        settings.stream() << settings.indentation(indents + 1) << "\"name\" : \"pNext\",\n";
        settings.stream() << settings.indentation(indents + 1) << "\"value\" : \"UNKNOWN (" << (int64_t) (static_cast<const VkBaseInStructure*>(object)->sType) << ")\"\n";
        settings.stream() << settings.indentation(indents) << "}";
    }
}

//========================= Function Implementations ========================//

void dump_json_vkCreateInstance(ApiDumpInstance& dump_inst, VkResult result, const VkInstanceCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkInstance* pInstance)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_pointer<const VkInstanceCreateInfo>(pCreateInfo, settings, "const VkInstanceCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkInstanceCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkInstance>(pInstance, settings, "VkInstance*", "pInstance", false, false, 4, dump_json_VkInstance);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyInstance(ApiDumpInstance& dump_inst, VkInstance instance, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkEnumeratePhysicalDevices(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, uint32_t* pPhysicalDeviceCount, VkPhysicalDevice* pPhysicalDevices)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPhysicalDeviceCount, settings, "uint32_t*", "pPhysicalDeviceCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkPhysicalDevice>(pPhysicalDevices, *pPhysicalDeviceCount, settings, "VkPhysicalDevice*", "VkPhysicalDevice", "pPhysicalDevices", false, false, 4, dump_json_VkPhysicalDevice); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceFeatures(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, VkPhysicalDeviceFeatures* pFeatures)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceFeatures>(pFeatures, settings, "VkPhysicalDeviceFeatures*", "pFeatures", true, false, 4, dump_json_VkPhysicalDeviceFeatures);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceFormatProperties(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, VkFormat format, VkFormatProperties* pFormatProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkFormat>(format, NULL, settings, "VkFormat", "format", false, false, 4, dump_json_VkFormat);
        settings.stream() << ",\n";
        dump_json_pointer<const VkFormatProperties>(pFormatProperties, settings, "VkFormatProperties*", "pFormatProperties", true, false, 4, dump_json_VkFormatProperties);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceImageFormatProperties(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, VkFormat format, VkImageType type, VkImageTiling tiling, VkImageUsageFlags usage, VkImageCreateFlags flags, VkImageFormatProperties* pImageFormatProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkFormat>(format, NULL, settings, "VkFormat", "format", false, false, 4, dump_json_VkFormat);
        settings.stream() << ",\n";
        dump_json_value<const VkImageType>(type, NULL, settings, "VkImageType", "type", false, false, 4, dump_json_VkImageType);
        settings.stream() << ",\n";
        dump_json_value<const VkImageTiling>(tiling, NULL, settings, "VkImageTiling", "tiling", false, false, 4, dump_json_VkImageTiling);
        settings.stream() << ",\n";
        dump_json_value<const VkImageUsageFlags>(usage, NULL, settings, "VkImageUsageFlags", "usage", false, false, 4, dump_json_VkImageUsageFlags);
        settings.stream() << ",\n";
        dump_json_value<const VkImageCreateFlags>(flags, NULL, settings, "VkImageCreateFlags", "flags", false, false, 4, dump_json_VkImageCreateFlags);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageFormatProperties>(pImageFormatProperties, settings, "VkImageFormatProperties*", "pImageFormatProperties", true, false, 4, dump_json_VkImageFormatProperties);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceProperties(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, VkPhysicalDeviceProperties* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceProperties>(pProperties, settings, "VkPhysicalDeviceProperties*", "pProperties", true, false, 4, dump_json_VkPhysicalDeviceProperties);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceQueueFamilyProperties(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, uint32_t* pQueueFamilyPropertyCount, VkQueueFamilyProperties* pQueueFamilyProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pQueueFamilyPropertyCount, settings, "uint32_t*", "pQueueFamilyPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkQueueFamilyProperties>(pQueueFamilyProperties, *pQueueFamilyPropertyCount, settings, "VkQueueFamilyProperties*", "VkQueueFamilyProperties", "pQueueFamilyProperties", true, false, 4, dump_json_VkQueueFamilyProperties); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceMemoryProperties(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, VkPhysicalDeviceMemoryProperties* pMemoryProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceMemoryProperties>(pMemoryProperties, settings, "VkPhysicalDeviceMemoryProperties*", "pMemoryProperties", true, false, 4, dump_json_VkPhysicalDeviceMemoryProperties);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateDevice(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, const VkDeviceCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkDevice* pDevice)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceCreateInfo>(pCreateInfo, settings, "const VkDeviceCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkDeviceCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDevice>(pDevice, settings, "VkDevice*", "pDevice", false, false, 4, dump_json_VkDevice);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyDevice(ApiDumpInstance& dump_inst, VkDevice device, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkEnumerateInstanceExtensionProperties(ApiDumpInstance& dump_inst, VkResult result, const char* pLayerName, uint32_t* pPropertyCount, VkExtensionProperties* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const char*>(pLayerName, NULL, settings, "const char*", "pLayerName", false, false, 4, dump_json_cstring);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPropertyCount, settings, "uint32_t*", "pPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkExtensionProperties>(pProperties, *pPropertyCount, settings, "VkExtensionProperties*", "VkExtensionProperties", "pProperties", true, false, 4, dump_json_VkExtensionProperties); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkEnumerateDeviceExtensionProperties(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, const char* pLayerName, uint32_t* pPropertyCount, VkExtensionProperties* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const char*>(pLayerName, NULL, settings, "const char*", "pLayerName", false, false, 4, dump_json_cstring);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPropertyCount, settings, "uint32_t*", "pPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkExtensionProperties>(pProperties, *pPropertyCount, settings, "VkExtensionProperties*", "VkExtensionProperties", "pProperties", true, false, 4, dump_json_VkExtensionProperties); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkEnumerateInstanceLayerProperties(ApiDumpInstance& dump_inst, VkResult result, uint32_t* pPropertyCount, VkLayerProperties* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_pointer<const uint32_t>(pPropertyCount, settings, "uint32_t*", "pPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkLayerProperties>(pProperties, *pPropertyCount, settings, "VkLayerProperties*", "VkLayerProperties", "pProperties", true, false, 4, dump_json_VkLayerProperties); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkEnumerateDeviceLayerProperties(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t* pPropertyCount, VkLayerProperties* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPropertyCount, settings, "uint32_t*", "pPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkLayerProperties>(pProperties, *pPropertyCount, settings, "VkLayerProperties*", "VkLayerProperties", "pProperties", true, false, 4, dump_json_VkLayerProperties); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceQueue(ApiDumpInstance& dump_inst, VkDevice device, uint32_t queueFamilyIndex, uint32_t queueIndex, VkQueue* pQueue)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(queueFamilyIndex, NULL, settings, "uint32_t", "queueFamilyIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(queueIndex, NULL, settings, "uint32_t", "queueIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_pointer<const VkQueue>(pQueue, settings, "VkQueue*", "pQueue", false, false, 4, dump_json_VkQueue);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkQueueSubmit(ApiDumpInstance& dump_inst, VkResult result, VkQueue queue, uint32_t submitCount, const VkSubmitInfo* pSubmits, VkFence fence)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkQueue>(queue, NULL, settings, "VkQueue", "queue", false, false, 4, dump_json_VkQueue);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(submitCount, NULL, settings, "uint32_t", "submitCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkSubmitInfo>(pSubmits, submitCount, settings, "const VkSubmitInfo*", "const VkSubmitInfo", "pSubmits", true, false, 4, dump_json_VkSubmitInfo); // PQA
        settings.stream() << ",\n";
        dump_json_value<const VkFence>(fence, NULL, settings, "VkFence", "fence", false, false, 4, dump_json_VkFence);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkQueueWaitIdle(ApiDumpInstance& dump_inst, VkResult result, VkQueue queue)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkQueue>(queue, NULL, settings, "VkQueue", "queue", false, false, 4, dump_json_VkQueue);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDeviceWaitIdle(ApiDumpInstance& dump_inst, VkResult result, VkDevice device)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkAllocateMemory(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkMemoryAllocateInfo* pAllocateInfo, const VkAllocationCallbacks* pAllocator, VkDeviceMemory* pMemory)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryAllocateInfo>(pAllocateInfo, settings, "const VkMemoryAllocateInfo*", "pAllocateInfo", true, false, 4, dump_json_VkMemoryAllocateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceMemory>(pMemory, settings, "VkDeviceMemory*", "pMemory", false, false, 4, dump_json_VkDeviceMemory);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkFreeMemory(ApiDumpInstance& dump_inst, VkDevice device, VkDeviceMemory memory, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceMemory>(memory, NULL, settings, "VkDeviceMemory", "memory", false, false, 4, dump_json_VkDeviceMemory);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkMapMemory(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDeviceMemory memory, VkDeviceSize offset, VkDeviceSize size, VkMemoryMapFlags flags, void** ppData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceMemory>(memory, NULL, settings, "VkDeviceMemory", "memory", false, false, 4, dump_json_VkDeviceMemory);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(size, NULL, settings, "VkDeviceSize", "size", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkMemoryMapFlags>(flags, NULL, settings, "VkMemoryMapFlags", "flags", false, false, 4, dump_json_VkMemoryMapFlags);
        settings.stream() << ",\n";
        dump_json_pointer<const void*>(ppData, settings, "void**", "ppData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkUnmapMemory(ApiDumpInstance& dump_inst, VkDevice device, VkDeviceMemory memory)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceMemory>(memory, NULL, settings, "VkDeviceMemory", "memory", false, false, 4, dump_json_VkDeviceMemory);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkFlushMappedMemoryRanges(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t memoryRangeCount, const VkMappedMemoryRange* pMemoryRanges)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(memoryRangeCount, NULL, settings, "uint32_t", "memoryRangeCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkMappedMemoryRange>(pMemoryRanges, memoryRangeCount, settings, "const VkMappedMemoryRange*", "const VkMappedMemoryRange", "pMemoryRanges", true, false, 4, dump_json_VkMappedMemoryRange); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkInvalidateMappedMemoryRanges(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t memoryRangeCount, const VkMappedMemoryRange* pMemoryRanges)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(memoryRangeCount, NULL, settings, "uint32_t", "memoryRangeCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkMappedMemoryRange>(pMemoryRanges, memoryRangeCount, settings, "const VkMappedMemoryRange*", "const VkMappedMemoryRange", "pMemoryRanges", true, false, 4, dump_json_VkMappedMemoryRange); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceMemoryCommitment(ApiDumpInstance& dump_inst, VkDevice device, VkDeviceMemory memory, VkDeviceSize* pCommittedMemoryInBytes)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceMemory>(memory, NULL, settings, "VkDeviceMemory", "memory", false, false, 4, dump_json_VkDeviceMemory);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceSize>(pCommittedMemoryInBytes, settings, "VkDeviceSize*", "pCommittedMemoryInBytes", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkBindBufferMemory(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkBuffer buffer, VkDeviceMemory memory, VkDeviceSize memoryOffset)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceMemory>(memory, NULL, settings, "VkDeviceMemory", "memory", false, false, 4, dump_json_VkDeviceMemory);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(memoryOffset, NULL, settings, "VkDeviceSize", "memoryOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkBindImageMemory(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkImage image, VkDeviceMemory memory, VkDeviceSize memoryOffset)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(image, NULL, settings, "VkImage", "image", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceMemory>(memory, NULL, settings, "VkDeviceMemory", "memory", false, false, 4, dump_json_VkDeviceMemory);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(memoryOffset, NULL, settings, "VkDeviceSize", "memoryOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetBufferMemoryRequirements(ApiDumpInstance& dump_inst, VkDevice device, VkBuffer buffer, VkMemoryRequirements* pMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryRequirements>(pMemoryRequirements, settings, "VkMemoryRequirements*", "pMemoryRequirements", true, false, 4, dump_json_VkMemoryRequirements);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetImageMemoryRequirements(ApiDumpInstance& dump_inst, VkDevice device, VkImage image, VkMemoryRequirements* pMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(image, NULL, settings, "VkImage", "image", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryRequirements>(pMemoryRequirements, settings, "VkMemoryRequirements*", "pMemoryRequirements", true, false, 4, dump_json_VkMemoryRequirements);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetImageSparseMemoryRequirements(ApiDumpInstance& dump_inst, VkDevice device, VkImage image, uint32_t* pSparseMemoryRequirementCount, VkSparseImageMemoryRequirements* pSparseMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(image, NULL, settings, "VkImage", "image", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pSparseMemoryRequirementCount, settings, "uint32_t*", "pSparseMemoryRequirementCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkSparseImageMemoryRequirements>(pSparseMemoryRequirements, *pSparseMemoryRequirementCount, settings, "VkSparseImageMemoryRequirements*", "VkSparseImageMemoryRequirements", "pSparseMemoryRequirements", true, false, 4, dump_json_VkSparseImageMemoryRequirements); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceSparseImageFormatProperties(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, VkFormat format, VkImageType type, VkSampleCountFlagBits samples, VkImageUsageFlags usage, VkImageTiling tiling, uint32_t* pPropertyCount, VkSparseImageFormatProperties* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkFormat>(format, NULL, settings, "VkFormat", "format", false, false, 4, dump_json_VkFormat);
        settings.stream() << ",\n";
        dump_json_value<const VkImageType>(type, NULL, settings, "VkImageType", "type", false, false, 4, dump_json_VkImageType);
        settings.stream() << ",\n";
        dump_json_value<const VkSampleCountFlagBits>(samples, NULL, settings, "VkSampleCountFlagBits", "samples", false, false, 4, dump_json_VkSampleCountFlagBits);
        settings.stream() << ",\n";
        dump_json_value<const VkImageUsageFlags>(usage, NULL, settings, "VkImageUsageFlags", "usage", false, false, 4, dump_json_VkImageUsageFlags);
        settings.stream() << ",\n";
        dump_json_value<const VkImageTiling>(tiling, NULL, settings, "VkImageTiling", "tiling", false, false, 4, dump_json_VkImageTiling);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPropertyCount, settings, "uint32_t*", "pPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkSparseImageFormatProperties>(pProperties, *pPropertyCount, settings, "VkSparseImageFormatProperties*", "VkSparseImageFormatProperties", "pProperties", true, false, 4, dump_json_VkSparseImageFormatProperties); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkQueueBindSparse(ApiDumpInstance& dump_inst, VkResult result, VkQueue queue, uint32_t bindInfoCount, const VkBindSparseInfo* pBindInfo, VkFence fence)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkQueue>(queue, NULL, settings, "VkQueue", "queue", false, false, 4, dump_json_VkQueue);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(bindInfoCount, NULL, settings, "uint32_t", "bindInfoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBindSparseInfo>(pBindInfo, bindInfoCount, settings, "const VkBindSparseInfo*", "const VkBindSparseInfo", "pBindInfo", true, false, 4, dump_json_VkBindSparseInfo); // PQA
        settings.stream() << ",\n";
        dump_json_value<const VkFence>(fence, NULL, settings, "VkFence", "fence", false, false, 4, dump_json_VkFence);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateFence(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkFenceCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkFence* pFence)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkFenceCreateInfo>(pCreateInfo, settings, "const VkFenceCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkFenceCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkFence>(pFence, settings, "VkFence*", "pFence", false, false, 4, dump_json_VkFence);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyFence(ApiDumpInstance& dump_inst, VkDevice device, VkFence fence, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkFence>(fence, NULL, settings, "VkFence", "fence", false, false, 4, dump_json_VkFence);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkResetFences(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t fenceCount, const VkFence* pFences)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(fenceCount, NULL, settings, "uint32_t", "fenceCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkFence>(pFences, fenceCount, settings, "const VkFence*", "const VkFence", "pFences", false, false, 4, dump_json_VkFence); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetFenceStatus(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkFence fence)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkFence>(fence, NULL, settings, "VkFence", "fence", false, false, 4, dump_json_VkFence);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkWaitForFences(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t fenceCount, const VkFence* pFences, VkBool32 waitAll, uint64_t timeout)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(fenceCount, NULL, settings, "uint32_t", "fenceCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkFence>(pFences, fenceCount, settings, "const VkFence*", "const VkFence", "pFences", false, false, 4, dump_json_VkFence); // PQA
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(waitAll, NULL, settings, "VkBool32", "waitAll", false, false, 4, dump_json_VkBool32);
        settings.stream() << ",\n";
        dump_json_value<const uint64_t>(timeout, NULL, settings, "uint64_t", "timeout", false, false, 4, dump_json_uint64_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateSemaphore(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkSemaphoreCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSemaphore* pSemaphore)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSemaphoreCreateInfo>(pCreateInfo, settings, "const VkSemaphoreCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkSemaphoreCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSemaphore>(pSemaphore, settings, "VkSemaphore*", "pSemaphore", false, false, 4, dump_json_VkSemaphore);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroySemaphore(ApiDumpInstance& dump_inst, VkDevice device, VkSemaphore semaphore, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSemaphore>(semaphore, NULL, settings, "VkSemaphore", "semaphore", false, false, 4, dump_json_VkSemaphore);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateEvent(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkEventCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkEvent* pEvent)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkEventCreateInfo>(pCreateInfo, settings, "const VkEventCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkEventCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkEvent>(pEvent, settings, "VkEvent*", "pEvent", false, false, 4, dump_json_VkEvent);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyEvent(ApiDumpInstance& dump_inst, VkDevice device, VkEvent event, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkEvent>(event, NULL, settings, "VkEvent", "event", false, false, 4, dump_json_VkEvent);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetEventStatus(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkEvent event)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkEvent>(event, NULL, settings, "VkEvent", "event", false, false, 4, dump_json_VkEvent);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkSetEvent(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkEvent event)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkEvent>(event, NULL, settings, "VkEvent", "event", false, false, 4, dump_json_VkEvent);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkResetEvent(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkEvent event)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkEvent>(event, NULL, settings, "VkEvent", "event", false, false, 4, dump_json_VkEvent);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateQueryPool(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkQueryPoolCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkQueryPool* pQueryPool)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkQueryPoolCreateInfo>(pCreateInfo, settings, "const VkQueryPoolCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkQueryPoolCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkQueryPool>(pQueryPool, settings, "VkQueryPool*", "pQueryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyQueryPool(ApiDumpInstance& dump_inst, VkDevice device, VkQueryPool queryPool, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetQueryPoolResults(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkQueryPool queryPool, uint32_t firstQuery, uint32_t queryCount, size_t dataSize, void* pData, VkDeviceSize stride, VkQueryResultFlags flags)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstQuery, NULL, settings, "uint32_t", "firstQuery", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(queryCount, NULL, settings, "uint32_t", "queryCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const size_t>(dataSize, NULL, settings, "size_t", "dataSize", false, false, 4, dump_json_size_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(stride, NULL, settings, "VkDeviceSize", "stride", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryResultFlags>(flags, NULL, settings, "VkQueryResultFlags", "flags", false, false, 4, dump_json_VkQueryResultFlags);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateBuffer(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkBufferCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkBuffer* pBuffer)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBufferCreateInfo>(pCreateInfo, settings, "const VkBufferCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkBufferCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBuffer>(pBuffer, settings, "VkBuffer*", "pBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyBuffer(ApiDumpInstance& dump_inst, VkDevice device, VkBuffer buffer, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateBufferView(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkBufferViewCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkBufferView* pView)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBufferViewCreateInfo>(pCreateInfo, settings, "const VkBufferViewCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkBufferViewCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBufferView>(pView, settings, "VkBufferView*", "pView", false, false, 4, dump_json_VkBufferView);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyBufferView(ApiDumpInstance& dump_inst, VkDevice device, VkBufferView bufferView, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkBufferView>(bufferView, NULL, settings, "VkBufferView", "bufferView", false, false, 4, dump_json_VkBufferView);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateImage(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkImageCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkImage* pImage)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageCreateInfo>(pCreateInfo, settings, "const VkImageCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkImageCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImage>(pImage, settings, "VkImage*", "pImage", false, false, 4, dump_json_VkImage);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyImage(ApiDumpInstance& dump_inst, VkDevice device, VkImage image, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(image, NULL, settings, "VkImage", "image", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetImageSubresourceLayout(ApiDumpInstance& dump_inst, VkDevice device, VkImage image, const VkImageSubresource* pSubresource, VkSubresourceLayout* pLayout)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(image, NULL, settings, "VkImage", "image", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageSubresource>(pSubresource, settings, "const VkImageSubresource*", "pSubresource", true, false, 4, dump_json_VkImageSubresource);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSubresourceLayout>(pLayout, settings, "VkSubresourceLayout*", "pLayout", true, false, 4, dump_json_VkSubresourceLayout);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateImageView(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkImageViewCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkImageView* pView)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageViewCreateInfo>(pCreateInfo, settings, "const VkImageViewCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkImageViewCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageView>(pView, settings, "VkImageView*", "pView", false, false, 4, dump_json_VkImageView);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyImageView(ApiDumpInstance& dump_inst, VkDevice device, VkImageView imageView, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkImageView>(imageView, NULL, settings, "VkImageView", "imageView", false, false, 4, dump_json_VkImageView);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateShaderModule(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkShaderModuleCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkShaderModule* pShaderModule)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkShaderModuleCreateInfo>(pCreateInfo, settings, "const VkShaderModuleCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkShaderModuleCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkShaderModule>(pShaderModule, settings, "VkShaderModule*", "pShaderModule", false, false, 4, dump_json_VkShaderModule);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyShaderModule(ApiDumpInstance& dump_inst, VkDevice device, VkShaderModule shaderModule, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkShaderModule>(shaderModule, NULL, settings, "VkShaderModule", "shaderModule", false, false, 4, dump_json_VkShaderModule);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreatePipelineCache(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkPipelineCacheCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkPipelineCache* pPipelineCache)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPipelineCacheCreateInfo>(pCreateInfo, settings, "const VkPipelineCacheCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkPipelineCacheCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPipelineCache>(pPipelineCache, settings, "VkPipelineCache*", "pPipelineCache", false, false, 4, dump_json_VkPipelineCache);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyPipelineCache(ApiDumpInstance& dump_inst, VkDevice device, VkPipelineCache pipelineCache, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineCache>(pipelineCache, NULL, settings, "VkPipelineCache", "pipelineCache", false, false, 4, dump_json_VkPipelineCache);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPipelineCacheData(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkPipelineCache pipelineCache, size_t* pDataSize, void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineCache>(pipelineCache, NULL, settings, "VkPipelineCache", "pipelineCache", false, false, 4, dump_json_VkPipelineCache);
        settings.stream() << ",\n";
        dump_json_pointer<const size_t>(pDataSize, settings, "size_t*", "pDataSize", false, false, 4, dump_json_size_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkMergePipelineCaches(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkPipelineCache dstCache, uint32_t srcCacheCount, const VkPipelineCache* pSrcCaches)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineCache>(dstCache, NULL, settings, "VkPipelineCache", "dstCache", false, false, 4, dump_json_VkPipelineCache);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(srcCacheCount, NULL, settings, "uint32_t", "srcCacheCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkPipelineCache>(pSrcCaches, srcCacheCount, settings, "const VkPipelineCache*", "const VkPipelineCache", "pSrcCaches", false, false, 4, dump_json_VkPipelineCache); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateGraphicsPipelines(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkPipelineCache pipelineCache, uint32_t createInfoCount, const VkGraphicsPipelineCreateInfo* pCreateInfos, const VkAllocationCallbacks* pAllocator, VkPipeline* pPipelines)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineCache>(pipelineCache, NULL, settings, "VkPipelineCache", "pipelineCache", false, false, 4, dump_json_VkPipelineCache);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(createInfoCount, NULL, settings, "uint32_t", "createInfoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkGraphicsPipelineCreateInfo>(pCreateInfos, createInfoCount, settings, "const VkGraphicsPipelineCreateInfo*", "const VkGraphicsPipelineCreateInfo", "pCreateInfos", true, false, 4, dump_json_VkGraphicsPipelineCreateInfo); // PQA
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_array<const VkPipeline>(pPipelines, createInfoCount, settings, "VkPipeline*", "VkPipeline", "pPipelines", false, false, 4, dump_json_VkPipeline); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateComputePipelines(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkPipelineCache pipelineCache, uint32_t createInfoCount, const VkComputePipelineCreateInfo* pCreateInfos, const VkAllocationCallbacks* pAllocator, VkPipeline* pPipelines)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineCache>(pipelineCache, NULL, settings, "VkPipelineCache", "pipelineCache", false, false, 4, dump_json_VkPipelineCache);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(createInfoCount, NULL, settings, "uint32_t", "createInfoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkComputePipelineCreateInfo>(pCreateInfos, createInfoCount, settings, "const VkComputePipelineCreateInfo*", "const VkComputePipelineCreateInfo", "pCreateInfos", true, false, 4, dump_json_VkComputePipelineCreateInfo); // PQA
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_array<const VkPipeline>(pPipelines, createInfoCount, settings, "VkPipeline*", "VkPipeline", "pPipelines", false, false, 4, dump_json_VkPipeline); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyPipeline(ApiDumpInstance& dump_inst, VkDevice device, VkPipeline pipeline, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipeline>(pipeline, NULL, settings, "VkPipeline", "pipeline", false, false, 4, dump_json_VkPipeline);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreatePipelineLayout(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkPipelineLayoutCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkPipelineLayout* pPipelineLayout)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPipelineLayoutCreateInfo>(pCreateInfo, settings, "const VkPipelineLayoutCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkPipelineLayoutCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPipelineLayout>(pPipelineLayout, settings, "VkPipelineLayout*", "pPipelineLayout", false, false, 4, dump_json_VkPipelineLayout);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyPipelineLayout(ApiDumpInstance& dump_inst, VkDevice device, VkPipelineLayout pipelineLayout, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineLayout>(pipelineLayout, NULL, settings, "VkPipelineLayout", "pipelineLayout", false, false, 4, dump_json_VkPipelineLayout);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateSampler(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkSamplerCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSampler* pSampler)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSamplerCreateInfo>(pCreateInfo, settings, "const VkSamplerCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkSamplerCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSampler>(pSampler, settings, "VkSampler*", "pSampler", false, false, 4, dump_json_VkSampler);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroySampler(ApiDumpInstance& dump_inst, VkDevice device, VkSampler sampler, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSampler>(sampler, NULL, settings, "VkSampler", "sampler", false, false, 4, dump_json_VkSampler);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateDescriptorSetLayout(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkDescriptorSetLayoutCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkDescriptorSetLayout* pSetLayout)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorSetLayoutCreateInfo>(pCreateInfo, settings, "const VkDescriptorSetLayoutCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkDescriptorSetLayoutCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorSetLayout>(pSetLayout, settings, "VkDescriptorSetLayout*", "pSetLayout", false, false, 4, dump_json_VkDescriptorSetLayout);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyDescriptorSetLayout(ApiDumpInstance& dump_inst, VkDevice device, VkDescriptorSetLayout descriptorSetLayout, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDescriptorSetLayout>(descriptorSetLayout, NULL, settings, "VkDescriptorSetLayout", "descriptorSetLayout", false, false, 4, dump_json_VkDescriptorSetLayout);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateDescriptorPool(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkDescriptorPoolCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkDescriptorPool* pDescriptorPool)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorPoolCreateInfo>(pCreateInfo, settings, "const VkDescriptorPoolCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkDescriptorPoolCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorPool>(pDescriptorPool, settings, "VkDescriptorPool*", "pDescriptorPool", false, false, 4, dump_json_VkDescriptorPool);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyDescriptorPool(ApiDumpInstance& dump_inst, VkDevice device, VkDescriptorPool descriptorPool, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDescriptorPool>(descriptorPool, NULL, settings, "VkDescriptorPool", "descriptorPool", false, false, 4, dump_json_VkDescriptorPool);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkResetDescriptorPool(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDescriptorPool descriptorPool, VkDescriptorPoolResetFlags flags)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDescriptorPool>(descriptorPool, NULL, settings, "VkDescriptorPool", "descriptorPool", false, false, 4, dump_json_VkDescriptorPool);
        settings.stream() << ",\n";
        dump_json_value<const VkDescriptorPoolResetFlags>(flags, NULL, settings, "VkDescriptorPoolResetFlags", "flags", false, false, 4, dump_json_VkDescriptorPoolResetFlags);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkAllocateDescriptorSets(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkDescriptorSetAllocateInfo* pAllocateInfo, VkDescriptorSet* pDescriptorSets)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorSetAllocateInfo>(pAllocateInfo, settings, "const VkDescriptorSetAllocateInfo*", "pAllocateInfo", true, false, 4, dump_json_VkDescriptorSetAllocateInfo);
        settings.stream() << ",\n";
        dump_json_array<const VkDescriptorSet>(pDescriptorSets, pAllocateInfo->descriptorSetCount, settings, "VkDescriptorSet*", "VkDescriptorSet", "pDescriptorSets", false, false, 4, dump_json_VkDescriptorSet); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkFreeDescriptorSets(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDescriptorPool descriptorPool, uint32_t descriptorSetCount, const VkDescriptorSet* pDescriptorSets)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDescriptorPool>(descriptorPool, NULL, settings, "VkDescriptorPool", "descriptorPool", false, false, 4, dump_json_VkDescriptorPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(descriptorSetCount, NULL, settings, "uint32_t", "descriptorSetCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkDescriptorSet>(pDescriptorSets, descriptorSetCount, settings, "const VkDescriptorSet*", "const VkDescriptorSet", "pDescriptorSets", false, false, 4, dump_json_VkDescriptorSet); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkUpdateDescriptorSets(ApiDumpInstance& dump_inst, VkDevice device, uint32_t descriptorWriteCount, const VkWriteDescriptorSet* pDescriptorWrites, uint32_t descriptorCopyCount, const VkCopyDescriptorSet* pDescriptorCopies)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(descriptorWriteCount, NULL, settings, "uint32_t", "descriptorWriteCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkWriteDescriptorSet>(pDescriptorWrites, descriptorWriteCount, settings, "const VkWriteDescriptorSet*", "const VkWriteDescriptorSet", "pDescriptorWrites", true, false, 4, dump_json_VkWriteDescriptorSet); // PQA
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(descriptorCopyCount, NULL, settings, "uint32_t", "descriptorCopyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkCopyDescriptorSet>(pDescriptorCopies, descriptorCopyCount, settings, "const VkCopyDescriptorSet*", "const VkCopyDescriptorSet", "pDescriptorCopies", true, false, 4, dump_json_VkCopyDescriptorSet); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateFramebuffer(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkFramebufferCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkFramebuffer* pFramebuffer)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkFramebufferCreateInfo>(pCreateInfo, settings, "const VkFramebufferCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkFramebufferCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkFramebuffer>(pFramebuffer, settings, "VkFramebuffer*", "pFramebuffer", false, false, 4, dump_json_VkFramebuffer);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyFramebuffer(ApiDumpInstance& dump_inst, VkDevice device, VkFramebuffer framebuffer, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkFramebuffer>(framebuffer, NULL, settings, "VkFramebuffer", "framebuffer", false, false, 4, dump_json_VkFramebuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateRenderPass(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkRenderPassCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkRenderPass* pRenderPass)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRenderPassCreateInfo>(pCreateInfo, settings, "const VkRenderPassCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkRenderPassCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRenderPass>(pRenderPass, settings, "VkRenderPass*", "pRenderPass", false, false, 4, dump_json_VkRenderPass);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyRenderPass(ApiDumpInstance& dump_inst, VkDevice device, VkRenderPass renderPass, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkRenderPass>(renderPass, NULL, settings, "VkRenderPass", "renderPass", false, false, 4, dump_json_VkRenderPass);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetRenderAreaGranularity(ApiDumpInstance& dump_inst, VkDevice device, VkRenderPass renderPass, VkExtent2D* pGranularity)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkRenderPass>(renderPass, NULL, settings, "VkRenderPass", "renderPass", false, false, 4, dump_json_VkRenderPass);
        settings.stream() << ",\n";
        dump_json_pointer<const VkExtent2D>(pGranularity, settings, "VkExtent2D*", "pGranularity", true, false, 4, dump_json_VkExtent2D);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateCommandPool(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkCommandPoolCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkCommandPool* pCommandPool)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCommandPoolCreateInfo>(pCreateInfo, settings, "const VkCommandPoolCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkCommandPoolCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCommandPool>(pCommandPool, settings, "VkCommandPool*", "pCommandPool", false, false, 4, dump_json_VkCommandPool);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyCommandPool(ApiDumpInstance& dump_inst, VkDevice device, VkCommandPool commandPool, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkCommandPool>(commandPool, NULL, settings, "VkCommandPool", "commandPool", false, false, 4, dump_json_VkCommandPool);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkResetCommandPool(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkCommandPool commandPool, VkCommandPoolResetFlags flags)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkCommandPool>(commandPool, NULL, settings, "VkCommandPool", "commandPool", false, false, 4, dump_json_VkCommandPool);
        settings.stream() << ",\n";
        dump_json_value<const VkCommandPoolResetFlags>(flags, NULL, settings, "VkCommandPoolResetFlags", "flags", false, false, 4, dump_json_VkCommandPoolResetFlags);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkAllocateCommandBuffers(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkCommandBufferAllocateInfo* pAllocateInfo, VkCommandBuffer* pCommandBuffers)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCommandBufferAllocateInfo>(pAllocateInfo, settings, "const VkCommandBufferAllocateInfo*", "pAllocateInfo", true, false, 4, dump_json_VkCommandBufferAllocateInfo);
        settings.stream() << ",\n";
        dump_json_array<const VkCommandBuffer>(pCommandBuffers, pAllocateInfo->commandBufferCount, settings, "VkCommandBuffer*", "VkCommandBuffer", "pCommandBuffers", false, false, 4, dump_json_VkCommandBuffer); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkFreeCommandBuffers(ApiDumpInstance& dump_inst, VkDevice device, VkCommandPool commandPool, uint32_t commandBufferCount, const VkCommandBuffer* pCommandBuffers)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkCommandPool>(commandPool, NULL, settings, "VkCommandPool", "commandPool", false, false, 4, dump_json_VkCommandPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(commandBufferCount, NULL, settings, "uint32_t", "commandBufferCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkCommandBuffer>(pCommandBuffers, commandBufferCount, settings, "const VkCommandBuffer*", "const VkCommandBuffer", "pCommandBuffers", false, false, 4, dump_json_VkCommandBuffer); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkBeginCommandBuffer(ApiDumpInstance& dump_inst, VkResult result, VkCommandBuffer commandBuffer, const VkCommandBufferBeginInfo* pBeginInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        ApiDumpInstance::current().setCmdBuffer(commandBuffer);
        dump_json_pointer<const VkCommandBufferBeginInfo>(pBeginInfo, settings, "const VkCommandBufferBeginInfo*", "pBeginInfo", true, false, 4, dump_json_VkCommandBufferBeginInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkEndCommandBuffer(ApiDumpInstance& dump_inst, VkResult result, VkCommandBuffer commandBuffer)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkResetCommandBuffer(ApiDumpInstance& dump_inst, VkResult result, VkCommandBuffer commandBuffer, VkCommandBufferResetFlags flags)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkCommandBufferResetFlags>(flags, NULL, settings, "VkCommandBufferResetFlags", "flags", false, false, 4, dump_json_VkCommandBufferResetFlags);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBindPipeline(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkPipelineBindPoint pipelineBindPoint, VkPipeline pipeline)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineBindPoint>(pipelineBindPoint, NULL, settings, "VkPipelineBindPoint", "pipelineBindPoint", false, false, 4, dump_json_VkPipelineBindPoint);
        settings.stream() << ",\n";
        dump_json_value<const VkPipeline>(pipeline, NULL, settings, "VkPipeline", "pipeline", false, false, 4, dump_json_VkPipeline);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetViewport(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstViewport, uint32_t viewportCount, const VkViewport* pViewports)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstViewport, NULL, settings, "uint32_t", "firstViewport", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(viewportCount, NULL, settings, "uint32_t", "viewportCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkViewport>(pViewports, viewportCount, settings, "const VkViewport*", "const VkViewport", "pViewports", true, false, 4, dump_json_VkViewport); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetScissor(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstScissor, uint32_t scissorCount, const VkRect2D* pScissors)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstScissor, NULL, settings, "uint32_t", "firstScissor", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(scissorCount, NULL, settings, "uint32_t", "scissorCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkRect2D>(pScissors, scissorCount, settings, "const VkRect2D*", "const VkRect2D", "pScissors", true, false, 4, dump_json_VkRect2D); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetLineWidth(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, float lineWidth)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const float>(lineWidth, NULL, settings, "float", "lineWidth", false, false, 4, dump_json_float);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthBias(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, float depthBiasConstantFactor, float depthBiasClamp, float depthBiasSlopeFactor)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const float>(depthBiasConstantFactor, NULL, settings, "float", "depthBiasConstantFactor", false, false, 4, dump_json_float);
        settings.stream() << ",\n";
        dump_json_value<const float>(depthBiasClamp, NULL, settings, "float", "depthBiasClamp", false, false, 4, dump_json_float);
        settings.stream() << ",\n";
        dump_json_value<const float>(depthBiasSlopeFactor, NULL, settings, "float", "depthBiasSlopeFactor", false, false, 4, dump_json_float);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetBlendConstants(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const float blendConstants[4])
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_array<const float>(blendConstants, 4, settings, "const float[4]", "const float", "blendConstants", false, false, 4, dump_json_float); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthBounds(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, float minDepthBounds, float maxDepthBounds)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const float>(minDepthBounds, NULL, settings, "float", "minDepthBounds", false, false, 4, dump_json_float);
        settings.stream() << ",\n";
        dump_json_value<const float>(maxDepthBounds, NULL, settings, "float", "maxDepthBounds", false, false, 4, dump_json_float);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetStencilCompareMask(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkStencilFaceFlags faceMask, uint32_t compareMask)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkStencilFaceFlags>(faceMask, NULL, settings, "VkStencilFaceFlags", "faceMask", false, false, 4, dump_json_VkStencilFaceFlags);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(compareMask, NULL, settings, "uint32_t", "compareMask", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetStencilWriteMask(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkStencilFaceFlags faceMask, uint32_t writeMask)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkStencilFaceFlags>(faceMask, NULL, settings, "VkStencilFaceFlags", "faceMask", false, false, 4, dump_json_VkStencilFaceFlags);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(writeMask, NULL, settings, "uint32_t", "writeMask", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetStencilReference(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkStencilFaceFlags faceMask, uint32_t reference)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkStencilFaceFlags>(faceMask, NULL, settings, "VkStencilFaceFlags", "faceMask", false, false, 4, dump_json_VkStencilFaceFlags);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(reference, NULL, settings, "uint32_t", "reference", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBindDescriptorSets(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkPipelineBindPoint pipelineBindPoint, VkPipelineLayout layout, uint32_t firstSet, uint32_t descriptorSetCount, const VkDescriptorSet* pDescriptorSets, uint32_t dynamicOffsetCount, const uint32_t* pDynamicOffsets)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineBindPoint>(pipelineBindPoint, NULL, settings, "VkPipelineBindPoint", "pipelineBindPoint", false, false, 4, dump_json_VkPipelineBindPoint);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineLayout>(layout, NULL, settings, "VkPipelineLayout", "layout", false, false, 4, dump_json_VkPipelineLayout);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstSet, NULL, settings, "uint32_t", "firstSet", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(descriptorSetCount, NULL, settings, "uint32_t", "descriptorSetCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkDescriptorSet>(pDescriptorSets, descriptorSetCount, settings, "const VkDescriptorSet*", "const VkDescriptorSet", "pDescriptorSets", false, false, 4, dump_json_VkDescriptorSet); // PQA
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(dynamicOffsetCount, NULL, settings, "uint32_t", "dynamicOffsetCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const uint32_t>(pDynamicOffsets, dynamicOffsetCount, settings, "const uint32_t*", "const uint32_t", "pDynamicOffsets", false, false, 4, dump_json_uint32_t); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBindIndexBuffer(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset, VkIndexType indexType)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkIndexType>(indexType, NULL, settings, "VkIndexType", "indexType", false, false, 4, dump_json_VkIndexType);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBindVertexBuffers(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstBinding, uint32_t bindingCount, const VkBuffer* pBuffers, const VkDeviceSize* pOffsets)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstBinding, NULL, settings, "uint32_t", "firstBinding", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(bindingCount, NULL, settings, "uint32_t", "bindingCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBuffer>(pBuffers, bindingCount, settings, "const VkBuffer*", "const VkBuffer", "pBuffers", false, false, 4, dump_json_VkBuffer); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkDeviceSize>(pOffsets, bindingCount, settings, "const VkDeviceSize*", "const VkDeviceSize", "pOffsets", false, false, 4, dump_json_VkDeviceSize); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDraw(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t vertexCount, uint32_t instanceCount, uint32_t firstVertex, uint32_t firstInstance)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(vertexCount, NULL, settings, "uint32_t", "vertexCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(instanceCount, NULL, settings, "uint32_t", "instanceCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstVertex, NULL, settings, "uint32_t", "firstVertex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstInstance, NULL, settings, "uint32_t", "firstInstance", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawIndexed(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t indexCount, uint32_t instanceCount, uint32_t firstIndex, int32_t vertexOffset, uint32_t firstInstance)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(indexCount, NULL, settings, "uint32_t", "indexCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(instanceCount, NULL, settings, "uint32_t", "instanceCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstIndex, NULL, settings, "uint32_t", "firstIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const int32_t>(vertexOffset, NULL, settings, "int32_t", "vertexOffset", false, false, 4, dump_json_int32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstInstance, NULL, settings, "uint32_t", "firstInstance", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawIndirect(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset, uint32_t drawCount, uint32_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(drawCount, NULL, settings, "uint32_t", "drawCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawIndexedIndirect(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset, uint32_t drawCount, uint32_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(drawCount, NULL, settings, "uint32_t", "drawCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDispatch(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t groupCountX, uint32_t groupCountY, uint32_t groupCountZ)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCountX, NULL, settings, "uint32_t", "groupCountX", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCountY, NULL, settings, "uint32_t", "groupCountY", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCountZ, NULL, settings, "uint32_t", "groupCountZ", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDispatchIndirect(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyBuffer(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer srcBuffer, VkBuffer dstBuffer, uint32_t regionCount, const VkBufferCopy* pRegions)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(srcBuffer, NULL, settings, "VkBuffer", "srcBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(dstBuffer, NULL, settings, "VkBuffer", "dstBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(regionCount, NULL, settings, "uint32_t", "regionCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBufferCopy>(pRegions, regionCount, settings, "const VkBufferCopy*", "const VkBufferCopy", "pRegions", true, false, 4, dump_json_VkBufferCopy); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyImage(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkImage srcImage, VkImageLayout srcImageLayout, VkImage dstImage, VkImageLayout dstImageLayout, uint32_t regionCount, const VkImageCopy* pRegions)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(srcImage, NULL, settings, "VkImage", "srcImage", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_value<const VkImageLayout>(srcImageLayout, NULL, settings, "VkImageLayout", "srcImageLayout", false, false, 4, dump_json_VkImageLayout);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(dstImage, NULL, settings, "VkImage", "dstImage", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_value<const VkImageLayout>(dstImageLayout, NULL, settings, "VkImageLayout", "dstImageLayout", false, false, 4, dump_json_VkImageLayout);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(regionCount, NULL, settings, "uint32_t", "regionCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkImageCopy>(pRegions, regionCount, settings, "const VkImageCopy*", "const VkImageCopy", "pRegions", true, false, 4, dump_json_VkImageCopy); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBlitImage(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkImage srcImage, VkImageLayout srcImageLayout, VkImage dstImage, VkImageLayout dstImageLayout, uint32_t regionCount, const VkImageBlit* pRegions, VkFilter filter)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(srcImage, NULL, settings, "VkImage", "srcImage", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_value<const VkImageLayout>(srcImageLayout, NULL, settings, "VkImageLayout", "srcImageLayout", false, false, 4, dump_json_VkImageLayout);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(dstImage, NULL, settings, "VkImage", "dstImage", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_value<const VkImageLayout>(dstImageLayout, NULL, settings, "VkImageLayout", "dstImageLayout", false, false, 4, dump_json_VkImageLayout);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(regionCount, NULL, settings, "uint32_t", "regionCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkImageBlit>(pRegions, regionCount, settings, "const VkImageBlit*", "const VkImageBlit", "pRegions", true, false, 4, dump_json_VkImageBlit); // PQA
        settings.stream() << ",\n";
        dump_json_value<const VkFilter>(filter, NULL, settings, "VkFilter", "filter", false, false, 4, dump_json_VkFilter);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyBufferToImage(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer srcBuffer, VkImage dstImage, VkImageLayout dstImageLayout, uint32_t regionCount, const VkBufferImageCopy* pRegions)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(srcBuffer, NULL, settings, "VkBuffer", "srcBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(dstImage, NULL, settings, "VkImage", "dstImage", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_value<const VkImageLayout>(dstImageLayout, NULL, settings, "VkImageLayout", "dstImageLayout", false, false, 4, dump_json_VkImageLayout);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(regionCount, NULL, settings, "uint32_t", "regionCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBufferImageCopy>(pRegions, regionCount, settings, "const VkBufferImageCopy*", "const VkBufferImageCopy", "pRegions", true, false, 4, dump_json_VkBufferImageCopy); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyImageToBuffer(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkImage srcImage, VkImageLayout srcImageLayout, VkBuffer dstBuffer, uint32_t regionCount, const VkBufferImageCopy* pRegions)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(srcImage, NULL, settings, "VkImage", "srcImage", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_value<const VkImageLayout>(srcImageLayout, NULL, settings, "VkImageLayout", "srcImageLayout", false, false, 4, dump_json_VkImageLayout);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(dstBuffer, NULL, settings, "VkBuffer", "dstBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(regionCount, NULL, settings, "uint32_t", "regionCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBufferImageCopy>(pRegions, regionCount, settings, "const VkBufferImageCopy*", "const VkBufferImageCopy", "pRegions", true, false, 4, dump_json_VkBufferImageCopy); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdUpdateBuffer(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer dstBuffer, VkDeviceSize dstOffset, VkDeviceSize dataSize, const void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(dstBuffer, NULL, settings, "VkBuffer", "dstBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(dstOffset, NULL, settings, "VkDeviceSize", "dstOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(dataSize, NULL, settings, "VkDeviceSize", "dataSize", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "const void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdFillBuffer(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer dstBuffer, VkDeviceSize dstOffset, VkDeviceSize size, uint32_t data)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(dstBuffer, NULL, settings, "VkBuffer", "dstBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(dstOffset, NULL, settings, "VkDeviceSize", "dstOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(size, NULL, settings, "VkDeviceSize", "size", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(data, NULL, settings, "uint32_t", "data", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdClearColorImage(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkImage image, VkImageLayout imageLayout, const VkClearColorValue* pColor, uint32_t rangeCount, const VkImageSubresourceRange* pRanges)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(image, NULL, settings, "VkImage", "image", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_value<const VkImageLayout>(imageLayout, NULL, settings, "VkImageLayout", "imageLayout", false, false, 4, dump_json_VkImageLayout);
        settings.stream() << ",\n";
        dump_json_pointer<const VkClearColorValue>(pColor, settings, "const VkClearColorValue*", "pColor", false, true, 4, dump_json_VkClearColorValue);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(rangeCount, NULL, settings, "uint32_t", "rangeCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkImageSubresourceRange>(pRanges, rangeCount, settings, "const VkImageSubresourceRange*", "const VkImageSubresourceRange", "pRanges", true, false, 4, dump_json_VkImageSubresourceRange); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdClearDepthStencilImage(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkImage image, VkImageLayout imageLayout, const VkClearDepthStencilValue* pDepthStencil, uint32_t rangeCount, const VkImageSubresourceRange* pRanges)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(image, NULL, settings, "VkImage", "image", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_value<const VkImageLayout>(imageLayout, NULL, settings, "VkImageLayout", "imageLayout", false, false, 4, dump_json_VkImageLayout);
        settings.stream() << ",\n";
        dump_json_pointer<const VkClearDepthStencilValue>(pDepthStencil, settings, "const VkClearDepthStencilValue*", "pDepthStencil", true, false, 4, dump_json_VkClearDepthStencilValue);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(rangeCount, NULL, settings, "uint32_t", "rangeCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkImageSubresourceRange>(pRanges, rangeCount, settings, "const VkImageSubresourceRange*", "const VkImageSubresourceRange", "pRanges", true, false, 4, dump_json_VkImageSubresourceRange); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdClearAttachments(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t attachmentCount, const VkClearAttachment* pAttachments, uint32_t rectCount, const VkClearRect* pRects)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(attachmentCount, NULL, settings, "uint32_t", "attachmentCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkClearAttachment>(pAttachments, attachmentCount, settings, "const VkClearAttachment*", "const VkClearAttachment", "pAttachments", true, false, 4, dump_json_VkClearAttachment); // PQA
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(rectCount, NULL, settings, "uint32_t", "rectCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkClearRect>(pRects, rectCount, settings, "const VkClearRect*", "const VkClearRect", "pRects", true, false, 4, dump_json_VkClearRect); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdResolveImage(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkImage srcImage, VkImageLayout srcImageLayout, VkImage dstImage, VkImageLayout dstImageLayout, uint32_t regionCount, const VkImageResolve* pRegions)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(srcImage, NULL, settings, "VkImage", "srcImage", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_value<const VkImageLayout>(srcImageLayout, NULL, settings, "VkImageLayout", "srcImageLayout", false, false, 4, dump_json_VkImageLayout);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(dstImage, NULL, settings, "VkImage", "dstImage", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_value<const VkImageLayout>(dstImageLayout, NULL, settings, "VkImageLayout", "dstImageLayout", false, false, 4, dump_json_VkImageLayout);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(regionCount, NULL, settings, "uint32_t", "regionCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkImageResolve>(pRegions, regionCount, settings, "const VkImageResolve*", "const VkImageResolve", "pRegions", true, false, 4, dump_json_VkImageResolve); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetEvent(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkEvent event, VkPipelineStageFlags stageMask)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkEvent>(event, NULL, settings, "VkEvent", "event", false, false, 4, dump_json_VkEvent);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineStageFlags>(stageMask, NULL, settings, "VkPipelineStageFlags", "stageMask", false, false, 4, dump_json_VkPipelineStageFlags);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdResetEvent(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkEvent event, VkPipelineStageFlags stageMask)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkEvent>(event, NULL, settings, "VkEvent", "event", false, false, 4, dump_json_VkEvent);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineStageFlags>(stageMask, NULL, settings, "VkPipelineStageFlags", "stageMask", false, false, 4, dump_json_VkPipelineStageFlags);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdWaitEvents(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t eventCount, const VkEvent* pEvents, VkPipelineStageFlags srcStageMask, VkPipelineStageFlags dstStageMask, uint32_t memoryBarrierCount, const VkMemoryBarrier* pMemoryBarriers, uint32_t bufferMemoryBarrierCount, const VkBufferMemoryBarrier* pBufferMemoryBarriers, uint32_t imageMemoryBarrierCount, const VkImageMemoryBarrier* pImageMemoryBarriers)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(eventCount, NULL, settings, "uint32_t", "eventCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkEvent>(pEvents, eventCount, settings, "const VkEvent*", "const VkEvent", "pEvents", false, false, 4, dump_json_VkEvent); // PQA
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineStageFlags>(srcStageMask, NULL, settings, "VkPipelineStageFlags", "srcStageMask", false, false, 4, dump_json_VkPipelineStageFlags);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineStageFlags>(dstStageMask, NULL, settings, "VkPipelineStageFlags", "dstStageMask", false, false, 4, dump_json_VkPipelineStageFlags);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(memoryBarrierCount, NULL, settings, "uint32_t", "memoryBarrierCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkMemoryBarrier>(pMemoryBarriers, memoryBarrierCount, settings, "const VkMemoryBarrier*", "const VkMemoryBarrier", "pMemoryBarriers", true, false, 4, dump_json_VkMemoryBarrier); // PQA
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(bufferMemoryBarrierCount, NULL, settings, "uint32_t", "bufferMemoryBarrierCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBufferMemoryBarrier>(pBufferMemoryBarriers, bufferMemoryBarrierCount, settings, "const VkBufferMemoryBarrier*", "const VkBufferMemoryBarrier", "pBufferMemoryBarriers", true, false, 4, dump_json_VkBufferMemoryBarrier); // PQA
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(imageMemoryBarrierCount, NULL, settings, "uint32_t", "imageMemoryBarrierCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkImageMemoryBarrier>(pImageMemoryBarriers, imageMemoryBarrierCount, settings, "const VkImageMemoryBarrier*", "const VkImageMemoryBarrier", "pImageMemoryBarriers", true, false, 4, dump_json_VkImageMemoryBarrier); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdPipelineBarrier(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkPipelineStageFlags srcStageMask, VkPipelineStageFlags dstStageMask, VkDependencyFlags dependencyFlags, uint32_t memoryBarrierCount, const VkMemoryBarrier* pMemoryBarriers, uint32_t bufferMemoryBarrierCount, const VkBufferMemoryBarrier* pBufferMemoryBarriers, uint32_t imageMemoryBarrierCount, const VkImageMemoryBarrier* pImageMemoryBarriers)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineStageFlags>(srcStageMask, NULL, settings, "VkPipelineStageFlags", "srcStageMask", false, false, 4, dump_json_VkPipelineStageFlags);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineStageFlags>(dstStageMask, NULL, settings, "VkPipelineStageFlags", "dstStageMask", false, false, 4, dump_json_VkPipelineStageFlags);
        settings.stream() << ",\n";
        dump_json_value<const VkDependencyFlags>(dependencyFlags, NULL, settings, "VkDependencyFlags", "dependencyFlags", false, false, 4, dump_json_VkDependencyFlags);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(memoryBarrierCount, NULL, settings, "uint32_t", "memoryBarrierCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkMemoryBarrier>(pMemoryBarriers, memoryBarrierCount, settings, "const VkMemoryBarrier*", "const VkMemoryBarrier", "pMemoryBarriers", true, false, 4, dump_json_VkMemoryBarrier); // PQA
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(bufferMemoryBarrierCount, NULL, settings, "uint32_t", "bufferMemoryBarrierCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBufferMemoryBarrier>(pBufferMemoryBarriers, bufferMemoryBarrierCount, settings, "const VkBufferMemoryBarrier*", "const VkBufferMemoryBarrier", "pBufferMemoryBarriers", true, false, 4, dump_json_VkBufferMemoryBarrier); // PQA
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(imageMemoryBarrierCount, NULL, settings, "uint32_t", "imageMemoryBarrierCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkImageMemoryBarrier>(pImageMemoryBarriers, imageMemoryBarrierCount, settings, "const VkImageMemoryBarrier*", "const VkImageMemoryBarrier", "pImageMemoryBarriers", true, false, 4, dump_json_VkImageMemoryBarrier); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBeginQuery(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkQueryPool queryPool, uint32_t query, VkQueryControlFlags flags)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(query, NULL, settings, "uint32_t", "query", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryControlFlags>(flags, NULL, settings, "VkQueryControlFlags", "flags", false, false, 4, dump_json_VkQueryControlFlags);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdEndQuery(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkQueryPool queryPool, uint32_t query)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(query, NULL, settings, "uint32_t", "query", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdResetQueryPool(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkQueryPool queryPool, uint32_t firstQuery, uint32_t queryCount)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstQuery, NULL, settings, "uint32_t", "firstQuery", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(queryCount, NULL, settings, "uint32_t", "queryCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdWriteTimestamp(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkPipelineStageFlagBits pipelineStage, VkQueryPool queryPool, uint32_t query)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineStageFlagBits>(pipelineStage, NULL, settings, "VkPipelineStageFlagBits", "pipelineStage", false, false, 4, dump_json_VkPipelineStageFlagBits);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(query, NULL, settings, "uint32_t", "query", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyQueryPoolResults(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkQueryPool queryPool, uint32_t firstQuery, uint32_t queryCount, VkBuffer dstBuffer, VkDeviceSize dstOffset, VkDeviceSize stride, VkQueryResultFlags flags)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstQuery, NULL, settings, "uint32_t", "firstQuery", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(queryCount, NULL, settings, "uint32_t", "queryCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(dstBuffer, NULL, settings, "VkBuffer", "dstBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(dstOffset, NULL, settings, "VkDeviceSize", "dstOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(stride, NULL, settings, "VkDeviceSize", "stride", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryResultFlags>(flags, NULL, settings, "VkQueryResultFlags", "flags", false, false, 4, dump_json_VkQueryResultFlags);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdPushConstants(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkPipelineLayout layout, VkShaderStageFlags stageFlags, uint32_t offset, uint32_t size, const void* pValues)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineLayout>(layout, NULL, settings, "VkPipelineLayout", "layout", false, false, 4, dump_json_VkPipelineLayout);
        settings.stream() << ",\n";
        dump_json_value<const VkShaderStageFlags>(stageFlags, NULL, settings, "VkShaderStageFlags", "stageFlags", false, false, 4, dump_json_VkShaderStageFlags);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(offset, NULL, settings, "uint32_t", "offset", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(size, NULL, settings, "uint32_t", "size", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pValues, NULL, settings, "const void*", "pValues", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBeginRenderPass(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkRenderPassBeginInfo* pRenderPassBegin, VkSubpassContents contents)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRenderPassBeginInfo>(pRenderPassBegin, settings, "const VkRenderPassBeginInfo*", "pRenderPassBegin", true, false, 4, dump_json_VkRenderPassBeginInfo);
        settings.stream() << ",\n";
        dump_json_value<const VkSubpassContents>(contents, NULL, settings, "VkSubpassContents", "contents", false, false, 4, dump_json_VkSubpassContents);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdNextSubpass(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkSubpassContents contents)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkSubpassContents>(contents, NULL, settings, "VkSubpassContents", "contents", false, false, 4, dump_json_VkSubpassContents);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdEndRenderPass(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdExecuteCommands(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t commandBufferCount, const VkCommandBuffer* pCommandBuffers)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(commandBufferCount, NULL, settings, "uint32_t", "commandBufferCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkCommandBuffer>(pCommandBuffers, commandBufferCount, settings, "const VkCommandBuffer*", "const VkCommandBuffer", "pCommandBuffers", false, false, 4, dump_json_VkCommandBuffer); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkBindBufferMemory2(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t bindInfoCount, const VkBindBufferMemoryInfo* pBindInfos)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(bindInfoCount, NULL, settings, "uint32_t", "bindInfoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBindBufferMemoryInfo>(pBindInfos, bindInfoCount, settings, "const VkBindBufferMemoryInfo*", "const VkBindBufferMemoryInfo", "pBindInfos", true, false, 4, dump_json_VkBindBufferMemoryInfo); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkBindImageMemory2(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t bindInfoCount, const VkBindImageMemoryInfo* pBindInfos)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(bindInfoCount, NULL, settings, "uint32_t", "bindInfoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBindImageMemoryInfo>(pBindInfos, bindInfoCount, settings, "const VkBindImageMemoryInfo*", "const VkBindImageMemoryInfo", "pBindInfos", true, false, 4, dump_json_VkBindImageMemoryInfo); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceGroupPeerMemoryFeatures(ApiDumpInstance& dump_inst, VkDevice device, uint32_t heapIndex, uint32_t localDeviceIndex, uint32_t remoteDeviceIndex, VkPeerMemoryFeatureFlags* pPeerMemoryFeatures)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(heapIndex, NULL, settings, "uint32_t", "heapIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(localDeviceIndex, NULL, settings, "uint32_t", "localDeviceIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(remoteDeviceIndex, NULL, settings, "uint32_t", "remoteDeviceIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPeerMemoryFeatureFlags>(pPeerMemoryFeatures, settings, "VkPeerMemoryFeatureFlags*", "pPeerMemoryFeatures", false, false, 4, dump_json_VkPeerMemoryFeatureFlags);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDeviceMask(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t deviceMask)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(deviceMask, NULL, settings, "uint32_t", "deviceMask", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDispatchBase(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t baseGroupX, uint32_t baseGroupY, uint32_t baseGroupZ, uint32_t groupCountX, uint32_t groupCountY, uint32_t groupCountZ)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(baseGroupX, NULL, settings, "uint32_t", "baseGroupX", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(baseGroupY, NULL, settings, "uint32_t", "baseGroupY", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(baseGroupZ, NULL, settings, "uint32_t", "baseGroupZ", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCountX, NULL, settings, "uint32_t", "groupCountX", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCountY, NULL, settings, "uint32_t", "groupCountY", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCountZ, NULL, settings, "uint32_t", "groupCountZ", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkEnumeratePhysicalDeviceGroups(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, uint32_t* pPhysicalDeviceGroupCount, VkPhysicalDeviceGroupProperties* pPhysicalDeviceGroupProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPhysicalDeviceGroupCount, settings, "uint32_t*", "pPhysicalDeviceGroupCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkPhysicalDeviceGroupProperties>(pPhysicalDeviceGroupProperties, *pPhysicalDeviceGroupCount, settings, "VkPhysicalDeviceGroupProperties*", "VkPhysicalDeviceGroupProperties", "pPhysicalDeviceGroupProperties", true, false, 4, dump_json_VkPhysicalDeviceGroupProperties); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetImageMemoryRequirements2(ApiDumpInstance& dump_inst, VkDevice device, const VkImageMemoryRequirementsInfo2* pInfo, VkMemoryRequirements2* pMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageMemoryRequirementsInfo2>(pInfo, settings, "const VkImageMemoryRequirementsInfo2*", "pInfo", true, false, 4, dump_json_VkImageMemoryRequirementsInfo2);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryRequirements2>(pMemoryRequirements, settings, "VkMemoryRequirements2*", "pMemoryRequirements", true, false, 4, dump_json_VkMemoryRequirements2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetBufferMemoryRequirements2(ApiDumpInstance& dump_inst, VkDevice device, const VkBufferMemoryRequirementsInfo2* pInfo, VkMemoryRequirements2* pMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBufferMemoryRequirementsInfo2>(pInfo, settings, "const VkBufferMemoryRequirementsInfo2*", "pInfo", true, false, 4, dump_json_VkBufferMemoryRequirementsInfo2);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryRequirements2>(pMemoryRequirements, settings, "VkMemoryRequirements2*", "pMemoryRequirements", true, false, 4, dump_json_VkMemoryRequirements2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetImageSparseMemoryRequirements2(ApiDumpInstance& dump_inst, VkDevice device, const VkImageSparseMemoryRequirementsInfo2* pInfo, uint32_t* pSparseMemoryRequirementCount, VkSparseImageMemoryRequirements2* pSparseMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageSparseMemoryRequirementsInfo2>(pInfo, settings, "const VkImageSparseMemoryRequirementsInfo2*", "pInfo", true, false, 4, dump_json_VkImageSparseMemoryRequirementsInfo2);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pSparseMemoryRequirementCount, settings, "uint32_t*", "pSparseMemoryRequirementCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkSparseImageMemoryRequirements2>(pSparseMemoryRequirements, *pSparseMemoryRequirementCount, settings, "VkSparseImageMemoryRequirements2*", "VkSparseImageMemoryRequirements2", "pSparseMemoryRequirements", true, false, 4, dump_json_VkSparseImageMemoryRequirements2); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceFeatures2(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, VkPhysicalDeviceFeatures2* pFeatures)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceFeatures2>(pFeatures, settings, "VkPhysicalDeviceFeatures2*", "pFeatures", true, false, 4, dump_json_VkPhysicalDeviceFeatures2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceProperties2(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, VkPhysicalDeviceProperties2* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceProperties2>(pProperties, settings, "VkPhysicalDeviceProperties2*", "pProperties", true, false, 4, dump_json_VkPhysicalDeviceProperties2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceFormatProperties2(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, VkFormat format, VkFormatProperties2* pFormatProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkFormat>(format, NULL, settings, "VkFormat", "format", false, false, 4, dump_json_VkFormat);
        settings.stream() << ",\n";
        dump_json_pointer<const VkFormatProperties2>(pFormatProperties, settings, "VkFormatProperties2*", "pFormatProperties", true, false, 4, dump_json_VkFormatProperties2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceImageFormatProperties2(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, const VkPhysicalDeviceImageFormatInfo2* pImageFormatInfo, VkImageFormatProperties2* pImageFormatProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceImageFormatInfo2>(pImageFormatInfo, settings, "const VkPhysicalDeviceImageFormatInfo2*", "pImageFormatInfo", true, false, 4, dump_json_VkPhysicalDeviceImageFormatInfo2);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageFormatProperties2>(pImageFormatProperties, settings, "VkImageFormatProperties2*", "pImageFormatProperties", true, false, 4, dump_json_VkImageFormatProperties2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceQueueFamilyProperties2(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, uint32_t* pQueueFamilyPropertyCount, VkQueueFamilyProperties2* pQueueFamilyProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pQueueFamilyPropertyCount, settings, "uint32_t*", "pQueueFamilyPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkQueueFamilyProperties2>(pQueueFamilyProperties, *pQueueFamilyPropertyCount, settings, "VkQueueFamilyProperties2*", "VkQueueFamilyProperties2", "pQueueFamilyProperties", true, false, 4, dump_json_VkQueueFamilyProperties2); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceMemoryProperties2(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, VkPhysicalDeviceMemoryProperties2* pMemoryProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceMemoryProperties2>(pMemoryProperties, settings, "VkPhysicalDeviceMemoryProperties2*", "pMemoryProperties", true, false, 4, dump_json_VkPhysicalDeviceMemoryProperties2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceSparseImageFormatProperties2(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, const VkPhysicalDeviceSparseImageFormatInfo2* pFormatInfo, uint32_t* pPropertyCount, VkSparseImageFormatProperties2* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceSparseImageFormatInfo2>(pFormatInfo, settings, "const VkPhysicalDeviceSparseImageFormatInfo2*", "pFormatInfo", true, false, 4, dump_json_VkPhysicalDeviceSparseImageFormatInfo2);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPropertyCount, settings, "uint32_t*", "pPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkSparseImageFormatProperties2>(pProperties, *pPropertyCount, settings, "VkSparseImageFormatProperties2*", "VkSparseImageFormatProperties2", "pProperties", true, false, 4, dump_json_VkSparseImageFormatProperties2); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkTrimCommandPool(ApiDumpInstance& dump_inst, VkDevice device, VkCommandPool commandPool, VkCommandPoolTrimFlags flags)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkCommandPool>(commandPool, NULL, settings, "VkCommandPool", "commandPool", false, false, 4, dump_json_VkCommandPool);
        settings.stream() << ",\n";
        dump_json_value<const VkCommandPoolTrimFlags>(flags, NULL, settings, "VkCommandPoolTrimFlags", "flags", false, false, 4, dump_json_VkCommandPoolTrimFlags);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceQueue2(ApiDumpInstance& dump_inst, VkDevice device, const VkDeviceQueueInfo2* pQueueInfo, VkQueue* pQueue)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceQueueInfo2>(pQueueInfo, settings, "const VkDeviceQueueInfo2*", "pQueueInfo", true, false, 4, dump_json_VkDeviceQueueInfo2);
        settings.stream() << ",\n";
        dump_json_pointer<const VkQueue>(pQueue, settings, "VkQueue*", "pQueue", false, false, 4, dump_json_VkQueue);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateSamplerYcbcrConversion(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkSamplerYcbcrConversionCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSamplerYcbcrConversion* pYcbcrConversion)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSamplerYcbcrConversionCreateInfo>(pCreateInfo, settings, "const VkSamplerYcbcrConversionCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkSamplerYcbcrConversionCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSamplerYcbcrConversion>(pYcbcrConversion, settings, "VkSamplerYcbcrConversion*", "pYcbcrConversion", false, false, 4, dump_json_VkSamplerYcbcrConversion);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroySamplerYcbcrConversion(ApiDumpInstance& dump_inst, VkDevice device, VkSamplerYcbcrConversion ycbcrConversion, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSamplerYcbcrConversion>(ycbcrConversion, NULL, settings, "VkSamplerYcbcrConversion", "ycbcrConversion", false, false, 4, dump_json_VkSamplerYcbcrConversion);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateDescriptorUpdateTemplate(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkDescriptorUpdateTemplateCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkDescriptorUpdateTemplate* pDescriptorUpdateTemplate)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorUpdateTemplateCreateInfo>(pCreateInfo, settings, "const VkDescriptorUpdateTemplateCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkDescriptorUpdateTemplateCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorUpdateTemplate>(pDescriptorUpdateTemplate, settings, "VkDescriptorUpdateTemplate*", "pDescriptorUpdateTemplate", false, false, 4, dump_json_VkDescriptorUpdateTemplate);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyDescriptorUpdateTemplate(ApiDumpInstance& dump_inst, VkDevice device, VkDescriptorUpdateTemplate descriptorUpdateTemplate, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDescriptorUpdateTemplate>(descriptorUpdateTemplate, NULL, settings, "VkDescriptorUpdateTemplate", "descriptorUpdateTemplate", false, false, 4, dump_json_VkDescriptorUpdateTemplate);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkUpdateDescriptorSetWithTemplate(ApiDumpInstance& dump_inst, VkDevice device, VkDescriptorSet descriptorSet, VkDescriptorUpdateTemplate descriptorUpdateTemplate, const void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDescriptorSet>(descriptorSet, NULL, settings, "VkDescriptorSet", "descriptorSet", false, false, 4, dump_json_VkDescriptorSet);
        settings.stream() << ",\n";
        dump_json_value<const VkDescriptorUpdateTemplate>(descriptorUpdateTemplate, NULL, settings, "VkDescriptorUpdateTemplate", "descriptorUpdateTemplate", false, false, 4, dump_json_VkDescriptorUpdateTemplate);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "const void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceExternalBufferProperties(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, const VkPhysicalDeviceExternalBufferInfo* pExternalBufferInfo, VkExternalBufferProperties* pExternalBufferProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceExternalBufferInfo>(pExternalBufferInfo, settings, "const VkPhysicalDeviceExternalBufferInfo*", "pExternalBufferInfo", true, false, 4, dump_json_VkPhysicalDeviceExternalBufferInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkExternalBufferProperties>(pExternalBufferProperties, settings, "VkExternalBufferProperties*", "pExternalBufferProperties", true, false, 4, dump_json_VkExternalBufferProperties);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceExternalFenceProperties(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, const VkPhysicalDeviceExternalFenceInfo* pExternalFenceInfo, VkExternalFenceProperties* pExternalFenceProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceExternalFenceInfo>(pExternalFenceInfo, settings, "const VkPhysicalDeviceExternalFenceInfo*", "pExternalFenceInfo", true, false, 4, dump_json_VkPhysicalDeviceExternalFenceInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkExternalFenceProperties>(pExternalFenceProperties, settings, "VkExternalFenceProperties*", "pExternalFenceProperties", true, false, 4, dump_json_VkExternalFenceProperties);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceExternalSemaphoreProperties(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, const VkPhysicalDeviceExternalSemaphoreInfo* pExternalSemaphoreInfo, VkExternalSemaphoreProperties* pExternalSemaphoreProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceExternalSemaphoreInfo>(pExternalSemaphoreInfo, settings, "const VkPhysicalDeviceExternalSemaphoreInfo*", "pExternalSemaphoreInfo", true, false, 4, dump_json_VkPhysicalDeviceExternalSemaphoreInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkExternalSemaphoreProperties>(pExternalSemaphoreProperties, settings, "VkExternalSemaphoreProperties*", "pExternalSemaphoreProperties", true, false, 4, dump_json_VkExternalSemaphoreProperties);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDescriptorSetLayoutSupport(ApiDumpInstance& dump_inst, VkDevice device, const VkDescriptorSetLayoutCreateInfo* pCreateInfo, VkDescriptorSetLayoutSupport* pSupport)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorSetLayoutCreateInfo>(pCreateInfo, settings, "const VkDescriptorSetLayoutCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkDescriptorSetLayoutCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorSetLayoutSupport>(pSupport, settings, "VkDescriptorSetLayoutSupport*", "pSupport", true, false, 4, dump_json_VkDescriptorSetLayoutSupport);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawIndirectCount(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset, VkBuffer countBuffer, VkDeviceSize countBufferOffset, uint32_t maxDrawCount, uint32_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(countBuffer, NULL, settings, "VkBuffer", "countBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(countBufferOffset, NULL, settings, "VkDeviceSize", "countBufferOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(maxDrawCount, NULL, settings, "uint32_t", "maxDrawCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawIndexedIndirectCount(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset, VkBuffer countBuffer, VkDeviceSize countBufferOffset, uint32_t maxDrawCount, uint32_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(countBuffer, NULL, settings, "VkBuffer", "countBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(countBufferOffset, NULL, settings, "VkDeviceSize", "countBufferOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(maxDrawCount, NULL, settings, "uint32_t", "maxDrawCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateRenderPass2(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkRenderPassCreateInfo2* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkRenderPass* pRenderPass)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRenderPassCreateInfo2>(pCreateInfo, settings, "const VkRenderPassCreateInfo2*", "pCreateInfo", true, false, 4, dump_json_VkRenderPassCreateInfo2);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRenderPass>(pRenderPass, settings, "VkRenderPass*", "pRenderPass", false, false, 4, dump_json_VkRenderPass);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBeginRenderPass2(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkRenderPassBeginInfo*      pRenderPassBegin, const VkSubpassBeginInfo*      pSubpassBeginInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRenderPassBeginInfo>(pRenderPassBegin, settings, "const VkRenderPassBeginInfo*", "pRenderPassBegin", true, false, 4, dump_json_VkRenderPassBeginInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSubpassBeginInfo>(pSubpassBeginInfo, settings, "const VkSubpassBeginInfo*", "pSubpassBeginInfo", true, false, 4, dump_json_VkSubpassBeginInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdNextSubpass2(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkSubpassBeginInfo*      pSubpassBeginInfo, const VkSubpassEndInfo*        pSubpassEndInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSubpassBeginInfo>(pSubpassBeginInfo, settings, "const VkSubpassBeginInfo*", "pSubpassBeginInfo", true, false, 4, dump_json_VkSubpassBeginInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSubpassEndInfo>(pSubpassEndInfo, settings, "const VkSubpassEndInfo*", "pSubpassEndInfo", true, false, 4, dump_json_VkSubpassEndInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdEndRenderPass2(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkSubpassEndInfo*        pSubpassEndInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSubpassEndInfo>(pSubpassEndInfo, settings, "const VkSubpassEndInfo*", "pSubpassEndInfo", true, false, 4, dump_json_VkSubpassEndInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkResetQueryPool(ApiDumpInstance& dump_inst, VkDevice device, VkQueryPool queryPool, uint32_t firstQuery, uint32_t queryCount)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstQuery, NULL, settings, "uint32_t", "firstQuery", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(queryCount, NULL, settings, "uint32_t", "queryCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetSemaphoreCounterValue(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkSemaphore semaphore, uint64_t* pValue)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSemaphore>(semaphore, NULL, settings, "VkSemaphore", "semaphore", false, false, 4, dump_json_VkSemaphore);
        settings.stream() << ",\n";
        dump_json_pointer<const uint64_t>(pValue, settings, "uint64_t*", "pValue", false, false, 4, dump_json_uint64_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkWaitSemaphores(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkSemaphoreWaitInfo* pWaitInfo, uint64_t timeout)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSemaphoreWaitInfo>(pWaitInfo, settings, "const VkSemaphoreWaitInfo*", "pWaitInfo", true, false, 4, dump_json_VkSemaphoreWaitInfo);
        settings.stream() << ",\n";
        dump_json_value<const uint64_t>(timeout, NULL, settings, "uint64_t", "timeout", false, false, 4, dump_json_uint64_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkSignalSemaphore(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkSemaphoreSignalInfo* pSignalInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSemaphoreSignalInfo>(pSignalInfo, settings, "const VkSemaphoreSignalInfo*", "pSignalInfo", true, false, 4, dump_json_VkSemaphoreSignalInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetBufferDeviceAddress(ApiDumpInstance& dump_inst, VkDeviceAddress result, VkDevice device, const VkBufferDeviceAddressInfo* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkDeviceAddress(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBufferDeviceAddressInfo>(pInfo, settings, "const VkBufferDeviceAddressInfo*", "pInfo", true, false, 4, dump_json_VkBufferDeviceAddressInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetBufferOpaqueCaptureAddress(ApiDumpInstance& dump_inst, uint64_t result, VkDevice device, const VkBufferDeviceAddressInfo* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_uint64_t(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBufferDeviceAddressInfo>(pInfo, settings, "const VkBufferDeviceAddressInfo*", "pInfo", true, false, 4, dump_json_VkBufferDeviceAddressInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceMemoryOpaqueCaptureAddress(ApiDumpInstance& dump_inst, uint64_t result, VkDevice device, const VkDeviceMemoryOpaqueCaptureAddressInfo* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_uint64_t(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceMemoryOpaqueCaptureAddressInfo>(pInfo, settings, "const VkDeviceMemoryOpaqueCaptureAddressInfo*", "pInfo", true, false, 4, dump_json_VkDeviceMemoryOpaqueCaptureAddressInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceToolProperties(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t* pToolCount, VkPhysicalDeviceToolProperties* pToolProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pToolCount, settings, "uint32_t*", "pToolCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkPhysicalDeviceToolProperties>(pToolProperties, *pToolCount, settings, "VkPhysicalDeviceToolProperties*", "VkPhysicalDeviceToolProperties", "pToolProperties", true, false, 4, dump_json_VkPhysicalDeviceToolProperties); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreatePrivateDataSlot(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkPrivateDataSlotCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkPrivateDataSlot* pPrivateDataSlot)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPrivateDataSlotCreateInfo>(pCreateInfo, settings, "const VkPrivateDataSlotCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkPrivateDataSlotCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPrivateDataSlot>(pPrivateDataSlot, settings, "VkPrivateDataSlot*", "pPrivateDataSlot", false, false, 4, dump_json_VkPrivateDataSlot);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyPrivateDataSlot(ApiDumpInstance& dump_inst, VkDevice device, VkPrivateDataSlot privateDataSlot, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPrivateDataSlot>(privateDataSlot, NULL, settings, "VkPrivateDataSlot", "privateDataSlot", false, false, 4, dump_json_VkPrivateDataSlot);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkSetPrivateData(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkObjectType objectType, uint64_t objectHandle, VkPrivateDataSlot privateDataSlot, uint64_t data)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkObjectType>(objectType, NULL, settings, "VkObjectType", "objectType", false, false, 4, dump_json_VkObjectType);
        settings.stream() << ",\n";
        dump_json_value<const uint64_t>(objectHandle, NULL, settings, "uint64_t", "objectHandle", false, false, 4, dump_json_uint64_t);
        settings.stream() << ",\n";
        dump_json_value<const VkPrivateDataSlot>(privateDataSlot, NULL, settings, "VkPrivateDataSlot", "privateDataSlot", false, false, 4, dump_json_VkPrivateDataSlot);
        settings.stream() << ",\n";
        dump_json_value<const uint64_t>(data, NULL, settings, "uint64_t", "data", false, false, 4, dump_json_uint64_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPrivateData(ApiDumpInstance& dump_inst, VkDevice device, VkObjectType objectType, uint64_t objectHandle, VkPrivateDataSlot privateDataSlot, uint64_t* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkObjectType>(objectType, NULL, settings, "VkObjectType", "objectType", false, false, 4, dump_json_VkObjectType);
        settings.stream() << ",\n";
        dump_json_value<const uint64_t>(objectHandle, NULL, settings, "uint64_t", "objectHandle", false, false, 4, dump_json_uint64_t);
        settings.stream() << ",\n";
        dump_json_value<const VkPrivateDataSlot>(privateDataSlot, NULL, settings, "VkPrivateDataSlot", "privateDataSlot", false, false, 4, dump_json_VkPrivateDataSlot);
        settings.stream() << ",\n";
        dump_json_pointer<const uint64_t>(pData, settings, "uint64_t*", "pData", false, false, 4, dump_json_uint64_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetEvent2(ApiDumpInstance& dump_inst, VkCommandBuffer                   commandBuffer, VkEvent                                             event, const VkDependencyInfo*                             pDependencyInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkEvent>(event, NULL, settings, "VkEvent", "event", false, false, 4, dump_json_VkEvent);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDependencyInfo>(pDependencyInfo, settings, "const VkDependencyInfo*", "pDependencyInfo", true, false, 4, dump_json_VkDependencyInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdResetEvent2(ApiDumpInstance& dump_inst, VkCommandBuffer                   commandBuffer, VkEvent                                             event, VkPipelineStageFlags2               stageMask)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkEvent>(event, NULL, settings, "VkEvent", "event", false, false, 4, dump_json_VkEvent);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineStageFlags2>(stageMask, NULL, settings, "VkPipelineStageFlags2", "stageMask", false, false, 4, dump_json_VkPipelineStageFlags2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdWaitEvents2(ApiDumpInstance& dump_inst, VkCommandBuffer                   commandBuffer, uint32_t                                            eventCount, const VkEvent*                     pEvents, const VkDependencyInfo*            pDependencyInfos)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(eventCount, NULL, settings, "uint32_t", "eventCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkEvent>(pEvents, eventCount, settings, "const VkEvent*", "const VkEvent", "pEvents", false, false, 4, dump_json_VkEvent); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkDependencyInfo>(pDependencyInfos, eventCount, settings, "const VkDependencyInfo*", "const VkDependencyInfo", "pDependencyInfos", true, false, 4, dump_json_VkDependencyInfo); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdPipelineBarrier2(ApiDumpInstance& dump_inst, VkCommandBuffer                   commandBuffer, const VkDependencyInfo*                             pDependencyInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDependencyInfo>(pDependencyInfo, settings, "const VkDependencyInfo*", "pDependencyInfo", true, false, 4, dump_json_VkDependencyInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdWriteTimestamp2(ApiDumpInstance& dump_inst, VkCommandBuffer                   commandBuffer, VkPipelineStageFlags2               stage, VkQueryPool                                         queryPool, uint32_t                                            query)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineStageFlags2>(stage, NULL, settings, "VkPipelineStageFlags2", "stage", false, false, 4, dump_json_VkPipelineStageFlags2);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(query, NULL, settings, "uint32_t", "query", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkQueueSubmit2(ApiDumpInstance& dump_inst, VkResult result, VkQueue                           queue, uint32_t                            submitCount, const VkSubmitInfo2*              pSubmits, VkFence           fence)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkQueue>(queue, NULL, settings, "VkQueue", "queue", false, false, 4, dump_json_VkQueue);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(submitCount, NULL, settings, "uint32_t", "submitCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkSubmitInfo2>(pSubmits, submitCount, settings, "const VkSubmitInfo2*", "const VkSubmitInfo2", "pSubmits", true, false, 4, dump_json_VkSubmitInfo2); // PQA
        settings.stream() << ",\n";
        dump_json_value<const VkFence>(fence, NULL, settings, "VkFence", "fence", false, false, 4, dump_json_VkFence);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyBuffer2(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCopyBufferInfo2* pCopyBufferInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyBufferInfo2>(pCopyBufferInfo, settings, "const VkCopyBufferInfo2*", "pCopyBufferInfo", true, false, 4, dump_json_VkCopyBufferInfo2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyImage2(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCopyImageInfo2* pCopyImageInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyImageInfo2>(pCopyImageInfo, settings, "const VkCopyImageInfo2*", "pCopyImageInfo", true, false, 4, dump_json_VkCopyImageInfo2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyBufferToImage2(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCopyBufferToImageInfo2* pCopyBufferToImageInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyBufferToImageInfo2>(pCopyBufferToImageInfo, settings, "const VkCopyBufferToImageInfo2*", "pCopyBufferToImageInfo", true, false, 4, dump_json_VkCopyBufferToImageInfo2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyImageToBuffer2(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCopyImageToBufferInfo2* pCopyImageToBufferInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyImageToBufferInfo2>(pCopyImageToBufferInfo, settings, "const VkCopyImageToBufferInfo2*", "pCopyImageToBufferInfo", true, false, 4, dump_json_VkCopyImageToBufferInfo2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBlitImage2(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkBlitImageInfo2* pBlitImageInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBlitImageInfo2>(pBlitImageInfo, settings, "const VkBlitImageInfo2*", "pBlitImageInfo", true, false, 4, dump_json_VkBlitImageInfo2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdResolveImage2(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkResolveImageInfo2* pResolveImageInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkResolveImageInfo2>(pResolveImageInfo, settings, "const VkResolveImageInfo2*", "pResolveImageInfo", true, false, 4, dump_json_VkResolveImageInfo2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBeginRendering(ApiDumpInstance& dump_inst, VkCommandBuffer                   commandBuffer, const VkRenderingInfo*                              pRenderingInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRenderingInfo>(pRenderingInfo, settings, "const VkRenderingInfo*", "pRenderingInfo", true, false, 4, dump_json_VkRenderingInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdEndRendering(ApiDumpInstance& dump_inst, VkCommandBuffer                   commandBuffer)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetCullMode(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkCullModeFlags cullMode)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkCullModeFlags>(cullMode, NULL, settings, "VkCullModeFlags", "cullMode", false, false, 4, dump_json_VkCullModeFlags);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetFrontFace(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkFrontFace frontFace)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkFrontFace>(frontFace, NULL, settings, "VkFrontFace", "frontFace", false, false, 4, dump_json_VkFrontFace);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetPrimitiveTopology(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkPrimitiveTopology primitiveTopology)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPrimitiveTopology>(primitiveTopology, NULL, settings, "VkPrimitiveTopology", "primitiveTopology", false, false, 4, dump_json_VkPrimitiveTopology);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetViewportWithCount(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t viewportCount, const VkViewport* pViewports)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(viewportCount, NULL, settings, "uint32_t", "viewportCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkViewport>(pViewports, viewportCount, settings, "const VkViewport*", "const VkViewport", "pViewports", true, false, 4, dump_json_VkViewport); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetScissorWithCount(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t scissorCount, const VkRect2D* pScissors)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(scissorCount, NULL, settings, "uint32_t", "scissorCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkRect2D>(pScissors, scissorCount, settings, "const VkRect2D*", "const VkRect2D", "pScissors", true, false, 4, dump_json_VkRect2D); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBindVertexBuffers2(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstBinding, uint32_t bindingCount, const VkBuffer* pBuffers, const VkDeviceSize* pOffsets, const VkDeviceSize* pSizes, const VkDeviceSize* pStrides)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstBinding, NULL, settings, "uint32_t", "firstBinding", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(bindingCount, NULL, settings, "uint32_t", "bindingCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBuffer>(pBuffers, bindingCount, settings, "const VkBuffer*", "const VkBuffer", "pBuffers", false, false, 4, dump_json_VkBuffer); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkDeviceSize>(pOffsets, bindingCount, settings, "const VkDeviceSize*", "const VkDeviceSize", "pOffsets", false, false, 4, dump_json_VkDeviceSize); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkDeviceSize>(pSizes, bindingCount, settings, "const VkDeviceSize*", "const VkDeviceSize", "pSizes", false, false, 4, dump_json_VkDeviceSize); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkDeviceSize>(pStrides, bindingCount, settings, "const VkDeviceSize*", "const VkDeviceSize", "pStrides", false, false, 4, dump_json_VkDeviceSize); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthTestEnable(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 depthTestEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(depthTestEnable, NULL, settings, "VkBool32", "depthTestEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthWriteEnable(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 depthWriteEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(depthWriteEnable, NULL, settings, "VkBool32", "depthWriteEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthCompareOp(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkCompareOp depthCompareOp)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkCompareOp>(depthCompareOp, NULL, settings, "VkCompareOp", "depthCompareOp", false, false, 4, dump_json_VkCompareOp);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthBoundsTestEnable(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 depthBoundsTestEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(depthBoundsTestEnable, NULL, settings, "VkBool32", "depthBoundsTestEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetStencilTestEnable(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 stencilTestEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(stencilTestEnable, NULL, settings, "VkBool32", "stencilTestEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetStencilOp(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkStencilFaceFlags faceMask, VkStencilOp failOp, VkStencilOp passOp, VkStencilOp depthFailOp, VkCompareOp compareOp)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkStencilFaceFlags>(faceMask, NULL, settings, "VkStencilFaceFlags", "faceMask", false, false, 4, dump_json_VkStencilFaceFlags);
        settings.stream() << ",\n";
        dump_json_value<const VkStencilOp>(failOp, NULL, settings, "VkStencilOp", "failOp", false, false, 4, dump_json_VkStencilOp);
        settings.stream() << ",\n";
        dump_json_value<const VkStencilOp>(passOp, NULL, settings, "VkStencilOp", "passOp", false, false, 4, dump_json_VkStencilOp);
        settings.stream() << ",\n";
        dump_json_value<const VkStencilOp>(depthFailOp, NULL, settings, "VkStencilOp", "depthFailOp", false, false, 4, dump_json_VkStencilOp);
        settings.stream() << ",\n";
        dump_json_value<const VkCompareOp>(compareOp, NULL, settings, "VkCompareOp", "compareOp", false, false, 4, dump_json_VkCompareOp);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetRasterizerDiscardEnable(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 rasterizerDiscardEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(rasterizerDiscardEnable, NULL, settings, "VkBool32", "rasterizerDiscardEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthBiasEnable(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 depthBiasEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(depthBiasEnable, NULL, settings, "VkBool32", "depthBiasEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetPrimitiveRestartEnable(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 primitiveRestartEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(primitiveRestartEnable, NULL, settings, "VkBool32", "primitiveRestartEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceBufferMemoryRequirements(ApiDumpInstance& dump_inst, VkDevice device, const VkDeviceBufferMemoryRequirements* pInfo, VkMemoryRequirements2* pMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceBufferMemoryRequirements>(pInfo, settings, "const VkDeviceBufferMemoryRequirements*", "pInfo", true, false, 4, dump_json_VkDeviceBufferMemoryRequirements);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryRequirements2>(pMemoryRequirements, settings, "VkMemoryRequirements2*", "pMemoryRequirements", true, false, 4, dump_json_VkMemoryRequirements2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceImageMemoryRequirements(ApiDumpInstance& dump_inst, VkDevice device, const VkDeviceImageMemoryRequirements* pInfo, VkMemoryRequirements2* pMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceImageMemoryRequirements>(pInfo, settings, "const VkDeviceImageMemoryRequirements*", "pInfo", true, false, 4, dump_json_VkDeviceImageMemoryRequirements);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryRequirements2>(pMemoryRequirements, settings, "VkMemoryRequirements2*", "pMemoryRequirements", true, false, 4, dump_json_VkMemoryRequirements2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceImageSparseMemoryRequirements(ApiDumpInstance& dump_inst, VkDevice device, const VkDeviceImageMemoryRequirements* pInfo, uint32_t* pSparseMemoryRequirementCount, VkSparseImageMemoryRequirements2* pSparseMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceImageMemoryRequirements>(pInfo, settings, "const VkDeviceImageMemoryRequirements*", "pInfo", true, false, 4, dump_json_VkDeviceImageMemoryRequirements);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pSparseMemoryRequirementCount, settings, "uint32_t*", "pSparseMemoryRequirementCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkSparseImageMemoryRequirements2>(pSparseMemoryRequirements, *pSparseMemoryRequirementCount, settings, "VkSparseImageMemoryRequirements2*", "VkSparseImageMemoryRequirements2", "pSparseMemoryRequirements", true, false, 4, dump_json_VkSparseImageMemoryRequirements2); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroySurfaceKHR(ApiDumpInstance& dump_inst, VkInstance instance, VkSurfaceKHR surface, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_value<const VkSurfaceKHR>(surface, NULL, settings, "VkSurfaceKHR", "surface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceSurfaceSupportKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t queueFamilyIndex, VkSurfaceKHR surface, VkBool32* pSupported)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(queueFamilyIndex, NULL, settings, "uint32_t", "queueFamilyIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const VkSurfaceKHR>(surface, NULL, settings, "VkSurfaceKHR", "surface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBool32>(pSupported, settings, "VkBool32*", "pSupported", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceSurfaceCapabilitiesKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, VkSurfaceKHR surface, VkSurfaceCapabilitiesKHR* pSurfaceCapabilities)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSurfaceKHR>(surface, NULL, settings, "VkSurfaceKHR", "surface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceCapabilitiesKHR>(pSurfaceCapabilities, settings, "VkSurfaceCapabilitiesKHR*", "pSurfaceCapabilities", true, false, 4, dump_json_VkSurfaceCapabilitiesKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceSurfaceFormatsKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, VkSurfaceKHR surface, uint32_t* pSurfaceFormatCount, VkSurfaceFormatKHR* pSurfaceFormats)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSurfaceKHR>(surface, NULL, settings, "VkSurfaceKHR", "surface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pSurfaceFormatCount, settings, "uint32_t*", "pSurfaceFormatCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkSurfaceFormatKHR>(pSurfaceFormats, *pSurfaceFormatCount, settings, "VkSurfaceFormatKHR*", "VkSurfaceFormatKHR", "pSurfaceFormats", true, false, 4, dump_json_VkSurfaceFormatKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceSurfacePresentModesKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, VkSurfaceKHR surface, uint32_t* pPresentModeCount, VkPresentModeKHR* pPresentModes)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSurfaceKHR>(surface, NULL, settings, "VkSurfaceKHR", "surface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPresentModeCount, settings, "uint32_t*", "pPresentModeCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkPresentModeKHR>(pPresentModes, *pPresentModeCount, settings, "VkPresentModeKHR*", "VkPresentModeKHR", "pPresentModes", false, false, 4, dump_json_VkPresentModeKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateSwapchainKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkSwapchainCreateInfoKHR* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSwapchainKHR* pSwapchain)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSwapchainCreateInfoKHR>(pCreateInfo, settings, "const VkSwapchainCreateInfoKHR*", "pCreateInfo", true, false, 4, dump_json_VkSwapchainCreateInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSwapchainKHR>(pSwapchain, settings, "VkSwapchainKHR*", "pSwapchain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroySwapchainKHR(ApiDumpInstance& dump_inst, VkDevice device, VkSwapchainKHR swapchain, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSwapchainKHR>(swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetSwapchainImagesKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkSwapchainKHR swapchain, uint32_t* pSwapchainImageCount, VkImage* pSwapchainImages)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSwapchainKHR>(swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pSwapchainImageCount, settings, "uint32_t*", "pSwapchainImageCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkImage>(pSwapchainImages, *pSwapchainImageCount, settings, "VkImage*", "VkImage", "pSwapchainImages", false, false, 4, dump_json_VkImage); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkAcquireNextImageKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkSwapchainKHR swapchain, uint64_t timeout, VkSemaphore semaphore, VkFence fence, uint32_t* pImageIndex)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSwapchainKHR>(swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << ",\n";
        dump_json_value<const uint64_t>(timeout, NULL, settings, "uint64_t", "timeout", false, false, 4, dump_json_uint64_t);
        settings.stream() << ",\n";
        dump_json_value<const VkSemaphore>(semaphore, NULL, settings, "VkSemaphore", "semaphore", false, false, 4, dump_json_VkSemaphore);
        settings.stream() << ",\n";
        dump_json_value<const VkFence>(fence, NULL, settings, "VkFence", "fence", false, false, 4, dump_json_VkFence);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pImageIndex, settings, "uint32_t*", "pImageIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkQueuePresentKHR(ApiDumpInstance& dump_inst, VkResult result, VkQueue queue, const VkPresentInfoKHR* pPresentInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkQueue>(queue, NULL, settings, "VkQueue", "queue", false, false, 4, dump_json_VkQueue);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPresentInfoKHR>(pPresentInfo, settings, "const VkPresentInfoKHR*", "pPresentInfo", true, false, 4, dump_json_VkPresentInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceGroupPresentCapabilitiesKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDeviceGroupPresentCapabilitiesKHR* pDeviceGroupPresentCapabilities)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceGroupPresentCapabilitiesKHR>(pDeviceGroupPresentCapabilities, settings, "VkDeviceGroupPresentCapabilitiesKHR*", "pDeviceGroupPresentCapabilities", true, false, 4, dump_json_VkDeviceGroupPresentCapabilitiesKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceGroupSurfacePresentModesKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkSurfaceKHR surface, VkDeviceGroupPresentModeFlagsKHR* pModes)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSurfaceKHR>(surface, NULL, settings, "VkSurfaceKHR", "surface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceGroupPresentModeFlagsKHR>(pModes, settings, "VkDeviceGroupPresentModeFlagsKHR*", "pModes", false, false, 4, dump_json_VkDeviceGroupPresentModeFlagsKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDevicePresentRectanglesKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, VkSurfaceKHR surface, uint32_t* pRectCount, VkRect2D* pRects)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSurfaceKHR>(surface, NULL, settings, "VkSurfaceKHR", "surface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pRectCount, settings, "uint32_t*", "pRectCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkRect2D>(pRects, *pRectCount, settings, "VkRect2D*", "VkRect2D", "pRects", true, false, 4, dump_json_VkRect2D); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkAcquireNextImage2KHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkAcquireNextImageInfoKHR* pAcquireInfo, uint32_t* pImageIndex)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAcquireNextImageInfoKHR>(pAcquireInfo, settings, "const VkAcquireNextImageInfoKHR*", "pAcquireInfo", true, false, 4, dump_json_VkAcquireNextImageInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pImageIndex, settings, "uint32_t*", "pImageIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceDisplayPropertiesKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t* pPropertyCount, VkDisplayPropertiesKHR* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPropertyCount, settings, "uint32_t*", "pPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkDisplayPropertiesKHR>(pProperties, *pPropertyCount, settings, "VkDisplayPropertiesKHR*", "VkDisplayPropertiesKHR", "pProperties", true, false, 4, dump_json_VkDisplayPropertiesKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceDisplayPlanePropertiesKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t* pPropertyCount, VkDisplayPlanePropertiesKHR* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPropertyCount, settings, "uint32_t*", "pPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkDisplayPlanePropertiesKHR>(pProperties, *pPropertyCount, settings, "VkDisplayPlanePropertiesKHR*", "VkDisplayPlanePropertiesKHR", "pProperties", true, false, 4, dump_json_VkDisplayPlanePropertiesKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDisplayPlaneSupportedDisplaysKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t planeIndex, uint32_t* pDisplayCount, VkDisplayKHR* pDisplays)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(planeIndex, NULL, settings, "uint32_t", "planeIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pDisplayCount, settings, "uint32_t*", "pDisplayCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkDisplayKHR>(pDisplays, *pDisplayCount, settings, "VkDisplayKHR*", "VkDisplayKHR", "pDisplays", false, false, 4, dump_json_VkDisplayKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDisplayModePropertiesKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, VkDisplayKHR display, uint32_t* pPropertyCount, VkDisplayModePropertiesKHR* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDisplayKHR>(display, NULL, settings, "VkDisplayKHR", "display", false, false, 4, dump_json_VkDisplayKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPropertyCount, settings, "uint32_t*", "pPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkDisplayModePropertiesKHR>(pProperties, *pPropertyCount, settings, "VkDisplayModePropertiesKHR*", "VkDisplayModePropertiesKHR", "pProperties", true, false, 4, dump_json_VkDisplayModePropertiesKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateDisplayModeKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, VkDisplayKHR display, const VkDisplayModeCreateInfoKHR* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkDisplayModeKHR* pMode)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDisplayKHR>(display, NULL, settings, "VkDisplayKHR", "display", false, false, 4, dump_json_VkDisplayKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDisplayModeCreateInfoKHR>(pCreateInfo, settings, "const VkDisplayModeCreateInfoKHR*", "pCreateInfo", true, false, 4, dump_json_VkDisplayModeCreateInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDisplayModeKHR>(pMode, settings, "VkDisplayModeKHR*", "pMode", false, false, 4, dump_json_VkDisplayModeKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDisplayPlaneCapabilitiesKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, VkDisplayModeKHR mode, uint32_t planeIndex, VkDisplayPlaneCapabilitiesKHR* pCapabilities)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDisplayModeKHR>(mode, NULL, settings, "VkDisplayModeKHR", "mode", false, false, 4, dump_json_VkDisplayModeKHR);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(planeIndex, NULL, settings, "uint32_t", "planeIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDisplayPlaneCapabilitiesKHR>(pCapabilities, settings, "VkDisplayPlaneCapabilitiesKHR*", "pCapabilities", true, false, 4, dump_json_VkDisplayPlaneCapabilitiesKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateDisplayPlaneSurfaceKHR(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkDisplaySurfaceCreateInfoKHR* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSurfaceKHR* pSurface)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDisplaySurfaceCreateInfoKHR>(pCreateInfo, settings, "const VkDisplaySurfaceCreateInfoKHR*", "pCreateInfo", true, false, 4, dump_json_VkDisplaySurfaceCreateInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceKHR>(pSurface, settings, "VkSurfaceKHR*", "pSurface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateSharedSwapchainsKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t swapchainCount, const VkSwapchainCreateInfoKHR* pCreateInfos, const VkAllocationCallbacks* pAllocator, VkSwapchainKHR* pSwapchains)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(swapchainCount, NULL, settings, "uint32_t", "swapchainCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkSwapchainCreateInfoKHR>(pCreateInfos, swapchainCount, settings, "const VkSwapchainCreateInfoKHR*", "const VkSwapchainCreateInfoKHR", "pCreateInfos", true, false, 4, dump_json_VkSwapchainCreateInfoKHR); // PQA
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_array<const VkSwapchainKHR>(pSwapchains, swapchainCount, settings, "VkSwapchainKHR*", "VkSwapchainKHR", "pSwapchains", false, false, 4, dump_json_VkSwapchainKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_XLIB_KHR)
void dump_json_vkCreateXlibSurfaceKHR(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkXlibSurfaceCreateInfoKHR* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSurfaceKHR* pSurface)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkXlibSurfaceCreateInfoKHR>(pCreateInfo, settings, "const VkXlibSurfaceCreateInfoKHR*", "pCreateInfo", true, false, 4, dump_json_VkXlibSurfaceCreateInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceKHR>(pSurface, settings, "VkSurfaceKHR*", "pSurface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_XLIB_KHR
#if defined(VK_USE_PLATFORM_XLIB_KHR)
void dump_json_vkGetPhysicalDeviceXlibPresentationSupportKHR(ApiDumpInstance& dump_inst, VkBool32 result, VkPhysicalDevice physicalDevice, uint32_t queueFamilyIndex, Display* dpy, VisualID visualID)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkBool32(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(queueFamilyIndex, NULL, settings, "uint32_t", "queueFamilyIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const Display*>(dpy, NULL, settings, "Display*", "dpy", false, false, 4, dump_json_Display);
        settings.stream() << ",\n";
        dump_json_value<const VisualID>(visualID, NULL, settings, "VisualID", "visualID", false, false, 4, dump_json_VisualID);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_XLIB_KHR
#if defined(VK_USE_PLATFORM_XCB_KHR)
void dump_json_vkCreateXcbSurfaceKHR(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkXcbSurfaceCreateInfoKHR* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSurfaceKHR* pSurface)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkXcbSurfaceCreateInfoKHR>(pCreateInfo, settings, "const VkXcbSurfaceCreateInfoKHR*", "pCreateInfo", true, false, 4, dump_json_VkXcbSurfaceCreateInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceKHR>(pSurface, settings, "VkSurfaceKHR*", "pSurface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_XCB_KHR
#if defined(VK_USE_PLATFORM_XCB_KHR)
void dump_json_vkGetPhysicalDeviceXcbPresentationSupportKHR(ApiDumpInstance& dump_inst, VkBool32 result, VkPhysicalDevice physicalDevice, uint32_t queueFamilyIndex, xcb_connection_t* connection, xcb_visualid_t visual_id)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkBool32(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(queueFamilyIndex, NULL, settings, "uint32_t", "queueFamilyIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const xcb_connection_t*>(connection, NULL, settings, "xcb_connection_t*", "connection", false, false, 4, dump_json_xcb_connection_t);
        settings.stream() << ",\n";
        dump_json_value<const xcb_visualid_t>(visual_id, NULL, settings, "xcb_visualid_t", "visual_id", false, false, 4, dump_json_xcb_visualid_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_XCB_KHR
#if defined(VK_USE_PLATFORM_WAYLAND_KHR)
void dump_json_vkCreateWaylandSurfaceKHR(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkWaylandSurfaceCreateInfoKHR* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSurfaceKHR* pSurface)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkWaylandSurfaceCreateInfoKHR>(pCreateInfo, settings, "const VkWaylandSurfaceCreateInfoKHR*", "pCreateInfo", true, false, 4, dump_json_VkWaylandSurfaceCreateInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceKHR>(pSurface, settings, "VkSurfaceKHR*", "pSurface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WAYLAND_KHR
#if defined(VK_USE_PLATFORM_WAYLAND_KHR)
void dump_json_vkGetPhysicalDeviceWaylandPresentationSupportKHR(ApiDumpInstance& dump_inst, VkBool32 result, VkPhysicalDevice physicalDevice, uint32_t queueFamilyIndex, struct wl_display* display)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkBool32(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(queueFamilyIndex, NULL, settings, "uint32_t", "queueFamilyIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const wl_display*>(display, NULL, settings, "struct wl_display*", "display", false, false, 4, dump_json_wl_display);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WAYLAND_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_vkCreateAndroidSurfaceKHR(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkAndroidSurfaceCreateInfoKHR* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSurfaceKHR* pSurface)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAndroidSurfaceCreateInfoKHR>(pCreateInfo, settings, "const VkAndroidSurfaceCreateInfoKHR*", "pCreateInfo", true, false, 4, dump_json_VkAndroidSurfaceCreateInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceKHR>(pSurface, settings, "VkSurfaceKHR*", "pSurface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_vkCreateWin32SurfaceKHR(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkWin32SurfaceCreateInfoKHR* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSurfaceKHR* pSurface)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkWin32SurfaceCreateInfoKHR>(pCreateInfo, settings, "const VkWin32SurfaceCreateInfoKHR*", "pCreateInfo", true, false, 4, dump_json_VkWin32SurfaceCreateInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceKHR>(pSurface, settings, "VkSurfaceKHR*", "pSurface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_vkGetPhysicalDeviceWin32PresentationSupportKHR(ApiDumpInstance& dump_inst, VkBool32 result, VkPhysicalDevice physicalDevice, uint32_t queueFamilyIndex)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkBool32(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(queueFamilyIndex, NULL, settings, "uint32_t", "queueFamilyIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WIN32_KHR
void dump_json_vkCreateDebugReportCallbackEXT(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkDebugReportCallbackCreateInfoEXT* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkDebugReportCallbackEXT* pCallback)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDebugReportCallbackCreateInfoEXT>(pCreateInfo, settings, "const VkDebugReportCallbackCreateInfoEXT*", "pCreateInfo", true, false, 4, dump_json_VkDebugReportCallbackCreateInfoEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDebugReportCallbackEXT>(pCallback, settings, "VkDebugReportCallbackEXT*", "pCallback", false, false, 4, dump_json_VkDebugReportCallbackEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyDebugReportCallbackEXT(ApiDumpInstance& dump_inst, VkInstance instance, VkDebugReportCallbackEXT callback, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_value<const VkDebugReportCallbackEXT>(callback, NULL, settings, "VkDebugReportCallbackEXT", "callback", false, false, 4, dump_json_VkDebugReportCallbackEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDebugReportMessageEXT(ApiDumpInstance& dump_inst, VkInstance instance, VkDebugReportFlagsEXT flags, VkDebugReportObjectTypeEXT objectType, uint64_t object, size_t location, int32_t messageCode, const char* pLayerPrefix, const char* pMessage)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_value<const VkDebugReportFlagsEXT>(flags, NULL, settings, "VkDebugReportFlagsEXT", "flags", false, false, 4, dump_json_VkDebugReportFlagsEXT);
        settings.stream() << ",\n";
        dump_json_value<const VkDebugReportObjectTypeEXT>(objectType, NULL, settings, "VkDebugReportObjectTypeEXT", "objectType", false, false, 4, dump_json_VkDebugReportObjectTypeEXT);
        settings.stream() << ",\n";
        dump_json_value<const uint64_t>(object, NULL, settings, "uint64_t", "object", false, false, 4, dump_json_uint64_t);
        settings.stream() << ",\n";
        dump_json_value<const size_t>(location, NULL, settings, "size_t", "location", false, false, 4, dump_json_size_t);
        settings.stream() << ",\n";
        dump_json_value<const int32_t>(messageCode, NULL, settings, "int32_t", "messageCode", false, false, 4, dump_json_int32_t);
        settings.stream() << ",\n";
        dump_json_value<const char*>(pLayerPrefix, NULL, settings, "const char*", "pLayerPrefix", false, false, 4, dump_json_cstring);
        settings.stream() << ",\n";
        dump_json_value<const char*>(pMessage, NULL, settings, "const char*", "pMessage", false, false, 4, dump_json_cstring);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDebugMarkerSetObjectTagEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkDebugMarkerObjectTagInfoEXT* pTagInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDebugMarkerObjectTagInfoEXT>(pTagInfo, settings, "const VkDebugMarkerObjectTagInfoEXT*", "pTagInfo", true, false, 4, dump_json_VkDebugMarkerObjectTagInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDebugMarkerSetObjectNameEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkDebugMarkerObjectNameInfoEXT* pNameInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDebugMarkerObjectNameInfoEXT>(pNameInfo, settings, "const VkDebugMarkerObjectNameInfoEXT*", "pNameInfo", true, false, 4, dump_json_VkDebugMarkerObjectNameInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDebugMarkerBeginEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkDebugMarkerMarkerInfoEXT* pMarkerInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDebugMarkerMarkerInfoEXT>(pMarkerInfo, settings, "const VkDebugMarkerMarkerInfoEXT*", "pMarkerInfo", true, false, 4, dump_json_VkDebugMarkerMarkerInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDebugMarkerEndEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDebugMarkerInsertEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkDebugMarkerMarkerInfoEXT* pMarkerInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDebugMarkerMarkerInfoEXT>(pMarkerInfo, settings, "const VkDebugMarkerMarkerInfoEXT*", "pMarkerInfo", true, false, 4, dump_json_VkDebugMarkerMarkerInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceVideoCapabilitiesKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, const VkVideoProfileInfoKHR* pVideoProfile, VkVideoCapabilitiesKHR* pCapabilities)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkVideoProfileInfoKHR>(pVideoProfile, settings, "const VkVideoProfileInfoKHR*", "pVideoProfile", true, false, 4, dump_json_VkVideoProfileInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkVideoCapabilitiesKHR>(pCapabilities, settings, "VkVideoCapabilitiesKHR*", "pCapabilities", true, false, 4, dump_json_VkVideoCapabilitiesKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceVideoFormatPropertiesKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, const VkPhysicalDeviceVideoFormatInfoKHR* pVideoFormatInfo, uint32_t* pVideoFormatPropertyCount, VkVideoFormatPropertiesKHR* pVideoFormatProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceVideoFormatInfoKHR>(pVideoFormatInfo, settings, "const VkPhysicalDeviceVideoFormatInfoKHR*", "pVideoFormatInfo", true, false, 4, dump_json_VkPhysicalDeviceVideoFormatInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pVideoFormatPropertyCount, settings, "uint32_t*", "pVideoFormatPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkVideoFormatPropertiesKHR>(pVideoFormatProperties, *pVideoFormatPropertyCount, settings, "VkVideoFormatPropertiesKHR*", "VkVideoFormatPropertiesKHR", "pVideoFormatProperties", true, false, 4, dump_json_VkVideoFormatPropertiesKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateVideoSessionKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkVideoSessionCreateInfoKHR* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkVideoSessionKHR* pVideoSession)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkVideoSessionCreateInfoKHR>(pCreateInfo, settings, "const VkVideoSessionCreateInfoKHR*", "pCreateInfo", true, false, 4, dump_json_VkVideoSessionCreateInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkVideoSessionKHR>(pVideoSession, settings, "VkVideoSessionKHR*", "pVideoSession", false, false, 4, dump_json_VkVideoSessionKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyVideoSessionKHR(ApiDumpInstance& dump_inst, VkDevice device, VkVideoSessionKHR videoSession, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkVideoSessionKHR>(videoSession, NULL, settings, "VkVideoSessionKHR", "videoSession", false, false, 4, dump_json_VkVideoSessionKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetVideoSessionMemoryRequirementsKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkVideoSessionKHR videoSession, uint32_t* pMemoryRequirementsCount, VkVideoSessionMemoryRequirementsKHR* pMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkVideoSessionKHR>(videoSession, NULL, settings, "VkVideoSessionKHR", "videoSession", false, false, 4, dump_json_VkVideoSessionKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pMemoryRequirementsCount, settings, "uint32_t*", "pMemoryRequirementsCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkVideoSessionMemoryRequirementsKHR>(pMemoryRequirements, *pMemoryRequirementsCount, settings, "VkVideoSessionMemoryRequirementsKHR*", "VkVideoSessionMemoryRequirementsKHR", "pMemoryRequirements", true, false, 4, dump_json_VkVideoSessionMemoryRequirementsKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkBindVideoSessionMemoryKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkVideoSessionKHR videoSession, uint32_t bindSessionMemoryInfoCount, const VkBindVideoSessionMemoryInfoKHR* pBindSessionMemoryInfos)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkVideoSessionKHR>(videoSession, NULL, settings, "VkVideoSessionKHR", "videoSession", false, false, 4, dump_json_VkVideoSessionKHR);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(bindSessionMemoryInfoCount, NULL, settings, "uint32_t", "bindSessionMemoryInfoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBindVideoSessionMemoryInfoKHR>(pBindSessionMemoryInfos, bindSessionMemoryInfoCount, settings, "const VkBindVideoSessionMemoryInfoKHR*", "const VkBindVideoSessionMemoryInfoKHR", "pBindSessionMemoryInfos", true, false, 4, dump_json_VkBindVideoSessionMemoryInfoKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateVideoSessionParametersKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkVideoSessionParametersCreateInfoKHR* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkVideoSessionParametersKHR* pVideoSessionParameters)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkVideoSessionParametersCreateInfoKHR>(pCreateInfo, settings, "const VkVideoSessionParametersCreateInfoKHR*", "pCreateInfo", true, false, 4, dump_json_VkVideoSessionParametersCreateInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkVideoSessionParametersKHR>(pVideoSessionParameters, settings, "VkVideoSessionParametersKHR*", "pVideoSessionParameters", false, false, 4, dump_json_VkVideoSessionParametersKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkUpdateVideoSessionParametersKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkVideoSessionParametersKHR videoSessionParameters, const VkVideoSessionParametersUpdateInfoKHR* pUpdateInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkVideoSessionParametersKHR>(videoSessionParameters, NULL, settings, "VkVideoSessionParametersKHR", "videoSessionParameters", false, false, 4, dump_json_VkVideoSessionParametersKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkVideoSessionParametersUpdateInfoKHR>(pUpdateInfo, settings, "const VkVideoSessionParametersUpdateInfoKHR*", "pUpdateInfo", true, false, 4, dump_json_VkVideoSessionParametersUpdateInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyVideoSessionParametersKHR(ApiDumpInstance& dump_inst, VkDevice device, VkVideoSessionParametersKHR videoSessionParameters, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkVideoSessionParametersKHR>(videoSessionParameters, NULL, settings, "VkVideoSessionParametersKHR", "videoSessionParameters", false, false, 4, dump_json_VkVideoSessionParametersKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBeginVideoCodingKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkVideoBeginCodingInfoKHR* pBeginInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkVideoBeginCodingInfoKHR>(pBeginInfo, settings, "const VkVideoBeginCodingInfoKHR*", "pBeginInfo", true, false, 4, dump_json_VkVideoBeginCodingInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdEndVideoCodingKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkVideoEndCodingInfoKHR* pEndCodingInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkVideoEndCodingInfoKHR>(pEndCodingInfo, settings, "const VkVideoEndCodingInfoKHR*", "pEndCodingInfo", true, false, 4, dump_json_VkVideoEndCodingInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdControlVideoCodingKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkVideoCodingControlInfoKHR* pCodingControlInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkVideoCodingControlInfoKHR>(pCodingControlInfo, settings, "const VkVideoCodingControlInfoKHR*", "pCodingControlInfo", true, false, 4, dump_json_VkVideoCodingControlInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDecodeVideoKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkVideoDecodeInfoKHR* pDecodeInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkVideoDecodeInfoKHR>(pDecodeInfo, settings, "const VkVideoDecodeInfoKHR*", "pDecodeInfo", true, false, 4, dump_json_VkVideoDecodeInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBindTransformFeedbackBuffersEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstBinding, uint32_t bindingCount, const VkBuffer* pBuffers, const VkDeviceSize* pOffsets, const VkDeviceSize* pSizes)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstBinding, NULL, settings, "uint32_t", "firstBinding", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(bindingCount, NULL, settings, "uint32_t", "bindingCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBuffer>(pBuffers, bindingCount, settings, "const VkBuffer*", "const VkBuffer", "pBuffers", false, false, 4, dump_json_VkBuffer); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkDeviceSize>(pOffsets, bindingCount, settings, "const VkDeviceSize*", "const VkDeviceSize", "pOffsets", false, false, 4, dump_json_VkDeviceSize); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkDeviceSize>(pSizes, bindingCount, settings, "const VkDeviceSize*", "const VkDeviceSize", "pSizes", false, false, 4, dump_json_VkDeviceSize); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBeginTransformFeedbackEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstCounterBuffer, uint32_t counterBufferCount, const VkBuffer* pCounterBuffers, const VkDeviceSize* pCounterBufferOffsets)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstCounterBuffer, NULL, settings, "uint32_t", "firstCounterBuffer", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(counterBufferCount, NULL, settings, "uint32_t", "counterBufferCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBuffer>(pCounterBuffers, counterBufferCount, settings, "const VkBuffer*", "const VkBuffer", "pCounterBuffers", false, false, 4, dump_json_VkBuffer); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkDeviceSize>(pCounterBufferOffsets, counterBufferCount, settings, "const VkDeviceSize*", "const VkDeviceSize", "pCounterBufferOffsets", false, false, 4, dump_json_VkDeviceSize); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdEndTransformFeedbackEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstCounterBuffer, uint32_t counterBufferCount, const VkBuffer* pCounterBuffers, const VkDeviceSize* pCounterBufferOffsets)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstCounterBuffer, NULL, settings, "uint32_t", "firstCounterBuffer", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(counterBufferCount, NULL, settings, "uint32_t", "counterBufferCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBuffer>(pCounterBuffers, counterBufferCount, settings, "const VkBuffer*", "const VkBuffer", "pCounterBuffers", false, false, 4, dump_json_VkBuffer); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkDeviceSize>(pCounterBufferOffsets, counterBufferCount, settings, "const VkDeviceSize*", "const VkDeviceSize", "pCounterBufferOffsets", false, false, 4, dump_json_VkDeviceSize); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBeginQueryIndexedEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkQueryPool queryPool, uint32_t query, VkQueryControlFlags flags, uint32_t index)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(query, NULL, settings, "uint32_t", "query", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryControlFlags>(flags, NULL, settings, "VkQueryControlFlags", "flags", false, false, 4, dump_json_VkQueryControlFlags);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(index, NULL, settings, "uint32_t", "index", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdEndQueryIndexedEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkQueryPool queryPool, uint32_t query, uint32_t index)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(query, NULL, settings, "uint32_t", "query", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(index, NULL, settings, "uint32_t", "index", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawIndirectByteCountEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t instanceCount, uint32_t firstInstance, VkBuffer counterBuffer, VkDeviceSize counterBufferOffset, uint32_t counterOffset, uint32_t vertexStride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(instanceCount, NULL, settings, "uint32_t", "instanceCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstInstance, NULL, settings, "uint32_t", "firstInstance", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(counterBuffer, NULL, settings, "VkBuffer", "counterBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(counterBufferOffset, NULL, settings, "VkDeviceSize", "counterBufferOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(counterOffset, NULL, settings, "uint32_t", "counterOffset", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(vertexStride, NULL, settings, "uint32_t", "vertexStride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateCuModuleNVX(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkCuModuleCreateInfoNVX* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkCuModuleNVX* pModule)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCuModuleCreateInfoNVX>(pCreateInfo, settings, "const VkCuModuleCreateInfoNVX*", "pCreateInfo", true, false, 4, dump_json_VkCuModuleCreateInfoNVX);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCuModuleNVX>(pModule, settings, "VkCuModuleNVX*", "pModule", false, false, 4, dump_json_VkCuModuleNVX);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateCuFunctionNVX(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkCuFunctionCreateInfoNVX* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkCuFunctionNVX* pFunction)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCuFunctionCreateInfoNVX>(pCreateInfo, settings, "const VkCuFunctionCreateInfoNVX*", "pCreateInfo", true, false, 4, dump_json_VkCuFunctionCreateInfoNVX);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCuFunctionNVX>(pFunction, settings, "VkCuFunctionNVX*", "pFunction", false, false, 4, dump_json_VkCuFunctionNVX);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyCuModuleNVX(ApiDumpInstance& dump_inst, VkDevice device, VkCuModuleNVX module, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkCuModuleNVX>(module, NULL, settings, "VkCuModuleNVX", "module", false, false, 4, dump_json_VkCuModuleNVX);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyCuFunctionNVX(ApiDumpInstance& dump_inst, VkDevice device, VkCuFunctionNVX function, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkCuFunctionNVX>(function, NULL, settings, "VkCuFunctionNVX", "function", false, false, 4, dump_json_VkCuFunctionNVX);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCuLaunchKernelNVX(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCuLaunchInfoNVX* pLaunchInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCuLaunchInfoNVX>(pLaunchInfo, settings, "const VkCuLaunchInfoNVX*", "pLaunchInfo", true, false, 4, dump_json_VkCuLaunchInfoNVX);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetImageViewHandleNVX(ApiDumpInstance& dump_inst, uint32_t result, VkDevice device, const VkImageViewHandleInfoNVX* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_uint32_t(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageViewHandleInfoNVX>(pInfo, settings, "const VkImageViewHandleInfoNVX*", "pInfo", true, false, 4, dump_json_VkImageViewHandleInfoNVX);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetImageViewAddressNVX(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkImageView imageView, VkImageViewAddressPropertiesNVX* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkImageView>(imageView, NULL, settings, "VkImageView", "imageView", false, false, 4, dump_json_VkImageView);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageViewAddressPropertiesNVX>(pProperties, settings, "VkImageViewAddressPropertiesNVX*", "pProperties", true, false, 4, dump_json_VkImageViewAddressPropertiesNVX);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawIndirectCountAMD(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset, VkBuffer countBuffer, VkDeviceSize countBufferOffset, uint32_t maxDrawCount, uint32_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(countBuffer, NULL, settings, "VkBuffer", "countBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(countBufferOffset, NULL, settings, "VkDeviceSize", "countBufferOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(maxDrawCount, NULL, settings, "uint32_t", "maxDrawCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawIndexedIndirectCountAMD(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset, VkBuffer countBuffer, VkDeviceSize countBufferOffset, uint32_t maxDrawCount, uint32_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(countBuffer, NULL, settings, "VkBuffer", "countBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(countBufferOffset, NULL, settings, "VkDeviceSize", "countBufferOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(maxDrawCount, NULL, settings, "uint32_t", "maxDrawCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetShaderInfoAMD(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkPipeline pipeline, VkShaderStageFlagBits shaderStage, VkShaderInfoTypeAMD infoType, size_t* pInfoSize, void* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipeline>(pipeline, NULL, settings, "VkPipeline", "pipeline", false, false, 4, dump_json_VkPipeline);
        settings.stream() << ",\n";
        dump_json_value<const VkShaderStageFlagBits>(shaderStage, NULL, settings, "VkShaderStageFlagBits", "shaderStage", false, false, 4, dump_json_VkShaderStageFlagBits);
        settings.stream() << ",\n";
        dump_json_value<const VkShaderInfoTypeAMD>(infoType, NULL, settings, "VkShaderInfoTypeAMD", "infoType", false, false, 4, dump_json_VkShaderInfoTypeAMD);
        settings.stream() << ",\n";
        dump_json_pointer<const size_t>(pInfoSize, settings, "size_t*", "pInfoSize", false, false, 4, dump_json_size_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pInfo, NULL, settings, "void*", "pInfo", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBeginRenderingKHR(ApiDumpInstance& dump_inst, VkCommandBuffer                   commandBuffer, const VkRenderingInfo*                              pRenderingInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRenderingInfo>(pRenderingInfo, settings, "const VkRenderingInfo*", "pRenderingInfo", true, false, 4, dump_json_VkRenderingInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdEndRenderingKHR(ApiDumpInstance& dump_inst, VkCommandBuffer                   commandBuffer)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_GGP)
void dump_json_vkCreateStreamDescriptorSurfaceGGP(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkStreamDescriptorSurfaceCreateInfoGGP* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSurfaceKHR* pSurface)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkStreamDescriptorSurfaceCreateInfoGGP>(pCreateInfo, settings, "const VkStreamDescriptorSurfaceCreateInfoGGP*", "pCreateInfo", true, false, 4, dump_json_VkStreamDescriptorSurfaceCreateInfoGGP);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceKHR>(pSurface, settings, "VkSurfaceKHR*", "pSurface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_GGP
void dump_json_vkGetPhysicalDeviceExternalImageFormatPropertiesNV(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, VkFormat format, VkImageType type, VkImageTiling tiling, VkImageUsageFlags usage, VkImageCreateFlags flags, VkExternalMemoryHandleTypeFlagsNV externalHandleType, VkExternalImageFormatPropertiesNV* pExternalImageFormatProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkFormat>(format, NULL, settings, "VkFormat", "format", false, false, 4, dump_json_VkFormat);
        settings.stream() << ",\n";
        dump_json_value<const VkImageType>(type, NULL, settings, "VkImageType", "type", false, false, 4, dump_json_VkImageType);
        settings.stream() << ",\n";
        dump_json_value<const VkImageTiling>(tiling, NULL, settings, "VkImageTiling", "tiling", false, false, 4, dump_json_VkImageTiling);
        settings.stream() << ",\n";
        dump_json_value<const VkImageUsageFlags>(usage, NULL, settings, "VkImageUsageFlags", "usage", false, false, 4, dump_json_VkImageUsageFlags);
        settings.stream() << ",\n";
        dump_json_value<const VkImageCreateFlags>(flags, NULL, settings, "VkImageCreateFlags", "flags", false, false, 4, dump_json_VkImageCreateFlags);
        settings.stream() << ",\n";
        dump_json_value<const VkExternalMemoryHandleTypeFlagsNV>(externalHandleType, NULL, settings, "VkExternalMemoryHandleTypeFlagsNV", "externalHandleType", false, false, 4, dump_json_VkExternalMemoryHandleTypeFlagsNV);
        settings.stream() << ",\n";
        dump_json_pointer<const VkExternalImageFormatPropertiesNV>(pExternalImageFormatProperties, settings, "VkExternalImageFormatPropertiesNV*", "pExternalImageFormatProperties", true, false, 4, dump_json_VkExternalImageFormatPropertiesNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_vkGetMemoryWin32HandleNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDeviceMemory memory, VkExternalMemoryHandleTypeFlagsNV handleType, HANDLE* pHandle)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceMemory>(memory, NULL, settings, "VkDeviceMemory", "memory", false, false, 4, dump_json_VkDeviceMemory);
        settings.stream() << ",\n";
        dump_json_value<const VkExternalMemoryHandleTypeFlagsNV>(handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagsNV", "handleType", false, false, 4, dump_json_VkExternalMemoryHandleTypeFlagsNV);
        settings.stream() << ",\n";
        dump_json_pointer<const HANDLE>(pHandle, settings, "HANDLE*", "pHandle", false, false, 4, dump_json_HANDLE);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WIN32_KHR
void dump_json_vkGetPhysicalDeviceFeatures2KHR(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, VkPhysicalDeviceFeatures2* pFeatures)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceFeatures2>(pFeatures, settings, "VkPhysicalDeviceFeatures2*", "pFeatures", true, false, 4, dump_json_VkPhysicalDeviceFeatures2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceProperties2KHR(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, VkPhysicalDeviceProperties2* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceProperties2>(pProperties, settings, "VkPhysicalDeviceProperties2*", "pProperties", true, false, 4, dump_json_VkPhysicalDeviceProperties2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceFormatProperties2KHR(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, VkFormat format, VkFormatProperties2* pFormatProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkFormat>(format, NULL, settings, "VkFormat", "format", false, false, 4, dump_json_VkFormat);
        settings.stream() << ",\n";
        dump_json_pointer<const VkFormatProperties2>(pFormatProperties, settings, "VkFormatProperties2*", "pFormatProperties", true, false, 4, dump_json_VkFormatProperties2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceImageFormatProperties2KHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, const VkPhysicalDeviceImageFormatInfo2* pImageFormatInfo, VkImageFormatProperties2* pImageFormatProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceImageFormatInfo2>(pImageFormatInfo, settings, "const VkPhysicalDeviceImageFormatInfo2*", "pImageFormatInfo", true, false, 4, dump_json_VkPhysicalDeviceImageFormatInfo2);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageFormatProperties2>(pImageFormatProperties, settings, "VkImageFormatProperties2*", "pImageFormatProperties", true, false, 4, dump_json_VkImageFormatProperties2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceQueueFamilyProperties2KHR(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, uint32_t* pQueueFamilyPropertyCount, VkQueueFamilyProperties2* pQueueFamilyProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pQueueFamilyPropertyCount, settings, "uint32_t*", "pQueueFamilyPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkQueueFamilyProperties2>(pQueueFamilyProperties, *pQueueFamilyPropertyCount, settings, "VkQueueFamilyProperties2*", "VkQueueFamilyProperties2", "pQueueFamilyProperties", true, false, 4, dump_json_VkQueueFamilyProperties2); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceMemoryProperties2KHR(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, VkPhysicalDeviceMemoryProperties2* pMemoryProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceMemoryProperties2>(pMemoryProperties, settings, "VkPhysicalDeviceMemoryProperties2*", "pMemoryProperties", true, false, 4, dump_json_VkPhysicalDeviceMemoryProperties2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceSparseImageFormatProperties2KHR(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, const VkPhysicalDeviceSparseImageFormatInfo2* pFormatInfo, uint32_t* pPropertyCount, VkSparseImageFormatProperties2* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceSparseImageFormatInfo2>(pFormatInfo, settings, "const VkPhysicalDeviceSparseImageFormatInfo2*", "pFormatInfo", true, false, 4, dump_json_VkPhysicalDeviceSparseImageFormatInfo2);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPropertyCount, settings, "uint32_t*", "pPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkSparseImageFormatProperties2>(pProperties, *pPropertyCount, settings, "VkSparseImageFormatProperties2*", "VkSparseImageFormatProperties2", "pProperties", true, false, 4, dump_json_VkSparseImageFormatProperties2); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceGroupPeerMemoryFeaturesKHR(ApiDumpInstance& dump_inst, VkDevice device, uint32_t heapIndex, uint32_t localDeviceIndex, uint32_t remoteDeviceIndex, VkPeerMemoryFeatureFlags* pPeerMemoryFeatures)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(heapIndex, NULL, settings, "uint32_t", "heapIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(localDeviceIndex, NULL, settings, "uint32_t", "localDeviceIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(remoteDeviceIndex, NULL, settings, "uint32_t", "remoteDeviceIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPeerMemoryFeatureFlags>(pPeerMemoryFeatures, settings, "VkPeerMemoryFeatureFlags*", "pPeerMemoryFeatures", false, false, 4, dump_json_VkPeerMemoryFeatureFlags);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDeviceMaskKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t deviceMask)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(deviceMask, NULL, settings, "uint32_t", "deviceMask", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDispatchBaseKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t baseGroupX, uint32_t baseGroupY, uint32_t baseGroupZ, uint32_t groupCountX, uint32_t groupCountY, uint32_t groupCountZ)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(baseGroupX, NULL, settings, "uint32_t", "baseGroupX", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(baseGroupY, NULL, settings, "uint32_t", "baseGroupY", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(baseGroupZ, NULL, settings, "uint32_t", "baseGroupZ", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCountX, NULL, settings, "uint32_t", "groupCountX", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCountY, NULL, settings, "uint32_t", "groupCountY", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCountZ, NULL, settings, "uint32_t", "groupCountZ", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_VI_NN)
void dump_json_vkCreateViSurfaceNN(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkViSurfaceCreateInfoNN* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSurfaceKHR* pSurface)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkViSurfaceCreateInfoNN>(pCreateInfo, settings, "const VkViSurfaceCreateInfoNN*", "pCreateInfo", true, false, 4, dump_json_VkViSurfaceCreateInfoNN);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceKHR>(pSurface, settings, "VkSurfaceKHR*", "pSurface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_VI_NN
void dump_json_vkTrimCommandPoolKHR(ApiDumpInstance& dump_inst, VkDevice device, VkCommandPool commandPool, VkCommandPoolTrimFlags flags)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkCommandPool>(commandPool, NULL, settings, "VkCommandPool", "commandPool", false, false, 4, dump_json_VkCommandPool);
        settings.stream() << ",\n";
        dump_json_value<const VkCommandPoolTrimFlags>(flags, NULL, settings, "VkCommandPoolTrimFlags", "flags", false, false, 4, dump_json_VkCommandPoolTrimFlags);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkEnumeratePhysicalDeviceGroupsKHR(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, uint32_t* pPhysicalDeviceGroupCount, VkPhysicalDeviceGroupProperties* pPhysicalDeviceGroupProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPhysicalDeviceGroupCount, settings, "uint32_t*", "pPhysicalDeviceGroupCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkPhysicalDeviceGroupProperties>(pPhysicalDeviceGroupProperties, *pPhysicalDeviceGroupCount, settings, "VkPhysicalDeviceGroupProperties*", "VkPhysicalDeviceGroupProperties", "pPhysicalDeviceGroupProperties", true, false, 4, dump_json_VkPhysicalDeviceGroupProperties); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceExternalBufferPropertiesKHR(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, const VkPhysicalDeviceExternalBufferInfo* pExternalBufferInfo, VkExternalBufferProperties* pExternalBufferProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceExternalBufferInfo>(pExternalBufferInfo, settings, "const VkPhysicalDeviceExternalBufferInfo*", "pExternalBufferInfo", true, false, 4, dump_json_VkPhysicalDeviceExternalBufferInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkExternalBufferProperties>(pExternalBufferProperties, settings, "VkExternalBufferProperties*", "pExternalBufferProperties", true, false, 4, dump_json_VkExternalBufferProperties);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_vkGetMemoryWin32HandleKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkMemoryGetWin32HandleInfoKHR* pGetWin32HandleInfo, HANDLE* pHandle)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryGetWin32HandleInfoKHR>(pGetWin32HandleInfo, settings, "const VkMemoryGetWin32HandleInfoKHR*", "pGetWin32HandleInfo", true, false, 4, dump_json_VkMemoryGetWin32HandleInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const HANDLE>(pHandle, settings, "HANDLE*", "pHandle", false, false, 4, dump_json_HANDLE);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_vkGetMemoryWin32HandlePropertiesKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkExternalMemoryHandleTypeFlagBits handleType, HANDLE handle, VkMemoryWin32HandlePropertiesKHR* pMemoryWin32HandleProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkExternalMemoryHandleTypeFlagBits>(handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagBits", "handleType", false, false, 4, dump_json_VkExternalMemoryHandleTypeFlagBits);
        settings.stream() << ",\n";
        dump_json_value<const HANDLE>(handle, NULL, settings, "HANDLE", "handle", false, false, 4, dump_json_HANDLE);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryWin32HandlePropertiesKHR>(pMemoryWin32HandleProperties, settings, "VkMemoryWin32HandlePropertiesKHR*", "pMemoryWin32HandleProperties", true, false, 4, dump_json_VkMemoryWin32HandlePropertiesKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WIN32_KHR
void dump_json_vkGetMemoryFdKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkMemoryGetFdInfoKHR* pGetFdInfo, int* pFd)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryGetFdInfoKHR>(pGetFdInfo, settings, "const VkMemoryGetFdInfoKHR*", "pGetFdInfo", true, false, 4, dump_json_VkMemoryGetFdInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const int>(pFd, settings, "int*", "pFd", false, false, 4, dump_json_int);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetMemoryFdPropertiesKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkExternalMemoryHandleTypeFlagBits handleType, int fd, VkMemoryFdPropertiesKHR* pMemoryFdProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkExternalMemoryHandleTypeFlagBits>(handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagBits", "handleType", false, false, 4, dump_json_VkExternalMemoryHandleTypeFlagBits);
        settings.stream() << ",\n";
        dump_json_value<const int>(fd, NULL, settings, "int", "fd", false, false, 4, dump_json_int);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryFdPropertiesKHR>(pMemoryFdProperties, settings, "VkMemoryFdPropertiesKHR*", "pMemoryFdProperties", true, false, 4, dump_json_VkMemoryFdPropertiesKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceExternalSemaphorePropertiesKHR(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, const VkPhysicalDeviceExternalSemaphoreInfo* pExternalSemaphoreInfo, VkExternalSemaphoreProperties* pExternalSemaphoreProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceExternalSemaphoreInfo>(pExternalSemaphoreInfo, settings, "const VkPhysicalDeviceExternalSemaphoreInfo*", "pExternalSemaphoreInfo", true, false, 4, dump_json_VkPhysicalDeviceExternalSemaphoreInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkExternalSemaphoreProperties>(pExternalSemaphoreProperties, settings, "VkExternalSemaphoreProperties*", "pExternalSemaphoreProperties", true, false, 4, dump_json_VkExternalSemaphoreProperties);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_vkImportSemaphoreWin32HandleKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkImportSemaphoreWin32HandleInfoKHR* pImportSemaphoreWin32HandleInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImportSemaphoreWin32HandleInfoKHR>(pImportSemaphoreWin32HandleInfo, settings, "const VkImportSemaphoreWin32HandleInfoKHR*", "pImportSemaphoreWin32HandleInfo", true, false, 4, dump_json_VkImportSemaphoreWin32HandleInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_vkGetSemaphoreWin32HandleKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkSemaphoreGetWin32HandleInfoKHR* pGetWin32HandleInfo, HANDLE* pHandle)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSemaphoreGetWin32HandleInfoKHR>(pGetWin32HandleInfo, settings, "const VkSemaphoreGetWin32HandleInfoKHR*", "pGetWin32HandleInfo", true, false, 4, dump_json_VkSemaphoreGetWin32HandleInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const HANDLE>(pHandle, settings, "HANDLE*", "pHandle", false, false, 4, dump_json_HANDLE);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WIN32_KHR
void dump_json_vkImportSemaphoreFdKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkImportSemaphoreFdInfoKHR* pImportSemaphoreFdInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImportSemaphoreFdInfoKHR>(pImportSemaphoreFdInfo, settings, "const VkImportSemaphoreFdInfoKHR*", "pImportSemaphoreFdInfo", true, false, 4, dump_json_VkImportSemaphoreFdInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetSemaphoreFdKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkSemaphoreGetFdInfoKHR* pGetFdInfo, int* pFd)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSemaphoreGetFdInfoKHR>(pGetFdInfo, settings, "const VkSemaphoreGetFdInfoKHR*", "pGetFdInfo", true, false, 4, dump_json_VkSemaphoreGetFdInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const int>(pFd, settings, "int*", "pFd", false, false, 4, dump_json_int);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdPushDescriptorSetKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkPipelineBindPoint pipelineBindPoint, VkPipelineLayout layout, uint32_t set, uint32_t descriptorWriteCount, const VkWriteDescriptorSet* pDescriptorWrites)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineBindPoint>(pipelineBindPoint, NULL, settings, "VkPipelineBindPoint", "pipelineBindPoint", false, false, 4, dump_json_VkPipelineBindPoint);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineLayout>(layout, NULL, settings, "VkPipelineLayout", "layout", false, false, 4, dump_json_VkPipelineLayout);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(set, NULL, settings, "uint32_t", "set", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(descriptorWriteCount, NULL, settings, "uint32_t", "descriptorWriteCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkWriteDescriptorSet>(pDescriptorWrites, descriptorWriteCount, settings, "const VkWriteDescriptorSet*", "const VkWriteDescriptorSet", "pDescriptorWrites", true, false, 4, dump_json_VkWriteDescriptorSet); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdPushDescriptorSetWithTemplateKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkDescriptorUpdateTemplate descriptorUpdateTemplate, VkPipelineLayout layout, uint32_t set, const void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDescriptorUpdateTemplate>(descriptorUpdateTemplate, NULL, settings, "VkDescriptorUpdateTemplate", "descriptorUpdateTemplate", false, false, 4, dump_json_VkDescriptorUpdateTemplate);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineLayout>(layout, NULL, settings, "VkPipelineLayout", "layout", false, false, 4, dump_json_VkPipelineLayout);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(set, NULL, settings, "uint32_t", "set", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "const void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBeginConditionalRenderingEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkConditionalRenderingBeginInfoEXT* pConditionalRenderingBegin)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkConditionalRenderingBeginInfoEXT>(pConditionalRenderingBegin, settings, "const VkConditionalRenderingBeginInfoEXT*", "pConditionalRenderingBegin", true, false, 4, dump_json_VkConditionalRenderingBeginInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdEndConditionalRenderingEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateDescriptorUpdateTemplateKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkDescriptorUpdateTemplateCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkDescriptorUpdateTemplate* pDescriptorUpdateTemplate)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorUpdateTemplateCreateInfo>(pCreateInfo, settings, "const VkDescriptorUpdateTemplateCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkDescriptorUpdateTemplateCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorUpdateTemplate>(pDescriptorUpdateTemplate, settings, "VkDescriptorUpdateTemplate*", "pDescriptorUpdateTemplate", false, false, 4, dump_json_VkDescriptorUpdateTemplate);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyDescriptorUpdateTemplateKHR(ApiDumpInstance& dump_inst, VkDevice device, VkDescriptorUpdateTemplate descriptorUpdateTemplate, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDescriptorUpdateTemplate>(descriptorUpdateTemplate, NULL, settings, "VkDescriptorUpdateTemplate", "descriptorUpdateTemplate", false, false, 4, dump_json_VkDescriptorUpdateTemplate);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkUpdateDescriptorSetWithTemplateKHR(ApiDumpInstance& dump_inst, VkDevice device, VkDescriptorSet descriptorSet, VkDescriptorUpdateTemplate descriptorUpdateTemplate, const void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDescriptorSet>(descriptorSet, NULL, settings, "VkDescriptorSet", "descriptorSet", false, false, 4, dump_json_VkDescriptorSet);
        settings.stream() << ",\n";
        dump_json_value<const VkDescriptorUpdateTemplate>(descriptorUpdateTemplate, NULL, settings, "VkDescriptorUpdateTemplate", "descriptorUpdateTemplate", false, false, 4, dump_json_VkDescriptorUpdateTemplate);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "const void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetViewportWScalingNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstViewport, uint32_t viewportCount, const VkViewportWScalingNV* pViewportWScalings)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstViewport, NULL, settings, "uint32_t", "firstViewport", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(viewportCount, NULL, settings, "uint32_t", "viewportCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkViewportWScalingNV>(pViewportWScalings, viewportCount, settings, "const VkViewportWScalingNV*", "const VkViewportWScalingNV", "pViewportWScalings", true, false, 4, dump_json_VkViewportWScalingNV); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkReleaseDisplayEXT(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, VkDisplayKHR display)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDisplayKHR>(display, NULL, settings, "VkDisplayKHR", "display", false, false, 4, dump_json_VkDisplayKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_XLIB_XRANDR_EXT)
void dump_json_vkAcquireXlibDisplayEXT(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, Display* dpy, VkDisplayKHR display)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const Display*>(dpy, NULL, settings, "Display*", "dpy", false, false, 4, dump_json_Display);
        settings.stream() << ",\n";
        dump_json_value<const VkDisplayKHR>(display, NULL, settings, "VkDisplayKHR", "display", false, false, 4, dump_json_VkDisplayKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_XLIB_XRANDR_EXT
#if defined(VK_USE_PLATFORM_XLIB_XRANDR_EXT)
void dump_json_vkGetRandROutputDisplayEXT(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, Display* dpy, RROutput rrOutput, VkDisplayKHR* pDisplay)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const Display*>(dpy, NULL, settings, "Display*", "dpy", false, false, 4, dump_json_Display);
        settings.stream() << ",\n";
        dump_json_value<const RROutput>(rrOutput, NULL, settings, "RROutput", "rrOutput", false, false, 4, dump_json_RROutput);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDisplayKHR>(pDisplay, settings, "VkDisplayKHR*", "pDisplay", false, false, 4, dump_json_VkDisplayKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_XLIB_XRANDR_EXT
void dump_json_vkGetPhysicalDeviceSurfaceCapabilities2EXT(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, VkSurfaceKHR surface, VkSurfaceCapabilities2EXT* pSurfaceCapabilities)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSurfaceKHR>(surface, NULL, settings, "VkSurfaceKHR", "surface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceCapabilities2EXT>(pSurfaceCapabilities, settings, "VkSurfaceCapabilities2EXT*", "pSurfaceCapabilities", true, false, 4, dump_json_VkSurfaceCapabilities2EXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDisplayPowerControlEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDisplayKHR display, const VkDisplayPowerInfoEXT* pDisplayPowerInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDisplayKHR>(display, NULL, settings, "VkDisplayKHR", "display", false, false, 4, dump_json_VkDisplayKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDisplayPowerInfoEXT>(pDisplayPowerInfo, settings, "const VkDisplayPowerInfoEXT*", "pDisplayPowerInfo", true, false, 4, dump_json_VkDisplayPowerInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkRegisterDeviceEventEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkDeviceEventInfoEXT* pDeviceEventInfo, const VkAllocationCallbacks* pAllocator, VkFence* pFence)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceEventInfoEXT>(pDeviceEventInfo, settings, "const VkDeviceEventInfoEXT*", "pDeviceEventInfo", true, false, 4, dump_json_VkDeviceEventInfoEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkFence>(pFence, settings, "VkFence*", "pFence", false, false, 4, dump_json_VkFence);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkRegisterDisplayEventEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDisplayKHR display, const VkDisplayEventInfoEXT* pDisplayEventInfo, const VkAllocationCallbacks* pAllocator, VkFence* pFence)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDisplayKHR>(display, NULL, settings, "VkDisplayKHR", "display", false, false, 4, dump_json_VkDisplayKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDisplayEventInfoEXT>(pDisplayEventInfo, settings, "const VkDisplayEventInfoEXT*", "pDisplayEventInfo", true, false, 4, dump_json_VkDisplayEventInfoEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkFence>(pFence, settings, "VkFence*", "pFence", false, false, 4, dump_json_VkFence);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetSwapchainCounterEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkSwapchainKHR swapchain, VkSurfaceCounterFlagBitsEXT counter, uint64_t* pCounterValue)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSwapchainKHR>(swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << ",\n";
        dump_json_value<const VkSurfaceCounterFlagBitsEXT>(counter, NULL, settings, "VkSurfaceCounterFlagBitsEXT", "counter", false, false, 4, dump_json_VkSurfaceCounterFlagBitsEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const uint64_t>(pCounterValue, settings, "uint64_t*", "pCounterValue", false, false, 4, dump_json_uint64_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetRefreshCycleDurationGOOGLE(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkSwapchainKHR swapchain, VkRefreshCycleDurationGOOGLE* pDisplayTimingProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSwapchainKHR>(swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRefreshCycleDurationGOOGLE>(pDisplayTimingProperties, settings, "VkRefreshCycleDurationGOOGLE*", "pDisplayTimingProperties", true, false, 4, dump_json_VkRefreshCycleDurationGOOGLE);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPastPresentationTimingGOOGLE(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkSwapchainKHR swapchain, uint32_t* pPresentationTimingCount, VkPastPresentationTimingGOOGLE* pPresentationTimings)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSwapchainKHR>(swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPresentationTimingCount, settings, "uint32_t*", "pPresentationTimingCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkPastPresentationTimingGOOGLE>(pPresentationTimings, *pPresentationTimingCount, settings, "VkPastPresentationTimingGOOGLE*", "VkPastPresentationTimingGOOGLE", "pPresentationTimings", true, false, 4, dump_json_VkPastPresentationTimingGOOGLE); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDiscardRectangleEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstDiscardRectangle, uint32_t discardRectangleCount, const VkRect2D* pDiscardRectangles)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstDiscardRectangle, NULL, settings, "uint32_t", "firstDiscardRectangle", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(discardRectangleCount, NULL, settings, "uint32_t", "discardRectangleCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkRect2D>(pDiscardRectangles, discardRectangleCount, settings, "const VkRect2D*", "const VkRect2D", "pDiscardRectangles", true, false, 4, dump_json_VkRect2D); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDiscardRectangleEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 discardRectangleEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(discardRectangleEnable, NULL, settings, "VkBool32", "discardRectangleEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDiscardRectangleModeEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkDiscardRectangleModeEXT discardRectangleMode)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDiscardRectangleModeEXT>(discardRectangleMode, NULL, settings, "VkDiscardRectangleModeEXT", "discardRectangleMode", false, false, 4, dump_json_VkDiscardRectangleModeEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkSetHdrMetadataEXT(ApiDumpInstance& dump_inst, VkDevice device, uint32_t swapchainCount, const VkSwapchainKHR* pSwapchains, const VkHdrMetadataEXT* pMetadata)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(swapchainCount, NULL, settings, "uint32_t", "swapchainCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkSwapchainKHR>(pSwapchains, swapchainCount, settings, "const VkSwapchainKHR*", "const VkSwapchainKHR", "pSwapchains", false, false, 4, dump_json_VkSwapchainKHR); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkHdrMetadataEXT>(pMetadata, swapchainCount, settings, "const VkHdrMetadataEXT*", "const VkHdrMetadataEXT", "pMetadata", true, false, 4, dump_json_VkHdrMetadataEXT); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateRenderPass2KHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkRenderPassCreateInfo2* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkRenderPass* pRenderPass)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRenderPassCreateInfo2>(pCreateInfo, settings, "const VkRenderPassCreateInfo2*", "pCreateInfo", true, false, 4, dump_json_VkRenderPassCreateInfo2);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRenderPass>(pRenderPass, settings, "VkRenderPass*", "pRenderPass", false, false, 4, dump_json_VkRenderPass);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBeginRenderPass2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkRenderPassBeginInfo*      pRenderPassBegin, const VkSubpassBeginInfo*      pSubpassBeginInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRenderPassBeginInfo>(pRenderPassBegin, settings, "const VkRenderPassBeginInfo*", "pRenderPassBegin", true, false, 4, dump_json_VkRenderPassBeginInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSubpassBeginInfo>(pSubpassBeginInfo, settings, "const VkSubpassBeginInfo*", "pSubpassBeginInfo", true, false, 4, dump_json_VkSubpassBeginInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdNextSubpass2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkSubpassBeginInfo*      pSubpassBeginInfo, const VkSubpassEndInfo*        pSubpassEndInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSubpassBeginInfo>(pSubpassBeginInfo, settings, "const VkSubpassBeginInfo*", "pSubpassBeginInfo", true, false, 4, dump_json_VkSubpassBeginInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSubpassEndInfo>(pSubpassEndInfo, settings, "const VkSubpassEndInfo*", "pSubpassEndInfo", true, false, 4, dump_json_VkSubpassEndInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdEndRenderPass2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkSubpassEndInfo*        pSubpassEndInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSubpassEndInfo>(pSubpassEndInfo, settings, "const VkSubpassEndInfo*", "pSubpassEndInfo", true, false, 4, dump_json_VkSubpassEndInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetSwapchainStatusKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkSwapchainKHR swapchain)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSwapchainKHR>(swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceExternalFencePropertiesKHR(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, const VkPhysicalDeviceExternalFenceInfo* pExternalFenceInfo, VkExternalFenceProperties* pExternalFenceProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceExternalFenceInfo>(pExternalFenceInfo, settings, "const VkPhysicalDeviceExternalFenceInfo*", "pExternalFenceInfo", true, false, 4, dump_json_VkPhysicalDeviceExternalFenceInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkExternalFenceProperties>(pExternalFenceProperties, settings, "VkExternalFenceProperties*", "pExternalFenceProperties", true, false, 4, dump_json_VkExternalFenceProperties);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_vkImportFenceWin32HandleKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkImportFenceWin32HandleInfoKHR* pImportFenceWin32HandleInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImportFenceWin32HandleInfoKHR>(pImportFenceWin32HandleInfo, settings, "const VkImportFenceWin32HandleInfoKHR*", "pImportFenceWin32HandleInfo", true, false, 4, dump_json_VkImportFenceWin32HandleInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_vkGetFenceWin32HandleKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkFenceGetWin32HandleInfoKHR* pGetWin32HandleInfo, HANDLE* pHandle)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkFenceGetWin32HandleInfoKHR>(pGetWin32HandleInfo, settings, "const VkFenceGetWin32HandleInfoKHR*", "pGetWin32HandleInfo", true, false, 4, dump_json_VkFenceGetWin32HandleInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const HANDLE>(pHandle, settings, "HANDLE*", "pHandle", false, false, 4, dump_json_HANDLE);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WIN32_KHR
void dump_json_vkImportFenceFdKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkImportFenceFdInfoKHR* pImportFenceFdInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImportFenceFdInfoKHR>(pImportFenceFdInfo, settings, "const VkImportFenceFdInfoKHR*", "pImportFenceFdInfo", true, false, 4, dump_json_VkImportFenceFdInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetFenceFdKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkFenceGetFdInfoKHR* pGetFdInfo, int* pFd)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkFenceGetFdInfoKHR>(pGetFdInfo, settings, "const VkFenceGetFdInfoKHR*", "pGetFdInfo", true, false, 4, dump_json_VkFenceGetFdInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const int>(pFd, settings, "int*", "pFd", false, false, 4, dump_json_int);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkEnumeratePhysicalDeviceQueueFamilyPerformanceQueryCountersKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t queueFamilyIndex, uint32_t* pCounterCount, VkPerformanceCounterKHR* pCounters, VkPerformanceCounterDescriptionKHR* pCounterDescriptions)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(queueFamilyIndex, NULL, settings, "uint32_t", "queueFamilyIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pCounterCount, settings, "uint32_t*", "pCounterCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkPerformanceCounterKHR>(pCounters, *pCounterCount, settings, "VkPerformanceCounterKHR*", "VkPerformanceCounterKHR", "pCounters", true, false, 4, dump_json_VkPerformanceCounterKHR); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkPerformanceCounterDescriptionKHR>(pCounterDescriptions, *pCounterCount, settings, "VkPerformanceCounterDescriptionKHR*", "VkPerformanceCounterDescriptionKHR", "pCounterDescriptions", true, false, 4, dump_json_VkPerformanceCounterDescriptionKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceQueueFamilyPerformanceQueryPassesKHR(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, const VkQueryPoolPerformanceCreateInfoKHR* pPerformanceQueryCreateInfo, uint32_t* pNumPasses)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkQueryPoolPerformanceCreateInfoKHR>(pPerformanceQueryCreateInfo, settings, "const VkQueryPoolPerformanceCreateInfoKHR*", "pPerformanceQueryCreateInfo", true, false, 4, dump_json_VkQueryPoolPerformanceCreateInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pNumPasses, settings, "uint32_t*", "pNumPasses", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkAcquireProfilingLockKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkAcquireProfilingLockInfoKHR* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAcquireProfilingLockInfoKHR>(pInfo, settings, "const VkAcquireProfilingLockInfoKHR*", "pInfo", true, false, 4, dump_json_VkAcquireProfilingLockInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkReleaseProfilingLockKHR(ApiDumpInstance& dump_inst, VkDevice device)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceSurfaceCapabilities2KHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, const VkPhysicalDeviceSurfaceInfo2KHR* pSurfaceInfo, VkSurfaceCapabilities2KHR* pSurfaceCapabilities)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceSurfaceInfo2KHR>(pSurfaceInfo, settings, "const VkPhysicalDeviceSurfaceInfo2KHR*", "pSurfaceInfo", true, false, 4, dump_json_VkPhysicalDeviceSurfaceInfo2KHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceCapabilities2KHR>(pSurfaceCapabilities, settings, "VkSurfaceCapabilities2KHR*", "pSurfaceCapabilities", true, false, 4, dump_json_VkSurfaceCapabilities2KHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceSurfaceFormats2KHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, const VkPhysicalDeviceSurfaceInfo2KHR* pSurfaceInfo, uint32_t* pSurfaceFormatCount, VkSurfaceFormat2KHR* pSurfaceFormats)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceSurfaceInfo2KHR>(pSurfaceInfo, settings, "const VkPhysicalDeviceSurfaceInfo2KHR*", "pSurfaceInfo", true, false, 4, dump_json_VkPhysicalDeviceSurfaceInfo2KHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pSurfaceFormatCount, settings, "uint32_t*", "pSurfaceFormatCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkSurfaceFormat2KHR>(pSurfaceFormats, *pSurfaceFormatCount, settings, "VkSurfaceFormat2KHR*", "VkSurfaceFormat2KHR", "pSurfaceFormats", true, false, 4, dump_json_VkSurfaceFormat2KHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceDisplayProperties2KHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t* pPropertyCount, VkDisplayProperties2KHR* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPropertyCount, settings, "uint32_t*", "pPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkDisplayProperties2KHR>(pProperties, *pPropertyCount, settings, "VkDisplayProperties2KHR*", "VkDisplayProperties2KHR", "pProperties", true, false, 4, dump_json_VkDisplayProperties2KHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceDisplayPlaneProperties2KHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t* pPropertyCount, VkDisplayPlaneProperties2KHR* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPropertyCount, settings, "uint32_t*", "pPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkDisplayPlaneProperties2KHR>(pProperties, *pPropertyCount, settings, "VkDisplayPlaneProperties2KHR*", "VkDisplayPlaneProperties2KHR", "pProperties", true, false, 4, dump_json_VkDisplayPlaneProperties2KHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDisplayModeProperties2KHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, VkDisplayKHR display, uint32_t* pPropertyCount, VkDisplayModeProperties2KHR* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDisplayKHR>(display, NULL, settings, "VkDisplayKHR", "display", false, false, 4, dump_json_VkDisplayKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPropertyCount, settings, "uint32_t*", "pPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkDisplayModeProperties2KHR>(pProperties, *pPropertyCount, settings, "VkDisplayModeProperties2KHR*", "VkDisplayModeProperties2KHR", "pProperties", true, false, 4, dump_json_VkDisplayModeProperties2KHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDisplayPlaneCapabilities2KHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, const VkDisplayPlaneInfo2KHR* pDisplayPlaneInfo, VkDisplayPlaneCapabilities2KHR* pCapabilities)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDisplayPlaneInfo2KHR>(pDisplayPlaneInfo, settings, "const VkDisplayPlaneInfo2KHR*", "pDisplayPlaneInfo", true, false, 4, dump_json_VkDisplayPlaneInfo2KHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDisplayPlaneCapabilities2KHR>(pCapabilities, settings, "VkDisplayPlaneCapabilities2KHR*", "pCapabilities", true, false, 4, dump_json_VkDisplayPlaneCapabilities2KHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_IOS_MVK)
void dump_json_vkCreateIOSSurfaceMVK(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkIOSSurfaceCreateInfoMVK* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSurfaceKHR* pSurface)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkIOSSurfaceCreateInfoMVK>(pCreateInfo, settings, "const VkIOSSurfaceCreateInfoMVK*", "pCreateInfo", true, false, 4, dump_json_VkIOSSurfaceCreateInfoMVK);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceKHR>(pSurface, settings, "VkSurfaceKHR*", "pSurface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_IOS_MVK
#if defined(VK_USE_PLATFORM_MACOS_MVK)
void dump_json_vkCreateMacOSSurfaceMVK(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkMacOSSurfaceCreateInfoMVK* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSurfaceKHR* pSurface)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMacOSSurfaceCreateInfoMVK>(pCreateInfo, settings, "const VkMacOSSurfaceCreateInfoMVK*", "pCreateInfo", true, false, 4, dump_json_VkMacOSSurfaceCreateInfoMVK);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceKHR>(pSurface, settings, "VkSurfaceKHR*", "pSurface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_MACOS_MVK
void dump_json_vkSetDebugUtilsObjectNameEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkDebugUtilsObjectNameInfoEXT* pNameInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDebugUtilsObjectNameInfoEXT>(pNameInfo, settings, "const VkDebugUtilsObjectNameInfoEXT*", "pNameInfo", true, false, 4, dump_json_VkDebugUtilsObjectNameInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkSetDebugUtilsObjectTagEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkDebugUtilsObjectTagInfoEXT* pTagInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDebugUtilsObjectTagInfoEXT>(pTagInfo, settings, "const VkDebugUtilsObjectTagInfoEXT*", "pTagInfo", true, false, 4, dump_json_VkDebugUtilsObjectTagInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkQueueBeginDebugUtilsLabelEXT(ApiDumpInstance& dump_inst, VkQueue queue, const VkDebugUtilsLabelEXT* pLabelInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkQueue>(queue, NULL, settings, "VkQueue", "queue", false, false, 4, dump_json_VkQueue);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDebugUtilsLabelEXT>(pLabelInfo, settings, "const VkDebugUtilsLabelEXT*", "pLabelInfo", true, false, 4, dump_json_VkDebugUtilsLabelEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkQueueEndDebugUtilsLabelEXT(ApiDumpInstance& dump_inst, VkQueue queue)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkQueue>(queue, NULL, settings, "VkQueue", "queue", false, false, 4, dump_json_VkQueue);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkQueueInsertDebugUtilsLabelEXT(ApiDumpInstance& dump_inst, VkQueue queue, const VkDebugUtilsLabelEXT* pLabelInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkQueue>(queue, NULL, settings, "VkQueue", "queue", false, false, 4, dump_json_VkQueue);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDebugUtilsLabelEXT>(pLabelInfo, settings, "const VkDebugUtilsLabelEXT*", "pLabelInfo", true, false, 4, dump_json_VkDebugUtilsLabelEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBeginDebugUtilsLabelEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkDebugUtilsLabelEXT* pLabelInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDebugUtilsLabelEXT>(pLabelInfo, settings, "const VkDebugUtilsLabelEXT*", "pLabelInfo", true, false, 4, dump_json_VkDebugUtilsLabelEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdEndDebugUtilsLabelEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdInsertDebugUtilsLabelEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkDebugUtilsLabelEXT* pLabelInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDebugUtilsLabelEXT>(pLabelInfo, settings, "const VkDebugUtilsLabelEXT*", "pLabelInfo", true, false, 4, dump_json_VkDebugUtilsLabelEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateDebugUtilsMessengerEXT(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkDebugUtilsMessengerCreateInfoEXT* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkDebugUtilsMessengerEXT* pMessenger)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDebugUtilsMessengerCreateInfoEXT>(pCreateInfo, settings, "const VkDebugUtilsMessengerCreateInfoEXT*", "pCreateInfo", true, false, 4, dump_json_VkDebugUtilsMessengerCreateInfoEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDebugUtilsMessengerEXT>(pMessenger, settings, "VkDebugUtilsMessengerEXT*", "pMessenger", false, false, 4, dump_json_VkDebugUtilsMessengerEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyDebugUtilsMessengerEXT(ApiDumpInstance& dump_inst, VkInstance instance, VkDebugUtilsMessengerEXT messenger, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_value<const VkDebugUtilsMessengerEXT>(messenger, NULL, settings, "VkDebugUtilsMessengerEXT", "messenger", false, false, 4, dump_json_VkDebugUtilsMessengerEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkSubmitDebugUtilsMessageEXT(ApiDumpInstance& dump_inst, VkInstance instance, VkDebugUtilsMessageSeverityFlagBitsEXT messageSeverity, VkDebugUtilsMessageTypeFlagsEXT messageTypes, const VkDebugUtilsMessengerCallbackDataEXT* pCallbackData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_value<const VkDebugUtilsMessageSeverityFlagBitsEXT>(messageSeverity, NULL, settings, "VkDebugUtilsMessageSeverityFlagBitsEXT", "messageSeverity", false, false, 4, dump_json_VkDebugUtilsMessageSeverityFlagBitsEXT);
        settings.stream() << ",\n";
        dump_json_value<const VkDebugUtilsMessageTypeFlagsEXT>(messageTypes, NULL, settings, "VkDebugUtilsMessageTypeFlagsEXT", "messageTypes", false, false, 4, dump_json_VkDebugUtilsMessageTypeFlagsEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDebugUtilsMessengerCallbackDataEXT>(pCallbackData, settings, "const VkDebugUtilsMessengerCallbackDataEXT*", "pCallbackData", true, false, 4, dump_json_VkDebugUtilsMessengerCallbackDataEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_vkGetAndroidHardwareBufferPropertiesANDROID(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const struct AHardwareBuffer* buffer, VkAndroidHardwareBufferPropertiesANDROID* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const AHardwareBuffer*>(buffer, NULL, settings, "const struct AHardwareBuffer*", "buffer", false, false, 4, dump_json_AHardwareBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAndroidHardwareBufferPropertiesANDROID>(pProperties, settings, "VkAndroidHardwareBufferPropertiesANDROID*", "pProperties", true, false, 4, dump_json_VkAndroidHardwareBufferPropertiesANDROID);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_USE_PLATFORM_ANDROID_KHR)
void dump_json_vkGetMemoryAndroidHardwareBufferANDROID(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkMemoryGetAndroidHardwareBufferInfoANDROID* pInfo, struct AHardwareBuffer** pBuffer)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryGetAndroidHardwareBufferInfoANDROID>(pInfo, settings, "const VkMemoryGetAndroidHardwareBufferInfoANDROID*", "pInfo", true, false, 4, dump_json_VkMemoryGetAndroidHardwareBufferInfoANDROID);
        settings.stream() << ",\n";
        dump_json_pointer<const AHardwareBuffer*>(pBuffer, settings, "struct AHardwareBuffer**", "pBuffer", false, false, 4, dump_json_AHardwareBuffer);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_ANDROID_KHR
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_vkCreateExecutionGraphPipelinesAMDX(ApiDumpInstance& dump_inst, VkResult result, VkDevice                                        device, VkPipelineCache                 pipelineCache, uint32_t                                        createInfoCount, const VkExecutionGraphPipelineCreateInfoAMDX* pCreateInfos, const VkAllocationCallbacks*    pAllocator, VkPipeline*               pPipelines)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineCache>(pipelineCache, NULL, settings, "VkPipelineCache", "pipelineCache", false, false, 4, dump_json_VkPipelineCache);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(createInfoCount, NULL, settings, "uint32_t", "createInfoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkExecutionGraphPipelineCreateInfoAMDX>(pCreateInfos, createInfoCount, settings, "const VkExecutionGraphPipelineCreateInfoAMDX*", "const VkExecutionGraphPipelineCreateInfoAMDX", "pCreateInfos", true, false, 4, dump_json_VkExecutionGraphPipelineCreateInfoAMDX); // PQA
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_array<const VkPipeline>(pPipelines, createInfoCount, settings, "VkPipeline*", "VkPipeline", "pPipelines", false, false, 4, dump_json_VkPipeline); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_vkGetExecutionGraphPipelineScratchSizeAMDX(ApiDumpInstance& dump_inst, VkResult result, VkDevice                                        device, VkPipeline                                      executionGraph, VkExecutionGraphPipelineScratchSizeAMDX*         pSizeInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipeline>(executionGraph, NULL, settings, "VkPipeline", "executionGraph", false, false, 4, dump_json_VkPipeline);
        settings.stream() << ",\n";
        dump_json_pointer<const VkExecutionGraphPipelineScratchSizeAMDX>(pSizeInfo, settings, "VkExecutionGraphPipelineScratchSizeAMDX*", "pSizeInfo", true, false, 4, dump_json_VkExecutionGraphPipelineScratchSizeAMDX);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_vkGetExecutionGraphPipelineNodeIndexAMDX(ApiDumpInstance& dump_inst, VkResult result, VkDevice                                        device, VkPipeline                                      executionGraph, const VkPipelineShaderStageNodeCreateInfoAMDX*   pNodeInfo, uint32_t*                                       pNodeIndex)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipeline>(executionGraph, NULL, settings, "VkPipeline", "executionGraph", false, false, 4, dump_json_VkPipeline);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPipelineShaderStageNodeCreateInfoAMDX>(pNodeInfo, settings, "const VkPipelineShaderStageNodeCreateInfoAMDX*", "pNodeInfo", true, false, 4, dump_json_VkPipelineShaderStageNodeCreateInfoAMDX);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pNodeIndex, settings, "uint32_t*", "pNodeIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_vkCmdInitializeGraphScratchMemoryAMDX(ApiDumpInstance& dump_inst, VkCommandBuffer                                 commandBuffer, VkDeviceAddress                                 scratch)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceAddress>(scratch, NULL, settings, "VkDeviceAddress", "scratch", false, false, 4, dump_json_VkDeviceAddress);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_vkCmdDispatchGraphAMDX(ApiDumpInstance& dump_inst, VkCommandBuffer                                 commandBuffer, VkDeviceAddress                                 scratch, const VkDispatchGraphCountInfoAMDX*              pCountInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceAddress>(scratch, NULL, settings, "VkDeviceAddress", "scratch", false, false, 4, dump_json_VkDeviceAddress);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDispatchGraphCountInfoAMDX>(pCountInfo, settings, "const VkDispatchGraphCountInfoAMDX*", "pCountInfo", true, false, 4, dump_json_VkDispatchGraphCountInfoAMDX);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_vkCmdDispatchGraphIndirectAMDX(ApiDumpInstance& dump_inst, VkCommandBuffer                                 commandBuffer, VkDeviceAddress                                 scratch, const VkDispatchGraphCountInfoAMDX*              pCountInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceAddress>(scratch, NULL, settings, "VkDeviceAddress", "scratch", false, false, 4, dump_json_VkDeviceAddress);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDispatchGraphCountInfoAMDX>(pCountInfo, settings, "const VkDispatchGraphCountInfoAMDX*", "pCountInfo", true, false, 4, dump_json_VkDispatchGraphCountInfoAMDX);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_ENABLE_BETA_EXTENSIONS
#if defined(VK_ENABLE_BETA_EXTENSIONS)
void dump_json_vkCmdDispatchGraphIndirectCountAMDX(ApiDumpInstance& dump_inst, VkCommandBuffer                                 commandBuffer, VkDeviceAddress                                 scratch, VkDeviceAddress                                 countInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceAddress>(scratch, NULL, settings, "VkDeviceAddress", "scratch", false, false, 4, dump_json_VkDeviceAddress);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceAddress>(countInfo, NULL, settings, "VkDeviceAddress", "countInfo", false, false, 4, dump_json_VkDeviceAddress);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_ENABLE_BETA_EXTENSIONS
void dump_json_vkCmdSetSampleLocationsEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkSampleLocationsInfoEXT* pSampleLocationsInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSampleLocationsInfoEXT>(pSampleLocationsInfo, settings, "const VkSampleLocationsInfoEXT*", "pSampleLocationsInfo", true, false, 4, dump_json_VkSampleLocationsInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceMultisamplePropertiesEXT(ApiDumpInstance& dump_inst, VkPhysicalDevice physicalDevice, VkSampleCountFlagBits samples, VkMultisamplePropertiesEXT* pMultisampleProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSampleCountFlagBits>(samples, NULL, settings, "VkSampleCountFlagBits", "samples", false, false, 4, dump_json_VkSampleCountFlagBits);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMultisamplePropertiesEXT>(pMultisampleProperties, settings, "VkMultisamplePropertiesEXT*", "pMultisampleProperties", true, false, 4, dump_json_VkMultisamplePropertiesEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetImageMemoryRequirements2KHR(ApiDumpInstance& dump_inst, VkDevice device, const VkImageMemoryRequirementsInfo2* pInfo, VkMemoryRequirements2* pMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageMemoryRequirementsInfo2>(pInfo, settings, "const VkImageMemoryRequirementsInfo2*", "pInfo", true, false, 4, dump_json_VkImageMemoryRequirementsInfo2);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryRequirements2>(pMemoryRequirements, settings, "VkMemoryRequirements2*", "pMemoryRequirements", true, false, 4, dump_json_VkMemoryRequirements2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetBufferMemoryRequirements2KHR(ApiDumpInstance& dump_inst, VkDevice device, const VkBufferMemoryRequirementsInfo2* pInfo, VkMemoryRequirements2* pMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBufferMemoryRequirementsInfo2>(pInfo, settings, "const VkBufferMemoryRequirementsInfo2*", "pInfo", true, false, 4, dump_json_VkBufferMemoryRequirementsInfo2);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryRequirements2>(pMemoryRequirements, settings, "VkMemoryRequirements2*", "pMemoryRequirements", true, false, 4, dump_json_VkMemoryRequirements2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetImageSparseMemoryRequirements2KHR(ApiDumpInstance& dump_inst, VkDevice device, const VkImageSparseMemoryRequirementsInfo2* pInfo, uint32_t* pSparseMemoryRequirementCount, VkSparseImageMemoryRequirements2* pSparseMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageSparseMemoryRequirementsInfo2>(pInfo, settings, "const VkImageSparseMemoryRequirementsInfo2*", "pInfo", true, false, 4, dump_json_VkImageSparseMemoryRequirementsInfo2);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pSparseMemoryRequirementCount, settings, "uint32_t*", "pSparseMemoryRequirementCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkSparseImageMemoryRequirements2>(pSparseMemoryRequirements, *pSparseMemoryRequirementCount, settings, "VkSparseImageMemoryRequirements2*", "VkSparseImageMemoryRequirements2", "pSparseMemoryRequirements", true, false, 4, dump_json_VkSparseImageMemoryRequirements2); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateAccelerationStructureKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice                                           device, const VkAccelerationStructureCreateInfoKHR*        pCreateInfo, const VkAllocationCallbacks*       pAllocator, VkAccelerationStructureKHR*                        pAccelerationStructure)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAccelerationStructureCreateInfoKHR>(pCreateInfo, settings, "const VkAccelerationStructureCreateInfoKHR*", "pCreateInfo", true, false, 4, dump_json_VkAccelerationStructureCreateInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAccelerationStructureKHR>(pAccelerationStructure, settings, "VkAccelerationStructureKHR*", "pAccelerationStructure", false, false, 4, dump_json_VkAccelerationStructureKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyAccelerationStructureKHR(ApiDumpInstance& dump_inst, VkDevice device, VkAccelerationStructureKHR accelerationStructure, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkAccelerationStructureKHR>(accelerationStructure, NULL, settings, "VkAccelerationStructureKHR", "accelerationStructure", false, false, 4, dump_json_VkAccelerationStructureKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBuildAccelerationStructuresKHR(ApiDumpInstance& dump_inst, VkCommandBuffer                                    commandBuffer, uint32_t infoCount, const VkAccelerationStructureBuildGeometryInfoKHR* pInfos, const VkAccelerationStructureBuildRangeInfoKHR* const* ppBuildRangeInfos)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(infoCount, NULL, settings, "uint32_t", "infoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkAccelerationStructureBuildGeometryInfoKHR>(pInfos, infoCount, settings, "const VkAccelerationStructureBuildGeometryInfoKHR*", "const VkAccelerationStructureBuildGeometryInfoKHR", "pInfos", true, false, 4, dump_json_VkAccelerationStructureBuildGeometryInfoKHR); // PQA
        settings.stream() << ",\n";
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBuildAccelerationStructuresIndirectKHR(ApiDumpInstance& dump_inst, VkCommandBuffer                  commandBuffer, uint32_t                                           infoCount, const VkAccelerationStructureBuildGeometryInfoKHR* pInfos, const VkDeviceAddress*             pIndirectDeviceAddresses, const uint32_t*                    pIndirectStrides, const uint32_t* const*             ppMaxPrimitiveCounts)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(infoCount, NULL, settings, "uint32_t", "infoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkAccelerationStructureBuildGeometryInfoKHR>(pInfos, infoCount, settings, "const VkAccelerationStructureBuildGeometryInfoKHR*", "const VkAccelerationStructureBuildGeometryInfoKHR", "pInfos", true, false, 4, dump_json_VkAccelerationStructureBuildGeometryInfoKHR); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkDeviceAddress>(pIndirectDeviceAddresses, infoCount, settings, "const VkDeviceAddress*", "const VkDeviceAddress", "pIndirectDeviceAddresses", false, false, 4, dump_json_VkDeviceAddress); // PQA
        settings.stream() << ",\n";
        dump_json_array<const uint32_t>(pIndirectStrides, infoCount, settings, "const uint32_t*", "const uint32_t", "pIndirectStrides", false, false, 4, dump_json_uint32_t); // PQA
        settings.stream() << ",\n";
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkBuildAccelerationStructuresKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice                                           device, VkDeferredOperationKHR deferredOperation, uint32_t infoCount, const VkAccelerationStructureBuildGeometryInfoKHR* pInfos, const VkAccelerationStructureBuildRangeInfoKHR* const* ppBuildRangeInfos)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeferredOperationKHR>(deferredOperation, NULL, settings, "VkDeferredOperationKHR", "deferredOperation", false, false, 4, dump_json_VkDeferredOperationKHR);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(infoCount, NULL, settings, "uint32_t", "infoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkAccelerationStructureBuildGeometryInfoKHR>(pInfos, infoCount, settings, "const VkAccelerationStructureBuildGeometryInfoKHR*", "const VkAccelerationStructureBuildGeometryInfoKHR", "pInfos", true, false, 4, dump_json_VkAccelerationStructureBuildGeometryInfoKHR); // PQA
        settings.stream() << ",\n";
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCopyAccelerationStructureKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDeferredOperationKHR deferredOperation, const VkCopyAccelerationStructureInfoKHR* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeferredOperationKHR>(deferredOperation, NULL, settings, "VkDeferredOperationKHR", "deferredOperation", false, false, 4, dump_json_VkDeferredOperationKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyAccelerationStructureInfoKHR>(pInfo, settings, "const VkCopyAccelerationStructureInfoKHR*", "pInfo", true, false, 4, dump_json_VkCopyAccelerationStructureInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCopyAccelerationStructureToMemoryKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDeferredOperationKHR deferredOperation, const VkCopyAccelerationStructureToMemoryInfoKHR* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeferredOperationKHR>(deferredOperation, NULL, settings, "VkDeferredOperationKHR", "deferredOperation", false, false, 4, dump_json_VkDeferredOperationKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyAccelerationStructureToMemoryInfoKHR>(pInfo, settings, "const VkCopyAccelerationStructureToMemoryInfoKHR*", "pInfo", true, false, 4, dump_json_VkCopyAccelerationStructureToMemoryInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCopyMemoryToAccelerationStructureKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDeferredOperationKHR deferredOperation, const VkCopyMemoryToAccelerationStructureInfoKHR* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeferredOperationKHR>(deferredOperation, NULL, settings, "VkDeferredOperationKHR", "deferredOperation", false, false, 4, dump_json_VkDeferredOperationKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyMemoryToAccelerationStructureInfoKHR>(pInfo, settings, "const VkCopyMemoryToAccelerationStructureInfoKHR*", "pInfo", true, false, 4, dump_json_VkCopyMemoryToAccelerationStructureInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkWriteAccelerationStructuresPropertiesKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t accelerationStructureCount, const VkAccelerationStructureKHR* pAccelerationStructures, VkQueryType  queryType, size_t       dataSize, void* pData, size_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(accelerationStructureCount, NULL, settings, "uint32_t", "accelerationStructureCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkAccelerationStructureKHR>(pAccelerationStructures, accelerationStructureCount, settings, "const VkAccelerationStructureKHR*", "const VkAccelerationStructureKHR", "pAccelerationStructures", false, false, 4, dump_json_VkAccelerationStructureKHR); // PQA
        settings.stream() << ",\n";
        dump_json_value<const VkQueryType>(queryType, NULL, settings, "VkQueryType", "queryType", false, false, 4, dump_json_VkQueryType);
        settings.stream() << ",\n";
        dump_json_value<const size_t>(dataSize, NULL, settings, "size_t", "dataSize", false, false, 4, dump_json_size_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << ",\n";
        dump_json_value<const size_t>(stride, NULL, settings, "size_t", "stride", false, false, 4, dump_json_size_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyAccelerationStructureKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCopyAccelerationStructureInfoKHR* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyAccelerationStructureInfoKHR>(pInfo, settings, "const VkCopyAccelerationStructureInfoKHR*", "pInfo", true, false, 4, dump_json_VkCopyAccelerationStructureInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyAccelerationStructureToMemoryKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCopyAccelerationStructureToMemoryInfoKHR* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyAccelerationStructureToMemoryInfoKHR>(pInfo, settings, "const VkCopyAccelerationStructureToMemoryInfoKHR*", "pInfo", true, false, 4, dump_json_VkCopyAccelerationStructureToMemoryInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyMemoryToAccelerationStructureKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCopyMemoryToAccelerationStructureInfoKHR* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyMemoryToAccelerationStructureInfoKHR>(pInfo, settings, "const VkCopyMemoryToAccelerationStructureInfoKHR*", "pInfo", true, false, 4, dump_json_VkCopyMemoryToAccelerationStructureInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetAccelerationStructureDeviceAddressKHR(ApiDumpInstance& dump_inst, VkDeviceAddress result, VkDevice device, const VkAccelerationStructureDeviceAddressInfoKHR* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkDeviceAddress(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAccelerationStructureDeviceAddressInfoKHR>(pInfo, settings, "const VkAccelerationStructureDeviceAddressInfoKHR*", "pInfo", true, false, 4, dump_json_VkAccelerationStructureDeviceAddressInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdWriteAccelerationStructuresPropertiesKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t accelerationStructureCount, const VkAccelerationStructureKHR* pAccelerationStructures, VkQueryType queryType, VkQueryPool queryPool, uint32_t firstQuery)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(accelerationStructureCount, NULL, settings, "uint32_t", "accelerationStructureCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkAccelerationStructureKHR>(pAccelerationStructures, accelerationStructureCount, settings, "const VkAccelerationStructureKHR*", "const VkAccelerationStructureKHR", "pAccelerationStructures", false, false, 4, dump_json_VkAccelerationStructureKHR); // PQA
        settings.stream() << ",\n";
        dump_json_value<const VkQueryType>(queryType, NULL, settings, "VkQueryType", "queryType", false, false, 4, dump_json_VkQueryType);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstQuery, NULL, settings, "uint32_t", "firstQuery", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceAccelerationStructureCompatibilityKHR(ApiDumpInstance& dump_inst, VkDevice device, const VkAccelerationStructureVersionInfoKHR* pVersionInfo, VkAccelerationStructureCompatibilityKHR* pCompatibility)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAccelerationStructureVersionInfoKHR>(pVersionInfo, settings, "const VkAccelerationStructureVersionInfoKHR*", "pVersionInfo", true, false, 4, dump_json_VkAccelerationStructureVersionInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAccelerationStructureCompatibilityKHR>(pCompatibility, settings, "VkAccelerationStructureCompatibilityKHR*", "pCompatibility", false, false, 4, dump_json_VkAccelerationStructureCompatibilityKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetAccelerationStructureBuildSizesKHR(ApiDumpInstance& dump_inst, VkDevice                                            device, VkAccelerationStructureBuildTypeKHR                 buildType, const VkAccelerationStructureBuildGeometryInfoKHR*  pBuildInfo, const uint32_t*  pMaxPrimitiveCounts, VkAccelerationStructureBuildSizesInfoKHR*           pSizeInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkAccelerationStructureBuildTypeKHR>(buildType, NULL, settings, "VkAccelerationStructureBuildTypeKHR", "buildType", false, false, 4, dump_json_VkAccelerationStructureBuildTypeKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAccelerationStructureBuildGeometryInfoKHR>(pBuildInfo, settings, "const VkAccelerationStructureBuildGeometryInfoKHR*", "pBuildInfo", true, false, 4, dump_json_VkAccelerationStructureBuildGeometryInfoKHR);
        settings.stream() << ",\n";
        dump_json_array<const uint32_t>(pMaxPrimitiveCounts, pBuildInfo->geometryCount, settings, "const uint32_t*", "const uint32_t", "pMaxPrimitiveCounts", false, false, 4, dump_json_uint32_t); // PQA
        settings.stream() << ",\n";
        dump_json_pointer<const VkAccelerationStructureBuildSizesInfoKHR>(pSizeInfo, settings, "VkAccelerationStructureBuildSizesInfoKHR*", "pSizeInfo", true, false, 4, dump_json_VkAccelerationStructureBuildSizesInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateSamplerYcbcrConversionKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkSamplerYcbcrConversionCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSamplerYcbcrConversion* pYcbcrConversion)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSamplerYcbcrConversionCreateInfo>(pCreateInfo, settings, "const VkSamplerYcbcrConversionCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkSamplerYcbcrConversionCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSamplerYcbcrConversion>(pYcbcrConversion, settings, "VkSamplerYcbcrConversion*", "pYcbcrConversion", false, false, 4, dump_json_VkSamplerYcbcrConversion);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroySamplerYcbcrConversionKHR(ApiDumpInstance& dump_inst, VkDevice device, VkSamplerYcbcrConversion ycbcrConversion, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSamplerYcbcrConversion>(ycbcrConversion, NULL, settings, "VkSamplerYcbcrConversion", "ycbcrConversion", false, false, 4, dump_json_VkSamplerYcbcrConversion);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkBindBufferMemory2KHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t bindInfoCount, const VkBindBufferMemoryInfo* pBindInfos)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(bindInfoCount, NULL, settings, "uint32_t", "bindInfoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBindBufferMemoryInfo>(pBindInfos, bindInfoCount, settings, "const VkBindBufferMemoryInfo*", "const VkBindBufferMemoryInfo", "pBindInfos", true, false, 4, dump_json_VkBindBufferMemoryInfo); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkBindImageMemory2KHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t bindInfoCount, const VkBindImageMemoryInfo* pBindInfos)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(bindInfoCount, NULL, settings, "uint32_t", "bindInfoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBindImageMemoryInfo>(pBindInfos, bindInfoCount, settings, "const VkBindImageMemoryInfo*", "const VkBindImageMemoryInfo", "pBindInfos", true, false, 4, dump_json_VkBindImageMemoryInfo); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetImageDrmFormatModifierPropertiesEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkImage image, VkImageDrmFormatModifierPropertiesEXT* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(image, NULL, settings, "VkImage", "image", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageDrmFormatModifierPropertiesEXT>(pProperties, settings, "VkImageDrmFormatModifierPropertiesEXT*", "pProperties", true, false, 4, dump_json_VkImageDrmFormatModifierPropertiesEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateValidationCacheEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkValidationCacheCreateInfoEXT* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkValidationCacheEXT* pValidationCache)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkValidationCacheCreateInfoEXT>(pCreateInfo, settings, "const VkValidationCacheCreateInfoEXT*", "pCreateInfo", true, false, 4, dump_json_VkValidationCacheCreateInfoEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkValidationCacheEXT>(pValidationCache, settings, "VkValidationCacheEXT*", "pValidationCache", false, false, 4, dump_json_VkValidationCacheEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyValidationCacheEXT(ApiDumpInstance& dump_inst, VkDevice device, VkValidationCacheEXT validationCache, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkValidationCacheEXT>(validationCache, NULL, settings, "VkValidationCacheEXT", "validationCache", false, false, 4, dump_json_VkValidationCacheEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkMergeValidationCachesEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkValidationCacheEXT dstCache, uint32_t srcCacheCount, const VkValidationCacheEXT* pSrcCaches)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkValidationCacheEXT>(dstCache, NULL, settings, "VkValidationCacheEXT", "dstCache", false, false, 4, dump_json_VkValidationCacheEXT);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(srcCacheCount, NULL, settings, "uint32_t", "srcCacheCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkValidationCacheEXT>(pSrcCaches, srcCacheCount, settings, "const VkValidationCacheEXT*", "const VkValidationCacheEXT", "pSrcCaches", false, false, 4, dump_json_VkValidationCacheEXT); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetValidationCacheDataEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkValidationCacheEXT validationCache, size_t* pDataSize, void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkValidationCacheEXT>(validationCache, NULL, settings, "VkValidationCacheEXT", "validationCache", false, false, 4, dump_json_VkValidationCacheEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const size_t>(pDataSize, settings, "size_t*", "pDataSize", false, false, 4, dump_json_size_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBindShadingRateImageNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkImageView imageView, VkImageLayout imageLayout)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkImageView>(imageView, NULL, settings, "VkImageView", "imageView", false, false, 4, dump_json_VkImageView);
        settings.stream() << ",\n";
        dump_json_value<const VkImageLayout>(imageLayout, NULL, settings, "VkImageLayout", "imageLayout", false, false, 4, dump_json_VkImageLayout);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetViewportShadingRatePaletteNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstViewport, uint32_t viewportCount, const VkShadingRatePaletteNV* pShadingRatePalettes)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstViewport, NULL, settings, "uint32_t", "firstViewport", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(viewportCount, NULL, settings, "uint32_t", "viewportCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkShadingRatePaletteNV>(pShadingRatePalettes, viewportCount, settings, "const VkShadingRatePaletteNV*", "const VkShadingRatePaletteNV", "pShadingRatePalettes", true, false, 4, dump_json_VkShadingRatePaletteNV); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetCoarseSampleOrderNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkCoarseSampleOrderTypeNV sampleOrderType, uint32_t customSampleOrderCount, const VkCoarseSampleOrderCustomNV* pCustomSampleOrders)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkCoarseSampleOrderTypeNV>(sampleOrderType, NULL, settings, "VkCoarseSampleOrderTypeNV", "sampleOrderType", false, false, 4, dump_json_VkCoarseSampleOrderTypeNV);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(customSampleOrderCount, NULL, settings, "uint32_t", "customSampleOrderCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkCoarseSampleOrderCustomNV>(pCustomSampleOrders, customSampleOrderCount, settings, "const VkCoarseSampleOrderCustomNV*", "const VkCoarseSampleOrderCustomNV", "pCustomSampleOrders", true, false, 4, dump_json_VkCoarseSampleOrderCustomNV); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateAccelerationStructureNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkAccelerationStructureCreateInfoNV* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkAccelerationStructureNV* pAccelerationStructure)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAccelerationStructureCreateInfoNV>(pCreateInfo, settings, "const VkAccelerationStructureCreateInfoNV*", "pCreateInfo", true, false, 4, dump_json_VkAccelerationStructureCreateInfoNV);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAccelerationStructureNV>(pAccelerationStructure, settings, "VkAccelerationStructureNV*", "pAccelerationStructure", false, false, 4, dump_json_VkAccelerationStructureNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyAccelerationStructureNV(ApiDumpInstance& dump_inst, VkDevice device, VkAccelerationStructureNV accelerationStructure, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkAccelerationStructureNV>(accelerationStructure, NULL, settings, "VkAccelerationStructureNV", "accelerationStructure", false, false, 4, dump_json_VkAccelerationStructureNV);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetAccelerationStructureMemoryRequirementsNV(ApiDumpInstance& dump_inst, VkDevice device, const VkAccelerationStructureMemoryRequirementsInfoNV* pInfo, VkMemoryRequirements2KHR* pMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAccelerationStructureMemoryRequirementsInfoNV>(pInfo, settings, "const VkAccelerationStructureMemoryRequirementsInfoNV*", "pInfo", true, false, 4, dump_json_VkAccelerationStructureMemoryRequirementsInfoNV);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryRequirements2>(pMemoryRequirements, settings, "VkMemoryRequirements2KHR*", "pMemoryRequirements", true, false, 4, dump_json_VkMemoryRequirements2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkBindAccelerationStructureMemoryNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t bindInfoCount, const VkBindAccelerationStructureMemoryInfoNV* pBindInfos)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(bindInfoCount, NULL, settings, "uint32_t", "bindInfoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBindAccelerationStructureMemoryInfoNV>(pBindInfos, bindInfoCount, settings, "const VkBindAccelerationStructureMemoryInfoNV*", "const VkBindAccelerationStructureMemoryInfoNV", "pBindInfos", true, false, 4, dump_json_VkBindAccelerationStructureMemoryInfoNV); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBuildAccelerationStructureNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkAccelerationStructureInfoNV* pInfo, VkBuffer instanceData, VkDeviceSize instanceOffset, VkBool32 update, VkAccelerationStructureNV dst, VkAccelerationStructureNV src, VkBuffer scratch, VkDeviceSize scratchOffset)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAccelerationStructureInfoNV>(pInfo, settings, "const VkAccelerationStructureInfoNV*", "pInfo", true, false, 4, dump_json_VkAccelerationStructureInfoNV);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(instanceData, NULL, settings, "VkBuffer", "instanceData", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(instanceOffset, NULL, settings, "VkDeviceSize", "instanceOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(update, NULL, settings, "VkBool32", "update", false, false, 4, dump_json_VkBool32);
        settings.stream() << ",\n";
        dump_json_value<const VkAccelerationStructureNV>(dst, NULL, settings, "VkAccelerationStructureNV", "dst", false, false, 4, dump_json_VkAccelerationStructureNV);
        settings.stream() << ",\n";
        dump_json_value<const VkAccelerationStructureNV>(src, NULL, settings, "VkAccelerationStructureNV", "src", false, false, 4, dump_json_VkAccelerationStructureNV);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(scratch, NULL, settings, "VkBuffer", "scratch", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(scratchOffset, NULL, settings, "VkDeviceSize", "scratchOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyAccelerationStructureNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkAccelerationStructureNV dst, VkAccelerationStructureNV src, VkCopyAccelerationStructureModeKHR mode)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkAccelerationStructureNV>(dst, NULL, settings, "VkAccelerationStructureNV", "dst", false, false, 4, dump_json_VkAccelerationStructureNV);
        settings.stream() << ",\n";
        dump_json_value<const VkAccelerationStructureNV>(src, NULL, settings, "VkAccelerationStructureNV", "src", false, false, 4, dump_json_VkAccelerationStructureNV);
        settings.stream() << ",\n";
        dump_json_value<const VkCopyAccelerationStructureModeKHR>(mode, NULL, settings, "VkCopyAccelerationStructureModeKHR", "mode", false, false, 4, dump_json_VkCopyAccelerationStructureModeKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdTraceRaysNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer raygenShaderBindingTableBuffer, VkDeviceSize raygenShaderBindingOffset, VkBuffer missShaderBindingTableBuffer, VkDeviceSize missShaderBindingOffset, VkDeviceSize missShaderBindingStride, VkBuffer hitShaderBindingTableBuffer, VkDeviceSize hitShaderBindingOffset, VkDeviceSize hitShaderBindingStride, VkBuffer callableShaderBindingTableBuffer, VkDeviceSize callableShaderBindingOffset, VkDeviceSize callableShaderBindingStride, uint32_t width, uint32_t height, uint32_t depth)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(raygenShaderBindingTableBuffer, NULL, settings, "VkBuffer", "raygenShaderBindingTableBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(raygenShaderBindingOffset, NULL, settings, "VkDeviceSize", "raygenShaderBindingOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(missShaderBindingTableBuffer, NULL, settings, "VkBuffer", "missShaderBindingTableBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(missShaderBindingOffset, NULL, settings, "VkDeviceSize", "missShaderBindingOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(missShaderBindingStride, NULL, settings, "VkDeviceSize", "missShaderBindingStride", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(hitShaderBindingTableBuffer, NULL, settings, "VkBuffer", "hitShaderBindingTableBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(hitShaderBindingOffset, NULL, settings, "VkDeviceSize", "hitShaderBindingOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(hitShaderBindingStride, NULL, settings, "VkDeviceSize", "hitShaderBindingStride", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(callableShaderBindingTableBuffer, NULL, settings, "VkBuffer", "callableShaderBindingTableBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(callableShaderBindingOffset, NULL, settings, "VkDeviceSize", "callableShaderBindingOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(callableShaderBindingStride, NULL, settings, "VkDeviceSize", "callableShaderBindingStride", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(width, NULL, settings, "uint32_t", "width", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(height, NULL, settings, "uint32_t", "height", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(depth, NULL, settings, "uint32_t", "depth", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateRayTracingPipelinesNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkPipelineCache pipelineCache, uint32_t createInfoCount, const VkRayTracingPipelineCreateInfoNV* pCreateInfos, const VkAllocationCallbacks* pAllocator, VkPipeline* pPipelines)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineCache>(pipelineCache, NULL, settings, "VkPipelineCache", "pipelineCache", false, false, 4, dump_json_VkPipelineCache);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(createInfoCount, NULL, settings, "uint32_t", "createInfoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkRayTracingPipelineCreateInfoNV>(pCreateInfos, createInfoCount, settings, "const VkRayTracingPipelineCreateInfoNV*", "const VkRayTracingPipelineCreateInfoNV", "pCreateInfos", true, false, 4, dump_json_VkRayTracingPipelineCreateInfoNV); // PQA
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_array<const VkPipeline>(pPipelines, createInfoCount, settings, "VkPipeline*", "VkPipeline", "pPipelines", false, false, 4, dump_json_VkPipeline); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetRayTracingShaderGroupHandlesKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkPipeline pipeline, uint32_t firstGroup, uint32_t groupCount, size_t dataSize, void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipeline>(pipeline, NULL, settings, "VkPipeline", "pipeline", false, false, 4, dump_json_VkPipeline);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstGroup, NULL, settings, "uint32_t", "firstGroup", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCount, NULL, settings, "uint32_t", "groupCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const size_t>(dataSize, NULL, settings, "size_t", "dataSize", false, false, 4, dump_json_size_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetRayTracingShaderGroupHandlesNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkPipeline pipeline, uint32_t firstGroup, uint32_t groupCount, size_t dataSize, void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipeline>(pipeline, NULL, settings, "VkPipeline", "pipeline", false, false, 4, dump_json_VkPipeline);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstGroup, NULL, settings, "uint32_t", "firstGroup", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCount, NULL, settings, "uint32_t", "groupCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const size_t>(dataSize, NULL, settings, "size_t", "dataSize", false, false, 4, dump_json_size_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetAccelerationStructureHandleNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkAccelerationStructureNV accelerationStructure, size_t dataSize, void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkAccelerationStructureNV>(accelerationStructure, NULL, settings, "VkAccelerationStructureNV", "accelerationStructure", false, false, 4, dump_json_VkAccelerationStructureNV);
        settings.stream() << ",\n";
        dump_json_value<const size_t>(dataSize, NULL, settings, "size_t", "dataSize", false, false, 4, dump_json_size_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdWriteAccelerationStructuresPropertiesNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t accelerationStructureCount, const VkAccelerationStructureNV* pAccelerationStructures, VkQueryType queryType, VkQueryPool queryPool, uint32_t firstQuery)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(accelerationStructureCount, NULL, settings, "uint32_t", "accelerationStructureCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkAccelerationStructureNV>(pAccelerationStructures, accelerationStructureCount, settings, "const VkAccelerationStructureNV*", "const VkAccelerationStructureNV", "pAccelerationStructures", false, false, 4, dump_json_VkAccelerationStructureNV); // PQA
        settings.stream() << ",\n";
        dump_json_value<const VkQueryType>(queryType, NULL, settings, "VkQueryType", "queryType", false, false, 4, dump_json_VkQueryType);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstQuery, NULL, settings, "uint32_t", "firstQuery", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCompileDeferredNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkPipeline pipeline, uint32_t shader)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipeline>(pipeline, NULL, settings, "VkPipeline", "pipeline", false, false, 4, dump_json_VkPipeline);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(shader, NULL, settings, "uint32_t", "shader", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDescriptorSetLayoutSupportKHR(ApiDumpInstance& dump_inst, VkDevice device, const VkDescriptorSetLayoutCreateInfo* pCreateInfo, VkDescriptorSetLayoutSupport* pSupport)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorSetLayoutCreateInfo>(pCreateInfo, settings, "const VkDescriptorSetLayoutCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkDescriptorSetLayoutCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorSetLayoutSupport>(pSupport, settings, "VkDescriptorSetLayoutSupport*", "pSupport", true, false, 4, dump_json_VkDescriptorSetLayoutSupport);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawIndirectCountKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset, VkBuffer countBuffer, VkDeviceSize countBufferOffset, uint32_t maxDrawCount, uint32_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(countBuffer, NULL, settings, "VkBuffer", "countBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(countBufferOffset, NULL, settings, "VkDeviceSize", "countBufferOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(maxDrawCount, NULL, settings, "uint32_t", "maxDrawCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawIndexedIndirectCountKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset, VkBuffer countBuffer, VkDeviceSize countBufferOffset, uint32_t maxDrawCount, uint32_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(countBuffer, NULL, settings, "VkBuffer", "countBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(countBufferOffset, NULL, settings, "VkDeviceSize", "countBufferOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(maxDrawCount, NULL, settings, "uint32_t", "maxDrawCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetMemoryHostPointerPropertiesEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkExternalMemoryHandleTypeFlagBits handleType, const void* pHostPointer, VkMemoryHostPointerPropertiesEXT* pMemoryHostPointerProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkExternalMemoryHandleTypeFlagBits>(handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagBits", "handleType", false, false, 4, dump_json_VkExternalMemoryHandleTypeFlagBits);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pHostPointer, NULL, settings, "const void*", "pHostPointer", false, false, 4, dump_json_void);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryHostPointerPropertiesEXT>(pMemoryHostPointerProperties, settings, "VkMemoryHostPointerPropertiesEXT*", "pMemoryHostPointerProperties", true, false, 4, dump_json_VkMemoryHostPointerPropertiesEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdWriteBufferMarkerAMD(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkPipelineStageFlagBits pipelineStage, VkBuffer dstBuffer, VkDeviceSize dstOffset, uint32_t marker)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineStageFlagBits>(pipelineStage, NULL, settings, "VkPipelineStageFlagBits", "pipelineStage", false, false, 4, dump_json_VkPipelineStageFlagBits);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(dstBuffer, NULL, settings, "VkBuffer", "dstBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(dstOffset, NULL, settings, "VkDeviceSize", "dstOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(marker, NULL, settings, "uint32_t", "marker", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceCalibrateableTimeDomainsKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t* pTimeDomainCount, VkTimeDomainKHR* pTimeDomains)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pTimeDomainCount, settings, "uint32_t*", "pTimeDomainCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkTimeDomainKHR>(pTimeDomains, *pTimeDomainCount, settings, "VkTimeDomainKHR*", "VkTimeDomainKHR", "pTimeDomains", false, false, 4, dump_json_VkTimeDomainKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceCalibrateableTimeDomainsEXT(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t* pTimeDomainCount, VkTimeDomainKHR* pTimeDomains)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pTimeDomainCount, settings, "uint32_t*", "pTimeDomainCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkTimeDomainKHR>(pTimeDomains, *pTimeDomainCount, settings, "VkTimeDomainKHR*", "VkTimeDomainKHR", "pTimeDomains", false, false, 4, dump_json_VkTimeDomainKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetCalibratedTimestampsKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t timestampCount, const VkCalibratedTimestampInfoKHR* pTimestampInfos, uint64_t* pTimestamps, uint64_t* pMaxDeviation)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(timestampCount, NULL, settings, "uint32_t", "timestampCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkCalibratedTimestampInfoKHR>(pTimestampInfos, timestampCount, settings, "const VkCalibratedTimestampInfoKHR*", "const VkCalibratedTimestampInfoKHR", "pTimestampInfos", true, false, 4, dump_json_VkCalibratedTimestampInfoKHR); // PQA
        settings.stream() << ",\n";
        dump_json_array<const uint64_t>(pTimestamps, timestampCount, settings, "uint64_t*", "uint64_t", "pTimestamps", false, false, 4, dump_json_uint64_t); // PQA
        settings.stream() << ",\n";
        dump_json_pointer<const uint64_t>(pMaxDeviation, settings, "uint64_t*", "pMaxDeviation", false, false, 4, dump_json_uint64_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetCalibratedTimestampsEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t timestampCount, const VkCalibratedTimestampInfoKHR* pTimestampInfos, uint64_t* pTimestamps, uint64_t* pMaxDeviation)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(timestampCount, NULL, settings, "uint32_t", "timestampCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkCalibratedTimestampInfoKHR>(pTimestampInfos, timestampCount, settings, "const VkCalibratedTimestampInfoKHR*", "const VkCalibratedTimestampInfoKHR", "pTimestampInfos", true, false, 4, dump_json_VkCalibratedTimestampInfoKHR); // PQA
        settings.stream() << ",\n";
        dump_json_array<const uint64_t>(pTimestamps, timestampCount, settings, "uint64_t*", "uint64_t", "pTimestamps", false, false, 4, dump_json_uint64_t); // PQA
        settings.stream() << ",\n";
        dump_json_pointer<const uint64_t>(pMaxDeviation, settings, "uint64_t*", "pMaxDeviation", false, false, 4, dump_json_uint64_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawMeshTasksNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t taskCount, uint32_t firstTask)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(taskCount, NULL, settings, "uint32_t", "taskCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstTask, NULL, settings, "uint32_t", "firstTask", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawMeshTasksIndirectNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset, uint32_t drawCount, uint32_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(drawCount, NULL, settings, "uint32_t", "drawCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawMeshTasksIndirectCountNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset, VkBuffer countBuffer, VkDeviceSize countBufferOffset, uint32_t maxDrawCount, uint32_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(countBuffer, NULL, settings, "VkBuffer", "countBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(countBufferOffset, NULL, settings, "VkDeviceSize", "countBufferOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(maxDrawCount, NULL, settings, "uint32_t", "maxDrawCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetExclusiveScissorEnableNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstExclusiveScissor, uint32_t exclusiveScissorCount, const VkBool32* pExclusiveScissorEnables)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstExclusiveScissor, NULL, settings, "uint32_t", "firstExclusiveScissor", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(exclusiveScissorCount, NULL, settings, "uint32_t", "exclusiveScissorCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBool32>(pExclusiveScissorEnables, exclusiveScissorCount, settings, "const VkBool32*", "const VkBool32", "pExclusiveScissorEnables", false, false, 4, dump_json_VkBool32); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetExclusiveScissorNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstExclusiveScissor, uint32_t exclusiveScissorCount, const VkRect2D* pExclusiveScissors)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstExclusiveScissor, NULL, settings, "uint32_t", "firstExclusiveScissor", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(exclusiveScissorCount, NULL, settings, "uint32_t", "exclusiveScissorCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkRect2D>(pExclusiveScissors, exclusiveScissorCount, settings, "const VkRect2D*", "const VkRect2D", "pExclusiveScissors", true, false, 4, dump_json_VkRect2D); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetCheckpointNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const void* pCheckpointMarker)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pCheckpointMarker, NULL, settings, "const void*", "pCheckpointMarker", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetQueueCheckpointDataNV(ApiDumpInstance& dump_inst, VkQueue queue, uint32_t* pCheckpointDataCount, VkCheckpointDataNV* pCheckpointData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkQueue>(queue, NULL, settings, "VkQueue", "queue", false, false, 4, dump_json_VkQueue);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pCheckpointDataCount, settings, "uint32_t*", "pCheckpointDataCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkCheckpointDataNV>(pCheckpointData, *pCheckpointDataCount, settings, "VkCheckpointDataNV*", "VkCheckpointDataNV", "pCheckpointData", true, false, 4, dump_json_VkCheckpointDataNV); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetSemaphoreCounterValueKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkSemaphore semaphore, uint64_t* pValue)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSemaphore>(semaphore, NULL, settings, "VkSemaphore", "semaphore", false, false, 4, dump_json_VkSemaphore);
        settings.stream() << ",\n";
        dump_json_pointer<const uint64_t>(pValue, settings, "uint64_t*", "pValue", false, false, 4, dump_json_uint64_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkWaitSemaphoresKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkSemaphoreWaitInfo* pWaitInfo, uint64_t timeout)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSemaphoreWaitInfo>(pWaitInfo, settings, "const VkSemaphoreWaitInfo*", "pWaitInfo", true, false, 4, dump_json_VkSemaphoreWaitInfo);
        settings.stream() << ",\n";
        dump_json_value<const uint64_t>(timeout, NULL, settings, "uint64_t", "timeout", false, false, 4, dump_json_uint64_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkSignalSemaphoreKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkSemaphoreSignalInfo* pSignalInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSemaphoreSignalInfo>(pSignalInfo, settings, "const VkSemaphoreSignalInfo*", "pSignalInfo", true, false, 4, dump_json_VkSemaphoreSignalInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkInitializePerformanceApiINTEL(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkInitializePerformanceApiInfoINTEL* pInitializeInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkInitializePerformanceApiInfoINTEL>(pInitializeInfo, settings, "const VkInitializePerformanceApiInfoINTEL*", "pInitializeInfo", true, false, 4, dump_json_VkInitializePerformanceApiInfoINTEL);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkUninitializePerformanceApiINTEL(ApiDumpInstance& dump_inst, VkDevice device)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetPerformanceMarkerINTEL(ApiDumpInstance& dump_inst, VkResult result, VkCommandBuffer commandBuffer, const VkPerformanceMarkerInfoINTEL* pMarkerInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPerformanceMarkerInfoINTEL>(pMarkerInfo, settings, "const VkPerformanceMarkerInfoINTEL*", "pMarkerInfo", true, false, 4, dump_json_VkPerformanceMarkerInfoINTEL);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetPerformanceStreamMarkerINTEL(ApiDumpInstance& dump_inst, VkResult result, VkCommandBuffer commandBuffer, const VkPerformanceStreamMarkerInfoINTEL* pMarkerInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPerformanceStreamMarkerInfoINTEL>(pMarkerInfo, settings, "const VkPerformanceStreamMarkerInfoINTEL*", "pMarkerInfo", true, false, 4, dump_json_VkPerformanceStreamMarkerInfoINTEL);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetPerformanceOverrideINTEL(ApiDumpInstance& dump_inst, VkResult result, VkCommandBuffer commandBuffer, const VkPerformanceOverrideInfoINTEL* pOverrideInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPerformanceOverrideInfoINTEL>(pOverrideInfo, settings, "const VkPerformanceOverrideInfoINTEL*", "pOverrideInfo", true, false, 4, dump_json_VkPerformanceOverrideInfoINTEL);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkAcquirePerformanceConfigurationINTEL(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkPerformanceConfigurationAcquireInfoINTEL* pAcquireInfo, VkPerformanceConfigurationINTEL* pConfiguration)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPerformanceConfigurationAcquireInfoINTEL>(pAcquireInfo, settings, "const VkPerformanceConfigurationAcquireInfoINTEL*", "pAcquireInfo", true, false, 4, dump_json_VkPerformanceConfigurationAcquireInfoINTEL);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPerformanceConfigurationINTEL>(pConfiguration, settings, "VkPerformanceConfigurationINTEL*", "pConfiguration", false, false, 4, dump_json_VkPerformanceConfigurationINTEL);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkReleasePerformanceConfigurationINTEL(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkPerformanceConfigurationINTEL configuration)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPerformanceConfigurationINTEL>(configuration, NULL, settings, "VkPerformanceConfigurationINTEL", "configuration", false, false, 4, dump_json_VkPerformanceConfigurationINTEL);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkQueueSetPerformanceConfigurationINTEL(ApiDumpInstance& dump_inst, VkResult result, VkQueue queue, VkPerformanceConfigurationINTEL configuration)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkQueue>(queue, NULL, settings, "VkQueue", "queue", false, false, 4, dump_json_VkQueue);
        settings.stream() << ",\n";
        dump_json_value<const VkPerformanceConfigurationINTEL>(configuration, NULL, settings, "VkPerformanceConfigurationINTEL", "configuration", false, false, 4, dump_json_VkPerformanceConfigurationINTEL);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPerformanceParameterINTEL(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkPerformanceParameterTypeINTEL parameter, VkPerformanceValueINTEL* pValue)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPerformanceParameterTypeINTEL>(parameter, NULL, settings, "VkPerformanceParameterTypeINTEL", "parameter", false, false, 4, dump_json_VkPerformanceParameterTypeINTEL);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPerformanceValueINTEL>(pValue, settings, "VkPerformanceValueINTEL*", "pValue", true, false, 4, dump_json_VkPerformanceValueINTEL);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkSetLocalDimmingAMD(ApiDumpInstance& dump_inst, VkDevice device, VkSwapchainKHR swapChain, VkBool32 localDimmingEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSwapchainKHR>(swapChain, NULL, settings, "VkSwapchainKHR", "swapChain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(localDimmingEnable, NULL, settings, "VkBool32", "localDimmingEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_vkCreateImagePipeSurfaceFUCHSIA(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkImagePipeSurfaceCreateInfoFUCHSIA* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSurfaceKHR* pSurface)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImagePipeSurfaceCreateInfoFUCHSIA>(pCreateInfo, settings, "const VkImagePipeSurfaceCreateInfoFUCHSIA*", "pCreateInfo", true, false, 4, dump_json_VkImagePipeSurfaceCreateInfoFUCHSIA);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceKHR>(pSurface, settings, "VkSurfaceKHR*", "pSurface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_vkCreateMetalSurfaceEXT(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkMetalSurfaceCreateInfoEXT* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSurfaceKHR* pSurface)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMetalSurfaceCreateInfoEXT>(pCreateInfo, settings, "const VkMetalSurfaceCreateInfoEXT*", "pCreateInfo", true, false, 4, dump_json_VkMetalSurfaceCreateInfoEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceKHR>(pSurface, settings, "VkSurfaceKHR*", "pSurface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_METAL_EXT
void dump_json_vkGetPhysicalDeviceFragmentShadingRatesKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t* pFragmentShadingRateCount, VkPhysicalDeviceFragmentShadingRateKHR* pFragmentShadingRates)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pFragmentShadingRateCount, settings, "uint32_t*", "pFragmentShadingRateCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkPhysicalDeviceFragmentShadingRateKHR>(pFragmentShadingRates, *pFragmentShadingRateCount, settings, "VkPhysicalDeviceFragmentShadingRateKHR*", "VkPhysicalDeviceFragmentShadingRateKHR", "pFragmentShadingRates", true, false, 4, dump_json_VkPhysicalDeviceFragmentShadingRateKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetFragmentShadingRateKHR(ApiDumpInstance& dump_inst, VkCommandBuffer           commandBuffer, const VkExtent2D*                           pFragmentSize, const VkFragmentShadingRateCombinerOpKHR    combinerOps[2])
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkExtent2D>(pFragmentSize, settings, "const VkExtent2D*", "pFragmentSize", true, false, 4, dump_json_VkExtent2D);
        settings.stream() << ",\n";
        dump_json_array<const VkFragmentShadingRateCombinerOpKHR>(combinerOps, 2, settings, "const VkFragmentShadingRateCombinerOpKHR[2]", "const VkFragmentShadingRateCombinerOpKHR", "combinerOps", false, false, 4, dump_json_VkFragmentShadingRateCombinerOpKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetRenderingAttachmentLocationsKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkRenderingAttachmentLocationInfoKHR* pLocationInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRenderingAttachmentLocationInfoKHR>(pLocationInfo, settings, "const VkRenderingAttachmentLocationInfoKHR*", "pLocationInfo", true, false, 4, dump_json_VkRenderingAttachmentLocationInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetRenderingInputAttachmentIndicesKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkRenderingInputAttachmentIndexInfoKHR* pInputAttachmentIndexInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRenderingInputAttachmentIndexInfoKHR>(pInputAttachmentIndexInfo, settings, "const VkRenderingInputAttachmentIndexInfoKHR*", "pInputAttachmentIndexInfo", true, false, 4, dump_json_VkRenderingInputAttachmentIndexInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetBufferDeviceAddressEXT(ApiDumpInstance& dump_inst, VkDeviceAddress result, VkDevice device, const VkBufferDeviceAddressInfo* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkDeviceAddress(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBufferDeviceAddressInfo>(pInfo, settings, "const VkBufferDeviceAddressInfo*", "pInfo", true, false, 4, dump_json_VkBufferDeviceAddressInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceToolPropertiesEXT(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t* pToolCount, VkPhysicalDeviceToolProperties* pToolProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pToolCount, settings, "uint32_t*", "pToolCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkPhysicalDeviceToolProperties>(pToolProperties, *pToolCount, settings, "VkPhysicalDeviceToolProperties*", "VkPhysicalDeviceToolProperties", "pToolProperties", true, false, 4, dump_json_VkPhysicalDeviceToolProperties); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkWaitForPresentKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkSwapchainKHR swapchain, uint64_t presentId, uint64_t timeout)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSwapchainKHR>(swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << ",\n";
        dump_json_value<const uint64_t>(presentId, NULL, settings, "uint64_t", "presentId", false, false, 4, dump_json_uint64_t);
        settings.stream() << ",\n";
        dump_json_value<const uint64_t>(timeout, NULL, settings, "uint64_t", "timeout", false, false, 4, dump_json_uint64_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceCooperativeMatrixPropertiesNV(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t* pPropertyCount, VkCooperativeMatrixPropertiesNV* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPropertyCount, settings, "uint32_t*", "pPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkCooperativeMatrixPropertiesNV>(pProperties, *pPropertyCount, settings, "VkCooperativeMatrixPropertiesNV*", "VkCooperativeMatrixPropertiesNV", "pProperties", true, false, 4, dump_json_VkCooperativeMatrixPropertiesNV); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceSupportedFramebufferMixedSamplesCombinationsNV(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t* pCombinationCount, VkFramebufferMixedSamplesCombinationNV* pCombinations)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pCombinationCount, settings, "uint32_t*", "pCombinationCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkFramebufferMixedSamplesCombinationNV>(pCombinations, *pCombinationCount, settings, "VkFramebufferMixedSamplesCombinationNV*", "VkFramebufferMixedSamplesCombinationNV", "pCombinations", true, false, 4, dump_json_VkFramebufferMixedSamplesCombinationNV); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_vkGetPhysicalDeviceSurfacePresentModes2EXT(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, const VkPhysicalDeviceSurfaceInfo2KHR* pSurfaceInfo, uint32_t* pPresentModeCount, VkPresentModeKHR* pPresentModes)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceSurfaceInfo2KHR>(pSurfaceInfo, settings, "const VkPhysicalDeviceSurfaceInfo2KHR*", "pSurfaceInfo", true, false, 4, dump_json_VkPhysicalDeviceSurfaceInfo2KHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPresentModeCount, settings, "uint32_t*", "pPresentModeCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkPresentModeKHR>(pPresentModes, *pPresentModeCount, settings, "VkPresentModeKHR*", "VkPresentModeKHR", "pPresentModes", false, false, 4, dump_json_VkPresentModeKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_vkAcquireFullScreenExclusiveModeEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkSwapchainKHR swapchain)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSwapchainKHR>(swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_vkReleaseFullScreenExclusiveModeEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkSwapchainKHR swapchain)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSwapchainKHR>(swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_vkGetDeviceGroupSurfacePresentModes2EXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkPhysicalDeviceSurfaceInfo2KHR* pSurfaceInfo, VkDeviceGroupPresentModeFlagsKHR* pModes)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceSurfaceInfo2KHR>(pSurfaceInfo, settings, "const VkPhysicalDeviceSurfaceInfo2KHR*", "pSurfaceInfo", true, false, 4, dump_json_VkPhysicalDeviceSurfaceInfo2KHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceGroupPresentModeFlagsKHR>(pModes, settings, "VkDeviceGroupPresentModeFlagsKHR*", "pModes", false, false, 4, dump_json_VkDeviceGroupPresentModeFlagsKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WIN32_KHR
void dump_json_vkCreateHeadlessSurfaceEXT(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkHeadlessSurfaceCreateInfoEXT* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSurfaceKHR* pSurface)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkHeadlessSurfaceCreateInfoEXT>(pCreateInfo, settings, "const VkHeadlessSurfaceCreateInfoEXT*", "pCreateInfo", true, false, 4, dump_json_VkHeadlessSurfaceCreateInfoEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceKHR>(pSurface, settings, "VkSurfaceKHR*", "pSurface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetBufferDeviceAddressKHR(ApiDumpInstance& dump_inst, VkDeviceAddress result, VkDevice device, const VkBufferDeviceAddressInfo* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkDeviceAddress(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBufferDeviceAddressInfo>(pInfo, settings, "const VkBufferDeviceAddressInfo*", "pInfo", true, false, 4, dump_json_VkBufferDeviceAddressInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetBufferOpaqueCaptureAddressKHR(ApiDumpInstance& dump_inst, uint64_t result, VkDevice device, const VkBufferDeviceAddressInfo* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_uint64_t(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBufferDeviceAddressInfo>(pInfo, settings, "const VkBufferDeviceAddressInfo*", "pInfo", true, false, 4, dump_json_VkBufferDeviceAddressInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceMemoryOpaqueCaptureAddressKHR(ApiDumpInstance& dump_inst, uint64_t result, VkDevice device, const VkDeviceMemoryOpaqueCaptureAddressInfo* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_uint64_t(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceMemoryOpaqueCaptureAddressInfo>(pInfo, settings, "const VkDeviceMemoryOpaqueCaptureAddressInfo*", "pInfo", true, false, 4, dump_json_VkDeviceMemoryOpaqueCaptureAddressInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetLineStippleKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t lineStippleFactor, uint16_t lineStipplePattern)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(lineStippleFactor, NULL, settings, "uint32_t", "lineStippleFactor", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint16_t>(lineStipplePattern, NULL, settings, "uint16_t", "lineStipplePattern", false, false, 4, dump_json_uint16_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetLineStippleEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t lineStippleFactor, uint16_t lineStipplePattern)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(lineStippleFactor, NULL, settings, "uint32_t", "lineStippleFactor", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint16_t>(lineStipplePattern, NULL, settings, "uint16_t", "lineStipplePattern", false, false, 4, dump_json_uint16_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkResetQueryPoolEXT(ApiDumpInstance& dump_inst, VkDevice device, VkQueryPool queryPool, uint32_t firstQuery, uint32_t queryCount)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstQuery, NULL, settings, "uint32_t", "firstQuery", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(queryCount, NULL, settings, "uint32_t", "queryCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetCullModeEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkCullModeFlags cullMode)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkCullModeFlags>(cullMode, NULL, settings, "VkCullModeFlags", "cullMode", false, false, 4, dump_json_VkCullModeFlags);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetFrontFaceEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkFrontFace frontFace)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkFrontFace>(frontFace, NULL, settings, "VkFrontFace", "frontFace", false, false, 4, dump_json_VkFrontFace);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetPrimitiveTopologyEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkPrimitiveTopology primitiveTopology)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPrimitiveTopology>(primitiveTopology, NULL, settings, "VkPrimitiveTopology", "primitiveTopology", false, false, 4, dump_json_VkPrimitiveTopology);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetViewportWithCountEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t viewportCount, const VkViewport* pViewports)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(viewportCount, NULL, settings, "uint32_t", "viewportCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkViewport>(pViewports, viewportCount, settings, "const VkViewport*", "const VkViewport", "pViewports", true, false, 4, dump_json_VkViewport); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetScissorWithCountEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t scissorCount, const VkRect2D* pScissors)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(scissorCount, NULL, settings, "uint32_t", "scissorCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkRect2D>(pScissors, scissorCount, settings, "const VkRect2D*", "const VkRect2D", "pScissors", true, false, 4, dump_json_VkRect2D); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBindVertexBuffers2EXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstBinding, uint32_t bindingCount, const VkBuffer* pBuffers, const VkDeviceSize* pOffsets, const VkDeviceSize* pSizes, const VkDeviceSize* pStrides)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstBinding, NULL, settings, "uint32_t", "firstBinding", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(bindingCount, NULL, settings, "uint32_t", "bindingCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBuffer>(pBuffers, bindingCount, settings, "const VkBuffer*", "const VkBuffer", "pBuffers", false, false, 4, dump_json_VkBuffer); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkDeviceSize>(pOffsets, bindingCount, settings, "const VkDeviceSize*", "const VkDeviceSize", "pOffsets", false, false, 4, dump_json_VkDeviceSize); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkDeviceSize>(pSizes, bindingCount, settings, "const VkDeviceSize*", "const VkDeviceSize", "pSizes", false, false, 4, dump_json_VkDeviceSize); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkDeviceSize>(pStrides, bindingCount, settings, "const VkDeviceSize*", "const VkDeviceSize", "pStrides", false, false, 4, dump_json_VkDeviceSize); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthTestEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 depthTestEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(depthTestEnable, NULL, settings, "VkBool32", "depthTestEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthWriteEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 depthWriteEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(depthWriteEnable, NULL, settings, "VkBool32", "depthWriteEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthCompareOpEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkCompareOp depthCompareOp)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkCompareOp>(depthCompareOp, NULL, settings, "VkCompareOp", "depthCompareOp", false, false, 4, dump_json_VkCompareOp);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthBoundsTestEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 depthBoundsTestEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(depthBoundsTestEnable, NULL, settings, "VkBool32", "depthBoundsTestEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetStencilTestEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 stencilTestEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(stencilTestEnable, NULL, settings, "VkBool32", "stencilTestEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetStencilOpEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkStencilFaceFlags faceMask, VkStencilOp failOp, VkStencilOp passOp, VkStencilOp depthFailOp, VkCompareOp compareOp)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkStencilFaceFlags>(faceMask, NULL, settings, "VkStencilFaceFlags", "faceMask", false, false, 4, dump_json_VkStencilFaceFlags);
        settings.stream() << ",\n";
        dump_json_value<const VkStencilOp>(failOp, NULL, settings, "VkStencilOp", "failOp", false, false, 4, dump_json_VkStencilOp);
        settings.stream() << ",\n";
        dump_json_value<const VkStencilOp>(passOp, NULL, settings, "VkStencilOp", "passOp", false, false, 4, dump_json_VkStencilOp);
        settings.stream() << ",\n";
        dump_json_value<const VkStencilOp>(depthFailOp, NULL, settings, "VkStencilOp", "depthFailOp", false, false, 4, dump_json_VkStencilOp);
        settings.stream() << ",\n";
        dump_json_value<const VkCompareOp>(compareOp, NULL, settings, "VkCompareOp", "compareOp", false, false, 4, dump_json_VkCompareOp);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateDeferredOperationKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkAllocationCallbacks* pAllocator, VkDeferredOperationKHR* pDeferredOperation)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeferredOperationKHR>(pDeferredOperation, settings, "VkDeferredOperationKHR*", "pDeferredOperation", false, false, 4, dump_json_VkDeferredOperationKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyDeferredOperationKHR(ApiDumpInstance& dump_inst, VkDevice device, VkDeferredOperationKHR operation, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeferredOperationKHR>(operation, NULL, settings, "VkDeferredOperationKHR", "operation", false, false, 4, dump_json_VkDeferredOperationKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeferredOperationMaxConcurrencyKHR(ApiDumpInstance& dump_inst, uint32_t result, VkDevice device, VkDeferredOperationKHR operation)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_uint32_t(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeferredOperationKHR>(operation, NULL, settings, "VkDeferredOperationKHR", "operation", false, false, 4, dump_json_VkDeferredOperationKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeferredOperationResultKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDeferredOperationKHR operation)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeferredOperationKHR>(operation, NULL, settings, "VkDeferredOperationKHR", "operation", false, false, 4, dump_json_VkDeferredOperationKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDeferredOperationJoinKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDeferredOperationKHR operation)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeferredOperationKHR>(operation, NULL, settings, "VkDeferredOperationKHR", "operation", false, false, 4, dump_json_VkDeferredOperationKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPipelineExecutablePropertiesKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice                        device, const VkPipelineInfoKHR*        pPipelineInfo, uint32_t* pExecutableCount, VkPipelineExecutablePropertiesKHR* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPipelineInfoKHR>(pPipelineInfo, settings, "const VkPipelineInfoKHR*", "pPipelineInfo", true, false, 4, dump_json_VkPipelineInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pExecutableCount, settings, "uint32_t*", "pExecutableCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkPipelineExecutablePropertiesKHR>(pProperties, *pExecutableCount, settings, "VkPipelineExecutablePropertiesKHR*", "VkPipelineExecutablePropertiesKHR", "pProperties", true, false, 4, dump_json_VkPipelineExecutablePropertiesKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPipelineExecutableStatisticsKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice                        device, const VkPipelineExecutableInfoKHR*  pExecutableInfo, uint32_t* pStatisticCount, VkPipelineExecutableStatisticKHR* pStatistics)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPipelineExecutableInfoKHR>(pExecutableInfo, settings, "const VkPipelineExecutableInfoKHR*", "pExecutableInfo", true, false, 4, dump_json_VkPipelineExecutableInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pStatisticCount, settings, "uint32_t*", "pStatisticCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkPipelineExecutableStatisticKHR>(pStatistics, *pStatisticCount, settings, "VkPipelineExecutableStatisticKHR*", "VkPipelineExecutableStatisticKHR", "pStatistics", true, false, 4, dump_json_VkPipelineExecutableStatisticKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPipelineExecutableInternalRepresentationsKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice                        device, const VkPipelineExecutableInfoKHR*  pExecutableInfo, uint32_t* pInternalRepresentationCount, VkPipelineExecutableInternalRepresentationKHR* pInternalRepresentations)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPipelineExecutableInfoKHR>(pExecutableInfo, settings, "const VkPipelineExecutableInfoKHR*", "pExecutableInfo", true, false, 4, dump_json_VkPipelineExecutableInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pInternalRepresentationCount, settings, "uint32_t*", "pInternalRepresentationCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkPipelineExecutableInternalRepresentationKHR>(pInternalRepresentations, *pInternalRepresentationCount, settings, "VkPipelineExecutableInternalRepresentationKHR*", "VkPipelineExecutableInternalRepresentationKHR", "pInternalRepresentations", true, false, 4, dump_json_VkPipelineExecutableInternalRepresentationKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCopyMemoryToImageEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkCopyMemoryToImageInfoEXT* pCopyMemoryToImageInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyMemoryToImageInfoEXT>(pCopyMemoryToImageInfo, settings, "const VkCopyMemoryToImageInfoEXT*", "pCopyMemoryToImageInfo", true, false, 4, dump_json_VkCopyMemoryToImageInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCopyImageToMemoryEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkCopyImageToMemoryInfoEXT* pCopyImageToMemoryInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyImageToMemoryInfoEXT>(pCopyImageToMemoryInfo, settings, "const VkCopyImageToMemoryInfoEXT*", "pCopyImageToMemoryInfo", true, false, 4, dump_json_VkCopyImageToMemoryInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCopyImageToImageEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkCopyImageToImageInfoEXT* pCopyImageToImageInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyImageToImageInfoEXT>(pCopyImageToImageInfo, settings, "const VkCopyImageToImageInfoEXT*", "pCopyImageToImageInfo", true, false, 4, dump_json_VkCopyImageToImageInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkTransitionImageLayoutEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t transitionCount, const VkHostImageLayoutTransitionInfoEXT* pTransitions)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(transitionCount, NULL, settings, "uint32_t", "transitionCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkHostImageLayoutTransitionInfoEXT>(pTransitions, transitionCount, settings, "const VkHostImageLayoutTransitionInfoEXT*", "const VkHostImageLayoutTransitionInfoEXT", "pTransitions", true, false, 4, dump_json_VkHostImageLayoutTransitionInfoEXT); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetImageSubresourceLayout2KHR(ApiDumpInstance& dump_inst, VkDevice device, VkImage image, const VkImageSubresource2KHR* pSubresource, VkSubresourceLayout2KHR* pLayout)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(image, NULL, settings, "VkImage", "image", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageSubresource2KHR>(pSubresource, settings, "const VkImageSubresource2KHR*", "pSubresource", true, false, 4, dump_json_VkImageSubresource2KHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSubresourceLayout2KHR>(pLayout, settings, "VkSubresourceLayout2KHR*", "pLayout", true, false, 4, dump_json_VkSubresourceLayout2KHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetImageSubresourceLayout2EXT(ApiDumpInstance& dump_inst, VkDevice device, VkImage image, const VkImageSubresource2KHR* pSubresource, VkSubresourceLayout2KHR* pLayout)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(image, NULL, settings, "VkImage", "image", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageSubresource2KHR>(pSubresource, settings, "const VkImageSubresource2KHR*", "pSubresource", true, false, 4, dump_json_VkImageSubresource2KHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSubresourceLayout2KHR>(pLayout, settings, "VkSubresourceLayout2KHR*", "pLayout", true, false, 4, dump_json_VkSubresourceLayout2KHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkMapMemory2KHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkMemoryMapInfoKHR* pMemoryMapInfo, void** ppData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryMapInfoKHR>(pMemoryMapInfo, settings, "const VkMemoryMapInfoKHR*", "pMemoryMapInfo", true, false, 4, dump_json_VkMemoryMapInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const void*>(ppData, settings, "void**", "ppData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkUnmapMemory2KHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkMemoryUnmapInfoKHR* pMemoryUnmapInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryUnmapInfoKHR>(pMemoryUnmapInfo, settings, "const VkMemoryUnmapInfoKHR*", "pMemoryUnmapInfo", true, false, 4, dump_json_VkMemoryUnmapInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkReleaseSwapchainImagesEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkReleaseSwapchainImagesInfoEXT* pReleaseInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkReleaseSwapchainImagesInfoEXT>(pReleaseInfo, settings, "const VkReleaseSwapchainImagesInfoEXT*", "pReleaseInfo", true, false, 4, dump_json_VkReleaseSwapchainImagesInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetGeneratedCommandsMemoryRequirementsNV(ApiDumpInstance& dump_inst, VkDevice device, const VkGeneratedCommandsMemoryRequirementsInfoNV* pInfo, VkMemoryRequirements2* pMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkGeneratedCommandsMemoryRequirementsInfoNV>(pInfo, settings, "const VkGeneratedCommandsMemoryRequirementsInfoNV*", "pInfo", true, false, 4, dump_json_VkGeneratedCommandsMemoryRequirementsInfoNV);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryRequirements2>(pMemoryRequirements, settings, "VkMemoryRequirements2*", "pMemoryRequirements", true, false, 4, dump_json_VkMemoryRequirements2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdPreprocessGeneratedCommandsNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkGeneratedCommandsInfoNV* pGeneratedCommandsInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkGeneratedCommandsInfoNV>(pGeneratedCommandsInfo, settings, "const VkGeneratedCommandsInfoNV*", "pGeneratedCommandsInfo", true, false, 4, dump_json_VkGeneratedCommandsInfoNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdExecuteGeneratedCommandsNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 isPreprocessed, const VkGeneratedCommandsInfoNV* pGeneratedCommandsInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(isPreprocessed, NULL, settings, "VkBool32", "isPreprocessed", false, false, 4, dump_json_VkBool32);
        settings.stream() << ",\n";
        dump_json_pointer<const VkGeneratedCommandsInfoNV>(pGeneratedCommandsInfo, settings, "const VkGeneratedCommandsInfoNV*", "pGeneratedCommandsInfo", true, false, 4, dump_json_VkGeneratedCommandsInfoNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBindPipelineShaderGroupNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkPipelineBindPoint pipelineBindPoint, VkPipeline pipeline, uint32_t groupIndex)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineBindPoint>(pipelineBindPoint, NULL, settings, "VkPipelineBindPoint", "pipelineBindPoint", false, false, 4, dump_json_VkPipelineBindPoint);
        settings.stream() << ",\n";
        dump_json_value<const VkPipeline>(pipeline, NULL, settings, "VkPipeline", "pipeline", false, false, 4, dump_json_VkPipeline);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupIndex, NULL, settings, "uint32_t", "groupIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateIndirectCommandsLayoutNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkIndirectCommandsLayoutCreateInfoNV* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkIndirectCommandsLayoutNV* pIndirectCommandsLayout)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkIndirectCommandsLayoutCreateInfoNV>(pCreateInfo, settings, "const VkIndirectCommandsLayoutCreateInfoNV*", "pCreateInfo", true, false, 4, dump_json_VkIndirectCommandsLayoutCreateInfoNV);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkIndirectCommandsLayoutNV>(pIndirectCommandsLayout, settings, "VkIndirectCommandsLayoutNV*", "pIndirectCommandsLayout", false, false, 4, dump_json_VkIndirectCommandsLayoutNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyIndirectCommandsLayoutNV(ApiDumpInstance& dump_inst, VkDevice device, VkIndirectCommandsLayoutNV indirectCommandsLayout, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkIndirectCommandsLayoutNV>(indirectCommandsLayout, NULL, settings, "VkIndirectCommandsLayoutNV", "indirectCommandsLayout", false, false, 4, dump_json_VkIndirectCommandsLayoutNV);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthBias2EXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkDepthBiasInfoEXT*         pDepthBiasInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDepthBiasInfoEXT>(pDepthBiasInfo, settings, "const VkDepthBiasInfoEXT*", "pDepthBiasInfo", true, false, 4, dump_json_VkDepthBiasInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkAcquireDrmDisplayEXT(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, int32_t drmFd, VkDisplayKHR display)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const int32_t>(drmFd, NULL, settings, "int32_t", "drmFd", false, false, 4, dump_json_int32_t);
        settings.stream() << ",\n";
        dump_json_value<const VkDisplayKHR>(display, NULL, settings, "VkDisplayKHR", "display", false, false, 4, dump_json_VkDisplayKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDrmDisplayEXT(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, int32_t drmFd, uint32_t connectorId, VkDisplayKHR* display)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const int32_t>(drmFd, NULL, settings, "int32_t", "drmFd", false, false, 4, dump_json_int32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(connectorId, NULL, settings, "uint32_t", "connectorId", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDisplayKHR>(display, settings, "VkDisplayKHR*", "display", false, false, 4, dump_json_VkDisplayKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreatePrivateDataSlotEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkPrivateDataSlotCreateInfo* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkPrivateDataSlot* pPrivateDataSlot)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPrivateDataSlotCreateInfo>(pCreateInfo, settings, "const VkPrivateDataSlotCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkPrivateDataSlotCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPrivateDataSlot>(pPrivateDataSlot, settings, "VkPrivateDataSlot*", "pPrivateDataSlot", false, false, 4, dump_json_VkPrivateDataSlot);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyPrivateDataSlotEXT(ApiDumpInstance& dump_inst, VkDevice device, VkPrivateDataSlot privateDataSlot, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPrivateDataSlot>(privateDataSlot, NULL, settings, "VkPrivateDataSlot", "privateDataSlot", false, false, 4, dump_json_VkPrivateDataSlot);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkSetPrivateDataEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkObjectType objectType, uint64_t objectHandle, VkPrivateDataSlot privateDataSlot, uint64_t data)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkObjectType>(objectType, NULL, settings, "VkObjectType", "objectType", false, false, 4, dump_json_VkObjectType);
        settings.stream() << ",\n";
        dump_json_value<const uint64_t>(objectHandle, NULL, settings, "uint64_t", "objectHandle", false, false, 4, dump_json_uint64_t);
        settings.stream() << ",\n";
        dump_json_value<const VkPrivateDataSlot>(privateDataSlot, NULL, settings, "VkPrivateDataSlot", "privateDataSlot", false, false, 4, dump_json_VkPrivateDataSlot);
        settings.stream() << ",\n";
        dump_json_value<const uint64_t>(data, NULL, settings, "uint64_t", "data", false, false, 4, dump_json_uint64_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPrivateDataEXT(ApiDumpInstance& dump_inst, VkDevice device, VkObjectType objectType, uint64_t objectHandle, VkPrivateDataSlot privateDataSlot, uint64_t* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkObjectType>(objectType, NULL, settings, "VkObjectType", "objectType", false, false, 4, dump_json_VkObjectType);
        settings.stream() << ",\n";
        dump_json_value<const uint64_t>(objectHandle, NULL, settings, "uint64_t", "objectHandle", false, false, 4, dump_json_uint64_t);
        settings.stream() << ",\n";
        dump_json_value<const VkPrivateDataSlot>(privateDataSlot, NULL, settings, "VkPrivateDataSlot", "privateDataSlot", false, false, 4, dump_json_VkPrivateDataSlot);
        settings.stream() << ",\n";
        dump_json_pointer<const uint64_t>(pData, settings, "uint64_t*", "pData", false, false, 4, dump_json_uint64_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceVideoEncodeQualityLevelPropertiesKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, const VkPhysicalDeviceVideoEncodeQualityLevelInfoKHR* pQualityLevelInfo, VkVideoEncodeQualityLevelPropertiesKHR* pQualityLevelProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPhysicalDeviceVideoEncodeQualityLevelInfoKHR>(pQualityLevelInfo, settings, "const VkPhysicalDeviceVideoEncodeQualityLevelInfoKHR*", "pQualityLevelInfo", true, false, 4, dump_json_VkPhysicalDeviceVideoEncodeQualityLevelInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkVideoEncodeQualityLevelPropertiesKHR>(pQualityLevelProperties, settings, "VkVideoEncodeQualityLevelPropertiesKHR*", "pQualityLevelProperties", true, false, 4, dump_json_VkVideoEncodeQualityLevelPropertiesKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetEncodedVideoSessionParametersKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkVideoEncodeSessionParametersGetInfoKHR* pVideoSessionParametersInfo, VkVideoEncodeSessionParametersFeedbackInfoKHR* pFeedbackInfo, size_t* pDataSize, void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkVideoEncodeSessionParametersGetInfoKHR>(pVideoSessionParametersInfo, settings, "const VkVideoEncodeSessionParametersGetInfoKHR*", "pVideoSessionParametersInfo", true, false, 4, dump_json_VkVideoEncodeSessionParametersGetInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkVideoEncodeSessionParametersFeedbackInfoKHR>(pFeedbackInfo, settings, "VkVideoEncodeSessionParametersFeedbackInfoKHR*", "pFeedbackInfo", true, false, 4, dump_json_VkVideoEncodeSessionParametersFeedbackInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const size_t>(pDataSize, settings, "size_t*", "pDataSize", false, false, 4, dump_json_size_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdEncodeVideoKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkVideoEncodeInfoKHR* pEncodeInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkVideoEncodeInfoKHR>(pEncodeInfo, settings, "const VkVideoEncodeInfoKHR*", "pEncodeInfo", true, false, 4, dump_json_VkVideoEncodeInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateCudaModuleNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkCudaModuleCreateInfoNV* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkCudaModuleNV* pModule)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCudaModuleCreateInfoNV>(pCreateInfo, settings, "const VkCudaModuleCreateInfoNV*", "pCreateInfo", true, false, 4, dump_json_VkCudaModuleCreateInfoNV);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCudaModuleNV>(pModule, settings, "VkCudaModuleNV*", "pModule", false, false, 4, dump_json_VkCudaModuleNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetCudaModuleCacheNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkCudaModuleNV module, size_t* pCacheSize, void* pCacheData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkCudaModuleNV>(module, NULL, settings, "VkCudaModuleNV", "module", false, false, 4, dump_json_VkCudaModuleNV);
        settings.stream() << ",\n";
        dump_json_pointer<const size_t>(pCacheSize, settings, "size_t*", "pCacheSize", false, false, 4, dump_json_size_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pCacheData, NULL, settings, "void*", "pCacheData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateCudaFunctionNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkCudaFunctionCreateInfoNV* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkCudaFunctionNV* pFunction)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCudaFunctionCreateInfoNV>(pCreateInfo, settings, "const VkCudaFunctionCreateInfoNV*", "pCreateInfo", true, false, 4, dump_json_VkCudaFunctionCreateInfoNV);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCudaFunctionNV>(pFunction, settings, "VkCudaFunctionNV*", "pFunction", false, false, 4, dump_json_VkCudaFunctionNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyCudaModuleNV(ApiDumpInstance& dump_inst, VkDevice device, VkCudaModuleNV module, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkCudaModuleNV>(module, NULL, settings, "VkCudaModuleNV", "module", false, false, 4, dump_json_VkCudaModuleNV);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyCudaFunctionNV(ApiDumpInstance& dump_inst, VkDevice device, VkCudaFunctionNV function, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkCudaFunctionNV>(function, NULL, settings, "VkCudaFunctionNV", "function", false, false, 4, dump_json_VkCudaFunctionNV);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCudaLaunchKernelNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCudaLaunchInfoNV* pLaunchInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCudaLaunchInfoNV>(pLaunchInfo, settings, "const VkCudaLaunchInfoNV*", "pLaunchInfo", true, false, 4, dump_json_VkCudaLaunchInfoNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_METAL_EXT)
void dump_json_vkExportMetalObjectsEXT(ApiDumpInstance& dump_inst, VkDevice device, VkExportMetalObjectsInfoEXT* pMetalObjectsInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkExportMetalObjectsInfoEXT>(pMetalObjectsInfo, settings, "VkExportMetalObjectsInfoEXT*", "pMetalObjectsInfo", true, false, 4, dump_json_VkExportMetalObjectsInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_METAL_EXT
void dump_json_vkCmdSetEvent2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer                   commandBuffer, VkEvent                                             event, const VkDependencyInfo*                             pDependencyInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkEvent>(event, NULL, settings, "VkEvent", "event", false, false, 4, dump_json_VkEvent);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDependencyInfo>(pDependencyInfo, settings, "const VkDependencyInfo*", "pDependencyInfo", true, false, 4, dump_json_VkDependencyInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdResetEvent2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer                   commandBuffer, VkEvent                                             event, VkPipelineStageFlags2               stageMask)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkEvent>(event, NULL, settings, "VkEvent", "event", false, false, 4, dump_json_VkEvent);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineStageFlags2>(stageMask, NULL, settings, "VkPipelineStageFlags2", "stageMask", false, false, 4, dump_json_VkPipelineStageFlags2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdWaitEvents2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer                   commandBuffer, uint32_t                                            eventCount, const VkEvent*                     pEvents, const VkDependencyInfo*            pDependencyInfos)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(eventCount, NULL, settings, "uint32_t", "eventCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkEvent>(pEvents, eventCount, settings, "const VkEvent*", "const VkEvent", "pEvents", false, false, 4, dump_json_VkEvent); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkDependencyInfo>(pDependencyInfos, eventCount, settings, "const VkDependencyInfo*", "const VkDependencyInfo", "pDependencyInfos", true, false, 4, dump_json_VkDependencyInfo); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdPipelineBarrier2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer                   commandBuffer, const VkDependencyInfo*                             pDependencyInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDependencyInfo>(pDependencyInfo, settings, "const VkDependencyInfo*", "pDependencyInfo", true, false, 4, dump_json_VkDependencyInfo);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdWriteTimestamp2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer                   commandBuffer, VkPipelineStageFlags2               stage, VkQueryPool                                         queryPool, uint32_t                                            query)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineStageFlags2>(stage, NULL, settings, "VkPipelineStageFlags2", "stage", false, false, 4, dump_json_VkPipelineStageFlags2);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(query, NULL, settings, "uint32_t", "query", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkQueueSubmit2KHR(ApiDumpInstance& dump_inst, VkResult result, VkQueue                           queue, uint32_t                            submitCount, const VkSubmitInfo2*              pSubmits, VkFence           fence)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkQueue>(queue, NULL, settings, "VkQueue", "queue", false, false, 4, dump_json_VkQueue);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(submitCount, NULL, settings, "uint32_t", "submitCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkSubmitInfo2>(pSubmits, submitCount, settings, "const VkSubmitInfo2*", "const VkSubmitInfo2", "pSubmits", true, false, 4, dump_json_VkSubmitInfo2); // PQA
        settings.stream() << ",\n";
        dump_json_value<const VkFence>(fence, NULL, settings, "VkFence", "fence", false, false, 4, dump_json_VkFence);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdWriteBufferMarker2AMD(ApiDumpInstance& dump_inst, VkCommandBuffer                   commandBuffer, VkPipelineStageFlags2               stage, VkBuffer                                            dstBuffer, VkDeviceSize                                        dstOffset, uint32_t                                            marker)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineStageFlags2>(stage, NULL, settings, "VkPipelineStageFlags2", "stage", false, false, 4, dump_json_VkPipelineStageFlags2);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(dstBuffer, NULL, settings, "VkBuffer", "dstBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(dstOffset, NULL, settings, "VkDeviceSize", "dstOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(marker, NULL, settings, "uint32_t", "marker", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetQueueCheckpointData2NV(ApiDumpInstance& dump_inst, VkQueue queue, uint32_t* pCheckpointDataCount, VkCheckpointData2NV* pCheckpointData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkQueue>(queue, NULL, settings, "VkQueue", "queue", false, false, 4, dump_json_VkQueue);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pCheckpointDataCount, settings, "uint32_t*", "pCheckpointDataCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkCheckpointData2NV>(pCheckpointData, *pCheckpointDataCount, settings, "VkCheckpointData2NV*", "VkCheckpointData2NV", "pCheckpointData", true, false, 4, dump_json_VkCheckpointData2NV); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDescriptorSetLayoutSizeEXT(ApiDumpInstance& dump_inst, VkDevice device, VkDescriptorSetLayout layout, VkDeviceSize* pLayoutSizeInBytes)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDescriptorSetLayout>(layout, NULL, settings, "VkDescriptorSetLayout", "layout", false, false, 4, dump_json_VkDescriptorSetLayout);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceSize>(pLayoutSizeInBytes, settings, "VkDeviceSize*", "pLayoutSizeInBytes", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDescriptorSetLayoutBindingOffsetEXT(ApiDumpInstance& dump_inst, VkDevice device, VkDescriptorSetLayout layout, uint32_t binding, VkDeviceSize* pOffset)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDescriptorSetLayout>(layout, NULL, settings, "VkDescriptorSetLayout", "layout", false, false, 4, dump_json_VkDescriptorSetLayout);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(binding, NULL, settings, "uint32_t", "binding", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceSize>(pOffset, settings, "VkDeviceSize*", "pOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDescriptorEXT(ApiDumpInstance& dump_inst, VkDevice device, const VkDescriptorGetInfoEXT* pDescriptorInfo, size_t dataSize, void* pDescriptor)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorGetInfoEXT>(pDescriptorInfo, settings, "const VkDescriptorGetInfoEXT*", "pDescriptorInfo", true, false, 4, dump_json_VkDescriptorGetInfoEXT);
        settings.stream() << ",\n";
        dump_json_value<const size_t>(dataSize, NULL, settings, "size_t", "dataSize", false, false, 4, dump_json_size_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pDescriptor, NULL, settings, "void*", "pDescriptor", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBindDescriptorBuffersEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t bufferCount, const VkDescriptorBufferBindingInfoEXT* pBindingInfos)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(bufferCount, NULL, settings, "uint32_t", "bufferCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkDescriptorBufferBindingInfoEXT>(pBindingInfos, bufferCount, settings, "const VkDescriptorBufferBindingInfoEXT*", "const VkDescriptorBufferBindingInfoEXT", "pBindingInfos", true, false, 4, dump_json_VkDescriptorBufferBindingInfoEXT); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDescriptorBufferOffsetsEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkPipelineBindPoint pipelineBindPoint, VkPipelineLayout layout, uint32_t firstSet, uint32_t setCount, const uint32_t* pBufferIndices, const VkDeviceSize* pOffsets)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineBindPoint>(pipelineBindPoint, NULL, settings, "VkPipelineBindPoint", "pipelineBindPoint", false, false, 4, dump_json_VkPipelineBindPoint);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineLayout>(layout, NULL, settings, "VkPipelineLayout", "layout", false, false, 4, dump_json_VkPipelineLayout);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstSet, NULL, settings, "uint32_t", "firstSet", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(setCount, NULL, settings, "uint32_t", "setCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const uint32_t>(pBufferIndices, setCount, settings, "const uint32_t*", "const uint32_t", "pBufferIndices", false, false, 4, dump_json_uint32_t); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkDeviceSize>(pOffsets, setCount, settings, "const VkDeviceSize*", "const VkDeviceSize", "pOffsets", false, false, 4, dump_json_VkDeviceSize); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBindDescriptorBufferEmbeddedSamplersEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkPipelineBindPoint pipelineBindPoint, VkPipelineLayout layout, uint32_t set)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineBindPoint>(pipelineBindPoint, NULL, settings, "VkPipelineBindPoint", "pipelineBindPoint", false, false, 4, dump_json_VkPipelineBindPoint);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineLayout>(layout, NULL, settings, "VkPipelineLayout", "layout", false, false, 4, dump_json_VkPipelineLayout);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(set, NULL, settings, "uint32_t", "set", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetBufferOpaqueCaptureDescriptorDataEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkBufferCaptureDescriptorDataInfoEXT* pInfo, void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBufferCaptureDescriptorDataInfoEXT>(pInfo, settings, "const VkBufferCaptureDescriptorDataInfoEXT*", "pInfo", true, false, 4, dump_json_VkBufferCaptureDescriptorDataInfoEXT);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetImageOpaqueCaptureDescriptorDataEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkImageCaptureDescriptorDataInfoEXT* pInfo, void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageCaptureDescriptorDataInfoEXT>(pInfo, settings, "const VkImageCaptureDescriptorDataInfoEXT*", "pInfo", true, false, 4, dump_json_VkImageCaptureDescriptorDataInfoEXT);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetImageViewOpaqueCaptureDescriptorDataEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkImageViewCaptureDescriptorDataInfoEXT* pInfo, void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageViewCaptureDescriptorDataInfoEXT>(pInfo, settings, "const VkImageViewCaptureDescriptorDataInfoEXT*", "pInfo", true, false, 4, dump_json_VkImageViewCaptureDescriptorDataInfoEXT);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetSamplerOpaqueCaptureDescriptorDataEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkSamplerCaptureDescriptorDataInfoEXT* pInfo, void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSamplerCaptureDescriptorDataInfoEXT>(pInfo, settings, "const VkSamplerCaptureDescriptorDataInfoEXT*", "pInfo", true, false, 4, dump_json_VkSamplerCaptureDescriptorDataInfoEXT);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetAccelerationStructureOpaqueCaptureDescriptorDataEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkAccelerationStructureCaptureDescriptorDataInfoEXT* pInfo, void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAccelerationStructureCaptureDescriptorDataInfoEXT>(pInfo, settings, "const VkAccelerationStructureCaptureDescriptorDataInfoEXT*", "pInfo", true, false, 4, dump_json_VkAccelerationStructureCaptureDescriptorDataInfoEXT);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetFragmentShadingRateEnumNV(ApiDumpInstance& dump_inst, VkCommandBuffer           commandBuffer, VkFragmentShadingRateNV                     shadingRate, const VkFragmentShadingRateCombinerOpKHR    combinerOps[2])
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkFragmentShadingRateNV>(shadingRate, NULL, settings, "VkFragmentShadingRateNV", "shadingRate", false, false, 4, dump_json_VkFragmentShadingRateNV);
        settings.stream() << ",\n";
        dump_json_array<const VkFragmentShadingRateCombinerOpKHR>(combinerOps, 2, settings, "const VkFragmentShadingRateCombinerOpKHR[2]", "const VkFragmentShadingRateCombinerOpKHR", "combinerOps", false, false, 4, dump_json_VkFragmentShadingRateCombinerOpKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawMeshTasksEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t groupCountX, uint32_t groupCountY, uint32_t groupCountZ)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCountX, NULL, settings, "uint32_t", "groupCountX", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCountY, NULL, settings, "uint32_t", "groupCountY", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCountZ, NULL, settings, "uint32_t", "groupCountZ", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawMeshTasksIndirectEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset, uint32_t drawCount, uint32_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(drawCount, NULL, settings, "uint32_t", "drawCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawMeshTasksIndirectCountEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset, VkBuffer countBuffer, VkDeviceSize countBufferOffset, uint32_t maxDrawCount, uint32_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(countBuffer, NULL, settings, "VkBuffer", "countBuffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(countBufferOffset, NULL, settings, "VkDeviceSize", "countBufferOffset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(maxDrawCount, NULL, settings, "uint32_t", "maxDrawCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyBuffer2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCopyBufferInfo2* pCopyBufferInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyBufferInfo2>(pCopyBufferInfo, settings, "const VkCopyBufferInfo2*", "pCopyBufferInfo", true, false, 4, dump_json_VkCopyBufferInfo2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyImage2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCopyImageInfo2* pCopyImageInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyImageInfo2>(pCopyImageInfo, settings, "const VkCopyImageInfo2*", "pCopyImageInfo", true, false, 4, dump_json_VkCopyImageInfo2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyBufferToImage2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCopyBufferToImageInfo2* pCopyBufferToImageInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyBufferToImageInfo2>(pCopyBufferToImageInfo, settings, "const VkCopyBufferToImageInfo2*", "pCopyBufferToImageInfo", true, false, 4, dump_json_VkCopyBufferToImageInfo2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyImageToBuffer2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCopyImageToBufferInfo2* pCopyImageToBufferInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyImageToBufferInfo2>(pCopyImageToBufferInfo, settings, "const VkCopyImageToBufferInfo2*", "pCopyImageToBufferInfo", true, false, 4, dump_json_VkCopyImageToBufferInfo2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBlitImage2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkBlitImageInfo2* pBlitImageInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBlitImageInfo2>(pBlitImageInfo, settings, "const VkBlitImageInfo2*", "pBlitImageInfo", true, false, 4, dump_json_VkBlitImageInfo2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdResolveImage2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkResolveImageInfo2* pResolveImageInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkResolveImageInfo2>(pResolveImageInfo, settings, "const VkResolveImageInfo2*", "pResolveImageInfo", true, false, 4, dump_json_VkResolveImageInfo2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceFaultInfoEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDeviceFaultCountsEXT* pFaultCounts, VkDeviceFaultInfoEXT* pFaultInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceFaultCountsEXT>(pFaultCounts, settings, "VkDeviceFaultCountsEXT*", "pFaultCounts", true, false, 4, dump_json_VkDeviceFaultCountsEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceFaultInfoEXT>(pFaultInfo, settings, "VkDeviceFaultInfoEXT*", "pFaultInfo", true, false, 4, dump_json_VkDeviceFaultInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_vkAcquireWinrtDisplayNV(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, VkDisplayKHR display)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDisplayKHR>(display, NULL, settings, "VkDisplayKHR", "display", false, false, 4, dump_json_VkDisplayKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_WIN32_KHR)
void dump_json_vkGetWinrtDisplayNV(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t deviceRelativeId, VkDisplayKHR* pDisplay)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(deviceRelativeId, NULL, settings, "uint32_t", "deviceRelativeId", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDisplayKHR>(pDisplay, settings, "VkDisplayKHR*", "pDisplay", false, false, 4, dump_json_VkDisplayKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_WIN32_KHR
#if defined(VK_USE_PLATFORM_DIRECTFB_EXT)
void dump_json_vkCreateDirectFBSurfaceEXT(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkDirectFBSurfaceCreateInfoEXT* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSurfaceKHR* pSurface)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDirectFBSurfaceCreateInfoEXT>(pCreateInfo, settings, "const VkDirectFBSurfaceCreateInfoEXT*", "pCreateInfo", true, false, 4, dump_json_VkDirectFBSurfaceCreateInfoEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceKHR>(pSurface, settings, "VkSurfaceKHR*", "pSurface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_DIRECTFB_EXT
#if defined(VK_USE_PLATFORM_DIRECTFB_EXT)
void dump_json_vkGetPhysicalDeviceDirectFBPresentationSupportEXT(ApiDumpInstance& dump_inst, VkBool32 result, VkPhysicalDevice physicalDevice, uint32_t queueFamilyIndex, IDirectFB* dfb)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkBool32(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(queueFamilyIndex, NULL, settings, "uint32_t", "queueFamilyIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_pointer<const IDirectFB>(dfb, settings, "IDirectFB*", "dfb", false, false, 4, dump_json_IDirectFB);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_DIRECTFB_EXT
void dump_json_vkCmdTraceRaysKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkStridedDeviceAddressRegionKHR* pRaygenShaderBindingTable, const VkStridedDeviceAddressRegionKHR* pMissShaderBindingTable, const VkStridedDeviceAddressRegionKHR* pHitShaderBindingTable, const VkStridedDeviceAddressRegionKHR* pCallableShaderBindingTable, uint32_t width, uint32_t height, uint32_t depth)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkStridedDeviceAddressRegionKHR>(pRaygenShaderBindingTable, settings, "const VkStridedDeviceAddressRegionKHR*", "pRaygenShaderBindingTable", true, false, 4, dump_json_VkStridedDeviceAddressRegionKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkStridedDeviceAddressRegionKHR>(pMissShaderBindingTable, settings, "const VkStridedDeviceAddressRegionKHR*", "pMissShaderBindingTable", true, false, 4, dump_json_VkStridedDeviceAddressRegionKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkStridedDeviceAddressRegionKHR>(pHitShaderBindingTable, settings, "const VkStridedDeviceAddressRegionKHR*", "pHitShaderBindingTable", true, false, 4, dump_json_VkStridedDeviceAddressRegionKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkStridedDeviceAddressRegionKHR>(pCallableShaderBindingTable, settings, "const VkStridedDeviceAddressRegionKHR*", "pCallableShaderBindingTable", true, false, 4, dump_json_VkStridedDeviceAddressRegionKHR);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(width, NULL, settings, "uint32_t", "width", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(height, NULL, settings, "uint32_t", "height", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(depth, NULL, settings, "uint32_t", "depth", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateRayTracingPipelinesKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDeferredOperationKHR deferredOperation, VkPipelineCache pipelineCache, uint32_t createInfoCount, const VkRayTracingPipelineCreateInfoKHR* pCreateInfos, const VkAllocationCallbacks* pAllocator, VkPipeline* pPipelines)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeferredOperationKHR>(deferredOperation, NULL, settings, "VkDeferredOperationKHR", "deferredOperation", false, false, 4, dump_json_VkDeferredOperationKHR);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineCache>(pipelineCache, NULL, settings, "VkPipelineCache", "pipelineCache", false, false, 4, dump_json_VkPipelineCache);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(createInfoCount, NULL, settings, "uint32_t", "createInfoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkRayTracingPipelineCreateInfoKHR>(pCreateInfos, createInfoCount, settings, "const VkRayTracingPipelineCreateInfoKHR*", "const VkRayTracingPipelineCreateInfoKHR", "pCreateInfos", true, false, 4, dump_json_VkRayTracingPipelineCreateInfoKHR); // PQA
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_array<const VkPipeline>(pPipelines, createInfoCount, settings, "VkPipeline*", "VkPipeline", "pPipelines", false, false, 4, dump_json_VkPipeline); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetRayTracingCaptureReplayShaderGroupHandlesKHR(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkPipeline pipeline, uint32_t firstGroup, uint32_t groupCount, size_t dataSize, void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipeline>(pipeline, NULL, settings, "VkPipeline", "pipeline", false, false, 4, dump_json_VkPipeline);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstGroup, NULL, settings, "uint32_t", "firstGroup", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCount, NULL, settings, "uint32_t", "groupCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const size_t>(dataSize, NULL, settings, "size_t", "dataSize", false, false, 4, dump_json_size_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdTraceRaysIndirectKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkStridedDeviceAddressRegionKHR* pRaygenShaderBindingTable, const VkStridedDeviceAddressRegionKHR* pMissShaderBindingTable, const VkStridedDeviceAddressRegionKHR* pHitShaderBindingTable, const VkStridedDeviceAddressRegionKHR* pCallableShaderBindingTable, VkDeviceAddress indirectDeviceAddress)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkStridedDeviceAddressRegionKHR>(pRaygenShaderBindingTable, settings, "const VkStridedDeviceAddressRegionKHR*", "pRaygenShaderBindingTable", true, false, 4, dump_json_VkStridedDeviceAddressRegionKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkStridedDeviceAddressRegionKHR>(pMissShaderBindingTable, settings, "const VkStridedDeviceAddressRegionKHR*", "pMissShaderBindingTable", true, false, 4, dump_json_VkStridedDeviceAddressRegionKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkStridedDeviceAddressRegionKHR>(pHitShaderBindingTable, settings, "const VkStridedDeviceAddressRegionKHR*", "pHitShaderBindingTable", true, false, 4, dump_json_VkStridedDeviceAddressRegionKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkStridedDeviceAddressRegionKHR>(pCallableShaderBindingTable, settings, "const VkStridedDeviceAddressRegionKHR*", "pCallableShaderBindingTable", true, false, 4, dump_json_VkStridedDeviceAddressRegionKHR);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceAddress>(indirectDeviceAddress, NULL, settings, "VkDeviceAddress", "indirectDeviceAddress", false, false, 4, dump_json_VkDeviceAddress);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetRayTracingShaderGroupStackSizeKHR(ApiDumpInstance& dump_inst, VkDeviceSize result, VkDevice device, VkPipeline pipeline, uint32_t group, VkShaderGroupShaderKHR groupShader)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkDeviceSize(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkPipeline>(pipeline, NULL, settings, "VkPipeline", "pipeline", false, false, 4, dump_json_VkPipeline);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(group, NULL, settings, "uint32_t", "group", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const VkShaderGroupShaderKHR>(groupShader, NULL, settings, "VkShaderGroupShaderKHR", "groupShader", false, false, 4, dump_json_VkShaderGroupShaderKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetRayTracingPipelineStackSizeKHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t pipelineStackSize)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(pipelineStackSize, NULL, settings, "uint32_t", "pipelineStackSize", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetVertexInputEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t vertexBindingDescriptionCount, const VkVertexInputBindingDescription2EXT* pVertexBindingDescriptions, uint32_t vertexAttributeDescriptionCount, const VkVertexInputAttributeDescription2EXT* pVertexAttributeDescriptions)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(vertexBindingDescriptionCount, NULL, settings, "uint32_t", "vertexBindingDescriptionCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkVertexInputBindingDescription2EXT>(pVertexBindingDescriptions, vertexBindingDescriptionCount, settings, "const VkVertexInputBindingDescription2EXT*", "const VkVertexInputBindingDescription2EXT", "pVertexBindingDescriptions", true, false, 4, dump_json_VkVertexInputBindingDescription2EXT); // PQA
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(vertexAttributeDescriptionCount, NULL, settings, "uint32_t", "vertexAttributeDescriptionCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkVertexInputAttributeDescription2EXT>(pVertexAttributeDescriptions, vertexAttributeDescriptionCount, settings, "const VkVertexInputAttributeDescription2EXT*", "const VkVertexInputAttributeDescription2EXT", "pVertexAttributeDescriptions", true, false, 4, dump_json_VkVertexInputAttributeDescription2EXT); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_vkGetMemoryZirconHandleFUCHSIA(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkMemoryGetZirconHandleInfoFUCHSIA* pGetZirconHandleInfo, zx_handle_t* pZirconHandle)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryGetZirconHandleInfoFUCHSIA>(pGetZirconHandleInfo, settings, "const VkMemoryGetZirconHandleInfoFUCHSIA*", "pGetZirconHandleInfo", true, false, 4, dump_json_VkMemoryGetZirconHandleInfoFUCHSIA);
        settings.stream() << ",\n";
        dump_json_pointer<const zx_handle_t>(pZirconHandle, settings, "zx_handle_t*", "pZirconHandle", false, false, 4, dump_json_zx_handle_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_vkGetMemoryZirconHandlePropertiesFUCHSIA(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkExternalMemoryHandleTypeFlagBits handleType, zx_handle_t zirconHandle, VkMemoryZirconHandlePropertiesFUCHSIA* pMemoryZirconHandleProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkExternalMemoryHandleTypeFlagBits>(handleType, NULL, settings, "VkExternalMemoryHandleTypeFlagBits", "handleType", false, false, 4, dump_json_VkExternalMemoryHandleTypeFlagBits);
        settings.stream() << ",\n";
        dump_json_value<const zx_handle_t>(zirconHandle, NULL, settings, "zx_handle_t", "zirconHandle", false, false, 4, dump_json_zx_handle_t);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryZirconHandlePropertiesFUCHSIA>(pMemoryZirconHandleProperties, settings, "VkMemoryZirconHandlePropertiesFUCHSIA*", "pMemoryZirconHandleProperties", true, false, 4, dump_json_VkMemoryZirconHandlePropertiesFUCHSIA);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_vkImportSemaphoreZirconHandleFUCHSIA(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkImportSemaphoreZirconHandleInfoFUCHSIA* pImportSemaphoreZirconHandleInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImportSemaphoreZirconHandleInfoFUCHSIA>(pImportSemaphoreZirconHandleInfo, settings, "const VkImportSemaphoreZirconHandleInfoFUCHSIA*", "pImportSemaphoreZirconHandleInfo", true, false, 4, dump_json_VkImportSemaphoreZirconHandleInfoFUCHSIA);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_vkGetSemaphoreZirconHandleFUCHSIA(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkSemaphoreGetZirconHandleInfoFUCHSIA* pGetZirconHandleInfo, zx_handle_t* pZirconHandle)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSemaphoreGetZirconHandleInfoFUCHSIA>(pGetZirconHandleInfo, settings, "const VkSemaphoreGetZirconHandleInfoFUCHSIA*", "pGetZirconHandleInfo", true, false, 4, dump_json_VkSemaphoreGetZirconHandleInfoFUCHSIA);
        settings.stream() << ",\n";
        dump_json_pointer<const zx_handle_t>(pZirconHandle, settings, "zx_handle_t*", "pZirconHandle", false, false, 4, dump_json_zx_handle_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_vkCreateBufferCollectionFUCHSIA(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkBufferCollectionCreateInfoFUCHSIA* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkBufferCollectionFUCHSIA* pCollection)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBufferCollectionCreateInfoFUCHSIA>(pCreateInfo, settings, "const VkBufferCollectionCreateInfoFUCHSIA*", "pCreateInfo", true, false, 4, dump_json_VkBufferCollectionCreateInfoFUCHSIA);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBufferCollectionFUCHSIA>(pCollection, settings, "VkBufferCollectionFUCHSIA*", "pCollection", false, false, 4, dump_json_VkBufferCollectionFUCHSIA);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_vkSetBufferCollectionImageConstraintsFUCHSIA(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkBufferCollectionFUCHSIA collection, const VkImageConstraintsInfoFUCHSIA* pImageConstraintsInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkBufferCollectionFUCHSIA>(collection, NULL, settings, "VkBufferCollectionFUCHSIA", "collection", false, false, 4, dump_json_VkBufferCollectionFUCHSIA);
        settings.stream() << ",\n";
        dump_json_pointer<const VkImageConstraintsInfoFUCHSIA>(pImageConstraintsInfo, settings, "const VkImageConstraintsInfoFUCHSIA*", "pImageConstraintsInfo", true, false, 4, dump_json_VkImageConstraintsInfoFUCHSIA);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_vkSetBufferCollectionBufferConstraintsFUCHSIA(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkBufferCollectionFUCHSIA collection, const VkBufferConstraintsInfoFUCHSIA* pBufferConstraintsInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkBufferCollectionFUCHSIA>(collection, NULL, settings, "VkBufferCollectionFUCHSIA", "collection", false, false, 4, dump_json_VkBufferCollectionFUCHSIA);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBufferConstraintsInfoFUCHSIA>(pBufferConstraintsInfo, settings, "const VkBufferConstraintsInfoFUCHSIA*", "pBufferConstraintsInfo", true, false, 4, dump_json_VkBufferConstraintsInfoFUCHSIA);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_vkDestroyBufferCollectionFUCHSIA(ApiDumpInstance& dump_inst, VkDevice device, VkBufferCollectionFUCHSIA collection, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkBufferCollectionFUCHSIA>(collection, NULL, settings, "VkBufferCollectionFUCHSIA", "collection", false, false, 4, dump_json_VkBufferCollectionFUCHSIA);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_FUCHSIA
#if defined(VK_USE_PLATFORM_FUCHSIA)
void dump_json_vkGetBufferCollectionPropertiesFUCHSIA(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkBufferCollectionFUCHSIA collection, VkBufferCollectionPropertiesFUCHSIA* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkBufferCollectionFUCHSIA>(collection, NULL, settings, "VkBufferCollectionFUCHSIA", "collection", false, false, 4, dump_json_VkBufferCollectionFUCHSIA);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBufferCollectionPropertiesFUCHSIA>(pProperties, settings, "VkBufferCollectionPropertiesFUCHSIA*", "pProperties", true, false, 4, dump_json_VkBufferCollectionPropertiesFUCHSIA);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_FUCHSIA
void dump_json_vkGetDeviceSubpassShadingMaxWorkgroupSizeHUAWEI(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkRenderPass renderpass, VkExtent2D* pMaxWorkgroupSize)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkRenderPass>(renderpass, NULL, settings, "VkRenderPass", "renderpass", false, false, 4, dump_json_VkRenderPass);
        settings.stream() << ",\n";
        dump_json_array<const VkExtent2D>(pMaxWorkgroupSize, 1, settings, "VkExtent2D*", "VkExtent2D", "pMaxWorkgroupSize", true, false, 4, dump_json_VkExtent2D); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSubpassShadingHUAWEI(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBindInvocationMaskHUAWEI(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkImageView imageView, VkImageLayout imageLayout)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkImageView>(imageView, NULL, settings, "VkImageView", "imageView", false, false, 4, dump_json_VkImageView);
        settings.stream() << ",\n";
        dump_json_value<const VkImageLayout>(imageLayout, NULL, settings, "VkImageLayout", "imageLayout", false, false, 4, dump_json_VkImageLayout);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetMemoryRemoteAddressNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkMemoryGetRemoteAddressInfoNV* pMemoryGetRemoteAddressInfo, VkRemoteAddressNV* pAddress)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryGetRemoteAddressInfoNV>(pMemoryGetRemoteAddressInfo, settings, "const VkMemoryGetRemoteAddressInfoNV*", "pMemoryGetRemoteAddressInfo", true, false, 4, dump_json_VkMemoryGetRemoteAddressInfoNV);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRemoteAddressNV>(pAddress, settings, "VkRemoteAddressNV*", "pAddress", false, false, 4, dump_json_VkRemoteAddressNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPipelinePropertiesEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkPipelineInfoEXT* pPipelineInfo, VkBaseOutStructure* pPipelineProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPipelineInfoKHR>(pPipelineInfo, settings, "const VkPipelineInfoEXT*", "pPipelineInfo", true, false, 4, dump_json_VkPipelineInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBaseOutStructure>(pPipelineProperties, settings, "VkBaseOutStructure*", "pPipelineProperties", true, false, 4, dump_json_VkBaseOutStructure);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetPatchControlPointsEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t patchControlPoints)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(patchControlPoints, NULL, settings, "uint32_t", "patchControlPoints", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetRasterizerDiscardEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 rasterizerDiscardEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(rasterizerDiscardEnable, NULL, settings, "VkBool32", "rasterizerDiscardEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthBiasEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 depthBiasEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(depthBiasEnable, NULL, settings, "VkBool32", "depthBiasEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetLogicOpEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkLogicOp logicOp)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkLogicOp>(logicOp, NULL, settings, "VkLogicOp", "logicOp", false, false, 4, dump_json_VkLogicOp);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetPrimitiveRestartEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 primitiveRestartEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(primitiveRestartEnable, NULL, settings, "VkBool32", "primitiveRestartEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
void dump_json_vkCreateScreenSurfaceQNX(ApiDumpInstance& dump_inst, VkResult result, VkInstance instance, const VkScreenSurfaceCreateInfoQNX* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkSurfaceKHR* pSurface)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkInstance>(instance, NULL, settings, "VkInstance", "instance", false, false, 4, dump_json_VkInstance);
        settings.stream() << ",\n";
        dump_json_pointer<const VkScreenSurfaceCreateInfoQNX>(pCreateInfo, settings, "const VkScreenSurfaceCreateInfoQNX*", "pCreateInfo", true, false, 4, dump_json_VkScreenSurfaceCreateInfoQNX);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSurfaceKHR>(pSurface, settings, "VkSurfaceKHR*", "pSurface", false, false, 4, dump_json_VkSurfaceKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_SCREEN_QNX
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
void dump_json_vkGetPhysicalDeviceScreenPresentationSupportQNX(ApiDumpInstance& dump_inst, VkBool32 result, VkPhysicalDevice physicalDevice, uint32_t queueFamilyIndex, struct _screen_window* window)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkBool32(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(queueFamilyIndex, NULL, settings, "uint32_t", "queueFamilyIndex", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const _screen_window*>(window, NULL, settings, "struct _screen_window*", "window", false, false, 4, dump_json__screen_window);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_SCREEN_QNX
void dump_json_vkCmdSetColorWriteEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer       commandBuffer, uint32_t                                attachmentCount, const VkBool32*   pColorWriteEnables)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(attachmentCount, NULL, settings, "uint32_t", "attachmentCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBool32>(pColorWriteEnables, attachmentCount, settings, "const VkBool32*", "const VkBool32", "pColorWriteEnables", false, false, 4, dump_json_VkBool32); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdTraceRaysIndirect2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkDeviceAddress indirectDeviceAddress)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceAddress>(indirectDeviceAddress, NULL, settings, "VkDeviceAddress", "indirectDeviceAddress", false, false, 4, dump_json_VkDeviceAddress);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawMultiEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t drawCount, const VkMultiDrawInfoEXT* pVertexInfo, uint32_t instanceCount, uint32_t firstInstance, uint32_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(drawCount, NULL, settings, "uint32_t", "drawCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkMultiDrawInfoEXT>(pVertexInfo, drawCount, settings, "const VkMultiDrawInfoEXT*", "const VkMultiDrawInfoEXT", "pVertexInfo", true, false, 4, dump_json_VkMultiDrawInfoEXT); // PQA
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(instanceCount, NULL, settings, "uint32_t", "instanceCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstInstance, NULL, settings, "uint32_t", "firstInstance", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawMultiIndexedEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t drawCount, const VkMultiDrawIndexedInfoEXT* pIndexInfo, uint32_t instanceCount, uint32_t firstInstance, uint32_t stride, const int32_t* pVertexOffset)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(drawCount, NULL, settings, "uint32_t", "drawCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkMultiDrawIndexedInfoEXT>(pIndexInfo, drawCount, settings, "const VkMultiDrawIndexedInfoEXT*", "const VkMultiDrawIndexedInfoEXT", "pIndexInfo", true, false, 4, dump_json_VkMultiDrawIndexedInfoEXT); // PQA
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(instanceCount, NULL, settings, "uint32_t", "instanceCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstInstance, NULL, settings, "uint32_t", "firstInstance", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_pointer<const int32_t>(pVertexOffset, settings, "const int32_t*", "pVertexOffset", false, false, 4, dump_json_int32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateMicromapEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice                                           device, const VkMicromapCreateInfoEXT*        pCreateInfo, const VkAllocationCallbacks*       pAllocator, VkMicromapEXT*                        pMicromap)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMicromapCreateInfoEXT>(pCreateInfo, settings, "const VkMicromapCreateInfoEXT*", "pCreateInfo", true, false, 4, dump_json_VkMicromapCreateInfoEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMicromapEXT>(pMicromap, settings, "VkMicromapEXT*", "pMicromap", false, false, 4, dump_json_VkMicromapEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyMicromapEXT(ApiDumpInstance& dump_inst, VkDevice device, VkMicromapEXT micromap, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkMicromapEXT>(micromap, NULL, settings, "VkMicromapEXT", "micromap", false, false, 4, dump_json_VkMicromapEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBuildMicromapsEXT(ApiDumpInstance& dump_inst, VkCommandBuffer                                    commandBuffer, uint32_t infoCount, const VkMicromapBuildInfoEXT* pInfos)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(infoCount, NULL, settings, "uint32_t", "infoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkMicromapBuildInfoEXT>(pInfos, infoCount, settings, "const VkMicromapBuildInfoEXT*", "const VkMicromapBuildInfoEXT", "pInfos", true, false, 4, dump_json_VkMicromapBuildInfoEXT); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkBuildMicromapsEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice                                           device, VkDeferredOperationKHR deferredOperation, uint32_t infoCount, const VkMicromapBuildInfoEXT* pInfos)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeferredOperationKHR>(deferredOperation, NULL, settings, "VkDeferredOperationKHR", "deferredOperation", false, false, 4, dump_json_VkDeferredOperationKHR);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(infoCount, NULL, settings, "uint32_t", "infoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkMicromapBuildInfoEXT>(pInfos, infoCount, settings, "const VkMicromapBuildInfoEXT*", "const VkMicromapBuildInfoEXT", "pInfos", true, false, 4, dump_json_VkMicromapBuildInfoEXT); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCopyMicromapEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDeferredOperationKHR deferredOperation, const VkCopyMicromapInfoEXT* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeferredOperationKHR>(deferredOperation, NULL, settings, "VkDeferredOperationKHR", "deferredOperation", false, false, 4, dump_json_VkDeferredOperationKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyMicromapInfoEXT>(pInfo, settings, "const VkCopyMicromapInfoEXT*", "pInfo", true, false, 4, dump_json_VkCopyMicromapInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCopyMicromapToMemoryEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDeferredOperationKHR deferredOperation, const VkCopyMicromapToMemoryInfoEXT* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeferredOperationKHR>(deferredOperation, NULL, settings, "VkDeferredOperationKHR", "deferredOperation", false, false, 4, dump_json_VkDeferredOperationKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyMicromapToMemoryInfoEXT>(pInfo, settings, "const VkCopyMicromapToMemoryInfoEXT*", "pInfo", true, false, 4, dump_json_VkCopyMicromapToMemoryInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCopyMemoryToMicromapEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkDeferredOperationKHR deferredOperation, const VkCopyMemoryToMicromapInfoEXT* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeferredOperationKHR>(deferredOperation, NULL, settings, "VkDeferredOperationKHR", "deferredOperation", false, false, 4, dump_json_VkDeferredOperationKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyMemoryToMicromapInfoEXT>(pInfo, settings, "const VkCopyMemoryToMicromapInfoEXT*", "pInfo", true, false, 4, dump_json_VkCopyMemoryToMicromapInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkWriteMicromapsPropertiesEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t micromapCount, const VkMicromapEXT* pMicromaps, VkQueryType  queryType, size_t       dataSize, void* pData, size_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(micromapCount, NULL, settings, "uint32_t", "micromapCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkMicromapEXT>(pMicromaps, micromapCount, settings, "const VkMicromapEXT*", "const VkMicromapEXT", "pMicromaps", false, false, 4, dump_json_VkMicromapEXT); // PQA
        settings.stream() << ",\n";
        dump_json_value<const VkQueryType>(queryType, NULL, settings, "VkQueryType", "queryType", false, false, 4, dump_json_VkQueryType);
        settings.stream() << ",\n";
        dump_json_value<const size_t>(dataSize, NULL, settings, "size_t", "dataSize", false, false, 4, dump_json_size_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << ",\n";
        dump_json_value<const size_t>(stride, NULL, settings, "size_t", "stride", false, false, 4, dump_json_size_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyMicromapEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCopyMicromapInfoEXT* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyMicromapInfoEXT>(pInfo, settings, "const VkCopyMicromapInfoEXT*", "pInfo", true, false, 4, dump_json_VkCopyMicromapInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyMicromapToMemoryEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCopyMicromapToMemoryInfoEXT* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyMicromapToMemoryInfoEXT>(pInfo, settings, "const VkCopyMicromapToMemoryInfoEXT*", "pInfo", true, false, 4, dump_json_VkCopyMicromapToMemoryInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyMemoryToMicromapEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkCopyMemoryToMicromapInfoEXT* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkCopyMemoryToMicromapInfoEXT>(pInfo, settings, "const VkCopyMemoryToMicromapInfoEXT*", "pInfo", true, false, 4, dump_json_VkCopyMemoryToMicromapInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdWriteMicromapsPropertiesEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t micromapCount, const VkMicromapEXT* pMicromaps, VkQueryType queryType, VkQueryPool queryPool, uint32_t firstQuery)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(micromapCount, NULL, settings, "uint32_t", "micromapCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkMicromapEXT>(pMicromaps, micromapCount, settings, "const VkMicromapEXT*", "const VkMicromapEXT", "pMicromaps", false, false, 4, dump_json_VkMicromapEXT); // PQA
        settings.stream() << ",\n";
        dump_json_value<const VkQueryType>(queryType, NULL, settings, "VkQueryType", "queryType", false, false, 4, dump_json_VkQueryType);
        settings.stream() << ",\n";
        dump_json_value<const VkQueryPool>(queryPool, NULL, settings, "VkQueryPool", "queryPool", false, false, 4, dump_json_VkQueryPool);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstQuery, NULL, settings, "uint32_t", "firstQuery", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceMicromapCompatibilityEXT(ApiDumpInstance& dump_inst, VkDevice device, const VkMicromapVersionInfoEXT* pVersionInfo, VkAccelerationStructureCompatibilityKHR* pCompatibility)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMicromapVersionInfoEXT>(pVersionInfo, settings, "const VkMicromapVersionInfoEXT*", "pVersionInfo", true, false, 4, dump_json_VkMicromapVersionInfoEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAccelerationStructureCompatibilityKHR>(pCompatibility, settings, "VkAccelerationStructureCompatibilityKHR*", "pCompatibility", false, false, 4, dump_json_VkAccelerationStructureCompatibilityKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetMicromapBuildSizesEXT(ApiDumpInstance& dump_inst, VkDevice                                            device, VkAccelerationStructureBuildTypeKHR                 buildType, const VkMicromapBuildInfoEXT*  pBuildInfo, VkMicromapBuildSizesInfoEXT*           pSizeInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkAccelerationStructureBuildTypeKHR>(buildType, NULL, settings, "VkAccelerationStructureBuildTypeKHR", "buildType", false, false, 4, dump_json_VkAccelerationStructureBuildTypeKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMicromapBuildInfoEXT>(pBuildInfo, settings, "const VkMicromapBuildInfoEXT*", "pBuildInfo", true, false, 4, dump_json_VkMicromapBuildInfoEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMicromapBuildSizesInfoEXT>(pSizeInfo, settings, "VkMicromapBuildSizesInfoEXT*", "pSizeInfo", true, false, 4, dump_json_VkMicromapBuildSizesInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawClusterHUAWEI(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t groupCountX, uint32_t groupCountY, uint32_t groupCountZ)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCountX, NULL, settings, "uint32_t", "groupCountX", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCountY, NULL, settings, "uint32_t", "groupCountY", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(groupCountZ, NULL, settings, "uint32_t", "groupCountZ", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDrawClusterIndirectHUAWEI(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkSetDeviceMemoryPriorityEXT(ApiDumpInstance& dump_inst, VkDevice       device, VkDeviceMemory memory, float          priority)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceMemory>(memory, NULL, settings, "VkDeviceMemory", "memory", false, false, 4, dump_json_VkDeviceMemory);
        settings.stream() << ",\n";
        dump_json_value<const float>(priority, NULL, settings, "float", "priority", false, false, 4, dump_json_float);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceBufferMemoryRequirementsKHR(ApiDumpInstance& dump_inst, VkDevice device, const VkDeviceBufferMemoryRequirements* pInfo, VkMemoryRequirements2* pMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceBufferMemoryRequirements>(pInfo, settings, "const VkDeviceBufferMemoryRequirements*", "pInfo", true, false, 4, dump_json_VkDeviceBufferMemoryRequirements);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryRequirements2>(pMemoryRequirements, settings, "VkMemoryRequirements2*", "pMemoryRequirements", true, false, 4, dump_json_VkMemoryRequirements2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceImageMemoryRequirementsKHR(ApiDumpInstance& dump_inst, VkDevice device, const VkDeviceImageMemoryRequirements* pInfo, VkMemoryRequirements2* pMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceImageMemoryRequirements>(pInfo, settings, "const VkDeviceImageMemoryRequirements*", "pInfo", true, false, 4, dump_json_VkDeviceImageMemoryRequirements);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryRequirements2>(pMemoryRequirements, settings, "VkMemoryRequirements2*", "pMemoryRequirements", true, false, 4, dump_json_VkMemoryRequirements2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceImageSparseMemoryRequirementsKHR(ApiDumpInstance& dump_inst, VkDevice device, const VkDeviceImageMemoryRequirements* pInfo, uint32_t* pSparseMemoryRequirementCount, VkSparseImageMemoryRequirements2* pSparseMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceImageMemoryRequirements>(pInfo, settings, "const VkDeviceImageMemoryRequirements*", "pInfo", true, false, 4, dump_json_VkDeviceImageMemoryRequirements);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pSparseMemoryRequirementCount, settings, "uint32_t*", "pSparseMemoryRequirementCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkSparseImageMemoryRequirements2>(pSparseMemoryRequirements, *pSparseMemoryRequirementCount, settings, "VkSparseImageMemoryRequirements2*", "VkSparseImageMemoryRequirements2", "pSparseMemoryRequirements", true, false, 4, dump_json_VkSparseImageMemoryRequirements2); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDescriptorSetLayoutHostMappingInfoVALVE(ApiDumpInstance& dump_inst, VkDevice device, const VkDescriptorSetBindingReferenceVALVE* pBindingReference, VkDescriptorSetLayoutHostMappingInfoVALVE* pHostMapping)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorSetBindingReferenceVALVE>(pBindingReference, settings, "const VkDescriptorSetBindingReferenceVALVE*", "pBindingReference", true, false, 4, dump_json_VkDescriptorSetBindingReferenceVALVE);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDescriptorSetLayoutHostMappingInfoVALVE>(pHostMapping, settings, "VkDescriptorSetLayoutHostMappingInfoVALVE*", "pHostMapping", true, false, 4, dump_json_VkDescriptorSetLayoutHostMappingInfoVALVE);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDescriptorSetHostMappingVALVE(ApiDumpInstance& dump_inst, VkDevice device, VkDescriptorSet descriptorSet, void** ppData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkDescriptorSet>(descriptorSet, NULL, settings, "VkDescriptorSet", "descriptorSet", false, false, 4, dump_json_VkDescriptorSet);
        settings.stream() << ",\n";
        dump_json_pointer<const void*>(ppData, settings, "void**", "ppData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyMemoryIndirectNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkDeviceAddress copyBufferAddress, uint32_t copyCount, uint32_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceAddress>(copyBufferAddress, NULL, settings, "VkDeviceAddress", "copyBufferAddress", false, false, 4, dump_json_VkDeviceAddress);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(copyCount, NULL, settings, "uint32_t", "copyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdCopyMemoryToImageIndirectNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkDeviceAddress copyBufferAddress, uint32_t copyCount, uint32_t stride, VkImage dstImage, VkImageLayout dstImageLayout, const VkImageSubresourceLayers* pImageSubresources)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceAddress>(copyBufferAddress, NULL, settings, "VkDeviceAddress", "copyBufferAddress", false, false, 4, dump_json_VkDeviceAddress);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(copyCount, NULL, settings, "uint32_t", "copyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const VkImage>(dstImage, NULL, settings, "VkImage", "dstImage", false, false, 4, dump_json_VkImage);
        settings.stream() << ",\n";
        dump_json_value<const VkImageLayout>(dstImageLayout, NULL, settings, "VkImageLayout", "dstImageLayout", false, false, 4, dump_json_VkImageLayout);
        settings.stream() << ",\n";
        dump_json_array<const VkImageSubresourceLayers>(pImageSubresources, copyCount, settings, "const VkImageSubresourceLayers*", "const VkImageSubresourceLayers", "pImageSubresources", true, false, 4, dump_json_VkImageSubresourceLayers); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDecompressMemoryNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t decompressRegionCount, const VkDecompressMemoryRegionNV* pDecompressMemoryRegions)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(decompressRegionCount, NULL, settings, "uint32_t", "decompressRegionCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkDecompressMemoryRegionNV>(pDecompressMemoryRegions, decompressRegionCount, settings, "const VkDecompressMemoryRegionNV*", "const VkDecompressMemoryRegionNV", "pDecompressMemoryRegions", true, false, 4, dump_json_VkDecompressMemoryRegionNV); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdDecompressMemoryIndirectCountNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkDeviceAddress indirectCommandsAddress, VkDeviceAddress indirectCommandsCountAddress, uint32_t stride)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceAddress>(indirectCommandsAddress, NULL, settings, "VkDeviceAddress", "indirectCommandsAddress", false, false, 4, dump_json_VkDeviceAddress);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceAddress>(indirectCommandsCountAddress, NULL, settings, "VkDeviceAddress", "indirectCommandsCountAddress", false, false, 4, dump_json_VkDeviceAddress);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stride, NULL, settings, "uint32_t", "stride", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPipelineIndirectMemoryRequirementsNV(ApiDumpInstance& dump_inst, VkDevice device, const VkComputePipelineCreateInfo* pCreateInfo, VkMemoryRequirements2* pMemoryRequirements)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkComputePipelineCreateInfo>(pCreateInfo, settings, "const VkComputePipelineCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkComputePipelineCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkMemoryRequirements2>(pMemoryRequirements, settings, "VkMemoryRequirements2*", "pMemoryRequirements", true, false, 4, dump_json_VkMemoryRequirements2);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdUpdatePipelineIndirectBufferNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkPipelineBindPoint           pipelineBindPoint, VkPipeline                    pipeline)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPipelineBindPoint>(pipelineBindPoint, NULL, settings, "VkPipelineBindPoint", "pipelineBindPoint", false, false, 4, dump_json_VkPipelineBindPoint);
        settings.stream() << ",\n";
        dump_json_value<const VkPipeline>(pipeline, NULL, settings, "VkPipeline", "pipeline", false, false, 4, dump_json_VkPipeline);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPipelineIndirectDeviceAddressNV(ApiDumpInstance& dump_inst, VkDeviceAddress result, VkDevice device, const VkPipelineIndirectDeviceAddressInfoNV* pInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkDeviceAddress(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPipelineIndirectDeviceAddressInfoNV>(pInfo, settings, "const VkPipelineIndirectDeviceAddressInfoNV*", "pInfo", true, false, 4, dump_json_VkPipelineIndirectDeviceAddressInfoNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthClampEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 depthClampEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(depthClampEnable, NULL, settings, "VkBool32", "depthClampEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetPolygonModeEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkPolygonMode polygonMode)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkPolygonMode>(polygonMode, NULL, settings, "VkPolygonMode", "polygonMode", false, false, 4, dump_json_VkPolygonMode);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetRasterizationSamplesEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkSampleCountFlagBits  rasterizationSamples)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkSampleCountFlagBits>(rasterizationSamples, NULL, settings, "VkSampleCountFlagBits", "rasterizationSamples", false, false, 4, dump_json_VkSampleCountFlagBits);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetSampleMaskEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkSampleCountFlagBits  samples, const VkSampleMask*    pSampleMask)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkSampleCountFlagBits>(samples, NULL, settings, "VkSampleCountFlagBits", "samples", false, false, 4, dump_json_VkSampleCountFlagBits);
        settings.stream() << ",\n";
        dump_json_array<const VkSampleMask>(pSampleMask, (samples + 31) / 32, settings, "const VkSampleMask*", "const VkSampleMask", "pSampleMask", false, false, 4, dump_json_VkSampleMask); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetAlphaToCoverageEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 alphaToCoverageEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(alphaToCoverageEnable, NULL, settings, "VkBool32", "alphaToCoverageEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetAlphaToOneEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 alphaToOneEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(alphaToOneEnable, NULL, settings, "VkBool32", "alphaToOneEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetLogicOpEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 logicOpEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(logicOpEnable, NULL, settings, "VkBool32", "logicOpEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetColorBlendEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstAttachment, uint32_t attachmentCount, const VkBool32* pColorBlendEnables)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstAttachment, NULL, settings, "uint32_t", "firstAttachment", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(attachmentCount, NULL, settings, "uint32_t", "attachmentCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkBool32>(pColorBlendEnables, attachmentCount, settings, "const VkBool32*", "const VkBool32", "pColorBlendEnables", false, false, 4, dump_json_VkBool32); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetColorBlendEquationEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstAttachment, uint32_t attachmentCount, const VkColorBlendEquationEXT* pColorBlendEquations)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstAttachment, NULL, settings, "uint32_t", "firstAttachment", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(attachmentCount, NULL, settings, "uint32_t", "attachmentCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkColorBlendEquationEXT>(pColorBlendEquations, attachmentCount, settings, "const VkColorBlendEquationEXT*", "const VkColorBlendEquationEXT", "pColorBlendEquations", true, false, 4, dump_json_VkColorBlendEquationEXT); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetColorWriteMaskEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstAttachment, uint32_t attachmentCount, const VkColorComponentFlags* pColorWriteMasks)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstAttachment, NULL, settings, "uint32_t", "firstAttachment", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(attachmentCount, NULL, settings, "uint32_t", "attachmentCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkColorComponentFlags>(pColorWriteMasks, attachmentCount, settings, "const VkColorComponentFlags*", "const VkColorComponentFlags", "pColorWriteMasks", false, false, 4, dump_json_VkColorComponentFlags); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetTessellationDomainOriginEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkTessellationDomainOrigin domainOrigin)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkTessellationDomainOrigin>(domainOrigin, NULL, settings, "VkTessellationDomainOrigin", "domainOrigin", false, false, 4, dump_json_VkTessellationDomainOrigin);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetRasterizationStreamEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t rasterizationStream)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(rasterizationStream, NULL, settings, "uint32_t", "rasterizationStream", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetConservativeRasterizationModeEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkConservativeRasterizationModeEXT conservativeRasterizationMode)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkConservativeRasterizationModeEXT>(conservativeRasterizationMode, NULL, settings, "VkConservativeRasterizationModeEXT", "conservativeRasterizationMode", false, false, 4, dump_json_VkConservativeRasterizationModeEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetExtraPrimitiveOverestimationSizeEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, float extraPrimitiveOverestimationSize)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const float>(extraPrimitiveOverestimationSize, NULL, settings, "float", "extraPrimitiveOverestimationSize", false, false, 4, dump_json_float);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthClipEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 depthClipEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(depthClipEnable, NULL, settings, "VkBool32", "depthClipEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetSampleLocationsEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 sampleLocationsEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(sampleLocationsEnable, NULL, settings, "VkBool32", "sampleLocationsEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetColorBlendAdvancedEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstAttachment, uint32_t attachmentCount, const VkColorBlendAdvancedEXT* pColorBlendAdvanced)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstAttachment, NULL, settings, "uint32_t", "firstAttachment", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(attachmentCount, NULL, settings, "uint32_t", "attachmentCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkColorBlendAdvancedEXT>(pColorBlendAdvanced, attachmentCount, settings, "const VkColorBlendAdvancedEXT*", "const VkColorBlendAdvancedEXT", "pColorBlendAdvanced", true, false, 4, dump_json_VkColorBlendAdvancedEXT); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetProvokingVertexModeEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkProvokingVertexModeEXT provokingVertexMode)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkProvokingVertexModeEXT>(provokingVertexMode, NULL, settings, "VkProvokingVertexModeEXT", "provokingVertexMode", false, false, 4, dump_json_VkProvokingVertexModeEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetLineRasterizationModeEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkLineRasterizationModeEXT lineRasterizationMode)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkLineRasterizationModeKHR>(lineRasterizationMode, NULL, settings, "VkLineRasterizationModeEXT", "lineRasterizationMode", false, false, 4, dump_json_VkLineRasterizationModeKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetLineStippleEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 stippledLineEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(stippledLineEnable, NULL, settings, "VkBool32", "stippledLineEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDepthClipNegativeOneToOneEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 negativeOneToOne)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(negativeOneToOne, NULL, settings, "VkBool32", "negativeOneToOne", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetViewportWScalingEnableNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 viewportWScalingEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(viewportWScalingEnable, NULL, settings, "VkBool32", "viewportWScalingEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetViewportSwizzleNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t firstViewport, uint32_t viewportCount, const VkViewportSwizzleNV* pViewportSwizzles)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(firstViewport, NULL, settings, "uint32_t", "firstViewport", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(viewportCount, NULL, settings, "uint32_t", "viewportCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkViewportSwizzleNV>(pViewportSwizzles, viewportCount, settings, "const VkViewportSwizzleNV*", "const VkViewportSwizzleNV", "pViewportSwizzles", true, false, 4, dump_json_VkViewportSwizzleNV); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetCoverageToColorEnableNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 coverageToColorEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(coverageToColorEnable, NULL, settings, "VkBool32", "coverageToColorEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetCoverageToColorLocationNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t coverageToColorLocation)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(coverageToColorLocation, NULL, settings, "uint32_t", "coverageToColorLocation", false, false, 4, dump_json_uint32_t);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetCoverageModulationModeNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkCoverageModulationModeNV coverageModulationMode)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkCoverageModulationModeNV>(coverageModulationMode, NULL, settings, "VkCoverageModulationModeNV", "coverageModulationMode", false, false, 4, dump_json_VkCoverageModulationModeNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetCoverageModulationTableEnableNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 coverageModulationTableEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(coverageModulationTableEnable, NULL, settings, "VkBool32", "coverageModulationTableEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetCoverageModulationTableNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t coverageModulationTableCount, const float* pCoverageModulationTable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(coverageModulationTableCount, NULL, settings, "uint32_t", "coverageModulationTableCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const float>(pCoverageModulationTable, coverageModulationTableCount, settings, "const float*", "const float", "pCoverageModulationTable", false, false, 4, dump_json_float); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetShadingRateImageEnableNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 shadingRateImageEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(shadingRateImageEnable, NULL, settings, "VkBool32", "shadingRateImageEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetRepresentativeFragmentTestEnableNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBool32 representativeFragmentTestEnable)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBool32>(representativeFragmentTestEnable, NULL, settings, "VkBool32", "representativeFragmentTestEnable", false, false, 4, dump_json_VkBool32);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetCoverageReductionModeNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkCoverageReductionModeNV coverageReductionMode)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkCoverageReductionModeNV>(coverageReductionMode, NULL, settings, "VkCoverageReductionModeNV", "coverageReductionMode", false, false, 4, dump_json_VkCoverageReductionModeNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetShaderModuleIdentifierEXT(ApiDumpInstance& dump_inst, VkDevice device, VkShaderModule shaderModule, VkShaderModuleIdentifierEXT* pIdentifier)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkShaderModule>(shaderModule, NULL, settings, "VkShaderModule", "shaderModule", false, false, 4, dump_json_VkShaderModule);
        settings.stream() << ",\n";
        dump_json_pointer<const VkShaderModuleIdentifierEXT>(pIdentifier, settings, "VkShaderModuleIdentifierEXT*", "pIdentifier", true, false, 4, dump_json_VkShaderModuleIdentifierEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetShaderModuleCreateInfoIdentifierEXT(ApiDumpInstance& dump_inst, VkDevice device, const VkShaderModuleCreateInfo* pCreateInfo, VkShaderModuleIdentifierEXT* pIdentifier)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkShaderModuleCreateInfo>(pCreateInfo, settings, "const VkShaderModuleCreateInfo*", "pCreateInfo", true, false, 4, dump_json_VkShaderModuleCreateInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkShaderModuleIdentifierEXT>(pIdentifier, settings, "VkShaderModuleIdentifierEXT*", "pIdentifier", true, false, 4, dump_json_VkShaderModuleIdentifierEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceOpticalFlowImageFormatsNV(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, const VkOpticalFlowImageFormatInfoNV* pOpticalFlowImageFormatInfo, uint32_t* pFormatCount, VkOpticalFlowImageFormatPropertiesNV* pImageFormatProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkOpticalFlowImageFormatInfoNV>(pOpticalFlowImageFormatInfo, settings, "const VkOpticalFlowImageFormatInfoNV*", "pOpticalFlowImageFormatInfo", true, false, 4, dump_json_VkOpticalFlowImageFormatInfoNV);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pFormatCount, settings, "uint32_t*", "pFormatCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkOpticalFlowImageFormatPropertiesNV>(pImageFormatProperties, *pFormatCount, settings, "VkOpticalFlowImageFormatPropertiesNV*", "VkOpticalFlowImageFormatPropertiesNV", "pImageFormatProperties", true, false, 4, dump_json_VkOpticalFlowImageFormatPropertiesNV); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateOpticalFlowSessionNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkOpticalFlowSessionCreateInfoNV* pCreateInfo, const VkAllocationCallbacks* pAllocator, VkOpticalFlowSessionNV* pSession)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkOpticalFlowSessionCreateInfoNV>(pCreateInfo, settings, "const VkOpticalFlowSessionCreateInfoNV*", "pCreateInfo", true, false, 4, dump_json_VkOpticalFlowSessionCreateInfoNV);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_pointer<const VkOpticalFlowSessionNV>(pSession, settings, "VkOpticalFlowSessionNV*", "pSession", false, false, 4, dump_json_VkOpticalFlowSessionNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyOpticalFlowSessionNV(ApiDumpInstance& dump_inst, VkDevice device, VkOpticalFlowSessionNV session, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkOpticalFlowSessionNV>(session, NULL, settings, "VkOpticalFlowSessionNV", "session", false, false, 4, dump_json_VkOpticalFlowSessionNV);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkBindOpticalFlowSessionImageNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkOpticalFlowSessionNV session, VkOpticalFlowSessionBindingPointNV bindingPoint, VkImageView view, VkImageLayout layout)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkOpticalFlowSessionNV>(session, NULL, settings, "VkOpticalFlowSessionNV", "session", false, false, 4, dump_json_VkOpticalFlowSessionNV);
        settings.stream() << ",\n";
        dump_json_value<const VkOpticalFlowSessionBindingPointNV>(bindingPoint, NULL, settings, "VkOpticalFlowSessionBindingPointNV", "bindingPoint", false, false, 4, dump_json_VkOpticalFlowSessionBindingPointNV);
        settings.stream() << ",\n";
        dump_json_value<const VkImageView>(view, NULL, settings, "VkImageView", "view", false, false, 4, dump_json_VkImageView);
        settings.stream() << ",\n";
        dump_json_value<const VkImageLayout>(layout, NULL, settings, "VkImageLayout", "layout", false, false, 4, dump_json_VkImageLayout);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdOpticalFlowExecuteNV(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkOpticalFlowSessionNV session, const VkOpticalFlowExecuteInfoNV* pExecuteInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkOpticalFlowSessionNV>(session, NULL, settings, "VkOpticalFlowSessionNV", "session", false, false, 4, dump_json_VkOpticalFlowSessionNV);
        settings.stream() << ",\n";
        dump_json_pointer<const VkOpticalFlowExecuteInfoNV>(pExecuteInfo, settings, "const VkOpticalFlowExecuteInfoNV*", "pExecuteInfo", true, false, 4, dump_json_VkOpticalFlowExecuteInfoNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBindIndexBuffer2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkBuffer buffer, VkDeviceSize offset, VkDeviceSize size, VkIndexType indexType)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkBuffer>(buffer, NULL, settings, "VkBuffer", "buffer", false, false, 4, dump_json_VkBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(offset, NULL, settings, "VkDeviceSize", "offset", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkDeviceSize>(size, NULL, settings, "VkDeviceSize", "size", false, false, 4, dump_json_VkDeviceSize);
        settings.stream() << ",\n";
        dump_json_value<const VkIndexType>(indexType, NULL, settings, "VkIndexType", "indexType", false, false, 4, dump_json_VkIndexType);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetRenderingAreaGranularityKHR(ApiDumpInstance& dump_inst, VkDevice device, const VkRenderingAreaInfoKHR* pRenderingAreaInfo, VkExtent2D* pGranularity)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRenderingAreaInfoKHR>(pRenderingAreaInfo, settings, "const VkRenderingAreaInfoKHR*", "pRenderingAreaInfo", true, false, 4, dump_json_VkRenderingAreaInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkExtent2D>(pGranularity, settings, "VkExtent2D*", "pGranularity", true, false, 4, dump_json_VkExtent2D);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDeviceImageSubresourceLayoutKHR(ApiDumpInstance& dump_inst, VkDevice device, const VkDeviceImageSubresourceInfoKHR* pInfo, VkSubresourceLayout2KHR* pLayout)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkDeviceImageSubresourceInfoKHR>(pInfo, settings, "const VkDeviceImageSubresourceInfoKHR*", "pInfo", true, false, 4, dump_json_VkDeviceImageSubresourceInfoKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSubresourceLayout2KHR>(pLayout, settings, "VkSubresourceLayout2KHR*", "pLayout", true, false, 4, dump_json_VkSubresourceLayout2KHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCreateShadersEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, uint32_t createInfoCount, const VkShaderCreateInfoEXT* pCreateInfos, const VkAllocationCallbacks* pAllocator, VkShaderEXT* pShaders)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(createInfoCount, NULL, settings, "uint32_t", "createInfoCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkShaderCreateInfoEXT>(pCreateInfos, createInfoCount, settings, "const VkShaderCreateInfoEXT*", "const VkShaderCreateInfoEXT", "pCreateInfos", true, false, 4, dump_json_VkShaderCreateInfoEXT); // PQA
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << ",\n";
        dump_json_array<const VkShaderEXT>(pShaders, createInfoCount, settings, "VkShaderEXT*", "VkShaderEXT", "pShaders", false, false, 4, dump_json_VkShaderEXT); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkDestroyShaderEXT(ApiDumpInstance& dump_inst, VkDevice device, VkShaderEXT shader, const VkAllocationCallbacks* pAllocator)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkShaderEXT>(shader, NULL, settings, "VkShaderEXT", "shader", false, false, 4, dump_json_VkShaderEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const VkAllocationCallbacks>(pAllocator, settings, "const VkAllocationCallbacks*", "pAllocator", true, false, 4, dump_json_VkAllocationCallbacks);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetShaderBinaryDataEXT(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkShaderEXT shader, size_t* pDataSize, void* pData)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkShaderEXT>(shader, NULL, settings, "VkShaderEXT", "shader", false, false, 4, dump_json_VkShaderEXT);
        settings.stream() << ",\n";
        dump_json_pointer<const size_t>(pDataSize, settings, "size_t*", "pDataSize", false, false, 4, dump_json_size_t);
        settings.stream() << ",\n";
        dump_json_value<const void*>(pData, NULL, settings, "void*", "pData", false, false, 4, dump_json_void);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBindShadersEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, uint32_t stageCount, const VkShaderStageFlagBits* pStages, const VkShaderEXT* pShaders)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const uint32_t>(stageCount, NULL, settings, "uint32_t", "stageCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkShaderStageFlagBits>(pStages, stageCount, settings, "const VkShaderStageFlagBits*", "const VkShaderStageFlagBits", "pStages", false, false, 4, dump_json_VkShaderStageFlagBits); // PQA
        settings.stream() << ",\n";
        dump_json_array<const VkShaderEXT>(pShaders, stageCount, settings, "const VkShaderEXT*", "const VkShaderEXT", "pShaders", false, false, 4, dump_json_VkShaderEXT); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetFramebufferTilePropertiesQCOM(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkFramebuffer framebuffer, uint32_t* pPropertiesCount, VkTilePropertiesQCOM* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkFramebuffer>(framebuffer, NULL, settings, "VkFramebuffer", "framebuffer", false, false, 4, dump_json_VkFramebuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPropertiesCount, settings, "uint32_t*", "pPropertiesCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkTilePropertiesQCOM>(pProperties, *pPropertiesCount, settings, "VkTilePropertiesQCOM*", "VkTilePropertiesQCOM", "pProperties", true, false, 4, dump_json_VkTilePropertiesQCOM); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetDynamicRenderingTilePropertiesQCOM(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const VkRenderingInfo* pRenderingInfo, VkTilePropertiesQCOM* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const VkRenderingInfo>(pRenderingInfo, settings, "const VkRenderingInfo*", "pRenderingInfo", true, false, 4, dump_json_VkRenderingInfo);
        settings.stream() << ",\n";
        dump_json_pointer<const VkTilePropertiesQCOM>(pProperties, settings, "VkTilePropertiesQCOM*", "pProperties", true, false, 4, dump_json_VkTilePropertiesQCOM);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkSetLatencySleepModeNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkSwapchainKHR swapchain, const VkLatencySleepModeInfoNV* pSleepModeInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSwapchainKHR>(swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkLatencySleepModeInfoNV>(pSleepModeInfo, settings, "const VkLatencySleepModeInfoNV*", "pSleepModeInfo", true, false, 4, dump_json_VkLatencySleepModeInfoNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkLatencySleepNV(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, VkSwapchainKHR swapchain, const VkLatencySleepInfoNV* pSleepInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSwapchainKHR>(swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkLatencySleepInfoNV>(pSleepInfo, settings, "const VkLatencySleepInfoNV*", "pSleepInfo", true, false, 4, dump_json_VkLatencySleepInfoNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkSetLatencyMarkerNV(ApiDumpInstance& dump_inst, VkDevice device, VkSwapchainKHR swapchain, const VkSetLatencyMarkerInfoNV* pLatencyMarkerInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSwapchainKHR>(swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSetLatencyMarkerInfoNV>(pLatencyMarkerInfo, settings, "const VkSetLatencyMarkerInfoNV*", "pLatencyMarkerInfo", true, false, 4, dump_json_VkSetLatencyMarkerInfoNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetLatencyTimingsNV(ApiDumpInstance& dump_inst, VkDevice device, VkSwapchainKHR swapchain, VkGetLatencyMarkerInfoNV* pLatencyMarkerInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const VkSwapchainKHR>(swapchain, NULL, settings, "VkSwapchainKHR", "swapchain", false, false, 4, dump_json_VkSwapchainKHR);
        settings.stream() << ",\n";
        dump_json_pointer<const VkGetLatencyMarkerInfoNV>(pLatencyMarkerInfo, settings, "VkGetLatencyMarkerInfoNV*", "pLatencyMarkerInfo", true, false, 4, dump_json_VkGetLatencyMarkerInfoNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkQueueNotifyOutOfBandNV(ApiDumpInstance& dump_inst, VkQueue queue, const VkOutOfBandQueueTypeInfoNV* pQueueTypeInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkQueue>(queue, NULL, settings, "VkQueue", "queue", false, false, 4, dump_json_VkQueue);
        settings.stream() << ",\n";
        dump_json_pointer<const VkOutOfBandQueueTypeInfoNV>(pQueueTypeInfo, settings, "const VkOutOfBandQueueTypeInfoNV*", "pQueueTypeInfo", true, false, 4, dump_json_VkOutOfBandQueueTypeInfoNV);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkGetPhysicalDeviceCooperativeMatrixPropertiesKHR(ApiDumpInstance& dump_inst, VkResult result, VkPhysicalDevice physicalDevice, uint32_t* pPropertyCount, VkCooperativeMatrixPropertiesKHR* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkPhysicalDevice>(physicalDevice, NULL, settings, "VkPhysicalDevice", "physicalDevice", false, false, 4, dump_json_VkPhysicalDevice);
        settings.stream() << ",\n";
        dump_json_pointer<const uint32_t>(pPropertyCount, settings, "uint32_t*", "pPropertyCount", false, false, 4, dump_json_uint32_t);
        settings.stream() << ",\n";
        dump_json_array<const VkCooperativeMatrixPropertiesKHR>(pProperties, *pPropertyCount, settings, "VkCooperativeMatrixPropertiesKHR*", "VkCooperativeMatrixPropertiesKHR", "pProperties", true, false, 4, dump_json_VkCooperativeMatrixPropertiesKHR); // PQA
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetAttachmentFeedbackLoopEnableEXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, VkImageAspectFlags aspectMask)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_value<const VkImageAspectFlags>(aspectMask, NULL, settings, "VkImageAspectFlags", "aspectMask", false, false, 4, dump_json_VkImageAspectFlags);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#if defined(VK_USE_PLATFORM_SCREEN_QNX)
void dump_json_vkGetScreenBufferPropertiesQNX(ApiDumpInstance& dump_inst, VkResult result, VkDevice device, const struct _screen_buffer* buffer, VkScreenBufferPropertiesQNX* pProperties)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    settings.stream() << settings.indentation(3) << "\"returnValue\" : ";
    dump_json_VkResult(result, settings, 0);
    if(settings.showParams())
        settings.stream() << ",";
    settings.stream() << "\n";
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkDevice>(device, NULL, settings, "VkDevice", "device", false, false, 4, dump_json_VkDevice);
        settings.stream() << ",\n";
        dump_json_value<const _screen_buffer*>(buffer, NULL, settings, "const struct _screen_buffer*", "buffer", false, false, 4, dump_json__screen_buffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkScreenBufferPropertiesQNX>(pProperties, settings, "VkScreenBufferPropertiesQNX*", "pProperties", true, false, 4, dump_json_VkScreenBufferPropertiesQNX);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
#endif // VK_USE_PLATFORM_SCREEN_QNX
void dump_json_vkCmdBindDescriptorSets2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkBindDescriptorSetsInfoKHR* pBindDescriptorSetsInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBindDescriptorSetsInfoKHR>(pBindDescriptorSetsInfo, settings, "const VkBindDescriptorSetsInfoKHR*", "pBindDescriptorSetsInfo", true, false, 4, dump_json_VkBindDescriptorSetsInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdPushConstants2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkPushConstantsInfoKHR* pPushConstantsInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPushConstantsInfoKHR>(pPushConstantsInfo, settings, "const VkPushConstantsInfoKHR*", "pPushConstantsInfo", true, false, 4, dump_json_VkPushConstantsInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdPushDescriptorSet2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkPushDescriptorSetInfoKHR* pPushDescriptorSetInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPushDescriptorSetInfoKHR>(pPushDescriptorSetInfo, settings, "const VkPushDescriptorSetInfoKHR*", "pPushDescriptorSetInfo", true, false, 4, dump_json_VkPushDescriptorSetInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdPushDescriptorSetWithTemplate2KHR(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkPushDescriptorSetWithTemplateInfoKHR* pPushDescriptorSetWithTemplateInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkPushDescriptorSetWithTemplateInfoKHR>(pPushDescriptorSetWithTemplateInfo, settings, "const VkPushDescriptorSetWithTemplateInfoKHR*", "pPushDescriptorSetWithTemplateInfo", true, false, 4, dump_json_VkPushDescriptorSetWithTemplateInfoKHR);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdSetDescriptorBufferOffsets2EXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkSetDescriptorBufferOffsetsInfoEXT* pSetDescriptorBufferOffsetsInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkSetDescriptorBufferOffsetsInfoEXT>(pSetDescriptorBufferOffsetsInfo, settings, "const VkSetDescriptorBufferOffsetsInfoEXT*", "pSetDescriptorBufferOffsetsInfo", true, false, 4, dump_json_VkSetDescriptorBufferOffsetsInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}
void dump_json_vkCmdBindDescriptorBufferEmbeddedSamplers2EXT(ApiDumpInstance& dump_inst, VkCommandBuffer commandBuffer, const VkBindDescriptorBufferEmbeddedSamplersInfoEXT* pBindDescriptorBufferEmbeddedSamplersInfo)
{
    const ApiDumpSettings& settings(dump_inst.settings());
    // Display parameter values
    if(settings.showParams())
    {
        settings.stream() << settings.indentation(3) << "\"args\" :\n";
        settings.stream() << settings.indentation(3) << "[\n";
        dump_json_value<const VkCommandBuffer>(commandBuffer, NULL, settings, "VkCommandBuffer", "commandBuffer", false, false, 4, dump_json_VkCommandBuffer);
        settings.stream() << ",\n";
        dump_json_pointer<const VkBindDescriptorBufferEmbeddedSamplersInfoEXT>(pBindDescriptorBufferEmbeddedSamplersInfo, settings, "const VkBindDescriptorBufferEmbeddedSamplersInfoEXT*", "pBindDescriptorBufferEmbeddedSamplersInfo", true, false, 4, dump_json_VkBindDescriptorBufferEmbeddedSamplersInfoEXT);
        settings.stream() << "\n" << settings.indentation(3) << "]\n";
    }
    settings.stream() << settings.indentation(2) << "}";
    if (settings.shouldFlush()) settings.stream().flush();
}


